#!/bin/sh
#
# Shell script to customize RPM files for the Aic7xxx driver.
#
# $1 = Build Environment Root
#
# $2 = RPM destination directory
#
RPM_TEMPLATE_DIR=$1/pkg/rpm

SUMMARY="Adaptec Driver Binaries"
PRODUCT="aar1210"
location=`find $1/scsi/$PRODUCT -type f -name 'hr.c' -print`
class="1"
if [ "$location" == "" ];then
	location=`find $1/scsi/$PRODUCT -type f -name 'a320.c' -print`
	class="2"
	if [ "$location" == "" ];then
		exit
	fi
fi
if [ "$class" == "1" ];then
	MAJOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*HR_DRIVER_VERSION[[:space:]]*"\(.*\)".*/\1/p'`
	MINOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*HR_DRIVER_REVISION[[:space:]]*"\(.*\)".*/\1/p'`
	SUBMINOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*HR_DRIVER_SUBREVISION[[:space:]]*"\(.*\)".*/\1/p'`
	RELEASE=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*HR_DRIVER_VARIANT[[:space:]]*"\(.*\)".*/\1/p'`
else
	MAJOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_VERSION[[:space:]]*"\(.*\)".*/\1/p'`
	MINOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_REVISION[[:space:]]*"\(.*\)".*/\1/p'`
	SUBMINOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_SUBREVISION[[:space:]]*"\(.*\)".*/\1/p'`
	RELEASE=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_VARIANT[[:space:]]*"\(.*\)".*/\1/p'`
fi
RELEASE="$3"
idir=$1/scsi/$PRODUCT
if [ -e $idir/chimver.h ];then
	cp $idir/chimver.h $idir/chimver.h_
 	dos2unix -q $idir/chimver.h_
	chimversion=`grep Version: $idir/chimver.h_|awk -F 'Version: ' '{print $2}'|awk -F $ '{print $1}'`
	rm -f $idir/chimver.h_
	VERSION=${MAJOR_VERSION}.${MINOR_VERSION}.${SUBMINOR_VERSION}.${chimversion}
else
	VERSION=${MAJOR_VERSION}.${MINOR_VERSION}.${SUBMINOR_VERSION}
fi
COPYRIGHT="Adaptec Inc."
DESCRIPTION="This driver supports the Adaptec AAR1210 SATA RAID product."
MODULE_INSTALL_PATH=kernel/drivers/scsi

sed_cmds="
	s=%_SUMMARY_%=$SUMMARY=g
	s=%_DESCRIPTION_%=$DESCRIPTION=g
	s=%_PRODUCT_%=$PRODUCT=g
	s=%_VERSION_%=$VERSION=g
	s=%_RELEASE_%=$RELEASE=g
	s=%_COPYRIGHT_%=$COPYRIGHT=g
	s=%_MODULE_INSTALL_PATH_%=$MODULE_INSTALL_PATH=g"

cp $RPM_TEMPLATE_DIR/* $2
sed "$sed_cmds" $2/template.spec > $2/$PRODUCT.spec
rm $2/template.spec
sed "$sed_cmds" $2/install.sh > $2/install.sh.$$
mv $2/install.sh.$$ $2/install.sh
