#!/bin/sh
#
# Shell script to customize RPM files for the adpahci driver.
#
# $1 = Build Environment Root
#
# $2 = RPM destination directory
#
# $3 = RPM RELEASE NUMBER
#
RPM_TEMPLATE_DIR=$1/pkg/rpm

SUMMARY="Adaptec Driver Binaries"
PRODUCT="adpahci"
location=`find $1/scsi/adpahci -type f -maxdepth 1 -name 'hr.c' -print`
class="1"
if [ "$location" == "" ];then
	location=`find $1/scsi/adpahci -type f -maxdepth 1 -name 'a320.c' -print`
	class="2"
	if [ "$location" == "" ];then
		exit
	fi
fi
if [ "$class" == "1" ];then
	echo -n "Class="$class
	MAJOR_VERSION=`cat $location | \
		 sed -n 's/^#[[:space:]]*define[[:space:]]*ADPT_MAJOR_REV[[:space:]]*\([0-9]\+\).*/\1/p'`
	MINOR_VERSION=`cat $location | \
		 sed -n 's/^#[[:space:]]*define[[:space:]]*ADPT_MINOR_REV[[:space:]]*\([0-9]\+\).*/\1/p'`
	SUBMINOR_VERSION=`cat $location | \
		 sed -n 's/^#[[:space:]]*define[[:space:]]*ADPT_OSM_REV[[:space:]]*\([0-9]\+\).*/\1/p'`
	CHIM_VERSION=`cat $location | \
		 sed -n 's/^#[[:space:]]*define[[:space:]]*ADPT_CHIM_REV[[:space:]]*\([0-9]\+\).*/\1/p'`
else
	MAJOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_VERSION[[:space:]]*"\(.*\)"/\1/p'`
	MINOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_REVISION[[:space:]]*"\(.*\)"/\1/p'`
	SUBMINOR_VERSION=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_SUBREVISION[[:space:]]*"\(.*\)"/\1/p'`
	RELEASE=`cat $location | \
		 sed -n 's/#[[:space:]]*define[[:space:]]*A320_DRIVER_VARIANT[[:space:]]*"\(.*\)"/\1/p'`
fi
RELEASE="$3"
idir=$1/scsi/$PRODUCT
if [ -e $idir/chimver.h ];then
	cp $idir/chimver.h $idir/chimver.h_
 	dos2unix -q $idir/chimver.h_
	chimversion=`grep Version: $idir/chimver.h_|awk -F 'Version: ' '{print $2}'|awk -F $ '{print $1}'`
	rm -f $idir/chimver.h_
	VERSION=${MAJOR_VERSION}.${MINOR_VERSION}.${SUBMINOR_VERSION}.${CHIM_VERSION}
else
	VERSION=${MAJOR_VERSION}.${MINOR_VERSION}.${SUBMINOR_VERSION}
fi
COPYRIGHT="Adaptec Inc."
DESCRIPTION="This driver supports the Adaptec ADPAHCI SATA RAID product."
MODULE_INSTALL_PATH=kernel/drivers/scsi

sed_cmds="
	s=%_SUMMARY_%=$SUMMARY=g
	s=%_DESCRIPTION_%=$DESCRIPTION=g
	s=%_PRODUCT_%=$PRODUCT=g
	s=%_VERSION_%=$VERSION=g
	s=%_RELEASE_%=$RELEASE=g
	s=%_COPYRIGHT_%=$COPYRIGHT=g
	s=%_MODULE_INSTALL_PATH_%=$MODULE_INSTALL_PATH=g"

echo " Here ?!"
cp $RPM_TEMPLATE_DIR/* $2
sed "$sed_cmds" $2/template.spec > $2/$PRODUCT.spec
rm $2/template.spec
sed "$sed_cmds" $2/install.sh > $2/install.sh.$$
mv $2/install.sh.$$ $2/install.sh
echo " sed_cmds=="$sed_cmds
echo " Class="$class" Exit !!!"
