/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: address.h

File Description
Definition of structures related to the addressing of objects.

***********************************************************************

History :

05/20/2002 : 18:56   BUR11729 Farrukh 
Modified LDTree absolute path to 48 (from 16) characters for IBIS.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.


***********************************************************************/

#ifndef ADDRESS_INCLUDED
#define ADDRESS_INCLUDED

#include "sysdef.h"

/* Define the addressing that is possible for any logical device. */
/* Note that the LD_Depth field is modifiable. This is used to determine */
/* if the address is at the point of execution. Each level decrements this */
/* and if the result is zero, that level is responsible for processing of */
/* the request.*/

/*********************************************************************

	A D D R E S S I N G  O F  A L L  T A R G E T S

 *********************************************************************/


#define 	MAX_PATH 48

typedef struct _LDTree 	/* structure size is 20 bytes */
{					     
/* 0x0000 */	USHORT		LD_cBranches;
/* 0x0002 */	USHORT		LD_Depth;
/* 0x0004 */	UCHAR		LD_AbsPath[MAX_PATH];
} LDTree, DPTR pLDTree;

typedef struct _LDNear 	/* structure size is 4 bytes */
{
/* 0x0000 */	UCHAR		LD_NearPath;
/* 0x0002 */ 	UCHAR		Padding[3];	/* padding for dword alignment */
} LDNear, DPTR pLDNear;

	

typedef struct _Sas_Device_Address       /* structure size is 20 bytes */
{
/* 0x0000 */    UCHAR           PortNumber;
/* 0x0001 */    UCHAR           PortState;
/* 0x0002 */    USHORT          Sas_Flags;
/* 0x0004 */    UCHAR           Sas_WorldWideID[16];   /* should be 8bytes, 8 padding */
} Sas_Device_Address, DPTR pSas_Device_Address;

#endif
