/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: athena.h

File Description
Defines structures for the transmission/receipt of commands for remote
cards.

***********************************************************************

History :
01/26/2004 : 11:43 PM   BURN11729 Burney
Updated to support 64bit addressing. Filds that are not used are now 
reserved. Only 1 SGElement is allocated and rest of the space is now
reserved.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.


***********************************************************************/

#ifndef ATHENA_INCLUDED
#define ATHENA_INCLUDED

#include "sysdef.h"

#define	DATA_IN			0
#define	DATA_OUT		1
#define	BIDIRECTIONAL	2

typedef struct _AthenaPkt DPTR pAthenaPkt;

typedef struct _AthenaPkt 				/* structure size is 368 bytes */
{
/* 0x0000 */	ULONG			Reserved1;			/* for driver use only	*/
/* 0x0004 */	LDTree			AP_Target;			/* fully qualified path to target	*/
/*	0x0018 */	UCHAR			AP_Direction;		/* data in or out */
/* 0x0019 */	UCHAR			AP_RetryCount;		/* retry count for AP_SCSICmd */
/* 0x001A */	UCHAR	 		Padding[2];    		/* padding for dword alignment */
/* 0x001C */	SCSICDB12		AP_SCSICmd;			/* command to be sent to target	*/
/* 0x0028 */	USHORT			AP_szSCSICmd;  		/* size of CDB; 6,10, 12   */
/* 0x002A */	USHORT			AP_Timeout;    		/* maximum time to complete request */
/* 0x002C */	USHORT			AP_Priority;		/* priority of this request	*/
/* 0x003E */	USHORT			AP_Status;			/* driver supplies			*/
/* 0x0030 */	AthenaSGTable	AP_SGTable;			/* apps supplies 1 element only */
/* 0x0040 */	AthenaSGElement AP_SGList;			/* for driver use only 		*/
#if (OSM_CPU_ADDRESS_SIZE == 64)
				ULONG			reserved2[120];		
#else
				ULONG			reserved2[126];
#endif
/* 0x0140 */	SCSISenseData	AP_Sense;  			/* driver supplies if any	*/
/* 0x0160 */	ULONG			Reserved3;			/* Back pointer, for driver use only */
/* 0x0164 */	ULONG			Reserved4;			/* for driver use only 		*/
/* 0x0168 */	ULONG			Reserved5;			/* driver use - if alloc of OSD fails */
/* 0x016C */	UCHAR			AP_OSReserved[4]; 	/* Scratch pad for the OS */
} AthenaPkt;

/* HACB sturcture for NWPA driver support */
typedef struct _HACB_IO_CONTROL DPTR PHACB_IO_CONTROL;

typedef struct _HACB_IO_CONTROL
{
	UCHAR	signature[5];
	PVOID	ioctlPkt;
	USHORT	status;
}HACB_IO_CONTROL;

										
#endif
