/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/*********************************************************************
                  CSMI.H

OSM or HCI will need to include this file in their code to interface
with CDA as per CSMI specifications.

This file is intended for CSMI supports.


History:

11/20/2003: Tam
Initial checkin.

*********************************************************************/

#ifndef CSMI_INCLUDED
#define CSMI_INCLUDED

#include		"osmcda.h" 

#ifdef _WIN32
#include		"miniport.h"

#if _STORPORT
#include                "storport.h"
#else
#include		"srb.h"
#endif
#include                "ntddscsi.h"
#endif

#include "sysdef.h"
#include "cdadef.h"
#include "csmisas.h"
#include "physif.h"

/* tt - Add support for NVRAM access */
#include "csmisasa.h"

#define  SAS_30_CONNECTRATE        30000
#define  SAS_15_CONNECTRATE        15000

/* CSMI Prototypes */

//#define HIM_NULL_TSH    ((HIM_TASK_SET_HANDLE HIM_PTR) 0) /*CHIM Defines it now*/


UINT32
CSMI_Initialize(
    pADAPTER_DEV_EXT    pAdapterDevExt
    );

VOID
CSMI_Queue(
    POSD                pOSD
    );

VOID 
CSMI_GetDriverInfo(
    pADAPTER_DEV_EXT    pAdapterDevExt, 
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetSCSIAddress(
    pADAPTER_DEV_EXT    pAdapterDevExt, 
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetDeviceAddress(
    pADAPTER_DEV_EXT    pAdapterDevExt, 
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32
CSMI_FirmwareDownload(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetControllerStatus(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,  
    PVOID               pBuffer
    );

VOID
CSMI_GetControllerConfig(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetPhyInfo(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32
CSMI_SetPhyInfo(
    pADAPTER_DEV_EXT    pAdapterDevExt,    
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32
CSMI_GetLinkErrors(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetSATASignature(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetRAIDInfo(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32
CSMI_GetRAIDConfig(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32    
CSMI_SMP_PassThru(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32      
CSMI_SSP_PassThru(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32    
CSMI_STP_PassThru(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID 
CSMI_DisablePhy(
	pAdapterDevice      pAD,
    POSD                pOSD,
	HIM_TASK_SET_HANDLE phyTSH 
	);

VOID
CSMI_DisablePhyComplete(
    HIM_IOB HIM_PTR     pIOB
    );

VOID
CSMI_SendInquiry(
    pIODescriptor       pIOD
    );

VOID
CSMI_Inquiry(
    POSD                pOSD,
    pSysDevice          pSD,
    PVOID               pInquiryBuffer
    );

VOID
PostCSMI_SMP_PassThru(
    pIODescriptor       pIOD 
    );

VOID
PostCSMI_PassThru(
    pIODescriptor                   pIOD 
    );

VOID
PostCSMI_SSP_PassThru(
    pIODescriptor       pIOD 
    );

VOID
PostCSMI_STP_PassThru(
    pIODescriptor       pIOD 
    );

UINT32
PostCSMI_GetLinkErrors(
    pIODescriptor       pIOD 
    );

VOID
PostCSMI_ResetLinkErrors(
    pIODescriptor       pIOD 
    );

VOID
PostCSMI_PassThru(
    pIODescriptor                   pIOD 
    );

VOID
SendSMPPassThru(
    pADAPTER_DEV_EXT                pAdapterDevExt,
    POSD                            pOSD,
	PCSMI_SAS_SMP_PASSTHRU_BUFFER   pPassThruBuffer,
	HIM_TASK_SET_HANDLE             TargetTSH
    ); 


HIM_UINT32
SMPSendPassThru(
    pADAPTER_DEV_EXT                pAdapterDevExt,
    POSD                            pOSD,
    PCSMI_SAS_SMP_PASSTHRU_BUFFER   pPassThruBuffer,
    HIM_TASK_SET_HANDLE             TargetTSH
    ); 



HIM_UINT32
SSPSendPassThru(
    pADAPTER_DEV_EXT                pAdapterDevExt,
    POSD                            pOSD,
    PCSMI_SAS_SSP_PASSTHRU_BUFFER   pPassThruBuffer,
    pPhysDevice                     pPD
    ); 

/*
 *  ++Start: TASKMANAGEMENT FUNCTIONS definition
 */
UINT32
CSMI_PHY_Control(
    pADAPTER_DEV_EXT         pAdapterDevExt,
    POSD                     pOSD,
    PVOID                    pBuffer
    );

VOID
PostCSMI_PHY_Control(
    pIODescriptor       pIOD 
    );

VOID
PostCSMI_Pattern_Generator(
    pIODescriptor       pIOD 
    );

VOID
PostCSMI_PHY_ControlLinkOrHardReset(
    pIODescriptor       pIOD 
    );


VOID
PostCSMI_SAS_TaskManagement(
    pIODescriptor                   pIOD 
    );


UINT32    
Send_CSMI_SAS_HardResetSequence(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

UINT32    
Send_CSMI_SAS_ResetLU(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );


UINT32    
Send_CSMI_SAS_AbortTaskSetORClearAca(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer,
    unsigned long       function
    );


UINT32    
Send_CSMI_SAS_AbortORQueryTask(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer,
    unsigned long       function
    );


UINT32    
CSMI_SAS_TaskManagement(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

/*
 *  --End: TASKMANAGEMENT FUNCTIONS definition
 */


UINT32
CSMI_GetConnectorInfo(
    pADAPTER_DEV_EXT         pAdapterDevExt,
    POSD                     pOSD,
    PVOID                    pBuffer
    );

#if  CDA_SUPPORTS_BIOS_FLASHING
/* tt - Add support for NVRAM access */
VOID
CSMI_GetADPTControllerConfig(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_GetNVSegmentProperties(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_WriteNVSegment(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
CSMI_ReadNVSegment(
    pADAPTER_DEV_EXT    pAdapterDevExt,
    POSD                pOSD,
    PVOID               pBuffer
    );

VOID
MapCHIMSegmentID(
    HIM_UINT32 HIM_PTR SegmentID);

/* tt - End of adding support for NVRAM access */
#endif /* CDA_SUPPORTS_BIOS_FLASHING */


/*  
    ControlCode 
    OSM needs to map their control codes to these.
*/
#define CDA_CSMI_SAS_GET_DRIVER_INFO        1 
#define CDA_CSMI_SAS_GET_CNTLR_CONFIG       2 
#define CDA_CSMI_SAS_GET_CNTLR_STATUS       3
#define CDA_CSMI_SAS_FIRMWARE_DOWNLOAD      4
 
#define CDA_CSMI_SAS_GET_RAID_INFO          10
#define CDA_CSMI_SAS_GET_RAID_CONFIG        11
 
#define CDA_CSMI_SAS_GET_PHY_INFO           20
#define CDA_CSMI_SAS_SET_PHY_INFO           21 
#define CDA_CSMI_SAS_GET_LINK_ERRORS        22
#define CDA_CSMI_SAS_SMP_PASSTHRU           23
#define CDA_CSMI_SAS_SSP_PASSTHRU           24
#define CDA_CSMI_SAS_STP_PASSTHRU           25
#define CDA_CSMI_SAS_GET_SATA_SIGNATURE     26 
#define CDA_CSMI_SAS_GET_SCSI_ADDRESS       27
#define CDA_CSMI_SAS_GET_DEVICE_ADDRESS     28

/*
    States for CSMI_SendInquiry state machine
*/

#define CDA_CSMI_SEND_INQ       1
#define CDA_CSMI_RECEIVE_INQ    2
#define CDA_CSMI_SEND_USN       3
#define CDA_CSMI_RECEIVE_USN    4

/*
    Security Access
*/
#define CSMI_SECURITY_NO_ACCESS             0x01
#define CSMI_SECURITY_RESTRICTED_ACCESS     0x02
#define CSMI_SECURITY_LIMITED_ACCESS        0x03
#define CSMI_SECURITY_FULL_ACCESS           0x04

#define CDA_CSMI_INQUIRY_BUFFER_SIZE        64
#define CDA_CSMI_SERIAL_NUMBER_SIZE         40

/*
    SMP Function definitions    
*/
#define SMP_FUNCTION_CONFIGURE_ROUTE_INFO   0x90
#define SMP_FUNCTION_PHY_CONTROL            0x91
 

#endif /* CSMI_INCLUDED */
