/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/*********************************************************************
                  CDADEF.H

OSM or HCI will need to include this file in their code to interface
with CDA as per the new specifications. All core files will need these
definitions.

This file is not intended for RAID Management applications.


History:

03/18/2005 : Shing-Kin Pun
-Added #define "calcd.h" to support Multi-Level RAID.

12/16/04 : TonyR
change memory requirements for 64bit as it will not install with 800KB of 
CDA_HIBERNATION_CUT_OFF_MEMORY_SIZE.  This is a temporary fix for this release.
Memory allocation will be changed in future program.

12/15/04 : TonyR
Reduce memory requirements by 50K to 800KB for CDA_HIBERNATION_CUT_OFF_MEMORY_SIZE.

10/13/2004 : Madhuresh
Adding 0x41E Non-RAID Device ID in the table and
adding 40KB extra in CDA_SIZEOF_CHIM_ETC_MEMORY for memory calculations

10/11/2004 : Madhuresh
Increased CDA_HIBERNATION_CUT_OFF_MEMORY_SIZE to 850K

10/02/2004 : Madhuresh
Added pOSD_OsmNext for exclusive use by OSM.

09/15/2004 : Farrukh
Added prototype for CDASaveCrashdumpPtrs()

08/21/2004 : Madhuresh
Added three constants (smaller) for size of memory. Particularly to help
Hibernation and crashdump.

08/12/2004 : 16:01 NAR20028 Manjunath
Added prototype for CdaResetInitializationFlag().

11/24/2003: Madhuresh
OSD_Status is now 8 bit field. In OSD, CMP and CDP, DeviceHandle related
fields are made PVOID.

11/19/2003: Madhuresh
pOSD_Next has been moved to the top of OSD temporarily.

11/19/2003: Madhuresh
Significant code differences exist due to transport being SAS or U320.
CHIM definitions for these show up much later in CHIM.H but we need
to know for SG entries what format does the h/w follow.
SET EXACTLY ONE OF THESE AS TRUE BEFORE MOVING AHEAD.
	CDA_FOR_U320
	CDA_FOR_SAS

11/17/2003: Madhuresh
Some more errors defined for HIM failures. OSD_DataTransferLength added.

11/03/2003: Tam
Add defines for AddSenseCode, IO directions and	Engine's pause/unpause

10/29/2003: Madhuresh
New data structure for mapping between PTL and DeviceHandle for OSM.
CEP_DeviceExtensionSize. Added constants for DMA Transfer related flags.

10/16/2003: Madhuresh
Adjusted name, version etc in CEP to match CSMISAS needs and be able
to push into standard INQUIRY string for Athena.
Removed CdaAllocOSD and CdaFreeOSD.

06/10/2003: Madhuresh
Initial checkin.

*********************************************************************/
#ifndef CDADEF_INCLUDED
#define CDADEF_INCLUDED

/* Include file definitions */

#include "osmcda.h"

#define	CDA_SUCCESS                 0
#define	CDA_FAILURE                 1
#define	CDA_REBOOT_REQUIRED         2
#define	CDA_RETRY					3

/*
	Significant code differences exist due to transport being SAS or U320.
	CHIM definitions for these show up much later in CHIM.H but we need
	to know for SG entries what format does the h/w follow.

	PLEASE SET EXACTLY ONE OF THESE BEFORE MOVING AHEAD.
*/


/*
	OSD->CDB->OpCode should contain the following value before calling
	CdaExecuteOSD for IOCTL path.

	This is a special op code consumed by CDA. It will inhibit OS from
	making IO calls using op code as this value. Pass through users can
	still use such an op-code.
*/
#define	CDA_EXECUTE_IOCTL           0xFE
#define	CDA_CDB_ATHENA_IOCTL        0x00
#define	CDA_ATHENA_FUNCTION_DUMMY	0xFF

#define	CDA_CDB_CSMI_IOCTL          0x01
#define	CDA_CDB_PSUITE_IOCTL        0x02

#define	CDA_SAS_ADDRESS_SIZE        8
#define	CDA_LUN_ADDRESS_SIZE        8

#define	CDA_NON_RAID_DEVICE_ID1     0x04129005
#define	CDA_NON_RAID_DEVICE_ID2     0x041E9005
#define	CDA_NON_RAID_DEVICE_ID3     0x04329005
#define CDA_NON_RAID_DEVICE_ID4     0x043E9005
#define	CDA_OP_MODE_LEAST_MEMORY    0x01
#define	CDA_OP_MODE_NO_RAID         0x02
#define CDA_OP_MODE_NO_RAID10       0x04

#define	CDA_SIZEOF_MALLOC_DURING_CRASH_HIBER 1024 * 60
#define	CDA_SIZEOF_CRASHDUMP_MEMORY	1024 * 128

#define CDA_IOB_RESERVE_PADDING     8
#define CDA_1K_ALIGNMENT            0x3ff

/*
	Size of inquiry info from device we will return in CdaDeviceProperties.
	This will be standard Vendor Id, Product Id, Product Revision fields.
	(byte offsets 8 to 35 of standard inquiry data)
*/
#define	CDA_DEVICE_NAME_SIZE        32

/*
	Product name as known to the user. e.g. ASC-39320D etc. Size is aligned
	to what we can return via Product Id in standard inquiry. Bytes 16-31.
*/
#define	CDA_PRODUCT_NAME_SIZE       16

/*
	Filename of the driver. Size is per dos format 8.3
*/

#define	CDA_DRIVER_NAME_SIZE        12
/*
	CSMISAS needs 81 byte null terminated string for
	Driver Description.
*/
#define	CDA_DRIVER_DESCRIPTION_SIZE 81

/*
	Space reserved in two key data structures for future.
*/
#define	CDA_CDP_RESERVED_SIZE       32
#define	CDA_CEP_RESERVED_SIZE       12

/*
	Lengths of main SCSI structures that CDA works with.
*/
#define	CDA_MAX_CDB_LENGTH          12
#define	CDA_MAX_SENSE_LENGTH        32

/*
	Use to notify CDA Engine to pause and unpause.
*/
#define	CDA_PAUSE					0
#define	CDA_UNPAUSE					1


/*
	Maximum RSU count
*/
#define	CDA_MAX_RSU_IOD_COUNT           (CDA_MAX_RAIDS_PER_ADAPTER*2)
#define	CDA_MAX_RSU_BUFFER_COUNT        32

#define CDA_DEVICE_PORT_RESET_TIMEOUT       32 //Seconds



/*
	OSD_Status.
	TDE - The following error definition values are
	taken from the CHIM definition defined in
	chimdef.h.
	We expose these to OSM as CDA_ASC_XXX error.
*/
#define CDA_ASC_IOB_BUSY                0x03
#define CDA_ASC_UNSUPPORTED_REQUEST		0x0B 
#define CDA_ASC_INVALID                 0x10 
#define	CDA_ASC_NO_RESPONSE				0x20

#define CDA_ASC_DEVICE_PORT_UNREACHABLE 0x22

/* TDE - The following errors need IOB to be retried */
#define CDA_ASC_PROTOCOL_ERROR			0x23
#define	CDA_ASC_CONNECTION_FAILED		0x24
/* end of retry error definition */

#define CDA_ASC_DATA_UNDERRUN           0x27  /* Data underrun wasdetected.       */
#define CDA_ASC_DATA_OVERRUN            0x28  /* Data overrun was detected.       */

#define	CDA_ASC_ABORT_FAILED			0x40
#define CDA_ASC_ABORT_NOT_FOUND			0x41  
#define CDA_ASC_ABORT_STARTED			0x42  
#define CDA_ASC_ABORT_ALREADY_DONE		0x43  
#define CDA_ASC_ABORTED_REQ_LUR			0x4C 

#define CDA_ASC_ABORTED_ON_REQUEST           0x50           
#define CDA_ASC_ABORTED_RESET_DEVICE_PORT    0x51   
#define CDA_ASC_ABORTED_RESET_PORT           0x52  
#define CDA_ASC_ABORTED_RESET_ADAPTER        0x53 
#define CDA_ASC_RESET_HARDWARE               0x54
#define CDA_ASC_ABORTED_RESET_LOGICAL_UNIT   0x55 
#define CDA_ASC_ABORTED_BY_LOGICAL_UNIT      0x56 
#define CDA_ASC_ABORTED_CLEAR_TASK_SET       0x57 
#define CDA_ASC_ABORTED_ABORT_TASK_SET       0x58 

#define CDA_ASC_HOST_ADAPTER_FAILURE    0x90 
#define CDA_ASC_PORT_FAILURE            0x91 

/* The following status codes do not map 1-1 to HIM_IOB_xxx */
#define CDA_ASC_RETRY_IOB				0xFE
#define CDA_ASC_FAILED_SENDING_TMF      0xFD
#define CDA_ASC_INVALID_TMF_REQ			0xFC
#define CDA_ASC_ABORTED_REQ_LUR         0x4C 
#define CDA_ASC_IOB_TASK_SET_FULL       0x28  /* Make same as SCSI_STAT_QUEUEFULL */

/*
	OSD_Direction.
*/
#define	CDA_DIRECTION_DATA_IN			0x00
#define	CDA_DIRECTION_DATA_OUT			0x01
#define	CDA_DIRECTION_BIDIRECTIONAL		0x02

#include	"sg.h"	/* OSD requires Adaptec Standard SGList format declaration */


/******************************************************
    DEVICE UNIQUE NAME
 ******************************************************/
#define     IDENTIFIER_TYPE_NAA                             0x03
#define     IDENTIFIER_TYPE_SATA_MODEL_AND_SERIAL_NUMBER    0x08
#define     ASSOCIATION_BY_LOGICAL_UNIT                     0x00

typedef struct _CDA_INQUIRY_IDENTIFICATION_DESCRIPTOR
        CDA_INQUIRY_IDENTIFICATION_DESCRIPTOR, 
        *PCDA_INQUIRY_IDENTIFICATION_DESCRIPTOR;

typedef struct _CDA_SAS_SATA_INQUIRY_PAGE_DESCRIPTOR
        CDA_SAS_SATA_INQUIRY_PAGE_DESCRIPTOR, 
        *PCDA_SAS_SATA_INQUIRY_PAGE_DESCRIPTOR;

typedef struct _PIV_ASSOCIATION_IDTYPE 
        PIV_ASSOCIATION_IDTYPE, *PPIV_ASSOCIATION_IDTYPE;

typedef struct _CDA_SAS_INQUIRY_UINT_SERIAL_NUMBER
        CDA_SAS_INQUIRY_UINT_SERIAL_NUMBER, 
        *PCDA_SAS_INQUIRY_UINT_SERIAL_NUMBER;

#ifndef SCO_DDI8
typedef struct _PIV_ASSOCIATION_IDTYPE
#else		/* SCO_DDI8 */
struct _PIV_ASSOCIATION_IDTYPE
#endif	/* SCO_DDI8 */
{
    UINT8   idtype:4;
    UINT8   association:2;
    UINT8   reserved:1;
    UINT8   piv:1;
};

#ifndef SCO_DDI8
typedef struct _CDA_INQUIRY_IDENTIFICATION_DESCRIPTOR
#else		/* SCO_DDI8 */
struct _CDA_INQUIRY_IDENTIFICATION_DESCRIPTOR
#endif	/* SCO_DDI8 */
{
    UINT8                   protocolID_codeSet;
    PIV_ASSOCIATION_IDTYPE  piv_association_idType;
    UINT8                   reserved;
    UINT8                   idLength;
    UINT8                   luName[1];
};
#ifndef SCO_DDI8
typedef struct _CDA_SAS_SATA_INQUIRY_PAGE_DESCRIPTOR
#else		/* SCO_DDI8 */
struct _CDA_SAS_SATA_INQUIRY_PAGE_DESCRIPTOR
#endif	/* SCO_DDI8 */
{
  UINT8     peripheral_device;
  UINT8     pageCode;
  UINT16    pageLength;
  CDA_INQUIRY_IDENTIFICATION_DESCRIPTOR descriptor;
};

#ifndef SCO_DDI8
typedef struct _CDA_SAS_INQUIRY_UINT_SERIAL_NUMBER
#else		/* SCO_DDI8 */
struct _CDA_SAS_INQUIRY_UINT_SERIAL_NUMBER
#endif	/* SCO_DDI8 */
{
  UINT8     peripheral_qual_devicetype;
  UINT8     pageCode;
  UINT8     reserved;
  UINT8     pageLength;
  UINT8     luName[1];
};

typedef struct _UniqueName
{
    union
    {
        UINT8   GUID[8];            /* For Raid: Not Used */
        UINT8   SASWWN[16];          /* Used for SAS or SATA WWN to store : Copied from device Page 83*/
        UINT8   TargetPortName[8];
        UINT8   LunName[68];    /* driver is using this for SATA */
        UINT8   PortProfileWWN[8]; /* if LunName is not 8 byte long, Profile WWN will be used for SATA */
    }Name;
    UINT8   NameFormat;
    #define GUID_FORMAT                     0x01
    #define SAS_WWN_FORMAT                  0x02
    #define TARGET_PORT_NAME_FORMAT         0x03
    #define LUN_NAME_FORMAT            0x04
    #define PORT_PROFILE_WWN_FORMAT    0x05
    UINT8   Length;
    UINT8   Reserved[2]; /* to make it 8 byte align*/
}UniqueName, DPTR pUniqueName;

#define PTL_MAP_SIZE    112
typedef struct  _DeviceUniqueMap
{
    UniqueName  DevName;
    UINT16      PathId;
    UINT16      TargetId;
    UINT8       BootdevFlag;
    UINT8       Reserved[3];/* to make it 8 byte allign */
}DeviceUniqueMap, DPTR pDeviceUniqueMap;



/*************************************************************************

	CDA receives OSD as the main structure to execute any command.

*************************************************************************/
typedef struct _OSD {
    struct _OSD    *pOSD_Next ;
    struct _OSD    *pOSD_OsmBack; /* ptr to previous osd in the list */
    struct _OSD    *pOSD_OsmNext;/* Used only by OSM */

    PVOID           OSD_DeviceHandle ;
    UINT8           OSD_Direction ;
    UINT8           OSD_RetryCount ;
    UINT8           OSD_SenseLen ;
    UINT8           OSD_cCDB ;
    UINT8           OSD_CDB[ CDA_MAX_CDB_LENGTH ] ;
    UINT8           OSD_Sense[ CDA_MAX_SENSE_LENGTH ] ;
    void            *pOSD_Sense;  /* OSM will use this pointer to access sense data */
                                  /* CDA only modify this pointer for removable media */
    SGTable         OSD_SGTable ;

	UINT32			OSD_DataTransferLength ;
    void           *OSD_pSGTable ;
#ifdef SCO_DDI8
    void           *OSD_pDataBuffer ;
#else // SCO_DDI8
    DATA_N_PTR     *OSD_pDataBuffer ;
#endif // SCO_DDI8
    FNPTR           pOSD_Post ;
    void           *OSD_DeviceExtension ;
    void           *pOSRequest ;
	/*
		Fields below this line are for internal use of CDA RAID engine.
		OSM or HCI
	*/
	FNPTR           pOSD_Retry ;
	UINT32          OSD_StartBlock ;    /* IO starting block */
	UINT32          cOSD_Blocks ;       /* Total block count.*/
	UINT8			OSD_R1WriteControl ;/* Now used for Raid1 Write only.*/
	UINT8			cOSD_Reference ;    /* Call post when 0 */
	UINT8			iMember ;           /* Member number - used only by IssueStartStopUnitToPairs */
	UCHAR           OSD_Priority ;      /* Priority for array management */

    UINT16          OSD_Status ;
	UINT16          cOSD_TCount ;       /* OSD_cTCount ;        Number of ticks for operation to complete. */

    UINT16          OSD_OurTimeout;     /* Used for timing out the SRB */
    UCHAR			OSD_CustomFlag;		/* TDE - Indicates OSM processing I/Os with busy Logic */
    UCHAR           bOSD_fIOCtlRequest;

    UCHAR           OSD_QueueTag;
	UCHAR			SendStopUnitCmd;	/* Send stop unit command to device when shutdowning system */
    UCHAR           Padding[2];

	//
	// Following fields are additional fields for OSDecriptor only.
	//
    void           *pOSD_IODChain;		/* TDE - Needed to keep track of all IODs allocated */
										/* for this OSD in case TMF is issued by OSM.   	*/
	BYTE			MemberIOStat;

} OSD, *POSD ;

/* Definition of CDP_State */
#define DEVICE_OFFLINE  0
#define DEVICE_ONLINE	1

/*************************************************************************
	CDA returns the configuration of its logical devices using this
	data structure.
*************************************************************************/
typedef struct _CDA_DEVICE_PROPERTIES
{
    PVOID           CDP_DeviceHandle ;  /* IO handle */
    PVOID           CDP_DevUniqueHandle ;   /* CHIM Target TSH */
    UINT32          CDP_Capacity ;
    UINT32          CDP_Path ;
    UINT32          CDP_TargetId ;
    UINT8           CDP_Lun[ CDA_LUN_ADDRESS_SIZE ] ;
    UINT8           CDP_WWN[ CDA_SAS_ADDRESS_SIZE ] ;
    UINT8           CDP_ConnectedToWWN[ CDA_SAS_ADDRESS_SIZE ] ;
    UINT16          CDP_ConnectedToPortNumber ;
    UINT16          CDP_ConnectedToPhyId ;
    UINT8           CDP_DeviceType ;
    UINT8           CDP_Exposed ;
    UINT8           CDP_State ;		
    UINT8           CDP_MemberIndex ;   /* Member Index */
    UINT32          CDP_MaxSpeed ;
    UINT32          CDP_CurrentSpeed ;
    UINT32          CDP_BusWidth ;
	UINT32			Padding;
    PVOID           CDP_ParentHandle ;  /* Member's parent which is the Sysdevice handle of a RAID device */

    UINT8           CDP_Name[ CDA_DEVICE_NAME_SIZE ] ;
    UINT8           CDP_ReservedForFutureUse [ CDA_CDP_RESERVED_SIZE ] ;
    UniqueName      CDP_UniqueName;

} CDA_DEVICE_PROPERTIES, *PCDA_DEVICE_PROPERTIES ;
/*************************************************************************
	CDA returns the configuration of its RAID engine
	using this data structure.
*************************************************************************/
typedef struct _CDA_BAR_INFO
{
	DATA_N_PTR		BaseAddr;
	BOOL			MemoryMapped;
} CDA_BAR_INFO;

typedef struct _CDA_ENGINE_PROPERTIES
{
    UINT32          CEP_DeviceExtensionSize ;  /* CDA will provide this size to OSM for malloc */
    UINT32          CEP_ChipId ;               /* OSM MUST fill this */
	CDA_BAR_INFO    CEP_BAR[ 6 ] ;             /* OSM MUST fill this */
	UINT8           CEP_BusNumber ;            /* OSM should fill this if available */
	UINT8           CEP_SlotNumber ;           /* OSM should fill this if available */
	UINT16           CEP_SubVendorID ;           /* OSM should fill this if available */
	UINT16           CEP_SubsystemID ;           /* OSM should fill this if available */
    UINT8           CEP_ProductName[ CDA_PRODUCT_NAME_SIZE ] ;    /* OSM should fill this */
    UINT8           CEP_DriverName [ CDA_DRIVER_NAME_SIZE ] ;     /* OSM should fill this */
	UINT8           CEP_DriverDescription[ CDA_DRIVER_DESCRIPTION_SIZE ] ;/* OSM should fill this */
	USHORT          CEP_usMajorRevision ;
	USHORT          CEP_usMinorRevision ;
	USHORT          CEP_usBuildRevision ;
	USHORT          CEP_usReleaseRevision ;    /* 5 fields above are as per CSMISAS needs */

    UINT32          CEP_IoctlBufferSize ;
    UINT32          CEP_TimerInterval ;
    UINT32          CEP_MaxCommandsPerAdapter ;
    UINT32          CEP_MaxSGBreaks ;          /* to be replaced by CEP_MaximumSGList */
    UINT32          CEP_MaxOsds ;
    UINT32          CEP_MaxIods ;
    UINT32          CEP_OperationMode ;
    UINT32          CEP_RaidBufferSize ;
    UINT32          CEP_RaidBufferCountPerSystem ;
    UINT32          CEP_MemorySize ;           /* CDA needs this much non-contiguous memory */
    UINT32          CEP_DmableMemorySize ;     /* CDA needs this much contiguous memory */
	UINT32			Padding;

    UINT16          CEP_MaxNumberOfDevices ;
    UINT16          CEP_MaxNumberOfPhys ;
    UINT16          CEP_MaxNumberOfPorts ;
    UINT16          CEP_ITLNexusLossTimer ;

    UINT8           CEP_MaximumSGList ;
    UINT8           CEP_MaxTags ;              /* How many commands to send to each Lu (disk/target) */
    UINT8           CEP_CoercionPolicy ;       /* Method applied for Disk Size Coercion? */
    UINT8           CEP_MdStampingPolicy ;     /* OEM preference on stamping the MD with user warning or never stamping it? */

    UINT8           CEP_MemoryMapped ;         /* Program hardware/chim using memory mapping or IO mapping */
    UINT8           CEP_ForceFlushOnSata ;     /* Some SATA disks do not flush the cache correctly. Do we force? */
    UINT8           CEP_SupportCsmi ;          /* Per csmisas specs */
    UINT8           CEP_SuppportPsuite ;       /* PSUITENT ioctls are enabled or not */

	UINT8           CEP_SupportCoercion ;      /* Disk Size Coercion is supported? */
	UINT8           CEP_SupportHotSwapRebuild ;/* HotSwapRebuild Support is required? */
	UINT8           CEP_SupportCoalesce ;      /* Command Coalescing? */
	UINT8           CEP_IrqNumber ;      

    UINT8			CEP_TicksPerSec;           /* TDE */
	UINT8			CEP_DevIORetryCnt;         /* TDE */
	UINT8			CEP_SupportAutoReconstruct;/* Auto Rebuild support */
	/*UINT8			CEP_Reserved;*/
    UINT8			CEP_MaxTargetIdAllowed;
    UINT8           CEP_FaultToleranceTimeout;
    UINT8           CEP_PathFirst;
    UINT8           CEP_MaxLuns;
    UINT8           CEP_RaidPath;
    UINT8           CEP_ReservePaths;
    UINT8           CEP_VSAModePath;
    UINT8           CEP_OsmMaxPath;
    UINT16          CEP_MaxTargetsAllowed;
    UINT32          CEP_PTTableMemorySize; /* Out Param*/
    UINT32          CEP_LunTableMemorySize;/* Out Param*/
    UINT8           CEP_EnableBootDeviceMap;/* Added to expose the boot device at PT 0,0*/
#ifdef SES2_DEVICE
    UINT8           CEP_ExposeEnclosure;
    UINT8           CEP_EnableLED;
#endif
    UINT8           CEP_MaxSataQDepth;
    UINT8           CEP_PerformanceKnobe;

    UINT8           CEP_ReservedForFutureUse[ CDA_CEP_RESERVED_SIZE ] ;

	BOOL			CEP_AP_CanCross4G;
	UINT32			CEP_AP_MaxSGByteCount;

} CDA_ENGINE_PROPERTIES, *PCDA_ENGINE_PROPERTIES ;
/*************************************************************************
	CDA receives the allocated memory address and length using this
	structure.
*************************************************************************/
typedef struct _CDA_OS_MEMORY_REGIONS
{
#if _LME
    DATA_N_PTR      MEM_pDmableRegion ;
    DATA_N_PTR      MEM_pNonDmableRegion ;
    DATA_N_PTR      MEM_pPTTableNonDmableRegion;
    DATA_N_PTR      MEM_pLunTableNonDmableRegion;
    UINT32          MEM_DmableRegionSize ;
    UINT32          MEM_NonDmableRegionSize ;	
    UINT32          MEM_PTTableNonDmableRegionSize;
    UINT32          MEM_LunTableNonDmableRegionSize;
#else
    PVOID           MEM_pDmableRegion ;
    UINT32          MEM_DmableRegionSize ;
    PVOID           MEM_pNonDmableRegion ;
    UINT32          MEM_NonDmableRegionSize ;	
    PVOID           MEM_pPTTableNonDmableRegion;
    UINT32          MEM_PTTableNonDmableRegionSize;
    PVOID           MEM_pLunTableNonDmableRegion;
    UINT32          MEM_LunTableNonDmableRegionSize;
#endif
} CDA_OS_MEMORY_REGIONS, * PCDA_OS_MEMORY_REGIONS ;
/*************************************************************************
	To perform a DMA, CDA uses the following structure to send info.
*************************************************************************/
/*
	Bit positions in DMA_flags below
*/
#define	CDA_TRANSFER_TO_IOP     0x01  /* The destination address is in local memory. Only applies to IOP based RAID solution.*/
#define	CDA_TRANSFER_FROM_IOP   0x02  /* The source address is in local memory. Only applies to IOP based RAID solution.*/
#define	CDA_TRANSFER_TO_SG      0x04  /* The destination address points to standard Adaptec SG list. Transfer using the SG list.*/
#define	CDA_TRANSFER_FROM_SG    0x08  /* The source address points to standard Adaptec SG list. Transfer using the SG list.*/
#define	CDA_TRANSFER_BOTH_SG    0x10  /* Both addresses point to standard Adaptec SG lists. Transfer using the SG list.*/
#define	CDA_TRANSFER_IN_BG      0x20  /* CDA expects a DMA completion interrupt and so it can work on other thread while DMA happens in the background.*/

typedef struct _CDA_DMA_PARAMETERS
{
    PVOID           DMA_DeviceExtension ;
    UINT32          DMA_flags ;
    UINT32          DMA_srcAddressLow ;
    UINT32          DMA_srcAddressHigh ;
    UINT32          DMA_dstAddressLow ;
    UINT32          DMA_dstAddressHigh ;
    UINT32          DMA_byteCountLow ;
    UINT32          DMA_byteCountHigh ;
    UINT32          Padding;
    FNPTR           DMA_pXTransferDataComplete ;

} CDA_DMA_PARAMETERS ;
/*
   CDA mapping flags used for mapping PTL to/from DeviceHandle
*/
#define CDA_MAP_PTL_TO_DEVICEHANDLE 0
#define CDA_MAP_DEVICEHANDLE_TO_PTL 1

/*************************************************************************
 To perform a PTL to DeviceHandle mapping and vice versa, CDA uses the 
 following structure to send request to OSM .
*************************************************************************/
typedef struct _CDA_MAPPING_PARAMETERS
{
    UINT32          CMP_flags ;
    UINT32          Padding;
    PVOID           CMP_DeviceHandle ;
    UINT32          CMP_Path ;
    UINT32          CMP_TargetId ;
    UINT8           CMP_Lun[ CDA_LUN_ADDRESS_SIZE ] ;

} CDA_MAPPING_PARAMETERS, * PCDA_MAPPING_PARAMETERS ;
/*************************************************************************

	Exported Functions

*************************************************************************/
/*
	Classical CDA Sysinit for data and hardware initializations.
*/
UINT32
CdaSysinit(
    PVOID                   DeviceExtension,
    PCDA_OS_MEMORY_REGIONS   pmem ) ;

/*
	ISR to call when IO completion interrupt comes. CDA will in
	turn call CHIM's front end ISR and if its valid (our)
	interrupt, it would call the back end ISR.
*/
UINT8
CdaIoIsr( PVOID  DeviceExtension ) ;
/*
	It is essential that the OSM or OS call this function at
	regular interval as specified in CDA_ENGINE_PROPERTIES by
	the CEP_TimerInterval field.

	This is used for device polling, hot swap rebuild feature
	and command time out mechanism.
*/
void
CdaTimerIsr( PVOID  DeviceExtension ) ;

void
CdaTimerTask( PVOID  DeviceExtension ) ;

/*
	CdaExecuteOSD analyzes the parameters passed in the OSD
	data structure and starts a command using core functions.

	The call-back routine (function pointer kept in pOSD->pOSD_Post
	field) is executed for any failure to start the command.

	When command is completed by the core layers, by way of an
	interrupt from the underlying bus or direct execution,
	the call-back routine will be called with appropriate
	pOSD->OSD_Status and pOSD->OSD_Sense

	If debug_level is non-zero, this function will perform a
	NULL pointer check and trap.
*/
void
CdaExecuteOSD( POSD    pOSD ) ;

/*
	CdaAbortOSD attempts to kill a command that this OSD describes

	Currently not supported.
*/
UINT32
CdaAbortOSD( POSD     pOSD ) ;
/*
	CdaPauseUnpause is related to the background I/Os being performed
	by CDA for RAID rebuild or verify or for any internal operations
	on the devices without the knowledge of the operating system.

	Depending on the parameter, it will either put all I/Os on hold
	or restart such I/Os.
*/
void
CdaPauseUnpause(
    PVOID                   DeviceExtension,
    UINT32                  PauseFlag ) ;

/*
	The I/O channel under control of CDA is reset using this function,
	should the host ever need to do so.

	Currently not supported.
*/
void
CdaChannelReset(
    PVOID                   DeviceExtension,
    UINT32                  Channel ) ;


/*
	CdaShutdown issues a meta-data write (to notify safe shutdown) on
	all members of all RAIDs and a disk flush cache command to all
	disks controlled by CDA.

	This function should be called when OS unloads the drivers or issues
	a shutdown call. This implies there will be no more I/O or IOCTL
	calls from the OS or any application.
*/
void
CdaShutdown( POSD     pOSD ) ;

void
CdaShutdownOneSysDevice( POSD     pOSD ) ;

void
CdaShutdownDuringCrashTime( POSD     pOSD ) ;


/*
	CdaGetNextDevice picks up information about the logical devices
	under control of CDA RAID engine and passes to OSM or HCI for
	use by upper layers.

	If this function is called for the first time, it gets the first
	device from its list. Subsequent calls get the next nodes until
	all devices finish.
	
	When this function returns error, the device index is reset to zero.
	CDA_DEVICE_PROPERTIES contains information about the discovered
	logical devices, their SAS (WWN) and Parallel SCSI (Channel, Target)
	addresses and virtual handles to represent those logical devices.
*/
PVOID
CdaGetNextDevice (
    PVOID   DeviceExtension,
    PVOID	DeviceHandle,
	UCHAR	*Count);


/*
	CdaGetDeviceProperties gets the information page of a device.
*/
UINT32 
CdaGetDeviceProperties (
    PVOID                   DeviceHandle,
	USHORT					MemberIndex,
    PCDA_DEVICE_PROPERTIES  pCdaDeviceProperties
  ) ;

VOID
CdaSetPTLMapping (
	PVOID	DeviceHandle,
	PCDA_DEVICE_PROPERTIES  pCDP);

VOID UpdatePTLMapping(PVOID pRSIOD);
VOID PTLMapReadComplete(PVOID pRSIOD);
VOID PTLMapWriteToNextMember(PVOID pRSIOD);
VOID PTLMapWriteComplete(PVOID pRSIOD);


/*
	CdaGetEngineProperties supplies all configurable parameters of CDA.
	Some of these determine the performance or overall RAID engine
	feature support

	This function provides information about the capabilities of RAID
	engine to upper layers. It should be called before performing
	CdaSetEngineProperties.
*/
UINT32 
CdaGetEngineProperties (
    PVOID                   DeviceExtension,
    PCDA_ENGINE_PROPERTIES  pCdaEngineProperties
  ) ;

/*
	CdaSetEngineProperties picks up information about the logical devices
	under control of CDA RAID engine and passes to OSM or HCI for use
	by upper layers.

	This is the way OSM or HCI can modify the behavior, performance and
	some specific features that the CDA RAID engine would work with.

	Some features may not take effect immediately and require a restart
	of the CDA RAID engine (may be complete system reboot).
*/
UINT32 
CdaSetEngineProperties (
    PVOID                   DeviceExtension,
    PCDA_ENGINE_PROPERTIES  pCdaEngineProperties
  );

PVOID
CdaAllocOSDs (
	PVOID pCDADevExt,
	USHORT cOSDs);

VOID
CdaFreeOSD (
	PVOID pCDADevExt,
	PVOID pOSD);
#if 1
VOID
CdaTMFRequest (
	PVOID pAbortOSDesp,
	PVOID DeviceHandle,
	UCHAR OperationFlag);
#else
CdaTMFRequest (
	PVOID pAbortOSDesp);
#endif

UCHAR
CdaGetDeviceType(PVOID DeviceHandle);

/**********************************************************************
Function Name CdaStopAdapter

Function Description
Wait for all commands on the adapter to complete and quiesce the channel

Input Parameters
    Adapter Device Extension

Output Parameters
	CDA_SUCCESS or
	Coded Errors.
**********************************************************************/
UINT32
CdaStopAdapter( PVOID pCDADevExt ) ;

/**********************************************************************
Function Name CdaRestartAdapter

Function Description
This function will resume the adapter to its normal funcationality 
after system is awaken from S1, S2, S3, and S4 states. 

Input Parameters
    Adapter Device Extension

Output Parameters
	CDA_SUCCESS or
	Coded Errors.
**********************************************************************/
UINT32
CdaRestartAdapter( PVOID pCDADevExt ) ;

/**********************************************************************
Function Name CdaSaveCrashdumpPtrs

Function Description
It saves virtual and physical pointers of uncached memory for crashdump
driverin OCM. Crashdump driver will retrieve these pointer during its
initialization.
.
Input Parameters
    Adapter Device Extension

Output Parameters
	CDA_SUCCESS or
	Coded Errors.
**********************************************************************/
UINT32
CdaSaveCrashdumpPtrs( PVOID pAdapterDevExt );

/***********************************************************************
 File cda.c
***********************************************************************/

UINT32
CdaRestartAdapterCleanup(PVOID pAdptDevExt);

UINT32
CdaResetInitializatonFlag(PVOID pAdapterDevExt);

UINT32
CdaSetInitializatonFlag(PVOID pAdapterDevExt);

UINT32
CdaIntiTimeDiscoverDevices(PVOID pAdptDevExt);

UINT32
CdaGetPTMap (
    PVOID       pAdptDevExt,
    PVOID       pDevUniqueName,
    UCHAR       BootDevFlag
    );

UINT32
CdaSetPTMap (
    PVOID        pAdptDevExt,
    PVOID        pDevUniqueName,
    USHORT       PathId,
    USHORT       TargetId,
    UCHAR        BootDevFlag
    );

VOID CdaUpdatePTMapBootFlag(PVOID pAdapterDevExt);

UINT32
CdaRemovePTFromPTMap (
    PVOID        pAdptDevExt,
    PVOID        pDevUName,
    USHORT       PathId,
    USHORT       TargetId
    );

#define CDA_DEVICE_DEFAULT_IO_QUEUE_DEPTH       8
//
// The following is the queuedepth of a SATA1 RAID array, 
// this should be a number between (8 thru 16)
// it is also used in Windows to determine to call NextRequest or NextLuRequest 
// to avoid a SCSIPORT issue dealing with performance
//  
#define CDA_SATA1_ARRAY_Q_DEPTH                 16
#define CDA_DEVICE_UNDEFINED_IO_QUEUE_DEPTH     0

#define CDA_PERFORMANCE_KNOB_NORMAL     0x01
#define CDA_PERFORMANCE_KNOB_HIGH       0x02

/*
 *
 *        PROBLEMS:
 *        1. Insufficient IODCount and mismatched queueDepth, causing
 *           multiple AllocIODs failures and subsequent timeouts.
 *        2.
 *
 *        To resolve this problem:
 *
 *        1. Increase the amount of memory available for IODs
 *           We will reclaim 128K from memory reserved for CRASH DUMP
 *           that was not used during runtime. This memory area will be
             reclaimed by CRASH DUMP.
 *        2. Derive CDA_MAX_IODCOUNT from the maximum memory available
 *           for IODs.
 *        3. Derive QueueDepth from CDA_MAX_IODCOUNT, instead of the
 *           other way around.
 *
 *        For CDA_V495A10?:
 *
 *        CDA_MAX_MEMORY_FOR_IODS = 128K(minimum) + 128K(reclaimed & shared
 *                                                       with CrashDump)
 *                                = 256K 
 *
 *        CDA_MAX_IODCOUNT = CDA_MAX_MEMORY_FOR_IODS / SIZE_ALLOC_PER_IOD
 *
 *        SIZE_ALLOC_PER_IOD  = SIZEOF_IOD + SIZE_FOR_SGLIST
 *          For 32-Bit Driver = 488 + 256 = 760
 *          For 64-Bit Driver = 680 + 256 = 936
 *
 *                                 [------ MAX_IODCOUNT ------]
 *	  CDA_MAX_MEMORY_FOR_IODS  32-Bit Driver  64-Bit Driver
 *        -----------------------  -------------  -------------
 *         256K                       344             936
 *        
 *        Stripe  MAX_IODs_PER  MaxRAID_10  [--- CDA_MAX_CMD_QDEPTH ---]
 *        Size    _RAID_10      _SUPPORTED  32-Bit Driver  64-Bit Driver
 *        ------  ------------  ----------  -------------  -------------
 *        16K       10              2           17
 *        32K	     6              2           28
 *        64K        4              2           43
 *
 *        For future releases, we SHOULD dynamically re-adjust the
 *        CDA_MAX_CMD_QDEPTH based on current configuration.
 */


#define CDA_MAX_IODS_PER_RAID10			4



UINT32
CdaGetQueueDepth (
    PVOID        pAdptDevExt,
    PVOID        DeviceHandle
    );

UINT32
CdaUpdateMetaData( PVOID pAdapterDevExt );

/*************************************************************************

	Physical layer (Backend) types

*************************************************************************/
enum CDA_BACKEND_TYPES
{
    CDA_PHYS_SCSI2,
    CDA_PHYS_SSA,
    CDA_PHYS_FC,
    CDA_PHYS_P1394,
    CDA_PHYS_SAS,
    CDA_PHYS_ATA,
    CDA_PHYS_SATA
} ;
/*************************************************************************

	These events are sent to OSM for hot plug addition and removals

*************************************************************************/
enum CDA_NOTIFY_EVENTS
{
    CDA_NOTIFY_EVENT_DEVICE_ADDED,
    CDA_NOTIFY_EVENT_DEVICE_REMOVED,
    CDA_NOTIFY_EVENT_SUSPEND_IO,
    CDA_NOTIFY_EVENT_RESUME_IO,
	CDA_NOTIFY_EVENT_BUS_RESET,
	CDA_NOTIFY_EVENT_RAID_STATUS_CHANGE,
	CDA_NOTIFY_EVENT_AD_OFFLINE,
	CDA_NOTIFY_EVENT_DEVICE_ONLINE,
	CDA_NOTIFY_EVENT_DEVICE_OFFLINE,
	CDA_NOTIFY_EVENT_REPORT_DEVICE,
    CDA_NOTIFY_EVENT_ABORT_TASK,
    CDA_NOTIFY_EVENT_RESET_DEVICE_PORT,
    CDA_NOTIFY_EVENT_RESET_OCCURING_ON_ADAPTER,
    CDA_NOTIFY_EVENT_RESET_OCCURING_ON_LOGICAL_UNIT,
    CDA_NOTIFY_EVENT_RESET_OCCURING_ON_DEVICE_PORT
} ;


VOID XOSMWritePCIConfigurationByte(
	PVOID pCDADevExt ,
	UINT32 byteOffset,
	UINT8 value);

VOID XOSMWritePCIConfigurationWord(
	PVOID pCDADevExt ,
	UINT32 byteOffset,
	UINT16 value);

VOID XOSMWritePCIConfigurationDword(
	PVOID pCDADevExt ,
	UINT32 byteOffset,
	UINT32 value);

UINT32 XOSMReadPCIConfigurationByte(
	PVOID pCDADevExt ,
	UINT32 byteOffset);

UINT32 XOSMReadPCIConfigurationWord(
	PVOID pCDADevExt ,
	UINT32 byteOffset);

UINT32 XOSMReadPCIConfigurationDword(
	PVOID pCDADevExt ,
	UINT32 byteOffset);

void XNotify(
    PVOID	pCDADevExt,
	ULONG	event,
    PVOID	pDeviceHandle,
	USHORT  MemberIndex);

DATA_N_PTR  XReturnPhysAddr ( 
	PVOID 	            	pCDADevExt,
	UCHAR DPTR			    virtualAddress,
    PVOID                   pOSRequest); 

#ifdef SRB_TO_IOD_SGLIST
USHORT XGetSGList (
    PVOID pOSRequest, 
    PVOID psg);
#endif
DATA_N_PTR  XReturnPhysAddrAndLenght ( 
	PVOID 	            	pCDADevExt,
	UCHAR DPTR			    virtualAddress,
	UINT32 DPTR             length,
    PVOID                   pOSRequest); 


VOID	XMicrosecondDelay(PVOID pAdapterDevExt, UINT32 microSeconds);

ULONG XMemCmpDWord (
	ULONG   DPTR	pDest,
	ULONG   DPTR	pSrc,
	ULONG   nBytes);

ULONG XMemCmp (
	UCHAR   DPTR	pDest,
	UCHAR   DPTR	pSrc,
	ULONG   nBytes);

VOID	XMemCpy (
	UCHAR   DPTR dest,
	UCHAR   DPTR src,
	ULONG   n );

VOID XMemZero (
	UCHAR DPTR	bytePointer,
	ULONG		bytes );



UINT32 XBuildOSAdapterDevices(PVOID pAdapterDevExt);

/* TDE- changed return type */
//VOID XMapPTLandDeviceHandle(PVOID pCDADevExt, PCDA_MAPPING_PARAMETERS CDAStruct);
BOOL XMapPTLandDeviceHandle(PVOID pCDADevExt, PCDA_MAPPING_PARAMETERS CDAStruct);

VOID
XOSMDelay(
    VOID           *osmAdapterContext,
    ULONG          microSeconds
    );


#endif

