/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: CDAPTMAP.h

File Description
	This file deals with the various PT mapping schemes.


File Create: Tarun Singh,                              Dated: 01/27/2005

************************************************************************/


//
// Lun Device structure information
//
typedef struct _LUN_DEVICE
{
    UINT8       DevState;
    UINT8       LunNumber;
    UINT8       Padding[6]; // Added make the structure 8 byte align
    PVOID       pIOHandle;  // Stores the SD handle to issue I/O
    PVOID       pNext;
    PVOID       pPrev;
}LUN_DEVICE, *PLUN_DEVICE;




//
// PTMap structure information
//
typedef struct _PTSLOT_DATA
{
    UINT8       SlotState;
    UINT8       DeviceTMFPendingFlag;
    UINT8       Padding[6]; // Added make the structure 8 byte align
    UINT8       WWN[8];
    PVOID       DevUniqueHandle;
    PLUN_DEVICE  pLunDevice;
}PTSLOT_DATA, *PPTSLOT_DATA;



//
// Lun memory and related information
//
typedef struct  _LUN_DATA
{
    PVOID       pFreeListHead;
    PVOID       pFreeListTail;
    DATA_N_PTR  pLunMemoryPtr;
    ULONG       MaxLunListSize;
    ULONG       Padding; // Added to make the structure 8 byte align
}LUN_DATA, *PLUN_DATA;



typedef struct _IO_HANDLE
{
    PPTSLOT_DATA    pPTSlot;
    PLUN_DEVICE     pLunDevice;
    PVOID           pHandle;
}IO_HANDLE, *PIO_HANDLE;


//
// macros
//
#define GET_DEVICE_SLOT(PathIndex, TargetIndex, WidthCount) \
                (((PPTSLOT_DATA)pAdapterDevExt->pPTMemPtr) + ((PathIndex * WidthCount) + TargetIndex))


/*
 * Following values are used with DevState, DevSlotState,
 * and 
 */

//
// Device state changes in the following order
// 1. DEV_STATE_UNINITIALIZED ->DEV_STATE_ONLINE->DEV_STATE_OFFLINE
//    ->DEV_STATE_UNINITIALIZED
// 2. DEV_STATE_UNINITIALIZED ->DEV_STATE_ONLINE->DEV_STATE_IN_RECOVERY
//    ->DEV_STATE_ONLINE
// 3. DEV_STATE_UNINITIALIZED ->DEV_STATE_ONLINE->DEV_STATE_IN_RECOVERY
//    ->DEV_STATE_OFFLINE->DEV_STATE_UNINITIALIZED
// 4. DEV_STATE_UNINITIALIZED ->DEV_STATE_ONLINE->DEV_STATE_OFFLINE
//    ->DEV_STATE_ONLINE
//
#define     DEV_STATE_UNINITIALIZED                         0x0001
#define     DEV_STATE_ONLINE                                0x0002
#define     DEV_STATE_IN_RECOVERY                           0x0004 //Not used yet
#define     DEV_STATE_OFFLINE                               0x0008
#define     DEV_STATE_REMOVED                               0x0010
#define     DEV_STATE_CHANGED                               0x0020

//
// Dev Slot State changes in the following order:
// 1. SLOT_STATE_EMPTY ->SLOT_STATE_DEVICE_VALID->SLOT_STATE_DEVICE_REMOVED
//    ->SLOT_STATE_EMPTY
//    OR
// 2. SLOT_STATE_EMPTY ->SLOT_STATE_DEVICE_VALID->SLOT_STATE_DEVICE_REMOVED
//    ->SLOT_STATE_DEVICE_CHANGED->SLOT_STATE_DEVICE_VALID->SLOT_STATE_EMPTY
//
// It has the following meanings:
// SLOT_STATE_EMPTY: Slot is free to use, OS also has freed it up if it was used before
// SLOT_STATE_DEVICE_VALID: Slot is busy  and in use from driver and OS View
// SLOT_STATE_DEVICE_REMOVED: Dev removed from the slot(from OSM view), but OS still 
// recognizes it the same old removed target(OS view)
// SLOT_STATE_RESERVED: Slot is reserved for a particular B and T map
// SLOT_STATE_DEVICE_CHANGED: Slot has got the new target in this case and driver 
// knows about it, but from OS point of view it still recognizes the OLD target at this
// slot, it's driver responsibility to handle the data corruption in this case
#define     SLOT_STATE_EMPTY                            0x01
#define     SLOT_STATE_DEVICE_VALID                     0x02
#define     SLOT_STATE_DEVICE_CHANGED                   0x04
#define     SLOT_STATE_DEVICE_REMOVED                   0x08
#define     SLOT_STATE_TEMPORARY_RESERVED               0x10
#define     SLOT_STATE_PERMANENT_RESERVED               0x20



//
// Flags definitions for dynamically Reporting the Device state to the OS
#define     SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_NEW_SLOT                        0x010000
#define     SLOTS_LASTSTATE_OLD_DEVICE_REAPPEARED_AT_OLD_SLOT                   0x020000
#define     SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_OLD_SLOT_OR_DEVICE_CHANGED      0x040000
#define     SLOTS_LASTSTATE_OLD_DEVICE_REMOVED_AT_OLD_SLOT                      0x080000
#define     SLOTS_LASTSTATE_OLD_DEVICE_GOT_OFFLINE_AT_OLD_SLOT                  0x100000


//
// defines used in UpdateBTMap function
//
#define     OSM_DEVICE_ADD          0x01
#define     OSM_DEVICE_REMOVE       0x02
#define     OSM_DEVICE_OFFLINE      0x03
#define     OSM_DEVICE_ONLINE       0x04



VOID
CdaInitBTMap(
    PVOID   HwDeviceExtension
    );


VOID
CdaUpdateBTSlot(
    PVOID                   pAdptDevExt,
    ULONG                   event_code,
    PCDA_DEVICE_PROPERTIES  pDeviceProperties
    );


VOID
CdaPutLunResourceToFreeList(
    PVOID               pAdptDevExt,
    PLUN_DEVICE         pLunDevice
    );


PLUN_DEVICE
GetLunResourceFromPool(
    PVOID               pAdptDevExt
    );


VOID 
CdaMakeFreeLunsLinkList(
    PVOID               pAdptDevExt
    );


/*INLINE*/ VOID
CdaGetPTSlot(
    PVOID           pAdptDevExt,
    UINT16          OsBusId,    /*IN*/
    UINT16          OsTgtId,  /*IN*/
    UINT16          OsLun, /*IN*/
    PIO_HANDLE      pIoHandle  /*OUT*/
 );


void 
CdaNotifyDeviceState(
    PVOID                   pCDADevExt,
    ULONG                   event,
    PVOID					pDeviceHandle,
    USHORT                  MemberIndex
    );

PPTSLOT_DATA
CdaGetSlotInfo(
    PVOID           pAdptDevExt,/*IN*/
    UINT16          OsBusId,    /*IN*/
    UINT16          OsTgtId    /*IN*/
 );

VOID
CdaDiscoverAndMapDevice(
    PVOID           pAdptDevExt
    );

VOID
CdaAcknowledgeLunDeletion(
        PVOID            pAdptDevExt,
        UINT16           PathId,
        UINT16           TargetId,
        PLUN_DEVICE      pLunDevice);

VOID
CdaAcknowledgeTargetDeletion(
        PVOID           pAdptDevExt,
        UINT16          PathId,
        UINT16          TargetId);

VOID
CdaAcknowledgeLunChangeAtSameSlot(
        PVOID           pAdptDevExt,
        UINT16          PathId,
        UINT16          TargetId,
        PLUN_DEVICE      pLunDevice);

VOID
CdaAcknowledgeDeviceChangeAtSameSlot(
        PVOID           pAdptDevExt,
        UINT16          PathId,
        UINT16          TargetId);

