/* $Header: X:\chimdev\sas\src\chimcom.h   \main\v1040a\1   Tue Oct  3 20:44:36 2006   LUU $ */
/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/***************************************************************************
*
*  Module Name:   CHIMCOM.H
*
*  Description:   Definitions which are common to all implementations
*
*  Owners:        SSG CHIM Team
*
*  Notes:         Contains basic data structures 
*                    (quadlet, him_bus_address, him_host_id, etc.)
*                 Contains #defines for pseudo-enum values
*                    (All s/g defines, host bus types, 
*                     memory categories, etc.)
*
***************************************************************************/


/***************************************************************************
* Definitions for two byte entity
***************************************************************************/
#if OSM_CPU_LITTLE_ENDIAN
   typedef union HIM_DOUBLET_             /* two orders entity              */
   {
      struct
      {
         HIM_UEXACT8  order0;
         HIM_UEXACT8  order1;
      } u8;
   } HIM_DOUBLET;
#else /* !OSM_CPU_LITTLE_ENDIAN */
   typedef union HIM_DOUBLET_             /* two orders entity              */
   {
      struct
      {
         HIM_UEXACT8  order1;
         HIM_UEXACT8  order0;
      } u8;
   } HIM_DOUBLET;
#endif /* OSM_CPU_LITTLE_ENDIAN */


/***************************************************************************
* Definitions for four byte entity
***************************************************************************/
#if OSM_CPU_LITTLE_ENDIAN
   typedef union HIM_QUADLET_             /* four orders entity             */
   {
      struct
      {
         HIM_UEXACT8  order0;
         HIM_UEXACT8  order1;
         HIM_UEXACT8  order2;
         HIM_UEXACT8  order3;
      } u8;
      struct
      {
         HIM_UEXACT16 order0;
         HIM_UEXACT16 order1;
      } u16;
   } HIM_QUADLET;
#else /* !OSM_CPU_LITTLE_ENDIAN */
   typedef union HIM_QUADLET_             /* four orders entity             */
   {
      struct
      {
         HIM_UEXACT8  order3;
         HIM_UEXACT8  order2;
         HIM_UEXACT8  order1;
         HIM_UEXACT8  order0;
      } u8;
      struct
      {
         HIM_UEXACT16 order1;
         HIM_UEXACT16 order0;
      } u16;
   } HIM_QUADLET;
#endif /* OSM_CPU_LITTLE_ENDIAN */


/***************************************************************************
* Definitions for eight byte entity
***************************************************************************/
#if OSM_CPU_LITTLE_ENDIAN
   typedef union HIM_OCTLET_              /* eight orders entity            */
   {
      struct
      {
         HIM_UEXACT8 order0;
         HIM_UEXACT8 order1;
         HIM_UEXACT8 order2;
         HIM_UEXACT8 order3;
         HIM_UEXACT8 order4;
         HIM_UEXACT8 order5;
         HIM_UEXACT8 order6;
         HIM_UEXACT8 order7;
      } u8;
      struct
      {
         HIM_UEXACT16 order0;
         HIM_UEXACT16 order1;
         HIM_UEXACT16 order2;
         HIM_UEXACT16 order3;
      } u16;
      struct
      {
         HIM_UEXACT32 order0;
         HIM_UEXACT32 order1;
      } u32;
   } HIM_OCTLET;
#else /* !OSM_CPU_LITTLE_ENDIAN */
   typedef union HIM_OCTLET_              /* eight orders entity            */
   {
      struct
      {
#if (OSM_CPU_ADDRESS_SIZE == 32)
         /* 32 bit CPU addressing */
         HIM_UEXACT8 order3;
         HIM_UEXACT8 order2;
         HIM_UEXACT8 order1;
         HIM_UEXACT8 order0;
         HIM_UEXACT8 order7;
         HIM_UEXACT8 order6;
         HIM_UEXACT8 order5;
         HIM_UEXACT8 order4;
#else /* (OSM_CPU_ADDRESS_SIZE == 64) */
         /* 64 bit CPU addressing */
         HIM_UEXACT8 order7;
         HIM_UEXACT8 order6;
         HIM_UEXACT8 order5;
         HIM_UEXACT8 order4;
         HIM_UEXACT8 order3;
         HIM_UEXACT8 order2;
         HIM_UEXACT8 order1;
         HIM_UEXACT8 order0;
#endif /* (OSM_CPU_ADDRESS_SIZE == 32) */
      } u8;
      struct
      {
#if (OSM_CPU_ADDRESS_SIZE == 32)
         /* 32 bit CPU addressing */
         HIM_UEXACT16 order1;
         HIM_UEXACT16 order0;
         HIM_UEXACT16 order3;
         HIM_UEXACT16 order2;
#else /* (OSM_CPU_ADDRESS_SIZE == 64) */
         /* 64 bit CPU addressing */
         HIM_UEXACT16 order3;
         HIM_UEXACT16 order2;
         HIM_UEXACT16 order1;
         HIM_UEXACT16 order0;
#endif /* (OSM_CPU_ADDRESS_SIZE == 32) */
      } u16;
      struct
      {
#if (OSM_CPU_ADDRESS_SIZE == 32)
         /* 32 bit CPU addressing */
         HIM_UEXACT32 order0;
         HIM_UEXACT32 order1;
#else /* (OSM_CPU_ADDRESS_SIZE == 64) */
         /* 64 bit CPU addressing */
         HIM_UEXACT32 order1;
         HIM_UEXACT32 order0;
#endif /* (OSM_CPU_ADDRESS_SIZE == 32) */
      } u32;
   } HIM_OCTLET;
#endif /* OSM_CPU_LITTLE_ENDIAN */


/***************************************************************************
* Buffer descriptor
***************************************************************************/
typedef struct HIM_BUFFER_DESCRIPTOR_
{
   HIM_BUS_ADDRESS busAddress;            /* bus address                   */
   HIM_UINT32      bufferSize;            /* data buffer size in bytes     */
   HIM_UINT8       bufferFlags;           /* data buffer flags             */
   void HIM_PTR    virtualAddress;        /* virtual address               */
} HIM_BUFFER_DESCRIPTOR;

/***************************************************************************
* Data bufferFlags definition
***************************************************************************/
#define  HIM_BF_DS_OFF_CHIP      0        /* off-chip memory domain        */
#define  HIM_BF_DS_ON_CHIP       1        /* on-chip memory domain         */
#define  HIM_BF_INVALID_BUS_ADDRESS    2  /* busAddress field is invalid   */

/***************************************************************************
* Miscellaneous definitions
***************************************************************************/
typedef HIM_UEXACT32 HIM_HOST_ID;

/***************************************************************************
* Host bus type definitions 
***************************************************************************/
#define HIM_HOST_BUS_PCI     0            /* PCI bus                       */
#define HIM_HOST_BUS_EISA    1            /* EISA bus                      */
#define HIM_HOST_BUS_RBI     2            /* RBI bus                       */

/***************************************************************************
* Memory category code definition
***************************************************************************/
#define  HIM_MC_UNLOCKED   0        /* unlocked, not DMAable, cached */
#define  HIM_MC_LOCKED     1        /* locked, DMAable memory, non-cached */
