/* $Header: X:\chimdev\sas\src\chimdef.h   \main\v1040a\1   Tue Oct  3 20:44:37 2006   LUU $ */
/*
 * Copyright  2005-2006 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/***************************************************************************
*
*  Module Name:   CHIMDEF.H
*
*  Description:   Main data structures of Common HIM Interface
*                 (Particularly those that use basic CHIM types)
*
*  Owners:        SSG CHIM Team
*
*  Notes:         Contains the following:
*                 Main, complicated CHIM structures:
*                    HIM_IOB, HIM_CONFIGURATION, profile structures
*                 #defines associated with IOB:
*                    taskStatus, function codes, etc.
*                 function-specific defines
*                    event types for OSMevent etc.
*                 him_func_ptrs, him_osm_func_ptrs
*                    function prototypes in handy structures
*                    for him-osm passoff
*
*  History:
*
*      03/06/03 TSL    Initial version.
*
***************************************************************************/
/***************************************************************************
* Miscellaneous definitions
***************************************************************************/
typedef void    HIM_PTR         HIM_TASK_SET_HANDLE;
typedef struct  HIM_IOB_        HIM_IOB;


/***************************************************************************
* Debug definitions
***************************************************************************/
/* Defines to be used with OSMDebugMode variable. */
#define  HIM_DEBUG_ERROR                (1U << 0) /* output some error      */
                                                  /* information            */
#define  HIM_DEBUG_FUNCTION_ENTRY       (1U << 1) /* output function entry  */
                                                  /* information            */
#define  HIM_DEBUG_FUNCTION_EXIT        (1U << 2) /* output function exit   */
                                                  /* information            */
#define  HIM_DEBUG_FUNCTION_ERROR_EXIT  (1U << 3) /* output function exit   */
                                                  /* with error             */
#define  HIM_DEBUG_INFO                 (1U << 4) /* output information     */
                                                  /* example data structure */
                                                  /* address                */
#define  HIM_DEBUG_WARNING              (1U << 5) /* output warning         */
                                                  /* information            */
#define  HIM_DEBUG_DEBUG                (1U << 6) /* output debug           */
                                                  /* information            */
#define  HIM_DEBUG_DEVICE_DISCOVERY     (1U << 7) /* output device discovery*/
                                                  /* information            */
#define  HIM_DEBUG_DUMP                 (1U << 8) /* output chip resources  */
                                                  /* like registers, scratch*/
                                                  /* mem, etc., when chip is*/
                                                  /* not running            */
#define  HIM_DEBUG_ALL   (HIM_DEBUG_ERROR | \
                          HIM_DEBUG_FUNCTION_ENTRY | \
                          HIM_DEBUG_FUNCTION_EXIT | \
                          HIM_DEBUG_FUNCTION_ERROR_EXIT | \
                          HIM_DEBUG_INFO | \
                          HIM_DEBUG_WARNING | \
                          HIM_DEBUG_DEBUG | \
                          HIM_DEBUG_DEVICE_DISCOVERY | \
                          HIM_DEBUG_DUMP)

/* Shared structures */
#define  HIM_SHARED_STRUCT_SCB             (1U << 0)
#define  HIM_SHARED_STRUCT_DONE_LIST       (1U << 1)
#define  HIM_SHARED_STRUCT_ERROR_BUFFERS   (1U << 2)
#define  HIM_SHARED_STRUCT_NEXUS_BUFFERS   (1U << 3)
#define  HIM_SHARED_STRUCT_RESERVED1       (1U << 4)
#define  HIM_SHARED_STRUCT_RESERVED2       (1U << 5)
#define  HIM_SHARED_STRUCT_RESERVED3       (1U << 6)
#define  HIM_SHARED_STRUCT_RESERVED4       (1U << 7)
#define  HIM_SHARED_STRUCT_ALL   (HIM_SHARED_STRUCT_SCB | \
                                       HIM_SHARED_STRUCT_DONE_LIST | \
                                       HIM_SHARED_STRUCT_ERROR_BUFFERS | \
                                       HIM_SHARED_STRUCT_NEXUS_BUFFERS)
                                       
/* Shared structure qualifiers */                                       
#define HIM_SHARED_STRUCT_DONE_LIST_ALL 0xFFFF
                                       
/* Chip Memory */
#define  HIM_CHIP_MEMORY_DDB                    (1UL << 0)
#define  HIM_CHIP_MEMORY_SCB                    (1UL << 1)
#define  HIM_CHIP_MEMORY_CSEQ_SCRATCH           (1UL << 2)
#define  HIM_CHIP_MEMORY_LSEQ0_SCRATCH          (1UL << 3)
#define  HIM_CHIP_MEMORY_LSEQ1_SCRATCH          (1UL << 4)
#define  HIM_CHIP_MEMORY_LSEQ2_SCRATCH          (1UL << 5)
#define  HIM_CHIP_MEMORY_LSEQ3_SCRATCH          (1UL << 6)
#define  HIM_CHIP_MEMORY_LSEQ4_SCRATCH          (1UL << 7)
#define  HIM_CHIP_MEMORY_LSEQ5_SCRATCH          (1UL << 8)
#define  HIM_CHIP_MEMORY_LSEQ6_SCRATCH          (1UL << 9)
#define  HIM_CHIP_MEMORY_LSEQ7_SCRATCH          (1UL << 10)
#define  HIM_CHIP_MEMORY_ALL_SEEPROM            (1UL << 11)
#define  HIM_CHIP_MEMORY_SEEPROM_ASI            (1UL << 12)
#define  HIM_CHIP_MEMORY_ALL_OCM                (1UL << 13)
#define  HIM_CHIP_MEMORY_OCM_DIRECTORY          (1UL << 14)
#define  HIM_CHIP_MEMORY_ALL_NVRAM              (1UL << 15)
#define  HIM_CHIP_MEMORY_NVRAM_DIRECTORY        (1UL << 16)

#define  HIM_CHIP_MEMORY_ALL_LSEQ_SCRATCH  (HIM_CHIP_MEMORY_LSEQ0_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ1_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ2_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ3_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ4_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ5_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ6_SCRATCH | \
                                            HIM_CHIP_MEMORY_LSEQ7_SCRATCH)

#define  HIM_CHIP_MEMORY_ALL    (HIM_CHIP_MEMORY_DDB | \
                                 HIM_CHIP_MEMORY_SCB| \
                                 HIM_CHIP_MEMORY_CSEQ_SCRATCH | \
                                 HIM_CHIP_MEMORY_ALL_LSEQ_SCRATCH | \
                                 HIM_CHIP_MEMORY_ALL_SEEPROM | \
                                 HIM_CHIP_MEMORY_SEEPROM_ASI | \
                                 HIM_CHIP_MEMORY_ALL_OCM | \
                                 HIM_CHIP_MEMORY_OCM_DIRECTORY | \
                                 HIM_CHIP_MEMORY_ALL_NVRAM | \
                                 HIM_CHIP_MEMORY_NVRAM_DIRECTORY)
                                 
/* Registers*/                                 
#define  HIM_REGISTERS_COMSAS       (1U << 0)
#define  HIM_REGISTERS_DATAFIFO     (1U << 1)
#define  HIM_REGISTERS_CSEQSTATE    (1U << 2)
#define  HIM_REGISTERS_LSEQSTATE    (1U << 3)
#define  HIM_REGISTERS_LSEQDMA      (1U << 4)
#define  HIM_REGISTERS_OOB          (1U << 5)
#define  HIM_REGISTERS_LSEQTRACE    (1U << 6)
#define  HIM_REGISTERS_CSEQTRACE    (1U << 7)
#define  HIM_REGISTERS_CSEQ         (HIM_REGISTERS_CSEQSTATE | \
                                     HIM_REGISTERS_CSEQTRACE)
#define  HIM_REGISTERS_LSEQ         (HIM_REGISTERS_LSEQSTATE | \
                                     HIM_REGISTERS_LSEQDMA | \
                                     HIM_REGISTERS_LSEQTRACE)
#define  HIM_REGISTERS_ALL          (HIM_REGISTERS_COMSAS | \
                                     HIM_REGISTERS_DATAFIFO | \
                                     HIM_REGISTERS_CSEQ | \
                                     HIM_REGISTERS_LSEQ | \
                                     HIM_REGISTERS_OOB)

/* Register Set Qualifiers*/                                 
#define  HIM_REGISTERS_PHY0         (1U << 0)
#define  HIM_REGISTERS_PHY1         (1U << 1)
#define  HIM_REGISTERS_PHY2         (1U << 2)
#define  HIM_REGISTERS_PHY3         (1U << 3)
#define  HIM_REGISTERS_PHY4         (1U << 4)
#define  HIM_REGISTERS_PHY5         (1U << 5)
#define  HIM_REGISTERS_PHY6         (1U << 6)
#define  HIM_REGISTERS_PHY7         (1U << 7)      

#define  HIM_REGISTERS_ALL_PHYS (HIM_REGISTERS_PHY0 | HIM_REGISTERS_PHY1 | \
                                 HIM_REGISTERS_PHY2 | HIM_REGISTERS_PHY3 | \
                                 HIM_REGISTERS_PHY4 | HIM_REGISTERS_PHY5 | \
                                 HIM_REGISTERS_PHY6 | HIM_REGISTERS_PHY7)
                                 
#define  HIM_QUEUE_SCAN_RESOURCES   (1UL << 0)
#define  HIM_QUEUE_ERROR_RECOVER    (1UL << 1)
#define  HIM_QUEUE_ADAPTER          (1UL << 2)
#define  HIM_QUEUE_DEVICE_PORT      (1UL << 3)
#define  HIM_QUEUE_LU               (1UL << 4)
#define  HIM_QUEUE_ACTIVE           (1UL << 5)
#define  HIM_QUEUE_DONE             (1UL << 6)
#define  HIM_QUEUE_TRANSLATION      (1UL << 7)
#define  HIM_QUEUE_POST_DELAY       (1UL << 8)

#define  HIM_QUEUE_ALL    (HIM_QUEUE_SCAN_RESOURCES | \
                           HIM_QUEUE_ERROR_RECOVER | \
                           HIM_QUEUE_ADAPTER | \
                           HIM_QUEUE_DEVICE_PORT | \
                           HIM_QUEUE_LU | \
                           HIM_QUEUE_ACTIVE | \
                           HIM_QUEUE_DONE | \
                           HIM_QUEUE_TRANSLATION | \
                           HIM_QUEUE_POST_DELAY)
                           
#if HIM_SCATTER_GATHER_FORMAT0 == 1
/***************************************************************************
* Scatter Gather Format 0 definition 
***************************************************************************/
#define  HIM_SGF0_DS_OFF_CHIP 0x00000000  /* off-chip memory domain        */
#define  HIM_SGF0_DS_ON_CHIP  0x10000000  /* on-chip memory domain         */
#define  HIM_SGF0_EOL         0x40000000  /* last element in the S/G list  */
#define  HIM_SGF0_EOS         0x80000000  /* last buffer element in the S/G sub-list  */
#endif /* HIM_SCATTER_GATHER_FORMAT0 == 1 */

/***************************************************************************
* Definitions for HIM_CONFIGURATION
* Used for HIMGetConfiguration()
***************************************************************************/
#define HIM_VERSION_CONFIGURATION 0
typedef struct HIM_CONFIGURATION_
{
   HIM_UINT32   versionNumber;         /* version # of this structure.        */

   /* modifiable parameters */

   /* mode independent fields */
   HIM_UINT8    initiatorProtocol;     /* Default based on compile option.
                                        * When set to 0 , all initiator protocols
                                        * shall be disabled.
                                        */
   HIM_UINT8    targetProtocol;        /* Default based on compile option.
                                        * When set to 0 , all target protocols
                                        * shall be disabled.
                                        */
                                       /* The initiatorProtocol and/or
                                        * targetProtocol may be set to at
                                        * least one protocol.  But, both cannot
                                        * be set to no protocol i.e. both modes
                                        * are disable.
                                        */
   HIM_UINT16   numActiveTasks;        /* Number of simultaneous active
                                        * tasks the HIM can execute.
                                        */
   HIM_UINT16   numTaskDeliveryBuffers;
                                       /* Number of internal buffers the HIM
                                        * has available for task delivery
                                        * to the adapter.
                                        */
   HIM_UINT16   numInternalReceiveBuffers;
                                       /* Number of internal buffers reserved
                                        * for hardware to store data received
                                        * unexpectedly.
                                        */
   HIM_UINT16   numDevicePorts;        /* Number of device port resources
                                        * supported by adapter.
                                        */
   HIM_UINT8    numPhys;               /* Number of physical interfaces (or PHYs)
                                        * supported by adapter and HIM.
                                        */
   HIM_UINT8    numPorts;              /* Number of ports supported by
                                        * adapter and HIM.
                                        */
   HIM_UINT8    memoryMapped;          /* Allows OSM to choose:
                                        *   HIM_IOSPACE - I/O mapped
                                        *   HIM_MEMORYSPACE - memory mapped
                                        *   HIM_NO_CHOICE_AVAILABLE - HIM does not offer
                                        *                             choice
                                        */

   /* initiator mode fields */
   HIM_UINT16   numTargetPorts;        /* Number of device port resources
                                        * to be used for devices with
                                        * only target protocols.
                                        */
   HIM_UINT16   numExpanders;          /* Number of device port resources
                                        * to be used for SAS expander ports.
                                        */
   HIM_UINT8    numPhysPerExpander;    /* Number of resources the HIM
                                        * allocates for phys per expander.
                                        */
   HIM_UINT8    numInternalScanResources;
                                       /* Number of internal resources that
                                        * HIM uses for device port discovery.
                                        */
   HIM_UINT16   numATATranslationBuffers;
                                       /* Number of buffers used by the
                                        * SCSI to ATA translator.
                                        */

   /* target mode fields */
   HIM_UINT16   numInitiatorPorts;     /* Number of initiator ports supported
                                        * by the adapter.
                                        */
   HIM_UINT16   numSCSINexusTSHs;      /* Number of SCSI nexus task set handles
                                        * that can be used by the adapter.
                                        */
   HIM_UINT16   numSMPNexusTSHs;       /* Number of SMP nexus task set handles
                                        * that can be used by the adapter.
                                        */
   HIM_UINT16   maxReceiveCDBLength;   /* Maximum number of bytes of CDB
                                        * supported by the HIM for CDBs
                                        * received when operating in
                                        * target mode for SCSI.
                                        */
   HIM_UINT16   numInternalRequestBuffers;
                                       /* Number of internal buffers reserved
                                        * for the adapter to store requests
                                        * received in target mode.
                                        */
   HIM_UINT16   maxReceiveSMPRequestLength;
                                       /* Maximum number of bytes of SMP
                                        * request frame supported by the HIM
                                        * for SMP requests received when
                                        * operating in target mode for SMP.
                                        */
   HIM_BOOLEAN  scsiAutoResponseNoResource;
                                       /* Only be valid if targetProtocol
                                        * includes HIM_PROTOCOL_SCSI.
                                        */


   /* unmodifiable parameters */
   HIM_UINT32   maxTransferSize;       /* Maximum number of bytes transferred
                                        * in a single IOB request.
                                        */
   HIM_UINT32   maxSGElements;         /* Maximum number of scatter/gather
                                        * elements that may be referenced
                                        * by an IOB request.
                                        */
   HIM_UINT32   maxSGSublists;         /* Maximum number of scatter/gather
                                        * sub-lists within a scatter/gather list
                                        * that may be referenced by an IOB request.
                                        */
   HIM_UINT16   maxDevicePortsOnChip;  /* Maximum number of device port resources
                                        * supported by the adapter hardware's
                                        * internal memory.
                                        */
   HIM_UINT16   maxDevicePortsOffChip; /* Maximum number of devices supported
                                        * by the hardware using OSM allocated
                                        * memory.
                                        */
   HIM_UINT16   maxActiveTasksOnChip;  /* Maximum number of simultaneous
                                        * commands the hardware's memory
                                        * supports.
                                        */
   HIM_UINT16   maxActiveTasksOffChip;
                                       /* Maximum number of simultaneous
                                        * commands the hardware supports
                                        * using OSM allocated memory.
                                        */
   HIM_UINT8    maxIOHandles;          /* Maximum number of I/O handles
                                        * the HIM will need.
                                        */
} HIM_CONFIGURATION;

/***************************************************************************
* Definition for the function called by the OSMWatchdog routine
***************************************************************************/
typedef void (* HIM_WATCHDOG_FUNC)(HIM_TASK_SET_HANDLE adapterTSH);


/***************************************************************************
* Definition for the HIM_IOB post routine
***************************************************************************/
typedef HIM_UINT32 (* HIM_POST_PTR)(HIM_IOB HIM_PTR pIob);


/***************************************************************************
* Definitions for HIM_IOB 
***************************************************************************/
struct HIM_IOB_
{
   HIM_BUFFER_DESCRIPTOR   iobReserve;
   HIM_UINT8               function;
   HIM_UINT8               priority;

   /* A bit-map field */
   HIM_UEXACT32            iobFlags;
   void HIM_PTR            transportSpecific;
   void HIM_PTR            osmIobContext;
   void HIM_PTR            himEventContext;
   void HIM_PTR            osmDevicePortContext;
   HIM_IOB HIM_PTR         relatedIob;
   HIM_TASK_SET_HANDLE     taskSetHandle;
   HIM_UINT16              taskStatus;
   HIM_UINT32              sortTagBegin;
   HIM_UINT32              sortTagEnd;
   HIM_POST_PTR            postRoutine;
   HIM_BUFFER_DESCRIPTOR   targetCommand;
   HIM_UINT16              targetCommandLength;
   HIM_BUFFER_DESCRIPTOR   data;
   HIM_UINT32              ioLength;
   HIM_UINT32              relativeOffset;
   HIM_BUFFER_DESCRIPTOR   errorData;
   HIM_UEXACT32            residual;
   HIM_UEXACT32            residualError;
   HIM_UINT8               taskAttribute;
};

/***************************************************************************
* Definitions for HIM_ATA_TARGET_COMMAND
***************************************************************************/
typedef struct HIM_ATA_TARGET_COMMAND_
{
     HIM_UEXACT8 command;
     HIM_UEXACT8 features;
     HIM_UEXACT8 sectorNumber;
     HIM_UEXACT8 cylinderLow;
     HIM_UEXACT8 cylinderHigh;
     HIM_UEXACT8 deviceHead;
     HIM_UEXACT8 sectorNumberExp;
     HIM_UEXACT8 cylinderLowExp;
     HIM_UEXACT8 cylinderHighExp;
     HIM_UEXACT8 featuresExp;
     HIM_UEXACT8 sectorCount;
     HIM_UEXACT8 sectorCountExp;
     HIM_UEXACT8 reserved12;
     HIM_UEXACT8 control;
     HIM_UEXACT8 reserved14;
     HIM_UEXACT8 reserved15;
     HIM_UEXACT8 reserved16;
     HIM_UEXACT8 reserved17;
} HIM_ATA_TARGET_COMMAND;

/***************************************************************************
* Definitions for HIM_ATA_ERROR_DATA
***************************************************************************/
typedef struct HIM_ATA_ERROR_DATA_
{
     HIM_UEXACT8 status;
     HIM_UEXACT8 error;
     HIM_UEXACT8 sectorNumber;
     HIM_UEXACT8 cylinderLow;
     HIM_UEXACT8 cylinderHigh;
     HIM_UEXACT8 deviceHead;
     HIM_UEXACT8 sectorNumberExp;
     HIM_UEXACT8 cylinderLowExp;
     HIM_UEXACT8 cylinderHighExp;
     HIM_UEXACT8 reserved9;
     HIM_UEXACT8 sectorCount;
     HIM_UEXACT8 sectorCountExp;
     HIM_UEXACT8 reserved12;
     HIM_UEXACT8 reserved13;
     HIM_UEXACT8 reserved14;
     HIM_UEXACT8 reserved15;
     HIM_UEXACT8 reserved16;
     HIM_UEXACT8 reserved17;
} HIM_ATA_ERROR_DATA;

/* function definitions */
/* the functions have been divided into different categories 
   with extra functional values available for future expansion */
/* functions sending tasks to devices  codes 0 to 0x0f*/
#define  HIM_INITIATE_SCSI_TASK        0x00
#define  HIM_INITIATE_ATA_TASK         0x01
#define  HIM_INITIATE_SMP_TASK         0x02
#define  HIM_INITIATE_SCSI_BIDI_TASK   0x03
#define  HIM_INITIATE_IPMB_TASK        0x04

/* functions for device probe and scan; codes 0x10 to 0x1f */
#define  HIM_PROTOCOL_AUTO_CONFIG      0x10
#define  HIM_PROBE                     0x11
#define  HIM_PROBE_LU                  0x12

/* management functions internal and/or device; codes 0x20 0x3f */
#define  HIM_ABORT_TASK                0x20
#define  HIM_ABORT_TASK_SET            0x21
#define  HIM_CLEAR_ACA                 0x22
#define  HIM_QUERY_TASK                0x23
#define  HIM_SUSPEND                   0x24
#define  HIM_RESUME                    0x25
#define  HIM_QUIESCE                   0x26
#define  HIM_UNFREEZE_QUEUE            0x27
#define  HIM_CLEAR_TASK_SET            0x2b
#define  HIM_ABORT_NEXUS               0x2c
#define  HIM_PERFORM_LU_RECOVERY       0x2d

/* hardware related functions; codes 0x40 to 0x4f */
#define  HIM_RESET_LOGICAL_UNIT        0x40
#define  HIM_RESET_DEVICE_PORT         0x41
#define  HIM_RESET_PORT                0x42
#define  HIM_RESET_ADAPTER             0x43
#define  HIM_RESET_HARDWARE            0x44
#define  HIM_ENABLE_PHY                0x45
#define  HIM_DISABLE_PHY               0x46
#define  HIM_COPY_MEMORY               0x47

/* Miscellaneous functions; codes 0x50 to 0x5f */
#define  HIM_UPDATE_ASYNC_PROFILE_FIELDS 0x50
#define  HIM_INITIALIZE_DEVICE_PORT    0x51

/* functions for target mode */
#define  HIM_ESTABLISH_SCSI_CONNECTION      0x60
#define  HIM_SCSI_REESTABLISH_AND_COMPLETE  0x61
#define  HIM_SCSI_REESTABLISH_INTERMEDIATE  0x62
#define  HIM_ESTABLISH_SMP_CONNECTION       0x63
#define  HIM_SMP_RESPONSE                   0x64

/*********************************************************************/
/*                  HIM internal iob functions                       */
/* The range of function codes from 0xE0 - 0xFF are reserved for     */
/* HIM internal use and these internal functions are defined in      */
/* sasdef.h file.  These internal functions are defined as SASxxx    */
/* instead of HIMxxx.  If functions defined in CHIM interface spec   */
/* reach this range, sasdef.h file needs to be revisited.            */
/*********************************************************************/

/* For DEBUG purposes only */
#define  HIM_INITIATE_SSP_TASK         0xc0

/* iobFlags definitions */
#define  HIM_INBOUND_DATA           0x00000001
#define  HIM_OUTBOUND_DATA          0x00000002
#define  HIM_OUT_OF_ORDER_TRANSFER  0x00000004
#define  HIM_RAID_MIRROR            0x00000008
#define  HIM_REQUEST_TYPE_MASK      0x00000030

/* iobFlags 'Target Request Type' definitions */
#define  HIM_REQUEST_TYPE_COMMAND   0x00000010     /* Device Command */ 
#define  HIM_REQUEST_TYPE_TMF       0x00000020     /* Task Management Function */

#define  HIM_ATA_PIO                0x00000040
#define  HIM_IOB_SORT_FIFO          0x00000080
#define  HIM_NO_INTERRUPT_REQUIRED  0x00000100
#define  HIM_ATA_NCQ                0x00000200

/* task attribute definitions */
#define  HIM_TASK_SIMPLE            1
#define  HIM_TASK_HEAD_OF_QUEUE     2
#define  HIM_TASK_ORDERED           3
#define  HIM_TASK_ACA               4
#define  HIM_TASK_UNTAGGED          5
#define  HIM_TASK_UNKNOWN           9

/* probe transport specific definitions */
#define  HIM_INVALID_PHY_ID           0xFFFF

/***************************************************************************
*  Flash return definitions
***************************************************************************/
/* #define  HIM_ADAPTER_NOT_IDLE     2 */
#define  HIM_INVALID_SEGMENT_ID    3
#define  HIM_INVALID_REQUEST       4
#define  HIM_WRITE_NOT_SUPPORTED   5
#define  HIM_NO_NVRAM              6

#define  HIM_SEGMENT_ATTRIBUTE_ERASEWRITE 1

/***************************************************************************
*  taskStatus definitions
***************************************************************************/

/* Device Port reported status values */
#define HIM_IOB_GOOD                0x01  /* IO/function is successful.       */

#define HIM_IOB_ERRORDATA_VALID     0x02  /* I/O finished with an error or
                                          unusual status.  error data is
                                          valid.                              */

#define HIM_IOB_BUSY                0x03  /* LU reported BUSY status.         */
#define HIM_IOB_TASK_SET_FULL       0x04  /* LU reported TASK SET FULL status.*/
#define HIM_IOB_TARGET_RESERVED     0x05  /* LU reported RESERVATION CONFLICT
                                          status.                             */
#define HIM_IOB_CONDITION_MET       0x06  /* LU reported CONDITION MET status.*/
#define HIM_IOB_ACA_ACTIVE          0x07  /* LU reported ACA ACTIVE status.   */
#define HIM_IOB_TASK_ABORTED        0x08  /* LU reported TASK ABORTED status. */

#define HIM_IOB_PROTOCOL_NOT_SUPPORTED       0x09  /* Protocol not supported  */

#define HIM_IOB_RATE_NOT_SUPPORTED           0x0A  /* Connection rate not
                                                   supported.                 */

#define HIM_IOB_TRANSLATION_NOT_SUPPORTED    0x0B  /* The task or task
                                                   management function IOB
                                                   could not be translated
                                                   to the protocol of the
                                                   IOB's intended object.     */

#define HIM_IOB_INVALID_DEVICE_PORT_RESPONSE 0x0C  /* Device Port provided
                                                   an invalid or reserved
                                                   value for a response.      */

#define HIM_IOB_ERRORDATA_REQUIRED  0x0D  /* Same as HIM_IOB_ERRORDATA_VALID,
                                          but this error data has not yet
                                          been collected.                     */

/* IOB errors and support issues */
#define HIM_IOB_INVALID             0x10  /* IOB is incorrectly formed.       */
#define HIM_IOB_UNSUPPORTED         0x11  /* The CHIM implementation
                                          does not support the requested
                                          function. However, IOB form is
                                          not in error.                       */
#define HIM_IOB_TSH_INVALID         0x12  /* HIM does not recognize TSH       */
#define HIM_IOB_TSH_NOT_VALIDATED   0x13  /* An IOB was sent to a target TSH
                                          in between execution of protocol
                                          HIM_PROTOCOL_AUTO_CONFIG and
                                          HIMValidateTargetTSH.               */
#define HIM_IOB_ADAPTER_NOT_IDLE    0x14  /* Some functions require the
                                          adapterTSH in question to have
                                          no active I/O's.
                                          This is the return code used
                                          if this rule is violated.           */
#define HIM_IOB_PORT_NOT_IDLE       0x15  /* Some functions require the
                                          portTSH in question to have
                                          no active I/O's.
                                          This is the return code used
                                          if this rule is violated.           */
#define HIM_IOB_QUEUE_SUSPENDED     0x16  /* Adapter received suspend or
                                          quiesce before this IOB.  Send
                                          resume or auto config,
                                          respectively, before continuing.    */
#define HIM_IOB_UNDEFINED_TASKSTATUS         0x17  /* An error occurred that
                                                   could not be translated to
                                                   any of the current
                                                   taskStatus values.         */
#define HIM_IOB_INVALID_STATE       0x18  /* IOB format is valid and is
                                          supported by the HIM for this TSH,
                                          but the intended object or related
                                          object is in a state that prevents
                                          the IOB from being performed at
                                          this time.                          */

/* Transport errors */
#define HIM_IOB_NO_RESPONSE         0x20  /* Requested device port does not
                                          respond.                            */
#define HIM_IOB_COMMAND_DELIVERY_FAILURE     0x21  /* Command failed to be
                                                   delivered due to a
                                                   transport error.           */
#define HIM_IOB_DEVICE_PORT_UNREACHABLE      0x22  /* service delivery
                                                   subsystem was unable to
                                                   establish a connection with
                                                   the intended device port.  */
#define HIM_IOB_PROTOCOL_ERROR      0x23  /* CHIM detects an unrecoverable
                                          protocol error on the part of the
                                          device port.  (It is assumed that if
                                          the target detects an error by the
                                          host adapter, it will be reported
                                          as erro data.)                      */
#define HIM_IOB_CONNECTION_FAILED   0x24  /* Connection between CHIM and
                                          device port failed.                 */
#define HIM_IOB_ADAPTER_DETECTED_CRC_ERROR   0x25  /* Adapter detected
                                                   CRC error on the part
                                                   of the device port.        */
#define HIM_IOB_DEVICE_PORT_DETECTED_CRC_ERROR  0x26
                                                   /* Device port detected
                                                   a CRC error on the part
                                                   of the adapter.            */
#define HIM_IOB_DATA_UNDERRUN       0x27  /* Data underrun wasdetected.       */
#define HIM_IOB_DATA_OVERRUN        0x28  /* Data overrun was detected.       */
#define HIM_IOB_ERRORDATA_UNDERRUN  0x29  /* Error data underrun wasdetected. */
#define HIM_IOB_ERRORDATA_OVERRUN   0x2A  /* Error data overrun was detected. */
#define HIM_IOB_FAILED_TRANSPORT_TIMEOUT     0x2B  /* A transport timeout
                                                   occurred prior to
                                                   command delivery resulting
                                                   in the IOB failure.        */

/* Task Management Function IOB errors */
#define HIM_IOB_INVALID_LUN         0x30  /* Device port reported that
                                          the logical unit does not exist.    */
#define HIM_IOB_UNSUPPORTED_REQUEST 0x31  /* Device port reported that the
                                          request is not supported or
                                          unrecognized.                       */
#define HIM_IOB_REQUEST_FAILED      0x32  /* Device port reported that the
                                          task failed. There is no sense data
                                          associated with this taskStatus.    */
#define HIM_IOB_REQUEST_COMPLETE    0x33  /* Requested function is complete.  */



/* Aborting task status values */
/* NOTE: Additional abort task status added must be in the range from */
/* HIM_IOB_ABORTED_ON_REQUEST to HIM_IOB_ABORTED_RX_CLEAR_ACA */
#define HIM_IOB_ABORT_FAILED        0x40  /* Device port did not respond
                                          properly to the abort sequence.     */
#define HIM_IOB_ABORT_NOT_FOUND     0x41  /* Related IOB to be aborted has not
                                          been found.                         */
#define HIM_IOB_ABORT_STARTED       0x42  /* HIM has found the IOB to be
                                          aborted and has started the abort
                                          procedure.                          */
#define HIM_IOB_ABORT_ALREADY_DONE  0x43  /* IOB to be aborted has already
                                          completed on the I/O channel.
                                          No abort is performed.              */

/* Aborted task status values */
#define HIM_IOB_ABORTED_ON_REQUEST  0x50  /* IOB was aborted as a result of
                                          an OSM requested HIM_ABORT_TASK IOB.*/
#define HIM_IOB_ABORTED_RESET_DEVICE_PORT    0x51  /* IOB was aborted as a
                                                   result of an OSM requested
                                                   HIM_RESET_DEVICE_PORT IOB. */
#define HIM_IOB_ABORTED_RESET_PORT  0x52  /* IOB was aborted as a result of
                                          an OSM request to reset a port.
                                          The IOB should not be retried until
                                          after a port
                                          HIM_PROTOCOL_AUTO_CONFIG IOB
                                          completes.                          */
#define HIM_IOB_ABORTED_RESET_ADAPTER        0x53  /* IOB was aborted as a
                                                   result of an OSM request
                                                   to execute a
                                                   HIM_RESET_ADAPTER IOB.
                                                   The IOB should not be
                                                   retried until after
                                                   an adapter
                                                   HIM_PROTOCOL_AUTO_CONFIG
                                                   IOB completes.             */
#define HIM_IOB_ABORTED_RESET_HARDWARE       0x54  /* IOB was aborted as a
                                                   result of an OSM request
                                                   to execute a
                                                   HIM_RESET_HARDWARE IOB.    */
#define HIM_IOB_ABORTED_RESET_LOGICAL_UNIT   0x55  /* IOB was aborted due to
                                                   a logical unit reset
                                                   requested by the OSM.      */
#define HIM_IOB_ABORTED_BY_LOGICAL_UNIT      0x56  /* Logical unit aborted
                                                   the IOB due to an error
                                                   for another task.
                                                   The IOB itself was not
                                                   the cause of the error.
                                                   A retry is recommended.    */
#define HIM_IOB_ABORTED_CLEAR_TASK_SET       0x57  /* IOB was aborted due to
                                                   a clear task set
                                                   requested by the OSM.      */
#define HIM_IOB_ABORTED_ABORT_TASK_SET       0x58  /* IOB was aborted due to
                                                   a abort task set
                                                   requested by the OSM.      */
#define HIM_IOB_ABORTED_BY_CHIM     0x59  /* HIM decided to abort the
                                          command for internal reasons.       */

#define HIM_IOB_ABORTED_RX_HARD_RESET        0x60  /* IOB was aborted due to
                                                    a port received hard reset
                                                    event.                    */
#define HIM_IOB_ABORTED_QUIESCE              0x61  /* The IOB was aborted as
                                                    a result of an OSM request
                                                    to execute a HIM_QUIESCE.
                                                    This taskStatus is returned
                                                    for IOB functions 
                                                    HIM_ESTABLISH_SCSI_CONNECTION
                                                    and
                                                    HIM_ESTABLISH_SMP_COONECTION
                                                    only.                     */

/* NOTE: Additional abort task status added must be in the range from */
/* HIM_IOB_ABORTED_ON_REQUEST to HIM_IOB_ABORTED_RX_CLEAR_ACA */

/* Target Mode Task Management Function Abort Status values */
#define HIM_IOB_ABORTED_RX_ABORT             0x64 /* IOB was aborted due to
                                                     receipt of an Abort Task
                                                     request.                 */
#define HIM_IOB_ABORTED_RX_ABORT_TASK_SET    0x65 /* IOB was aborted due to
                                                     receipt of an Abort Task
                                                     Set request  .           */
#define HIM_IOB_ABORTED_RX_CLEAR_TASK_SET    0x66  /* IOB was aborted due to
                                                      receipt of a Clear Task
                                                      Set request.            */
#define HIM_IOB_ABORTED_RX_LOGICAL_UNIT_RESET 0x67 /* IOB was aborted due to
                                                      receipt of a Logical Unit
                                                      Reset request.          */
#define HIM_IOB_ABORTED_RX_CLEAR_ACA         0x68  /* IOB was aborted due to
                                                      receipt of a Clear ACA
                                                      request.                */

/* Target Mode Errors */
#define HIM_IOB_TARGETCOMMANDBUFFER_OVERRUN  0x70  /* Information received
                                                   from the initiator into
                                                   the targetCommand buffer
                                                   was truncated due to
                                                   value of IOB field
                                                   targetCommandBufferSize.   */
#define HIM_IOB_OUT_OF_ORDER_TRANSFER_REJECTED  0x71
                                                   /* A
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            or HIM_SCSIREESTABLISH_INTERMEDIATE
                                            IOB with the outOfOrder iobFlags set
                                            to one was rejected by the
                                            initiator.                        */
#define HIM_IOB_INITIATOR_DETECTED_ERROR     0x73  /* Initiator detected some
                                            error other than a parity error.
                                            Returned for
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            or HIM_SCSI_REESTABLISH_INTERMEDIATE
                                            IOBs only.                        */
#define HIM_IOB_DATA_OFFSET_ERROR            0x74  /* A Data Frame with an
                                            unexpected data offset value was
                                            received by a target port. Returned
                                            for
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE or
                                            HIM_SCSI_REESTABLISH_INTERMEDIATE
                                            IOB functions.
                                            The target mode application shall
                                            issue a
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB with a response frame that has
                                            a status of Check Condition, a
                                            Sense Key of Aborted Command (0Bh),
                                            ASC value of 4Bh, and ASCQ value
                                            of 05h indicating DATA OFFSET
                                            ERROR.                            */
#define HIM_IOB_INITIATOR_RESPONSE_TIMEOUT   0x75  /* The Initiator Response
                                            Timeout Timer expired while waiting
                                            to receive data from an initiator.
                                            This taskStatus shall only be
                                            returned by
                                            HIM_SCSI_REESTABLISH_INTERMEDIATE
                                            and
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB functions.

                                            The target mode application shall
                                            issue a
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB with a response frame that has
                                            a status of Check Condition, a
                                            Sense Key of Aborted Command (0Bh),
                                            ASC value of 4Bh, and ASCQ value
                                            of 06h indicating INITIATOR
                                            RESPONSE TIMEOUT.                 */
#define HIM_IOB_INFORMATION_UNIT_TOO_SHORT   0x76  /* The target port received
                                            a zero length data frame.
                                            This taskStatus shall only be
                                            returned by
                                            HIM_SCSI_REESTABLISH_INTERMEDIATE
                                            and
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB functions.

                                            The target mode application shall
                                            issue a
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB with a response frame that has
                                            a status of Check Condition, a
                                            Sense Key of Aborted Command (0Bh),
                                            ASC value of 0Eh, and ASCQ value
                                            of 01h indicating INFORMATION
                                            UNIT TOO SHORT.                   */
#define HIM_IOB_ACK_NAK_TIMEOUT     0x77  /* The target port did not receive an
                                            ACK/NAK for a read DATA frame or
                                            the connection was broken.
                                            This taskStatus shall only be
                                            returned by
                                            HIM_SCSI_REESTABLISH_INTERMEDIATE
                                            and
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB functions.

                                            The target mode application shall
                                            issue a
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB with a response frame that has
                                            a status of Check Condition, a
                                            Sense Key of Aborted Command (0Bh),
                                            ASC value of 4Bh, and ASCQ value
                                            of 03h indicating
                                            ACK/NAK TIMEOUT.                 */
#define HIM_IOB_NAK_RECEIVED        0x78  /* The target port received a NAK
                                            for a Frame (and any retries
                                            attempted) sent to an Initiator.
                                            This taskStatus shall only be
                                            returned by
                                            HIM_SCSI_REESTABLISH_INTERMEDIATE
                                            and
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB functions.

                                            The target mode application shall
                                            issue a
                                            HIM_SCSI_REESTABLISH_AND_COMPLETE
                                            IOB with a response frame that
                                            has a status of Check Condition,
                                            a Sense Key of Aborted Command
                                            (0Bh), ASC value of 4Bh, and ASCQ
                                            value of 04h indicating NAK
                                            RECEIVED.
                                                                            */


/* Primary I/O Interface Errors */
#define HIM_IOB_PCI_OR_PCIX_ERROR   0x80  /* IOB was executing when a PCI or
                                          PCI-X error was detected.
                                          The PCI/PCI-X error may be due to
                                          a memory address associated
                                          with this IOB.                      */
#define HIM_IOB_ABORTED_PCI_ERROR   0x81  /* A PCI error caused the IOB to
                                          fail.  This IOB should be retried.  */
#define HIM_IOB_ABORTED_PCIX_ERROR  0x82  /* A PCI-X error caused the IOB to
                                          fail.  This IOB should be retried.  */

/* Miscellaneous: */
#define HIM_IOB_ADAPTER_FAILURE     0x90  /* An adapter failure occurred
                                          causing this command to fail.       */
#define HIM_IOB_PORT_FAILURE        0x91  /* A port failure occurred
                                          causing this command to fail.       */
#define HIM_IOB_PORT_RESOURCES_EXCEEDED      0x92  /* Current configuration
                                                   attached to the adapter
                                                   has exceeded the HIM's
                                                   internal port resources.   */
#define HIM_IOB_DEVICE_PORT_RESOURCES_EXCEEDED 0x93
                                                   /* Current configuration
                                                   attached to the adapter
                                                   has exceeded the HIM's
                                                   internal device port
                                                   resources.                 */

/*********************************************************************/
/*                  HIM internal iob taskStatus                      */
/* The range of taskStatus codes from 0xE0 - 0xFF are reserved for   */
/* HIM internal use and these internal taskStatus are defined in     */
/* sasdef.h file.  These internal taskStatus are defined as SASxxx   */
/* instead of HIMxxx.  If taskStatus defined in CHIM interface spec  */
/* reach this range, sasdef.h file needs to be revisited.            */
/*********************************************************************/



/***************************************************************************
* General Definitions for Profiles
***************************************************************************/
#define HIM_SAS_ADDRESS_SIZE   8
#define HIM_MAX_WWID_SIZE      8
#define HIM_LUN_SIZE           8
#define HIM_SAS_MAX_PHYIDS_PER_DEVICE 128
/* Definition for the invalid link rate */
#define HIM_INVALID_RATE       0x00

/***************************************************************************
* Definitions for ADAPTER PROFILE Pages
***************************************************************************/
#define HIM_MAJOR_VERSION_ADAPTER_PROFILE   1
#define HIM_MINOR_VERSION_ADAPTER_PROFILE   0

/* Page Code defines */
#define HIM_APC_GENERAL                1
#define HIM_APC_RESOURCE_LIMITS        2
#define HIM_APC_HARDWARE_CONSTRAINTS   3
#define HIM_APC_IOB                    4
#define HIM_APC_BIOS                   5
#define HIM_APC_PCI                    6
#define HIM_APC_SAS_GENERAL            7
#define HIM_APC_FLASH                  8
#define HIM_APC_SATA_GENERAL           9

/* Page Code define specific to initiator mode */
#define HIM_APC_INITIATOR_MODE         0x10
#define HIM_APC_SAS_INITIATOR_MODE     0x11

/* Page Code define specific to target mode */
#define HIM_APC_TARGET_MODE         0x20
#define HIM_APC_SAS_SSP_TARGET_MODE 0x21
#define HIM_APC_SCSI_TARGET_MODE    0x22
#define HIM_APC_SMP_TARGET_MODE     0x23

/* Page structures */
/* General Adapter Profile Page */
#if !SAS_DOWNSHIFT_SUPPORT

typedef struct HIM_AP_GENERAL_PAGE_
{
   HIM_UINT16   AP_MajorVersion;
   HIM_UINT16   AP_MinorVersion;
   HIM_UINT8    AP_Transport;
   HIM_UINT8    AP_InitiatorProtocol;
   HIM_UINT8    AP_TargetProtocol;
   HIM_BOOLEAN  AP_SaveRestoreSequencer;
   HIM_UINT32   AP_StateSize;
   HIM_BOOLEAN  AP_ChipReset;
   HIM_UINT8    AP_MemoryMapped;
   HIM_UINT8    AP_MaxIOHandles;
   HIM_UEXACT8  AP_WorldWideName[8];
   HIM_UEXACT8  AP_HBASerialNumber[12];
   HIM_UEXACT8  AP_UsageModelID;
   HIM_UINT8    AP_InterruptMethod;
} HIM_AP_GENERAL_PAGE;

/* Resource Limits Adapter Profile Page */
typedef struct HIM_AP_RESOURCE_LIMITS_PAGE_
{
   HIM_UINT8    AP_NumPhys;
   HIM_UINT8    AP_NumPorts;
   HIM_UINT16   AP_NumDevicePorts;
   HIM_UINT16   AP_NumActiveTasks;
   HIM_UINT16   AP_NumTaskDeliveryBuffers;
   HIM_UINT16   AP_NumInternalReceiveBuffers;
   HIM_UINT16   AP_NumATATranslationBuffers;
} HIM_AP_RESOURCE_LIMITS_PAGE;

/* Harware Constraints Adapter Profile Page */
typedef struct HIM_AP_HARDWARE_CONSTRAINTS_PAGE_
{
   HIM_UEXACT32 AP_AlignmentMask;      /* Alignment req's for s/g lists, iob
                                        * reserve, and buffer descriptors.
                                        */
   HIM_UINT8    AP_SGFormatSupported;
   HIM_UINT32   AP_MaxSGElements;      /* Max S/G elements allowed */
   HIM_UINT32   AP_MaxSGElementsPerSublist;
   HIM_UINT32   AP_MaxSGSublists;      /* Max S/G sub-lists allowed */
   HIM_UINT32   AP_MaxTransferSize;    /* Max transfer, in bytes, per IOB */
   HIM_UINT32   AP_MaxSGByteCount;     /* Maximum value to be assigned to a */
                                       /* byteCount field of a SG buffer element */
                                       /* 0xFFFFFFFF indicates no limit */
   HIM_BOOLEAN  AP_CanCross4G;         /* If HIM_TRUE, DMA can cross 32-bit
                                        * boundary.
                                        */
   HIM_UINT16   AP_MaxActiveTasksOnChip;
   HIM_UINT16   AP_MaxActiveTasksOffChip;
   HIM_UINT16   AP_MaxDevicePortsOnChip;
   HIM_UINT16   AP_MaxDevicePortsOffChip;
} HIM_AP_HARDWARE_CONSTRAINTS_PAGE;

/* IOB Adapter Profile Page */
typedef struct HIM_AP_IOB_PAGE_
{
   HIM_BOOLEAN  AP_ErrorDataBusAddressSupported;
   HIM_BOOLEAN  AP_CopyMemoryIOBSupported;
                                       /* If HIM_TRUE, indicates the HIM supports
                                        * HIM_COPY_MEMORY IOB function.
                                        */
   HIM_UINT32   AP_MaxCopyMemoryTransferSize;
                                       /* The maximum size of data transfer
                                        * for the HIM_COPY_MEMORY IOB function.
                                        */
   HIM_UINT32   AP_SCSICommandLockedSize;
                                       /* Specifies the maximum number of bytes
                                        * a SCSI CDB can be in a HIM_MC_LOCKED
                                        * or HIM_MC_UNLOCKED buffer.
                                        */
   HIM_UINT16   AP_MinEstSCSICommandSize;
                                       /* The minimum size of the targetCommand
                                        * buffer for HIM_ESTABLISH_SCSI_CONNECTION
                                        * IOBs to avoid
                                        * HIM_IOB_TARGETCOMMANDBUFFER_OVERRUN
                                        * taskStatus results.
                                        */
   /* Recommended IOB timeout values */
   HIM_UINT16   AP_AdapterAutoConfigTimeout; /* HIM recommendation,in seconds */
   HIM_UINT16   AP_PortAutoConfigTimeout;    /* HIM recommendation,in seconds */
   HIM_UINT16   AP_InitiateSMPTaskTimeout;   /* HIM recommendation,in seconds */
   HIM_UINT16   AP_ResetPortTimeout;         /* HIM recommendation,in seconds */
   HIM_UINT16   AP_ResetAdapterTimeout;      /* HIM recommendation,in seconds */
   HIM_UINT16   AP_EnablePhyTimeout;         /* HIM recommendation,in seconds */
   HIM_UINT16   AP_CopyMemoryTimeout;        /* HIM recommendation,in seconds */
} HIM_AP_IOB_PAGE;

/* Flash Adapter Profile Page */
typedef struct HIM_AP_FLASH_PAGE_ 
{
   HIM_BOOLEAN  AP_FlashPresent;
   HIM_BOOLEAN  AP_FlashReadOnly;
   HIM_UEXACT16 AP_FlashManufacturerID;
   HIM_UEXACT16 AP_FlashDeviceID;

} HIM_AP_FLASH_PAGE;

/* BIOS Adapter Profile Page */
typedef struct HIM_AP_BIOS_PAGE_
{
   HIM_BOOLEAN  AP_BiosActive;
   HIM_UINT8    AP_LowestScanLu;
   HIM_UINT8    AP_BiosVersionFormat;
   struct HIM_BIOS_VERSION_
   {
      union
      {
         struct HIM_BIOS_VERSION_FORMAT1_
         {
            HIM_UINT8   AP_MajorNumber;
            HIM_UINT8   AP_MinorNumber;
            HIM_UINT32  AP_BuildNumber;
         } BV_FORMAT1;
      } himu;
   } AP_BiosVersion;
} HIM_AP_BIOS_PAGE;

/* PCI Adapter Profile Page */
typedef struct HIM_AP_PCI_PAGE_
{
   HIM_BOOLEAN  AP_ClearConfigurationStatus; 
                                       /* For PCI/PCI-X error recovery. If
                                        * HIM_TRUE (default), the HIM clears
                                        * the PCI Config space's Status
                                        * register(s). If HIM_FALSE the OSM has
                                        * accepted the responsibility for
                                        * clearing the Status register(s)
                                        * prior to sending the a
                                        * HIM_RESET_HARDWARE IOB.
                                        */  
   HIM_UINT8    AP_SlotNumber;         /* Physical placement of the adapter's 
                                        * slot on the motherboard.
                                        */
   HIM_BOOLEAN AP_EnableSplitTransactionTimer;
                                       /* If the adapter hardware implements 
                                        * a timer for a PCI/X split transaction, 
                                        * the value of this field shall determine 
                                        * whether the timer is enabled or 
                                        * disabled.
                                        */
} HIM_AP_PCI_PAGE;

/* SAS General Adapter Profile Page */
typedef struct HIM_AP_SAS_GENERAL_PAGE_
{
   HIM_UEXACT8   AP_SASBasePortID[HIM_SAS_ADDRESS_SIZE];
   HIM_UEXACT8   AP_SASPortIDMask;
   HIM_UEXACT8   AP_SASBaseDeviceName[HIM_SAS_ADDRESS_SIZE];
   HIM_UEXACT8   AP_SASDeviceNameMask;
} HIM_AP_SAS_GENERAL_PAGE;

typedef struct HIM_AP_SATA_GENERAL_PAGE_
{
   HIM_BOOLEAN   AP_SATADMAForPIOTransfersSupported;
} HIM_AP_SATA_GENERAL_PAGE;

/* Initiator Mode specific Profile pages */
/* Initiator Mode Adapter Profile Page */
typedef struct HIM_AP_INITIATOR_MODE_PAGE_
{
   HIM_UINT16    AP_NumTargetPorts;
   HIM_UINT16    AP_NumInternalScanResources;
} HIM_AP_INITIATOR_MODE_PAGE;

/* SAS Initiator Mode Adapter Profile Page */
typedef struct HIM_AP_SAS_INITIATOR_MODE_PAGE_
{
   HIM_UINT16    AP_SASNumExpanders;   /* Indicates the maximum number of 
                                        * expanders the HIM has been 
                                        * configured to operate.
                                        */
   HIM_UINT8     AP_SASNumPhysPerExpander;
                                       /* Indicates the maximum number of
                                        * phys for an expander the HIM
                                        * has been configured to operate.
                                        */
   HIM_UINT8     AP_SASWidePortRateMismatch;
                                       /* This field allows the OSM to choose
                                        * the behaviour desired if the HIM
                                        * detects a rate mismatch within a
                                        * SAS wide link.
                                        * Possible values are:
                                        * HIM_WPRM_OPERATE_WITH_MISMATCH
                                        * HIM_WPRM_HIGHEST_COMMON_RATE
                                        * HIM_WPRM_HIGHEST_RATE
                                        */
   HIM_BOOLEAN   AP_SASExpanderDiscoveryOptimization;
                                       /* This field allows the OSM to
                                        * choose the expander configuration
                                        * method used during PAC.
                                        * If set to HIM_TRUE (default), the
                                        * expander discovery optimization
                                        * defined in SAS 1.1 shall be enabled.
                                        * If set to HIM_FALSE, the expander
                                        * discovery method defined in SAS
                                        * 1.0 shall be enabled.
                                        */
   HIM_UINT8     AP_SASResetEndDeviceInitiatorProtocol;
                                       /* This field allows the OSM to
                                        * select the set of device ports
                                        * reset while performing HIM_RESET_PORT
                                        * and HIM_RESET_ADAPTER IOBs. This
                                        * field only applies to device ports
                                        * with DP_DeviceType equal to
                                        * HIM_END_DEVICE.
                                        */
} HIM_AP_SAS_INITIATOR_MODE_PAGE;

/* Target mode specific profile pages */
/* Target Mode Adapter Profile Page */
typedef struct HIM_AP_TARGET_MODE_PAGE_
{
   HIM_UINT16    AP_NumInitiatorPorts;
   HIM_UINT16    AP_NumInternalRequestBuffers;
} HIM_AP_TARGET_MODE_PAGE;

/* SAS SSP Target Mode Adapter Profile Page */
typedef struct HIM_AP_SAS_SSP_TARGET_MODE_PAGE_
{
   HIM_BOOLEAN   AP_SAS_SSPCheckTargetPortXferTag;
   HIM_BOOLEAN   AP_SAS_SSPTransportLayerRetriesSupported;
   HIM_BOOLEAN   AP_SAS_SSPTransportLayerRetriesEnabled;
   HIM_BOOLEAN   AP_SAS_SSPReadyLEDMeaning;
   HIM_UINT16    AP_SAS_SSPBusInactivityLimit;
   HIM_UINT16    AP_SAS_SSPMaxConnectTimeLimit;
   HIM_UINT16    AP_SAS_SSPInitResponseTimeout;
   HIM_UINT16    AP_SAS_SSP_I_T_NexusLossTime;
} HIM_AP_SAS_SSP_TARGET_MODE_PAGE;

/* SCSI Target Mode Adapter Profile Page */
typedef struct HIM_AP_SCSI_TARGET_MODE_PAGE_
{
   HIM_UINT16    AP_NumSCSINexusTSHs;
   HIM_UINT16    AP_MaxReceiveCdbLength;
   HIM_UINT8     AP_SCSITaskSetType;
   HIM_BOOLEAN   AP_SCSIAutoResponseNoResource;
   HIM_BOOLEAN   AP_SCSICheckReservedFieldsForZero;
   HIM_BOOLEAN   AP_SCSIOutofOrderTransfers;
   HIM_UINT32    AP_NumEstablishSCSIConnectionIOBs;
   HIM_UINT32    AP_NumAvailSCSINexusTSHs;
} HIM_AP_SCSI_TARGET_MODE_PAGE;

/* SMP Target Mode Adapter Profile Page */
typedef struct HIM_AP_SMP_TARGET_MODE_PAGE_
{
   HIM_UINT16    AP_NumSMPNexusTSHs;
   HIM_UINT16    AP_MaxReceiveSMPRequestLength;
   HIM_UINT32    AP_NumEstablishSMPConnectionIOBs;
   HIM_UINT32    AP_NumAvailSMPNexusTSHs;
} HIM_AP_SMP_TARGET_MODE_PAGE;

/***************************************************************************
* Definitions for General Adapter Profile Page
***************************************************************************/
/* Interrupt Method defines */
#define HIM_IM_POLLING_ONLY            0x00
#define HIM_IM_INTERRUPT_PIN           0x01
#define HIM_IM_MULTIPLE_INTERRUPT_PINS 0x02
#define HIM_IM_MSI                     0x03
#define HIM_IM_MSIX                    0x04

#else /* SAS_DOWNSHIFT_SUPPORT */

typedef struct HIM_AP_GENERAL_PAGE_
{
   HIM_UINT8    AP_InitiatorProtocol;
   HIM_UEXACT8  AP_WorldWideName[8];
} HIM_AP_GENERAL_PAGE;

/* Resource Limits Adapter Profile Page */
typedef struct HIM_AP_RESOURCE_LIMITS_PAGE_
{
   HIM_UINT8    AP_NumPhys;
   HIM_UINT8    AP_NumPorts;
} HIM_AP_RESOURCE_LIMITS_PAGE;

#endif /* !SAS_DOWNSHIFT_SUPPORT */

/***************************************************************************
* Definitions for Port PROFILE Pages
***************************************************************************/
#define HIM_MAJOR_VERSION_PORT_PROFILE   0
#define HIM_MINOR_VERSION_PORT_PROFILE   0

/* Page Code defines */
#define HIM_PPC_GENERAL                1
#define HIM_PPC_SAS_GENERAL            2
#define HIM_PPC_SATA_GENERAL           3 
 
/* Initiator Mode specific Page Code defines */
#define HIM_PPC_SAS_INITIATOR_MODE     0x10

/* Target Mode specific Page Code defines */
#define HIM_PPC_SAS_SSP_TARGET_MODE    0x20

/* Page structures */
/* General Port Profile Page */
#if !SAS_DOWNSHIFT_SUPPORT
typedef struct HIM_PP_GENERAL_PAGE_
{
   HIM_UINT16   PP_MajorVersion;
   HIM_UINT16   PP_MinorVersion;
   HIM_UINT8    PP_Transport;
   HIM_UINT8    PP_InitiatorProtocol;
   HIM_UINT8    PP_TargetProtocol;
   HIM_UINT8    PP_PortState;
   HIM_UINT8    PP_LinkWidth;
   HIM_UINT8    PP_AttachedDeviceType;
   HIM_UINT8    PP_AttachedInitiatorProtocol;
   HIM_UINT8    PP_AttachedTargetProtocol;
} HIM_PP_GENERAL_PAGE;

/* SAS General Port Profile Page */
typedef struct HIM_PP_SAS_GENERAL_PAGE_
{
   HIM_UEXACT32   PP_SASPhyIDBitMask;
   HIM_UEXACT8    PP_SASAttachedAddress[HIM_SAS_ADDRESS_SIZE];
                                                   /* SAS address of the attached
                                                    * device.
                                                    */
} HIM_PP_SAS_GENERAL_PAGE;

/* SATA General Port Profile Page */
typedef struct HIM_PP_SATA_GENERAL_PAGE_
{
   HIM_UINT8    PP_SATAPhyID; 
} HIM_PP_SATA_GENERAL_PAGE;


/* SAS Initiator Mode Port Profile Page */
typedef struct HIM_PP_SAS_INITIATOR_MODE_PAGE_
{
   HIM_UINT16   PP_SASNumSSPInitiators;
   HIM_UINT16   PP_SASNumSMPInitiators;
   HIM_UINT16   PP_SASNumSTPInitiators;
} HIM_PP_SAS_INITIATOR_MODE_PAGE;

/* SAS SSP Target Mode Port Profile Page */
typedef struct HIM_PP_SAS_SSP_TARGET_MODE_PAGE_
{
   HIM_UINT16   PP_SAS_SSPBusInactivityLimit;
   HIM_UINT16   PP_SAS_SSPMaxConnectTimeLimit;
   HIM_UINT16   PP_SAS_SSPInitResponseTimeout;
   HIM_UINT16   PP_SAS_SSP_I_T_NexusLossTime;
} HIM_PP_SAS_SSP_TARGET_MODE_PAGE;

#endif /* !SAS_DOWNSHIFT_SUPPORT */


#define HIM_MAJOR_VERSION_PHY_PROFILE 0
#define HIM_MINOR_VERSION_PHY_PROFILE 0

/***************************************************************************
* Definitions for PHY PROFILE Pages
***************************************************************************/

/* Page Code defines */
#define HIM_YPC_GENERAL                0x00
#define HIM_YPC_CONNECTOR              0x01
#define HIM_YPC_SAS_GENERAL            0x02
#define HIM_YPC_SAS_STATISTICS         0x03
#define HIM_YPC_SATA_STATISTICS        0x04
#define HIM_YPC_SAS_HARDWARE_CONTROL   0x05
#define HIM_YPC_SATA_HARDWARE_CONTROL  0x06
#define HIM_YPC_STP_GENERAL            0x07


/* Page Code specific to initiator mode defines */
#define HIM_YPC_SAS_INITIATOR_MODE     0x10
#define HIM_YPC_SATA_INITIATOR_MODE    0x11

/* Page Code specific to target mode defines */
#define HIM_YPC_SAS_TARGET_MODE        0x20

#if !SAS_DOWNSHIFT_SUPPORT
/* Page structures */
typedef struct HIM_YP_GENERAL_PAGE_
{
   HIM_UINT16              YP_MajorVersion;
   HIM_UINT16              YP_MinorVersion; 
   HIM_UINT8               YP_Transport;
   HIM_UINT8               YP_EnabledTransports; 
   HIM_UINT8               YP_InitiatorProtocol; 
   HIM_UINT8               YP_TargetProtocol; 
   HIM_UINT8               YP_PhyState; 
   HIM_TASK_SET_HANDLE     YP_PortTSH; 
   HIM_BOOLEAN             YP_VirtualPhy; 
   HIM_UINT32              YP_NegotiatedPhyLinkRate;
} HIM_YP_GENERAL_PAGE;

#endif /* !SAS_DOWNSHIFT_SUPPORT */

/* This page is used for DOWNSHIFT mode */
typedef struct HIM_YP_CONNECTOR_PAGE_
{
   HIM_UINT8               YP_ConnectorType; 
   HIM_UINT8               YP_ConnectorLaneNum; 
   HIM_UINT8               YP_ConnectorLocation; 
   HIM_UINT8               YP_ConnectorIndex; 
   HIM_UEXACT8             YP_ConnectorName[16];
} HIM_YP_CONNECTOR_PAGE;

#if !SAS_DOWNSHIFT_SUPPORT

typedef struct HIM_YP_SAS_GENERAL_PAGE_
{
   HIM_UEXACT8             YP_SASAddress[HIM_SAS_ADDRESS_SIZE];
   HIM_UEXACT8             YP_SASGroupingIndex;
   HIM_UEXACT8             YP_SASPhyID;
   HIM_UEXACT8             YP_SASAttachedPhyID;
   HIM_UINT8               YP_SASMaxIdentifySequenceRetries;  
} HIM_YP_SAS_GENERAL_PAGE;

typedef struct HIM_YP_SAS_STATISTICS_PAGE_
{
   HIM_BOOLEAN             YP_SASResetPhyErrorCounters; 
   HIM_UEXACT32            YP_SASNumInvalidDwords;
   HIM_UEXACT32            YP_SASNumDisparityErrors;
   HIM_UEXACT32            YP_SASNumLossOfDwordSync;
   HIM_UEXACT32            YP_SASNumPhyResetProblem;
   HIM_UEXACT32            YP_SASNumIdentifySequenceProblem;
   HIM_UEXACT32            YP_SASNumBroadcastChangeRx;
} HIM_YP_SAS_STATISTICS_PAGE;

typedef struct HIM_YP_SATA_STATISTICS_PAGE_
{
   HIM_BOOLEAN             YP_SATAResetPhyErrorCounters; 
   HIM_UINT32              YP_SATANumRecDataErrors;
   HIM_UINT32              YP_SATANumRecCommErrors;
   HIM_UINT32              YP_SATANumTransientUnrecDataErrors;
   HIM_UINT32              YP_SATANumPersistentUnrecErrors;
   HIM_UINT32              YP_SATANumProtocolErrors;
   HIM_UINT32              YP_SATANumUnknownFISRx;
   HIM_UINT32              YP_SATANumTxCRCErrors;
   HIM_UINT32              YP_SATANumRxCRCErrors;
   HIM_UINT32              YP_SATANumDisparityErrors;
   HIM_UINT32              YP_SATANumInvalidDwords;
} HIM_YP_SATA_STATISTICS_PAGE;

typedef struct HIM_YP_SAS_HARDWARE_CONTROL_PAGE_
{
   HIM_UINT32              YP_SASProgrammedMinPhyLinkRate;
   HIM_UINT32              YP_SASProgrammedMaxPhyLinkRate;
   HIM_UINT32              YP_SASHardwareMinPhyLinkRate;
   HIM_UINT32              YP_SASHardwareMaxPhyLinkRate;

   HIM_BOOLEAN             YP_SASDisableCRCChecking; 
   HIM_UINT16              YP_SASAlignRate;
   HIM_UINT16              YP_SASHotPlugTimeout;
   HIM_UINT16              YP_SASComInitTimeout;

   /* Receiver phy settings */
   HIM_SINT8               YP_SASRxCommonModeIndex;
   HIM_UINT16              YP_SASRxCommonModeValue;
   HIM_SINT8               YP_SASRxSquelchDetectIndex;
   HIM_UINT16              YP_SASRxSquelchDetectValue;
   HIM_BOOLEAN             YP_SASEnableAutomaticRxOffsetCancellation; 

   /* Transmitter phy settings */
   HIM_SINT8               YP_SASTxDriveAmplitudeIndex;
   HIM_UINT16              YP_SASTxDriveAmplitudeValue;
   HIM_SINT8               YP_SASTxPreemphasisIndex;
   HIM_UINT16              YP_SASTxPreemphasisValue;
   HIM_SINT8               YP_SASTxSlewRateIndex;
   HIM_UINT16              YP_SASTxSlewRateValue;
   HIM_BOOLEAN             YP_SASReduceTxDriveLevel; 
   HIM_BOOLEAN             YP_SASTxInternalCommonVoltage; 
} HIM_YP_SAS_HARDWARE_CONTROL_PAGE;

typedef struct HIM_YP_SATA_HARDWARE_CONTROL_PAGE_
{
   HIM_UINT32              YP_SATAProgrammedMinPhyLinkRate;
   HIM_UINT32              YP_SATAProgrammedMaxPhyLinkRate;
   HIM_UINT32              YP_SATAHardwareMinPhyLinkRate;
   HIM_UINT32              YP_SATAHardwareMaxPhyLinkRate;

   HIM_BOOLEAN             YP_SATADisableCRCChecking; 
   HIM_UINT16              YP_SATAAlignRate;
   HIM_UINT16              YP_SATAComInitTimeout;

   /* Receiver phy settings */
   HIM_SINT8               YP_SATARxCommonModeIndex;
   HIM_UINT16              YP_SATARxCommonModeValue;
   HIM_SINT8               YP_SATARxSquelchDetectIndex;
   HIM_UINT16              YP_SATARxSquelchDetectValue;
   HIM_BOOLEAN             YP_SATAEnableAutomaticRxOffsetCancellation; 

   /* Transmitter phy settings */
   HIM_SINT8               YP_SATATxDriveAmplitudeIndex;
   HIM_UINT16              YP_SATATxDriveAmplitudeValue;
   HIM_SINT8               YP_SATATxPreemphasisIndex;
   HIM_UINT16              YP_SATATxPreemphasisValue;
   HIM_SINT8               YP_SATATxSlewRateIndex;
   HIM_UINT16              YP_SATATxSlewRateValue;
   HIM_BOOLEAN             YP_SATAReduceTxDriveLevel; 
   HIM_BOOLEAN             YP_SATATxInternalCommonVoltage; 
} HIM_YP_SATA_HARDWARE_CONTROL_PAGE;

typedef struct HIM_YP_STP_GENERAL_PAGE_
{
   HIM_UINT32              YP_STPShutdownTimer;
} HIM_YP_STP_GENERAL_PAGE;

/* Page structures specific to initiator mode */
typedef struct HIM_YP_SAS_INITIATOR_MODE_PAGE_
{
   HIM_UINT16              YP_SASNotifyEnableSpinupInterval;
   HIM_UINT16              YP_SASNotifyEnableSpinupInitialDelay;
} HIM_YP_SAS_INITIATOR_MODE_PAGE;

typedef struct HIM_YP_SATA_INITIATOR_MODE_PAGE_
{
   HIM_BOOLEAN             YP_SATAEnableSpinUpHoldState;
   HIM_BOOLEAN             YP_SATAPortSelectorPresent;
} HIM_YP_SATA_INITIATOR_MODE_PAGE;

/* Page structures specific to target mode */
typedef struct HIM_YP_SAS_TARGET_MODE_PAGE_
{
   HIM_BOOLEAN             YP_SASReportEnableSpinup;
} HIM_YP_SAS_TARGET_MODE_PAGE;
#endif /* !SAS_DOWNSHIFT_SUPPORT */

/***************************************************************************
*  #defines used by Phy Profile
***************************************************************************/
/***************************************************************************
* Definitions for PHY GENERAL Pages
***************************************************************************/
/* YP_PhyState      */
#define HIM_PHY_REPORTED           0
#define HIM_PHY_DISABLED           1
#define HIM_PHY_DOWN               2
#define HIM_PHY_SPINUP_HOLD        3
#define HIM_PHY_UP                 4
#define HIM_PHY_PATTERN_GENERATION 5

/***************************************************************************
* Definitions for PHY CONNECTOR Pages
***************************************************************************/
/* Connector Type defines */
#define HIM_CT_UNKNOWN                 0x00
#define HIM_CT_SFF8470                 0x08
#define HIM_CT_SFF8482                 0x09
#define HIM_CT_SFF8484                 0x0A
#define HIM_CT_SFF8087                 0x0B
#define HIM_CT_SFF8088                 0x0C
#define HIM_CT_DEVICE                  0x0D

/* Connector Type specific to vendor unique defines */
#define HIM_CT_VENDOR_UNIQUE00         0x80
#define HIM_CT_VENDOR_UNIQUE01         0x81
#define HIM_CT_VENDOR_UNIQUE02         0x82
#define HIM_CT_VENDOR_UNIQUE03         0x83
#define HIM_CT_VENDOR_UNIQUE04         0x84
#define HIM_CT_VENDOR_UNIQUE05         0x85
#define HIM_CT_VENDOR_UNIQUE7F         0xFF

/* Connector Location defines */
#define HIM_CL_UNKNOWN                 0x00
#define HIM_CL_INTERNAL                0x01
#define HIM_CL_EXTERNAL                0x02

/* Connector Index defines */
#define HIM_CI_UNKNOWN                 (0xFF)   /* 8-bit unknown Connector Index */

#define HIM_MAJOR_VERSION_DEVICEPORT_PROFILE 0
#define HIM_MINOR_VERSION_DEVICEPORT_PROFILE 1

/***************************************************************************
* Definitions for DevicePort Profile pages
***************************************************************************/
/* Page Code defines */
#define HIM_DPC_GENERAL             1
#define HIM_DPC_IOB                 2
#define HIM_DPC_SAS_GENERAL         3
#define HIM_DPC_SAS_EXPANDER        4
#define HIM_DPC_SATA_GENERAL        5
#define HIM_DPC_SGPIO_LOCATION      6

/* Page Code defines specific to initiator mode */
#define HIM_DPC_INITIATOR_MODE      0x10
#define HIM_DPC_SAS_INITIATOR_MODE  0x11
#define HIM_DPC_SATA_INITIATOR_MODE 0x12

/* Page structures */

/* SAS Expander Device Port Profile Page */
typedef struct HIM_DP_SAS_EXPANDER_PAGE_
{
         HIM_UINT8      DP_SASNumPhys; /* Number of physical interfaces
                                        * supported by this expander if the
                                        * value of DP_DeviceType is
                                        * HIM_DEVICETYPE_EXPANDER.
                                        */
         HIM_BOOLEAN    DP_SASConfigurableRouteTable;
                                       /* If HIM_TRUE, expander device supports
                                        * a configurable route table.
                                        */
} HIM_DP_SAS_EXPANDER_PAGE;

/* General Device Port Profile Page */
#if !SAS_DOWNSHIFT_SUPPORT

typedef struct HIM_DP_GENERAL_PAGE_
{
   HIM_UINT16     DP_MajorVersion;     /* Profile major version number     */
   HIM_UINT16     DP_MinorVersion;     /* Profile minor version number     */
   HIM_UINT8      DP_Transport;        /* The basic transport layer supported
                                        * by the device port. Possible values
                                        * include HIM_TRANSPORT_SAS or
                                        * HIM_TRANSPORT_SATA.  Required to
                                        * interpret the transport-specific union
                                        * portion of the deviceport profile.
                                        */
   
   HIM_UINT8      DP_DeviceType;       /* A device type, possible values are:
                                        * HIM_NO_DEVICE             
                                        * HIM_END_DEVICE            
                                        * HIM_EDGE_EXPANDER_DEVICE  
                                        * HIM_FANOUT_EXPANDER_DEVICE
                                        * HIM_UNSUPPORTED_DEVICE
                                        * HIM_UNKNOWN_DEVICE
                                        */
   HIM_UINT8      DP_InitiatorProtocol;/* I/O protocol of initiator which
                                        * command and error data format are
                                        * understood by a device operating as
                                        * an initiator. Possible protocols:
                                        * HIM_PROTOCOL_SCSI, HIM_PROTOCOL_ATA or
                                        * HIM_PROTOCOL_SMP.
                                        */
   HIM_UINT8      DP_TargetProtocol;   /* I/O protocol of target which command
                                        * and error data format are understood
                                        * by target. Possible protocols:
                                        * HIM_PROTOCOL_SCSI, HIM_PROTOCOL_ATA or
                                        * HIM_PROTOCOL_SMP.
                                        */
   HIM_UINT8      DP_NativeTargetProtocol;
                                       /* I/O protocol of target which command
                                        * and error data format are understood
                                        * by target. Possible protocols:
                                        * HIM_PROTOCOL_SCSI, HIM_PROTOCOL_ATA or
                                        * HIM_PROTOCOL_SMP.
                                        */
   HIM_TASK_SET_HANDLE  DP_PortTSH;    /* TSH of initiator port this device
                                        * is attached.
                                        */
} HIM_DP_GENERAL_PAGE;

/* IOB Device Port Profile Page */

typedef struct HIM_DP_IOB_PAGE_
{
   /* Recommended IOB timeout values */
   HIM_UINT16     DP_AbortTaskTimeout; /* recommended timeout value,
                                        * in seconds, for a 
                                        * HIM_ABORT_TASK IOB.
                                        */
   HIM_UINT16     DP_ResetLogicalUnitTimeout;  
                                       /* recommended timeout value,
                                        * in seconds, for a 
                                        * HIM_RESET_LOGICAL_UNIT IOB.
                                        */
   HIM_UINT16     DP_ResetDevicePortTimeout;
                                       /* recommended timeout value,
                                        * in seconds, for a 
                                        * HIM_RESET_DEVICE_PORT IOB.
                                        */
   HIM_UINT16     DP_ProbeLuTimeout;   /* recommended timeout value, 
                                        * in seconds, for a 
                                        * HIM_PROBE_LU IOB.
                                        */
   HIM_UINT16     DP_InitializeDevicePortTimeout;
                                       /* recommended timeout value,
                                        * in seconds, for a 
                                        * HIM_INITIALIZE_DEVICE_PORT IOB.
                                        */
   HIM_UINT16     DP_PerformLuRecoveryTimeout;
                                       /* recommended timeout value,
                                        * in seconds, for a
                                        * HIM_PERFORM_LU_RECOVERY IOB.
                                        */
} HIM_DP_IOB_PAGE;

/* SAS General Device Port Profile Page */
typedef struct HIM_DP_SAS_GENERAL_PAGE_
{
   HIM_UINT16   DP_SASI_T_NexusLossTime;
                                       /* This field indicates the amount
                                        * of time (in milliseconds) that 
                                        * connection requests are retried 
                                        * by the adapter to this device port 
                                        * before recognizing an I_T nexus loss
                                        * condition has occurred (see SAM-3).
                                        * This field shall default with a 
                                        * value of 2 000 ms (i.e. 2 seconds).
                                        * A value of 0000h indicates an I_T 
                                        * nexus loss condition is to occur 
                                        * immediately or with a minimum 
                                        * timeout value. A value of FFFFh
                                        * indicates connection requests are
                                        * to be retried indefiniatly for 
                                        * this device port. The SAS CHIM 
                                        * implementation shall support this 
                                        * field.
                                        */
   HIM_UEXACT8    DP_SASAddress[HIM_SAS_ADDRESS_SIZE];
                                       /* Unique ID of this SAS device */
} HIM_DP_SAS_GENERAL_PAGE;

/* SATA General Device Port Profile Page */
typedef struct HIM_DP_SATA_GENERAL_PAGE_
{
   HIM_UEXACT8    DP_SATAPortID[HIM_SAS_ADDRESS_SIZE];
                                       /* Port identifier device */
} HIM_DP_SATA_GENERAL_PAGE;

/* Initiator mode specific device port pages */

/* Initiator Mode Device Port Profile Page */
typedef struct HIM_DP_INITIATOR_MODE_PAGE_
{
   HIM_TASK_SET_HANDLE  DP_AttachedDevicePortTSH;
                                       /* device port TSH of the expander
                                        * attached to the target.
                                        */
   HIM_BOOLEAN    DP_AttachedVirtualPhy; 
                                       /* If HIM_TRUE, indicates the target is
                                        * attached to the Virtual Phy of an
                                        * expander device.
                                        * If HIM_FALSE, the target is attached
                                        * to the external Phy of an expander
                                        * device.  The field is valid only if
                                        * DP_AttachedDevicePortTSH is non-zero.
                                        */
   /* modifiable fields */
   HIM_UINT16     DP_ResetSettleDelay; /* Specifies the number of milliseconds
                                        * the HIM shall delay before attempting
                                        * to communicate witha device port
                                        * following a device port reset.
                                        * This delay shall occur once during
                                        * HIM_RESET_DEVICE_PORT, and may occur
                                        * multiple times during HIM_RESET_PORT
                                        * and HIM_RESET_ADAPTER.
                                        * The default value of this field
                                        * is transport specific.
                                        */
} HIM_DP_INITIATOR_MODE_PAGE;

/* SAS Initiator Mode Device Port Profile Page */
typedef struct HIM_DP_SAS_INITIATOR_MODE_PAGE_
{
   HIM_UINT32     DP_SASDefaultConnectionRate;
                                       /* The currently negotiated link rate
                                        * between the target and its attached
                                        * physical interface.
                                        */
   HIM_UINT32     DP_SASEffectivePhyLinkRate;
                                       /* Specifies the requested connection
                                        * rate when requesting a connection to
                                        * a SAS target port.
                                        */
   HIM_UINT32     DP_SASNumHops;          /* The number of links between the 
                                        * devicePort and the port.
                                        */    
   HIM_UINT32     DP_SASDiscoveryIndex; /* Order devicePort was discovered 
                                         * using PAC
                                         */
   HIM_BOOLEAN    DP_SASEnableTransportLayerRetries;
                                        /* SAS device setting for transport
                                         * layer retries. If HIM_TRUE, the
                                         * HIM shall enable transport layer
                                         * retries if the feature is supported
                                         * by the device.
                                         */
   HIM_BOOLEAN    DP_SASTransportLayerRetriesEnabled;
                                        /* If HIM_TRUE, indicates transport
                                         * layer retries are supported by
                                         * the device port and port, and has
                                         * been successfully enabled.
                                         */
   HIM_UEXACT8    DP_SASAttachedPhyIDMask[HIM_SAS_MAX_PHYIDS_PER_DEVICE/8];
                                        /* A bitmask representing the phy
                                         * identifier(s) of the port
                                         * attached to this device port.
                                         */
   HIM_UEXACT8    DP_SASPhyIDMask[HIM_SAS_MAX_PHYIDS_PER_DEVICE/8];
                                        /* A bitmask representing the phy
                                         * identifier(s) of the port
                                         * of this device port.
                                         */
} HIM_DP_SAS_INITIATOR_MODE_PAGE;

/* SATA Initiator Mode Device Port Profile Page */
typedef struct HIM_DP_SATA_INITIATOR_MODE_PAGE_
{
   /* modifiable fields */
   HIM_BOOLEAN    DP_SATAAutoDMAActivate;
                                       /* If HIM_TRUE, SATA-2 auto-activate is
                                        * enabled.
                                        */
   HIM_UINT8      DP_SATAAdjustWriteCache;
                                       /* This field indicates the
                                        * SATA write cache enabled.
                                        */
   HIM_BOOLEAN    DP_SATAEnableNativeCommandQueuing;
                                       /* If HIM_TRUE (default if supported),
                                        * the HIM shall enable native command
                                        * queuing in the SATA device if the
                                        * feature is supported by the device.
                                        */
   HIM_UINT8      DP_SATACommandFeatureSet;
                                       /* This field indicates the
                                        * results of the SATA device
                                        * signature. Supported Values are
                                        * HIM_SATA_CFS_INVALID     
                                        * HIM_SATA_CFS_UNKNOWN     
                                        * HIM_SATA_CFS_ATA         
                                        * HIM_SATA_CFS_ATAPI       
                                        * HIM_SATA_CFS_SEP         
                                        * HIM_SATA_CFS_PORT_MULTIPLIER
                                        */
   HIM_UEXACT8    DP_SATAAttachedPhyID; /* The phy identifier of the device to
                                         * which this device port is attached.
                                         */
   HIM_UINT32     DP_SATADiscoveryIndex; /* Order devicePort was discovered 
                                         * using PAC
                                         */
   HIM_UEXACT8    DP_SATASignatureFIS[20];
                                       /* This field contains the register
                                        * Device to Host FIS with the 
                                        * device signature returned from 
                                        * the SATA device.
                                        */
   HIM_UINT32     DP_SATANegotiatedLinkRate;
                                       /* Specifies the negotiated connection
                                        * rate established to a SATA target port.
                                        */
   HIM_UEXACT8    DP_SATAPhyID;        /* The phy identifier of the port
                                        * of this device port.
                                        */
} HIM_DP_SATA_INITIATOR_MODE_PAGE;

#else /* SAS_DOWNSHIFT_SUPPORT */

typedef struct HIM_DP_GENERAL_PAGE_
{
   HIM_UINT8      DP_Transport;        /* The basic transport layer supported
                                        * by the device port. Possible values
                                        * include HIM_TRANSPORT_SAS or
                                        * HIM_TRANSPORT_SATA.  Required to
                                        * interpret the transport-specific union
                                        * portion of the deviceport profile.
                                        */
   
   HIM_UINT8      DP_DeviceType;       /* A device type, possible values are:
                                        * HIM_NO_DEVICE             
                                        * HIM_END_DEVICE            
                                        * HIM_EDGE_EXPANDER_DEVICE  
                                        * HIM_FANOUT_EXPANDER_DEVICE
                                        * HIM_UNSUPPORTED_DEVICE  
                                        */

   HIM_UINT8      DP_TargetProtocol;   /* I/O protocol of target which command
                                        * and error data format are understood
                                        * by target. Possible protocols:
                                        * HIM_PROTOCOL_SCSI, HIM_PROTOCOL_ATA or
                                        * HIM_PROTOCOL_SMP.
                                        */
} HIM_DP_GENERAL_PAGE;

/* Initiator Mode Device Port Profile Page */
typedef struct HIM_DP_INITIATOR_MODE_PAGE_
{
   HIM_BOOLEAN    DP_AttachedVirtualPhy; 
                                       /* If HIM_TRUE, indicates the target is
                                        * attached to the Virtual Phy of an
                                        * expander device.
                                        * If HIM_FALSE, the target is attached
                                        * to the external Phy of an expander
                                        * device.  The field is valid only if
                                        * DP_AttachedDevicePortTSH is non-zero.
                                        */

} HIM_DP_INITIATOR_MODE_PAGE;

#endif /* !SAS_DOWNSHIFT_SUPPORT */

/* SGPIO Location Device Port Profile Page */
typedef struct HIM_DP_SGPIO_LOCATION_PAGE_
{
  HIM_UINT8     DP_ConnectorIndex;
                                       /* This field indicates the
                                        * instance of the connector
                                        */
  HIM_UINT8     DP_BoxID;
                                       /* This field indicates the
                                        * box identifier
                                        */
  HIM_UINT8     DP_BayID;
                                       /* This field indicates the
                                        * SGPIO bay identifier
                                        */
  HIM_UINT16    DP_TargetID;
                                       /* This field indicates the
                                        * HIM derived target ID
                                        */
} HIM_DP_SGPIO_LOCATION_PAGE;


#define HIM_BOXID_UNKNOWN              (0xFF)   /* 8-bit unknown Box Number */
#define HIM_BAYID_UNKNOWN              (0x00)   /* 8-bit unknown Bay Number */
#define HIM_TARGETID_UNKNOWN           (0xFFFF) /*16-bit unknown Target ID  */

/***************************************************************************
* Definitions for Logical Unit PROFILE Pages
***************************************************************************/
#define HIM_MAJOR_VERSION_LU_PROFILE   1
#define HIM_MINOR_VERSION_LU_PROFILE   0

#define HIM_SATA_WWID_SIZE    16
/* Page Code defines */
#define HIM_LPC_GENERAL                1
#define HIM_LPC_SATA_GENERAL           2

/* Page structures */
/* General Logical Unit Profile Page */
#if !SAS_DOWNSHIFT_SUPPORT

typedef struct HIM_LP_GENERAL_PAGE_
{
   HIM_UINT16            LP_MajorVersion;  /* Profile major version number     */
   HIM_UINT16            LP_MinorVersion;  /* Profile minor version number     */
   HIM_UEXACT8           LP_Lun[HIM_LUN_SIZE];/* 8-byte LUN defined in SAM-3   */
   HIM_UINT16            LP_MaxActiveCommands;/* Max number of commands HIM can
                                               * have simultaneously pending to the
                                               * logical unit. This number can be
                                               * changed by the OSM or by the HIM
                                               * in order to avoid task-set-full
                                               * conditions.
                                               */
   HIM_UINT8             LP_SortMethod;
   HIM_TASK_SET_HANDLE   LP_DevicePortTSH; /* device port task set handle      */
   HIM_BOOLEAN    LP_AutoSenseNotSupported; 
                                       /* If HIM_TRUE, indicates the target
                                        * does not support Auto Sense feature.
                                        */
} HIM_LP_GENERAL_PAGE;

/* SATA General Logical Unit Profile Page */
typedef struct HIM_LP_SATA_GENERAL_PAGE_
{
   HIM_BOOLEAN   LP_SATAAutoDMAActivateEnabled;
                                          /* If HIM_TRUE, SATA-2 auto-activate
                                           * is enabled.
                                           */
   HIM_BOOLEAN   LP_SATAWriteCacheEnabled;
                                          /* If HIM_TRUE, indicates the SATA
                                           * write cache enabled.
                                           */
   
   HIM_BOOLEAN   LP_SATANativeCommandQueuingEnabled;
                                          /* If HIM_TRUE, indicates the SATA
                                           * will use ATA tagged queuing.
                                           */
   HIM_BOOLEAN   LP_SATAFlushCacheAfterWrites;
                                          /* If HIM_TRUE, selected Write cmds
                                           * are followed by flush Write cache
                                           * commands
                                           */
   HIM_UINT8     LP_SATAATAPICommandPacketLength;
   HIM_UEXACT8   LP_SATAWorldWideID[HIM_SATA_WWID_SIZE];
} HIM_LP_SATA_GENERAL_PAGE;

#else /* SAS_DOWNSHIFT_SUPPORT */

typedef struct HIM_LP_GENERAL_PAGE_
{
   HIM_UEXACT8          LP_Lun[HIM_LUN_SIZE];/* 8-byte LUN defined in SAM-3    */
} HIM_LP_GENERAL_PAGE;

#endif /* !SAS_DOWNSHIFT_SUPPORT */

/***************************************************************************
* Definitions for NEXUS PROFILE Pages
***************************************************************************/
#define HIM_MAJOR_VERSION_NEXUS_PROFILE   0
#define HIM_MINOR_VERSION_NEXUS_PROFILE   0

/* Page Code defines */
#define HIM_XPC_GENERAL        1
#define HIM_XPC_CONTEXT        2
#define HIM_XPC_SCSI_PROTOCOL  3
#define HIM_XPC_SMP_PROTOCOL   4
#define HIM_XPC_SAS_GENERAL    5
#if !SAS_DOWNSHIFT_SUPPORT
/* Page structures */
typedef struct HIM_XP_GENERAL_PAGE_
{
   HIM_UINT16   XP_MajorVersion;
   HIM_UINT16   XP_MinorVersion; 
   HIM_UINT8    XP_Transport;
   HIM_UINT8    XP_Protocol; 
} HIM_XP_GENERAL_PAGE;

typedef struct HIM_XP_CONTEXT_PAGE_
{
   HIM_TASK_SET_HANDLE   XP_AdapterTSH;
   HIM_TASK_SET_HANDLE   XP_DevicePortTSH;
   HIM_TASK_SET_HANDLE   XP_PortTSH;
   HIM_TASK_SET_HANDLE   XP_PhyTSH;
   void HIM_PTR          XP_OSMDevicePortContext;
} HIM_XP_CONTEXT_PAGE;

typedef struct HIM_XP_SCSI_PROTOCOL_PAGE_
{
   HIM_UEXACT8   XP_SCSILun[8];
   HIM_UINT8     XP_SCSITaskAttribute;
   HIM_UEXACT16  XP_SCSITag;
   HIM_BOOLEAN   XP_SCSIFirstBurstEnabled;
} HIM_XP_SCSI_PROTOCOL_PAGE;

/* Nothing defined - therefore make a dummy page for now */
typedef struct HIM_XP_SMP_PROTOCOL_PAGE_
{
   HIM_UEXACT8   XP_SMPDummy;
} HIM_XP_SMP_PROTOCOL_PAGE;

typedef struct HIM_XP_SAS_GENERAL_PAGE_
{
   HIM_UEXACT8   XP_SASInitiatorPortAddress[HIM_SAS_ADDRESS_SIZE];
   HIM_UEXACT8   XP_SASTargetPortAddress[HIM_SAS_ADDRESS_SIZE];
} HIM_XP_SAS_GENERAL_PAGE;
#endif /* !SAS_DOWNSHIFT_SUPPORT */

/***************************************************************************
*  #defines used by Adapter Profile
***************************************************************************/
/* AP_Transport, PP_Transport, YP_Transport, DP_Transport, LP_Transport */
/* and XP_Transport */
#define HIM_TRANSPORT_SAS        0x01
#define HIM_TRANSPORT_SATA       0x02
#define HIM_TRANSPORT_SGPIO      0x04
#define HIM_TRANSPORT_I2C        0x08
#define HIM_TRANSPORT_GPIO       0x10
/* AP_UsageModelID */
#define HIM_USAGEMODEL_DEFAULT   0
#define HIM_USAGEMODEL_HIGHEST_NUMBER 0xff
#define HIM_USAGEMODEL_LOWEST_NUMBER HIM_USAGEMODEL_DEFAULT
/* AP_Memorymapped            */
#define HIM_IOSPACE              0
#define HIM_MEMORYSPACE          1
#define HIM_NO_CHOICE_AVAILABLE  2

/* AP_BiosVersionFormat      */
#define HIM_BV_FORMAT1           1
#define HIM_BV_FORMAT_UNKNOWN    0xff

/* AP_LowestScanLu           */
#define HIM_LU_SCANORDER_UNKNOWN 0xff
/* AP_InitiatorModeProtocol, AP_TargetModeProtocol */
#define HIM_PROTOCOL_SCSI        0x01
#define HIM_PROTOCOL_ATA         0x02
#define HIM_PROTOCOL_SMP         0x04
#define HIM_PROTOCOL_ATAPI       0x08
#define HIM_PROTOCOL_I2C         0x10

/* AP_SASWidePortRateMismatch */
#define HIM_WPRM_HIGHEST_COMMON_RATE   0
#define HIM_WPRM_HIGHEST_RATE          1
#define HIM_WPRM_OPERATE_WITH_MISMATCH 2

/* AP_FlashManufacturerID */
#define HIM_INVALID_FLASH_MANUFACTURER_ID 0xffff

/* AP_FlashDeviceID */
#define HIM_INVALID_FLASH_DEVICE_ID  0xffff

/* AP_SCSITaskSetType - Mode Sense/Select Control Mode page - TST field */
#define HIM_SINGLE_TASK_SET        0   /* (TST = 0) */
#define HIM_SEPARATE_TASK_SETS     1   /* (TST = 1) */

/***************************************************************************
*  #defines used by Port and DevicePort Profile
***************************************************************************/
/* PP_AttachedDeviceType      */
/* DP_DeviceType              */
#define HIM_NO_DEVICE              0
#define HIM_END_DEVICE             1
#define HIM_EDGE_EXPANDER_DEVICE   2
#define HIM_FANOUT_EXPANDER_DEVICE 3
#define HIM_UNSUPPORTED_DEVICE     4
#define HIM_UNKNOWN_DEVICE         5

/***************************************************************************
*  #defines used by Device Port Profile
***************************************************************************/
/* DP_SortMethod              */
#define HIM_SORT_FIFO            0
#define HIM_SORT_ELEVATOR        1

/* DP_SATAComandFeatureSet */
#define HIM_SATA_CFS_INVALID     0
#define HIM_SATA_CFS_UNKNOWN     1
#define HIM_SATA_CFS_ATA         2
#define HIM_SATA_CFS_ATAPI       3
#define HIM_SATA_CFS_SEP         4
#define HIM_SATA_CFS_PORT_MULTIPLIER    5
#define HIM_SATA_WC_NOT_ADJUSTED 0
#define HIM_SATA_WC_DISABLE      1
#define HIM_SATA_WC_ENABLE       2
#define HIM_INVALID_TSH          ((HIM_TASK_SET_HANDLE) OSM_NULL)


/***************************************************************************
*  #defines used by Nexus Profile
***************************************************************************/
/* Null Pointer define - used in XP_OSMNodeContext */
#define  HIM_NULL                0

/****************************************************************************/
/* specialized return codes and parameters                                  */
/****************************************************************************/
/* General results            */
#define  HIM_SUCCESS             0
#define  HIM_FAILURE             1

/* HIMVerifyAdapter           */
#define  HIM_ADAPTER_NOT_SUPPORTED          ((HIM_STATUS)5)
#define  HIM_INVALID_NON_VOLATILE_STRUCTURE ((HIM_STATUS)14)
#define  HIM_BIST_FAILURE                   ((HIM_STATUS)20)
#define  HIM_COUNTERFEIT_ADAPTER            ((HIM_STATUS)21)

/* HIMInitialize              */
#define  HIM_APPLY_MEMORY_FAILURE  2
#define  HIM_FIRMWARE_FAILURE      3
#define  HIM_TIMEOUT_FAILURE       4
#define  HIM_HOST_BUS_ERROR        5

/* HIMCheckDevicePortTSCBNeeded   */
#define  HIM_NO_NEW_DEVICES      0
#define  HIM_NEW_DEVICE_DETECTED 1

/* index for HIMCreateDevicePortTSCB */
#define  HIM_PROBED_DEVICE       0xffff

/* HIMValidateDevicePortTSH, and DP_DevicePortState */
#define  HIM_DEVICE_VALID                0
#define  HIM_DEVICE_CHANGED              1
#define  HIM_DEVICE_MISSING              2
#define  HIM_DEVICE_INVALID              3
#define  HIM_DEVICE_NOT_OPERATIONAL      4
#define  HIM_DEVICE_DEGRADED_OPERATION   5
#define  HIM_DEVICE_STATE_UNAVAILABLE    6
#define  HIM_DEVICE_SPINUP_HOLD          7
#define  HIM_DEVICE_INACTIVE             8

/* HIMClearDevicePortTSH, HIMAdjustDevicePortProfile */
#define  HIM_DEVICE_NOT_IDLE     6

/* HIMValidatePortTSH */
#define  HIM_PORT_VALID          0
#define  HIM_PORT_CHANGED        1
#define  HIM_PORT_MISSING        2
#define  HIM_PORT_INVALID        3

/* HIMAdjustPhyProfile, HIMAdjustPortProfile,
 * HIMClearPortTSH
 */
#define  HIM_PORT_NOT_IDLE       7

/* HIMPollIRQ, HIMFrontEndISR */
#define  HIM_NOTHING_PENDING        ((HIM_STATUS)0)
#define  HIM_INTERRUPT_PENDING      ((HIM_STATUS)1)
#define  HIM_LONG_INTERRUPT_PENDING ((HIM_STATUS)2)
#define  HIM_ADAPTER_INACCESSIBLE   ((HIM_STATUS)3)

/* HIMProfileAdapter, HIMProfilePhy, HIMProfilePort, HIMProfileDevicePort,
 * HIMProfileNexus.
 */
#define  HIM_PAGE_CODE_NOT_SUPPORTED  ((HIM_STATUS)10)

/* HIMAdjustAdapterProfile */
#define  HIM_ADAPTER_NOT_IDLE    2

/* HIMAdjustAdapterProfile, HIMAdjustTargetProfile,
 * HIMSetConfiguration
 */
#define  HIM_ILLEGAL_CHANGE      ((HIM_STATUS)8)
#define  HIM_CHANGE_NOT_SUPPORTED   ((HIM_STATUS)15)

/* HIMProfileAdapter, HIMAdjustAdapterProfile,
 * HIMProfilePhy, HIMAdjustPhyProfile,
 * HIMProfilePort, HIMAdjustPortProfile,
 * HIMProfileDevicePort, HIMAdjustDevicePortProfile,
 * HIMProfileLu, HIMAdjustLuProfile,
 * HIMProfileNexus, HIMAdjustNexusProfile,
 * HIMGetPortTSH
 */
#define  HIM_TSH_INVALID   ((HIM_STATUS)16)

/* HIMClearNexusTSH specific results */
#define  HIM_NEXUS_HOLDING_IOBUS 2
#define  HIM_NEXUS_NOT_IDLE      3

/* HIMGetNVOSMSegment         */
#define  HIM_NO_OSM_SEGMENT      2

/* HIM_TS_SAS specific defines. */
#define  HIM_SSP_RESPONSE_HEADER_SIZE    24

/* HIMQueueIOB: transport-specific, if OSM wishes to include it */
typedef struct HIM_TS_SAS_ {
   HIM_UINT8    targetTransport;    /* set to HIM_TRANSPORT_SAS               */
   HIM_UINT8    targetProtocol;     /* HIM_PROTOCOL_SCSI...                   */
   HIM_UINT8    initiatorProtocol;  /*            */
   HIM_UINT8    deviceType;         /* END_DEVICE - EDGE/FANOUT EXPANDER      */
   HIM_UINT32   connectionRate;     /* in tenths of megabits per second       */
   HIM_UEXACT8  SASAddress[HIM_SAS_ADDRESS_SIZE];

   HIM_UEXACT16 phyID;              /* phy id of the SMP target to be probed  */
   HIM_UEXACT8  LUN[HIM_LUN_SIZE];  /* SAM-3 LUN format, 8-byte LUN           */
   HIM_UEXACT8  connectionStatus;   /* protocol status: scsi status or ssp
                                     * response code depending on taskStatus.
                                     */
   HIM_UEXACT8  sspResponseIUHeader[HIM_SSP_RESPONSE_HEADER_SIZE];
                                    /* first 24 bytes of SSP RESPONSE IU */
} HIM_TS_SAS;

/***************************************************************************/
/* Definations for connectionStatus field                                  */
/***************************************************************************/
#define HIM_CONNECTION_STATUS_INVALID                  0x00
#define HIM_SAS_OPEN_REJECT_BAD_DESTINATION            0x01
#define HIM_SAS_OPEN_REJECT_RESERVED_ABANDON0          0x06
#define HIM_SAS_OPEN_REJECT_RESERVED_ABANDON1          0x07
#define HIM_SAS_OPEN_REJECT_RESERVED_ABANDON2          0x08
#define HIM_SAS_OPEN_REJECT_RESERVED_ABANDON3          0x09
#define HIM_SAS_OPEN_REJECT_WRONG_DESTINATION          0x0A
#define HIM_SAS_OPEN_REJECT_NO_DESTINATION             0x20
#define HIM_SAS_OPEN_REJECT_PATHWAY_BLOCKED            0x21
#define HIM_SAS_OPEN_REJECT_RESERVED_CONTINUE0         0x22
#define HIM_SAS_OPEN_REJECT_RESERVED_CONTINUE1         0x23
#define HIM_SAS_OPEN_REJECT_RESERVED_INITIALIZE0       0x24
#define HIM_SAS_OPEN_REJECT_RESERVED_INITIALIZE1       0x25
#define HIM_SAS_OPEN_REJECT_RESERVED_STOP0             0x26
#define HIM_SAS_OPEN_REJECT_RESERVED_STOP1             0x27
#define HIM_SAS_OPEN_REJECT_RETRY                      0x28

/***************************************************************************
* Structure for for transport specific - SATA case
* HIMQueueIOB: SATA transport-specific, if OSM wishes to include it
***************************************************************************/
typedef struct HIM_TS_SATA_ {
   HIM_UINT8    targetTransport;    /* set to HIM_TRANSPORT_SATA              */
   HIM_UINT8    deviceType;         /* Only be valid for a HIM_PROBE IOB.     */
   HIM_UINT32   connectionRate;
   /* ATA register file for transport dependent ATA status */
   HIM_UEXACT8  sataStatus;
   HIM_UEXACT8  sataError;
   HIM_UEXACT8  sataSectorNumber;
   HIM_UEXACT8  sataCylinderLow;
   HIM_UEXACT8  sataCylinderHigh;
   HIM_UEXACT8  sataDeviceHead;
   HIM_UEXACT8  sataSectorNumberExp;
   HIM_UEXACT8  sataCylinderLowExp;
   HIM_UEXACT8  sataCylinderHighExp;
   HIM_UEXACT8  sataReserved9;
   HIM_UEXACT8  sataSectorCount;
   HIM_UEXACT8  sataSectorCountExp;
   HIM_UEXACT8  sataReserved12;
   HIM_UEXACT8  sataReserved13;
   HIM_UEXACT8  sataReserved14;
   HIM_UEXACT8  sataReserved15;
   HIM_UEXACT8  sataReserved16;
   HIM_UEXACT8  sataReserved17;
   HIM_UEXACT8  sataSStatus0;
   HIM_UEXACT8  sataSStatus1;
   HIM_UEXACT8  sataSStatus2;
   HIM_UEXACT8  sataSStatus3;
   HIM_UEXACT8  sataSError0;
   HIM_UEXACT8  sataSError1;
   HIM_UEXACT8  sataSError2;
   HIM_UEXACT8  sataSError3;
   HIM_UEXACT8  sataSControl0;
   HIM_UEXACT8  sataSControl1;
   HIM_UEXACT8  sataSControl2;
   HIM_UEXACT8  sataSControl3;
   HIM_UEXACT8  sataSActive0;
   HIM_UEXACT8  sataSActive1;
   HIM_UEXACT8  sataSActive2;
   HIM_UEXACT8  sataSActive3;
} HIM_TS_SATA;

#define  SATA_FIRST_REG                    sataStatus


/***************************************************************************
* Definitions for HIM_PHY_OPERATIONS
* Structure used to indicate what phy operation is to be performed
***************************************************************************/
typedef struct HIM_PHY_OPERATIONS_ 
{
   HIM_UINT8    PhyOperation;
   HIM_UINT8    FixedPattern;
   HIM_BOOLEAN  DisableScrambling;
   HIM_BOOLEAN  DisableAlign;
   HIM_BOOLEAN  DisablePhyAfterError;
} HIM_PHY_OPERATIONS;

/* PhyOperation code */
#define  HIM_PHY_OP_ENABLE                      0
#define  HIM_PHY_OP_SWITCH_ACTIVE_PORT          1
#define  HIM_PHY_OP_HARD_RESET                  2
#define  HIM_PHY_OP_DATA_PATTERN_GENERATE       3
#define  HIM_PHY_OP_PRIMITIVE_PATTERN_GENERATE  4
#define  HIM_PHY_OP_FIXED_PATTERN_GENERATE      5

/* FixedPattern code */
#define  HIM_FP_CJTPAT                          0
#define  HIM_FP_ALIGN1                          1

/* Task Management specific structure for HIM_IOB function HIM_ESTABLISH_SCSI_CONNECTION.
 * This is the format of the area pointed to by void HIM_PTR commandBuffer.
 */
typedef struct HIM_SCSI_TMF_REQUEST_
{
   HIM_UEXACT8           tmfCode;   /* TMF code received from initiator */
   HIM_TASK_SET_HANDLE   nexusTSH;  /* nexus TSH associated with the TMF */
} HIM_SCSI_TMF_REQUEST;

 
/***************************************************************************
* Structures for Event Notification
***************************************************************************/
typedef struct HIM_INVALID_SDS_EVENT_
{
    HIM_TASK_SET_HANDLE   portTSH;
    HIM_UEXACT8           SASAddress[HIM_SAS_ADDRESS_SIZE];
    HIM_UINT8             phyID;
  
} HIM_INVALID_SDS_EVENT;

typedef struct HIM_CORRECTED_SDS_EVENT_
{
    HIM_UEXACT8           SASAddress[HIM_SAS_ADDRESS_SIZE];
    HIM_UINT8             phyID;
} HIM_CORRECTED_SDS_EVENT;

/*****************************************************************
* PCI-X error event context
*****************************************************************/
typedef struct HIM_PCIX_ERROR_EVENT_
{
   HIM_UEXACT32 PCIXStatus;
   HIM_UEXACT32 ECCControlAndStatus;
   HIM_UEXACT32 ECCFirstAddress;
   HIM_UEXACT32 ECCSecondAddress;
   HIM_UEXACT32 ECCAttribute;
} HIM_PCIX_ERROR_EVENT;

/* Event codes for OSMAdapterEvent(), OSMPortEvent, OSMIobEvent and OSMLuEvent */
#define HIM_EVENT_SEQUENCE                               0x01  /* Event sequence */

/* Event codes for OSMAdapterEvent() */
#define HIM_EVENT_COMMUNICATION_FAILURE                  0x10  /* An error occurred in the communication    */
                                                               /* protocol between HIM and the adapter or   */
                                                               /* firmware operating the adapter.           */
#define HIM_EVENT_ADAPTER_FIRMWARE_BREAKPOINT            0x11  /* A firmware breakpoint interrupt occurred  */
                                                               /* on the adapter.                           */
#define HIM_EVENT_RECOVERABLE_ADAPTER_FAILURE            0x12  /* The adapter has failed due to a           */
                                                               /* recoverable error.                        */
#define HIM_EVENT_UNRECOVERABLE_ADAPTER_FAILURE          0x13  /* A catastrophic adapter failure has        */
                                                               /* occurred.                                 */
#define HIM_EVENT_NEW_PORT                               0x14  /* A new port was detected and created.      */
#define HIM_EVENT_PORT_RESOURCES_NOT_AVAILABLE           0x15  /* A new port was detected, but due          */
                                                               /* to a lack of port resources the           */
                                                               /* HIM was unable to report a                */
                                                               /* HIM_EVENT_NEW_PORT event.                 */
#define HIM_EVENT_PCI_ERROR                              0x16  /* A PCI error detected by the adapter.      */
#define HIM_EVENT_PCIX_ERROR                             0x17  /* A PCI-X error detected by the adapter.    */
#define HIM_EVENT_UNRECOVERABLE_PHY_FAILURE              0x18  /* Instructs OSM that the phy has been       */
                                                               /* disabled due to persistant error on link  */
                                                               /* reset sequencer and configured retries    */
                                                               /* didn't help.                              */

#define HIM_EVENT_CONFIGURATION_CHANGE                   0x22  /* Transport reported a configuration        */
                                                               /* change event occurred.                    */

/* Event codes for OSMPortEvent() */
#define HIM_EVENT_HARD_RESET                             0x21  /* A port received a hard reset event.       */

#define HIM_EVENT_STORAGE_ENCLOSURE_SERVICES             0x23  /* Transport reported a storage enclosure    */
                                                               /* services event occurred.                  */
#define HIM_EVENT_DISCOVERED_DEVICE_NOT_SUPPORTED        0x24  /* During device discovery if a device port  */
                                                               /* is discovered but not supported.          */
#define HIM_EVENT_INVALID_SERVICE_DELIVERY_SUBSYSTEM     0x25  /* A problem was detected within the service */
                                                               /* delivery subsystem that may prevent       */
                                                               /* discovery for all the targets attached    */
                                                               /* to the port.                              */
#define HIM_EVENT_CORRECTED_SERVICE_DELIVERY_SUBSYSTEM   0x26  /* A problem was detected and corrected      */
                                                               /* within the service delivery subsystem.    */
#define HIM_EVENT_PORT_CHANGED                           0x27  /* The number of links within a port has     */
                                                               /* changed or the speed of one or more       */
                                                               /* links have changed.                       */
#define HIM_EVENT_PORT_DOWN                              0x28  /* All the links of a port are missing and   */
                                                               /* IOBs are still pending to the port or     */
                                                               /* related TSH.                              */
#define HIM_EVENT_PORT_UP                                0x29  /* A link of a port has not been cleared     */
                                                               /* by HIMClearPortTSH has returned.          */
#define HIM_EVENT_PORT_INVALID                           0x2A  /* All the links of a port are missing and   */
                                                               /* no IOBs are pending to the port and       */
                                                               /* related TSH.                              */
#define HIM_EVENT_DEVICE_PORT_RESOURCES_NOT_AVAILABLE    0x2B  /* A new device port was detected, but due   */
                                                               /* to a lack of device port resources the    */
                                                               /* HIM was unable to report the device port. */
#define HIM_EVENT_UNKNOWN_PRIMITIVE_RECEIVED             0x2C  /* An unknown primitive was received.        */
#define HIM_EVENT_ENABLE_SPINUP                          0x2D  /* Instructs the target that it may utilize  */
                                                               /* additional power to transition to a       */
                                                               /* different power state.  Target mode only. */

/* Event codes for IOB event */
#define HIM_EVENT_DEVICE_PROTOCOL_ERROR                  0x40  /* The task resulted in a protocol error,    */
                                                               /* but the task remains in the               */
                                                               /* logical unit's task set.                  */
#define HIM_EVENT_TRANSPORT_ERROR                        0x41  /* The task was active when a transport      */
                                                               /* error occurred.                           */
#define HIM_EVENT_DEVICE_PORT_DETECTED_CRC_ERROR         0x42  /* The device port reported a CRC error.     */
#define HIM_EVENT_TRANSPORT_TIMEOUT                      0x43  /* A transport layer timeout occurred with   */

/* Event codes for OSMLuEvent() */
#define HIM_EVENT_NCQ_COMMAND_ERROR                      0x50  /* Indicates an error occurred with a        */
                                                               /* native command queuing task pending       */
                                                               /* to this logical unit.                     */
#define HIM_EVENT_TAG_COMMAND_ERROR                      0x60  /* Indicates an error occurred with a        */
                                                               /* tag command queued to this logical        */
                                                               /* unit                                      */  
/* Advanced PCI Express Error Events - Aurora */
#define HIM_EVENT_PCIE_RECOVERABLE_ERROR                 0x70  /* Indicates a recoverable PCI express occured*/
#define HIM_EVENT_PCIE_UNRECOVERABLE_ERROR               0x71  /* Indicates a unrecoverable PCI express      */
                                                               /* occured                                    */
#define HIM_EVENT_RBI_UNRECOVERABLE_ERROR                0x72  /* Indicates a unrecoverable internal RBI     */
                                                               /* error occured                              */
#define HIM_EVENT_RBI_RECOVERABLE_ERROR                  0x73  /* Indicates a recoverable internal RBI       */
                                                               /* error occured                              */

/* Action codes for OSMAdapterEvent */
/* OSM to stop sending IOBs for the adapter */
#define HIM_ACTION_FREEZE_ADAPTER                           ((HIM_UEXACT32)1 << 0)
/* Debug OSM to enter debugger, production OSM to ignore */
#define HIM_ACTION_ENTER_DEBUGGER                           ((HIM_UEXACT32)1 << 5)
/* OSM to take the adapter out of service */
#define HIM_ACTION_FAIL_ADAPTER                             ((HIM_UEXACT32)1 << 6)
/* OSM to send HIM_RESET_HARDWARE IOB */
#define HIM_ACTION_RESET_HARDWARE                           ((HIM_UEXACT32)1 << 10)
/* OSM to send HIM_RESET_ADAPTER IOB */
#define HIM_ACTION_RESET_ADAPTER                            ((HIM_UEXACT32)1 << 11)
#if HIM_REPORT_ACTION_FAIL_PHY
/* Instructs OSM that phy is disabled due to errors, need to be enabled before usage */
#define HIM_ACTION_FAIL_PHY                                 ((HIM_UEXACT32)1 << 12)
#endif /* HIM_REPORT_ACTION_FAIL_PHY */
/* OSM to send HIM_ENABLE_PHY IOB */
#define HIM_ACTION_ENABLE_PHY                               ((HIM_UEXACT32)1 << 13)
/* OSM to do HIMValidatePortTSH for all port TSH */
#define HIM_ACTION_VALIDATE_PORT_ON_ADAPTER                 ((HIM_UEXACT32)1 << 15)
/* OSM to check for new/create port TSCBs */
#define HIM_ACTION_GET_NEW_PORTS                            ((HIM_UEXACT32)1 << 16)
/* OSM to perform device port discovery for the adapter */
#define HIM_ACTION_PERFORM_DEVICE_PORT_DISCOVERY_ON_ADAPTER ((HIM_UEXACT32)1 << 20)
/* OSM to validate all the device port TSH for the adapter */
#define HIM_ACTION_VALIDATE_DEVICE_PORTS_ON_ADAPTER         ((HIM_UEXACT32)1 << 21)
/* OSM to check for new device port TSCBs */
#define HIM_ACTION_CREATE_NEW_DEVICE_PORTS_ON_ADAPTER       ((HIM_UEXACT32)1 << 22)
/* BY pass to set topology changed */
#define HIM_ACTION_BYPASS_TOPOLOGY_CHANGED                  ((HIM_UEXACT32)1 << 23)
/* Some reseved actions bits */    
#define HIM_ACTION_RESERVED0                                ((HIM_UEXACT32)1 << 24)
#define HIM_ACTION_RESERVED1                                ((HIM_UEXACT32)1 << 25)
#define HIM_ACTION_RESERVED2                                ((HIM_UEXACT32)1 << 26)
#define HIM_ACTION_RESERVED3                                ((HIM_UEXACT32)1 << 27)
/* OSM to resume sending IOBs for the adapter */
#define HIM_ACTION_UNFREEZE_ADAPTER                         ((HIM_UEXACT32)1 << 31)

/* Action codes for OSMPortEvent */
/* OSM to stop sending IOBs for the port */
#define HIM_ACTION_FREEZE_PORT                           ((HIM_UEXACT32)1 << 0)
/* OSM to do hard reset detected cleanup */
#define HIM_ACTION_HARD_RESET                            ((HIM_UEXACT32)1 << 5)
/* OSM to enable all phys for the adapter */
#define HIM_ACTION_ENABLE_PHYS                           ((HIM_UEXACT32)1 << 6)
/* OSM to send HIM_RESET_PORT IOB */
#define HIM_ACTION_RESET_PORT                            ((HIM_UEXACT32)1 << 10)
/* OSM to HIMValidatePortTSH */
#define HIM_ACTION_VALIDATE_PORT                         ((HIM_UEXACT32)1 << 11)
/* OSM to perform device port discovery for the port */
#define HIM_ACTION_PERFORM_DEVICE_PORT_DISCOVERY_ON_PORT ((HIM_UEXACT32)1 << 15)
/* OSM to validate all the device port TSH for the port */
#define HIM_ACTION_VALIDATE_DEVICE_PORTS_ON_PORT         ((HIM_UEXACT32)1 << 16)
/* OSM to check for new/create device port TSCBs */
#define HIM_ACTION_CREATE_NEW_DEVICE_PORTS_ON_PORT       ((HIM_UEXACT32)1 << 17)
/* OSM to service storage enclosure devices */
#define HIM_ACTION_SERVICE_ENCLOSURE_SERVICE_DEVICES     ((HIM_UEXACT32)1 << 20)
/* OSM to resume sending IOBs for the adapter */
#define HIM_ACTION_UNFREEZE_PORT                         ((HIM_UEXACT32)1 << 31)

/* Action codes for OSMIobEvent */
/* OSM to send HIM_RESET_DEVICE_PORT */
#define HIM_ACTION_RESET_DEVICE_PORT   ((HIM_UEXACT32)1 << 0)
/* OSM to send HIM_ABORT_TASK */
#define HIM_ACTION_ABORT_TASK          ((HIM_UEXACT32)1 << 5)
/* OSM to send HIM_QUERY_TASK */
#define HIM_ACTION_QUERY_TASK          ((HIM_UEXACT32)1 << 10)

/* Action codes for OSMLuEvent */
/* OSM to stop sending IOBs for the logical unit */
#define HIM_ACTION_FREEZE_LOGICAL_UNIT                   ((HIM_UEXACT32)1 << 0)
/* OSM to send HIM_RESET_LOGICAL_UNIT IOB */
#define HIM_ACTION_RESET_LOGICAL_UNIT                    ((HIM_UEXACT32)1 << 10)
/* OSM to send HIM_PERFORM_LU_RECOVERY IOB */
#define HIM_ACTION_PERFORM_LU_RECOVERY                   ((HIM_UEXACT32)1 << 15)
/* OSM to send HIM_ABORT_TASK_SET IOB */
#define HIM_ACTION_ABORT_TASK_SET                        ((HIM_UEXACT32)1 << 20)
/* OSM to resume sending IOBs for the adapter */
#define HIM_ACTION_UNFREEZE_LOGICAL_UNIT                 ((HIM_UEXACT32)1 << 31)

/* Attributes of OSMMapIOHandle */
#define HIM_IO_BIG_ENDIAN            1
#define HIM_IO_LITTLE_ENDIAN         2
#define HIM_IO_STRICTORDER           4 /* Register accesses must be in order */
#define HIM_IO_MERGING_OK            8 /* Merge consecutive loads/stores in  
                                        * single, larger operation           */
#define HIM_IO_LOAD_CACHING_OK       16 /* Loads can be cached and reused until
                                         * next store                        */
#define HIM_IO_STORE_CACHING_OK      32 /* Writes can be cached, and written 
                                         * at a later time                   */
/* OCM segment Ids*/
#define HIM_SEGMENT_ID_BIOS_AND_DRIVER_RAM0           0x03
#define HIM_SEGMENT_ID_BIOS_AND_DRIVER_CHIM_RAM0      0x02
#define HIM_SEGMENT_ID_DRIVER_AND_DRIVER_RAM0         0x01

/* Flash segment Ids */

#define HIM_SEGMENT_ID_CTRL_A_USER_SETTINGS0    0xe0
#define HIM_SEGMENT_ID_MANUFACTURING_SECTOR0    0x120
#define HIM_SEGMENT_ID_FLASH0                   0xff
#define HIM_SEGMENT_SIZE_INVALID                0xffffffff
#define HIM_SEGMENT_IMAGE_SIZE_INVALID          0xffffffff

/* HIMQueueIOB: transport-specific */
typedef struct HIM_TS_I2CTDI_
{
   HIM_UEXACT8 HIM_PTR commandBuffer;
   HIM_UEXACT8 HIM_PTR responseBuffer;
   HIM_BUS_ADDRESS     commandBusAddress;
   HIM_BUS_ADDRESS     responseBusAddress;

   HIM_UEXACT32     commandLength;
   HIM_UEXACT32     responseLength;
   HIM_UEXACT32     receivedLength;
   HIM_UINT8        targetTransport;    /* set to HIM_TRANSPORT_I2C         */
   HIM_UINT8        targetProtocol;     /* HIM_PROTOCOL_I2C...              */
   HIM_UINT8        initiatorProtocol;
   HIM_UINT8        deviceType;
   HIM_UEXACT8      I2CAddress;
} HIM_TS_I2CTDI;



/****************************************************************************/
/*  Macros  definitions                                                     */
/****************************************************************************/
#define HIM_PROBE_LU_SUCCESS(probeLuIOB) \
            ((((probeLuIOB)->taskStatus == HIM_IOB_GOOD) || \
              ((probeLuIOB)->taskStatus == HIM_IOB_DATA_UNDERRUN) || \
              ((probeLuIOB)->taskStatus == HIM_IOB_DATA_OVERRUN) || \
              ((probeLuIOB)->taskStatus == HIM_IOB_ERRORDATA_VALID) || \
              ((probeLuIOB)->taskStatus == HIM_IOB_ERRORDATA_REQUIRED) || \
              ((probeLuIOB)->taskStatus == HIM_IOB_ERRORDATA_UNDERRUN) || \
              ((probeLuIOB)->taskStatus == HIM_IOB_ERRORDATA_OVERRUN)) \
             ? (HIM_TRUE) : (HIM_FALSE))


/****************************************************************************/
/*  function pointer packets                                                */
/****************************************************************************/

/* ----- HIM -> OSM functions ----- */

#define HIM_VERSION_OSM_FUNC_PTRS 1

#define HIM_VERSION_FUNC_PTRS 1

#if SAS_DOWNSHIFT_SUPPORT

/* ----- OSM -> HIM functions ----- */

typedef struct HIM_OSM_FUNC_PTRS_ {

   HIM_UINT32     versionNumber;

   HIM_STATUS     (HIM_PTR OSMMapIOHandle)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT8     rangeIndex,
                                     HIM_IO_HANDLE HIM_PTR handle);

   void           (HIM_PTR OSMAdjustBusAddress)
                                    (HIM_BUS_ADDRESS HIM_PTR busAddress,
                                     HIM_SINT32    value);

   HIM_STATUS     (HIM_PTR OSMWriteNVSegment)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    segmentID,
                                     HIM_UINT32    destinationOffset,
                                     void HIM_PTR  source,
                                     HIM_UINT32    length);

   HIM_STATUS     (HIM_PTR OSMReadNVSegment)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    segmentID,
                                     void HIM_PTR  destination,
                                     HIM_UINT32    sourceOffset,
                                     HIM_UINT32    bytesToRead,
                                     HIM_UINT32 HIM_PTR bytestRead);

   HIM_STATUS     (HIM_PTR OSMGetNVSegmentProperties)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    segmentID,
                                     HIM_UINT16 HIM_PTR segmentAttributes,
                                     HIM_UINT32 HIM_PTR segmentSize,
                                     HIM_UINT32 HIM_PTR segmentImageSize);

   HIM_UEXACT8    (HIM_PTR OSMReadUExact8)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset);

   HIM_UEXACT16   (HIM_PTR OSMReadUExact16)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset);

   HIM_UEXACT32   (HIM_PTR OSMReadUExact32)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset);

   void           (HIM_PTR OSMWriteUExact8)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT8   value);

   void           (HIM_PTR OSMWriteUExact16)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT16  value);

   void           (HIM_PTR OSMWriteUExact32)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT32  value);

   HIM_UEXACT32   (HIM_PTR OSMReadPCIConfigurationDword)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber);

   void           (HIM_PTR OSMWritePCIConfigurationDword)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber,
                                     HIM_UEXACT32  registerValue);

   void           (HIM_PTR OSMDelay)(void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    microSeconds);

   void           (HIM_PTR OSMPortEvent)(void HIM_PTR  osmPortContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

   void           (HIM_PTR OSMAdapterEvent)(void HIM_PTR  osmAdapterContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

   void           (HIM_PTR OSMIobEvent)(void HIM_PTR  osmTaskContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

   void           (HIM_PTR OSMLuEvent)(void HIM_PTR  osmLuContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

} HIM_OSM_FUNC_PTRS;

/* ----- HIM -> OSM functions ----- */

typedef struct HIM_FUNC_PTRS_ {
   HIM_UINT32     versionNumber;


   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateInitializationTSCB)
                                    (void HIM_PTR         pHIMInitTSCB );

   void           (HIM_PTR HIMGetConfiguration)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_CONFIGURATION HIM_PTR pConfig,
                                     HIM_HOST_ID          productID);

   HIM_STATUS     (HIM_PTR HIMSetConfiguration)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_CONFIGURATION HIM_PTR pConfig,
                                     HIM_HOST_ID          productID);
                                     
   HIM_UINT32     (HIM_PTR HIMSizeAdapterTSCB)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_HOST_ID          productID);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateAdapterTSCB)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     void HIM_PTR         pAdapterTSCB,
                                     void HIM_PTR         osmAdapterContext,
                                     HIM_HOST_ID          productID);

   HIM_STATUS     (HIM_PTR HIMSetupAdapterTSCB)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_OSM_FUNC_PTRS HIM_PTR pOsmRoutines,
                                     HIM_UINT16           osmFuncLength);

   HIM_UINT32     (HIM_PTR HIMCheckMemoryNeeded)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_HOST_ID          productID,
                                     HIM_UINT16           index,
                                     HIM_UINT8 HIM_PTR    category,
                                     HIM_UINT32 HIM_PTR   minimumBytes,
                                     HIM_UINT32 HIM_PTR   granularity,
                                     HIM_UEXACT32 HIM_PTR alignmentMask,
                                     HIM_UINT8 HIM_PTR    type,
                                     HIM_UINT8 HIM_PTR    speed);

   HIM_UINT32     (HIM_PTR HIMCheckIOBMemoryNeeded) 
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_HOST_ID          productID,    
                                     HIM_UINT8            index,
                                     HIM_UINT8            iobFunction,
                                     HIM_UINT16 HIM_PTR   offsetOfIobField,
                                     HIM_UINT8 HIM_PTR    category,    
                                     HIM_UEXACT32 HIM_PTR alignmentMask,
                                     HIM_UINT8 HIM_PTR    speed);

   HIM_STATUS     (HIM_PTR HIMSetMemoryPointer)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT16           index,
                                     HIM_UINT8            category,
                                     HIM_BUFFER_DESCRIPTOR HIM_PTR pMemoryDescriptor);

   HIM_STATUS     (HIM_PTR HIMVerifyAdapter)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMGetPhyTSH)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT8            index,
                                     void HIM_PTR         osmPhyContext);

   HIM_STATUS     (HIM_PTR HIMInitialize)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMGetPortTSH)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     void HIM_PTR         osmPortContext);

   HIM_UINT32     (HIM_PTR HIMSizeDevicePortTSCB)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_STATUS     (HIM_PTR HIMCheckDevicePortTSCBNeeded)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT16           index);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateDevicePortTSCB)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT16           index,
                                     void HIM_PTR         devicePortTSCB,
                                     void HIM_PTR         osmDevicePortContext);

   HIM_UINT32     (HIM_PTR HIMSizeLuTSCB)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateLuTSCB)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH,
                                     void HIM_PTR         luTSCB,
                                     HIM_IOB HIM_PTR      probeLuIOB,
                                     void HIM_PTR         osmLuContext);

/* run time interfaces */

   void           (HIM_PTR HIMDisableIRQ)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_BOOLEAN HIM_PTR pLastIRQState);

   void           (HIM_PTR HIMEnableIRQ)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_STATUS     (HIM_PTR HIMPollIRQ)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_STATUS     (HIM_PTR HIMFrontEndISR)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   void           (HIM_PTR HIMBackEndISR)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   void           (HIM_PTR HIMQueueIOB)
                                    (HIM_IOB HIM_PTR      iob);

   HIM_STATUS     (HIM_PTR HIMValidatePortTSH)
                                    (HIM_TASK_SET_HANDLE  portTSH);

   HIM_STATUS     (HIM_PTR HIMClearPortTSH)
                                    (HIM_TASK_SET_HANDLE  portTSH);

   HIM_STATUS     (HIM_PTR HIMValidateDevicePortTSH)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH);

   HIM_STATUS     (HIM_PTR HIMClearDevicePortTSH)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH);

   HIM_STATUS     (HIM_PTR HIMClearLuTSH)
                                    (HIM_TASK_SET_HANDLE  luTSH);

#if SAS_SAVE_RESTORE_STATE
   HIM_STATUS    (HIM_PTR HIMSaveState)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     void HIM_PTR  pState);

   HIM_STATUS    (HIM_PTR HIMRestoreState)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     void HIM_PTR  pState);
#endif /* SAS_SAVE_RESTORE_STATE */

   HIM_STATUS     (HIM_PTR HIMProfileAdapter)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMProfilePhy)
                                    (HIM_TASK_SET_HANDLE  phyTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMProfileDevicePort)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMProfileLu)
                                    (HIM_TASK_SET_HANDLE  luTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMReadNVSegment)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     void HIM_PTR         destination,
                                     HIM_UINT32           sourceOffset,
                                     HIM_UINT32           bytesToRead,
                                     HIM_UINT32 HIM_PTR   bytesRead);

#if SAS_OSM_DEBUG_MODE
   HIM_STATUS     (HIM_PTR HIMDumpObject)
                                    (HIM_TASK_SET_HANDLE  debugTSH,
                                     HIM_UINT8            dumpFormat,  
                                     void HIM_PTR         traceInfo,
                                     void HIM_PTR         dumpOptions);

#endif /* SAS_OSM_DEBUG_MODE */
} HIM_FUNC_PTRS;

#else /* !SAS_DOWNSHIFT_SUPPORT */

/* ----- OSM -> HIM functions ----- */

typedef struct HIM_OSM_FUNC_PTRS_ {

   HIM_UINT32     versionNumber;

   HIM_STATUS     (HIM_PTR OSMMapIOHandle)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT8     rangeIndex,
                                     HIM_UINT32    offset,
                                     HIM_UINT32    length,
                                     HIM_UINT32    pacing,
                                     HIM_UINT16    attributes,
                                     HIM_IO_HANDLE HIM_PTR handle);

   HIM_STATUS     (HIM_PTR OSMReleaseIOHandle)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_IO_HANDLE handle);

   void           (HIM_PTR OSMAdjustBusAddress)
                                    (HIM_BUS_ADDRESS HIM_PTR busAddress,
                                     HIM_SINT32    value);

   HIM_STATUS     (HIM_PTR OSMWriteNVSegment)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    segmentID,
                                     HIM_UINT32    destinationOffset,
                                     void HIM_PTR  source,
                                     HIM_UINT32    length);
   
   HIM_STATUS     (HIM_PTR OSMReadNVSegment)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    segmentID,
                                     void HIM_PTR  destination,
                                     HIM_UINT32    sourceOffset,
                                     HIM_UINT32    bytesToRead,
                                     HIM_UINT32 HIM_PTR bytestRead);

   HIM_STATUS     (HIM_PTR OSMGetNVSegmentProperties)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    segmentID,
                                     HIM_UINT16 HIM_PTR segmentAttributes,
                                     HIM_UINT32 HIM_PTR segmentSize,
                                     HIM_UINT32 HIM_PTR segmentImageSize);

   HIM_UEXACT8    (HIM_PTR OSMReadUExact8)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset);

   HIM_UEXACT16   (HIM_PTR OSMReadUExact16)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset);

   HIM_UEXACT32   (HIM_PTR OSMReadUExact32)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset);

   void           (HIM_PTR OSMReadStringUExact8)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT8 HIM_PTR destBuffer,
                                     HIM_UINT32    count,
                                     HIM_UINT8     stride);

   void           (HIM_PTR OSMReadStringUExact16)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT16 HIM_PTR destBuffer,
                                     HIM_UINT32    count,
                                     HIM_UINT8     stride);

   void           (HIM_PTR OSMReadStringUExact32)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT32 HIM_PTR destBuffer,
                                     HIM_UINT32    count,
                                     HIM_UINT8     stride);

   void           (HIM_PTR OSMWriteUExact8)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT8   value);

   void           (HIM_PTR OSMWriteUExact16)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT16  value);

   void           (HIM_PTR OSMWriteUExact32)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT32  value);

   void           (HIM_PTR OSMWriteStringUExact8)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT8 HIM_PTR sourceBuffer,
                                     HIM_UINT32    count,
                                     HIM_UINT8     stride);

   void           (HIM_PTR OSMWriteStringUExact16)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT16 HIM_PTR sourceBuffer,
                                     HIM_UINT32    count,
                                     HIM_UINT8     stride);

   void           (HIM_PTR OSMWriteStringUExact32)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UEXACT32 HIM_PTR sourceBuffer,
                                     HIM_UINT32    count,
                                     HIM_UINT8     stride);

   void           (HIM_PTR OSMSynchronizeRange)
                                    (HIM_IO_HANDLE baseHandle,
                                     HIM_UINT32    offset,
                                     HIM_UINT32    length);

   void           (HIM_PTR OSMWatchdog)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_WATCHDOG_FUNC watchdogProcedure,
                                     HIM_UINT32    microSeconds);

   HIM_UEXACT32   (HIM_PTR OSMReadPCIConfigurationDword)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber);

   HIM_UEXACT16   (HIM_PTR OSMReadPCIConfigurationWord)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber);

   HIM_UEXACT8    (HIM_PTR OSMReadPCIConfigurationByte)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber);

   void           (HIM_PTR OSMWritePCIConfigurationDword)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber,
                                     HIM_UEXACT32  registerValue);

   void           (HIM_PTR OSMWritePCIConfigurationWord)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber,
                                     HIM_UEXACT16  registerValue);

   void           (HIM_PTR OSMWritePCIConfigurationByte)
                                    (void HIM_PTR  osmAdapterContext,
                                     HIM_UINT16    registerNumber,
                                     HIM_UEXACT8   registerValue);

   void           (HIM_PTR OSMDelay)(void HIM_PTR  osmAdapterContext,
                                     HIM_UINT32    microSeconds);

   void           (HIM_PTR OSMPortEvent)(void HIM_PTR  osmPortContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

   void           (HIM_PTR OSMAdapterEvent)(void HIM_PTR  osmAdapterContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

   void           (HIM_PTR OSMIobEvent)(void HIM_PTR  osmTaskContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);

   void           (HIM_PTR OSMLuEvent)(void HIM_PTR  osmLuContext,
                                     void HIM_PTR  himEventContext,
                                     HIM_UINT16    eventCode,
                                     HIM_UEXACT32  actionSet,
                                     ...);
} HIM_OSM_FUNC_PTRS;


/* ----- HIM -> OSM functions ----- */

typedef struct HIM_FUNC_PTRS_ {
   HIM_UINT32     versionNumber;

   HIM_HOST_ID    (HIM_PTR HIMGetNextHostDeviceType)
                                    (HIM_UINT16           index,
                                     HIM_UINT8 HIM_PTR    pHostBusType,
                                     HIM_HOST_ID HIM_PTR  pMask);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateInitializationTSCB)
                                    (void HIM_PTR         pHIMInitTSCB );

   void           (HIM_PTR HIMGetConfiguration)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_CONFIGURATION HIM_PTR pConfig,
                                     HIM_HOST_ID          productID);

   HIM_STATUS     (HIM_PTR HIMSetConfiguration)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_CONFIGURATION HIM_PTR pConfig,
                                     HIM_HOST_ID          productID);

   HIM_UINT32     (HIM_PTR HIMSizeAdapterTSCB)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_HOST_ID          productID);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateAdapterTSCB)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     void HIM_PTR         pAdapterTSCB,
                                     void HIM_PTR         osmAdapterContext,
                                     HIM_HOST_ID          productID);

   HIM_STATUS     (HIM_PTR HIMSetupAdapterTSCB)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_OSM_FUNC_PTRS HIM_PTR pOsmRoutines,
                                     HIM_UINT16           osmFuncLength);

   HIM_UINT32     (HIM_PTR HIMCheckMemoryNeeded)
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_HOST_ID          productID,
                                     HIM_UINT16           index,
                                     HIM_UINT8 HIM_PTR    category,
                                     HIM_UINT32 HIM_PTR   minimumBytes,
                                     HIM_UINT32 HIM_PTR   granularity,
                                     HIM_UEXACT32 HIM_PTR alignmentMask,
                                     HIM_UINT8 HIM_PTR    type,
                                     HIM_UINT8 HIM_PTR    speed);

   HIM_UINT32     (HIM_PTR HIMCheckIOBMemoryNeeded) 
                                    (HIM_TASK_SET_HANDLE  initializationTSH,
                                     HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_HOST_ID          productID,    
                                     HIM_UINT8            index,
                                     HIM_UINT8            iobFunction,
                                     HIM_UINT16 HIM_PTR   offsetOfIobField,
                                     HIM_UINT8 HIM_PTR    category,    
                                     HIM_UEXACT32 HIM_PTR alignmentMask,
                                     HIM_UINT8 HIM_PTR    speed);

   HIM_STATUS     (HIM_PTR HIMSetMemoryPointer)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT16           index,
                                     HIM_UINT8            category,
                                     HIM_BUFFER_DESCRIPTOR HIM_PTR pMemoryDescriptor);

   HIM_STATUS     (HIM_PTR HIMVerifyAdapter)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMGetPhyTSH)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT8            index,
                                     void HIM_PTR         osmPhyContext);

   HIM_STATUS     (HIM_PTR HIMInitialize)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMGetPortTSH)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     void HIM_PTR         osmPortContext);

   HIM_UINT32     (HIM_PTR HIMSizeDevicePortTSCB)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_STATUS     (HIM_PTR HIMCheckDevicePortTSCBNeeded)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT16           index);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateDevicePortTSCB)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT16           index,
                                     void HIM_PTR         devicePortTSCB,
                                     void HIM_PTR         osmDevicePortContext);

   HIM_UINT32     (HIM_PTR HIMSizeLuTSCB)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH);

   HIM_TASK_SET_HANDLE (HIM_PTR HIMCreateLuTSCB)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH,
                                     void HIM_PTR         luTSCB,
                                     HIM_IOB HIM_PTR      probeLuIOB,
                                     void HIM_PTR         osmLuContext);

/* run time interfaces */

   void           (HIM_PTR HIMDisableIRQ)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_BOOLEAN HIM_PTR pLastIRQState);

   void           (HIM_PTR HIMEnableIRQ)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_STATUS     (HIM_PTR HIMPollIRQ)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   HIM_STATUS     (HIM_PTR HIMFrontEndISR)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

   void           (HIM_PTR HIMBackEndISR)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);

#if SAS_MESSAGE_SIGNALED_INTERRUPTS_SUPPORT
   HIM_STATUS     (HIM_PTR HIMFrontEndMSIISR)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           messageData);

   void           (HIM_PTR HIMBackEndMSIISR)
                                    (HIM_TASK_SET_HANDLE  adapterTSH);
#endif /* SAS_MESSAGE_SIGNALED_INTERRUPTS_SUPPORT */

   void           (HIM_PTR HIMQueueIOB)
                                    (HIM_IOB HIM_PTR      pIob);

   HIM_STATUS     (HIM_PTR HIMValidatePortTSH)
                                    (HIM_TASK_SET_HANDLE  portTSH);

   HIM_STATUS     (HIM_PTR HIMClearPortTSH)
                                    (HIM_TASK_SET_HANDLE  portTSH);

   HIM_STATUS     (HIM_PTR HIMValidateDevicePortTSH)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH);

   HIM_STATUS     (HIM_PTR HIMClearDevicePortTSH)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH);

   HIM_STATUS     (HIM_PTR HIMClearLuTSH)
                                    (HIM_TASK_SET_HANDLE  luTSH);

#if SAS_SAVE_RESTORE_STATE
   HIM_STATUS    (HIM_PTR HIMSaveState)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     void HIM_PTR  pState);

   HIM_STATUS    (HIM_PTR HIMRestoreState)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     void HIM_PTR  pState);
#endif /* SAS_SAVE_RESTORE_STATE */

   HIM_STATUS     (HIM_PTR HIMProfileAdapter)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMAdjustAdapterProfile)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           pageCode,
                                     HIM_UINT32           fieldOffset,
                                     void HIM_PTR         pValue);

   HIM_STATUS     (HIM_PTR HIMProfilePort)
                                    (HIM_TASK_SET_HANDLE  portTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);
 
   HIM_STATUS     (HIM_PTR HIMAdjustPortProfile)
                                    (HIM_TASK_SET_HANDLE  portTSH,
                                     HIM_UINT32           pageCode,
                                     HIM_UINT32           fieldOffset,
                                     void HIM_PTR         pValue);

   HIM_STATUS     (HIM_PTR HIMProfilePhy)
                                    (HIM_TASK_SET_HANDLE  phyTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMAdjustPhyProfile)
                                    (HIM_TASK_SET_HANDLE  phyTSH,
                                     HIM_UINT32           pageCode,
                                     HIM_UINT32           fieldOffset,
                                     void HIM_PTR         pValue);

   HIM_STATUS     (HIM_PTR HIMProfileDevicePort)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMAdjustDevicePortProfile)
                                    (HIM_TASK_SET_HANDLE  devicePortTSH,
                                     HIM_UINT32           pageCode,
                                     HIM_UINT32           fieldOffset,
                                     void HIM_PTR         pValue);

   HIM_STATUS     (HIM_PTR HIMProfileLu)
                                    (HIM_TASK_SET_HANDLE  luTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR         pProfilePage);

   HIM_STATUS     (HIM_PTR HIMAdjustLuProfile)
                                    (HIM_TASK_SET_HANDLE  luTSH,
                                     HIM_UINT32           pageCode,
                                     HIM_UINT32           fieldOffset,
                                     void HIM_PTR         pValue);

   HIM_STATUS     (HIM_PTR HIMReadNVSegment)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     void HIM_PTR         destination,
                                     HIM_UINT32           sourceOffset,
                                     HIM_UINT32           bytesToRead,
                                     HIM_UINT32 HIM_PTR   bytesRead);

   HIM_STATUS     (HIM_PTR HIMWriteNVSegment)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     HIM_UINT32           destinationOffset,
                                     void HIM_PTR         source,
                                     HIM_UINT32           length);

   HIM_STATUS     (HIM_PTR HIMGetNVSegmentProperties)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     HIM_UINT16 HIM_PTR   segmentAttributes,
                                     HIM_UINT32 HIM_PTR   segmentSize,
                                     HIM_UINT32 HIM_PTR   segmentImageSize);

   HIM_STATUS     (HIM_PTR HIMReadRAMSegment)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     void HIM_PTR         destination,
                                     HIM_UINT32           sourceOffset,
                                     HIM_UINT32           length);

   HIM_STATUS     (HIM_PTR HIMWriteRAMSegment)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     HIM_UINT32           destinationOffset,
                                     void HIM_PTR         source,
                                     HIM_UINT32           length);

   HIM_STATUS     (HIM_PTR HIMGetRAMSegmentProperties)
                                    (HIM_TASK_SET_HANDLE  adapterTSH,
                                     HIM_UINT32           segmentID,
                                     HIM_UINT16 HIM_PTR   segmentAttributes,
                                     HIM_UINT32 HIM_PTR   segmentSize);
   
   HIM_STATUS     (HIM_PTR HIMProfileNexus)
                                    (HIM_TASK_SET_HANDLE  nexusTSH,
                                     HIM_UINT32           pageCode,
                                     void HIM_PTR  pProfilePage);

   HIM_STATUS     (HIM_PTR HIMClearNexusTSH)
                                    (HIM_TASK_SET_HANDLE  nexusTSH);

#if SAS_OSM_DEBUG_MODE
   HIM_STATUS     (HIM_PTR HIMDumpObject)
                                    (HIM_TASK_SET_HANDLE  debugTSH,
                                     HIM_UINT8            dumpFormat,  
                                     void HIM_PTR         traceInfo,
                                     void HIM_PTR         dumpOptions);
#endif /* SAS_OSM_DEBUG_MODE */

} HIM_FUNC_PTRS;

#endif /* SAS_DOWNSHIFT_SUPPORT */


/****************************************************************************/
/*  entryname function prototypes                                           */
/****************************************************************************/
#ifdef HIM_INCLUDE_SAS
    void HIM_ENTRYNAME_SAS(HIM_FUNC_PTRS HIM_PTR sasFuncPointers,
                           HIM_UINT16 length);
#endif /* HIM_INCLUDE_SAS */


#if SAS_958X_IOP_MODE
#define I2CTDI_MAX_SEP_DEVICES      (2)   /* Maximum SEPs supported, 1 per */
                                          /* internal connector            */
#define SEMB_I2C_ADDRESS            (0xD0)/* Our own I2C address           */
#endif /* SAS_958X_IOP_MODE */
#define SAS_CONNECTOR_INDEX_SAFTE   (0x80)
#define SAS_CONNECTOR_INDEX_UNKNOWN (0xFF)
#if SAS_I2C_SUPPORT
    
#if SAS_I2C_INTERNAL_TDI_SUPPORT
#define SAS_TDI_GET_FUNCTION_POINTERS(ptr,id) SASxInternalTdiGetFunctionPointers(ptr, id)
#elif SAS_I2C_EXTERNAL_TDI_SUPPORT
#define SAS_TDI_GET_FUNCTION_POINTERS(ptr,id) I2CTdiGetFunctionPointers(ptr)
#endif

#if !SAS_958X_IOP_MODE
#define I2CTDI_MAX_SEP_DEVICES      (2)   /* Maximum SEPs supported, 1 per */
                                          /* internal connector            */
#define SEMB_I2C_ADDRESS            (0xD0)   /* Our own I2C address */
#endif /* !SAS_958X_IOP_MODE */

#define I2C_MAX_CMD_LENGTH          (16)  /* Maximum I2C cmd frame length  */
#define I2C_MAX_RETRY_COUNT         (3)   /* retry count not used currently*/

#define IPMB_RQST_SIZE              (0x7)
#define IPMB_RESP_SIZE              (0x7)

#define IPMB_STATUS_GOOD            (0x50)

#define IPMB_CMD_TYPE_SES_READ      (0x02)
#define IPMB_CMD_TYPE_SES_WRITE     (0x82)
#define IPMB_CMD_TYPE_SAFTE_READ    (0x00)
#define IPMB_CMD_TYPE_SAFTE_WRITE   (0x80)

#define IPMP_SEP_CMD_IDENTIFY_SEP   (0xEC)


/***************************************************************************
* I2C TDI Function Table
***************************************************************************/
typedef struct I2CTDI_FUNC_PTRS_
{
   HIM_UINT32     (HIM_PTR I2CTDIVerifyAdapter)    (HIM_TASK_SET_HANDLE);
   HIM_UINT32     (HIM_PTR I2CTDISizeAdapterTSCB)(void);
   HIM_TASK_SET_HANDLE (HIM_PTR I2CTDICreateAdapterTSCB)
                                                   (void HIM_PTR,
                     HIM_TASK_SET_HANDLE);
   HIM_TASK_SET_HANDLE (HIM_PTR I2CTDIGetPhyTSH)   (HIM_TASK_SET_HANDLE);
   HIM_TASK_SET_HANDLE (HIM_PTR I2CTDIGetPortTSH)  (HIM_TASK_SET_HANDLE);
   HIM_UINT32  (HIM_PTR I2CTDISizeDevicePortTSCB)(void);
   HIM_UINT32  (HIM_PTR I2CTDISizeLuTSCB)(void);
   HIM_TASK_SET_HANDLE (HIM_PTR I2CTDICreateDevicePortTSCB)
                                                   (HIM_TASK_SET_HANDLE,
                                                    HIM_UINT16,
                                                    void HIM_PTR);
   HIM_TASK_SET_HANDLE (HIM_PTR I2CTDICreateLuTSCB)
                                                   (HIM_TASK_SET_HANDLE,
                                                    void HIM_PTR,
                                                    HIM_UEXACT8,
                                                    HIM_UEXACT8);
   HIM_UINT32  (HIM_PTR I2CTDICheckIOBMemoryNeeded)(void);
   void        (HIM_PTR I2CTDIQueueIOB)            (HIM_IOB HIM_PTR);
   HIM_UINT32  (HIM_PTR I2CTDIInterruptHandler)    (HIM_TASK_SET_HANDLE);
   HIM_UINT32  (HIM_PTR I2CTDIEnableInterrupt)     (HIM_TASK_SET_HANDLE);
   HIM_UINT32  (HIM_PTR I2CTDIDisableInterrupt)    (HIM_TASK_SET_HANDLE);
} I2CTDI_FUNC_PTRS;

#define I2CTdiVerifyAdapter         SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIVerifyAdapter
#define I2CTdiSizeAdapterTSCB       SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDISizeAdapterTSCB
#define I2CTdiCreateAdapterTSCB     SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDICreateAdapterTSCB
#define I2CTdiGetPhyTSH             SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIGetPhyTSH
#define I2CTdiGetPortTSH            SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIGetPortTSH
#define I2CTdiSizeDevicePortTSCB    SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDISizeDevicePortTSCB
#define I2CTdiSizeLuTSCB            SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDISizeLuTSCB
#define I2CTdiCreateDevicePortTSCB  SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDICreateDevicePortTSCB
#define I2CTdiCreateLuTSCB          SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDICreateLuTSCB
#define I2CTdiCheckIOBMemoryNeeded  SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDICheckIOBMemoryNeeded
#define I2CTdiQueueIOB              SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIQueueIOB
#define I2CTdiInterruptHandler      SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIInterruptHandler
#define I2CTdiEnableInterrupt       SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIEnableInterrupt
#define I2CTdiDisableInterrupt      SAS_ADPTSH(adapterTSH)->tdiBridge.tdiFunctions.I2CTDIDisableInterrupt

typedef struct IPMB_CMD_
{
   HIM_UEXACT8       sepAddress;
   HIM_UEXACT8       cmdType;
   HIM_UEXACT8       checksum1;
   HIM_UEXACT8       sembAddress;
   HIM_UEXACT8       sequence;
   HIM_UEXACT8       sepCommand;
   HIM_UEXACT8       checksum2;
} IPMB_CMD;


typedef struct IPMB_RESP_
{
   HIM_UEXACT8       sembAddress;
   HIM_UEXACT8       cmdType;
   HIM_UEXACT8       checksum1;
   HIM_UEXACT8       sepAddress;
   HIM_UEXACT8       sequence;
   HIM_UEXACT8       sepCommand;
   HIM_UEXACT8       sepStatus;
} IPMB_RESP;

#endif /* SAS_I2C_SUPPORT */
