/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/* $Header: H:\satanexgen\src\chimosm.h_t   \main\9   Tue Mar  8 20:15:58 2005   lin23747 $ */

/***************************************************************************
*
*  Module Name:   CHIMOSM.H
*
*  Description:   Definitions for customization by OSM writer
*
*  Owners:        Template provided by SSG CHIM Team
*
*  Notes:         Contains the following, as required by the 
*                 Serial CHIM Specification:
*                    him_include_xxx: Enables him-specific .h's.
*                    system-specific #defines
*                       OSM_CPU_LITTLE_ENDIAN, 
*                       OSM_CPU_ADDRESS_SIZE, etc.
*                    OSM-specified types
*                       HIM_PTR (near or far)
*                       HIM_UINT8, HIM_UEXACT16, etc.
*                       HIM_BUS_ADDRESS
*                       HIM_BOOLEAN
*                       HIM_IO_HANDLE
*                    macros
*                       HIM_FLUSH_CACHE, etc.
*                       OSMmemset, etc.
*                       OSMoffsetof, OSMAssert, OSMDebugPrint,
*                           OSMEnterDebugger   
*                       HIM_PUT_BIG_ENDIAN, HIM_PUT_LITTLE_ENDIAN, etc.
*
***************************************************************************/

#ifndef CHIMOSM_INCLUDED
#define CHIMOSM_INCLUDED

#include "osmcda.h"
extern void EnterDebugger();
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* HIM's to be included                                                   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#ifdef CDA_FOR_SATA
#define HIM_INCLUDE_SATA   1
#define CHIM_NEEDS_WATCHDOG_SUPPORT 1
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* definitions that needed by the OSM                                     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#define HIM_SCATTER_GATHER_FORMAT0  1        /* Adaptec Common Scatter Gather Format 0   */

#define SAS_PROFILE_INFORMATION     1        /* Support CHIM adapter,         */
                                             /*  deviceport and logical unit  */
                                             /*  profiles                     */
#define SAS_ADAPTER_PROFILE_PAGE_SUPPORT 1   /* Support Adapter Profile       */
                                             /* access using Page Codes       */ 
#define SAS_PHY_PROFILE_PAGE_SUPPORT     1   /* Support Phy Profile access    */
                                             /* using Page Codes              */ 
#define SAS_PORT_PROFILE_PAGE_SUPPORT    1   /* Support Port Profile access   */
                                             /* using Page Codes              */ 
#define SAS_DEVICEPORT_PROFILE_PAGE_SUPPORT 1 /* Support DevicePort Profile   */
                                             /* access using Page Codes       */
#define SAS_LU_PROFILE_PAGE_SUPPORT      1   /* Support Lu Profile access     */
                                             /* using Page Codes              */
#else
#define HIM_INCLUDE_SAS

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* Interface change and backward compatibility options                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/* New interface defines reporting adapter event on a phy which has */
/* persistant link reset errors and currently disabled. Since this is */
/* available in the previous CHIM versions. OSM can use this for backward */
/* compatibility */
#define HIM_REPORT_ACTION_FAIL_PHY               1

/* Sample for 32-bit platforms where size of pointer is size of long */
typedef unsigned long   HIM_UEXACTP; /* unsigned holder - exactly ptr size */
/* Sample for 64-bit platforms where size of pointer is size of longlong */
/*typedef longlong     HIM_UEXACTP;*//* unsigned holder - exactly ptr size */


#endif

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* OEM1's to be included.  It is comment out by default                   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*
 * #define HIM_OEM1_INCLUDE_SAS
 */
/************************************************************************/
/* define OSM_MAXDEVICES                                                */
/************************************************************************/
#if SAS_DOWNSHIFT_SUPPORT
#define OSM_MAXDEVICES 128
#endif /* SAS_DOWNSHIFT_SUPPORT */

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* Endian options                                                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#define OSM_CPU_LITTLE_ENDIAN  1 /*  0 - big    endian                  */
                                 /*  1 - little endian                  */
#define OSM_DMA_SWAP_ENDIAN    0 /*  0 - no swapping                    */
                                 /*  1 - swap every osm_swap_width      */
                                 /* -1 - swap by other means            */
#if _LME 
    #define OSM_CPU_ADDRESS_SIZE  64
    #define OSM_DMA_SWAP_WIDTH    64 /* Number of swapping bits. Used by   */
                                     /* HIM as a DMA-alignment requirement */

#else
    #define OSM_CPU_ADDRESS_SIZE  32
    #define OSM_DMA_SWAP_WIDTH    64 /* Number of swapping bits. Used by   */
                                     /* HIM as a DMA-alignment requirement */
#endif


#define OSM_NULL                -1  /* OSM definition of NULL              */
#define OSM_CACHELINE_SIZE	1

/***************************************************************************
* Definition for memory pointer reference
***************************************************************************/
#if defined(NT) || defined(__linux__) || defined(SCO) || defined (SCO_DDI8) || defined (NETWARE)
#define  HIM_PTR *      /* The pointer passed to all HIM calls and structs */
#else
#define  HIM_PTR far *  /* The pointer passed to all HIM calls and structs */
#endif
/***************************************************************************
* Definition for interface features
***************************************************************************/
#define OSM_EXTENDED_TUNABLE_PARAMETERS 1 /* Parameters tunable */
#define OSM_DIAGNOSTIC_FEATURES         1 /* Diagnosic parameters */  


/* Interface/Feature Option Validity --------------------------------------*/
#if ((SAS_PROGRAMABLE_ALIGN_RATE == 1) && (OSM_EXTENDED_TUNABLE_PARAMETERS == 0)) 
#error SAS_PROGRAMABLE_ALIGN_RATE requires OSM_EXTENDED_TUNABLE_PARAMETERS
#endif

/***************************************************************************
* Definition for data types
***************************************************************************/
#if 0   /* The following are examples of the optimum size for each */
    typedef char            HIM_SINT8;    /* signed   int  - min of   8       */
    typedef unsigned char   HIM_UINT8;    /* unsigned int  - min of   8       */
    typedef char            HIM_SEXACT8;  /* signed   byte - exactly  8       */
    typedef unsigned char   HIM_UEXACT8;  /* unsigned byte - exactly  8       */

    typedef short           HIM_SINT16;   /* signed   int  - min of   16      */
    typedef unsigned short  HIM_UINT16;   /* unsigned int  - min of   16      */
    typedef short           HIM_SEXACT16; /* signed   word - exactly  16      */
    typedef unsigned short  HIM_UEXACT16; /* unsigned word - exactly  16      */

    typedef long            HIM_SINT32;   /* signed   int   - min of  32      */
    typedef unsigned long   HIM_UINT32;   /* unsigned int   - min of  32      */
    typedef long            HIM_SEXACT32; /* signed   dword - exactly 32      */
    typedef unsigned long   HIM_UEXACT32; /* unsigned dword - exactly 32      */
#endif
#if defined(__linux__)
#include <linux/types.h>
/* Sizes optimized for speed */
typedef int32_t         HIM_SINT8;    /* signed   int  - min of   8       */
typedef uint32_t        HIM_UINT8;    /* unsigned int  - min of   8       */
typedef int8_t          HIM_SEXACT8;  /* signed   byte - exactly  8       */
typedef uint8_t         HIM_UEXACT8;  /* unsigned byte - exactly  8       */

typedef int32_t         HIM_SINT16;   /* signed   int  - min of   16      */
typedef uint32_t        HIM_UINT16;   /* unsigned int  - min of   16      */
typedef int16_t         HIM_SEXACT16; /* signed   word - exactly  16      */
typedef uint16_t        HIM_UEXACT16; /* unsigned word - exactly  16      */

typedef int32_t         HIM_SINT32;   /* signed   int   - min of  32      */
typedef uint32_t        HIM_UINT32;   /* unsigned int   - min of  32      */
typedef int32_t         HIM_SEXACT32; /* signed   dword - exactly 32      */
typedef uint32_t        HIM_UEXACT32; /* unsigned dword - exactly 32      */
#elif defined(SCO_DDI8)
typedef int8_t          HIM_SINT8;    /* signed   int  - min of   8       */
typedef uint8_t         HIM_UINT8;    /* unsigned int  - min of   8       */
typedef int8_t          HIM_SEXACT8;  /* signed   byte - exactly  8       */
typedef uint8_t         HIM_UEXACT8;  /* unsigned byte - exactly  8       */

typedef int16_t         HIM_SINT16;   /* signed   int  - min of   16      */
typedef uint16_t        HIM_UINT16;   /* unsigned int  - min of   16      */
typedef int16_t         HIM_SEXACT16; /* signed   word - exactly  16      */
typedef uint16_t        HIM_UEXACT16; /* unsigned word - exactly  16      */

typedef int32_t         HIM_SINT32;   /* signed   int   - min of  32      */
typedef uint32_t        HIM_UINT32;   /* unsigned int   - min of  32      */
typedef int32_t         HIM_SEXACT32; /* signed   dword - exactly 32      */
typedef uint32_t        HIM_UEXACT32; /* unsigned dword - exactly 32      */
#else
/* Sizes optimized for speed */
typedef long            HIM_SINT8;    /* signed   int  - min of   8       */
typedef unsigned long   HIM_UINT8;    /* unsigned int  - min of   8       */
typedef char            HIM_SEXACT8;  /* signed   byte - exactly  8       */
typedef unsigned char   HIM_UEXACT8;  /* unsigned byte - exactly  8       */

typedef long            HIM_SINT16;   /* signed   int  - min of   16      */
typedef unsigned long   HIM_UINT16;   /* unsigned int  - min of   16      */
typedef short           HIM_SEXACT16; /* signed   word - exactly  16      */
typedef unsigned short  HIM_UEXACT16; /* unsigned word - exactly  16      */

typedef long            HIM_SINT32;   /* signed   int   - min of  32      */
typedef unsigned long   HIM_UINT32;   /* unsigned int   - min of  32      */
typedef long            HIM_SEXACT32; /* signed   dword - exactly 32      */
typedef unsigned long   HIM_UEXACT32; /* unsigned dword - exactly 32      */
#endif

/***************************************************************************
* Define a HIM_STATUS
***************************************************************************/
typedef HIM_UINT8       HIM_STATUS;    /* HIM_STATUS is minimum of 8 bits  */


#ifdef _WIN64
typedef unsigned __int64 ULONG_PTR, *PULONG_PTR;
#else
typedef unsigned long ULONG_PTR, *PULONG_PTR;
#endif

// Win64 defines
#define WIN64_ALIGN 0x7 //! Win64 align works for all

#ifdef NT
typedef uintptr_t               HIM_DATANPTR;
#elif LINUX
typedef unsigned long		HIM_DATANPTR;
#else
typedef unsigned long       HIM_DATANPTR;
#endif
/***************************************************************************
* Define a BUS ADDRESS (Physical address)
***************************************************************************/
/* Sample of HIM_BUS_ADDRESS to support 64-bit Host Bus Addressing */
typedef struct HIM_BUS_ADDRESS_
{
   HIM_UEXACT32 u32entity0;
   HIM_UEXACT32 u32entity1;
} HIM_BUS_ADDRESS;

#if _LME

#define HIM_GET_BUS_ADDRESS(hba)		\
	((hba).u32entity0 | (((HIM_DATANPTR)(hba).u32entity1) << 32))
#define HIM_SET_BUS_ADDRESS(hba, ba)	\
do {									\
	(hba).u32entity0 = (HIM_UEXACT32)((ba) & 0xFFFFFFFFULL);	\
	(hba).u32entity1 = (HIM_UEXACT32)((ba) >> 32);			\
} while (0)
#define HIGH_32(x) (ULONG)(x >> 32)

#else // 32 bit

#define HIM_GET_BUS_ADDRESS(hba)		\
	((hba).u32entity0 )
#define HIM_SET_BUS_ADDRESS(hba, ba)	\
do {									\
	(hba).u32entity0 = (HIM_UEXACT32)(ba);	\
	(hba).u32entity1 = 0;			\
} while (0)

#endif

/***************************************************************************
* Define a boolean 
***************************************************************************/
typedef enum HIM_BOOLEAN_ENUM { HIM_FALSE, HIM_TRUE } HIM_BOOLEAN; /* boolean */

/***************************************************************************
* Definitions for the I/O handle
***************************************************************************/
typedef struct HIM_IO_HANDLE_
{
    HIM_DATANPTR    baseAddress; /* raw base address */
    HIM_UEXACT32    attributes;  /* mostly same as him #defines */
	HIM_BOOLEAN     memoryMapped;/* TRUE if memory mapped */
	HIM_BOOLEAN     handleUsed;  /* TRUE if this handle instance is valid,
                                  * FALSE if available as new handle */
} HIM_PTR HIM_IO_HANDLE;

/***************************************************************************
* Definitions for cache flushing and invalidating
***************************************************************************/
/* Intel platform -- define as null */
#define  HIM_FLUSH_CACHE(pMemory,length)
#define  HIM_INVALIDATE_CACHE(pMemory,length)

/***************************************************************************
* Definitions for standard ANSI-C functions
***************************************************************************/
#ifdef NT

#define  OSMmemset(buf,value,length)   memset((HIM_UEXACT8 HIM_PTR) buf,value,(HIM_UINT32) length)
#define  OSMmemcmp(buf1,buf2,length)   memcmp((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)
#define  OSMmemcpy(buf1,buf2,length)   memcpy((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)

#endif

#ifdef NETWARE

#define  OSMmemset(buf,value,length)   CSetB(value, (HIM_UEXACT8 HIM_PTR) buf, (HIM_UINT32) length)
#define  OSMmemcmp(buf1,buf2,length)   NWmemcmp((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)
#define  OSMmemcpy(buf1,buf2,length)   CMovB((HIM_UEXACT8 HIM_PTR) buf2,(HIM_UEXACT8 HIM_PTR)buf1,(HIM_UINT32)length)

#endif

#ifdef SCO_DDI8

#define  OSMmemset(buf,value,length)   XMemSet((HIM_UEXACT8 HIM_PTR) buf,value,(HIM_UINT32) length)
#define  OSMmemcmp(buf1,buf2,length)   bcmp((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)
#define  OSMmemcpy(buf1,buf2,length)   XMemCpy((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)

#endif

#ifdef __linux__

extern VOID XMemZero(UCHAR DPTR, ULONG);
extern VOID XMemCpy(UCHAR DPTR, UCHAR DPTR, ULONG);
extern UINT32 XMemcmp(UCHAR DPTR , UCHAR DPTR, UINT32);
#define  OSMmemset(buf,value,length)   XMemZero((HIM_UEXACT8 HIM_PTR) buf,(HIM_UINT32) length)
#define  OSMmemcmp(buf1,buf2,length)   XMemcmp((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)
#define  OSMmemcpy(buf1,buf2,length)   XMemCpy((HIM_UEXACT8 HIM_PTR) buf1,(HIM_UEXACT8 HIM_PTR)buf2,(HIM_UINT32)length)

#endif

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* miscellaneous system utilities */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#ifdef SCO_DDI8
#define OSMoffsetof(x,y) ((int )&((x *)0)->y)
#define OSMAssert(x)
#else // SCO_DDI8
#ifdef NT
#define OSMoffsetof                    offsetof
#else
#define OSMoffsetof(structDef,member)  (HIM_UEXACT16)((HIM_DATANPTR)&(((structDef *)0)->member))
#endif
#define OSMAssert(x)                   if(!(x)) EnterDebugger();
#endif // SCO_DDI8


/* By default OSMDebugPrint (and OSMDebugPrintWithNewline, which is same as the
 * OSMDebugPrint, except that it introduces a '\n' character after printing the
 * given string) is not defined, as we do not want any performance impact.  A
 * sample definition has been provided below to illustrate the usage of 
 * 'threshold' variable.
 */
extern void	XOSMDebugPrint(unsigned long dbg_level, char *dbg_msg, ...);
	
//#define OSMDebugPrint(threshold,string)
//#define OSMDebugPrintWithNewline(threshold,string)

/* Sample definition of OSMDebugPrint to be used with HIM_DEBUG_XXX mask bits */
/* #define OSMDebugPrint(threshold,string) \
 *  if (OSMDebugMode & threshold) \
 *  { \
 *     printf string; \
 *  }
 */

/* Sample definition of OSMDebugPrintWithNewline to be used with HIM_DEBUG_XXX
 * mask bits
 */
/* #define OSMDebugPrintWithNewline(threshold,string) \
 *  if (OSMDebugMode & threshold) \
 *  { \
 *     printf string; \
 *     printf("\n"); \
 *  }
*/

/*
 * Sample definition of OSMDebugPrint.
 *
 * extern  HIM_UINT32 OSMDebugMode;
 * extern  long       ScreenHandle;
 *
 * #define OSMDebugPrint(threshold,string) if ((threshold) <= OSMDebugMode) \
 *                                         OutputToScreen(ScreenHandle,string);
 */

#define OSMEnterDebugger()             EnterDebugger();

/* The following define is invoked by the CHIM to return
 * OSMEvent information related to the OSMEvent when the 
 * event value is HIM_EVENT_HA_FAILED or HIM_EVENT_IO_CHANNEL_FAILED.
 */

#define OSMDebugReportEventInfo(osmAdapterContext,eventInfo)  
/*
 * Sample definition
 *
 * void HIM_PTR osmAdapterContext;
 * HIM_UINT32 eventInfo; 
 *
 * #define OSMDebugReportEventInfo(osmAdapterContext,eventInfo)  \
 *           OSMLogCallerId((osmAdapterContext),(eventInfo))
 */

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* Endian-transformation macros    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#define HIM_PUT_BYTE_STRING(pContext,dmaStruct,offset,pSrc,length)\
        OSMmemcpy(((HIM_UEXACT8 *)(dmaStruct))+(offset),    \
                  (HIM_UEXACT8 *)pSrc,                      \
                  length)

#define HIM_PUT_LITTLE_ENDIAN8(adapterTSH,dmaStruct,offset,value)  \
            *(((HIM_UEXACT8 *)(dmaStruct))+(offset)) = (HIM_UEXACT8)(value)

#define HIM_PUT_LITTLE_ENDIAN16(adapterTSH,dmaStruct,offset,value) \
            *((HIM_UEXACT16 *)(((HIM_UEXACT8 *)\
               (dmaStruct))+(offset))) = (HIM_UEXACT16)(value)

#define HIM_PUT_LITTLE_ENDIAN24(adapterTSH,dmaStruct,offset,value) \
            *((HIM_UEXACT32 *)(((HIM_UEXACT8 *)(dmaStruct))+(offset))) &= \
                        0xFF000000; \
            *((HIM_UEXACT32 *)(((HIM_UEXACT8 *)(dmaStruct))+(offset))) |= \
                        ((HIM_UEXACT32)(value)) & 0x00FFFFFF

#define HIM_PUT_LITTLE_ENDIAN32(adapterTSH,dmaStruct,offset,value) \
            *((HIM_UEXACT32 *)(((HIM_UEXACT8 *)\
               (dmaStruct))+(offset))) = (HIM_UEXACT32)(value)
                                       
#define HIM_PUT_LITTLE_ENDIAN64(adapterTSH,dmaStruct,offset,value) \
            HIM_PUT_LITTLE_ENDIAN32((adapterTSH),(dmaStruct),(offset),\
                                       *((HIM_UEXACT32 *)&value));\
            HIM_PUT_LITTLE_ENDIAN32((adapterTSH),(dmaStruct),(offset)+4,\
                                       *(((HIM_UEXACT32 *)&value)+1))

#define HIM_GET_BYTE_STRING(pContext,pDest,dmaStruct,offset,length)\
        OSMmemcpy((HIM_UEXACT8 *)pDest,                      \
                  ((HIM_UEXACT8 *)(dmaStruct))+(offset),     \
                  length);

#define HIM_GET_LITTLE_ENDIAN8(adapterTSH,pValue,dmaStruct,offset)  \
            *((HIM_UEXACT8 *)pValue) = *(((HIM_UEXACT8 *)(dmaStruct))+(offset));

#define HIM_GET_LITTLE_ENDIAN16(adapterTSH,pValue,dmaStruct,offset) \
            *((HIM_UEXACT16 *)pValue) =\
               *((HIM_UEXACT16 *)(((HIM_UEXACT8 *)(dmaStruct))+(offset)))
                                                            

#define HIM_GET_LITTLE_ENDIAN24(adapterTSH,pValue,dmaStruct,offset) \
            *((HIM_UEXACT32 *)pValue) = *((HIM_UEXACT32 *)\
                  (((HIM_UEXACT8 *)(dmaStruct))+(offset))) & 0x00FFFFFF
                        
#define HIM_GET_LITTLE_ENDIAN32(adapterTSH,pValue,dmaStruct,offset) \
            *((HIM_UEXACT32 *)pValue) =  *((HIM_UEXACT32 *)\
                                 (((HIM_UEXACT8 *)(dmaStruct))+(offset)))

#define HIM_GET_LITTLE_ENDIAN64(adapterTSH,pValue,dmaStruct,offset) \
            HIM_GET_LITTLE_ENDIAN32((adapterTSH),(pValue),(dmaStruct),(offset));\
            HIM_GET_LITTLE_ENDIAN32((adapterTSH),((HIM_UEXACT32 *)\
                                          (pValue))+1,(dmaStruct),offset+4)

#define HIM_PUT_BIG_ENDIAN8(adapterTSH,dmaStruct,offset,value)\
            *(((HIM_UEXACT8 *)(dmaStruct))+(offset)) = \
                                       (HIM_UEXACT8)(value)

#define HIM_PUT_BIG_ENDIAN16(adapterTSH,dmaStruct,offset,value)\
            HIM_PUT_BIG_ENDIAN8((adapterTSH),(dmaStruct),(offset),\
                                      (HIM_UEXACT8)(value>>8));\
            HIM_PUT_BIG_ENDIAN8((adapterTSH),(dmaStruct),(offset)+1,\
                                      (HIM_UEXACT8)(value))

#define HIM_PUT_BIG_ENDIAN24(adapterTSH,dmaStruct,offset,value)\
            HIM_PUT_BIG_ENDIAN8((adapterTSH),(dmaStruct),(offset),\
                                   (HIM_UEXACT8)(value>>16));\
            HIM_PUT_BIG_ENDIAN8((adapterTSH),(dmaStruct),(offset)+1,\
                                   (HIM_UEXACT8)(value>>8));\
            HIM_PUT_BIG_ENDIAN8((adapterTSH),(dmaStruct),(offset)+2,\
                                   (HIM_UEXACT8)(value))

#define HIM_PUT_BIG_ENDIAN32(adapterTSH,dmaStruct,offset,value)\
            HIM_PUT_BIG_ENDIAN16(adapterTSH,dmaStruct,offset,\
                                  (HIM_UEXACT16)(value>>16));\
            HIM_PUT_BIG_ENDIAN16(adapterTSH,dmaStruct,offset+2,\
                                  (HIM_UEXACT16)(value))

#define HIM_PUT_BIG_ENDIAN64(adapterTSH,dmaStruct,offset,value) \
            HIM_PUT_BIG_ENDIAN32((adapterTSH),(dmaStruct),(offset),\
                              *(((HIM_UEXACT32 *)&value)+1));\
            HIM_PUT_BIG_ENDIAN32((adapterTSH),(dmaStruct),(offset)+4,\
                              *(((HIM_UEXACT32 *)&value)+0))

#define HIM_GET_BIG_ENDIAN8(adapterTSH,pValue,dmaStruct,offset)\
            *((HIM_UEXACT8 *)pValue) = *(((HIM_UEXACT8 *)(dmaStruct))+\
                                                            (offset))

#define HIM_GET_BIG_ENDIAN16(adapterTSH,pValue,dmaStruct,offset)\
            HIM_GET_BIG_ENDIAN8((adapterTSH),((HIM_UEXACT8 *)\
                                    (pValue))+1,(dmaStruct),(offset));\
            HIM_GET_BIG_ENDIAN8((adapterTSH),(pValue),(dmaStruct),(offset)+1);

#define HIM_GET_BIG_ENDIAN24(adapterTSH,pValue,dmaStruct,offset)\
            HIM_GET_BIG_ENDIAN8((adapterTSH),((HIM_UEXACT8 *)(pValue))+2,\
                                                (dmaStruct),(offset));\
            HIM_GET_BIG_ENDIAN8((adapterTSH),((HIM_UEXACT8 *)(pValue))+1,\
                                                (dmaStruct),(offset)+1);\
            HIM_GET_BIG_ENDIAN8((adapterTSH),(pValue),(dmaStruct),(offset)+2)

#define HIM_GET_BIG_ENDIAN32(adapterTSH,pValue,dmaStruct,offset)\
            HIM_GET_BIG_ENDIAN16((adapterTSH),((HIM_UEXACT16 *)(pValue))+1,\
                                                 (dmaStruct),(offset));\
            HIM_GET_BIG_ENDIAN16((adapterTSH),((HIM_UEXACT16 *)(pValue)),\
                                                (dmaStruct),(offset)+2);

#define HIM_GET_BIG_ENDIAN64(adapterTSH,pValue,dmaStruct,offset)\
            HIM_GET_BIG_ENDIAN32((adapterTSH),((HIM_UEXACT32 *)(pValue))+1,\
                                                (dmaStruct),(offset));\
            HIM_GET_BIG_ENDIAN32((adapterTSH),((HIM_UEXACT32 *)(pValue)),\
                                                (dmaStruct),(offset)+4)


#endif
