/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/***********************************************************************
Filename:confpage.h
This file contains the data structure of CDA style meta data called
ReservedSectorsLayout.



*** I M P O R T A N T *** I M P O R T A N T *** I M P O R T A N T ***

*** PLEASE MAINTAIN 4 BYTE ALIGNMENT FOR EVERY SINGLE STRUCTURE. ****

***********************************************************************

History :

03/16/2005 : 17:00  Shing-Kin Pun
1.  Added definitions for SH_irocFlags.

10/08/03: Madhuresh
MAX_ARRAYS_IN_META_DATA has been replaced by CDA_MAX_RAIDS_PER_DISK.

04/17/2003 18:00 NAR20038 Manjunath
Used hard coded values for RE_MAX_MEMBERS_PER_RAID and MAXARRAYMEMBERS.
We have to do this since now we have a requirement of single binary for
SCSI and SATA.

Aug 7, 2002:  1:40 PM   Madhuresh
Define SH_irocFlags in the metadata header area. This is copied to/from
irocFlags in header entry of Babylon type metadata. Also defined
VERIFY_WITH_AUTOFIX which will be used in cfginit.c defined SIZE_OF_RAID_NAME.

05/15/2002:  14:42 Manjunath	NAR20038
added new line at the end of the file, to avoid compilation error
under unix.

12/20/2001:  14:28 Manjunath	NAR20038
Removed unwanted #ifdefs

09/04/2001:  1:40 PM   Madhuresh
Only to make the size of new LR_Dirty field as UCHAR
from USHORT. This removes a compile time warning as well as
keeps one byte for future usage.

08/31/2001:  4:40 PM   Madhuresh
Defined a byte field for handling DIRTY state which defines if the system
was shutdown gracefully (a call to our shutdown routines). This helped
handling drive failure during shutdown in a cleaner way.

08/22/2001:  12:40 PM   Madhuresh
Removed OTCPage.

06/15/2001:  18:00 PM   Madhuresh
To store the SD_SpareDriveMagic what is the drivemagic of the spare to
show up later.

06/07/2001:  18:00 PM   Madhuresh
To increase the size of StoredMemberPage for Spare.

04/12/2001:  18:00 PM   Madhuresh
Added UCHAR SD_CheckpointActivity by stealing a byte from USHORT
SD_PhysDriveIndex.

04/06/2001:  11:00 AM   Madhuresh
Removed some more unwanted fields from the _OTCPage which is embedded
into the StoredDevicePage.

04/04/2001 : 18:00 PM   Madhuresh

Now the fields required for OTC meta data inside CDA are all under
StoredDevicePage only. Dropped some fields from header or member pages.
Create/Delete RAID now works from iomgr.

03/28/2001 : 18:00 PM   Madhuresh

Moved 3 fields from header to member page to fix a bug. Added checkpoint
field in the device page and removed number of logical arrays def from here.

03/16/2001 : 14:48	NAR20038 Manjunath
Updated all the structures to maintain 4 byte alignment. This is
required since this file is used by other components like IOMGR.
Compiler adds padding by default when IOMGR is compiled. This
option is disabled when driver is compiled since structure might
represent actual hardware registers.
RESERVED_SECTOR_COUNT is modified to 4 sectors since size of RS
dropped.
PLEASE MAKE SURE TO MAINTAIN 4 BYTE ALIGNMENT, whenever there is
a change in data structures here. Also please add offsets when 
updating.

03/09/2001 : 11:07 AM   NAR20038 Manjunath
Dropped not required fields for IROC product.

01/31/2001 : 10:31 AM   NAR20038 Manjunath
Added few more CDA related fields to that IOMgr would compile. Later
we can remove them as we clean up IOMgr.

12/13/2000 : 18:00 PM   Madhuresh

Shrunk the meta data size from 12KB down to 1KB by dropping UNUSED fields
and by dropping the number of array members down to 4 from 32.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.
*************************************************************************/

#ifndef CONFPAGE_INCLUDED
#define CONFPAGE_INCLUDED
#include	"sysdef.h"

#define CDA_MAX_SMP_PAGES       6

#define NUM_HEADS               0xFF	// metadata.c reports this constant
#define NUM_SPT                 0x3F	// -do-

#define	SIZE_OF_RAID_NAME		16
#define	MAX_OTC_ENTRIES			4 * 2 + 1
#define CONFIG_VERSION          1
#define CONFIG_REVISION         1

/*
 *>>SKP 03/15/2005
 *
 *  SH_irocFlags Definition.
 *
 *    Bit 0 = IROC_VERIFY_WITH AUTOFIX
 *    Bit 1 = Reserved
 *    Bit 2 = Reserved
 *    Bit 3 = IROC_DRIVE_WRITE_CACHE_ENABLED
 *    Bit 4 = Reserved
 *    Bit 5 = Reserved
 *    Bit 6 = Reserved
 *    Bit 7 = IROC_RS_SD_BOOTABLE 
 *
 *    Usage :  0 = Disable
 *             1 = Enable
 *
 *    SH_irocFlags is a member of the struct _StoredHeader that is the
 *    header part of the CDA internal metadata structure.
 *
 */
#define IROC_VERIFY_WITH_AUTOFIX             0x01   // Bit 0
#define IROC_DRIVE_WRITE_CACHE_ENABLED       0x08   // Bit 3
#define IROC_RS_SD_BOOTABLE                  0x80   // Bit 7
#define IROC_NO_CHANGE_IN_DRIVE_WRITE_CACHE  0x20   



/*>>SKP
 *  The following definition is not used in the code and should be removed.
 */
#define VERIFY_NO_NOTIFICATION  2

#define	RS_SIZE					(sizeof(ReservedSectorLayout))
#define RESERVED_SECTORS_COUNT  ((RS_SIZE) % 512 ? ((RS_SIZE/512)+1) : RS_SIZE/512)
#define RESERVED_SECTORS_BYTES	RESERVED_SECTORS_COUNT * 512

/*
	Maximum RAIDs a disk can participate into. In another word, how many
	containers can a disk have.
*/
#define	CDA_MAX_RAIDS_PER_DISK      1

typedef struct _SysDeviceCacheCapability
{
	ULONG				AvailQThreshold;
	ULONG				FlushQThreshold;
	ULONG				CacheSize;
	USHORT				AvailQCount;	// Total cache blks in this device
	UBYTE				CacheBlockSize;
	UBYTE				LookAheadSize;
	UBYTE				MaxDemandCache;
	UBYTE				MaxWriteBack;
	UBYTE				MaxWriteThrough;
	UBYTE				WriteBackPortion;
 	UBYTE	    		CacheFlag;		// 0 => Cache off, 1=>Cache on
 	UBYTE				Padding1;
	USHORT				Padding2;
	ULONG				UnusedCacheSize; // Cache available in the controller cache pool

} SysDeviceCacheCapability, DPTR pSysDeviceCacheCapability;
//*********************************************************************
//
// Each member will have a {channel, scsi-id} to reach it
//
// This structure hold such info for PhysicalDevicePage
//
//*********************************************************************
typedef struct _ParallelUnit					// 4 bytes
{
/* 0x0000 */	USHORT  PU_SCSIBusIndex;		// Relative to Board
/* 0x0002 */	USHORT  PU_Device;				// SCSI Id

} ParallelUnit, DPTR pParallelUnit;


//*********************************************************************
//
// Each member has properties such as
// {Capacity in blocks, Starting and Ending LBA, and {Channel, scsi-id} }
//
// This structure hold such info for PhysicalDevicePage
//
//*********************************************************************
typedef struct _PhysicalDevicePage			// 0x14 (20) bytes
{
/* 0x0000 */	ULONG   PD_Capacity;		// Useable capacity
/* 0x0004 */	ULONG   PD_Start;			// Where membership in LD starts
/* 0x000C */	ULONG   PD_End;				// Where membership in LD ends
/* 0x0010 */	union   _PD_PhysDesc		// 4 bytes
				{
/* 0x0010 */		ParallelUnit    PD_Scsi;
				
				} PD_PhysDesc;

} PhysicalDevicePage, DPTR pPhysicalDevicePage;
//*********************************************************************
//
// Define the devices that are members of logical arrays.
// These members may point to either logical array devices or physical devices.
//
//*********************************************************************
typedef struct _StoredMemberPage        // 0x18 (24) bytes
{
/* 0x0000 */	USHORT  SM_State;		// See stats.h: PHYS_STATE_ACTIVE = 0
										// PHYS_STATE_FAILED = 1
/* 0x0002 */	USHORT  SM_PhysType;
/* 0x0004 */	union   _SM_Device
				{
/* 0x0004 */		PhysicalDevicePage  SM_PhysDef; // 20 bytes, if SM_PhysType = PHYS_SCSI2
					ULONG               SM_ArrayId; // if SM_PhysType = PHYS_LOGICAL

				} SM_Device;

} StoredMemberPage, DPTR pStoredMemberPage;
//*********************************************************************
//
// The state and type of array, are preserved here.
//
// This structure is part of Device Page
//
//*********************************************************************
typedef struct _LogicalRedundancyPage			// 8 bytes
{
/* 0x0000 */	USHORT  LR_RedundancyStripe;	// blocks per redundancy stripe
/* 0x0002 */	USHORT  LR_RedundancyType;		// xor, copy, none, etc.
/* 0x0004 */	UCHAR   LR_RedundancyState;		// State of Array
/* 0x0005 */	UCHAR	LR_RedundancySubstate;	// Substate of Array
/* 0x0006 */	UCHAR	LR_Dirty;				// If TRUE, system was not SHUTDOWN gracefully.
/* 0x0007 */    UCHAR   RebuildFailedMemberIndex;

/* 0x0008 */
} LogicalRedundancyPage, DPTR pLogicalRedundancyPage;
//*********************************************************************
//
// This is the header part of Meta Data.
//
// All of useful array info is kept in Device Page. If SH_SpareFlag
// is set, most of the Device Page except ArrayID is to be ignored.
//
//*********************************************************************
typedef struct   _StoredHeader					// 0x14 (20) bytes
{
/* 0x0000 */	UCHAR   SH_Signature[16];// "ADAPTEC ARRAY   "
/* 0x0010 */	UCHAR   SH_Version;		// Fixed to 1
/* 0x0011 */	UCHAR   SH_Revision;	// Fixed to 1
/* 0x0012 */	UCHAR   SH_SpareFlag;	// If set, this drive is SPARE
/* 0x0013 */	UCHAR   SH_irocFlags ;	//
 
} StoredHeader,DPTR pStoredHeader;
//*********************************************************************
//
// All Important StoredDevicePage
//
// This contains fields specific to define the array and a index that
// tells position of current member (whose meta data has been just read)
// within the array. e.g. third member of array will have the index as 2.
//
// Notice the SD_Timestamp. Each member's meta data will bring this and
// all healthy members must have same timestamp which is higher of all.
// Those with lower timestamp and same ArrayId are such members which were
// removed sometime back without zapping their meta data.
//
//*********************************************************************
typedef struct _StoredDevicePage			// 0x130 (304) bytes
{
/* 0x0000 */	UCHAR   SD_Alias[SIZE_OF_RAID_NAME];// 16 byte ASCIIZ name (including Null)
/* 0x0010 */	ULONG   SD_ArrayId;			// All members must have this Unique identifier
/* 0x0014 */	ULONG   SD_Timestamp;		// All current members must have timestamp highest
/* 0x0018 */	ULONG   SD_TotalBlocks;		// Capacity of this array
/* 0x001C */	ULONG   SD_CheckpointLba;	// Verify/Rebuild has been completed till this lba.
/* 0x0020 */	ULONG   SD_VerifyDate;		// Used by IOMGR only
/* 0x0024 */	ULONG   SD_RecreateDate;	// Used by IOMGR only
/* 0x0028 */	USHORT  SD_MemberIndex;		// Position of this member in array
/* 0x002A */	USHORT  SD_NumMembers;		// Number of members in this array
/* 0x002C */	USHORT  SD_NumHeads;		// Heads - used by IOMGR only
/* 0x002E */	USHORT  SD_NumSPT;      	// Sectors Per Track - used by IOMGR only
/* 0x0030 */	USHORT  SD_NumCylinders;
/* 0x0032 */	USHORT  SD_DataStripe;
/* 0x0034 */	UCHAR	SD_PhysDriveIndex;	// To determine boot order
/* 0x0035 */	UCHAR	SD_CheckpointActivity;	// ARRAY_NO_OP / ARRAY_VERIFY_OP / ARRAY_REBUILD_OP / ARRAY_ZERO_OP
/* 0x0036 */	UCHAR   SD_SpareCnt;
/* 0x0037 */	UCHAR	SD_IVRPriority ;	// Init/Verify/Rebuild Priority as passed by BIOS

#if SAS
    /* TDE - added logical P T L */
/* 0x0038 */	UCHAR	SD_Path;			// Logical unit path id 
/* 0x0039 */	UCHAR	SD_TargetId;		// Logical unit target id
/* 0x003a */	UCHAR	SD_LUN;             // For configured SINGLE LUN value is real.
                                            // For RAID device LUN value is always 0.
/* 0x003b */	UCHAR	SD_Unused1;
#else
    /* This is currently unused, so rename it to save Logical unit PT value */
/* 0x0038 */	ULONG	SD_RCTBL_LBA ;		// Where the RCTBL starts on disk
#endif
/* 0x003C */	ULONG	SD_SpareDriveMagic ;// To preserve the drivemagic of spare disk.

/* 0x0040 */	LogicalRedundancyPage   SD_RPage ;			// 8 bytes
/* 0x0048 */	StoredMemberPage	SD_StoredMemberPage[CDA_MAX_SMP_PAGES] ; // 
/* 0x00F0 */	ULONG	SD_OTC_raidmagic [MAX_OTC_ENTRIES] ;		// 16*4 = 0x40 (64) bytes
/* 0x0130 */
}  StoredDevicePage, DPTR pStoredDevicePage;
//*********************************************************************
//
// Layout of Reserved Sectors
//
// Now this has the 20 byte header and one instance of StoredDevicePage
//
//*********************************************************************
typedef struct _ReservedSectorsLayout								// 0x144 (324)bytes
{
/* 0x0000 */	StoredHeader        Header;							// 0x014 (20) bytes
/* 0x0014 */	StoredDevicePage    SDP[ CDA_MAX_RAIDS_PER_DISK ];	// 0x0130 (304) bytes
/* 0x0144 */
} ReservedSectorLayout, DPTR pReservedSectorLayout;

#define CDA_CONFIG_BUFFER_SIZE          (1024)
//*********************************************************************

#endif

