
/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:coreprot.h

File Description

Prototype definitions for all routines in the core directory.
***********************************************************************

History :
05/25/2006: cha23416 Naveen Chandrasekaran
 Added macro ClearMEHFlags to clear MEH flags

3/24/2005: KenC
1. Added the following function prototypes:
UpdateWCEMetaData();
WCEMetaDataReadComplete();
WCEMetaDataWriteToNextMember();
WCEMetaDataWriteComplete();

03/18/2005 : 20:40   Shing-Kin Pun
1. Replaced Function Prototype in File cdar10.c.
2. Renamed OverlayRW() to ReadWriteSD().
3. Renamed PostOverlayRW() to PostReadWriteSD().

09/23/2004 : 14:40 BUR11729 Farrukh
Added prototypes for GetFreeTSCB() and FreeTSCB().

08/02/2004 : 10:27 NAR20038 Manjunath
Added prototypes for PhysRemoveSasDevice() and CdaRemoveExistingDevice()

07/20/2004 : 14:08 NAR20038 Manjunath
Added prototypes for Querying task.

07/20/2004 : 10:41 NAR20038 Manjunath
Added prototype for HandleIobEvents() . .. 

07/19/2004 : 14:05 NAR20038 Manjunath
Added prototypes for ResendPendingIOs(), CdaNotify(), FlushIOsOnPD()

07/13/2004 : 12:29 BUR11729 Farrukh
Added function prototypes for PhysQueueQuiesce and ResumeAdapter.

06/24/2004 : 10:10 NAR20038 Manjunath
Added prototypes for HandleAdapterEvents() and DiscoverNewPorts()

06/17/2004 : Tam
Added function prototypes for ResetLogicalUnit, ResetLogicalUnitComplete.
Other Reset prototypes added earlier includes ResetAdapter, ResetDevicePort
and ResetSASHardware.

06/17/2004 : 13:06 NAR20038 Manjunath
Added function prototypes for Port related events and hot-plug modules.
HandlePortEvents(), PostPacForPort(),
ProcessNewTargets(), PostProcessNewTargets(), UpdateArrayReservedSector(),
UpdateArraySysDeviceStatus().

11/21/2003 : Tam
New function protos CSMI_Init.

10/02/2003 : Madhuresh
New function protos IoctlPauseTasks, IoctlResumeTasks.

04/30/2003 : 17:05 BUR11729 Farrukh
Renamed function FixArrayMemberPosition to UpdateArrayReservedSector

04/10/2003 : 15:00  BUR11729 Farrukh
Added prototype RescanZapPartitionSector and RescanZapPartitionSectorDone.

04/08/2003 : 17:13  NAR20038 Manjunath
Added FixArrayMemberPosition under rescan.c


11/05/2002 : 18:35  NAR20038 Manjunath
Added prototype MoveRescanToWaitingList.

Oct 18, 2002: Madhuresh
LockNextRange: When background op like verify or rebuild enters a new range,
we should lock that range for the IOs. Whenever this happens
all such IODs are flagged and current RD is kept in SD for when
IOPull happens, it examins and tries to lock the range again. One
public call now kept here.

Sept 24, 25, 2002: Madhuresh.
Hot Swap Rebuild. When a disk is found at a scsi id which is
still referred by the meta data of any RAID1/10 as its missing
member, the newly found disk is moved in as a member and
a rebuild is initiated.

05/20/2002 : Farrukh
included raidglbl.h to remove compilation errors.

05/20/2002 : Madhuresh
Changed PVOIDs to pADAPTER_DEV_EXT for all prototypes who need these.

05/15/2002 : 13:45 Farrukh
Added defination for MDT_ReadWrite.

Removed SetInitIODSemaphore, ClearInitIODSemaphore and	AwaitInitIODSemaphore
Not used. 

A void pointer to ADAPTER_DEV_EXT is passed in many routine as parameter
so the Global which are now part of ADAPTER_DEV_EXT are accessible to the 
routine.

12/19/2001 : 17:01  NAR20038 Manjunath
Added NOMACRO definition for macros FreeIOD and FreeOSD. These most often
used functions are made macro to improve performance. Removed unwanted 
#ifdefs

12/18/2001 : 11:35  NAR20038 Manjunath
Updated IOPull proto to take only one parameter. Removed all #ifdef NT
lines. AdapterInterruptHandler() is made a macro intead of a function,
to improve performance.

12/07/2001 : 10:50  NAR20038 Manjunath
Removed _JAGUAR defines.

08/27/2001 : 17:15  Madhuresh
Added a routine AdjustSpareDisks which scans pOtherHead and
modifies the meta data of main members for missing links of DHS
or for any change of scsi id of the spares compared to whats on
their meta data. Removed these
WriteReservedSectorsComplete - Was not being used any more.
UpdatePoolSpareReserveSector - changed to AdjustSpareDisks.


04/17/2001 : 19:05  NAR20038 Manjunath
Added new function prototypes under raidsubr.c and recover.c

04/17/2001 : 13:40  NAR20038 Manjunath
Added prototype for ArrayOpCompleted() defined in ioctl.c

04/13/2001 : 12:07  Manjunath
Added prototypes for checkpointing modules under ioctl.c

04/10/2001 : 11:00 AM   Madhuresh
Added prefix MDT_ to ReadWriteMetaDataWithoutWait after making a new
module metadata.c for translation of reserved sector info.

03/16/2001 : 11:20 AM	BURN11729 Farrukh Burney
Removed pIOD2 parameter from function BuildAdapterBasedStorageDevs. 

01/31/2001 : 10:39 AM   NAR20038  Manjunath
Added ReadWriteMetaDataWithoutWait, Madhuresh's module for implementing
OTC meta data.

01/24/2001 : 02:22 PM   NAR20038  Manjunath
Removed #include "audit.h" and prototype GetSabreDataFromSD

12/13/2000 : 11:08 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Removed prototypes for functions in cache.c file
2. Removed prototypes for functions in sabre.c file
3. Removed prototypes for functions in r5ops.c file
4. Removed prototypes for functions in wrtmerge.c file

Added following prototypes:

IOCtlR10RecreateDevice
PostIOCtlR10RecreateDevice
IOCtlR10VerifyDevice
PostIOCtlR10VerifyDevice
IOCtlR10ChangeDrive
IVerifyCompareHostMemory

Removed following prototypes:

prototypes under #ifdef _DATACACHEON
prototypes under #ifdef _ATHENA
RAIDRecoveryAllocSabre
RAIDRecoveryClearSabre
IVerifyCheckHostMemoryZero
CheckHostMemoryZero
R1AllocSabre
R1BeginSetup
ControllerModePage1
ControllerModePage4
ControllerModePage6
IOCtlResyncPhysAddresses
RebaseSysRSU_IOD
RebaseHIMDataStruct
RebaseIODFields
RebaseOSDFields
RebaseCacheDescriptors
SabreRequestComplete
SabreReadRevision
SetCacheCapability
AllocateMoreCacheBlocks
AllocateLessCacheBlocks
PostSetCacheCapabilityDone
GetCacheCapabilityFromRSDone
SetCacheCapabilityFromRS
FlushForDisablingCacheDone
IsXORAdapter
InitDataCache
InitAllocateDataCacheBlocks
R1ResetInitialSabreRequest
R1MergeWriteDescriptors
R1CanProxyWrite
R1NextWD
R1FlushParityWriteQueues
R1PostDeadIO
BuildControllerBasedStorageDevs
CreateControllerStorageSet
CreateControllerStroageDev
SendInquiryToTestDisk
SendInquiryToTestDiskPresent
ConvertConf
ApplyLock
ReleaseRange
HashLookupOSD
OSInitTheController
InitPollController
DispathToAthena
ControllerInterruptHandler
ProcessCompleteRemPkts
AddRPToControllerQ
IOCtlControllerWrite
RemoteUpdateController
RemoteUpdateAll
RemoteUpdateNextController
BuildChildRPFromParentRP
RescanRemoteDevices
PostRescanRemoteDevices
RescanRemoteDevicesNextController
OSInquiryReadCapacityToRaidDrive
OSIssueToSabre
OSSabreClearComplete
OSSabreWriteComplete
OSInquiryReadCapacityToRaidDriveDone


***********************************************************************/

#ifndef COREPROT_INCLUDED
#define COREPROT_INCLUDED

#include "sysdef.h"

/***********************************************************************
 File cfginit.c
***********************************************************************/
ULONG				MDT_GetMetaDataSize() ;
VOID				SetDeviceEntryPoints (pPhysDevice pPD);
pPhysDevice	        AllocPhysDevice (pADAPTER_DEV_EXT pAdapterDevExt);
pSysDevice			AllocSysDevice (pADAPTER_DEV_EXT pAdapterDevExt, UCHAR AccessType);
USHORT				AllocRAIDResources( pSysDevice pSD );
pIODescriptor       CreateInitIOD (pADAPTER_DEV_EXT pAdapterDevExt);
VOID				FreeInitIOD (pIODescriptor pIOD);
pOSDescriptor       CreateInitOSD ();
VOID				FreeInitOSD (pOSDescriptor pOSD);
VOID				ClearInitOSDSemaphore (pOSDescriptor pOSD);
#ifndef SCO_DDI8
VOID				AwaitInitOSDSemaphore (pADAPTER_DEV_EXT pAdapterDevExt);

UINT32				BuildNonStorageDevs (pADAPTER_DEV_EXT pAdapterDevExt, pPCICfgHeader pConfigImage);
UINT32				BuildControllerDevs (pADAPTER_DEV_EXT pAdapterDevExt, pPCICfgHeader pConfigImage,UCHAR iTree, UCHAR LeafDepth[]);

UINT32				BuildAdapterDevs (pADAPTER_DEV_EXT pAdapterDevExt, pPCICfgHeader pConfigImage,UCHAR iTree, UCHAR LeafDepth[]);
#endif // SCO_DDI8
UINT32				BuildAdapterBasedStorageDevs (pSysDevice pAdapter, pOSDescriptor pOSD, pIODescriptor pIOD);

VOID				TestDevicePresent (pPhysDevice pNewPD, pIODescriptor pTestIOD);
VOID				TestInquiryComplete (pIODescriptor pTestIOD);
VOID				StartStopComplete (pIODescriptor pTestIOD);
VOID				TURComplete (pIODescriptor pTestIOD);
VOID				ReadCapComplete (pIODescriptor  pTestIOD);
VOID				ModeSenseForSmartComplete (pIODescriptor pTestIOD);
VOID				ModeSelectForSmartComplete (pIODescriptor pTestIOD);
VOID				ModeSenseForCacheComplete (pIODescriptor pTestIOD);
VOID				ModeSelectForCacheComplete (pIODescriptor pTestIOD);
VOID				ModeSenseErrRecoveryPageComplete(pIODescriptor pTestIOD);

VOID				AddToStorageDeviceList (pSysDevice pNewLD);
VOID				SetupPollIOD (pIODescriptor	  pTestIOD,UCHAR  cmd);
VOID				SetupToBuildLocalSysDev (pSysDevice	  pAdapter, pOSDescriptor pOSD, pIODescriptor pIOD);
VOID				MapToLocalSysDev (pADAPTER_DEV_EXT pAdapterDevExt, pIODescriptor pIOD, pPhysDevice pCrntPD);
pSysDevice			CreateAndLinkSysDevice(pADAPTER_DEV_EXT pAdapterDevExt);
VOID				LinkLocalSysDev (pSysDevice pCrntSD);
VOID				ReadBootSectorComplete (pIODescriptor   pTestIOD);
pSysDevice			ScanForTimestampMatch (pADAPTER_DEV_EXT pAdapterDevExt, ULONG Timestamp,ULONG ArrayID);
USHORT				FixupPhysDriveIndex (pADAPTER_DEV_EXT pAdapterDevExt);
ULONG				CreateR0OvrlaySDs(pSysDevice pMasterSD);
VOID				AssignFailedMembers(pSysDevice pSD);
USHORT				DetermineArrayStates(pIODescriptor pIOD, pOSDescriptor   pOSD);
VOID				CheckArrayMembersPosition(pSysDevice pSD, pIODescriptor pIOD, pOSDescriptor pOSD);

VOID				SetupPollAdapterIOD ( pIODescriptor pTestIOD, UCHAR cmd, UCHAR DPTR pseudoCDB, USHORT pseudoCDBlength );
VOID			    AdjustSpareDisks(pOSDescriptor	pOSD, pIODescriptor	pIOD) ;
pIODescriptor       Cfginit_CreateAndInitDDFIOD(pADAPTER_DEV_EXT pAdapterDevExt);

VOID                ModeSelectForITNexusLossTimeComplete (pIODescriptor   pTestIOD);
VOID                ModeSenseForITNexusLossTimeComplete (pIODescriptor   pTestIOD);

VOID                DetermineArraysKnownToOS(pADAPTER_DEV_EXT pAdapterDevExt);
VOID                LogEventsAfterDiscovery(pADAPTER_DEV_EXT pAdapterDevExt);

VOID                DoRecoverReservedSector(pADAPTER_DEV_EXT pAdapterDevExt,
                                                    pIODescriptor pIOD);
VOID                RescanZapPartitionSectorDone_InitializationTime(pIODescriptor   pIOD);
VOID				SortArraysbasedOnBIOSBootOrder(pADAPTER_DEV_EXT pAdptDevExt);

/***********************************************************************
 File hash.c
***********************************************************************/
USHORT				  HashInit (pSysDevice pParentSD);
pDHeader				HashLookup (pIODescriptor pIOD);

/***********************************************************************
 File himif.c
***********************************************************************/
VOID			IOSort (pIODescriptor pIOD, BOOL AllowProxy);
pIODescriptor   ProxyIO (pIODescriptor pProxyIOD, pIODescriptor pCandidateIOD);
VOID			PostProxyIO (pIODescriptor pIOD);
VOID			IOPull (pIODescriptor pIOD);
pIODescriptor   DispatchToHIM (pPhysDevice pPD);
VOID			SendToHIM (pIODescriptor pIOD);
VOID			IOFaultDevice ( pIODescriptor   pIOD,BOOL AllowProxy);

#define	AdapterInterruptHandler(pAD)									\
	if (pAD->AD_HIMFunctions.HIMFrontEndISR(pAD->AD_AdapterTSH) != 0)	\
	{																	\
		pAD->AD_HIMFunctions.HIMBackEndISR(pAD->AD_AdapterTSH);			\
		interruptAcknowledged = TRUE;									\
	}	


VOID			ReceiveFromHIM (pIODescriptor pIOD);
VOID			XlateIODToIOB (pIODescriptor pIOD);
pIODescriptor	SendIOD_UnfrzQ (pPhysDevice pPD);

VOID			ChkForIODTimeout (pSysDevice pController);
VOID			CdaTimer(pSysDevice pController);
VOID            CdaTimer_CheckIODTimeout(pADAPTER_DEV_EXT pAdapterDevExt);
VOID			HandleIOToResetFailure(pADAPTER_DEV_EXT pAdapterDevExt);

BOOL			SetupSCSIChannelReset(pAdapterDevice pAD);

VOID			RemoveIODFromIOQueue (pIODescriptor	  pIOD);
ULONG			OSInitTheHIM (pADAPTER_DEV_EXT pAdapterDevExt);
HIM_UINT32		OSMCompleteIOB(HIM_IOB HIM_PTR pIOB);
VOID			AbortIOD(pIODescriptor pIOD, pOSDescriptor, UCHAR);	/* TDE - added OSD and CustomFlag */
VOID			AbortCommandComplete(pIODescriptor pIOD);

VOID			QueryIOD(pIODescriptor pIOD);
VOID			QueryCommandComplete(pIODescriptor pIOD);

VOID			PACCompleteAfterReset (pSysDevice pSD);
VOID 			PostTestUnitReadyAfterResetComplete(pIODescriptor pTestIOD);
VOID			PostSendUnfrzQ(pIODescriptor pIOD);
VOID			UnlinkOFFLINESysDevices(pSysDevice pSD);
VOID			StartPendingIOsOnRemovedDevices(pSysDevice pSD);
VOID			PACCompleteAfterClearSCSIChannelReset(pOSDescriptor pOSD);
VOID			FreeOSDAfterPAC(pOSDescriptor pOSD);
VOID			SetupProxy(pIODescriptor pIOD);
VOID			MoveRescanToWaitingList(pIODescriptor pIOD);

UINT32          ResetLogicalUnit (
	                pPhysDevice pPD, 
	                pOSDescriptor,
	                UCHAR );		/* TDE - added OSD and CustomFlag */
VOID            ResetLogicalUnitComplete(
	                pIODescriptor pRetrnIOD);

VOID            ResetDevicePort (
	                pPhysDevice pPD, 
	                pOSDescriptor,
	                UCHAR );		/* TDE - added OSD and CustomFlag */

VOID            ResetDevicePortComplete(
	                pIODescriptor pResetIOD);
VOID            ResetSASHardware (
	                pAdapterDevice pAD );
VOID            ResetSASHardwareComplete(
	                pIODescriptor pResetIOD);
UCHAR           ResetAdapter (
	                pAdapterDevice pAD );
VOID            ResetAdapterComplete(
	                pIODescriptor pResetIOD);

UCHAR           ResetHardware (
	                pAdapterDevice pAD );
VOID            ResetHardwareComplete(
	                pIODescriptor pResetIOD);

VOID            RestartPendingIOs(
                    pPhysDevice         pPD,
                    pIODescriptor       pIOD);

VOID            TMFRequestComplete(
	                pIODescriptor pAbortIOD);


VOID            FailIO_TMFBusyLogicExceeds(
	                pSysDevice pSD,
	                pOSDescriptor pOSD);

VOID            SetIODTimeout(	
	                pIODescriptor pIOD,
	                pADAPTER_DEV_EXT pAdapterDevExt);

VOID            TimerCheckPendingRequest(pADAPTER_DEV_EXT pAdapterDevExt);
VOID            TimerResetDevicePort(pADAPTER_DEV_EXT pAdapterDevExt);
#if CDA_ENABLE_FTTT_SUPPORT
VOID            TimerValidateDevicePort(pADAPTER_DEV_EXT pAdapterDevExt);
#endif

VOID            TimerUpdateMetadata(pADAPTER_DEV_EXT pAdapterDevExt);

VOID            ValidateASasDevice(pADAPTER_DEV_EXT pAdapterDevExt, pSasDevice pSas);

HIM_UINT32 PostXCfgDiscoveryOnlyIF (HIM_IOB HIM_PTR pIOB);

VOID XCfgDiscoveryOnlyIF (
	pAdapterDevice  pAD,
	pIODescriptor   pIOD);

VOID PostAbortProbeLuOrDevicePortInitComplete(
	pIODescriptor pAbortIOD);

VOID PostResetDevicePortProbeLuOrDevicePortInitComplete(
    pIODescriptor pResetIOD);

VOID PostAbortForPACAtAdapterLevel(
	pIODescriptor pAbortIOD);

VOID PostAbortForPACAtPortLevel(
	pIODescriptor pAbortIOD);

/***********************************************************************
 File cdar1.c
***********************************************************************/
VOID					R1Write (pOSDescriptor) ;
VOID					R1Read (pOSDescriptor) ;
/***********************************************************************
 File cdar10.c
***********************************************************************/
VOID					R10Write (pOSDescriptor) ;
VOID					R10Read (pOSDescriptor) ;
VOID                    PostR10Write(pIODescriptor);

/***********************************************************************
 File raidsubr.c
***********************************************************************/
BOOL					FatalDiskError(pIODescriptor pIOD);
USHORT				  	MostSevereError(UCHAR DPTR pCurrentStatus, UCHAR UpdateStatus,
						pSCSISenseData pCurrentSense, pSCSISenseData pUpdateSense);
VOID					SetPostFunc (pIODescriptor pIOD, FPTR PostFunc);
VOID 					writeCompleteForMirror (pIODescriptor		pIOD);
VOID					SetupIOD (pIODescriptor pCrntIOD, USHORT priority,
						UCHAR XferDirection, ULONG LBA, USHORT cXfer);
VOID					ComposeCDB10 (pSCSICDB10 pCDB,UCHAR XferDirection,
						ULONG LBA,USHORT cXfer);
VOID					PlaceOSDWaiting (pControllerResourceDescriptor pCRD,
						pOSDescriptor pAddOSD);
VOID					RetryOSDDispatch();
VOID					PlaceRemPktWaiting (pControllerResourceDescriptor pCRD,
						pRemoteCtlPkt pAddRemPkt);
VOID					R0ReadWrite (pOSDescriptor pOSD);
VOID					ProcessRW (pOSDescriptor pOSD, pIODescriptor pIODChain);
VOID					PostGeneralRW (pIODescriptor pIOD);
VOID					NoOpRecovery (pIODescriptor pIOD);
VOID					NoOpErrHandle (pIODescriptor pIOD);
VOID					NoOpRecoveryChangeState(pIODescriptor pIOD);
VOID					SingleRW (pOSDescriptor pOSD);
VOID					SingleProcessRW (pOSDescriptor pOSD, pIODescriptor pIOD);
VOID					PostSingleRW (pIODescriptor	 pIOD);
VOID					OverlaySGSetup (pOSDescriptor pSrcOSD, pOSDescriptor pDstOSD,
						ULONG XferOffset,ULONG cXferBytes);
VOID					OtherRW (pOSDescriptor pOSD);
VOID					OtherProcessRW (pOSDescriptor pOSD,	  pIODescriptor pIOD);
VOID					PostOtherRW (pIODescriptor pIOD);
VOID					PauseRD (pRecoveryDescriptor pAddRD, pSysDevice pSD);
VOID					ResumeRD (pSysDevice pSD);
/* Manju */
VOID 					PlaceIODWaiting (pControllerResourceDescriptor pCRD, pIODescriptor pAddIOD);
VOID 					RetryIODDispatch( pControllerResourceDescriptor pCRD);
#ifdef SCO_DDI8
BOOL                    _LockpConfigBuffer(pSysDevice);
#define LockpConfigBuffer(a,b) _LockpConfigBuffer(a)
#else // SCO_DDI8
BOOL 					LockpConfigBuffer(pSysDevice pSD, FPTR	PostFunc);
#endif // SCO_DDI8
VOID 					ReleasepConfigBuffer(pSysDevice pSD);

                  pIODescriptor AllocateIODsNeeded(
                            pOSDescriptor   pOSD,
                            ULONG           totalTransactionBytes, 
                            ULONG           *stripeUnitSectors,
                            USHORT          *startIndex, 
                            ULONG           *stripeXfer,
                            ULONG           *startLBAs );

                  VOID BuildIODSGList(
                            pOSDescriptor    pOSD, 
                            pIODescriptor   pHeadIOD,
                            ULONG           totalTransactionBytes,  
                            ULONG           stripeUnitSectors,
                            ULONG           stripeXfer,
                            USHORT          mirrorFlag);

USHORT                  IOAddrBind ( pIODescriptor	pHeadIOD);

VOID                    InitializeIOD (
                            pIODescriptor   pHeadIOD,
                            pOSDescriptor   pOSD,
                            pPhysDevice	    pPhysParent,
                            USHORT          priority,
                            UCHAR           XferDirection,
                            ULONG           LBA,
		                    ULONG			absoluteLBA,
                            FPTR            PostFunc
                            );



/***********************************************************************
 File recover.c
***********************************************************************/
USHORT				  CreateRecoveryDescriptors (pSysDevice pParentSD);
USHORT				  CreateRDWaitDescriptors (pSysDevice pParentSD);
VOID					PlaceRDWaiting (pControllerResourceDescriptor pCRD,
						pIODescriptor pAddIOD);
VOID					RetryRDDispatch();
VOID					RetryRDDispatchToStartIOCtlOp (pControllerResourceDescriptor pCRD);
VOID					RAIDRecoveryBeforeSubmission (pIODescriptor pFailedIOD);
VOID					RAIDRecoveryAfterCompletion (pIODescriptor pFailedIOD);
VOID					RAIDRecovery (pIODescriptor pFailedIOD);
VOID					ProcessRAIDRecovery (pRecoveryDescriptor pRD);
USHORT				  RAIDRecoveryIssueRequest (pIODescriptor pCrntIOD, USHORT nextPhysDevice);
VOID					RAIDRecoveryStartRead (pIODescriptor pCrntIOD,
						pRecoveryDescriptor pRD, pSysDevice pSD, USHORT iMember);
VOID					RAIDRecoveryReadDone (pIODescriptor pIOD);
VOID					RAIDRecoveryReadTopHalf(pIODescriptor pCrntIOD,
						pRecoveryDescriptor pRD, pSysDevice pSD, USHORT iMember);
VOID					RAIDRecoveryReadTopHalfDone (pIODescriptor pIOD);
VOID					RAIDRecoveryVerifyReadDone (pIODescriptor pVerifyIOD);
VOID					RAIDRecoveryStartReassignment(pIODescriptor pIOD,
						pRecoveryDescriptor pRD, pSysDevice pSD);
VOID					RAIDRecoveryReassignmentDone(pIODescriptor pIOD);
VOID					RAIDRecoveryProcessIODFailedStatus(pIODescriptor pIOD);
VOID					RAIDRecoveryTryWriteVerify(pRecoveryDescriptor pRD, FPTR PostFunc);
VOID 					RAIDRecoveryTryWriteVerifyDone(pIODescriptor pIOD);
pSysDevice				RAIDRecoveryFindParentSDForPD(pSysDevice pMasterSD, pPhysDevice pPD);
VOID					RAIDRecoveryIncrementTimeStamp(pSysDevice pSD);
BOOL					SetCDAArrayStateOnDriveFailure(pIODescriptor pIOD);
BOOL 					RAIDRecoveryCompOSReqErr(pIODescriptor pIOD);
BOOL					RAIDRecoveryAbortArrayOp(pIODescriptor pIOD, pSysDevice pSD, 
											pPhysDevice pFailedPD, BOOL BkGroundOp);
BOOL					RAIDRecoveryRedirectOSRead(pIODescriptor pIOD);
BOOL					ChkMetaDataIO(pIODescriptor pIOD);
VOID					RAIDCheckDataRecoveryDone (pIODescriptor pIOD);
VOID					RAIDCheckRecoveryFunctionDone (pRecoveryDescriptor pRD);
VOID					RAIDRetryRecovery (pRecoveryDescriptor   pRD);
VOID					RAIDReleaseReadIODs (pRecoveryDescriptor		 pRD);
VOID					RAIDRecoveryUpdateReassignedBlock (pRecoveryDescriptor pRD);
VOID					RAIDRecoveryUpdateBlockDone (pIODescriptor pIOD);
VOID					RAIDRecoveryDone (pRecoveryDescriptor pRD);
VOID 					RAIDRecoveryReleaseRD(pRecoveryDescriptor pRD);
VOID					DeferredErrorRecoveryComplete (pIODescriptor pIOD);
VOID					LogDeviceEvent (pIODescriptor pIOD, UCHAR Qualifier,
						ULONG LBA,UCHAR Key,UCHAR Code);
VOID					ReadMemberReservedSector(pIODescriptor pIOD);
VOID					FlushSysDevIO(pSysDevice pSD);
VOID					ChangeArrayStateOnDriveFailure(pIODescriptor pIOD);
VOID					HandleIOToFailedDrive (pIODescriptor pIOD);
VOID 					RAIDRecoveryBlockReassignmentReadDone(pIODescriptor pIOD);
VOID 					FlushIOsOnPD(pPhysDevice pPD);
VOID                    FlushRSWaitQueue(pSysDevice pSD);
VOID					RAIDRecoveryReadWriteMDT(pIODescriptor pIOD, 
							FPTR PostFunc, UCHAR  ReadWrite, USHORT iMember);
VOID					RAIDRecoveryWriteMDTDone(pIODescriptor pIOD);
VOID					RAIDRecoveryReadMDTDone(pIODescriptor pIOD);
VOID					RAIDRecoverySyncMDTReadComplete(pIODescriptor pIOD);
VOID					RAIDRecoverySyncMDTWriteComplete(pIODescriptor pIOD);
BOOL 					RAIDRecoveryFindNextMemIdx(pIODescriptor pFailedIOD, pIODescriptor pRecoveryIOD, USHORT *pMember, UCHAR ReadWrite);
USHORT					FindMemId_IOD(pIODescriptor pIOD);
USHORT					FindMemId_PhysDev(pPhysDevice pPD);
VOID					RAIDRecoveryModifyFreezeFlags(pIODescriptor pIOD, UCHAR CDB_Opcode, BOOL SetFlags);
VOID					RAIDRecoveryScheduleSyncMDT(pIODescriptor pIOD);
BOOL					RAIDRecoveryIterateRWMDT(pIODescriptor pIOD, FPTR PostFunc, UCHAR  ReadWrite);
VOID					RAIDRecoveryIterateReadMDTDone(pIODescriptor pIOD);
VOID					RAIDRecoveryIterateWriteMDTDone(pIODescriptor pIOD);
VOID					RAIDRecoveryFailAllMembers(pIODescriptor pIOD);
USHORT					RAIDRecoveryFindMemIdR1Mirror(pIODescriptor pFailedIOD);
VOID					DumpArrayPDInfo(pSysDevice pSD);

/***********************************************************************
 File remoteif.c
***********************************************************************/

BOOL					CreateRemPkt(pSysDevice pParentSD);
pRemoteCtlPkt   AllocRemPkt (pControllerResourceDescriptor pCRD);
VOID					InitRemPkt (pRemoteCtlPkt pRemPkt);
VOID					FreeRemPkt (pControllerResourceDescriptor pCRD, pRemoteCtlPkt pRemPkt);
VOID					RetryRemPktDispatch (pControllerResourceDescriptor pCRD);
VOID					DispatchToLocal (pRemoteCtlPkt pRP);

VOID					CopyTreeAddress (pLDTree pSrcTree, pLDTree pDstTree);
VOID					CopySGTable (pSGTable  pSrcSG, pSGTable pDstSG);
VOID					RemoteRW (pOSDescriptor pOSDReq);
VOID					RemoteMgt( pRemoteCtlPkt  pRP);
VOID					RemoveRPFromControllerQ(pControllerDevice pCrntCD, pRemoteCtlPkt pCrntRP);
VOID					AbortRemPktsForReset(pControllerResourceDescriptor pCRD);

/***********************************************************************
 File resource.c
***********************************************************************/
USHORT				CreateIODescriptors (pSysDevice pParentSD);
VOID                Res_InitAndRebuildIODLink(
	                        pADAPTER_DEV_EXT	pAdapterDevExt,
                            pIODescriptor       pHead,
	                        ULONG				cStructures,
                            USHORT              sgMax);
VOID                Res_RebuildRSUBufferLink(
                            pRSUBuffer      pHead,
	                        ULONG			cStructures);
VOID                Res_InitIOD(
	                        pADAPTER_DEV_EXT	pAdapterDevExt,
                            pIODescriptor       pIOD,
                            USHORT              sgMax);

UCHAR DPTR			AllocStruc (pADAPTER_DEV_EXT pAdapterDevExt, ULONG cStructures, ULONG szStructure);
VOID				InitIOB (pADAPTER_DEV_EXT pAdapterDevExt, HIM_IOB HIM_PTR pIOB);
VOID                InitSpecialIOB (
	                        pADAPTER_DEV_EXT pAdapterDevExt,
	                        HIM_IOB HIM_PTR pIOB,
                            UINT32  function);
VOID              CreateDMASenseBuffers (pADAPTER_DEV_EXT pAdapterDevExt);
USHORT            AssignDMASenseBuffer(pADAPTER_DEV_EXT pAdapterDevExt, pPhysDevice pPD);
VOID              FreeDMASenseBuffer (pADAPTER_DEV_EXT pAdapterDevExt, pPhysDevice pPD);
USHORT              CreateRSUResources (pSysDevice pParentSD);
pIODescriptor       AllocRSUIOD (pADAPTER_DEV_EXT pAdapterDevExt);
pReservedSectorLayout   AllocRSURSBuffer (pADAPTER_DEV_EXT pAdapterDevExt);
VOID				FreeRSUResources (pSysDevice pSD);

pIODescriptor   	AllocIODs (pControllerResourceDescriptor pCRD, USHORT cIODs);
VOID				InitIOD (pIODescriptor pIOD);
ULONG				CheckForSufficientSysDevices
						(pControllerResourceDescriptor pCRD, ULONG MinNumSDs);
pSysDevice			GetFreeSysDevice (pControllerResourceDescriptor pCRD,
						pSysDevice pSD);
pPhysDevice			GetFreePhysDevice (pControllerResourceDescriptor pCRD,
						pSysDevice pParentSD);
PVOID				GetFreeTSCB (pAdapterDevice pAD,
								USHORT TSCB_RESOURCE);

VOID				FreeSysDevice (pControllerResourceDescriptor pCRD,
						pSysDevice pSD);
VOID				UnlinkSysDevice(pSysDevice pSD);
VOID				FreePhysDevice (pControllerResourceDescriptor pCRD,
						pPhysDevice pPD);
VOID				FreeTSCB (pAdapterDevice pAD, 
						PVOID pTSCB,
						USHORT TSCB_RESOURCE);

USHORT				CreateOSDescriptors (pSysDevice pParentSD);
pOSDescriptor   	AllocOSDs (pControllerResourceDescriptor pCRD, USHORT cOSDs);
VOID				InitOSD (pOSDescriptor pOSD);

#ifdef _NOMACRO

VOID				FreeIOD (pControllerResourceDescriptor pCRD, pIODescriptor pIOD);
VOID				FreeOSD (pControllerResourceDescriptor pCRD, pOSDescriptor pOSD);

#else

#define	FreeIOD(pCRD,pIOD)                                               \
{                                                                        \
	pControllerResourceDescriptor   pIODResource = pCRD + IOD_RESOURCE;  \
	pIOD->pIOD_Next = pIODResource->pFreeListHead;                       \
	pIODResource->pFreeListHead = pIOD;                                  \
	pIODResource->FreeDescriptorCount++;                                 \
}

#define	FreeOSD(pCRD,pOSD)                                              \
{                                                                       \
	pControllerResourceDescriptor pOSDResource = pCRD + OSD_RESOURCE;   \
	pOSD->pOSD_Next = (pOSDescriptor) pOSDResource->pFreeListHead;      \
	pOSDResource->pFreeListHead = (PVOID) pOSD;                         \
	pOSDResource->FreeDescriptorCount++;                                \
}
#endif



VOID				InitDHeader(pDHeader pDH);

pRecoveryDescriptor AllocRDs (pADAPTER_DEV_EXT pAdapterDevExt, pControllerResourceDescriptor pCRD, USHORT cRDs);
VOID				InitRD (pADAPTER_DEV_EXT pAdapterDevExt, pRecoveryDescriptor pRD);
VOID				FreeRD (pControllerResourceDescriptor pCRD, pRecoveryDescriptor pRD);
pRDWaitDescriptor   AllocWaitRDs (pControllerResourceDescriptor pCRD, USHORT cRDs);
VOID				FreeWaitRD (pControllerResourceDescriptor pCRD, pRDWaitDescriptor pWaitRD);

/* Manju - Added AllocSpecialIODs & FreeSpecialIODs */
USHORT				CreateSpecialIODescriptors (pSysDevice pParentSD);
pIODescriptor		AllocSpecialIODs (pControllerResourceDescriptor pCRD, USHORT cIODs);
VOID				FreeSpecialIOD (pControllerResourceDescriptor pCRD, pIODescriptor pIOD);

/***********************************************************************
 File sysinit.c
***********************************************************************/

UINT32				SysInit(pADAPTER_DEV_EXT pAdapterDevExt);
/*USHORT			OSInitTheAHardware(); */
#ifdef CDA_FOR_SATA
BOOL                ValidateCookie(pADAPTER_DEV_EXT pAdapterDevExt);
VOID                ValidateCookieFeatures(pADAPTER_DEV_EXT pAdapterDevExt);
#endif /* CDA_FOR_SATA */
VOID				ReturnSystemResources();
VOID				SetEntryPoints (pSysDevice pSD);
BOOL				GetMediumErrorRecoveryPath(pADAPTER_DEV_EXT pAdapterDevExt);

/***********************************************************************
 File gensubr.c
***********************************************************************/
pSysDevice		FindParentSysDeviceFromLDTree(pSysDevice pChildSD);
pSysDevice		FindSysDeviceFromLDTree(pLDTree pTarget,USHORT DPTR pDepth, pADAPTER_DEV_EXT pAdapterDevExt);
USHORT			DevSingleCmdSubmit(pOSDescriptor pOSD, pPhysDevice pPD, FPTR PostFunc);
VOID			PostDevSingleCmdSubmit(pIODescriptor pIOD);
USHORT			DevRaidCmdSubmit(pOSDescriptor pOSD, FPTR PostFunc);
USHORT			IssueStartStopUnitToPairs(pOSDescriptor   pChildOSD);
VOID			PostIssueStartStopUnitToPairs(pIODescriptor   pIOD);
VOID			PostDevRaidCmdSubmit(pIODescriptor pIOD);
VOID			RetrieveOSDFromRemPkt(pRemoteCtlPkt pRemPkt);
VOID			RetrieveRemPktFromOSD(pOSDescriptor pOSD);
VOID			InvalidIOCtl(pRemoteCtlPkt pRP, UCHAR Key, UCHAR Code);
VOID			InvalidLDTree(PVOID pPacket, UCHAR PacketType);
VOID			NoOpRead(pOSDescriptor pOSD);
VOID			NoOpWrite(pOSDescriptor pOSD);
VOID			NoOpIO(pOSDescriptor pOSD);
VOID			NoOpMgt(pRemoteCtlPkt pDrvPkt);
VOID			RetrieveParentOSDFromChildOSD(pOSDescriptor pOSD);
VOID			IOCtlRetry(pOSDescriptor pOSD, FPTR RetryFunc);
pSysDevice		FindSysDeviceFromStorageList(pADAPTER_DEV_EXT pAdapterDevExt, pLDTree pPath);
UCHAR			FindIndexInArraySD(pPhysDevice pMember, pSysDevice pArraySD);
VOID            ConvertHexToString(UCHAR DPTR Source,UCHAR DPTR Destination,USHORT SrcLength,USHORT DestLength);
VOID            ConstructAbortedStatus (pIODescriptor pIOD);

/***********************************************************************
 File osioctl.c
***********************************************************************/

VOID	OSLRaidIOCtl(pOSDescriptor pOSD);
VOID	OSLSingleIOCtl(pOSDescriptor pOSD);
VOID	OSLOtherIOCtl(pOSDescriptor	  pOSD);
VOID	OSRemoteIOCtl(pOSDescriptor pOSD);
VOID	OSInvalidIOCtl(pOSDescriptor pOSD);

/***********************************************************************
 File sioctl.c
***********************************************************************/

VOID			SingleMgt(pRemoteCtlPkt pRemPkt);
VOID			IOCtlConvertSingleToSpare(pRemoteCtlPkt pRemPkt);
VOID			IOCtlSingleToDevice(pRemoteCtlPkt pRemPkt);
VOID			IOCtlSingleDeviceBlockUnblockIO(pRemoteCtlPkt pRP);
VOID			SingleDevicePauseDone(pIODescriptor pIOD);
VOID			SingleDeviceUnPauseIO(pOSDescriptor pOSD);
VOID			SingleDeviceUnPauseIORetry(pRemoteCtlPkt pRP);
VOID			SingleDeviceUnPauseDone(pIODescriptor pIOD);
VOID			IOCtlConfigureSingleDrive(pRemoteCtlPkt pRemPkt);
VOID            RetryConfigureSingleDriveReadMetadata(pIODescriptor pIOD);

/***********************************************************************
 File oioctl.c
***********************************************************************/

VOID			OtherMgt(pRemoteCtlPkt pRemPkt);
VOID			IOCtlOtherToDevice(pRemoteCtlPkt pRemPkt);


/***********************************************************************
 File caioctl.c
***********************************************************************/
VOID			ControllerMgt (pRemoteCtlPkt pRP);
VOID			IOCtlControllerGetLog (pRemoteCtlPkt pRP);
VOID			LogIODError(pIODescriptor pIOD, UCHAR EventQual);
VOID			LogEventDuringInit (pADAPTER_DEV_EXT pAdapterDevExt, pLDTree pDevPath, UCHAR Status, UCHAR EventQual,
					SCSISenseData DPTR SenseData, UCHAR Key, UCHAR Code);
VOID			LogEvent (pADAPTER_DEV_EXT pAdapterDevExt, pLDTree pDevPath, UCHAR Status, UCHAR EventQual,
					SCSISenseData DPTR SenseData, UCHAR Key, UCHAR Code);
VOID			LogEventNoLDTree(pADAPTER_DEV_EXT pAdapterDevExt, UCHAR PCIBus, UCHAR Channel, UCHAR Id,
					UCHAR Lun, UCHAR Key, UCHAR Code);



VOID LogEventSasInfo(
	pADAPTER_DEV_EXT	pAdapterDevExt,
	UCHAR				PortNumber,
	UCHAR				PortState,
	USHORT				Sas_Flags,
	UCHAR	DPTR			WWN,
	UCHAR				Key,
	UCHAR				Code);

VOID			IOCtlControllerInquiry (pRemoteCtlPkt pRP);
VOID			IOCtlControllerModeSense (pRemoteCtlPkt pRP);
VOID			IOCtlControllerModeSelect (pRemoteCtlPkt pRP);
VOID			ControllerModePage0 (pRemoteCtlPkt pRP);
VOID			ControllerModePage2 (pRemoteCtlPkt pRP);
VOID			ControllerModePage3 (pRemoteCtlPkt pRP);
VOID			ControllerModePage5 (pRemoteCtlPkt pRP);
VOID			ControllerModePage8 (pRemoteCtlPkt pRP);
VOID			ControllerModePage9 (pRemoteCtlPkt pRP);
VOID			ControllerModePage10 (pRemoteCtlPkt pRP);
VOID			ControllerModePage11 (pRemoteCtlPkt pRP);
VOID			IOCtlControllerPauseUnpauseIO (pRemoteCtlPkt pRP);
VOID			PostIOCtlControllerPauseUnpauseIO(pOSDescriptor pOSD);
USHORT  		AdapterPauseUnpauseIO(pOSDescriptor pHostOSD,pAdapterDevice pAD);
VOID			PostCheckCDPause(pOSDescriptor pOSD);
UCHAR			FindNumberOfArrayOpActive(pAdapterDevice pAD, pSysDevice SDTable[MAX_PER_BUS], UCHAR DPTR index);
VOID			IOCtlControllerStartStopIO (pRemoteCtlPkt pRP);

VOID			IOCtlRescan(pRemoteCtlPkt pRP);
VOID			RebaseWDFields(pControllerResourceDescriptor pCRD);
VOID			RebaseRDFields(pControllerResourceDescriptor pCRD);
VOID			ReloadGlobalVars(pSysDevice pParentSD);

VOID			AdapterMgt (pRemoteCtlPkt pRP);
VOID			PhysDeviceIOCtl (pRemoteCtlPkt pRP, pSysDevice pSD);
VOID			PhysDeviceToSubmit (pRemoteCtlPkt pRP, pSysDevice pSD);
VOID			IOCtlAdapterInquiry (pRemoteCtlPkt pRP);
VOID			IOCtlAdapterModeSense (pRemoteCtlPkt pRP);
VOID			IOCtlAdapterModeSelect (pRemoteCtlPkt pRP);
VOID			IOCtlAdapterPauseUnpauseIO (pRemoteCtlPkt pRP);

VOID			IOCtlWaitChange(pRemoteCtlPkt pRP);
VOID			IOCtlPostChange(pADAPTER_DEV_EXT pAdapterDevExt);
VOID			QueueWaitChange(pADAPTER_DEV_EXT pAdapterDevExt);
VOID			ReceiveChange(pRemoteCtlPkt	  pRP);
VOID			ReceiveDeviceChanges (pRemoteCtlPkt	  pRP);
VOID		   CopyRelativePath( pLDTree pSrcTree, pLDTree pDstTree);

VOID			PACCompleteAfterPause(pOSDescriptor pOSD);
ULONG       getMaxAdapterSpeed(pAdapterDevice pAD);
VOID			CopyEventsLog(pRemoteCtlPkt pRP);

/***********************************************************************
 File stioctl.c
***********************************************************************/

VOID UpdatePhysDeviceStatistics( pIODescriptor  pIOD);


VOID IOCtlGetIOStatistics(pRemoteCtlPkt pRP);
VOID IOCtlResetStatistics(pSysDevice pSD);

VOID IOCtlSingleGetStatistics(pRemoteCtlPkt pRemPkt);
VOID IOCtlControllerGetStatistics (pRemoteCtlPkt pRP);
VOID PhysDeviceGetStatistics (pRemoteCtlPkt pRP, pSysDevice pSD);
VOID IOCtlAdapterGetStatistics (pRemoteCtlPkt pRP);
VOID IOCtlLRAIDGetStatistics (pRemoteCtlPkt pRP);
UCHAR AccumSysDevSttt ( pSysDeviceStats pSysDevSttt,
				pAdapterDevice  pAdptrDev);

/***********************************************************************
 File spioctl.c
***********************************************************************/
VOID			SpareMgt(pRemoteCtlPkt pRP);
VOID			IOCtlConvertSpareToSingle (pRemoteCtlPkt pRP);
VOID			IOCtlDefineArray (pRemoteCtlPkt pRP);

#ifdef SES2_DEVICE
/***********************************************************************
 File ses2ioctl.c
***********************************************************************/
VOID			SES2DeviceMgt(pRemoteCtlPkt pRP);
#endif

/***********************************************************************
 File r0ioctl.c
***********************************************************************/
VOID			IOCtlRaidSetPriority (pRemoteCtlPkt  pRP);
VOID			R0Mgt (pRemoteCtlPkt pRP);
VOID			IOCtlRaidInquiry(pRemoteCtlPkt pRP);
VOID			IOCtlRaidReadCapacity(pRemoteCtlPkt pRP);
VOID			IOCtlRaidBlockUnblockIO(pRemoteCtlPkt	 pRP);
VOID			IOCtlRaidModeSense (pRemoteCtlPkt pRP);
VOID			StorageDevicePage1(pRemoteCtlPkt pRP);
VOID			StorageDevicePage1ReadRSDone(pIODescriptor pIOD);
VOID			StorageDevFuncPage0(pRemoteCtlPkt pRP);
VOID			IOCtlRaidModeSelect (pRemoteCtlPkt pRP);
VOID			IOCtlRaidFormat (pRemoteCtlPkt pRP);
VOID			IOCtlRaidStartStop (pRemoteCtlPkt pRP);
VOID			IOCtlReactivateArray(pRemoteCtlPkt pRP);
VOID			PostIOCtlReactivateArray(pOSDescriptor pOSD);
VOID			IOCtlRaidFormatComplete(pIODescriptor pIOD);
VOID			CopyReadCapFromSysDev (pSysDevice pCrntSD, UCHAR DPTR pBuf);
VOID			CopyInquiryFromSysDev (pSysDevice pCrntSD, UCHAR DPTR pBuf);
VOID			AssignNextLogicalAddress (pSysDevice pCrntSD, pSysDevice pParentSD);
VOID			PostIOCtlReactivateArray (pOSDescriptor pOSD);
VOID			SetReactivateStateForR1 (pSysDevice pCrntSD);
BOOL			CheckAllMembersPresent (pSysDevice pCrntSD);
VOID 			PostReactivateSimpleVolume (pIODescriptor pIOD);
VOID 			ReactivateSimpleVolume (pRemoteCtlPkt pRP);
VOID			ReadSimpleVolumeMetadata(pIODescriptor pIOD);
VOID			ReadSimpleVolumeMetadataComplete(pIODescriptor pIOD);
USHORT		FindNoOfMembersPresent (pSysDevice pCrntSD);

/***********************************************************************
 File r1ioctl.c
***********************************************************************/
VOID R1Mgt(pRemoteCtlPkt pRP);

/***********************************************************************
 File r5ioctl.c
***********************************************************************/
VOID R5Mgt (pRemoteCtlPkt pRP);
VOID IOCtlR1andR5BlockUnblockIO(pRemoteCtlPkt pRP);
VOID IOCtlConvertArray(pRemoteCtlPkt pRP);
VOID IOCtlChangeDrive(pRemoteCtlPkt pRP);
VOID IOCtlDeleteArray(pRemoteCtlPkt pRP);
VOID IOCtlAbortArrayMgtOp(pRemoteCtlPkt pRP);
VOID IOCtlRedundancy(pRemoteCtlPkt pRP);
VOID PostIOCtlRedundancy(pOSDescriptor pOSD);
VOID IOCtlZeroCommand(pRemoteCtlPkt pRP);
VOID IZeroRSUpdateDone(pOSDescriptor pOSD);
VOID PostIOCtlZeroCommand(pOSDescriptor pOSD);
VOID RetryLocalReadMemberReservedSector(pRemoteCtlPkt pRP);
VOID PostRetryLocalReadMemberReservedSector(pRemoteCtlPkt pRP);
VOID IZeroArrayStateRSUpdateDone(pOSDescriptor pOSD); //TDE
VOID IOCtlUpdateDeviceStatus(pRemoteCtlPkt pRP);
VOID IZeroRSUpdateDone(pOSDescriptor pOSD);
VOID IOCtlCompleteDownADrive (pIODescriptor pIOD);
VOID ContinueIOCtlCompleteDownADrive (pRemoteCtlPkt pRP);

VOID IOCtlDriveCacheSettings(pRemoteCtlPkt pRP);			/* TDE - merged drive cache settings */
VOID StartDriveCacheSetting(pIODescriptor pIOD);            /* jz, 5/02/05 */
VOID GetCurrentCacheSettingsReadDone(pIODescriptor pIOD);
VOID IOCtlDriveCacheSettingsDone(pIODescriptor pIOD);
//
// @kenc: Added Support for MetaData Updates after User changes
// Write Cache Settings using ASM
// 
VOID UpdateWCEMetaData(PVOID pRSIOD);
VOID WCEMetaDataReadComplete(PVOID pRSIOD);
VOID WCEMetaDataWriteToNextMember(PVOID pRSIOD);
VOID WCEMetaDataWriteComplete(PVOID pRSIOD);

//
//@Maram : Added Support for Changing the Bootflag of a particular SD in metadata
//
VOID IOCtlSetBootFlag(pRemoteCtlPkt pRP);
VOID PostIOCtlSetBootFlag(pOSDescriptor pOSD);
VOID UpdateMetaDataForBootFlag(pOSDescriptor pOSD);
VOID ReadMetaDataForSV(pIODescriptor pIOD);
VOID WriteMetaDataForSV(pIODescriptor pIOD);
VOID WriteMetaDataForSVComplete(pIODescriptor pIOD);
VOID UpdateNvramEntry(pRemoteCtlPkt pRP, pOSDescriptor pOSD);


VOID PostIOCtlFlushCacheBeforeArrayDeletion (pOSDescriptor pOSD);
VOID IOCtlFlushCacheBeforeArrayDeletion( pRemoteCtlPkt pRP);
/***********************************************************************
 File cdatasks.c
***********************************************************************/
USHORT		  IOCtlAllocateRangeResources (pOSDescriptor pOSD, FPTR pPostFunc,
					USHORT cIODs, BOOL bFailedIODNeeded);
VOID			IOCtlAllocSabre(pRecoveryDescriptor pRD, ULONG cSectors, FPTR pNextFunction);
ULONG                   IVerifyInternal(pSysDevice pSD, UCHAR code,UCHAR verifyType);
VOID			IVerifyCommand (pOSDescriptor   pOSD);
VOID			IVerifySetup (pOSDescriptor  pOSD, pRecoveryDescriptor pRD);
VOID			IVerifyGainRange (pRecoveryDescriptor pRD);
VOID			IVerifyGainCache (pRecoveryDescriptor pRD);
VOID			IVerifySubmitVerifyRead (pIODescriptor pIOD);
VOID			IVerifyStartRecovery(pIODescriptor pIOD);
VOID			IVerifyRangeDone (pRecoveryDescriptor pRD);
VOID			IVerifySubmitParityXOR (pIODescriptor pIOD);

VOID			IVerifyCompareHostMemory (pRecoveryDescriptor pRD);

VOID			IVerifyParityXORDone (pRecoveryDescriptor pRD);
VOID			IVerifySabreImage (pRecoveryDescriptor pRD);
VOID			IVerifySabreImageDone (pRecoveryDescriptor pRD);
VOID			IVerifyCorrectParity (pRecoveryDescriptor pRD);
VOID			IVerifyUpdateImages (pRecoveryDescriptor pRD);
VOID			IVerifyReportDiskUpdate (pIODescriptor pIOD);
VOID			IVerifyToNextRange (pRecoveryDescriptor pRD);
VOID			IRebuildCommand (pOSDescriptor pOSD);
VOID			IRebuildSetup (pOSDescriptor  pOSD,
						pRecoveryDescriptor pRD);
VOID			IRebuildGainRange (pRecoveryDescriptor pRD);
VOID			IRebuildGainCache (pRecoveryDescriptor pRD);
VOID			IRebuildSetForSabre (pIODescriptor pIOD);
VOID			IRebuildSubmitSabre (pRecoveryDescriptor pRD);
VOID			IRebuildRangeDone (pRecoveryDescriptor pRD);
VOID			IRebuildUpdateImages (pRecoveryDescriptor pRD);
VOID			IRebuildReportDiskUpdate (pIODescriptor pIOD);
VOID			IRebuildToNextRange (pRecoveryDescriptor pRD);
ULONG                   IZeroInternal(pSysDevice pSD, UCHAR code);
VOID			IZeroCommand (pOSDescriptor pOSD);
VOID			IZeroSetup (pOSDescriptor pOSD,pRecoveryDescriptor pRD);
VOID			IZeroClearSabre (pRecoveryDescriptor   pRD);
VOID			IZeroGainRange (pRecoveryDescriptor pRD);
VOID			IZeroProcessRange (pRecoveryDescriptor pRD);
VOID			IZeroStartWrite (pRecoveryDescriptor pRD,pIODescriptor pCrntIOD,
							 pPhysDevice pCrntPhysDev);
VOID			IZeroWriteDone (pIODescriptor pIOD);
VOID			ICheckZeroRangeDone (pIODescriptor pIOD);
VOID			IZeroToNextRange (pRecoveryDescriptor pRD);
BOOL 			CheckPointArrayOp(pIODescriptor pIOD);
VOID 			UpdateCheckPoint(pRecoveryDescriptor pRD, pPhysDevice pReadMdtPD);
VOID            RetryUpdateCheckPoint(pSysDevice pSD);
VOID 			CheckPointReadComplete(pIODescriptor pRSIOD);
VOID 			CheckPointWriteToNextMember(pIODescriptor pRSIOD);
VOID 			CheckPointWriteComplete(pIODescriptor pRSIOD);
VOID			ArrayOpCompleted(pRecoveryDescriptor pRD);

BOOL            CheckRAIDBuffersAvailable(pADAPTER_DEV_EXT pAdapterDevExt, 
                               pSysDevice pSD, 
                               USHORT OperationType);

VOID            StartOverlappedIOsOnPD(pSysDevice pSD);

VOID RAIDRecoveryForceMediumError(pIODescriptor pIOD, pRecoveryDescriptor pRD,
				  pSysDevice pSD);
VOID RAIDRecoveryForceMediumErrorDone(pIODescriptor pIOD);
VOID RAIDRecoveryForceMediumErrorReadLong(pIODescriptor pIOD,
					  pIODescriptor pFailedIOD);
VOID RAIDRecoveryForceMediumErrorBuildRequest(pIODescriptor pIOD, 
					      pIODescriptor pFailedIOD);
VOID RAIDRecoveryForceMediumError(pIODescriptor pIOD, pRecoveryDescriptor pRD,
				  pSysDevice pSD);


/***********************************************************************
 File diskfunc.c
***********************************************************************/
VOID			LocalUpdateAll (pOSDescriptor pDriverOSD);
VOID			LocalUpdateAbortIOCtlComplete(pOSDescriptor pWriteOSD);
VOID			LocalUpdateAllReady(pOSDescriptor pDriverOSD, pOSDescriptor pWriteOSD);
VOID            LocalUpdateOneSysDevStart(pOSDescriptor pOSD);
VOID            CdaShutdownOneSysDeviceComplete( pOSDescriptor  pLocalOSD );
VOID			LocalUpdateOneSysDev (pOSDescriptor pWriteOSD);
BOOL			LocalReadMemberReservedSector(pSysDevice pSD, pOSDescriptor pOSD,
					FPTR PostFunc);
VOID			IssueLocalReadMemberReservedSector(pIODescriptor   pRS_IOD);
VOID			LocalReadMemberReservedSectorComplete(pIODescriptor   pRS_IOD);
VOID			LocalWriteToNextMemberReservedSector (pIODescriptor			 pRS_IOD);
VOID			LocalSyncCacheComplete (pIODescriptor		   pRS_IOD);
VOID			LocalUpdateNextArrayDevice (pOSDescriptor  pWriteOSD);
VOID			CopySysDevInfoToReservedSectors (pSysDevice pSD, USHORT Member);
UCHAR			FindActiveMemToUpdateRS(pSysDevice pSD, UCHAR IOFlag);
UCHAR			ChkErrorOnUpdateRS(pSysDevice pSD, UCHAR Index, UCHAR LogEventFlg,
					UCHAR code);
VOID			LocalReadToNextMemberReservedSector(pIODescriptor   pRS_IOD);
VOID			DetermineArrayStateAfterReactivation(pSysDevice   pSD);
VOID            RetryLocalReadMemberRSUpdate(pIODescriptor pIOD);
//VOID			MDT_ReadWriteMetaData( pIODescriptor	pIOD, pReservedSectorLayout pBuffer, USHORT InitializationTime ) ;
//VOID			MDT_Initialize (pADAPTER_DEV_EXT pAdapterDevExt) ;

/***********************************************************************
 File cdar10.c
***********************************************************************/
//VOID			OverlayRW(pOSDescriptor pOSD);
//* VOID			PostOverlayRW(pOSDescriptor	  pOSD);
USHORT			ReadWriteSD (pOSDescriptor pOSD);
USHORT			PostReadWriteSD (pIODescriptor pIOD);

/***********************************************************************
 File r10ioctl.c
***********************************************************************/

VOID			R10Mgt (pRemoteCtlPkt pRP);
VOID			IOCtlR10Redundancy (pRemoteCtlPkt pRP);
VOID			PostIOCtlR10Redundancy(pOSDescriptor pOSD);
VOID			IOCtlR10ModeSense (pRemoteCtlPkt pRP);
VOID			R10StorageDevFuncPage0 (pRemoteCtlPkt pRP);
VOID			IOCtlR10AbortMgtOp (pRemoteCtlPkt pRP);
VOID			IOCtlR10VerifyDevice(pRemoteCtlPkt	pRP);
VOID			IOCtlR10RecreateDevice(pRemoteCtlPkt	pRP);
VOID			PostR10VerifyDevice(pOSDescriptor	pOSD);
VOID			PostR10RecreateDevice(pOSDescriptor	pOSD);
VOID 			IOCtlR10ChangeDrive(pRemoteCtlPkt pRP);

/***********************************************************************
 File rescan.c
***********************************************************************/
VOID	BuildChildRPFromParentRPRescan(pRemoteCtlPkt pChildRP,
				pRemoteCtlPkt pParentRP, pSysDevice pChildSD);
VOID	RescanLocalNextDevice(pOSDescriptor pScanOSD);
VOID	RescanTestReplacedDevice(pOSDescriptor pScanOSD);
VOID	SetupForScan(pSysDevice pScanAdapter);
VOID	TestExistingDevice (pPhysDevice pTestPD, pIODescriptor  pScanIOD);
VOID	TestDevTURComplete (pIODescriptor pScanIOD);
VOID	TestDevInqComplete (pIODescriptor pScanIOD);
VOID	RemoveExistingDevice( pSysDevice pScanAdapter, pPhysDevice pExistPD);
VOID	UpdateArraySysDeviceStatus (pIODescriptor pScanIOD);
//VOID	UpdateArraySysDeviceComplete(pSysDevice pSD, pPhysDevice pTestPD);
VOID	UpdateArraySysDeviceComplete(pADAPTER_DEV_EXT pAdapterDevExt);
VOID	RescanReadReservedSector(pSysDevice pSD,pPhysDevice pPD,FPTR PostFunc);
VOID	RescanReadReservedSectorComplete(pIODescriptor  pIOD);
VOID	RetryRescanZapReservedSector(pIODescriptor   pIOD);
VOID	RescanZapReservedSector(pPhysDevice pPD,pIODescriptor pIOD,FPTR PostFunc, USHORT, USHORT);
VOID	RescanZapReservedSectorDone(pIODescriptor pIOD);
VOID	RescanZapPartitionSector(pPhysDevice pPD,pIODescriptor pIOD,FPTR PostFunc);
VOID	RescanZapPartitionSectorDone(pIODescriptor pIOD);
VOID	RescanCompleted(pOSDescriptor pDriverOSD);
VOID	RescanFreeResources(pOSDescriptor pScanOSD);
VOID	AdapterScan (pSysDevice pAdapter, pOSDescriptor pOSD, pIODescriptor pIOD);
VOID	CheckTargets ( pIODescriptor pIOD );
VOID	ScanTargetDevices (pSysDevice pAdapterSD);
VOID	ProbeNewDevice (pSysDevice pAdapterSD);
VOID	IssuePACToOneAdapter(pOSDescriptor pScanOSD);
VOID	IssuePACToAllAdapters(pOSDescriptor pScanOSD);
VOID	IssuePACToCHIM(pOSDescriptor pDriverOSD);
VOID	UpdateArrayReservedSector(pIODescriptor pScanIOD);

/***********************************************************************
 File metadata.c
***********************************************************************/
VOID	MDT_ReadWrite(pADAPTER_DEV_EXT pAdapterDevExt, USHORT InitializationTime) ;

#ifdef TRACE_DEBUG
VOID LogTrace(UCHAR Function, UCHAR Opcode, UCHAR Id, UCHAR HaStat, UCHAR TarStat,
				UCHAR Key, UCHAR Code, ULONG Lba, ULONG Block, ULONG Parent, ULONG HashKey,
				ULONG SabreBase, ULONG SGAddress, ULONG SGLength);
#endif

/***********************************************************************
Hot Swap Rebuild:
When a disk is found at a bus id and tareget id which are still referred by the meta data
of any RAID1/10 as its missing member, the newly found disk is moved in as
a member and a rebuild is initiated.

Auto Rebuild:
When a disk is failed and spare(global) is already configured in the system,
then the spare will move in as a failed member and a rebuild is initiated.

 File cdatasks.c
***********************************************************************/

ULONG	IRebuildInternal(pSysDevice pSD, UCHAR code, USHORT iMemberIndex ) ;

VOID    HsrRebuildProcess ( pADAPTER_DEV_EXT pAdapterDevExt ) ;

VOID	HsrRebuildStart (pIODescriptor pIOD);


VOID	AutoRebuildDegradedArray (pADAPTER_DEV_EXT pAdapterDevExt );
VOID	AutoRebuildProcessChangeDrive(pSysDevice pCrntSD, 
									  pSysDevice	pSpareSD, 
									  UCHAR		FailedMember,
									  UCHAR		GoodMember);
VOID	AutoRebuildStart (pIODescriptor pIOD);
VOID	AutoRebuildWriteToNextMember(pIODescriptor pIOD);
VOID    AutoRebuildUpdatePDE(pIODescriptor pIOD);
VOID    AutoRebuildUpdateMetadata(pIODescriptor pIOD, 
                                    pSysDevice pCrntSD,
                                    pPhysDevice pNewPD);

VOID    SetDriveCache(pOSDescriptor pOSD,pIODescriptor pIOD,pPhysDevice pPD);
VOID    GetDriveCache(pOSDescriptor pOSD,pIODescriptor pIOD,pPhysDevice pPD, FPTR PostFunc);
VOID    SetDriveCacheComplete(pIODescriptor pIOD);
VOID    GetDriveCacheComplete(pIODescriptor pIOD);
VOID	GetCacheSetgForHotAddDrive(pIODescriptor pTestIOD);
VOID	GetCacheSetgForHotAddDriveDone(pIODescriptor pTestIOD);


VOID    RetryRebuildUpdataMetadata(pIODescriptor pIOD);
/*
Start Verify and Clear from the driver itself instead on relying on the agent
*/
VOID AutoVerifyArray (pADAPTER_DEV_EXT pAdapterDevExt );
VOID AutoZeroArray (pADAPTER_DEV_EXT pAdapterDevExt );
void	HsrUpdatePDEinGlobalMD(pIODescriptor pIOD);
void	UpdatePDEInGlobalMD(pIODescriptor pIOD, pReservedSectorLayout pBuffer, BOOL ToAdd);
void	PostUpdatePDEInGlobalMD(pIODescriptor pIOD, BOOL AllowProxy);

/*
 * LockRange: When background op like verify or rebuild enters a new range,
 * we should lock that range for the IOs. Whenever this happens
 * all such IODs are flagged and current RD is kept in SD for when
 * IOPull happens, it examins and tries to lock the range again. One
 * public call now kept here.
 */
VOID	LockNextRangeDispatch( pSysDevice pSD ) ;
void	IoctlResumeTasks (pADAPTER_DEV_EXT pAdapterDevExt) ;
void	IoctlPauseTasks (pADAPTER_DEV_EXT pAdapterDevExt) ;

void	FillOutOSMPointers(HIM_OSM_FUNC_PTRS HIM_PTR osmRoutines, UCHAR AD_MemOrIO);

ULONG	PhysDiscoverNextLu ( pAdapterDevice  pAD, ULONG  DiscoveryState ) ;
ULONG	InitializeSasChim( pADAPTER_DEV_EXT pAdapterDevExt ) ;

VOID    OSMPortEvent(VOID HIM_PTR  pPortContext,VOID HIM_PTR pEventContext,HIM_UINT16 event,HIM_UEXACT32 actionSet, ...) ;
VOID    OSMAdapterEvent(VOID HIM_PTR  pPortContext,VOID HIM_PTR pEventContext,HIM_UINT16 event,HIM_UEXACT32 actionSet, ...) ;
VOID    OSMIobEvent (VOID HIM_PTR   osmTaskContext, VOID HIM_PTR   himEventContext, HIM_UINT16 eventCode, HIM_UEXACT32 actionSet, ...);
VOID    HandlePortEvents(pPortInformation pPortInfo);

#if CDA_ENABLE_LU_VALIDATION_SUPPORT
VOID OSMLocalLuEvent(pPhysDevice pPD, UINT32 actionSet);
VOID HandleLocalLuEvents(pPhysDevice pPD);
UINT32 CdaLuValidationToPort(pPortInformation pPortInfo);
/* Return value of above function are */
#define     PORT_DEVICES_MORE_VALIDATION_REQUIRED       0x01
#define     PORT_DEVICES_NO_MORE_VALIDATION_REQUIRED    0x02
UINT32 CdaDeviceLuValidation(pAdapterDevice   pAD, pPhysDevice pPD);
VOID CdaDeviceLuValidationComplete(pIODescriptor   pTestIOD);
#endif

#ifdef CDA_ENABLE_NCQ_SUPPORT
VOID OSMLuEvent(
        void HIM_PTR  osmLuContext,
        void HIM_PTR  himEventContext,
        HIM_UINT16    eventCode,
        HIM_UEXACT32  actionSet,
        ...);
VOID HandleLuEvents(pPhysDevice  pPD);
void PostCdaPerformLuRecovery(pIODescriptor pRetrnIOD);
void PostCdaAbortTaskSet(pIODescriptor pRetrnIOD);
#endif
VOID 	PostPacForPort(HIM_IOB HIM_PTR pIOB);
VOID 	ProcessNewTargets(pAdapterDevice pAD);
VOID    PostProcessNewTargets(pOSDescriptor pOSD);
VOID    UpdateArrayReservedSector(pIODescriptor pScanIOD);
VOID    UpdateArraySysDeviceStatus (pIODescriptor   pScanIOD);

VOID    HandleAdapterEvents(pAdapterDevice pAD);
UCHAR	DiscoverNewPorts(pAdapterDevice pAD);

VOID    HandleIobEvents(pIODescriptor pIOD);
VOID    HotPlug_SetInitPostProcessNewTargets(pOSDescriptor pOSD);
VOID    HotPlug_AwaitInitPostProcessNewTargets(pOSDescriptor pOSD);
VOID    HotPlug_ClearInitPostProcessNewTargets(pOSDescriptor pOSD);
VOID    HotPlug_InitTimeTestDevicePresent(pPhysDevice pPD, pADAPTER_DEV_EXT pAdapterDevExt);
VOID    HotPlug_AwaitInitProbeLU(pIODescriptor     pIOD);
VOID    HotPlug_InitTimeProbeLU(pPortInformation pPortInfo);

void
PerformAdapterEventActionSet (pAdapterDevice pAdapter);

UINT32	PhysClearDeviceContext( pPhysDevice pPD ) ;
UINT32	PhysPolledIo( pAdapterDevice   pAD, HIM_IOB HIM_PTR  pIOB ) ;

void	PhysInitIOB( pAdapterDevice pAD, pIODescriptor pIOD, UINT32 function ) ;
UINT32	PhysGetDevicePortTSH( pAdapterDevice  pAD, HIM_TASK_SET_HANDLE AdpterOrPortTSH ) ;
#ifdef CDA_ENABLE_NCQ_SUPPORT
UINT32	PhysGetLuTSH( pAdapterDevice  pAD, pSasDevice pTempSasDevice, pPhysDevice pPD ) ;
#else
UINT32	PhysGetLuTSH( pAdapterDevice  pAD, pSasDevice pTempSasDevice ) ;
#endif
UINT32  SizeIOBMemNeed(  pAdapterDevice pAD, HIM_UINT8 iobFunc );
ULONG	PhysStartAdapter( pAdapterDevice pAD );

/***********************************************************************
 File cdacsmi.c
***********************************************************************/
UINT32  CSMI_Initialize( pADAPTER_DEV_EXT pAdapterDevExt );

void	AthenaQueueCommand( POSD  pOSD ) ;
void	PsuiteQueueCommand( POSD  pOSD ) ;

/***********************************************************************
 File physif.c
***********************************************************************/

UINT32	PhysQueueQuiesce( pAdapterDevice  pAD );
VOID	CdaNotify( UINT32 Event, pADAPTER_DEV_EXT pAdapterDevExt, pPhysDevice	pPD);
VOID    CdaRemoveExistingDevice( pADAPTER_DEV_EXT  pAdapterDevExt, pPhysDevice pPD);
ULONG	PhysRemoveSasDevice(ULONG SasDeviceIndex, pAdapterDevice  pAD);
UCHAR   ValidatePort(pPortInformation pPortInfo);
USHORT  CdaEnablePort(PVOID pAdptDevExt,  PVOID PhyTSHandle);
USHORT  CdaDisablePort(PVOID pAdptDevExt, pPortInformation pPortInfo);
VOID    CdaStartScan (PVOID pAdptDevExt);
USHORT  CdaCheckForPortIOPending(PVOID pAdptDevExt, PVOID PortTSHandle);
USHORT	HsrFindMissingMember( pADAPTER_DEV_EXT pAdapterDevExt, pPhysDevice pPD);


VOID    AbortTaskForProbeLuComplete(pIODescriptor pIOD);

void    FlushAllCache( pADAPTER_DEV_EXT  pAdapterDevExt ) ;
void    FlushAllCacheComplete(pIODescriptor   pIOD);
BOOL    CheckPendingPhysDeviceRemval(pADAPTER_DEV_EXT  pAdapterDevExt, pPhysDevice pPD);
ULONG   AdjustDevicePortITNexusLossTime( pAdapterDevice  pAD,  pPhysDevice pPD, USHORT TimeinMSec);
/* New functions added for Hot Plug fixes */
UINT32  ValidateTargetsForPort(pPortInformation pPortInfo);
pIODescriptor AllocDiscoveryIODs(pAdapterDevice pAD, USHORT PurposeOfIOD);
VOID    FreeDiscoveryIODs(pAdapterDevice pAD, pIODescriptor pIODToFree, USHORT PurposeOfIOD);
VOID    PerformLuDiscovery(pAdapterDevice pAD);
VOID    PostProbeLuForSasDevice(HIM_IOB HIM_PTR  pIOB);
UCHAR   SendPACForPort(pPortInformation pPortInfo);
VOID    PostPacForPort(HIM_IOB HIM_PTR  pIOB);
UCHAR   SendPACForAdapter(pAdapterDevice pAD);
VOID    PostPacForAdapter(HIM_IOB HIM_PTR  pIOB);
BOOL    CheckForNewDevicePorts(pAdapterDevice pAD, HIM_TASK_SET_HANDLE AdpterOrPortTSH);
UCHAR   QueueResetAdapter(pAdapterDevice pAD, ULONG Action);
pPhysDevice CdaCreatePDForLuTSH( pAdapterDevice  pAD, pSasDevice pTempSasDevice );
VOID    PostTestDevicePresent_InitializationTime(pOSDescriptor pOSD);
VOID    PostPhysQueueEnablePhy(HIM_IOB HIM_PTR  pIOB);
VOID    PostPhysQueueQuiesce(HIM_IOB HIM_PTR  pIOB);

VOID    PerformDevicePortInitialization(pAdapterDevice pAD);
VOID    PostDevicePortInitialization( HIM_IOB HIM_PTR  pIOB);

void 	FlushAllCacheBeforeShutDown(pRemoteCtlPkt pShutdownPkt);
void 	PostCdaShutdown(pIODescriptor pIOD);
void    SendStopUnitToPD(pIODescriptor pIOD);
void    SendStopUnitToSD(pOSDescriptor pLocalOSD);
void    PostSendStopUnitToSD(pOSDescriptor pLocalOSD);

VOID    IssueTestDevicePresentSequence(pIODescriptor pIOD);

UCHAR   AdapterResetPending(pAdapterDevice pAD);
UCHAR   DevicePortResetPending(pPhysDevice pPD);
UCHAR PortFreeze(pSasDevice pSasInfo);
UCHAR   LogicalUnitResetPending(pPhysDevice pPD);

USHORT  CheckFailedPhyBitMask(pAdapterDevice pAD);
VOID    PostEnableFailedPhyDone(HIM_IOB HIM_PTR     pIOB);
VOID    DoCdaStartScan(pAdapterDevice pAD);

USHORT  DoResetPort(pAdapterDevice pAD, pPortInformation pPortInfo);
VOID    PostResetPortDone(HIM_IOB HIM_PTR     pIOB);


USHORT              CreateCSMIIODescriptors (pSysDevice pParentSD);
pIODescriptor       AllocCSMIIODs (pADAPTER_DEV_EXT pAdapterDevExt);
VOID                FreeCSMIIOD (pADAPTER_DEV_EXT pAdapterDevExt);

VOID   StampSimpleVolumeMetadata (pPhysDevice pPD, pIODescriptor pIOD, FPTR PostFunc, USHORT InitFlag, USHORT LockNeeded);
VOID   StampSimpleVolumeMetadataDone (pIODescriptor pIOD);
VOID   RetryStampSimpleVolumeMetadata (pIODescriptor pIOD);
#ifdef SES2_DEVICE
VOID    NotifyOSMMemberInsertedToNewLocation(pPhysDevice pArrayPD, pSysDevice pCrntSD);

/***********************************************************************
 File ses2dev.c
***********************************************************************/
VOID    EnumerateSES2EnclosureDevices ( pAdapterDevice   pAD);
VOID    DoEnumerateSES2EnclosureDevices ( pAdapterDevice   pAD);
VOID    IssueSES2ConfigurationDiagPage(pIODescriptor pIOD);
VOID    IssueSES2DeviceElementStatusPage(pIODescriptor pScanIOD);
VOID    PostSES2ConfigurationDiagPage (	pIODescriptor   pScanIOD);
VOID    PostSES2DeviceElementStatusPage (pIODescriptor   pScanIOD);
VOID    PostSES2ElementDescriptorPage (	pIODescriptor   pScanIOD);
VOID    AssignDeviceElementNumberToMatchingDevice(pSasDevice pSes2Sas, 
                                          UCHAR *pDeviceElementSasAddress, 
                                          UCHAR elementNumber,
                                          UINT16 BayNumber);
VOID        FindRelativePositionOfSES2Devices(pAdapterDevice pAD);
VOID        CdaAssignPTMapAndNotifyOSM(pAdapterDevice pAD);
VOID        CdaAssignPTMapForDirectAttachedDevice(pSasDevice pSas, pAdapterDevice pAD);
USHORT      GetSES2EnclosureStatusPage(pADAPTER_DEV_EXT pAdapterDevExt, pSasDevice pSes2Sas);
VOID        SetSES2EnclosureControlPage(pADAPTER_DEV_EXT pAdapterDevExt, pSasDevice pSes2Sas);
VOID        EnableLEDActivity(pADAPTER_DEV_EXT pAdapterDevExt);
VOID        DoEnableLEDActivity(pADAPTER_DEV_EXT pAdapterDevExt);
VOID        PostSES2ArrayElementControlPage (pIODescriptor   pIOD);
VOID        PostSES2EnclosureStatusPage (pIODescriptor   pIOD);
UCHAR       AsciiToNumber (UCHAR *pCh, UINT32 Length);


pSysDevice  FindSDFromLEDPd(pPhysDevice pPD);

#endif

#define InvokeMDMEH_IODPosts(pIOD)                                      \
{                                                                 \
    USHORT   CurrentStackVal = 0;                                    \
    if(pIOD->iIOD_PostStack == 0)                                 \
        CDA_ASSERT(FALSE);                                        \
    while (pIOD->iIOD_PostStack)                                  \
	{                                                             \
        pIOD->iIOD_PostStack--;                                   \
        CurrentStackVal = pIOD->iIOD_PostStack;                      \
		(CPTR(pIOD->pIOD_PostStack[pIOD->iIOD_PostStack]))(pIOD); \
        if(CurrentStackVal == 1)                                  \
        {                                                         \
             break;                                               \
        }                                                         \
	}                                                             \
}                                                                 

#define InvokeIODPosts(pIOD)                                      \
{                                                                 \
    USHORT   CurrentStackVal = 0;                                    \
    if(pIOD->iIOD_PostStack == 0)                                 \
        CDA_ASSERT(FALSE);                                        \
    while (pIOD->iIOD_PostStack)                                  \
	{                                                             \
        pIOD->iIOD_PostStack--;                                   \
        CurrentStackVal = pIOD->iIOD_PostStack;                      \
		(CPTR(pIOD->pIOD_PostStack[pIOD->iIOD_PostStack]))(pIOD); \
        if(CurrentStackVal == 0)                                  \
        {                                                         \
             break;                                               \
        }                                                         \
	}                                                             \
}                                                                 
 
 
#define ClearMEHFlags(pIOD) { \
	if (pIOD->IOD_BitFlags & CDA_IOD_FLAG_MEH_RECOVERY_IO) { \
		pIOD->IOD_BitFlags &= ~CDA_IOD_FLAG_MEH_RECOVERY_IO; \
		pIOD->IOD_BitFlags &= ~CDA_IOD_FLAG_METADATA_MEH_RECOVERY_IO; \
	} \
}
VOID RetryDevSingleCmdSubmitRSUpdate(pIODescriptor   pIOD);
UINT32 RetryPendingMetaDataUpdate(pSysDevice    pSD);
UINT32 SpecialMemoryAllocation(pADAPTER_DEV_EXT pAdapterDevExt);

#endif	/*COREPROT_INCLUDED*/


