/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/************************************************************************

Module Name:

      custom.h

Abstract:

      This is the main customization file.
      All OSM and OS needed definitions are defined here. 
      
      
Copyright (C) 2003 Adaptec, Inc.  All Rights Reserved.
 
This software contains the valuable trade secrets of Adaptec or its 
licensors. The software is protected under international copyright laws 
and treaties. This software may only be used in accordance with the 
terms of its accompanying license agreement.


History:

10/11/2004: Madhuresh
Increased CDA_MAX_IODS to 132.

01/04/2004: Madhuresh
Versioning modified.

10/29/2003: Madhuresh
Typical RD Constants that OSM need to provide to CDA are added.
Most of these provide maximum elements in an array.

10/28/2003: Tam Tran
Initial Check in.

************************************************************************/

#ifndef CUSTOM_H
#define CUSTOM_H


#ifndef _CUSTOMIZED
#define _CUSTOMIZED

/*##################################################################*/
/*# The following fields always change with every release          #*/
/*##################################################################*/

/** Internal Versioning label **/
#ifndef CDA_FOR_SATA
#define OSM_DESCRIPTION         "Fillmore 1.05s1/1.07 CDA_V303 CHIM_V627A5 SEQ_Alpha_7g17"
#else
#define OSM_DESCRIPTION         "RAGS 2 dev version - based on SAS CDA: 400_6 CHIM: V032"
#endif


/*##################################################################*/
/*# The following fields are flexible OSM fields                   #*/
/*##################################################################*/

/* When option eabled(1) the driver will use the srbext to buffer sense data,
   copy it to the LUExt, wher it waits for a request sense command to pick up
   the data from LUExt buffer.  Reason LUExt was not used directly was because
   it is not DMA-able.
*/
#define AUTOSENSE      1

/************************************************************************
	These are typical RD Constants that OSM need to provide to CDA.
	Most of these provide maximum elements in an array.
************************************************************************/
/*
	On one instance of the chip, how many total devices will be found.
	This number does NOT include adapter or controller device. All other
	devices such as disks, tapes, expanders etc are included.
*/
#ifndef CDA_FOR_SATA
#define	CDA_MAX_DEVICES_PER_ADAPTER     128
#else /* CDA_FOR_SATA */
#define	CDA_MAX_DEVICES_PER_ADAPTER     16
#endif
/*
	On one instance of the chip, how many RAIDs can the user configure.
*/
#define	CDA_MAX_RAIDS_PER_ADAPTER       2 /* Filmore 1.0 changed from 4 to 2 */
/*
	Maximum members a RAID can have.
*/
#define	CDA_MAX_DISKS_PER_RAID          6

/*
	Maximum RSU count
*/
#define	CDA_MAX_RSU_IOD_COUNT           (CDA_MAX_RAIDS_PER_ADAPTER*2)
#define	CDA_MAX_RSU_BUFFER_COUNT        32

/*
	Maximum number of IODs allocated by CDA Engine.
*/
#ifdef CDA_FOR_SATA
#define CDA_MAX_IODS                    64
#else
#define CDA_MAX_IODS                    132
#endif

/*
    Maximum number of retries for the CSMI pass through
*/
#define CSMI_MAX_RETRY                  3
 /*
   Diagnostic/Debug Includes - these defines should not be set to 1 for any production driver
   Setting NTSTRESS_CHK to 1 includes code used for debugging NTStress miscompares
 */
 #define     NTSTRESS_CHK        0

/* BIOS Flashing support */
#ifdef CDA_FOR_SATA
	#define  CDA_SUPPORTS_BIOS_FLASHING       FALSE
#else
	#define  CDA_SUPPORTS_BIOS_FLASHING       TRUE
#endif

/*##################################################
  # The following options do not change very often #
  ##################################################*/
/** Common definition **/
#ifdef CDA_FOR_SATA
#ifdef __linux__
	#if	defined(CDA_FOR_ADPAHCI)
		#define DRIVER_NAME		"adpahci"	
	#elif	defined(CDA_FOR_AAR81XX)
		#define DRIVER_NAME		"aar81xx"	
	#else
		#define DRIVER_NAME		"adpsata"	
	#endif
#else /* __linux__ */
	#define DRIVER_NAME		"adpAHCI"	
#endif /* __linux__ */
#else
#ifdef CDA_FOR_AURORA
#define DRIVER_NAME		"adp725x"	
#else
#define DRIVER_NAME		"adp94xx"	
#endif
#endif

/** OS version specific definition **/

#ifdef __linux__
	#if BITS_PER_LONG == 64
    	#define     _LME                1
	#endif
#endif

#ifdef SCO_DDI8
	#if ULONG_MAX > (1LL << 33)
    	#define     _LME                1
	#endif
#endif

#if defined (_WIN64) 
    #define     _LME                1
#endif

#if defined (_NT_40)

    #define     OS_STR              "NT40"

#elif defined (_NT_5)

    #define     OS_STR              "W2K "

#elif defined (_WINXP)

    #if defined (_ITANIUM)

        #define     OS_STR          "XP64"

    #else  // x86/IA32

        #define     OS_STR          "XP  "

    #endif /* (_ITANIUM) */

#elif defined (_WNET)
    //
    // .NET STORPORT info
    //
    // Let's take the sources compile option for STORPORT or Not
    // 
    #if defined (_STORPORT_ENABLE)
        //
        // Storport functions and libaries used
        //
        #define _STORPORT               1
    #else
        #define _STORPORT               0
    #endif
    
    #if _STORPORT
        // Support for the BuildIo function on Storport compiles
        #define _BUILDIO                    1 

        //
        // Target resets are supported but not LUN Resets at this time
        //
        #define _TGT_RESET_SUPPORT          0
        //
        // Used in early versions of Windows .net so that SCSIPortXX calls Map 
        // to StorportXX calls
        //
        #define STOR_USE_SCSI_ALIASES       

    #else
        //
        // Can not add the following to miniports
        //
        #define _BUILDIO                    0  
        #define _TGT_RESET_SUPPORT          0
  
    #endif //STORPORT

    #if defined (_ITANIUM) || (_AMD64)

        #define     OS_STR          ".NET64"

    #else  // x86/IA32

        #define     OS_STR          ".NET"

    #endif /* (_ITANIUM) || (_AMD64) */

#elif defined (_WIN_UNIVERSAL)
     
    #if defined (_ITANIUM) || (_AMD64)

        #define     OS_STR          "WIN64"

    #else  // x86/IA32

        #define     OS_STR          "WIN32"

        #define     HIM_PROBE_EXT_DBG       0
                       
    #endif /* (_ITANIUM) || (_AMD64) */

#endif  /* OS */
#endif  /* _CUSTOMIZED */

/***************************************************************
	The following are formulas and do not change often.
***************************************************************/

#undef 	DRIVER_VERSION
#define DRIVER_VERSION              FILE_VERSION
#define VER_FILEDESCRIPTION         FILE_DESCRIPTION_STR

#ifndef STR
#define STR(s)                      CONVERTTOSTRING(s)
#define CONVERTTOSTRING(s)          #s
#endif

#ifndef SCO_DDI8
#undef 	DRIVER_VERSION
#define DRIVER_VERSION              FILE_VERSION
#define VER_FILEDESCRIPTION         FILE_DESCRIPTION_STR

#define ADPT_MAJOR_REV_STR          STR(ADPT_MAJOR_REV)
#define ADPT_MINOR_REV_STR          STR(ADPT_MINOR_REV) 
#define ADPT_RELEASE_REV_STR        STR(ADPT_OSM_REV)
#define ADPT_BUILD_REV_STR          STR(ADPT_CHIM_REV)

/*#define FILE_VERSION_STR            "1.00.00.000"*/
#define FILE_VERSION_STR            ADPT_MAJOR_REV_STR \
                                    "." ADPT_MINOR_REV_STR \
                                    "." ADPT_RELEASE_REV_STR \
                                    "." ADPT_BUILD_REV_STR

/*#define FILE_VERSION                1,0,0,0*/
#define FILE_VERSION                ADPT_MAJOR_REV,ADPT_MINOR_REV,ADPT_OSM_REV,ADPT_CHIM_REV

#define ADAPTEC_STR                 "Adaptec, Inc."
#define SHORT_COMPANY_STR           "ADPT"
#define VER_LEGALCOPYRIGHT_STR      "Copyright \251 " VER_LEGALCOPYRIGHT_YEARS " Adaptec, Inc. All rights reserved." 
#define VER_LEGALCOPYRIGHT_YEARS    "2005"

#endif // SCO_DDI8

/** Product build no. **/

#if defined(_WINXP)
    #undef VER_PRODUCTBUILD
    #define VER_OS_STR "Microsoft\251 Windows XP"
    #define VER_PRODUCTBUILD 2600
#elif defined (_WNET)
    #undef VER_PRODUCTBUILD
    #define VER_OS_STR "Microsoft\251 Windows Server 2003"
    #define VER_PRODUCTBUILD 3790
#elif defined (_NT_5) 
    #undef VER_PRODUCTBUILD
    #define VER_OS_STR "Microsoft\251 Windows (TM) 2000"
    #define VER_PRODUCTBUILD 2195
#elif defined(_NT_40)
    #undef  VER_PRODUCTBUILD
    #define VER_OS_STR "Microsoft\251 Windows (TM) NT 4.0"
    #define VER_PRODUCTBUILD 4000
#elif defined (_WIN_UNIVERSAL)
    #undef VER_PRODUCTBUILD
    #define VER_OS_STR "Microsoft\251 Windows 2000, XP and Server 2003"
    #define VER_PRODUCTBUILD 3790
#endif


/** Product name and file description definition **/
#ifndef CDA_FOR_SATA

#if defined (_NT_40)
    /* WINDOWS NT 40 Descriptions */
    #define FILE_DESCRIPTION_STR        "Adaptec Windows SAS/SATA Driver"
    #define PRODUCTNAME_STR             "Adaptec Windows NT 4.0 SAS/SATA Family Driver"

#elif defined (_NT_5)
    /* WINDOWS 2000 Descriptions */
    #define FILE_DESCRIPTION_STR        "Adaptec Windows SAS/SATA Driver"
    #define PRODUCTNAME_STR             "Adaptec Windows 2000 SAS/SATA Family Driver"

#elif defined (_WINXP)
    /* WINDOWS XP Descriptions */
    #if defined (_ITANIUM)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver (IA-64)"
        #define PRODUCTNAME_STR         "Adaptec Windows XP SAS/SATA Family Driver"
    #else
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver"
        #define PRODUCTNAME_STR         "Adaptec Windows XP SAS/SATA Family Driver"
    #endif

#elif defined (_WNET)
    /* WINDOWS .NET Descriptions */
    //
    // Specify in description whether MINI or STORPORT
    //
    #if _STORPORT
        #if defined (_ITANIUM)
            #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Storport Driver (IA-64)"
            #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Storport Driver"
        #elif defined (_AMD64)
            #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Storport Driver (EM64T)"
            #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Storport Driver"
        #else
            #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Storport Driver"
            #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Storport Driver"
        #endif
    #else //MINIPORT
    #if defined (_ITANIUM)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver (IA-64)"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Driver"
    #elif defined (_AMD64)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver (AMD64)"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Driver"
    #else
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Driver"
    #endif
    #endif //_STORPORT

#elif defined (_WIN_UNIVERSAL)  
      /* WINDOWS 2K/XP/NET Descriptions	*/
    #if defined (_ITANIUM)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver (IA-64)"
        #define PRODUCTNAME_STR         "Adaptec Windows XP and Server 2003 SAS/SATA Family Driver"
    #elif defined (_AMD64)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver (AMD64)"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SAS/SATA Family Driver"
    #else
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SAS/SATA Driver"
        #define PRODUCTNAME_STR         "Adaptec Windows 2000, XP and Server 2003 SAS/SATA Family Driver"
    #endif
#endif /* (_NT_40) */


/** Orignal Filename definition **/
#define ORIGINALFILENAME_STR            "adp94xx.sys"

#else //CDA_FOR_SATA

#if defined (_NT_40)
    /* WINDOWS NT 40 Descriptions */
    #define FILE_DESCRIPTION_STR        "Adaptec Windows SATA Driver"
    #define PRODUCTNAME_STR             "Adaptec Windows NT 4.0 Embedded Serial ATA Family Driver"

#elif defined (_NT_5)
    /* WINDOWS 2000 Descriptions */
    #define FILE_DESCRIPTION_STR        "Adaptec Windows SATA Driver"
    #define PRODUCTNAME_STR             "Adaptec Windows 2000 Embedded Serial ATA Family Driver"

#elif defined (_WINXP)
    /* WINDOWS XP Descriptions */
    #if defined (_ITANIUM)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver (IA-64)"
        #define PRODUCTNAME_STR         "Adaptec Windows XP Embedded Serial ATA Family Driver"
    #else
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver"
        #define PRODUCTNAME_STR         "Adaptec Windows XP Embedded Serial ATA Family Driver"
    #endif

#elif defined (_WNET)
    /* WINDOWS .NET Descriptions */
    //
    // Specify in description whether MINI or STORPORT
    //
    #if _STORPORT
        #if defined (_ITANIUM)
            #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Storport Driver (IA-64)"
            #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SATA Family Storport Driver"
        #elif defined (_AMD64)
            #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Storport Driver (EM64T)"
            #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SATA Family Storport Driver"
        #else
            #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Storport Driver"
            #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 SATA Family Storport Driver"
        #endif
    #else //MINIPORT
    #if defined (_ITANIUM)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver (IA-64)"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 Embedded Serial ATA Family Driver"
    #elif defined (_AMD64)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver (AMD64)"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 Embedded Serial ATA Family Driver"
    #else
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 Embedded Serial ATA Family Driver"
    #endif
    #endif //_STORPORT

#elif defined (_WIN_UNIVERSAL)  
      /* WINDOWS 2K/XP/NET Descriptions	*/
    #if defined (_ITANIUM)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver (IA-64)"
        #define PRODUCTNAME_STR         "Adaptec Windows XP and Server 2003 Embedded Serial ATA Family Driver"
    #elif defined (_AMD64)
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver (AMD64)"
        #define PRODUCTNAME_STR         "Adaptec Windows Server 2003 Embedded Serial ATA Family Driver"
    #else
        #define FILE_DESCRIPTION_STR    "Adaptec Windows SATA Driver"
        #define PRODUCTNAME_STR         "Adaptec Windows 2000, XP and Server 2003 Embedded Serial ATA Family Driver"
    #endif
#endif /* (_NT_40) */


/** Orignal Filename definition **/
#define ORIGINALFILENAME_STR            "adpAHCI.sys"
/* SATA will always be not fillmore1.1 */
#define NOT_FILLMORE_1_1
#endif //CDA_FOR_SATA

#endif  /* CUSTOM_H */
