/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/*
 * HostRaid Linux OSM Data Structures and Function Declarations.
 *
 * $Id$
 */

#ifndef	_HR_H
#define _HR_H


/******************************** Linux Includes ******************************/
#include <asm/byteorder.h>
#include <asm/io.h>

#include <linux/types.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/pci.h>
#include <linux/smp_lock.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/config.h>
#include <linux/slab.h>
#include <linux/nmi.h>
#include <linux/hdreg.h>
#include <asm/uaccess.h>
#include <linux/ioctl.h>
#include <linux/init.h>

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(x,y,z)	(((x)<<16) + ((y)<<8) + (z))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#include <linux/blk.h>
#include "sd.h"
#include "hosts.h"
#include "scsi.h"
#else
#include <linux/moduleparam.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_driver.h>
#include <scsi/scsi_request.h>
#include <scsi/scsi.h>
#include <scsi/scsi_eh.h>
#include <scsi/scsi_tcq.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsicam.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15)
#include "scsi_priv.h"
#endif
#endif

#include <linux/blkdev.h>
#include "queue.h"

/****************************** CDA Core Includes *****************************/

#ifdef SUCCESS
#undef SUCCESS
#define SUCCESS	0
#endif

#include "custom.h"

#ifdef _CSMI
#include "cdacsmi.h"
#endif

#include "sysdef.h"
#include "errlog.h"
#include "scsisns.h"

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
typedef struct scsi_cmnd Scsi_Cmnd;
typedef struct scsi_device Scsi_Device;
typedef struct scsi_host_template Scsi_Host_Template;
#endif

#define UNREFERENCED_PARAMETER(x)
/********************************* SMP Support ********************************/
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0) || defined(SCSI_HAS_HOST_LOCK))
#define	HR_SCSI_HAS_HOST_LOCK 1

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
#define hr_assign_host_lock(hr) scsi_assign_lock((hr)->host, &(hr)->spin_lock);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,21)	\
   && defined(HR_RED_HAT_LINUX_KERNEL)
#define hr_assign_host_lock(hr)				\
do {							\
	(hr)->host->host_lock = &(hr)->spin_lock;	\
} while (0)
#else
#define hr_assign_host_lock(hr)			\
do {						\
	(hr)->host->lock = &(hr)->spin_lock;	\
} while (0)
#endif
#else	/* !HR_SCSI_HAS_HOST_LOCK */
#define	HR_SCSI_HAS_HOST_LOCK 0
#define hr_assign_host_lock(hr)
#endif	/* !HR_SCSI_HAS_HOST_LOCK */

/**************************** Debug Logging Macros ***************************/
#if HR_DEBUG

#define hr_log(log_var, log_level, fmt, ...) \
do {						   \
	if (log_var >= log_level) 		   \
		printk(fmt, ##__VA_ARGS__);	   \
} while (0)

#else 

#define hr_log(log_var, log_level, fmt, ...) /**/

#endif /* HR_DEBUG */

#ifndef ADAPTEC_IOCTL
#define ADAPTEC_IOCTL	0xAAAA
#endif

#ifndef	HR_BOOT_DEBUG
#define	HR_BOOT_DEBUG 		0
#endif

#ifndef HR_RUNTIME_DEBUG
#define HR_RUNTIME_DEBUG	0
#endif

#ifndef HR_CHIM_DEBUG
#define HR_CHIM_DEBUG 0
#endif

#ifndef	HR_OSMIF_DEBUG
#define	HR_OSMIF_DEBUG 0
#endif

#ifndef	HR_ERROR_DEBUG
#define	HR_ERROR_DEBUG		0
#endif

/* For initial debugging purpose only. */
#define IN	0
#define OUT	1
#define LINE	2

#define HR_DBG_FUNC(x)							\
do {									\
	if (x == IN) {							\
		printk("+++ Entering Function: %s:%d.\n", __FUNCTION__, \
			__LINE__);					\
	} else if (x == OUT) {						\
		printk("--- Exiting Function: %s:%d.\n", __FUNCTION__,  \
			__LINE__);					\
	} else {							\
		printk("*** %s (%d).\n", __FUNCTION__, __LINE__);	\
	}								\
} while (0)

/*************************** Linux Interrupt Support **************************/
#ifndef IRQ_RETVAL
typedef void irqreturn_t;
#define	IRQ_RETVAL(x)
#endif

/********************************* Misc Macros ********************************/
#ifndef HR_SATA_BUILD
#define HR_SKIP_SECTION(x)     ((x + 31) / 128) * 4
#else
#define HR_SKIP_SECTION(x)     x
#endif
#ifndef MIN
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#endif

#define hr_name(hr)	((hr)->unit_name)


#ifndef SCSI_DATA_READ
#define SCSI_DATA_READ  DMA_FROM_DEVICE
#define SCSI_DATA_WRITE DMA_TO_DEVICE
#define SCSI_DATA_UNKNOWN DMA_BIDIRECTIONAL
#define SCSI_DATA_NONE  DMA_NONE
#define scsi_to_pci_dma_dir(_a) (_a)
#endif

/***************************** Device Mapping *********************************/
#define OSM_MAX_CHANNEL			8
#ifdef CDA_FOR_SATA
#define OSM_MAX_TARGET_SUPPORTED	128	
#else
#define OSM_MAX_TARGET_SUPPORTED	256	
#endif
#define OSM_MAX_TARGET_PER_PATH		(OSM_MAX_TARGET_SUPPORTED/OSM_MAX_CHANNEL)
//#define OSM_MAX_TARGET_PER_PATH		16	
//mdt #define OSM_MAX_LUN			128
#define OSM_MAX_LUN			2	

/**************************** Forward Declarations ****************************/
struct hr_softc;
struct hr_io_request;

typedef struct pci_dev *hr_dev_softc_t;
typedef Scsi_Cmnd     *hr_io_ctx_t;
typedef void hr_eh_post_t(struct hr_softc *, struct hr_io_request *);

/**************************** Timer DataStructures ****************************/
#define HR_USECS_PER_JIFFY (1000000/HZ)
typedef struct timer_list hr_timer_t;

/******************************** Bus Space/DMA *******************************/
#if (BITS_PER_LONG > 32) && (!defined(HR_SATA_BUILD)) 
	#define HR_DMA_64BIT_SUPPORT		1
#else
	#define	HR_DMA_64BIT_SUPPORT		0
#endif
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,2,17)
typedef dma_addr_t bus_addr_t;
#else
typedef uint32_t bus_addr_t;
#endif
typedef uint32_t bus_size_t;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
#define	hr_alloc_coherent(hr, size, bus_addr_ptr) \
	dma_alloc_coherent(&hr->dev_softc->dev, size, bus_addr_ptr, /*flag*/0)

#define	hr_free_coherent(hr, size, vaddr, bus_addr) \
	dma_free_coherent(&hr->dev_softc->dev, size, vaddr, bus_addr)

#define	hr_map_single(hr, buf, size, direction) \
	dma_map_single(&hr->dev_softc->dev, buf, size, direction)

#define	hr_unmap_single(hr, busaddr, size, direction) \
	dma_unmap_single(&hr->dev_softc->dev, busaddr, size, direction)

#define	hr_map_sg(hr, sg_list, num_sg, direction) \
	dma_map_sg(&hr->dev_softc->dev, sg_list, num_sg, direction)

#define	hr_unmap_sg(hr, sg_list, num_sg, direction) \
	dma_unmap_sg(&hr->dev_softc->dev, sg_list, num_sg, direction)

#elif LINUX_VERSION_CODE > KERNEL_VERSION(2,3,0)

#define	hr_alloc_coherent(hr, size, bus_addr_ptr) \
	pci_alloc_consistent(hr->dev_softc, size, bus_addr_ptr)

#define	hr_free_coherent(hr, size, vaddr, bus_addr) \
	pci_free_consistent(hr->dev_softc, size, vaddr, bus_addr)

#define	hr_map_single(hr, buf, size, direction) \
	pci_map_single(hr->dev_softc, buf, size, direction)

#define	hr_unmap_single(hr, busaddr, size, direction) \
	pci_unmap_single(hr->dev_softc, busaddr, size, direction)

#define	hr_map_sg(hr, sg_list, num_sg, direction) \
	pci_map_sg(hr->dev_softc, sg_list, num_sg, direction)

#define	hr_unmap_sg(hr, sg_list, num_sg, direction) \
	pci_unmap_sg(hr->dev_softc, sg_list, num_sg, direction)

#else

static __inline void *hr_alloc_coherent(struct hr_softc *hr,
					bus_size_t size, bus_addr_t *baddrp);

/*
 * At least in 2.2.14, malloc is a slab allocator so all
 * allocations are aligned.  We assume for these kernel versions
 * that all allocations will be bellow 4Gig, physically contiguous,
 * and accessible via DMA by the controller.
 */
static __inline void *
hr_alloc_coherent(struct hr_softc *hr, bus_size_t size, bus_addr_t *baddrp)
{
	void *vaddr;

	vaddr = kmalloc(size, GFP_ATOMIC);
	if (vaddr != NULL)
		*baddrp = virt_to_bus(vaddr);
	return (vaddr);
}

#define	hr_free_coherent(hr, size, vaddr, bus_addr) \
	free(vaddr, M_DEVBUF)

#define hr_map_sg(pdev, sg_list, nseg, direction) (nseg)
#define hr_unmap_sg(pdev, sg_list, nseg, direction)
#define hr_map_single(pdev, buffer, bufflen, direction) \
	(VIRT_TO_BUS(buffer))
#define hr_unmap_single(pdev, buffer, buflen, direction)
#define sg_dma_address(sg) (VIRT_TO_BUS((sg)->address))
#define sg_dma_len(sg) ((sg)->length)
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)

#define hr_set_dma_mask(hr, mask) dma_set_mask(&hr->dev_softc->dev, mask)

#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,3)

/*
 * Device softc is NULL for EISA devices.
 */
#define hr_set_dma_mask(hr, mask) 			\
	((hr)->dev_softc == NULL ? 0 : pci_set_dma_mask(hr->dev_softc, mask))

#else
/*
 * Device softc is NULL for EISA devices.
 * Always "return" 0 for success.
 */
#define hr_set_dma_mask(hr, mask)			\
    (((hr)->dev_softc == NULL)				\
     ? 0						\
     : (((hr)->dev_softc->dma_mask = mask) && 0))
#endif

/**************************** Host Template Macros ****************************/
#define HR_BIOS_PARAM_PLATFORM \
	(defined(__i386__))

#if defined CONFIG_HIGHIO
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,10)
/* Assume RedHat Distribution with its different HIGHIO conventions. */
#define	HR_TEMPLATE_DMA_SETTINGS()	\
	.can_dma_32		= 1,	\
	.single_sg_okay		= 1,
#else
#define	HR_TEMPLATE_DMA_SETTINGS()	\
	.highmem_io		= 1,
#endif
#else
#define	HR_TEMPLATE_DMA_SETTINGS()
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,7)
#define	HR_TEMPLATE_MAX_SECTORS(sectors) \
	.max_sectors		= (sectors),
#else
#define	HR_TEMPLATE_MAX_SECTORS(sectors)
#endif

/***************************** OS Utility Wrappers ****************************/
static __inline void hr_delay(long);
static __inline void
hr_delay(long usec)
{
	/*
	 * udelay on Linux can have problems for
	 * multi-millisecond waits.  Wait at most
	 * 1024us per call.
	 */
	while (usec > 0) {
		udelay(usec % 1024);
		usec -= 1024;
	}
}

/********************************* Misc Macros ********************************/
#define	roundup(x, y)   ((((x)+((y)-1))/(y))*(y))
#define	powerof2(x)	((((x)-1)&(x))==0)

/****************************** Front End Queues ******************************/
/*
 * Data structure used to cast the Linux struct scsi_cmnd to something
 * that allows us to use the queue macros.  The linux structure has
 * plenty of space to hold the links fields as required by the queue
 * macros, but the queue macors require them to have the correct type.
 */
struct hr_cmd_internal {
	/* Area owned by the Linux scsi layer. */
	uint8_t	private[offsetof(struct scsi_cmnd, SCp.Status)];
	union {
		STAILQ_ENTRY(hr_cmd)	ste;
		TAILQ_ENTRY(hr_cmd)	tqe;
	} links;
	uint32_t			end;
};

struct hr_cmd {
	union {
		struct hr_cmd_internal	icmd;
		struct scsi_cmnd	scsi_cmd;
	} un;
};

#define hcmd_icmd(cmd) ((cmd)->un.icmd)
#define hcmd_scsi_cmd(cmd) ((cmd)->un.scsi_cmd)
#define hcmd_links un.icmd.links

/*************************** Device Data Structures ***************************/
#ifdef CDA_FOR_SATA
#define HR_MAX_OSD	136
#else
#define HR_MAX_OSD	156
#endif
#define HR_MAX_RSVD_OSD	8
#define HR_MAX_QUEUE	(HR_MAX_OSD - HR_MAX_RSVD_OSD)	

#ifdef	SCSI_MAX_PHYS_SEGMENTS
#define OSM_SG_MAX	SCSI_MAX_PHYS_SEGMENTS
#else
#define OSM_SG_MAX	128
#endif

#ifndef SG_MAX
#define SG_MAX	128
#endif

typedef enum {
	HR_EH_NONE		= 0x00,
	HR_EH_INITIATED		= 0x01,
	HR_EH_IN_PROGRESS	= 0x02,
	HR_EH_TIMEDOUT		= 0x04,
	HR_EH_DONE		= 0x08,
	HR_EH_ABORT_REQ		= 0x10,
	HR_EH_LU_RESET_REQ	= 0x20,
	HR_EH_DEV_RESET_REQ	= 0x40
} hr_eh_state;

#define HR_EH_LEVEL_MASK	0xF0

typedef enum {
	HR_EH_SUCCEED,
	HR_EH_FAILED
} hr_eh_status;

typedef enum {
	HR_IOREQ_TIMEDOUT	= 0x01,
	HR_IOREQ_TIMER_ACTIVE	= 0x02,
	HR_IOREQ_ACTIVE		= 0x04,	/* command is in CDA or lower layers */
	HR_IOREQ_RESET_LU	= 0x10
} hr_ioreq_flags;

/* 
 * IO Request Descriptor. 
 * OSD definitely has to be at the end as we don't
 * what cda is going to do our io_req ptr while we are not looking.
 */
struct hr_io_request {
	TAILQ_ENTRY(hr_io_request) 	links;
	TAILQ_ENTRY(hr_io_request) 	 timedout_links;
	struct hr_softc			*pAdapter;
	struct hr_linux_device		*dev;
	struct hr_cmd			*hcmd;

	/* Error recovery state and status info. */
	hr_ioreq_flags			 flags;
	hr_eh_state			 eh_state;
	hr_eh_status			 eh_status;
	hr_eh_post_t			*eh_post;
	struct timer_list 		eh_timeout;

	/* Embedded SG Lists. */
	SGTable                 	SGPool;
    	SGElement               	SGPoolElements[OSM_SG_MAX];
	OSD                     	OSD;
};

/*
 * A per core System Device structure used to deal with some error recovery
 * scenarios that the Linux mid-layer code just doesn't know how to
 * handle.  This structure is used in-lieu of a SysDevice in the pStorageTable.
 * This is allowed since the storage table is only used by the OSM.  This also
 * avoids an extra array lookup to get to our per-SysDevice OSM state.
 */
TAILQ_HEAD(hr_busyq, hr_cmd);
typedef enum {
	HR_DEV_UNCONFIGURED	  = 0x0001,
	HR_DEV_ON_RUN_LIST	  = 0x0002,
	HR_DEV_SLAVE_CONFIGURED   = 0x0004,
	HR_DEV_DESTROY_WAS_ACTIVE = 0x0008,
	HR_DEV_ONLINE	          = 0x0010,
	HR_DEV_OFFLINE	          = 0x0020,
	HR_DEV_IN_RECOVERY	  = 0x0040,
	HR_DEV_HOT_REMOVED	  = 0x0080,
	HR_DEV_DPC_ACTIVE	  = 0x0100,
	HR_DEV_TIMER_ACTIVE	  = 0x0200
} hr_linux_dev_flags;

struct hr_linux_device {
	TAILQ_ENTRY(hr_linux_device) 	links;

	struct hr_softc			*softc;
	/*
	 * Device Handle given from CDP.
	 */
	void				*handle;

	/*
	 * MemberIndex given from CDP.
	 */
	UniqueName			CDP_UniqueName;
	/*
	 * Queue of deferred commands in Linux
	 * Scsi_Cmnd format.
	 */
	struct hr_busyq			busyq;

	/*
	 * A positive count indicates that this
	 * device's queue is halted.
	 */
	u_int				qfrozen;
	
	struct timer_list	 	timer;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,41)	
	struct tq_struct	 	taskq;
#else
	struct work_struct	 	workq;
#endif	
	/*
	 * Cumulative command counter.
	 */
	u_long				commands_issued;
	int				active;

	hr_linux_dev_flags		flags;

	/*
	 * Link back to Linux SCSI device structure.
	 * May be NULL if we have not received OS
	 * initiated I/O yet.
	 */
	Scsi_Device	       		*scsi_device;

	/* device number as known to OS */
	int				rq_dev;

	uint16_t			incr_timeout;
};

/* Disk Info struct for supprting IOCTL_GET_DISK_INFO */
struct hr_disk_info {
	int32_t cnum;		/* controller num */
	int32_t bus;		/* bus num */
	int32_t target;		/* target ID */
	int32_t lun;		/* lun num */
	int8_t name[16];	/* special filename in "/dev" */
};

struct hr_linux_device	*hr_alloc_ldev(struct hr_softc *hr, void *handle);
void			hr_free_ldev(struct hr_softc *hr,
				     struct hr_linux_device *dev);

/*********************** Per-Controller Data Structures ***********************/
/*
 * Register mapping resources owned by our softc.
 */
struct hr_resource
{
	uint8_t			 *res_membase;
	u_long			 res_barbase;
	u_int			 res_len;
	int			 res_type;
	int			 res_bar;
	TAILQ_ENTRY(hr_resource) res_links;
};
TAILQ_HEAD(hr_res_tailq, hr_resource);

typedef enum {
	HR_RESUME_RUNQ			 = 0x01,
	HR_PAUSE_PKT_RECEIVED	 	 = 0x02,
	HR_RESOURCE_SHORTAGE		 = 0x04,
	HR_IOCTL_IN_PROGRESS		 = 0x08,
	HR_CORE_INITIALIZED		 = 0x10,
	HR_IRQ_ALLOCATED		 = 0x20,
	HR_TEARDOWN_IN_PROGRESS	 	 = 0x40,
	HR_SUSPEND_IO			 = 0x80
} hr_softc_flags;

struct dma_mem_region {
	
	SLIST_ENTRY(dma_mem_region)  links;
	uint8_t			*begin;
	uint8_t			*end;
	bus_addr_t		 physical;
	/* divide allocation into 32 pieces */
	uint32_t		 used;
};

struct hr_shutdown_info {
	spinlock_t		*lock;
	struct	semaphore 	shutdown_sem;
	u_int		  	shutdown_cnt;
	void			*pContext;
};

struct _ioctl_data_buf {
	struct	_AthenaPkt	AP;
	uint8_t			pbuf[0x2000];
	OSD			OSD;
};

typedef enum {
	HR_RECOVERY_SHUTDOWN	= 0x01
} hr_platform_flags;

struct hr_platform_data {
	/* Error Recovery Thread Support. */
	pid_t			ehandler_pid;
	struct semaphore	ehandler_sem;
	struct semaphore	ehandler_ending_sem;
	struct semaphore	eh_sem;
	struct semaphore	ioctl_sem;

	/* Tasklet support. */
	struct tasklet_struct	runq_tasklet;
	struct tasklet_struct	unblock_tasklet;

	hr_platform_flags	flags;
};

TAILQ_HEAD(hr_softc_tailq, hr_softc);
struct hr_softc
{
	/* Controller instance per pci function. */
	void				*pAdapterDevExt;

	/* Embedded CDA engine properties. */
	CDA_ENGINE_PROPERTIES   	cep;
	struct hr_linux_device		*device_table[OSM_MAX_CHANNEL]
						     [OSM_MAX_TARGET_PER_PATH]
						     [OSM_MAX_LUN];
	spinlock_t			spin_lock;
	hr_dev_softc_t			dev_softc;
	struct	_ioctl_data_buf		*pioctl_data_buf;
	CDA_OS_MEMORY_REGIONS   	os_mem_region;
	bus_addr_t			ioctl_data_busaddr;
	const char			*unit_name;
	const char			*hw_name;
	SLIST_HEAD(, dma_mem_region)	 dma_mem_regions;
	struct Scsi_Host		*host;
	struct hr_shutdown_info	shutdown_info;
	const struct pci_device_id	*pci_entry;
	u_int				qfrozen;
	u_int				max_cmds_per_lun;
	u_int				can_queue;
	u_int				max_sectors;
	u_int				max_scsi_id;
	u_int				max_lun;
	u_int				unit;
	u_int				adapter_scsi_id;
	struct hr_res_tailq		hr_resources;
	u_char				dma64_addr;
	bus_addr_t			hw_dma_mask;
	hr_softc_flags			flags;
	uint8_t				DriverState;
	u_int				pauseTCount;
	TAILQ_HEAD(, hr_linux_device)	device_runq;
	TAILQ_HEAD(, hr_io_request)	io_request_freeq;
	TAILQ_HEAD(, hr_io_request)	io_request_busyq;	
	TAILQ_HEAD(, hr_io_request)	io_request_timedoutq;	
	TAILQ_ENTRY(hr_softc)		links;

	struct hr_platform_data		*platform_data;
};

/******************************** Globals *************************************/
extern int hr_units;
extern struct hr_softc_tailq hr_tailq;
extern int hr_driver_unloading;
extern struct semaphore	hr_watchdog_interlock;

/******************************** Locking *************************************/
/* Lock protecting internal data structures */
static __inline void hr_lockinit(struct hr_softc *);
static __inline void hr_lock(struct hr_softc *, unsigned long *flags);
static __inline void hr_unlock(struct hr_softc *, unsigned long *flags);

/* Lock acquisition and release of the above lock in midlayer entry points. */
static __inline void hr_midlayer_entrypoint_lock(struct hr_softc *);
static __inline void hr_midlayer_entrypoint_unlock(struct hr_softc *);

/* Lock protecting Core shutdown notifications */
extern spinlock_t hr_shutdown_spinlock;
static __inline void hr_shutdown_lockinit(void);
static __inline void hr_shutdown_lock(struct hr_shutdown_info *sinfo,
				      unsigned long *flags);
static __inline void hr_shutdown_unlock(struct hr_shutdown_info *sinfo,
				      unsigned long *flags);

static __inline void
hr_lockinit(struct hr_softc *hr)
{
	spin_lock_init(&hr->spin_lock);
}

static __inline void
hr_lock(struct hr_softc *hr, unsigned long *flags)
{
	spin_lock_irqsave(&hr->spin_lock, *flags);
}

static __inline void
hr_unlock(struct hr_softc *hr, unsigned long *flags)
{
	spin_unlock_irqrestore(&hr->spin_lock, *flags);
}

static __inline void
hr_midlayer_entrypoint_lock(struct hr_softc *hr)
{
	/*
	 * In 2.5.X and some 2.4.X versions, the midlayer takes our
	 * lock just before calling us, so we avoid locking again.
	 * For other kernel versions, the io_request_lock is taken
	 * just before our entry point is called.  In this case, we
	 * trade the io_request_lock for our per-softc lock.
	 */
#if (HR_SCSI_HAS_HOST_LOCK == 0)
	spin_unlock(&io_request_lock);
	spin_lock(&hr->spin_lock);
#endif
}

static __inline void
hr_midlayer_entrypoint_unlock(struct hr_softc *hr)
{
#if (HR_SCSI_HAS_HOST_LOCK == 0)
	spin_unlock(&hr->spin_lock);
	spin_lock(&io_request_lock);
#endif
}

static __inline void
hr_shutdown_lockinit()
{
	spin_lock_init(&hr_shutdown_spinlock);
}

static __inline void
hr_shutdown_lock(struct hr_shutdown_info *sinfo, unsigned long *flags)
{
	if (sinfo->lock != NULL)
		spin_lock_irqsave(sinfo->lock, *flags);
}

static __inline void
hr_shutdown_unlock(struct hr_shutdown_info *sinfo, unsigned long *flags)
{
	if (sinfo->lock != NULL)
		spin_unlock_irqrestore(sinfo->lock, *flags);
}

/****************************** Timer Facilities ******************************/
typedef void hr_callback_t (u_long);  
void hr_linux_midlayer_timeout(struct scsi_cmnd *);

#define hr_timer_init init_timer
#define hr_timer_stop del_timer_sync
static __inline void hr_timer_reset(hr_timer_t *timer, uint32_t timeout,
				    hr_callback_t *func, void *arg);

static __inline void
hr_timer_reset(hr_timer_t *timer, uint32_t timeout,
	       hr_callback_t *func, void *arg)
{

	del_timer(timer);
	timer->data = (u_long)arg;
	timer->expires = jiffies + timeout;
	timer->function = (hr_callback_t*)func;
	add_timer(timer);
}

/****************************** KObject Wrappers ******************************/
#define	hr_dev_to_pci_dev(dev)		(dev)
#define	hr_pci_dev_to_dev(pci)		(pci)

#define	hr_pci_dev(hr)			hr_dev_to_pci_dev((hr)->dev_softc)

/******************************** PCI Routines ********************************/
static __inline uint32_t hr_pci_read_config(hr_dev_softc_t dev,
					     uint32_t reg, int width);
static __inline void hr_pci_write_config(hr_dev_softc_t dev,
					  uint32_t reg, uint32_t value,
					  int width);
static __inline int hr_get_pci_function(hr_dev_softc_t);
static __inline int hr_get_pci_slot(hr_dev_softc_t);
static __inline int hr_get_pci_bus(hr_dev_softc_t);

static __inline uint32_t
hr_pci_read_config(hr_dev_softc_t dev, uint32_t reg, int width)
{
	struct pci_dev *pci;

	pci = hr_dev_to_pci_dev(dev);
	switch (width) {
	case 1:
	{
		uint8_t retval;

		pci_read_config_byte(pci, reg, &retval);
		return (retval);
	}
	case 2:
	{
		uint16_t retval;
		pci_read_config_word(pci, reg, &retval);
		return (retval);
	}
	case 4:
	{
		uint32_t retval;
		pci_read_config_dword(pci, reg, &retval);
		return (retval);
	}
	default:
		panic("hr_pci_read_config: Read size too big");
		/* NOTREACHED */
		return (0);
	}
}

static __inline void
hr_pci_write_config(hr_dev_softc_t dev, uint32_t reg, uint32_t value, int width)
{
	struct pci_dev *pci;

	pci = hr_dev_to_pci_dev(dev);
	switch (width) {
	case 1:
		pci_write_config_byte(pci, reg, value);
		break;
	case 2:
		pci_write_config_word(pci, reg, value);
		break;
	case 4:
		pci_write_config_dword(pci, reg, value);
		break;
	default:
		panic("hr_pci_write_config: Write size too big");
		/* NOTREACHED */
	}
}

static __inline int
hr_get_pci_function(hr_dev_softc_t dev)
{
	struct pci_dev *pci;

	pci = hr_dev_to_pci_dev(dev);
	return (PCI_FUNC(pci->devfn));
}

static __inline int
hr_get_pci_slot(hr_dev_softc_t dev)
{
	struct pci_dev *pci;

	pci = hr_dev_to_pci_dev(dev);
	return (PCI_SLOT(pci->devfn));
}

static __inline int
hr_get_pci_bus(hr_dev_softc_t dev)
{
	struct pci_dev *pci;

	pci = hr_dev_to_pci_dev(dev);
	return (pci->bus->number);
}

/***************************** Large Disk Handling ****************************/
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static __inline int hr_sector_div(u_long capacity, int heads, int sectors);

static __inline int
hr_sector_div(u_long capacity, int heads, int sectors)
{
	return (capacity / (heads * sectors));
}
#else
static __inline int hr_sector_div(sector_t capacity, int heads, int sectors);

static __inline int
hr_sector_div(sector_t capacity, int heads, int sectors)
{
	/* ugly, ugly sector_div calling convention.. */
	sector_div(capacity, (heads * sectors));
	return (int)capacity;
}
#endif

/**************************** Magic SysReq Support ****************************/
#include <linux/sysrq.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
typedef void hr_sysrq_handler_t (int, struct pt_regs *, struct kbd_struct *,
				  struct tty_struct *);
#else
typedef void hr_sysrq_handler_t (int, struct pt_regs *, struct tty_struct *);
#endif

#ifdef CONFIG_MAGIC_SYSRQ
#define	hr_sysrq_key_op sysrq_key_op
#else
struct hr_sysrq_key_op {
	hr_sysrq_handler_t *handler;
	char *help_msg;
	char *action_msg;
};
#endif

hr_sysrq_handler_t	hr_sysrq_handler;
int			hr_install_sysrq(struct hr_sysrq_key_op *);
void			hr_remove_sysrq(int key,
					 struct hr_sysrq_key_op *key_op);

/**************************** SCSI Data Structures ****************************/
struct scsi_sense_data
{
	uint8_t error_code;
#define	SSD_ERRCODE			0x7F
#define		SSD_CURRENT_ERROR	0x70
#define		SSD_DEFERRED_ERROR	0x71
#define	SSD_ERRCODE_VALID       0x80
	uint8_t segment;  
	uint8_t flags;
#define	SSD_KEY				0x0F
#define		SSD_KEY_NO_SENSE	0x00
#define		SSD_KEY_RECOVERED_ERROR	0x01
#define		SSD_KEY_NOT_READY	0x02
#define		SSD_KEY_MEDIUM_ERROR	0x03
#define		SSD_KEY_HARDWARE_ERROR	0x04
#define		SSD_KEY_ILLEGAL_REQUEST	0x05
#define		SSD_KEY_UNIT_ATTENTION	0x06
#define		SSD_KEY_DATA_PROTECT	0x07
#define		SSD_KEY_BLANK_CHECK	0x08
#define		SSD_KEY_Vendor_Specific	0x09
#define		SSD_KEY_COPY_ABORTED	0x0a
#define		SSD_KEY_ABORTED_COMMAND	0x0b
#define		SSD_KEY_EQUAL		0x0c
#define		SSD_KEY_VOLUME_OVERFLOW	0x0d
#define		SSD_KEY_MISCOMPARE	0x0e
#define		SSD_KEY_RESERVED	0x0f
#define	SSD_ILI		0x20
#define	SSD_EOM		0x40
#define	SSD_FILEMARK	0x80
	uint8_t info[4];
	uint8_t extra_len;
	uint8_t cmd_spec_info[4];
	uint8_t add_sense_code;
	uint8_t add_sense_code_qual;
	uint8_t fru;
	uint8_t sense_key_spec[3];
#define	SSD_SCS_VALID	0x80
#define	SSD_FIELDPTR_CMD	0x40
#define	SSD_BITPTR_VALID	0x08
#define	SSD_BITPTR_VALUE	0x07
#define	SSD_MIN_SIZE 18
	uint8_t extra_bytes[14];
#define	SSD_FULL_SIZE sizeof(struct scsi_sense_data)
};

/************************** Prototypes definitions ****************************/
void 	hr_handle_tmf_req(struct hr_softc *hr, POSD pOSD, u_int event, 
			  u_short dev_flag);
void	hr_abort_ioreq_done(POSD pOSD);
void	hr_update_dev_status(struct hr_softc *hr, void *pDevHandle,
			     u_short member_idx, u_int evt);
void	hr_rp_shutdown_complete(POSD pOSD);
int	GetPTLFromLDTree(pLDTree pTarget, struct hr_disk_info *pdiskinfo,
			 void *pADE);
int	GetSysDeviceAccess(void *pIo_Handle);

#endif /*_HR_H */
