/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:ioctl.h
	 
File Description
Defines for ADAPTEC specific IOCtl command.

***********************************************************************

History :

04/14/2005: 18:30  SKP
Change the CDA internal rebuild watermark from 5% to 1%.
New code: #define CHECKPOINT_INTERVAL 1

03/11/2003: 10:50  Farrukh
Unroll last changes (IOCTL code changes) because it will cause incompatibility with old 
iomanager. Will revist this when required.

02/24/2003: 15:30  Farrukh
Modified IOCTL_ABORT and IOCTL_PAUSE_UNPAUSE_IO code because they were
overlapping with SCSI command.

04/13/2001: 12:04 	Manjunath
Added CHECKPOINT_INTERVAL, X percent when we should check point.

04/05/2001: 11:50 	Manjunath
IOCtl_DATA_Buffer size is made 2048. We need to have sufficient memory
for MAX_EVENTS also.

04/05/2001: 11:30 AM		Madhuresh
Moved the IOCTL_DATA_BUFFER definition from nt.h to here to expose it
to iomgr kind of application.

03/16/2001 : 15:00 NAR20038  Manjunath
new ioctls IOCTL_READ_RESERVED_SECTORS and IOCTL_WRITE_RESERVED_SECTORS 
added.

01/31/2001 : 10:33 PM   NAR20038  Manjunath
Added few cache related #defines so that IOMgr would compile. Later,
we can remove them once we clean up IOMgr.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Removed IOCTL_ZERO_UNIT
2. Removed IOCTL_GET_CACHE_STATISTICST
3. Removed IOCTL_GET_PHYS_STATISTICS
4. IOCTL_GET_CACHE_CAPABILITY
5. IOCTL_SET_CACHE_CAPABILITY 
6. IOCTL_CACHE_ON_OFF 
7. CACHE_ON



***********************************************************************/
#ifndef IOCTL_INCLUDED
#define IOCTL_INCLUDED

#include	"sysdef.h"

//
// ioctl buffer is malloced and its pointer is saved for all IOCTL calls
// from iomgr to transfer their data to this area in memory. This area
// will be for sure single scatter/gather.
//
// RESERVED_SECTORS_BYTES has been sector aligned and defined in confpage.h
// So the size of ioctl data buffer must become dependent on that definition.
// Note we have moved the IOCTL_DATA_BUFFER definition from nt.h to here.
//
#define		IOCTL_DATA_BUFFER       2048

#define IOCTL_TEST_UNIT_READY		0x00
#define IOCTL_WAIT_CHANGE			0x03
#define IOCTL_FORMAT				0x04
#define IOCTL_INQUIRY				0x12
#define IOCTL_RESERVE_DEVICE		0x16
#define IOCTL_ABORT					0x17   
/* SCSI START/STOP command */
#define IOCTL_START_STOP_UNIT		0x1B 	/* stop bit is set or cleared */
/* define byte 4 of CDB */
#define START_BIT					0x01    /* bit 0 set => start I/O */
	      		     	        			/* bit 0 cleared => stop I/O */


#define IOCTL_VERIFY				0x2F



/* Arbitarily have selected values, need to be changed later */
#define IOCTL_FLUSH_IO				0x21
#define IOCTL_BUS_DEVICE_RESET 	0x22					 

#define IOCTL_READ_CAPACITY		0x25

/* These are used to read/write reserved sector on the disk.
 * _WRITE_DEVICE is used as a special case to update controller
 * (Athena) code image or BIOS image.
 */
#define IOCTL_READ_DEVICE			0x28
#define IOCTL_WRITE_DEVICE			0x2A

/* ioctls for read/write reserved sectors */
#define	IOCTL_READ_RESERVED_SECTORS  0x98
#define	IOCTL_WRITE_RESERVED_SECTORS 0x99
			  
#define IOCTL_CATEGORY_STATUS		0x4D			  
#define IOCTL_GET_EVENT_ERROR_LOG	0x00	/* Page Code 0 */
#define IOCTL_GET_IO_STATISTICS		0x01	/* Page Code 1 */
#define IOCTL_COPY_EVENT_ERROR_LOG	0x02	/* Page 2, do not clear log */

#define IOCTL_GET_DISK_INFO	0xEE		/* linux get special char: /dev/sda ...etc */

#define IOCTL_CONFIGURE_SINGLE_DRIVE 0x4A
	#define CONFIGURE_DRIVE 		 0x00
	#define UNCONFIGURE_DRIVE		 0x01

#define IOCTL_DRIVE_CACHE_SETTINGS	   0x4C
#define IOCTL_DISABLE_WRITE_CACHE      0x00
#define IOCTL_ENABLE_WRITE_CACHE 	   0x01
#define IOCTL_NOCHANGE_IN_WRITE_CACHE  0x02

#define IOCTL_MODE_SELECT			0x55
#define IOCTL_MODE_SENSE			0x5A


/*6 Byte mode select and mode sense command code */
#define IOCTL_MODE_SELECT_6  		0x15
#define IOCTL_MODE_SENSE_6			0x1A


/* Define byte 5 of CDB bit 0 as IMMEDIATE_RTN bit.
 * If this bit is set, the control will immediately
 * return to the caller or the IO Manager before the IO
 * really get excecuted.
 */
#define IMMEDIATE_RTN			0x01

#define IOCTL_CATEGORY_REDUNDANCY_OUT  	0xBB
#define IOCTL_SERVICE_ARRAY_DELETE			0x02
#define IOCTL_SERVICE_RECREATE_DEVICE  	0x04
#define IOCTL_SERVICE_RECREATE_RANGE  		0x03

#define IOCTL_SERVICE_VERIFY_DEVICE    	0x05
#define IOCTL_SERVICE_VERIFY_RANGE    		0x06
/* define byte 2 of CDB */
#define VERIFY_LOG_BIT				0x01	/* bit 0 set => Log miscompare */
											/* bit 0 cleared => Do not log */
#define VERIFY_FIX_BIT				0x02    /* bit 1 set => Fix miscompare */
											/* bit 1 cleared => Do not fix */
#define VERIFY_NOFIX_NOLOG			0x00	/* Neither fix the parity
						   						nor even log it */
#define VERIFY_NOFIX_LOG			0x01	/* Do not fix the parity, but
											   log the location where
											   data miscompare is found. */
#define VERIFY_FIX_NOLOG			0x02	/* Fix the parity, but do not log */
#define VERIFY_FIX_LOG				0x03	/* Fix and log the parity location */

#define IOCTL_SERVICE_CONVERT_ARRAY    	0x18	   
/* define byte 10 of CDB */
#define PARITY_DISABLE				0x01 	/* bit 0 set => parity disabled */
											/* bit 0 cleared => parity enabled */
#define IOCTL_CATEGORY_PRIORITY	0xA4
#define IOCTL_SERVICE_SET_PRIORITY 0x19

					    
#define IOCTL_DEFINE_ARRAY			0xBC
					    
#define IOCTL_CATEGORY_SPARE_OUT	0xBD                  
#define IOCTL_SERVICE_CREATE_SPARE	0x01 
#define IOCTL_SERVICE_DELETE_SPARE	0x02
#define IOCTL_SERVICE_CHANGE_DRIVE 0x00
	      				    
#define IOCTL_CATEGORY_VOLUME_OUT	0xBF
#define IOCTL_SERVICE_ZERO_DEVICE	0x00
#define IOCTL_SERVICE_ZERO_RANGE	0x01
#define LOGICAL_DRIVE_QUICK_INIT	0x01	/* merged from DELL SR1 ROME branch */


/* Array Initialize (implies Unblock), Uninitialize (implies block IO),
 * and Block/Unblock ioctls usage is based on the targeted device. 
 * In case of array initialize/uninitialize
 * LDTree points to the intended array device, and
 * for Block/Unblock, LDTree points to the intended member
 * of the array whose I/Os has to be blocked/unblocked 
 * depending on the value of the UNBLOCK_BIT.
 */	     	    	  			   
#define IOCTL_BLOCK_UNBLOCK_IO		0xA0 	/* Block bit is set or cleared */
/* define byte 4 of CDB */
#define UNBLOCK_BIT				0x01    /* bit 0 set => unblock I/O */
	      		     	        			/* bit 0 cleared => block I/O */
#define CDBLen(x,y)  (x<<8)+y

/* Mark a member of an array or singles to failed state.
 * This is used when a bad status is returned by the driver
 * after a disk read/write request (usually read/write reserved
 * sector) completes unsuccessfully. Then IO manager issues
 * this request so that the driver can mark the device state as failed.
 */
#define IOCTL_FAIL_DRIVE			0xA1
#define IOCTL_RESCAN				0xA2      
#define IOCTL_REACTIVATE_ARRAY		0xA3
#define IOCTL_PAUSE_UNPAUSE_IO		0xA5
#define UNPAUSE_BIT				0x01    /* bit 0 set => unpause I/O */
#define IOCTL_START_STOP_IO		0xA6

/* This bit indicates whether to release resource.*/
/* If this bit and CACHE_ON bit(implies enabling cache), both bit is set */
/* the driver will reallocate cache blocks for the */
/* targeted sysdevice. */
/* If this bit is set and CACHE_ON bit is cleared(implies */
/* disabling cache), then the driver will deallocate cache */
/* blocks and return them to the cache pool. */
#define RESOURCE_MGT		0x02	

#define IOCTL_UPDATE_BOOTABLE_FLAG	0xA8	//@Maram
//#define IOCTL_RESET_IO_STATISTICS	0xA8	/* Currently not used */


/* Use scsisns.h file for error code definitions returned by the driver. */


typedef struct _ChangeDriveBlock 	/* structure size is 28 (0x1C) bytes */
{
/* 0x0000 */	LDTree				CB_Npath;	/* path of new drive being swapped in */
/* 0x0014 */	ULONG				CB_NstartLBA;	/* start LBA of drive being swapped in */
/* 0x0018 */	ULONG				CB_NendLBA;	/* end LBA of drive being swapped in */
} ChangeDriveBlock, DPTR pChangeDriveBlock;

typedef struct _DefineArrayBlock 	/* structure size is 900 (0x384) bytes */
{
/* 0x0000 */    USHORT				DA_cMembers;
/* 0x0002 */	 USHORT				Padding;
/* 0x0004 */    ChangeDriveBlock	DA_SysDevicePath[MAXARRAYMEMBERS];
} DefineArrayBlock, DPTR pDefineArrayBlock;

/*
 *  SKP 04/13/2005
 *  Changed the rebuild progress watermark from 5% to 1%.
 */
#define CHECKPOINT_INTERVAL  1
/* check point at every 5 percent completion */
/*
#define CHECKPOINT_INTERVAL 5
*/

//#define MAX_FUNC_SIZE		64
#define MAX_FUNC_SIZE		32

#ifdef MATH_16BIT
#define HOST_BUF_SIZE		Multiply32(MAX_FUNC_SIZE,BYTES_PER_SECTOR)
#else
#define HOST_BUF_SIZE		(MAX_FUNC_SIZE * BYTES_PER_SECTOR)
#endif

/* Included these definitions again so that IOMgr would compile.
 * But, these IOCtls won't be supported in the driver. In future
 * we can remove it when we remove references to these in IOMgr 
 * also.
 */
/*******************/ 
#define IOCTL_REZERO_UNIT			0x01
#define IOCTL_GET_CACHE_STATISTICS	0x03	/* Page Code 3 */ 
#define IOCTL_GET_PHYS_STATISTICS	0x04	/* Page Code 4 */ 
#define IOCTL_GET_CACHE_CAPABILITY 0x05	      	  	    	 	    
#define IOCTL_SET_CACHE_CAPABILITY 0x06	      	  	    	 	    
#define IOCTL_CACHE_ON_OFF		0xA7
/* define byte 4 of CDB */
#define CACHE_ON				0x01    /* bit 0 set => data cache */
										/* is on, bit 0 clear => data */
										/* cache is off.	*/

/* IOCTL codes that overlapped with SCSI code are mapped to new codes here.
 * This is necessory to keep the originally defined code unchanged for backward
 * compatibility of IOManager with the driver. 
 */
#define IOCTL_ABORT_MAPPED				0xD7
#define IOCTL_PAUSE_UNPAUSE_IO_MAPPED	0xF5
#define IOCTL_CATEGORY_STATUS_MAPPED	0xE2			  

#endif
