/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

#ifndef MDTDDF_INCLUDED
#define MDTDDF_INCLUDED

//============================================================================
//
// CONDITIONAL COMPILATION FLAGS
//
//============================================================================

//
//	DBG_TESTING	-	0 =	for normal driver build
//					1 = for debug testing with DDF Test App.
//					
#define	DBG_TESTING		0

/***************************************************************************** 
*
*							H E A D E R  F I L E
*
*									F O R
*	
*				Adaptec Dell's RAID Configuration Disk Data Format (DDF)
*
*
*	This header file is designed according to the specification as defined
*	in the "Maping Adaptec's CDA MetaData to Dell's RAID Configuration Disk
*	Data Format (DDF)" document (see Reference #1) that provides
*	implementation choices specific to Adaptec. You should also
*	consult the Dell's RAID Configuration Disk Data Format Specification.
*	(see Reference #2) for general DDF information.
*
*	NOTES FOR DEVELOPERS:
*
*	o DDF STRUCTURES ALIGNMENT:
*	  -	Each section within the DDF is aligned on a sector boundary.
*	  -	Major structures are aligned on 8 bytes boundary.
*
*	o GLOBAL/LOCAL CONTEXT:
*	  -	Sections with global context are stored on all physical disks.
*	  -	Sections with local context are stored only on relevant physical
*		disks.
*
*	o STRUCTURE SIZES:
*	  -	The size of the Physical Drive Records, Virtual Drive Records and
*		Configuration Records sections are affected by Adaptec implementation
*		limits chosen for Physical Drives (PD), Virtual Drives (VD), and
*		maximum number of PDs per VD.
*
*	o SIZE OF THE DDFs:
*		We have decided to compact the DDF area to create a smaller DDF
*		size by eliminating the reserved sectors in between each DDF section.
*		As a result, the overall size of the DDF area is reduced to
*		12 sectors instead of 4,096 sectors. This allows us to maintain the
*		entire DDF MetaData in memory and read/write the DDF sectors with
*		a single read/write command.
*
*
*	REFERENCES:
*
*	1.	Mapping Adaptec's CDA MetaData to Dell's RAID Configuration Disk Data
*		Format (DDF), Shing-Kin Pun, Revision 0.07, January 13, 2004.
*
*	2.	Dell Enterprise Systems, DDF RAID Configuration Disk Data Format Spec
*		Rev. 2.0.0, June 17, 2003
*
*****************************************************************************/



//============================================================================
//
//	REVISION HISTORY
//
//		Revision History should be entered in the following format, with the
//		most recent history on the top of the list.
//
//		MM/DD/YYYY : HH:MM	LAST_NAME
//		DESCRIPTION OF CHANGES
//
//============================================================================

/*

03/25/2005 : 15:30	Shing-Kin Pun
The following changes are for Multi-level RAID support. To turn on Multi-level
RAID support, set the Conditional Compilation Flag (MULTI_LEVEL_RAID) to TRUE
in sysdef.h.
This file is backward compatible to previous versions of mdtddf.h.
1.  Updated copyright and misc. comments.
2.  Added new PD_Types.
3.  Added new Error Codes.
4.  Added #defines for Primary & Secondary RAID Levels.
5.  Added Reserved field to align the VS_Log Structure.

03/16/2005 : 17:00  Shing-Kin Pun
1.  Redefined u64_CachePoliciesParam to uc_CachePoliciesParam[8] in VD_CFG_REC.
2.  Added #defines and defintions for DDF Cache Policies and Parameters.
3.  Misc. comments cleanup.

04/26/2004 : 19:30	Shing-Kin Pun
1.  Changed calculation of MAX_SAE_INTERNAL to adjust automatically according to
    the size of the VD_CFG_REC.

02/10/2004 : 13:50	Shing-Kin Pun
1.  Changes for compatibility with OTC:
    mdtddf.h: Changed DDF_REV to "02.00.00".

01/12/2004 : 10:21	Shing-Kin Pun
1. The CONFIG_RECORDS section has been expanded to include 2 additional blank
sectors. The size of this section has been changed from 2 sectors to 4 sectors.

2. The VD_CFG_REC and SA_CFG_REC sections are no longer 2 individual sections
within CONFIG_RECORDS. They are now mutually exclusive and are sharing the 
same LBA offset 5 from the Primary_DDF_Header. Either VD_CFG_REC or SA_CFG_REC
will exist inside the CONFIG_RECORDS section, but not both at the same time.

3. The total size of the DDF, including the Anchor_DDF_Header, has been 
increased from 11 sectors to 12 sectors.

12/10/2003 : 23:40 Shing-Kin Pun
o Moved Data Type U64 from typedefs.h into mdtddf.h.
o Removed references to typedefs.h.
o Changed Data Types U32, U16 and U8 to ULONG, USHORT and UCHAR.
o Changed variables prefix u32, u16 and u8 to ul, us, and uc.

10/16/2003 : 15:00  Shing-Kin
Added new Error Codes.

10/16/2003 : 15:00  Shing-Kin
Clean up compilation errors & warnings.



*/



//============================================================================
//
//	INCLUDE FILES
//
//============================================================================

#if	(DBG_TESTING)
#else
#include	"sysdef.h"
#endif

//============================================================================
//
//	DATA TYPES DEFINITIONS
//
//============================================================================

typedef	struct	_U64
{
/* Bytes 0-3 */	ULONG		ul_Low;
/* Bytes 4-7 */	ULONG		ul_High;
} U64,	DPTR PU64;

#define	MDT_STATUS		USHORT


//============================================================================
//
//	DDF DEFINES
//
//	These are Adaptec Specific DDF Implementation choices.
//
//============================================================================

#if (DBG_TESTING)
#define	MAX_DEV_MEMBERS			15
#define	DISK_BYTES_PER_BLOCK	512
#endif

#define	BLK_SIZE		(USHORT)DISK_BYTES_PER_BLOCK


//
//	MAX_PDE_INTERNAL -	Max number of PDEs (Physical Drive Entries)
//						supported internally in the PD_REC structure.
//						This number is selected to align the 
//						PD_REC structure on sector boundary.
//						Thus, the algorithm for calculating
//						MAX_PDE_INTERNAL = 7 + 8(N-1)
//						where,	N is the number of sectors to be allocated
//								for the PD_REC structure.
//						Example:
//								 N		MAX_PDE_INTERNAL
//								---		----------------
//								 1			 7
//								 2			15
//								 3			23
//								 4			31
//								 5			39
//								 6			47
//								 7			55
//								 8			63
//
//
//	MAX_PDE			 -	Max number of PDEs (Physical Drive Entries)
//						supported for this implementation.
//						It is set to equal to MAX_DEV_MEMBERS in SYSDEF.H.
//
#define MAX_PDE_INTERNAL	(USHORT)15

#define MAX_PDE				(USHORT)MAX_DEV_MEMBERS

//
//	MAX_VDE_INTERNAL -	Max number of VDEs (Virtual Drive Entries)
//						supported internally in the VD_REC structure.
//						This number is selected to align the 
//						VD_REC structure on sector boundary.
//						Thus, the algorithm for calculating
//						MAX_VDE_INTERNAL = 7 + 8(N-1)
//						where,	N is the number of sectors to be alocated
//								for the VD_REC structure.
//						Example:
//								 N		MAX_VDE_INTERNAL
//								---		----------------
//								 1			 7
//								 2			15
//
//	MAX_VDE			 -	Max number of VDEs (Virtual Drive Entries)
//						supported for this implementation.
//
#define MAX_VDE_INTERNAL	(USHORT)7

#define MAX_VDE				(USHORT)7
//#define MAX_VDE				(USHORT)2


#define MAX_MBE				(USHORT)61			// Max number of
												//	 Mapped Block Entries

#define	MAX_PARTITIONS		(USHORT)1			// Max number of Partitions

#define	VD_CONFIGURED		(USHORT)1			// Number of Virtual Drives
												//	Configured.
												// It is set to 1 for this
												//	implementation.

#define	SIZEOF_GUID		20						// Size of DDF_GUILD field.
#define	SIZEOF_DDF_REV	8						// Size of DDF_Rev field.

#define	SIZEOF_VD_NAME	16						// Size of VD_Name field

#define	DRIVER_DDF_REV          "02.00.00"


#define	SIZEOF_VSLOG_REV        8				// Size of DDF_Rev field.
#define	DRIVER_VSLOG_REV        "01.00.00"


//============================================================================
//
//	DDF HEADER STRUCTURE
//
//		This DDF_Hdr Structure is used by Anchor, Primary and Secondary DDFs.
//
//============================================================================

//	TBD - Comments

typedef	struct _DDF_HDR
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	UCHAR			uc_DDF_HdrGUID [SIZEOF_GUID];
/* 028 0x001C */	UCHAR			uc_Reserved_0 [4];
/* 032 0x0020 */	UCHAR			uc_DDF_Rev [SIZEOF_DDF_REV];
/* 040 0x0028 */	ULONG			ul_SequenceNum;
/* 044 0x002C */	ULONG			ul_TimeStamp;
/* 048 0x0030 */	UCHAR			uc_OpenFlag;
/* 049 0x0031 */	UCHAR			uc_ForeignFlag;
/* 050 0x0032 */	UCHAR			uc_DiskGrouping;
/* 051 0x0033 */	UCHAR			uc_Reserved_1 [13];
/* 064 0x0040 */	UCHAR			uc_HdrExt [32];
/* 096 0x0060 */	U64				u64_PrimaryHdrLBA;
/* 104 0x0068 */	U64				u64_SecondaryHdrLBA;
/* 112 0x0070 */	UCHAR			uc_HdrType;
/* 113 0x0071 */	UCHAR			uc_Reserved_2 [3];
/* 116 0x0074 */	ULONG			ul_WorkSpaceLength;
/* 120 0x0078 */	U64				u64_WorkSpaceLBA;
/* 128 0x0080 */	USHORT			us_MaxPDEs;
/* 130 0x0082 */	USHORT			us_MaxVDEs;
/* 132 0x0084 */	USHORT			us_MaxPartitions;
/* 134 0x0086 */	USHORT			us_CFG_RecLength;
/* 136 0x0088 */	UCHAR			uc_Reserved_3 [56];
/* 192 0x00C0 */	ULONG			ul_AdapterDataOffset;
/* 196 0x00C4 */	ULONG			ul_AdapterDataLength;
/* 200 0x00C8 */	ULONG			ul_PD_RecOffset;
/* 204 0x00CC */	ULONG			ul_PD_RecLength;
/* 208 0x00D0 */	ULONG			ul_VD_RecOffset;
/* 212 0x00D4 */	ULONG			ul_VD_RecLength;
/* 216 0x00D8 */	ULONG			ul_CFG_RecOffset;
/* 220 0x00DC */	ULONG			ul_CFG_RecLength;
/* 224 0x00E0 */	ULONG			ul_PD_DataOffset;
/* 228 0x00E4 */	ULONG			ul_PD_DataLength;
/* 232 0x00E8 */	ULONG			ul_BBM_LogOffset;
/* 236 0x00EC */	ULONG			ul_BBM_LogLength;
/* 240 0x00F0 */	ULONG			ul_DiagSpaceOffset;
/* 244 0x00F4 */	ULONG			ul_DiagSpaceLength;
/* 248 0x00F8 */	ULONG			ul_VS_LogOffset;
/* 252 0x00FC */	ULONG			ul_VS_LogLength;
/* 256 0x0100 */	UCHAR			uc_Reserved_4 [256];

/* 512 0x0200 */	//... SIZE OF THIS STRUCTURE

}	DDF_HDR, DPTR pDDF_HDR;



//============================================================================
//
//	ADAPTER DATA STRUCTURE
//
//============================================================================

//	TBD - Comments

typedef	struct _ADAPTER_DATA
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	UCHAR			uc_AdapterGUID [SIZEOF_GUID];
/* 028 0x001C */	UCHAR			uc_Reserved_0 [4];
/* 032 0x0020 */	UCHAR			uc_AdapterType [8];
/* 040 0x0028 */	UCHAR			uc_Reserved_1 [24];
/* 064 0x0040 */	UCHAR			uc_AdapterData [448];

/* 512 0x0200 */	//... SIZE OF THIS STRUCTURE

} ADAPTER_DATA, DPTR pADAPTER_DATA;



//============================================================================
//
//	PHYSICAL DRIVE ENTRY ( PDE )
//
//============================================================================

//	TBD - Comments

typedef	struct _PD_ENTRY
{
/* 000 0x0000 */	UCHAR			uc_PD_GUID [SIZEOF_GUID];
/* 020 0x0014 */	UCHAR			uc_Reserved_0 [4];
/* 024 0x0018 */	ULONG			ul_PD_Reference;
/* 028 0x001C */	USHORT			us_PD_Type;
/* 030 0x001E */	USHORT			us_PD_State;
/* 032 0x0020 */	U64				u64_PD_ConfiguredSize;
/* 040 0x0028 */	ULONG			ul_Path_0_Info;
/* 044 0x002C */	ULONG			ul_Path_1_Info;
/* 048 0x0030 */	UCHAR			uc_Reserved_1 [16];

/* 064 0x0040 */	//... SIZE OF THIS STRUCTURE

} PD_ENTRY, DPTR pPD_ENTRY;



//============================================================================
//
//	PHYSICAL DRIVE RECORDS
//
//============================================================================

//	TBD - Comments

typedef	struct _PD_REC
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	USHORT			us_PopulatedPDE;
/* 010 0x000A */	USHORT			us_MaxPDE;
/* 012 0x000C */	UCHAR			uc_Reserved_0 [52];
/* 064 0x0040 */	PD_ENTRY		PDE	[MAX_PDE_INTERNAL];

/*1024 0x0400 */	//... SIZE OF THIS STRUCTURE (FOR MAX_PDE = 15)

} PD_REC, DPTR pPD_REC;



//============================================================================
//
//	VIRTUAL DRIVE ENTRY ( VDE )
//
//============================================================================

//	TBD - Comments

typedef	struct _VD_ENTRY
{
/* 000 0x0000 */	UCHAR			uc_VD_GUID [SIZEOF_GUID];
/* 020 0x0014 */	UCHAR			uc_Reserved_0 [4];
/* 024 0x0018 */	USHORT			us_VD_Number;
/* 026 0x001A */	UCHAR			uc_Reserved_1 [2];
/* 028 0x001C */	ULONG			ul_VD_Type;
/* 032 0x0020 */	UCHAR			uc_VD_State;
/* 033 0x0021 */	UCHAR			uc_VD_InitState;
/* 034 0x0022 */	UCHAR			uc_Reserved_2 [14];
/* 048 0x0030 */	UCHAR			uc_VD_Name [SIZEOF_VD_NAME];

/* 064 0x0040 */	//... SIZE OF THIS STRUCTURE

} VD_ENTRY, DPTR pVD_ENTRY;



//============================================================================
//
//	VIRTUAL DRIVE RECORDS
//
//============================================================================

//	TBD - Comments

typedef	struct _VD_REC
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	USHORT			us_PopulatedVDE;
/* 010 0x000A */	USHORT			us_MaxVDE;
/* 012 0x000C */	UCHAR			uc_Reserved_0 [52];
/* 064 0x0040 */	VD_ENTRY		VDE	[MAX_VDE_INTERNAL];

/* 512 0x0200 */	//... SIZE OF THIS STRUCTURE (FOR MAX_VDE = 7)

} VD_REC, DPTR pVD_REC;



//============================================================================
//
//	VIRTUAL DRIVE CONFIGURATION RECORDS ( VD_CFG_REC )
//
//============================================================================

//	TBD - Comments

typedef	struct _VD_CFG_REC
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	UCHAR			uc_VD_GUID [SIZEOF_GUID];
/* 028 0x001C */	UCHAR			uc_Reserved_0 [4];
/* 032 0x0020 */	ULONG			ul_TimeStamp;
/* 036 0x0024 */	ULONG			ul_SequenceNumber;
/* 040 0x0028 */	UCHAR			uc_Reserved_1 [24];
/* 064 0x0040 */	USHORT			us_PrimaryElementCount;
/* 066 0x0042 */	UCHAR			uc_StripeSize;
/* 067 0x0043 */	UCHAR			uc_PrimaryRaidLevel;
/* 068 0x0044 */	UCHAR			uc_PrimaryRLQ;
/* 069 0x0045 */	UCHAR			uc_SecondaryElementCount;
/* 070 0x0046 */	UCHAR			uc_SecondaryElementSeq;
/* 071 0x0047 */	UCHAR			uc_SecondaryRaidLevel;
/* 072 0x0048 */	U64				u64_VD_PartitionSize;
/* 080 0x0050 */	U64				u64_VD_Size;
/* 088 0x0058 */	UCHAR			uc_Reserved_2 [8];
/* 096 0x0060 */	ULONG			ul_Associated_Spare[8];
/* 128 0x0080 */	UCHAR			uc_CachePoliciesParam[8];
/* 136 0x0088 */	UCHAR			uc_BG_Rate;
/* 137 0x0089 */	UCHAR			uc_Reserved_3 [3];
/* 140 0x008C */	UCHAR			uc_Reserved_4 [52];
/* 192 0x00C0 */	UCHAR			uc_Reserved_5 [192];
/* 384 0x0180 */	UCHAR			uc_V0 [32];
/* 416 0x01A0 */	UCHAR			uc_V1 [32];
/* 448 0x01C0 */	UCHAR			uc_V2 [16];
/* 464 0x01D0 */	UCHAR			uc_V3 [16];
/* 480 0x01E0 */	UCHAR			uc_V4 [32];

//	For MAX_PDE = 15
/* 512 0x0200 */	ULONG			ul_PD_Seq [MAX_PDE_INTERNAL];
/* 572 0x023C */	U64				u64_PD_StartLBA [MAX_PDE_INTERNAL];
/* 692 0x02B4 */	UCHAR			uc_Reserved_6 [332];

/* 1024       */	//... SIZE OF THIS STRUCTURE ( TBD )

} VD_CFG_REC, DPTR pVD_CFG_REC;



//============================================================================
//
//	VENDOR UNIQUE CONFIGURATION RECORDS ( VU_CFG_REC )
//
//============================================================================

//	TBD - Comments

typedef	struct _VU_CFG_REC
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	UCHAR			uc_VD_GUID [SIZEOF_GUID];
/* 028 0x001C */	UCHAR			uc_Reserved_0 [4];	// DELL? - Add this
														// field to align the
														// structure.

/* 032 0x0020 */	//... SIZE OF THIS STRUCTURE

} VU_CFG_REC, DPTR pVU_CFG_REC;



//============================================================================
//
//	SPARE ASSIGNMENT ENTRY ( SAE )
//
//============================================================================
//
//

typedef	struct _SA_ENTRY
{
/* 000 0x0000 */	UCHAR			uc_VD_GUID [SIZEOF_GUID];
/* 020 0x0014 */	UCHAR			uc_Reserved_0 [4];
/* 024 0x0018 */	USHORT			us_SecondaryElementNumber;
/* 026 0x001A */	UCHAR			uc_Reserved_1 [6];

/* 032 0x0020 */	//... SIZE OF THIS STRUCTURE

} SA_ENTRY, DPTR pSA_ENTRY;



//============================================================================
//
//	SPARE ASSIGNMENT CONFIGURATION RECORDS ( SA_CFG_REC )
//
//	*** NOTE *** THIS MUST BE DEFINED AFTER VD_CFG_REC.
//
//============================================================================
//
//	MAX_SAE_INTERNAL -	Max number of SAEs (Spare Assignment Entries)
//						supported internally in the SA_CFG_REC structure.
//
//						This number is depending on the size of the SAE Record
//						which is defined by the DDF SPEC that it MUST be the same
//						as the size of the VDF_Config_Record.
//
//						Thus, the algorithm for calculating
//						MAX_SAE_INTERNAL
//						= (sizeof(VD_CFG_REC)-sizeof(SA_REC_HDR)) / sizeof(SA_ENTRY)
//
//						Example:
//						MAX_SAE_INTERNAL = (1024 - 32) / 32 = 31
//
//	MAX_SAE			 -	Max number of SAEs (Spare Assignment Entries)
//						supported for this implementation.
//

#define MAX_SAE_INTERNAL (USHORT)((sizeof(VD_CFG_REC) -32) / sizeof(SA_ENTRY))
#define MAX_SAE			 (USHORT)1



typedef	struct _SA_CFG_REC
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	ULONG			ul_TimeStamp;
/* 012 0x000C */	ULONG			ul_ChannelAffinity;
/* 016 0x0010 */	UCHAR			uc_Reserved_0 [3];
/* 019 0x0013 */	UCHAR			uc_SpareType;
/* 020 0x0014 */	UCHAR			uc_Reserved_1 [12];
/* 032 0x0020 */	SA_ENTRY		SAE [MAX_SAE_INTERNAL];
/*1024 0x0400 */	//... SIZE OF THIS STRUCTURE (FOR MAX_SAE = 31 )

} SA_CFG_REC, DPTR pSA_CFG_REC;



//============================================================================
//
//	PHYSICAL DRIVE DATA ( PD_DATA )
//
//============================================================================

//	TBD - Comments

typedef	struct _PD_DATA
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0000 */	UCHAR			uc_PD_GUID [SIZEOF_GUID];
/* 028 0x001C */	UCHAR			uc_Reserved_0 [4];
/* 032 0x0020 */	ULONG			ul_PD_Reference;
/* 036 0x0024 */	UCHAR			uc_ForcedRefFlag;
/* 037 0x0025 */	UCHAR			uc_Forced_PD_GUID_Flag;
/* 038 0x0026 */	UCHAR			uc_Reserved_1 [474];

/* 512 0x0200 */	//... SIZE OF THIS STRUCTURE

} PD_DATA, DPTR pPD_DATA;



//============================================================================
//
//	BAD BLOCK MANAGEMENT LOG ( BBM_LOG )
//
//	NOTE:	This structure is not used in the current implementation.
//			It is defined here for completeness ONLY.
//
//============================================================================

//	TBD - Comments

typedef	struct _BBM_LOG
{
/* 000 0x0000 */	ULONG			ul_Signature;
/* 004 0x0004 */	ULONG			ul_CRC;
/* 008 0x0008 */	USHORT			us_PopulatedMBE;
/* 010 0x000A */	USHORT			us_SpareBlockCount;
/* 012 0x000C */	UCHAR			uc_Reserved_0 [4];
/* 016 0x0010 */	U64				u64_1stSpareLBA;
/* 024 0x0018 */	U64				u64_MBE [MAX_MBE];

/* 512 0x0200 */	//... SIZE OF THIS STRUCTURE
					//		= 1 sector =  512	(FOR MAX_MBE =  61)
					//		= 2 sector = 1024	(FOR MAX_MBE = 125)
					//		= 4 sector = 2048	(FOR MAX_MBE = 253)
					//		= 6 sector = 3072	(FOR MAX_MBE = 381)
					//		= 8 sector = 4096	(FOR MAX_MBE = 509)

} BBM_LOG, DPTR pBBM_LOG;



//============================================================================
//
//	VENDOR SPECIFIC LOG (VS_LOG )
//
//============================================================================

//	TBD - Comments

#if SAS
//#define MAX_MEMBER_COUNT 16

typedef  struct _Physical_PTLInfo_Entry
{
            USHORT PathId;
            USHORT TargetId;
            USHORT LUN;
            USHORT Reserved;    /* for future use */
} PHYSICAL_PTLINFO_ENTRY, *pPHYSICAL_PTLINFO_ENTRY; 

typedef struct _Logical_PTLInfo_Entry
{
            USHORT LogicalPathId;
            USHORT LogicalTargetId;
            USHORT LogicalLUN;
            USHORT Reserved;    /* for future use */
} LOGICAL_PTLINFO_ENTRY, *pLOGICAL_PTLINFO_ENTRY; 

typedef struct _PTLList
{
            LOGICAL_PTLINFO_ENTRY     LogicalDevice;
            PHYSICAL_PTLINFO_ENTRY    PhysicalDevice[MAX_DEV_MEMBERS];
}PTLList, *pPTLList;
#endif

typedef	struct _VS_LOG
{
/* 000 0x0000 */	ULONG	ul_Signature;
/* 004 0x0004 */	ULONG	ul_CRC;
/* 008 0x0008 */	UCHAR	uc_LogOwnerID [8];
/* 016 0x0010 */ UCHAR  uc_VSLog_Rev[8];
/* 024 0x0018 */ UCHAR  uc_Reserved_0 [8];

/* 032 0x0020 */	ULONG	SD_CheckpointLba;		// Verify/Rebuild has been
													//	completed till this lba.
/* 036 0x0024 */	ULONG	SD_VerifyDate;			// Used by IOMGR only
/* 040 0x0028 */	ULONG	SD_RecreateDate;		// Used by IOMGR only
/* 044 0x002C */	USHORT	SD_MemberIndex;			// Position of this member
													//	in array
/* 046 0x002E */	UCHAR	SD_PhysDriveIndex;		// To determine boot order
													//	used by IOMGR
/* 047 0x002F */	UCHAR	SD_CheckpointActivity;	// ARRAY_NO_OP
													//	ARRAY_VERIFY_OP
													//	ARRAY_REBUILD_OP
													//	ARRAY_ZERO_OP
/* 048 0x0030 */	UCHAR	SD_SpareCnt;
/* 049 0x0031 */	UCHAR	SH_irocFlags ;			// lsb has VERIFY_WITH_AUTOFIX
/* 050 0x0032 */	UCHAR	LR_RedundancySubstate;	// Substate of Array

/*
 *  SKP 2005-08-18   Part of the fix for ADPmp86388.
 */
/* 051 0x0033 */	UCHAR	RebuildFailedMemberIndex;
                   /* TDE - Start of Path,TargetId, and LUN information area
                     * for configured/RAID device.
                     * Total 256 bytes.
                     */
/* 052 0x0034 */    PTLList PTL_Info;
                    UCHAR   PTL_Reserved[120];

/* 308 0x0134 */	UCHAR	ucReserved_1 [12];		// Reserved, alignment.

                    /*
					 *  SKP 05/20/2005 Added VD_Ref and PD_Ref
					 *                 Used by CDA core only.
					 */
/* 320 0x0140 */	ULONG	ul_VD_Ref;				// SD_Array_ID 
/* 324 0x0144 */	ULONG	ul_PD_Ref[16];			// ArrayMember PD_Reference
/* 340 0x0154 */	UCHAR	ucReserved_2 [44];		// Reserved

/* 384 0x0180 */	UCHAR	uc_VS_LogData [128];	// MUST align to block boundary. 

/* 512 0x0200 */	//... SIZE OF THIS STRUCTURE (for 460 bytes of log data)

} VS_LOG, DPTR pVS_LOG;



//============================================================================
//
//	Define the size of each DDF Section based on its structure
//		size.
//
//		NOTE THAT THIS IS ADAPTEC IMPLEMENTATION SPECIFIC.
//
//		*** IMPORTANT NOTES FOR DEVELOPER ***
//
//		Each structure is and MUST BE setup on a sector boundary. 
//
//============================================================================

//
//	Size (in Bytes) of each DDF Section
//
#define	SIZEOF_DDF_HDR					(USHORT)sizeof( DDF_HDR )
#define	SIZEOF_ADAPTER_DATA				(USHORT)sizeof( ADAPTER_DATA )
#define	SIZEOF_PD_REC					(USHORT)sizeof( PD_REC )
#define	SIZEOF_VD_REC					(USHORT)sizeof( VD_REC )
#define	SIZEOF_VD_CFG_REC				(USHORT)sizeof( VD_CFG_REC )
#define	SIZEOF_VU_CFG_REC				(USHORT)0
#define	SIZEOF_SA_CFG_REC				(USHORT)sizeof( SA_CFG_REC )
#define	SIZEOF_PD_DATA					(USHORT)sizeof( PD_DATA )
#define	SIZEOF_BBM_LOG					(USHORT)0
//#define	SIZEOF_DIAGSPACE			(USHORT)1024
#define	SIZEOF_DIAGSPACE				(USHORT)0
#define	SIZEOF_VS_LOG					(USHORT)sizeof( VS_LOG )

//
//	Size (in Blocks) of Primary DDF	
//
#define	SIZEINBLKS_DDF_HDR				(USHORT)SIZEOF_DDF_HDR		/ BLK_SIZE
#define	SIZEINBLKS_ADAPTER_DATA			(USHORT)SIZEOF_ADAPTER_DATA	/ BLK_SIZE
#define	SIZEINBLKS_PD_REC				(USHORT)SIZEOF_PD_REC		/ BLK_SIZE
#define	SIZEINBLKS_VD_REC				(USHORT)SIZEOF_VD_REC		/ BLK_SIZE
#define	SIZEINBLKS_VD_CFG_REC			(USHORT)SIZEOF_VD_CFG_REC	/ BLK_SIZE
#define	SIZEINBLKS_VU_CFG_REC			(USHORT)0
#define	SIZEINBLKS_SA_CFG_REC			(USHORT)SIZEOF_SA_CFG_REC	/ BLK_SIZE
#define	SIZEINBLKS_PD_DATA				(USHORT)SIZEOF_PD_DATA		/ BLK_SIZE
#define	SIZEINBLKS_BBM_LOG				(USHORT)0
//#define	SIZEINBLKS_DIAGSPACE		(USHORT)SIZEOF_DIAGSPACE	/ BLK_SIZE
#define	SIZEINBLKS_DIAGSPACE			(USHORT)0
#define	SIZEINBLKS_VS_LOG				(USHORT)SIZEOF_VS_LOG		/ BLK_SIZE

//
//	Size (in Blocks) of Primary DDF
//
//	NOTE 1- The CONFIG_RECORD Section consists of
//			either all VD_CFG_RECs,
//			       all VU_CFG_RECs, or
//  		       one of them can be SA_CFG_REC.
//
//	NOTE 2-	SA_CFG_REC is stored only on the relevant Physical Drive.
//
//	NOTE 3-	The number of records stored on a drive
//			= 1 + MAX_PARTITIONS (defined in the DDF Header).
//
//	NOTE 4-	Since we have only 1 partition for this implementation,
//			the number of records we have for the
//				CONFIG_RECORD section = 1+1 = 2.
//			Hence we added SIZEINBLKS_VD_CFG_REC twice in the following
//			calculation.
//			CAUTION - THAT IS NOT A DUPLICATED LINE.
//
//	NOTE 5- Since OTC had implemented VD_CFG_REC and SA_CFG_REC as an
//			UNION of the 2 record types, we will only count the length
//			of the VD_CFG_REC.
//			Hence. SIZEINBLKS_SA_CFG_REC is not included in the following
//			calculation.
//

#define	SIZEINBLKS_PRIMARY_DDF			(USHORT)(					\
										SIZEINBLKS_DDF_HDR		+	\
										SIZEINBLKS_ADAPTER_DATA	+	\
										SIZEINBLKS_PD_REC		+	\
										SIZEINBLKS_VD_REC		+	\
										SIZEINBLKS_VD_CFG_REC	+	\
										SIZEINBLKS_VD_CFG_REC	+	\
										SIZEINBLKS_PD_DATA		+	\
										SIZEINBLKS_DIAGSPACE	+	\
										SIZEINBLKS_VS_LOG )

//
//	Size (in Blocks) of all DDF MetaData,
//		including the Anchor DDF Header.
//
#define	SIZEINBLKS_ALL_DDF_METADATA		(USHORT)(					\
										SIZEINBLKS_PRIMARY_DDF	+	\
										SIZEINBLKS_DDF_HDR )

//
//	Size (in Bytes) of Primary DDF
//
#define	SIZEOF_PRIMARY_DDF				(ULONG)(					\
										SIZEINBLKS_PRIMARY_DDF	*	\
										BLK_SIZE )

//
//	Size (in Bytes) of all DDF MetaData, including the Anchor DDF Header.
//
#define	SIZEOF_ALL_DDF_METADATA			(ULONG)(						\
										SIZEINBLKS_ALL_DDF_METADATA	*	\
										BLK_SIZE )


//
//	Size (in Blocks) of Reserved Area for DDF, as recommended by DELL.
//		This includes all DDF Metadata + Reserved areas.
//
#define	SIZEINBLKS_DDF_RESERVED_AREA	256 * 1024          // 128MB resevred for Capistrano and more compatibility
#define	SIZEINBLKS_HZ1_DDF_RESERVED_AREA	(64 * 1024)

//
//	Size (in Blocks) for Workspace
//		For this implementation, allocate 16 MBytes
//
#define SIZEINBLKS_WORKSPACE			(ULONG)32768		// 16M/512




//============================================================================
//
//	DDF MetaData Layout (in LBA) on the Disk
//
//		Macros for 64 bit calculations
//
//============================================================================

#define	DRIVE_CAPACITY			(U64)u64_DriveCapacity
#define	ANCHOR_DDFHDR_LBA 		(U64)(DRIVE_CAPACITY - 1)
#define	PRIMARY_DDFHDR_LBA		(U64)(DRIVE_CAPACITY - SIZEINBLKS_ALL_DDF_METADATA)
//#define	SECONDARY_DDFHDR_LBA

//
//	The Workspace is placed immediately above the Primary DDF.
//
#define	WORKSPACE_LBA			(U64)(PRIMARY_DDFHDR_LBA - SIZEINBLKS_WORKSPACE)



//============================================================================
//
//	Relative LBA Offset of each DDF section from the Primary DDF Header.
//
//============================================================================

#define	PRIMARY_DDFHDR_LBA_OFFSET		(ULONG)0
#define	SECONDARY_DDFHDR_LBA_OFFSET		(ULONG)0

#define	ADAPTER_DATA_LBA_OFFSET			(ULONG)1	
#define	PD_REC_LBA_OFFSET				(ULONG)2
#define	VD_REC_LBA_OFFSET				(ULONG)4
#define	VD_CFG_REC_LBA_OFFSET			(ULONG)5
#define	SA_CFG_REC_LBA_OFFSET			(ULONG)5
#define	PD_DATA_LBA_OFFSET				(ULONG)9
//#define	BBM_LOG_LBA_OFFSET
//#define	DIAGSPACE_LBA_OFFSET
#define	VS_LOG_LBA_OFFSET				(ULONG)10
#define	ANCHOR_DDFHDR_LBA_OFFSET		(ULONG)11


//============================================================================
//
//	Relative Byte Offset of each DDF section within the DDFbuffer in memory.
//
//============================================================================

#define	PRIMARY_DDFHDR_OFFSET		0
#define	SECONDARY_DDFHDR_OFFSET		0

#define	ADAPTER_DATA_OFFSET			(ULONG)(ADAPTER_DATA_LBA_OFFSET	* BLK_SIZE)
#define	PD_REC_OFFSET				(ULONG)(PD_REC_LBA_OFFSET		* BLK_SIZE)
#define	VD_REC_OFFSET				(ULONG)(VD_REC_LBA_OFFSET		* BLK_SIZE)
#define	VD_CFG_REC_OFFSET			(ULONG)(VD_CFG_REC_LBA_OFFSET	* BLK_SIZE)
#define	SA_CFG_REC_OFFSET			(ULONG)(SA_CFG_REC_LBA_OFFSET	* BLK_SIZE)
#define	PD_DATA_OFFSET				(ULONG)(PD_DATA_LBA_OFFSET		* BLK_SIZE)
//#define	BBM_LOG_OFFSET
//#define	DIAGSPACE_OFFSET		(ULONG)(DIAGSPACE_LBA_OFFSET	* BLK_SIZE)
#define	VS_LOG_OFFSET				(ULONG)(VS_LOG_LBA_OFFSET		* BLK_SIZE)

//#define	ANCHOR_DDFHDR_OFFSET	(ULONG)(SIZEOF_PRIMARY_DDF)
#define	ANCHOR_DDFHDR_OFFSET		(ULONG)(ANCHOR_DDFHDR_LBA_OFFSET * BLK_SIZE)


//============================================================================
//
//	DEFINITIONS FOR DDF HEADER TYPE
//
//============================================================================

#define	ANCHOR_DDF_HDR				(UCHAR)0x00
#define	PRIMARY_DDF_HDR				(UCHAR)0x01
#define	SECONDARY_DDF_HDR			(UCHAR)0x02



//============================================================================
//
//	DEFINITIONS DDF SECTIONS SIGNATURE 
//
//============================================================================

#define	DDF_HDR_SIGNATURE			(ULONG)0xDE11DE11	// Anchor, Primary &
														//	Seconary DDF HDR
														//	share the same
														//	DDF HDR signature.
#define	ADAPTER_DATA_SIGNATURE		(ULONG)0xAD111111		
#define	PD_REC_SIGNATURE			(ULONG)0x22222222
#define	VD_REC_SIGNATURE			(ULONG)0xDDDDDDDD
#define	VD_CFG_REC_SIGNATURE		(ULONG)0xEEEEEEEE
#define	VU_CFG_REC_SIGNATURE		(ULONG)0x88888888
#define	SA_CFG_REC_SIGNATURE		(ULONG)0x55555555
#define	PD_DATA_SIGNATURE			(ULONG)0x33333333
#define	BBM_LOG_SIGNATURE			(ULONG)0xABADB10C
#define	VS_LOG_SIGNATURE			(ULONG)0x01DBEEF0



//============================================================================
//
//	DEFINITIONS FOR DDF VIRTUAL DRIVE STATES
//
//============================================================================

//
//	BIT ASSIGNMENT FOR 	uc_VD_STATE
//

#define	VDS_BITMASK_ARRAY_STATES		(UCHAR)0x07	// BIT 0-2
#define	VDS_BITMASK_MORPHING			(UCHAR)0x08	// BIT 3
#define	VDS_BITMASK_DIRTY_CACHE			(UCHAR)0x10	// BIT 4

//
//	BIT 0-2:	ARRAY_STATES
//
#define	VDS_OPTIMAL						(UCHAR)0x00
#define	VDS_DEGRADED					(UCHAR)0x01
#define	VDS_DELETED						(UCHAR)0x02
#define	VDS_MISSING						(UCHAR)0x03
#define	VDS_FAILED						(UCHAR)0x04

//
//	BIT 3:	MORPHING
//
#define VDS_MORPHING					(UCHAR)0x08

//
//	BIT 4:	DIRTY_CACHE
//
#define VDS_DIRTY_CACHE					(UCHAR)0x10


//============================================================================
//
//	DEFINITIONS FOR DDF VIRTUAL DRIVE INIT STATES
//
//============================================================================

//		Bit 1-0		= FULL_INITIALIZED = 02
//		Bit 5-2		= RESERVED = 0
//		Bit 7-6		= USER_READ_WRITE_ALLOWED = 00

#define	VDIS_USER_READ_WRITE_ALLOWED	(UCHAR)0x00
#define	VDIS_UNINITIALIZED				(UCHAR)0x00
#define	VDIS_FULL_INITIALIZED			(UCHAR)0x02


//============================================================================
//
//	DEFINITIONS FOR DDF PD_TYPE
//
//============================================================================

#define PD_TYPE_NOT_PARTICIPATING_IN_VD	(USHORT)0x0000	// Bit 0
#define	PD_TYPE_PARTICIPATING_IN_VD		(USHORT)0x0002	// Bit 1
#define	PD_TYPE_GLOBAL_SPARE			(USHORT)0x0004	// Bit 2
#define	PD_TYPE_DEDICATED_SPARE			(USHORT)0x0008	// Bit 3
#define	PD_TYPE_FOREIGN_DISK			(USHORT)0x0010	// Bit 4
#define	PD_TYPE_PASSTHRU_LEGACY_DISK	(uSHORT)0x0020	// Bit 5

//... Bit 6 to 15 are Reserved.


//============================================================================
//
//	DEFINITIONS FOR DDF PD_STATE
//
//	This is the state of the physical drive as part of one or more Virtual
//	Drive.
//		Bit0:	Online
//		Bit1:	Failed 
//		Bit2:	Rebuild (drive rebuilding for a failed drive)
//		Bit3:	Transition (as in replacing a member by copy)
//		Bit4:	PFA (SMART error alerts)
//		Bit5:	Unrecovered ReadErrors
//		Bit6:	Missing (a drive can be failed-missing)
//		Bit7:	Reserved
//
//		Bits 8-15: Reserved
//		All reserved bits cleared.
//
//============================================================================

#define	DDF_PD_STATE_ON_LINE			(USHORT)0x0001
#define	DDF_PD_STATE_FAILED				(USHORT)0x0002
#define	DDF_PD_STATE_REBUILD			(USHORT)0x0004
#define	DDF_PD_STATE_TRANSITION			(USHORT)0x0008
#define	DDF_PD_STATE_PFA				(USHORT)0x0010
#define	DDF_PD_STATE_UNRECOVERED_RD_ERR	(USHORT)0x0020
#define	DDF_PD_STATE_MISSING			(USHORT)0x0040
#define DDF_PD_STATE_FAILED_MISSING		(USHORT)0x0042




//============================================================================
//
//	DEFINITIONS FOR DDF PHYS_TYPE
//	(REVISIT - This should be defined in CDA)
//
//============================================================================

#define	SCSI_DEVICES					(USHORT)0
#define	SAS_DEVICES						(USHORT)1
#define	SATA_DEVICES					(USHORT)2



//============================================================================
//
//	DEFINITIONS FOR DDF Spare_Type
//
//============================================================================

#define	DDF_GLOBAL_SPARE				(UCHAR)0	//	Bit 0
#define	DDF_DEDICATED_SPARE				(UCHAR)1	//	Bit 0


//============================================================================
//
//	DEFINITIONS FOR DDF PRIMARY RAID LEVELS
//
//  NOTE 1 - Not all definitions provided here are supported. They are
//           included ONLY for completeness.
//  
//============================================================================

#define PRL_RAID_0					(UCHAR)0x00
#define PRL_RAID_1					(UCHAR)0x01
#define PRL_RAID_3					(UCHAR)0x03
#define PRL_RAID_4					(UCHAR)0x04
#define PRL_RAID_5					(UCHAR)0x05
#define PRL_RAID_1E					(UCHAR)0x11
#define PRL_JBOD					(UCHAR)0x0F
#define PRL_CONCATENATION			(UCHAR)0x1F
#define PRL_RAID_5E					(UCHAR)0x15
#define PRL_RAID_5EE				(UCHAR)0x25
#define PRL_RAID_6					(UCHAR)0x16



//============================================================================
//
//	DEFINITIONS FOR DDF SECONDARY RAID LEVELS
//
//============================================================================

#define SRL_STRIPED					(UCHAR)0x00
#define SRL_MIRRORED				(UCHAR)0x01
#define SRL_CONCATENATED			(UCHAR)0x02
#define SRL_SPANNED					(UCHAR)0x03



//============================================================================
//
//	DEFINITIONS FOR DDF CACHE POLICIES & PARAMETERS
//
//  Bit usage for uc_CachePoliciesParam[0] in VD_CFG_REC
//
//  !!! IMPORTANT NOTES FOR DEVELOPERS !!!
//
//  (1) If bit 7 = 1, bits 0-3 SHOULD be left clear and ignored and
//                    bits 4-6 SHOULD still be used.
//					  Bytes 1-7, i.e. uc_CachePoliciesParam[1-7] are
//                               vendor specific.
//
//  (2) During Vendor migration, if bit 7 is found set, then the user SHOULD
//      be notified and bit 7 SHOULD be cleared;
//      bits 0-3 SHOULD be set to the controller default and
//      bits 4-6 SHOULD be preserved.
//
//  (3) Bit 4, 5 and 6 are Master Enable/Disable settings for Cache Parameters.
//
//============================================================================

#define DDF_VENDOR_SPEC_CACHING     (UCHAR)0x80 // Bit7, VENDOR_SPECIFIC_CACHING
                                            //  0 = NO_VENDOR_SPEC_CACHING
                                            //  1 = VENDOR_SPEC_CACHING

#define DDF_READ_CACHE_ENABLE       (UCHAR)0x40 // Bit6
                                            //  0 = DISABLE_READ_CACHE
                                            //  1 = ENABLE_READ_CACHE

#define DDF_WRITE_CACHE_ENABLE      (UCHAR)0x20 // Bit5
                                            //  0 = DISABLE_WRITE_CACHE
                                            //  1 = ENABLE_WRITE_CACHE

#define DDF_WRITE_CACHE_BATT_LOW    (UCHAR)0x10 // Bit4, if Battery is low
                                            //  0 = DISABLE_WC_BATTERY_LOW
                                            //  1 = ENABLE_WC_BATTERY_LOW

#define DDF_READ_AHEAD_ADAPTIVE     (UCHAR)0x08 // Bit3, ignored if Bit2 = 0
                                            //  0 = ALWAYS_READ_AHEAD
                                            //  1 = ADAPTIVE_READ_AHEAD

#define DDF_READ_AHEAD_ENABLE       (UCHAR)0x04 // Bit2
                                            //  0 = DISABLE_READ_AHEAD
                                            //  1 = ENABLE_READ_AHEAD

#define DDF_WRITE_BACK_ADAPTIVE     (UCHAR)0x02 // Bit1, ignored if Bit0 = 0
                                            //  0 = ALWAYS_WRITE_BACK
                                            //  1 = ADAPTIVE_WRITE_BACK

#define DDF_WRITE_BACK_ENABLE       (UCHAR)0x01 // Bit0
                                            //  0 = DISABLE_WRITE_BACK
											//      (i.e. WRITE_THRU)
                                            //  1 = ENABLE_WRITE_BACK

#define DDF_DEFAULT_CACHE_PARAM     (UCHAR) (DDF_WRITE_CACHE_ENABLE | DDF_WRITE_BACK_ENABLE);


//============================================================================
//
//	GLOBAL FUNCTION PROTOTYPES
//
//============================================================================

#if	(DBG_TESTING)
#else
MDT_STATUS
MDT_Initialize(
	pADAPTER_DEV_EXT		pAdapterDevExt
);

ULONG
MDT_GetMetaDataSize(
	VOID
);

//ULONG
//MDT_GetMetaDataSize(
//);


VOID
MDT_ReadWriteMetaData (
	pIODescriptor			pIOD,
	pReservedSectorLayout	pBuffer,
	BOOL					bWaitIOFlag
);

#endif



//============================================================================
//
//	DDF ERROR CODE DEFINITIONS
//
//  NOTE - MDTDDF ERROR Codes defined here MUST MATCH the ASC_MDTDDF_ERR Codes
//         defined in scsisns.h.
//
//============================================================================


#define	NO_ERR								(USHORT)0

#define	MDTDDF_ERR_BASE						(USHORT)0x2000


#define	ERR_MDTDDF_INIT_MEM_ALLOC			(USHORT)(MDTDDF_ERR_BASE + 1)
#define	ERR_MDTDDF_NO_SD_NUM_MEMBERS		(USHORT)(MDTDDF_ERR_BASE + 2)
#define	ERR_MDTDDF_NO_MATCHING_VD_GUID		(USHORT)(MDTDDF_ERR_BASE + 3)
#define	ERR_MDTDDF_NO_MATCHING_PD_REF		(USHORT)(MDTDDF_ERR_BASE + 4)
#define	ERR_MDTDDF_INVALID_PRIMARY_ELEMENT_COUNT (USHORT)(MDTDDF_ERR_BASE + 5)
#define ERR_MDTDDF_INVALID_SECONDARY_ELEMENT_SEQ (USHORT)(MDTDDF_ERR_BASE + 6)
#define	ERR_MDTDDF_INVALID_POPULATED_PDE	(USHORT)(MDTDDF_ERR_BASE + 7)
#define	ERR_MDTDDF_INVALID_PD_REF			(USHORT)(MDTDDF_ERR_BASE + 8)
#define	ERR_MDTDDF_INVALID_PD_SEQ			(USHORT)(MDTDDF_ERR_BASE + 9)
#define	ERR_MDTDDF_INVALID_DEVICE_ID		(USHORT)(MDTDDF_ERR_BASE + 10)
#define	ERR_MDTDDF_INVALID_MEMBER_INDEX		(USHORT)(MDTDDF_ERR_BASE + 11)
#define	ERR_MDTDDF_UNSUPPORTED_RAID_CFG		(USHORT)(MDTDDF_ERR_BASE + 12)
#define	ERR_MDTDDF_PD_REFERENCE_NOT_IN_PDE	(USHORT)(MDTDDF_ERR_BASE + 13)
#define	ERR_MDTDDF_EXCEED_MAX_VDE			(USHORT)(MDTDDF_ERR_BASE + 14)
#define	ERR_MDTDDF_EXCEED_CDA_MAX_RAIDS_PER_ADAPTER	(USHORT)(MDTDDF_ERR_BASE + 15)
#define	ERR_MDTDDF_INVALID_POPULATED_VDE	(USHORT)(MDTDDF_ERR_BASE + 16)
#define	ERR_MDTDDF_VD_NOTFOUND_IN_VDE		(USHORT)(MDTDDF_ERR_BASE + 17)
#define	ERR_MDTDDF_NULL_pPD					(USHORT)(MDTDDF_ERR_BASE + 18)
#define ERR_MDTDDF_INVALID_PRIM_RAID_LEVEL	(USHORT)(MDTDDF_ERR_BASE + 19)

#define ERR_MDTDDF_FOREIGN_DEVICE			(USHORT)(MDTDDF_ERR_BASE + 20)
#define ERR_MDTDDF_INVALID_ANCHOR_HDR_SIGNATURE	 (USHORT)(MDTDDF_ERR_BASE + 21)
#define ERR_MDTDDF_INVALID_PRIMARY_HDR_SIGNATURE (USHORT)(MDTDDF_ERR_BASE + 22)
#define ERR_MDTDDF_INCOMPATIBLE_DDF_REV     (USHORT)(MDTDDF_ERR_BASE +23)
#define ERR_MDTDDF_INCOMPATIBLE_VSLOG_REV   (USHORT)(MDTDDF_ERR_BASE +24)
#define ERR_MDTDDF_UNSUPPORTED_RAID_LEVEL   (USHORT)(MDTDDF_ERR_BASE +25)


#define ERR_MDTDDF_ANCHOR_HDR_CRC			(USHORT)(MDTDDF_ERR_BASE + 30)
#define ERR_MDTDDF_PRIMARY_HDR_CRC			(USHORT)(MDTDDF_ERR_BASE + 31)
#define ERR_MDTDDF_ADAPTER_DATA_CRC			(USHORT)(MDTDDF_ERR_BASE + 32)
#define ERR_MDTDDF_PD_REC_CRC				(USHORT)(MDTDDF_ERR_BASE + 33)
#define ERR_MDTDDF_VD_REC_CRC				(USHORT)(MDTDDF_ERR_BASE + 34)
#define ERR_MDTDDF_VD_CFG_REC_CRC			(USHORT)(MDTDDF_ERR_BASE + 35)
#define ERR_MDTDDF_SA_CFG_REC_CRC			(USHORT)(MDTDDF_ERR_BASE + 36)
#define ERR_MDTDDF_PD_DATA_CRC				(USHORT)(MDTDDF_ERR_BASE + 37)
#define ERR_MDTDDF_VS_LOG_CRC				(USHORT)(MDTDDF_ERR_BASE + 38)


#endif	// MDTDDF_INCLUDED

