/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/* 
 * This file contains OS specific support routines which will be called 
 * by lower layers of CDA.
 */

#ifndef SCO_DDI8
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/blkdev.h>      /* Definition of io_request_lock */
#include <linux/delay.h>    /* Definition of udelay & mdelay */
#include <linux/nmi.h>	    /* For touch_nmi_watchdog() */
#endif // SCO_DDI8

#include "hr.h"
#include "osmcda.h"
#include "randnum.h"

#ifdef SCO_DDI8
VOID
XMemSet(unsigned char *p, int val, int n)
{
	while (n--)
		*p++ = val;
}
#define memset(a, b, c) XMemSet(a, b, c)
#define memcpy(dest, src, cnt) bcopy(src, dest, cnt)
#define virt_to_bus(virt) (DATA_N_PTR)vtop(virt, NULL)
#define kmalloc(size, flag)	kmem_alloc(size, KM_NOSLEEP);
#define PAGE_SIZE	(ptob(1))
typedef uint32_t u32;
#endif // SCO_DDI8

/*
 *  Function:
 *      XBuildOSAdapterDevices
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.  
 *                               
 *  Return Value:  
 *      CDA_SUCCESS     -   if the function completed successfully.
 *                 
 *  Description:       
 *      Get the size of hwi data This function will be called before CDA attempt
 *	to initize the hardware using CHIM. Some OS require this to scan the 
 *	PCI bus completely and locate all such PCI device-functions. 
 */
UINT32
XBuildOSAdapterDevices(PVOID pCDADevExt)
{
	UNREFERENCED_PARAMETER(pCDADevExt);
	return CDA_SUCCESS;
}

/*
 *  Function:
 *      XReturnPhysAddr
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context. 
 *      pVirtualAddr    -   kernel virtual address to be translated. 
 *                               
 *  Return Value:  
 *      PhysAddr        -   Physical Address of the Virtual Address provided.
 *                 
 *  Description:       
 *      Translate a kernel virtual address to a physical address. 
 */
DATA_N_PTR
XReturnPhysAddr(PVOID pAdapterDevExt, UCHAR DPTR pVirtualAddr, PVOID pOSRequest)
{
	struct hr_softc  *hr;
	struct dma_mem_region	*region;
	u_long	offset;

	UNREFERENCED_PARAMETER(pOSRequest);
    

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pAdapterDevExt) {
			break;
		}
	}

	if(hr) {
		SLIST_FOREACH(region, &hr->dma_mem_regions, links) {
			if((pVirtualAddr >= region->begin) &&
					 ( pVirtualAddr < region->end)) {
				offset = pVirtualAddr - region->begin;
				return (DATA_N_PTR)(region->physical + offset);
			}
		}
	}
	
	/* should not be here... will remove in the future when deprecated*/
	return (DATA_N_PTR)virt_to_bus((caddr_t) pVirtualAddr);
}
#ifndef SCO_DDI8
VOID DPTR
XReturnVirtAddr(PVOID pAdapterDevExt, DATA_N_PTR PhysAddr, ULONG nBytes)
{
    	struct hr_softc  *hr;
	struct dma_mem_region	*region;
	u_long	offset;
	uint32_t	memsize;

	UNREFERENCED_PARAMETER(nBytes);

    	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pAdapterDevExt) {
			break;
		}
    	}

	if(hr) {
		SLIST_FOREACH(region, &hr->dma_mem_regions, links) {
			memsize = region->end - region->begin;	
			if((PhysAddr >= region->physical) &&
				( PhysAddr < (region->physical + memsize))) {
				offset = PhysAddr - region->physical;
				return (VOID DPTR)(region->begin + offset);
			}
		}
	}

	/* should not be here... will remove in the future when deprecated*/
	return (VOID DPTR) bus_to_virt((unsigned long) PhysAddr);
}


#endif /* SCO_DDI8 */
/*
 *  Function:
 *      XReturnPhysAddrAndLenght
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context. 
 *      pVirtualAddr    -   kernel virtual address to be translated.
 *      Length          -   length of address to be translated. 
 *                               
 *  Return Value:  
 *      PhysAddr        -   Physical Address of the Virtual Address provided.
 *                 
 *  Description:       
 *      Translate a kernel virtual address to a physical address. 
 */
DATA_N_PTR
XReturnPhysAddrAndLenght(PVOID pCDADevExt, UINT8 DPTR pVirtualAddr,
				UINT32 DPTR Length, PVOID pOSRequest)
{
	UNREFERENCED_PARAMETER(pCDADevExt);
	UNREFERENCED_PARAMETER(pOSRequest);

	/*
	 * have a lot of room to roam here:
	 * actual size is 8k minus some structures.
	 * Be conservative and give it 2k
	 */

 	DPTR Length = 0x800;
       
	return virt_to_bus((caddr_t) pVirtualAddr);
}

#ifdef HR_SATA_BUILD
/*
 * This routine transfers data between dma and user space.
 */
VOID
OSMsgCopyOut(HIM_UEXACT32 bcr, HIM_UEXACT32 padr,
		HIM_UEXACT32 puadr, PVOID ladr, PVOID iob)
{
	void *destaddr = NULL;
	pIODescriptor      pIOD = iob; 

	if (pIOD->bIOD_fIOCtlRequest)
	{
		destaddr = (void *) pIOD->BuffVirtAddr;
	}
	else
	{
    		pOSDescriptor      pOSD = pIOD->pIOD_IOParent;

		if(pOSD)
			destaddr = (void *) pOSD->OSD_pDataBuffer;

	}

	if((pIOD->pIOD_SGTable->SGList[0].SGLBufAddr) && 
		(padr > (HIM_UEXACT32)pIOD->pIOD_SGTable->SGList[0].SGLBufAddr))
	{
		HIM_UEXACT32 offset;
		offset = padr - (HIM_UEXACT32)pIOD->pIOD_SGTable->SGList[0].SGLBufAddr;
		destaddr += offset;
	}

	XMemCpy(destaddr, ladr, bcr);
}


VOID
OSMsgCopyIn(HIM_UEXACT32 bcr, HIM_UEXACT32 padr, 
		HIM_UEXACT32 puadr, PVOID ladr, PVOID iob)
{
    void *srcaddr = NULL;
    pIODescriptor      pIOD = iob; 

	if (pIOD->bIOD_fIOCtlRequest)
	{
		srcaddr = (void *) pIOD->BuffVirtAddr;
	}
	else
	{
    		pOSDescriptor      pOSD = pIOD->pIOD_IOParent;

		if(pOSD)
			srcaddr = (void *) pOSD->OSD_pDataBuffer;

	}

	if((pIOD->pIOD_SGTable->SGList[0].SGLBufAddr) && 
		(padr > (HIM_UEXACT32)pIOD->pIOD_SGTable->SGList[0].SGLBufAddr))
	{
		HIM_UEXACT32 offset;
		offset = padr - (HIM_UEXACT32)pIOD->pIOD_SGTable->SGList[0].SGLBufAddr;
		srcaddr += offset;
	}

    XMemCpy(ladr, srcaddr, bcr);
}
#endif /* HR_SATA_BUILD */

/*
 *  Function:
 *      XMapPTLandDeviceHandle 
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context. 
 *      pCMP            -   pointer to CDA Mapping Parameter struct.
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      This function converts the OSM Channel/Id/Lun and DeviceHandle. 
 */     
BOOL
XMapPTLandDeviceHandle(PVOID pCDADevExt,PCDA_MAPPING_PARAMETERS pCMP)
{
	return 0;
}

#ifndef SCO_DDI8
UINT32 XMemcmp(UINT8 DPTR buf1, UINT8 DPTR buf2, UINT32 length)
{
	return memcmp(buf1, buf2, length);
}
#endif // SCO_DDI8

/*
 *  Function:
 *      XMemZero
 *
 *  Parameters:    
 *      pBuffer         -   pointer to buffer to be zeroed out. 
 *      Bytes           -   no of bytes to be zeroed out.
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      Zeroed out the buffer for amount of n Bytes. 
 */
VOID 
XMemZero(UINT8 DPTR pBuffer, UINT32 Bytes)
{
	memset(pBuffer, 0, Bytes);
}

/*
 *  Function:
 *      XMemCpy
 *
 *  Parameters:    
 *      pDest           -   pointer to Destination buffer. 
 *      pSrc            -   pointer to Source buffer.
 *      Bytes           -   no of bytes to be copied.
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      Copy the Src buffer to Dest buffer for amount of n Bytes. 
 */
VOID	
XMemCpy(UINT8 DPTR pDest, UINT8 DPTR pSrc, UINT32 Bytes)
{
	/* can't trust anybody, check it first */
	if(pDest != NULL)
		memcpy(pDest, pSrc, Bytes);
}                    

/*
 *  Function:
 *      XMicrosecondDelay
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context. 
 *      MicroSeconds    -   # of MicroSeconds to delay.
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      Stall for the duration of MicroSeconds. 
 */
VOID
XMicrosecondDelay(PVOID pCDADevExt, UINT32 microSeconds)
{
#ifdef SCO_DDI8
	drv_usecwait(microSeconds);
#else // SCO_DDI8
	int     i;
	UNREFERENCED_PARAMETER(pCDADevExt);

#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,5)) || \
	(LINUX_VERSION_CODE == KERNEL_VERSION(2,4,21))) && \
	 ( BITS_PER_LONG > 32 )
	touch_nmi_watchdog();
#endif
	i = microSeconds / 1000;

	if (i) {
		mdelay(i);
	} else {
		i = microSeconds % 1000;
	if (i)
		udelay(i);
	}
#endif // SCO_DDI8
}

/*
 *  Function:
 *      XNotify()
 *                 
 *  Description:       
 *      XNotify is called with appropriate event number to report to the OS
 *      about a particular async event. 
 */
void
XNotify(void *pCDADevExt, u_int	event, void *pDeviceHandle, u_short MemberIndex)
{
	struct hr_softc  *hr;
	int		  found;

	found = FALSE;
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			found = TRUE;
			break;
		}
	}
	/* Sanity check. */
	if (found != TRUE)
		return;

	if((hr->DriverState & DRIVER_STATE_UP_AND_RUNNING) == 0)
		return;

	printk("XNotify: Event 0x%x\n", event);

	switch (event) 
	{
		case SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_NEW_SLOT:
			printk("SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_NEW_SLOT:\n");
			hr_update_dev_status(hr, pDeviceHandle, MemberIndex, event);
			break ;
		case SLOTS_LASTSTATE_OLD_DEVICE_REAPPEARED_AT_OLD_SLOT:
			printk("SLOTS_LASTSTATE_OLD_DEVICE_REAPPEARED_AT_OLD_SLOT:\n");
			hr_update_dev_status(hr, pDeviceHandle, MemberIndex, event);
			break ;
		case SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_OLD_SLOT_OR_DEVICE_CHANGED:
			printk(" SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_OLD_SLOT_OR_DEVICE_CHANGED:\n");
			hr_update_dev_status(hr, pDeviceHandle, MemberIndex, event);
			break ;
		case SLOTS_LASTSTATE_OLD_DEVICE_REMOVED_AT_OLD_SLOT:
		printk("SLOTS_LASTSTATE_OLD_DEVICE_REMOVED_AT_OLD_SLOT:\n");
			hr_update_dev_status(hr, pDeviceHandle, MemberIndex, event);
			break ;
		case SLOTS_LASTSTATE_OLD_DEVICE_GOT_OFFLINE_AT_OLD_SLOT:
		printk("SLOTS_LASTSTATE_OLD_DEVICE_GOT_OFFLINE_AT_OLD_SLOT:\n");
			hr_update_dev_status(hr, pDeviceHandle, MemberIndex, event);
			break ;
		case CDA_NOTIFY_EVENT_AD_OFFLINE:
			printk("%s: CDA_NOTIFY_EVENT_AD_OFFLINE\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_DEVICE_OFFLINE:
			printk("%s: CDA_NOTIFY_EVENT_DEVICE_OFFLINE\n",HR_DRIVER_NAME);
			//hr_update_dev_status(hr, pDeviceHandle, MemberIndex,
			//		       event);
			break ;
		case CDA_NOTIFY_EVENT_DEVICE_ONLINE:
			printk("%s: CDA_NOTIFY_EVENT_DEVICE_ONLINE\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_BUS_RESET:
			printk("%s: CDA_NOTIFY_EVENT_BUS_RESET\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_SUSPEND_IO:
			hr->flags |= HR_SUSPEND_IO;
			printk("%s: CDA_NOTIFY_EVENT_SUSPEND_IO\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_RESUME_IO:
			hr->flags &= ~HR_SUSPEND_IO;
			hr->flags |= HR_RESUME_RUNQ;
			printk("%s: CDA_NOTIFY_EVENT_RESUME_IO\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_RAID_STATUS_CHANGE:
			printk("%s: CDA_NOTIFY_EVENT_RAID_STATUS_CHANGE\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_DEVICE_ADDED:
			printk("%s: CDA_NOTIFY_EVENT_DEVICE_ADDED\n",HR_DRIVER_NAME);
			break ;
		case CDA_NOTIFY_EVENT_DEVICE_REMOVED:
			printk("%s: CDA_NOTIFY_EVENT_DEVICE_REMOVED\n",HR_DRIVER_NAME);
			//hr_update_dev_status(hr, pDeviceHandle, MemberIndex,
			//		       event);
			break ;
		case CDA_NOTIFY_EVENT_REPORT_DEVICE:			
			printk("%s: CDA_NOTIFY_EVENT_REPORT_DEVICE\n",HR_DRIVER_NAME);
			break;
		case CDA_NOTIFY_EVENT_ABORT_TASK:			
			printk("%s: CDA_NOTIFY_EVENT_ABORT_TASK\n",HR_DRIVER_NAME);
            		hr_handle_tmf_req(hr, (POSD) pDeviceHandle, event,
					  MemberIndex);
			break;
    		case CDA_NOTIFY_EVENT_RESET_DEVICE_PORT:
			printk("%s: CDA_NOTIFY_EVENT_RESET_DEVICE_PORT: pOSD: %p, Member: %x\n",
					HR_DRIVER_NAME, pDeviceHandle, MemberIndex);
            		hr_handle_tmf_req(hr, (POSD) pDeviceHandle, event,
					  MemberIndex);
			break;
    		case CDA_NOTIFY_EVENT_RESET_OCCURING_ON_ADAPTER:
            		hr_handle_tmf_req(hr, (POSD) pDeviceHandle, event, MemberIndex);
			printk("%s: CDA_NOTIFY_EVENT_RESET_OCCURING_ON_ADAPTER\n",HR_DRIVER_NAME);
			break;
    		case CDA_NOTIFY_EVENT_RESET_OCCURING_ON_LOGICAL_UNIT:
			printk("%s: CDA_NOTIFY_EVENT_RESET_OCCURING_ON_LOGICAL_UNIT\n",HR_DRIVER_NAME);
			break;
    		case CDA_NOTIFY_EVENT_RESET_OCCURING_ON_DEVICE_PORT:
			printk("%s: CDA_NOTIFY_EVENT_RESET_OCCURING_ON_DEVICE_PORT\n",HR_DRIVER_NAME);
			break;
		default:
			printk("XNotify: default\n");
			break;   
	}
}

/*
 *  Function:
 *      XTransferData
 *
 *  Parameters:    
 *      Parameters      -   pointer to CDA DMA parameters. 
 *                               
 *  Return Value:  
 *     
 *                 
 *  Description:       
 *      This function is called to move memory content from one area to 
 *      another. 
 */
UINT32	
XTransferData(CDA_DMA_PARAMETERS  Parameters)
{
	UNREFERENCED_PARAMETER(Parameters);
	return CDA_SUCCESS;
}

/*
 *  Function:
 *      XOSMReadPCIConfigurationByte
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.
 *      ByteOffset      -   position of byte to read from Config Space.
 *                               
 *  Return Value:  
 *      8 bits value of the PCI config space register.
 *                 
 *  Description:       
 *      Read 8 bits of PCI configuration space. 
 */
UINT32 
XOSMReadPCIConfigurationByte(PVOID pCDADevExt, UINT32 ByteOffset)
{
	struct hr_softc  *hr;

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			break;
		}
	}
   
	return (hr_pci_read_config(hr->dev_softc, ByteOffset, 1));
}

/*
 *  Function:
 *      XOSMReadPCIConfigurationWord
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.
 *      ByteOffset      -   position of byte to read from Config Space. 
 *                               
 *  Return Value:  
 *      16 bits value of the PCI config space register.
 *                 
 *  Description:       
 *      Read 16 bits of PCI configuration space. 
 */
UINT32 
XOSMReadPCIConfigurationWord(PVOID pCDADevExt, UINT32 ByteOffset)            
{
	struct hr_softc  *hr;

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			break;
		}
	}

	return (hr_pci_read_config(hr->dev_softc, ByteOffset, 2));
}

/*
 *  Function:
 *      XOSMReadPCIConfigurationDWord
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.
 *      ByteOffset      -   position of byte to read from Config Space. 
 *                               
 *  Return Value:  
 *      32 bits value of the PCI config space register.
 *                 
 *  Description:       
 *      Read 32 bits of PCI configuration space. 
 */
UINT32 
XOSMReadPCIConfigurationDword(PVOID pCDADevExt, UINT32 ByteOffset)
{
	struct hr_softc  *hr;
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			break;
		}
	}

	return (hr_pci_read_config(hr->dev_softc, ByteOffset, 4));
} 

/*
 *  Function:
 *      XOSMWritePCIConfigurationByte 
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.
 *      ByteOffset      -   position of byte to write to Config Space.
 *      Value           -   8-bit value to be written. 
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      Write 8 bits value to the PCI configuration space register. 
 */
VOID 
XOSMWritePCIConfigurationByte(PVOID pCDADevExt, UINT32 ByteOffset, UINT8 Value)
{
	struct hr_softc  *hr;

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			break;
		}
	}

	hr_pci_write_config(hr->dev_softc, ByteOffset, Value, 1);
}    
    
/*
 *  Function:
 *      XOSMWritePCIConfigurationWord 
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.
 *      ByteOffset      -   position of byte to write to Config Space.
 *      Value           -   16 bits value to be written. 
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      Write 16 bits value to the PCI configuration space register. 
 */
VOID 
XOSMWritePCIConfigurationWord(PVOID pCDADevExt, UINT32 ByteOffset, UINT16 Value)
{
	struct hr_softc  *hr;

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			break;
		}
	}

	hr_pci_write_config(hr->dev_softc, ByteOffset, Value, 2);
} 

/*
 *  Function:
 *      XOSMWritePCIConfigurationDWord 
 *
 *  Parameters:    
 *      pCDADevExt      -   pointer to CDA device context.
 *      ByteOffset      -   position of byte to write to Config Space.
 *      Value           -   32 bits value to be written. 
 *                               
 *  Return Value:  
 *      None.
 *                 
 *  Description:       
 *      Write 32 bits value to the PCI configuration space register. 
 */
VOID 
XOSMWritePCIConfigurationDword(PVOID pCDADevExt,
		 UINT32 ByteOffset, UINT32 Value)
{
	struct hr_softc  *hr;

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pCDADevExt) {
			break;
		}
	}

	hr_pci_write_config(hr->dev_softc, ByteOffset, Value, 4);
}


/***************** Memory allocation helper functions *****************/

/*
 * Allocates n bytes of zero'd memory.
 */
VOID DPTR
XMemCalloc(pADAPTER_DEV_EXT pADE, ULONG	size)
{
	VOID DPTR retVal;

	retVal = XMemAllocContiguous(pADE, size);

	if(retVal == NULL)
		printk("XMemCalloc: NULL, size %d\n",size);

	return (retVal);
}

/*
 * Allocates n bytes of memory on a paragraph boundry. Returns a pointer
 * that is expressed in terms of DPTR types. Allocation is to be
 * in a virtually contiguous buffer and will not be used for DMA operations
 */

VOID DPTR
#ifndef	SCO_DDI8
XMemAlloc(PVOID	pADE, ULONG size)
#else	/* SCO_DDI8 */
XMemAlloc(pADAPTER_DEV_EXT pADE, ULONG size)
#endif	/* SCO_DDI8 */
{
	uint8_t *pMem;

	if(size > 131072) {
		printk("XMemAlloc: Can not alloc mem this size %d\n", size);
		return NULL;
	}

	pMem = XMemAllocContiguous(pADE, size);

	if(pMem != NULL)
		memset(pMem, 0, size);

	return (PVOID) pMem;
	
}

/*
 * Allocates n bytes of memory on a paragraph boundry. Returns a pointer
 * that is expressed in terms of DPTR types. Allocation is to be
 * in a physically contiguous buffer and can be used for DMA
 */
VOID DPTR
XMemAllocContiguous(PVOID pADE, ULONG size)
{
	struct hr_softc		*hr;
	uint8_t			*virtual;
	struct dma_mem_region	*region;
	struct dma_mem_region	*previous;
	struct dma_mem_region	*after;
	dma_addr_t		physical;
	u_long			rounded_size;
	u_long			skip;
	u_long			offset;
	uint32_t		bit;

	if(size > 131072) {
		printk("XMemAllocContig: Can't alloc mem this size %d\n", size);
		return NULL;
	}

     	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == (PVOID) pADE) {
			break;
		}
    	}
	/* Any free entries in the pool? */
	SLIST_FOREACH(region, &hr->dma_mem_regions, links) {
		u_long section;

		if (region->used == (u32)-1L)
			continue;
		section = region->end - region->begin;
		if (section < size)
			continue;

		skip    = HR_SKIP_SECTION(section);
		bit     = 1;
		section = 0;
		virtual = region->begin;
		while (bit && (virtual < region->end)) {
			if (region->used & bit) {
				section = 0;
			} else if ((section += skip) >= size) {
				/* Found a segment */
				for (;;) {
					region->used |= bit;
					if ((section -= skip) <= 0)
						break;
					virtual -= skip;
					bit >>= 1;
				}
    				memset(virtual, 0, size);
				return (virtual);
			}
			bit <<= 1;
			virtual += skip;
		}
	}
	/* Get a new one */
 	region = kmalloc(sizeof(*region), GFP_ATOMIC);
	if (region == NULL)
		return (NULL);

	rounded_size = ((size - 1) | (PAGE_SIZE - 1)) + 1;
#ifdef SCO_DDI8
	virtual = (uint8_t *)kmem_alloc_phys(rounded_size, hr->hr_physreq, &physical, KM_FAIL_OK);
	if (virtual == NULL) {
		kmem_free(region, sizeof(*region));
		return (NULL);
	}
	region->size = rounded_size;
#else // SCO_DDI8
	virtual = (uint8_t *)pci_alloc_consistent(hr->dev_softc,
					  rounded_size, &physical);
	if (virtual == NULL) {
		kfree(region);
		return (NULL);
	}
#endif // SCO_DDI8

	/* Generate the mapping database entry */
	region->begin = virtual;
	region->end = virtual + rounded_size;
	region->physical = physical;
	region->used = (u32)1L;
	bit    = 1;
	skip   = HR_SKIP_SECTION(rounded_size);
	offset = 0;
	while (bit && (offset < size)) {
		region->used |= bit;
		bit <<= 1;
		offset += skip;
	}

	/* Place sorted into the virtual/physical mapping table */
	previous = NULL;
	SLIST_FOREACH(after, &hr->dma_mem_regions, links) {
		if (after->begin >= virtual)
			break;
		previous = after;
	}
	if (previous != NULL)
		SLIST_INSERT_AFTER(previous, region, links);
	else
		SLIST_INSERT_HEAD( &hr->dma_mem_regions, region, links);
    	memset(virtual, 0, size);
	return (virtual);
}

/*
 * Frees up ALL associated mapped memory resources remaining.
 */
VOID
XMemFreeAllContiguous(PVOID pADE)
{
	struct hr_softc 	*hr;
	struct dma_mem_region 	*region;

    	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pADE) {
			break;
		}
    	}

	while ((region = SLIST_FIRST(&hr->dma_mem_regions)) != NULL) {
		SLIST_REMOVE_HEAD(&hr->dma_mem_regions, links);
#ifdef SCO_DDI8
		kmem_free(region->begin, region->size);
		kmem_free(region, sizeof(*region));
#else // SCO_DDI8
		pci_free_consistent(hr->dev_softc, region->end - region->begin,
				    region->begin, region->physical);
		kfree(region);
#endif // SCO_DDI8
	}
}

/*
 * Returns a memory allocation to the operating system.
 */
VOID
XMemFree(UCHAR DPTR pBuf, ULONG	nBytes)
{
	struct hr_softc	      	*hr;
	struct dma_mem_region 	*previous;
	struct dma_mem_region 	*region;
	uint8_t		      	*pEnd;

	pEnd = (uint8_t *)pBuf + nBytes;

	/*
	 * The Core mixes frees of DMA safe memory with
	 * frees of other memory.  Search all of the softcs
	 * to see if this is a DMA safe allocation.
	 */
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		previous = NULL;
		SLIST_FOREACH(region, &hr->dma_mem_regions, links) {
			uint8_t   *start;
			u_long     size;
			u_long     skip;
			uint32_t   bit;

			if ((region->begin > (uint8_t *)pBuf)
			 || (pEnd > region->end)) {
				previous = region;
				continue;
			}

			start = region->begin;
			size  = region->end - region->begin;
			skip  = HR_SKIP_SECTION(size);
			bit   = 1;

			/*
			 * Clear the used bits referencing this
			 * memory
			 */
			while (bit && (start < pEnd)) {
				if ((uint8_t *)pBuf <= start) {
					region->used &= ~bit;
				}
				start += skip;
				bit <<= 1;
			}
			if (region->used == 0) {
				if (previous == NULL)
					SLIST_REMOVE_HEAD(&hr->dma_mem_regions,
							 links);
				else
					SLIST_NEXT(previous, links) =
					    SLIST_NEXT(region, links);
#ifdef SCO_DDI8
				kmem_free(region->begin, region->size);
				kmem_free(region, sizeof(*region));
#else // SCO_DDI8
				pci_free_consistent(hr->dev_softc,
						    region->end - region->begin,
						    region->begin,
						    region->physical);
				kfree(region);
#endif // SCO_DDI8
			}
			return;
		}
	}

	/* Fall Through to generic release */
#ifdef SCO_DDI8
	kmem_free(pBuf, nBytes);
#else // SCO_DDI8
	kfree(pBuf);
#endif // SCO_DDI8
}

USHORT
XGetSGList (PVOID pOSRequest, PVOID psg)
{
	SGElement	*cur_SG;
	uint16_t	cSGTotal = 0;
	POSD		pOSD;

#ifdef CDA_FOR_SATArags2
	pOSD = &((struct hr_io_request *) pOSRequest)->OSD;
#else
	pOSD = (POSD) pOSRequest;
#endif

	cur_SG = &(((SGTable *) pOSD->OSD_pSGTable)->SGList[0]);
	cSGTotal = ((SGTable *) pOSD->OSD_pSGTable)->SGCount;

        memcpy((SGElement *) psg, cur_SG, cSGTotal * sizeof(SGElement));

    	return (cSGTotal);
}


/*
 * FUNCTION: XMemFill
 *		Fill a buffer with the specified pattern. 
 * INPUT:	 UCHAR	*puc_Buf 	Pointer to buffer to be filled.
 *		 UCHAR		uc_Pattern Data pattern to be used.
 *		 USHORT		u16_BufSize Number of bytes to fill.
 * RETURNS: NONE
 */

VOID
XMemFill(UCHAR DPTR puc_Buf, UCHAR uc_Pattern, USHORT us_BufSize)
{
	while( us_BufSize-- )
	{
		DPTR puc_Buf++ = uc_Pattern;
	}
}

/*
 * This is the Maximum exponent supported
 * by the following 2^N conversion functions.
 */
#define	MAX_EXPONENT		15

/*
 * FUNCTION: XPowerOf2	LOCAL
 * This function returns 2^N for a given exponent
 * WARNING: This function returns a 16 bit number, hence, the maximum
 *		value for the exponent N is 15.
 *
 * INPUT:	UCHAR		The exponent (N), No negative numbers !!!
 *
 * OUTPUT: 	USHORT		2^N
 *
 *		Examples:
 *					N		2^N
 *					---		---
 *					0		1
 *					1		2
 *					2		4
 *					3		8
 *					4		16
 *					5		32
 *					6		64
 *					7		128
 *					8	    256
 *					9	    512
 *					10		1024
 *					11		2048
 *					12		4096
 *					13		8192
 *					14		16384
 *					15		32768
 *
 */

USHORT
XPowerOf2(UCHAR	uc_Exponent)
{
	UCHAR	uc_N = (uc_Exponent > MAX_EXPONENT)? MAX_EXPONENT : uc_Exponent;

	return( (USHORT)1 << uc_N );
}

/*
 * FUNCTION: XExponent	LOCAL
 *	This function returns the exponent N of rhe given 2^N number.
 *
 * WARNING:	Caller must ensure that the number is 2^N and
 *			N must be less than or equal to 15.
 *
 *		If a number is not an exact value of 2^N, the algorithm
 *		will truncate the value to the lower 2^N value.
 *
 *		There's no extensive checking within this function.
 * INPUT: 	USHORT		A Number that is 2 to the power N.
 *
 * OUTPUT:
 *		signed char		returns the exponent N
 *
 *	EXAMPLES:
 *					N		2^N
 *					---		---
 *					0		1
 *					1		2
 *					2		4
 *					3		8
 *					4		16
 *					5		32
 *					6		64
 *					7		128
 *					8	    256
 *					9	    512
 *					10		1024
 *					11		2048
 *					11		2048
 *					12		4096
 *					13		8192
 *					14		16384
 *					15		32768
 */

signed char
XExponentOf2(USHORT us_PowerOf2)
{
	signed char	sc_N;

	if( us_PowerOf2 == 0 ) return ( -1 );

	for( sc_N=MAX_EXPONENT; sc_N>=0; sc_N-- )
	{
		if( ( us_PowerOf2 >> sc_N ) == 1 )
			return( sc_N );
	}
	return( -1 );
}


/*
 * FUNCTION:		XRandomNumber
 *
 * DESCRIPTION:	Generate a new Random Number each time when this
 *		 function is called.
 *
 * INPUT:	none
 *
 * RETURN:	ULONG	ul_RandNum	Function returns a 32-bit integer
 *
 * COMMENT:	270,799
 */

ULONG XRandomNumber(ULONG ul_NewSeed)
{
	static	ULONG	ul_RandNum;
	static	ULONG	ul_RandCount = 0;
	static	ULONG	ul_RandSeed = 0;
	ULONG		j;

	if( ( ul_RandCount % MAX_RANDOM_NUM ) == 0 )
	{
	    if( ul_RandSeed != ul_NewSeed )
		{
			ul_RandSeed = ul_NewSeed & (ULONG)0xFFFFFFFF;
			ul_RandNum  = ul_RandSeed;
		}
		//
		//	Generates a new RandSeed
		//
		j = (ul_RandNum) % 32;
       	ul_RandNum = ((ul_RandNum ^ (RANDOM_SEED_MOD >> j)) * RANDOM_SEED_MOD) +
								 ul_RandCount;
		ul_RandCount = 0;
	}

	ul_RandNum ^= (ul_RandNum >> 11);
	ul_RandNum ^= (ul_RandNum << 7) & RAND_MASK1;
	ul_RandNum ^= (ul_RandNum << 15) & RAND_MASK2;
	ul_RandNum ^= (ul_RandNum >> 18);

	ul_RandCount++;

	return( ul_RandNum ); 

}


VOID
XOSMDelay(VOID *osmAdapterContext, ULONG microSeconds) 
{
#ifdef SCO_DDI8
	drv_usecwait(microSeconds);
#else // SCO_DDI8
	int i;
	i = microSeconds / 1000;
   
	if (i) {
		mdelay(i);
	} else {
		i = microSeconds % 1000;
		if (i)
			udelay(i);
	}
#endif // SCO_DDI8
}


UCHAR XGetDriverState(PVOID pADE)
{
	struct hr_softc  *hr;

	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pADE) {
			break;
		}
    	}

	return (hr->DriverState);
}

#ifndef SCO_DDI8
VOID XCoreDebugPrint(UCHAR *s)
{
	printk("**** %s ****\n",s);
}


VOID XCoreOSMDebugPrint(UCHAR *s, DATA_N_PTR param1,DATA_N_PTR param2)
{
	printk("**** %s %lx %lx ****\n",s,param1,param2);
}
#endif // SCO_DDI8


USHORT XGetDefaultITNexusLossTime(void)
{
	return 2;
}


VOID XLogOSError(PVOID pAdptDevExt, int errCode)
{
	
}
