/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:osifprot.h

File Description

Prototype definitions for all routines in the operating system
specific interface file.

***********************************************************************

History :

11/22/2003 Madhuresh
OSMDelay is gone. OSMCDA.H has XOSMDelay now.

11/03/2003 Tam
Remove prototype for XReturnPhysAddr.  Now being prototyped in osmcda.h.

10/31/2003 Madhuresh
Modified XNotifyRescanPauseInProgress.

02/10/2003 : 12:50 NAR20038  Manjunath
Added XMemAllocContiguous() - a macro which translates to XMemAlloc()

05/15/2002:  14:22  BUR11729 Farrukh
pAdapterDevExt is now passesed as a parameter in XFindConfig

Removed defination of XCfgXORIF since it is not used.

NTCrashdetected is moved to ADAPTER_DEV_EXT therefore we need to pass
pAdapterDevExt as parameter in some functions.

pAdapterDevExt is now passesed as a parameter in many functions.
Removed XGetPhysAddr defination since it is not used anywhere. 

02/11/2002 : 15:45 BURN11729 Farrukh
For Multi-controller support added aditional parameter to some routine. 
12/18/2001 : 14:20 NAR20038  Manjunath
Removed proto for XReceiveIO and updated XReturnVirtAddr.

12/06/2001 : 11:20 NAR20038  Manjunath
Cleaned up _JAGUAR lines.

05/21/2001:  12:00  BURN11729 Farrukh
Added prototype XMemoryReductionRequired to isolate NTCrashDetected code
to OS specific files.

03/01/2001 : 13:10 NAR20038  Manjunath
Added prototype GetAdapterDevExt for multichannel support.

02/02/2001 : 02:34 PM   NAR20038  Manjunath
Added XDebugPrint, which can be called from non-osm files for debugging.

Removed prototype XInquirePCIConfiguration
01/24/2001 : 02:59 PM   NAR20038  Manjunath
Removed prototype XInquirePCIConfiguration

12/13/2000 : 11:14AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

Removed prototypes

1. XSetSabreWatchdog
2. XInvokeSabreWatchdog
3. Removed #ifdef _VIKDRVR 
4. XCfgControllerIF

***********************************************************************/
#ifndef OSIFPROT_INCLUDED
#define OSIFPROT_INCLUDED

#include "sysdef.h"
/**********************************************************************

Function Name XNotifyRescanPauseInProgress

Function Description
After issuing rescan we call here to pause the 
controller. At this time we need to make sure that we hold off all the
requests from the operating system by setting PauseInProgress flag.
When rescan completes, we reset said flag.

Input Parameters
	pSysDevice		This should be a sysdevice of a controller.
	PauseFlag		This flag determines to pause or unpause the
					controller. 
					TRUE == Pause the controller
					FALSE == Unpause the controller

Output Parameters
	None

**********************************************************************/
#define	XNotifyRescanPauseInProgress(pSD,PauseFlag)	XNotify(pSD->pAdapterDevExt,PauseFlag?CDA_NOTIFY_EVENT_SUSPEND_IO:CDA_NOTIFY_EVENT_RESUME_IO,0,0);

/**********************************************************************

Function Name XRescanCompleted

Function Description
This is called when Rescan is completed or could not continue because
we are out of resources.

Input Parameters
	pOSDescriptor   pDriverOSD

Output Parameters
	None

**********************************************************************/
#define	XRescanCompleted(pOSD)	RescanCompleted(pOSD) ;
/**********************************************************************

Function Name  XGetAdapterDevices

Function Description
Copy the Adapter Device Setting information to the application.

Called from
	CAIOCTL.C::IOCtlAdapterModeSense() only

Input Parameters
	pRemoteCtlPkt   pRp;
	pSysDevice		pSD;

Output Parameters
	None

**********************************************************************/
#define	XGetAdapterDevices(pRP,pSD) ;
/**********************************************************************

Function Name XPauseComplete

Function Description
	Given a sysdevice, find its nt-dev-extn and clear all PAUSE related
	flags and parameters.

Input Parameters
	pSysDevice		This should be a sysdevice of a controller.

**********************************************************************/
#define	XPauseComplete(pSD)	;
/**********************************************************************

Function Name: XOSRemapBaseAddress

Function Description
	This function allows the OS to update the AD_BaseAddr field
	in the adapter device structure if the OS decides to remap 
	the base address.

Input Parameters
	pAD			pointer to the Adapter Device structure.

Output Parameters	
	None

**********************************************************************/
#define		XOSRemapBaseAddress(pCurrAD);

/**********************************************************************
Function Name XNotifyDeviceStatusChange

Function Description
This function is called so that the operating system can be notified
for the status change of the device.

Input Parameters
	pSD			Pointer to the sys device that has just been updated.

Output Parameters

**********************************************************************/
#define			XNotifyDeviceStatusChange(pScanSD);

/**********************************************************************
Function Name XEnableDataCache

Function Description
This function is a stub for enabling data cache, it is used only
in athena.

Input Parameters

Output Parameters

**********************************************************************/
#define			XEnableDataCache();

/**********************************************************************
Function Name XDisableDataCache

Function Description
This function is a stub for disabling data cache, it is used only
in athena.

Input Parameters

Output Parameters

**********************************************************************/
#define			XDisableDataCache();

/**********************************************************************

Function Name XChkBatteryState

Function Description
This function checks the state of the battery on athena.
It gets the current state of the battery, if state changes,
it saves the last state. It battery is ok, it sets SysWriteBack
flag for RAID5 devices, else it resets this flag.

Input Parameters

Output Parameters

**********************************************************************/

#define			XChkBatteryState();

/**********************************************************************

Function Name  XSetAdapterDevices

Function Description
Sets up the NVRam with new Adapter Device settings received from
application.

Input Parameters
	pRP			Pointer to remote control packet.
	pSD			Pointer to adapter sysdevice.

Output Parameters

**********************************************************************/
#define			XSetAdapterDevices(pRP, pSD);

/**********************************************************************

Function Name  XControllerSettings

Function Description
Currently only valid for Athena. This enables/disables Int13
devices in NVRam.

Input Parameters
	pRP			Pointer to remote control packet.

Output Parameters

**********************************************************************/
#define			XControllerSettings(pRP);

/**********************************************************************

Function Name  XGetControllerFault

Function Description
Get from the NVRam the controllor fault info for the application.

Input Parameters
	pRP				Pointer to remote control packet.
	pSD				Pointer to Controller sysdevice.

Output Parameters

**********************************************************************/
#define			XGetControllerFault(pRP, pSD);

/**********************************************************************

Function Name XCreateSemaphore

Function Description

Input Parameters

Output Parameters

**********************************************************************/
#define			XCreateSemaphore();

/**********************************************************************

Function Name XReleaseResources

Function Description
	Releases OS specific resources.

Input Parameters

Output Parameters
	none

**********************************************************************/
#define			XReleaseResources();

/**********************************************************************

Function Name XPostApiIOCtlDone

Function Description
	This is the post function for the driver athena packet. This function
	updates application packet status, and posts the IOCtl back to the OS.
	This file has to be moved to the OS specific IOCtl handling file.

Input Parameters
	pRemoteCtlPkt   pDrvPkt

Output Parameters
	none

**********************************************************************/
#define			XPostApiIOCtlDone(pDrvPkt);

/**********************************************************************

Function Name XPreparePktForRemote

Function Description
	This function takes an remote packet and modifies it for transmission
	to athena.
	Do OS specific manipulations on Scatter/gather list

Input Parameters
	pRemoteCtlPkt   pRemPkt

Output Parameters
	none

**********************************************************************/
#define			XPreparePktForRemote(pRemPkt);

/**********************************************************************

Function Name XCreateSGTable

Function Description
This function creates a SGTable for a buffer so that it can be accessed
by a DMA controller. Typically I/O requests will be received from the
operating system with a S/G list already created. IOCtls are issued without
knowledge about the physical memory associated with a buffer and need to
have the S/G list created for them.

It is assumed that the virtual buffer has already been locked prior to
calling this function.

Input Parameters
	pSGTable				A pointer to the SGTable that will be initialized
	virtualBufferAddr		The virtual address which needs a S/G list
	nBytes					The size of the the buffer

Output Parameters

**********************************************************************/
#define			XCreateSGTable(newSGTable, virtualBufferAddr, nBytes);

/**********************************************************************

Function Name XVirtAlloc

Function Description
This function allocates temporary memory from system's virtual memory
pool and locks the space. The allocated memory will be virtually
contigous.

Input Parameters
	vaddr   virtual address
	nbytes  number of bytes


Output Parameters
	SUCCESS				If allocated.
	FAILURE				If fail to allocate.

**********************************************************************/

#define			XVirtAlloc(vaddr, nbytes);

/**********************************************************************

Function Name XVirtFree

Function Description
This function frees temporary memory allocated from XVirtAlloc().

Input Parameters
	vaddr   virtual address
	nbytes  number of bytes

Output Parameters
	None, assumed to be successful

**********************************************************************/

#define			XVirtFree(vaddr, nbytes);


/**********************************************************************

Function Name XAckInterrupt

Function Description
This routine is called to perfrom the OS specific function needed to
acknowledge an interrupt.

Input Parameters
	IntLevel				The interrupt to acknowledge.

Output Parameters
	None

**********************************************************************/

#define			XAckInterrupt(InterruptLevel);

/**********************************************************************

 Function Name XDisplayLogError

 Function Description
 This function can be used to display an error message from the adapter.

 Input Parameters
	Fault Class

 Output Parameters
	none
 **********************************************************************/
#define			XDisplayLogError(FaultClass);





/**********************************************************************

 Function Name XBuildStorageDevs

 Function Description
 This function is used currently by the SCO driver to handle multiple cntrllrs

 Input Parameters
	none

 Output Parameters
	none
 **********************************************************************/
#define        XBuildStorageDevs();



/*VOID			XNotifyRescanPauseInProgress (pSysDevice pSD, USHORT PauseFlag);
VOID			XPauseComplete (pSysDevice pSD);
VOID			XRescanCompleted (pOSDescriptor   pDriverOSD);
ULONG			XGetSabreReservedSize (pSysDevice pParentSD) ;
VOID			XGetAdapterDevices (pRemoteCtlPkt pRP,pSysDevice pSD);*/

VOID			XCopyAthPktToRemPkt (pAthenaPkt pAthPkt, pRemoteCtlPkt pRemPkt,
					pSysDevice pSD, PVOID pAdapterDevExt);
VOID			XPostApiIOCTL(pRemoteCtlPkt pDrvPkt);
#ifdef NETWARE
UCHAR DPTR	XMemMapAdapter(ULONG BaseAddr, ULONG nBytes, void *	pCDADevExt);
ULONG XIOMapAdapter(ULONG BaseAddr, ULONG nBytes, void * pCDADevExt);
#else
USHORT DPTR		XMemMapAdapter (ULONG BaseAddr, USHORT nBytes);
ULONG			XIOMapAdapter (ULONG BaseAddr,USHORT nBytes);
#endif
VOID			XCfgAdapterIF (pPhysDevice pPD, pIODescriptor pIOD);
VOID			XCopyToApps (UCHAR DPTR pDest, UCHAR DPTR pSrc, ULONG nBytes);
BOOL			XMemoryReductionRequired(PVOID pAdapterDevExt);
USHORT		 	XCalculateOSDCount (PVOID pAdapterDevExt);
USHORT			XCalculateIODCount (PVOID pAdapterDevExt);
/* Manju */
USHORT			XCalculateSpecialIODCount (PVOID pAdapterDevExt);
USHORT			XCalculateAPCount(PVOID pAdapterDevExt);
/*ULONG			XBuildOSAdapterDevices(PVOID pAdapterDevExt);*/
VOID			ResyncPhysAddresses();
VOID			ResyncDone(pRemoteCtlPkt pRP);
pOSDescriptor 	XConvertRemPktToOSD(pRemoteCtlPkt pRemPkt,FPTR RetryFunc);
ULONG		  	ReturnMaxMemAlloc ();
ULONG		  	XPhysAlloc (PVOID pAdapterDevExt, ULONG   nBytes);

#ifdef SCO_DDI8
#define         XAlignedMemAlloc(pAdapterDevExt, nBytes, alignment) XMemAllocContiguous(pAdapterDevExt, nBytes)
VOID DPTR       XMemAllocContiguous( PVOID pADE, ULONG size);
VOID DPTR       XMemAlloc(pADAPTER_DEV_EXT, ULONG);
#else // SCO_DDI8

#ifndef LINUX
#define			XMemAllocContiguous(pAdapterDevExt, nBytes) XMemAlloc(pAdapterDevExt, nBytes)
#define			XMemAlloc(pAdapterDevExt, nBytes) XAlignedMemAlloc (pAdapterDevExt, nBytes, 0)
#define			XAlignedMemAlloc(pAdapterDevExt, nBytes, alignment) XAlignedMemAllocPastOH (pAdapterDevExt, nBytes, alignment, 0)
VOID DPTR		XAlignedMemAllocPastOH (PVOID pAdapterDevExt, ULONG nBytes, DATA_N_PTR alignment, DATA_N_PTR szOverHead);
#endif

#ifdef LINUX 
#define			XAlignedMemAlloc(pAdapterDevExt, nBytes, alignment) XMemAllocContiguous(pAdapterDevExt, nBytes)
VOID DPTR XMemAllocContiguous( PVOID pADE, ULONG size);
VOID DPTR  XMemAlloc(pAdapterDevExt, nBytes);
ULONG	XMemcmp(UCHAR DPTR pDest, UCHAR DPTR pSrc, ULONG nBytes);
#endif //LINUX
#endif // SCO_DDI8

VOID			XMemZero (UCHAR DPTR pSrc, ULONG nBytes);
VOID			XMemCpy (UCHAR DPTR pDest, UCHAR DPTR   pSrc, ULONG nBytes);
ULONG		  	XMemCmp (UCHAR DPTR pDest, UCHAR DPTR   pSrc, ULONG nBytes);
ULONG		  	XMemCmpDWord (ULONG DPTR pDest, ULONG DPTR   pSrc, ULONG nBytes);
USHORT			XSetWatchdog (PVOID HwDevExt, ULONG Period, VOID (CPTR pWatchdog)());

USHORT			XRegisterInterrupt (USHORT  IntLevel, BOOL  Exclusive);
USHORT			XDeregisterInterrupt (USHORT IntLevel, BOOL Exclusive);
VOID			XReceiveIOCTL (PVOID pSrb, PVOID pAdapterDevExt);
VOID			IssueIOCTL(pRemoteCtlPkt pDrvPkt);
//VOID			PostOSD (pOSDescriptor  pOSD);

/* TDE - PTL mapping change */
USHORT			XRemoveSystemReference (pSysDevice pSD, UCHAR MemberIndex);
USHORT			XCreateSystemReference (pSysDevice  pSD, UCHAR MemberIndex);

USHORT			LoadGlobalVars ();

/*
ULONG			XOSMReadPCIConfigurationDword (pAdapterDevice pCurrAD, ULONG registerNumber);
USHORT			XOSMReadPCIConfigurationWord (pAdapterDevice pCurrAD, ULONG registerNumber);
UCHAR			XOSMReadPCIConfigurationByte (pAdapterDevice pCurrAD, ULONG registerNumber);
VOID			XOSMWritePCIConfigurationDword (pAdapterDevice	pCurrAD, 
					ULONG registerNumber, ULONG registerValue) ;
VOID			XOSMWritePCIConfigurationWord (pAdapterDevice	pCurrAD, 
					ULONG registerNumber, USHORT registerValue) ;
VOID			XOSMWritePCIConfigurationByte (pAdapterDevice	pCurrAD, 
					ULONG registerNumber, UCHAR registerValue) ;
*/
pPCICfgHeader   XFindConfig(PVOID pAdapterDevExt);

VOID			XInitIobSend(HIM_IOB HIM_PTR pIOB);

HIM_UINT32	   	PostInitIobSend (HIM_IOB HIM_PTR pIOB);

VOID			EnterDebugger();
UCHAR			XWriteFlashRAM (UCHAR DPTR pBuf, ULONG nBytes); // WARN-FIX-FIX
USHORT			XRevertCodeImage ();
ULONG			XReturnImageRev (UCHAR DPTR pBuf,ULONG Func);
VOID			XReportStatusChange (pSysDevice pSD, UCHAR MemberIndex,
					UCHAR type, UCHAR param1, UCHAR param2);

VOID			XLogInitEvent (PVOID pAdapterDevExt, pLDTree pDevPath, UCHAR Status, UCHAR EventQual,
					pSCSISenseData pSenseData, UCHAR Key, UCHAR Code);

VOID			XRetrieveAthPktFromOSD(pOSDescriptor pOSD);
VOID			OSPostProc (pOSDescriptor   pOSD);

BOOL 			SabreKatanaTiedHardwareInterrupt(pSysDevice pSD);

//VOID XDebugPrint( UCHAR DPTR str, ULONG count, ULONG v1, ULONG v2, ULONG v3, ULONG v4, 
					//ULONG v5, ULONG v6, ULONG v7, ULONG v8, ULONG v9, ULONG v10 );

ULONG XGetBaseAddr(pSysDevice pSD, ULONG rangeIndex) ;
void	XNotifyOSAboutDeviceAdded(pSysDevice pSD) ;

#if	SAS_BRINGUP
void	XDebugPrint( int Priority, char *string ) ;
void	XOSMDebugPrint(unsigned long dbg_level, char *dbg_msg, ...);
#endif /* #if	SAS_BRINGUP */

VOID	XMemFill(
	UCHAR	DPTR	puc_Buf,
	UCHAR			uc_Pattern,
	USHORT			us_BufSize
);

USHORT	XPowerOf2(
	UCHAR	uc_Exponent
);

signed char	XExponentOf2(
	USHORT	us_PowerOf2
);


ULONG	XRandomNumber(
	ULONG	ul_NewSeed
);

BOOL XIsAdapterResetPending(
	PVOID	pCDADevExt
);

BOOL XIsHardwareResetPending(
	PVOID	pCDADevExt
);

BOOL XIsAllLunResetPending(
	PVOID	pCDADevExt
);

#define CORE_DEBUG 1
#if CORE_DEBUG
VOID XCoreDebugPrint(UCHAR *s);
VOID XCoreOSMDebugPrint(UCHAR *s, DATA_N_PTR param1,DATA_N_PTR param2);
#else
//#define XCoreDebugPrint(routineCode, pData1, pData2, INT) 
#endif


USHORT XGetDefaultITNexusLossTime(void);

UCHAR XGetDriverState(PVOID pCDADevExt);

VOID XLogOSError(PVOID pAdptDevExt, int errCode);



/* 
 * debug trace related definitions 
 */
//#define DBG_TRACE

#if _LME
    /* always disable the debug trace in 64 bits mode */
    #ifdef DBG_TRACE
    #undef DBG_TRACE
    #endif
#endif

#ifdef DBG_TRACE
void        log_entry(HIM_UEXACT32 p1, HIM_UEXACT32 p2);
void        log_error_entries(int save_flag);
void        Srb_In(void);
void        Srb_Out(void);
HIM_UEXACT32 Srb_Pending_count(void);

#else
#define     log_entry(p1, p2)
#define     log_error_entries(save_flag)
#define     Srb_In()
#define     Srb_Out()
#define     Srb_Pending_count()
#endif


typedef struct _dbglog
{
   HIM_UEXACT32 histamp;
   HIM_UEXACT32 lostamp;
   HIM_UEXACT32 p1;
   HIM_UEXACT32 p2;
} DBGLOG, *PDBGLOG;

#define MAX_DBG_LOG_ENTRY                   (8*1024)
#define MAX_ERR_LOG_ENTRY                   (16*1024)

#define LOG_ID_MASK                         0xFFFF0000
#define LOG_ID_HW_START_IO                  0x00010000
#define LOG_ID_HW_START_SCSI                0x00020000
#define LOG_ID_HW_START_IOCTL               0x00030000
#define LOG_ID_HW_SRB_COMPLETE              0x10010000
#define LOG_ID_HW_SRB_CMPT_ERROR            0x10040000

#define LOG_ID_RCV_FR_HIM_ERROR             0x10080000
#define LOG_ID_RCV_FR_HIM_ERROR2            0x10090000

#define LOG_ID_CSMI_TMF_ENTRY               0x000a0000
#define LOG_ID_CSMI_TMF_FUNC                0x000b0000
#define LOG_ID_CSMI_SAS_HW_RESET            0x000c0000
#define LOG_ID_CSMI_TMF_EXIT                0x100a0000
#define LOG_ID_CSMI_TMF_POST                0x100d0000

#define LOG_ID_CSMI_TMF_ABT                 0x00110000
#define LOG_ID_CSMI_TMF_ABTS                0x00120000
#define LOG_ID_CSMI_TMF_LUR                 0x00130000
#define LOG_ID_CSMI_TMF_HW_RESET            0x00150000

#define LOG_ID_RESET_BUS_ENTRY              0x00180000
#define LOG_ID_RESET_BUS_EXIT               0x10180000

#define LOG_ID_RESET_SAS_HW_ENTRY           0x001a0000
#define LOG_ID_RESET_SAS_HW_RET             0x001b0000
#define LOG_ID_RESET_SAS_HW_RET2            0x001c0000
#define LOG_ID_RESET_SAS_HW_ENTRY2          0x001f0000
#define LOG_ID_RESET_SAS_HW_CMPT            0x101f0000

#define LOG_ID_RESET_HW_CMPT                0x10ef0000

#define LOG_ID_RESET_ADAPTER_ENTRY          0x00200000
#define LOG_ID_RESET_ADAPTER_RET            0x00210000
#define LOG_ID_RESET_ADAPTER_RET2           0x00220000
#define LOG_ID_RESET_ADAPTER_ENTRY2         0x00280000
#define LOG_ID_RESET_ADAPTER_CMPT_ENTRY     0x10280000

#define LOG_ID_RESET_CHANNEL_ENTRY          0x00300000
#define LOG_ID_RESET_CHANNEL_CMPT           0x10300000
#define LOG_ID_RESET_CHANNEL_ENTRY2         0x00310000

#define LOG_ID_RESET_DPORT_ENTRY            0x00400000
#define LOG_ID_RESET_DPORT_CMPT             0x10400000
#define LOG_ID_RESET_DPORT_ENTRY2           0x00410000
#define LOG_ID_RESET_DPORT_RET              0x00420000

#define LOG_ID_RESET_LU_ENTRY               0x00500000
#define LOG_ID_RESET_LU_CMPT                0x10500000
#define LOG_ID_RESET_LU_ENTRY2              0x00510000
#define LOG_ID_RESET_LU_RET                 0x00520000

#define LOG_ID_REM_EXISTING_DEV             0x00600000
#define LOG_ID_REM_SAS_DEV                  0x00680000


#define LOG_ID_ADAPTER_EVENT_ENTRY          0x00700000
#define LOG_ID_ADAPTER_EVENT_EXIT           0x00710000
#define LOG_ID_PORT_EVENT_ENTRY             0x00780000
#define LOG_ID_PORT_EVENT_EXIT              0x00790000
#define LOG_ID_IOB_EVENT_ENTRY              0x007C0000
#define LOG_ID_IOB_EVENT_EXIT               0x007D0000


#define LOG_ID_CDA_TMF_REQ                  0x00800000

#define LOG_ID_TMF_CMPT                     0x10800000
#define LOG_ID_POST_OSM_TMF                 0x10810000
#define LOG_ID_POST_RESET_TMF               0x10820000

#define LOG_ID_ABORTED_STATUS               0x00900000
#define LOG_ID_ABORTED_STATUS1              0x00910000
#define LOG_ID_FLUSH_ALL_PD_IOS             0x00920000
#define LOG_ID_FLUSH_ALL_PD_IOS1            0x00930000
#define LOG_ID_FLUSH_ALL_SD_IOS             0x00940000
#define LOG_ID_FLUSH_ALL_SD_IOS1            0x00950000

#define SAVE_FLAG_ALL                       1
#define SAVE_FLAG_RELATED_ENTRIES           0
#define SAVE_FLAG_LAST_ENTRY                -1




#endif	/*OSIFPROT_INCLUDED*/
