/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: osmcda.h

File Description
		Imports required by the CDA RAID engine. 

***********************************************************************

History :
02/08/05 David
Ported osmcda.h for Linux environment.

07/24/04 Tarun
This file belongs to the NT environment only. Other Os should have their 
own osmcda.h file.

11/24/03: Tam
New prototype for XMapPTLandDeviceHandle.

11/03/03: Tam
New prototype for XReturnPhysAddr.

10/31/03: Madhuresh
Changed XNotify and XBuildOSAdapterDevices prototypes.

10/29/03: Madhuresh
Added correct SCSIPORT definitions.

08/14/2003 : RUIZ Tony 

Start first version of file to correspond to the CDA RAID Engine Interface Specification
Version .5.

***********************************************************************/
#ifndef OSMCDA_INCLUDED
#define OSMCDA_INCLUDED

/************************************************************************/
/*                     LINUX  DATA DEFINITIONS                          */
/*        THIS FILE IS SOLELY FOR LINUX ENVIRONMENT ONLY                */
/************************************************************************/


/* include files definitions */
#ifdef SCO_DDI8
#ifdef	_KERNEL_HEADERS
#include <util/types.h>
#include <util/cmn_err.h>
#include <util/debug.h>
#else /* _KERNEL_HEADERS */
#include <sys/types.h>
#include <sys/cmn_err.h>
#include <sys/debug.h>
#endif /* _KERNEL_HEADERS */
#else // SCO_DDI8
#include <asm/io.h>
#include <linux/types.h>
#endif // SCO_DDI8

#include "custom.h"


#define DebugPrintMem(x) 
#define DBG_ALWAYS  		0

/*
 * Set this to 1 to enable lower layer (CDA, CHIM) debugging, such as
 * printing, asserting.
 */
#define DEBUG_BUILD		0

#if DEBUG_BUILD
#ifndef SCO_DDI8
extern asmlinkage int printk(const char * fmt, ...)
	__attribute__ ((format (printf, 1, 2)));
#endif // SCO_DDI8

    #define OSM_DBGMSG_TRACE			(0x1)
    #define OSM_DBGMSG_WARNING			(0x5)
    #define OSM_DBGMSG_ERROR			(0xa)
    #define OSM_DBGMSG_FATAL			(0xf)
    #define OSM_DBGMSG_DUMP         (1U << 8)   /* same as HIM_DEBUG_DUMP */

    #define CDA_DBGMSG_DDF			(0x40)
    #define CDA_DBGMSG_ALL			(0x80)
    /* Assert macroes*/
    #define Assert(condition)                   OSMDebugAssert(condition)
    #define CDA_ASSERT(condition)               OSMDebugAssert(condition)
    #define OSM_ASSERT(condition)               OSMDebugAssert(condition)
    #define CDA_ASSERT_MSG(condition, message)  OSMDebugAssertMsg(condition, message)
    #define OSM_ASSERT_MSG(condition, message)  OSMDebugAssertMsg(condition, message)
    /*Macro for message printing into the CDA layer*/
    #define CDA_DEBUG_PRINT(sev, str)		OSMDebugPrint(sev, str)
    #define OSM_DEBUG_PRINT(sev, str)		OSMDebugPrint(sev, str)
    #define CHIM_DEBUG_PRINT(sev, str)		OSMDebugPrint(sev, str)
#else
    #define Assert(condition)			/**/
    #define CDA_ASSERT(condition)		/**/
    #define OSM_ASSERT(condition)		/**/
    #define CDA_ASSERT_MSG(condition, message)	/**/
    #define OSM_ASSERT_MSG(condition, message)	/**/
    /*Macro for message printing into the CDA layer*/
    #define CDA_DEBUG_PRINT(sev, str)		/**/
    #define OSM_DEBUG_PRINT(sev, str)		/**/
    #define CHIM_DEBUG_PRINT(sev, str)		/**/
#endif


/* Data definitions */
#define	DATA_N_PTR		unsigned long	

#ifndef	FALSE
#define FALSE                       (0)
#endif

#ifndef	TRUE
#define TRUE                        (1)
#endif

#ifdef SCO_DDI8
#ifndef	NULL
#define NULL                        0
#endif
#else // SCO_DDI8
#undef NULL
#ifndef	NULL
#define NULL                        ((void *) 0)
#endif
#endif // SCO_DDI8

#ifndef	DPTR
#define DPTR                        *
#endif

#ifndef	NP
#define NP                          *
#endif

#ifndef	CPTR
#define CPTR                        *
#endif


/* Define 8 bit data types and their pointers */
typedef unsigned char		UINT8, *PUINT8;
typedef int			BYTE, *PBYTE;
typedef int			BOOL;
typedef unsigned char		UBYTE, *PUBYTE, UCHAR, *PUCHAR;

/* Define 16 bit data types and their pointers */
typedef	unsigned short		UINT16, *PUINT16, USHORT, *PUSHORT,
				UWORD, *PUWORD;

/* Define 32 bit data types and their pointers */
#undef UINT32
#ifndef UINT32
#define UINT32		uint32_t 
#endif

#undef PUINT32
#ifndef PUINT32
#define PUINT32		uint32_t *
#endif

#ifndef DWORD
typedef uint32_t	DWORD;
#endif

#ifndef PDWORD
typedef uint32_t	*PDWORD;
#endif

#ifndef ULONG
typedef uint32_t	ULONG;
#endif

#ifndef PULONG
typedef uint32_t	*PULONG;
#endif


/* Define 64 bit data types and their pointers */
#undef UINT64
#ifndef UINT64
#define UINT64		uint64_t
#endif

#undef PUINT64
#ifndef PUINT64
#define PUINT64		uint64_t *
#endif


#ifndef	VOID
#define VOID		void
#endif

typedef VOID DPTR PVOID;

#ifndef	FNPTR
typedef VOID ((CPTR FNPTR) ());
#endif

#ifndef	FPTR
typedef VOID ((CPTR FPTR) ());
#endif

#define OSM_NEW_DEVICE_EVENT 1 //NT SPECIFIC ONLY

#define XDECLARE_INTS
#define XENTER_CRITICAL_REGION(X)
#define XEXIT_CRITICAL_REGION(X) 

#define XINBYTE(Port)           inb(Port) 
#define XINWORD(Port)           inw(Port) 
#define XINDWORD(Port)          inl(Port)    
#ifdef SCO_DDI8
#define XOUTBYTE(Port,Data)     outb(Port,Data)   
#define XOUTWORD(Port,Data)     outw(Port,Data)    
#define XOUTDWORD(Port,Data)    outl(Port,Data)    
#else // SCO_DDI8
#define XOUTBYTE(Port,Data)     outb(Data,Port)   
#define XOUTWORD(Port,Data)     outw(Data,Port)    
#define XOUTDWORD(Port,Data)    outl(Data,Port)    
#endif // SCO_DDI8

/* 
 * Printf support for debug environment
 * Macros definitions available to all the files
 */
#if DEBUG_BUILD


#define CDA_DBGMSG_TRACE	OSM_DBGMSG_TRACE
#define CDA_DBGMSG_WARNING	OSM_DBGMSG_WARNING
#define CDA_DBGMSG_ERROR	OSM_DBGMSG_ERROR
#define CDA_DBGMSG_FATAL	OSM_DBGMSG_FATAL

#define CHIM_DBGMSG_TRACE	OSM_DBGMSG_TRACE
#define CHIM_DBGMSG_WARNING	OSM_DBGMSG_WARNING
#define CHIM_DBGMSG_ERROR	OSM_DBGMSG_ERROR
#define CHIM_DBGMSG_FATAL	OSM_DBGMSG_FATAL
#define CHIM_DBGMSG_DUMP   OSM_DBGMSG_DUMP

/* SAS CHIM debug print options: to dump SAS CHIM object */
#define OSM_DBGMSG_SAS_DUMP

/* OSM Dump Mode Specific Options */
#define OSM_SAS_DUMP_SHARED_STRUCT
#define OSM_SAS_DUMP_CHIP_MEMORY
#define OSM_SAS_DUMP_REGISTERS
#define OSM_SAS_DUMP_QUEUE
#define OSM_SAS_DUMP_STRUCT


//
// debug build macros
//
extern u_int	DEBUG_LEVEL;
extern u_int	OSMDebugMode;
  
//
// Set the Debug Level ( for debugging Purpose)
//
#define SET_DEBUG_LEVEL()	DEBUG_LEVEL = (CDA_DBGMSG_TRACE |	\
					       CDA_DBGMSG_WARNING |	\
					       CDA_DBGMSG_ERROR |	\
					       CDA_DBGMSG_FATAL |	\
					       CHIM_DBGMSG_DUMP |	\
					       0);

#define OSMDebugPrint(sev, str)			\
do {						\
	if (sev & DEBUG_LEVEL)  		\
	{                           		\
#ifdef SCO_DDI8
		printf str;			\
#else // SCO_DDI8
		printk str;			\
#endif // SCO_DDI8
	}                           		\
} while(0)

#define OSMDebugPrintWithNewline(sev, str)			\
do {						\
	if (sev & DEBUG_LEVEL)  		\
	{                           		\
#ifdef SCO_DDI8
		printf str;			\
		printf("\n");			\
#else // SCO_DDI8
		printk str;			\
		printk("\n");			\
#endif // SCO_DDI8
	}                           		\
} while(0)

#define OSMDebugAssert(exp)						\
	if (!(exp)) { 							\
		panic("assertion failed: %s, file: %s, line: %d\n",	\
			#exp, __FILE__, __LINE__);			\
	}

#define OSMDebugAssertMsg(exp, fmt)		\
do {						\
	if (!(exp)) {				\
#ifdef SCO_DDI8
		printf(fmt);			\
#else // SCO_DDI8
		printk(fmt);			\
#endif // SCO_DDI8
		OSMDebugAssert(exp);		\
	}					\
} while(0)

#ifdef SCO_DDI8
#define DebugBreak()    call_demon()
#else // SCO_DDI8
#define DebugBreak()				\
		panic("DebugBreak !! file: %s, func: %s, line: %d\n",	\
		       __FILE__, __FUNCTION__,  __LINE__);

#define ASSERT(exp)						\
	if (!(exp)) { 							\
		panic("Assertion failed: %s, file: %s, line: %d\n",	\
			#exp, __FILE__, __LINE__);			\
	}
#endif // SCO_DDI8
#else

#define SET_DEBUG_LEVEL()	/**/

#define NOTHING			/**/

#define OSMDebugPrint(severity, String)		\
do {                        			\
	NOTHING;				\
} while(0)

#define OSMDebugAssertMsg(condition, message)	\
do {						\
	NOTHING;				\
} while(0)

#define DebugBreak()		/**/

#ifndef SCO_DDI8
#define ASSERT(exp)		/**/
#endif // SCO_DDI8

#endif /* DEBUG_BUILD */

#endif

#ifdef HR_SATA_BUILD 

VOID
OSMsgCopyIn(UINT32 bcr, UINT32 padr, 
		UINT32 puadr, PVOID ladr, PVOID iob);

VOID
OSMsgCopyOut(UINT32 bcr, UINT32 padr, UINT32 puadr,
    				PVOID ladr, PVOID iob);

#endif  //CDA_FOR_SATA
