/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:physdev.h

File Description
	Define those structures used for scsi device manipulation.
***********************************************************************

History :

10/05/2004	14:50	Kashif
A local (only-needed-fields) version of HIM_ADAPTER_PROFILE was introduced
called LocalAdapterProfile, since CHIM enabled SAS_ADAPTER_PROFILE_PAGE_SUPPORT
and now Adapter Profile is avaialble in pages rather then all at once.  We read
it page by page and save the needed values in this local structure to be used
throughout the code.

09/20/2004  Madhuesh
Defined PendingRemoval and adjusted alignment

06/24/2004  11:25  NAR20038  Manjunath
Updated PortInformation to include freeze flag. Added working action set to
AdapterDevice.

06/17/2004  13:30  NAR20038  Manjunath
Updated PhysDevice struct to include bConfiguredDrive.
Updated AdapterDevice struct to include PortTSHUnderDiscovery, Port details of all ports.
Updated SasDevice struct to include index into AD_SasDevices table.

11/21/2003 : Madhuresh
AD now holds SasDevice pointer for current PD temporarily.

11/19/2003 : Madhuresh
HIM task related IODs are now allocated and pointers are preserved in AD.
pAD_PhysMap has space for all PDs instead of allocating after AD.
AD_ChipId added. Now vendor and device ID are not filled in AD.

10/03/2003 : Madhuresh
SAS specific structure and field in PD added.

05/06/2003 : 12:10   NAR20038  Manjunath
Added PHYS_DEVICE_ERROR_THRESHOLD to elimate the target from the system 
if it consistantly report errors.

10/28/2002 : 01:25   NAR20038  Manjunath
Increased timeout counts for Reset, Abort, Device busy to 15 sec. We had
5 sec timeouts, and that caused us to hit abort and resets back to back
and made our recovery logic from resets complicated. With 15 sec we have
enough time cushion to recover from any resets.

12/20/2001 : 13:46   NAR20038  Manjunath
Removed #ifdef WOLFPACK, NETWARE and NT. For hibernate/standby Windows
can AD_Protectedmode state for saving adapter state.

12/06/2001 : 12:02   NAR20038  Manjunath
Added DEVICE_BUSY_COUNT and DEVICE_BUSY_PROCESSED counters and removed
DEVICE_BUSYCOUNT.

10/29/2001 : 14:32   NAR20038  Manjunath
Added #define for RESET_CHANNEL_TIMEOUT, BOARD_RESET_TIMEOUT and
DEVICE_BUSY_DELAY.

05/15/2001 : 03:02 PM   NAR20038  Manjunath
Reduced DELAY_AFTERRESET to 1 from 5. Now it's basically takes 5 secs.
Added ABORT_COMMAND_TIMEOUT (1 - 5 sec) for abort commands.

04/27/2001 : 01:40 PM   BURN1729  Farrukh 
Added a field in struct  _AdapterDevice for W2K to hold a pointer for a
location where chip state will be saved for hibernation and standby.

01/24/2001 : 03:02 PM   NAR20038  Manjunath
Removed Sabre related data structures

12/13/2000 : 11:08 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Removed pSabreDevData from ControllerDevice
2. Removed #ifdef ATHENA
3. Removed #include "mailbox"
4. Removed pRemoteQueue "pCD_Header"

***********************************************************************/

#ifndef PHYSDEV_INCLUDED
#define PHYSDEV_INCLUDED

#include "sysdef.h"

#define	MAX_ADAPTERS			8

#define SMART_DRIVE				4
/********************************************************************************
	SasDevice is a new structure defined to contain SAS specific properties
	of each SAS device discovered on the adapter (all sas domains).
	A table of these has been embedded in Device Extension (see RAIDGLBL.H).
*********************************************************************************/

typedef struct _SasDevice	SasDevice, DPTR pSasDevice;

struct	_SasDevice
{
	pSasDevice			pNextSasDevice; /* list of devices on port basis (per port) */

#ifdef SES2_DEVICE
    USHORT				Sas_Flags ;     /* End device, Expander, Working, failed, etc. */	
#else
    UCHAR				Sas_Flags ;     /* End device, Expander, Working, failed, etc. */
#endif

	UCHAR				SasDeviceIndex ; /* Index into AD_SasDevices list */

    UCHAR               Sas_PendingRemoval;
    //UCHAR               Reserved;

	USHORT				Sas_AttachedPortId ;
	USHORT				Sas_AttachedTargetId;

	HIM_TASK_SET_HANDLE Sas_AttachedPortTSH; /* PortTSH at the initiator */

	UCHAR				Sas_WorldWideID[ CDA_SAS_ADDRESS_SIZE ] ;

	pPhysDevice			Sas_pPD ;		/* Pointer to PD, it represents */
	HIM_TASK_SET_HANDLE Sas_DevicePortTSH ;
	HIM_TASK_SET_HANDLE	Sas_AttachedDevicePortTSH ;

	/* TDE added */
	PVOID				Sas_pPortInfo;
	
	USHORT				Sas_DevicePortConnectionRate;
	USHORT				Sas_DevicePortNegotiatedPhyLinkRate;
	USHORT				Sas_DevicePortITNexusLossTime;
	UCHAR				Sas_DevicePortTranportType;
	UCHAR				Sas_FaultToleranceTimeout;//Sas_Reserved2;
    #define                 FTTT_UNINITIALIZED_VALUE    0xFF

    PVOID               Sas_TaskIOD; /*Used to do the Probe Lu*/
    HIM_TASK_SET_HANDLE Sas_DeviceContext ;
    UCHAR               Sas_TargetProtocol ;
#ifdef SES2_DEVICE
    USHORT              Sas_PhysTagsMax ;
#else
    ULONG               Sas_PhysTagsMax ;
#endif
    USHORT              Sas_DeviceLUN ;

#ifdef SES2_DEVICE
    /****************************************************/
    /* Added fields for SES-2 support                   */
    /****************************************************/
    UCHAR               Sas_ConnectorType;        /* SES-2 Device connected to external/internal connector */

    /* Possible number of device elements and
     * array device elements supported by this SES-2 device
     * If this Sas device represents an EXPANDER, then
     * this indicates the Number of Phys the expander supports 
     */
    UCHAR               Sas_NumberOfPossibleElements; /* Total possible number of elements of all types */
    UCHAR               Sas_NumberOfPossibleArrayAndDeviceElements; /* Possible number of elements of */
                                                                        /* array and device types. */
    


    /* If the target is a SES-2 device, then this is the slot 
     * location of the target inside the SES-2 box. 
     * If the target is behind an EXPANDER, then this is the
     * Phy id of expander to whic this sas device port is attached to.
     */
    UCHAR               Sas_SlotNumber;               
                                                       
     
    UCHAR               Sas_RelativePosition;         /* This value indicates a SES-2 box position */
                                                      /* that is the first or second, etc. box from */
                                                      /* the external connector of an Initiator HBA. */
                                                      /* This value is calculated based on Sas_DevicePortDiscoveryIndex */
    USHORT              Sas_SumOfPossibleElementsOffset;    /* When more than one SES-2 boxes are cascaded, then */
                                                            /* this value indicates the Sum of Possible Elements */
                                                            /* of all its previous */
                                                            /* SES-2 devices in the chain. */
    UCHAR               Sas_PhyIdentifier;             /* Attached phy value */

    UCHAR               Sas_LEDActivityStarted;        /* Set to TRUE if LED command is send to I2C Enclosure */
    USHORT              Sas_DevicePortDiscoveryIndex; /* Order in which device ports are discovered */ 
    USHORT              ArrayElementOffset;           /* Store the offset of Array Element in    */
                                                      /* Enclosure Control Diagnostic Page (=02) */ 
                                                   
    
#endif
    /*********************************************/
#if CDA_ENABLE_FTTT_SUPPORT
    UINT8               Sas_AttachedPhyIdentifier;/* It is the Phy id where this device attached to */
    UINT8               Sas_AttachedDeviceWWN[ CDA_SAS_ADDRESS_SIZE ] ; /* WWN this device attached to,e.g. 
                                                                          exapander or HBA itself*/
#endif
    UniqueName          Sas_UniqueName;
};

/* TDE */
/* Definition of PhysTMFState */
#define PHYS_TMF_SUCCESS		0
#define PHYS_TMF_FAILED			1
#define PHYS_TMF_UNSUPPORTED	2
#define PHYS_TMF_RETRY			3
#define PHYS_TMF_TIMEOUT		4


/* Definition of bResetPending */
#define PHYS_TMF_NO_RESET			        0x00
#define PHYS_TMF_LUN_RESET_BY_OSM	        0x01
#define PHYS_TMF_LUN_RESET_BY_CDA	        0x02
#define PHYS_TMF_LUN_RESET_BY_CSMI	        0x04
#define PHYS_TMF_DEVICE_PORT_RESET_BY_OSM	0x08
#define PHYS_TMF_DEVICE_PORT_RESET_BY_CDA	0x10
#define PHYS_TMF_DEVICE_PORT_RESET_BY_CSMI	0x20
#define PHYS_TMF_LUN_ABORT_TASK_SET_CDA     0x40


/* Definition of PhysSubState */
#define PHYS_DEVICE_MISSING         0x01

/*********************************************************************************
	The Physical Device - PD

	This is created one for each device discovered in domain or bus. A unique
	PD node is also created one per Lu in a given device.

*********************************************************************************/
typedef struct	_PhysDevice
{
    pPhysDevice         pPhysNext;      /* Link pointer - used for Lu */
    pSysDevice          pSysParent;     /* Back pointer to owner */
    pAdapterDevice      pBusParent;     /* Pointer to AD */
    pSasDevice          pSasInfo ;      /* Points to SAS specific values */
    SenseBuffer         Sense;          /* Only used for removable media Devices for now */
                                        /* Points to a DMAble 256 byte buffer to support sense data */
                                        /* longer than the built-in 32 byte sense buffer in each command */
    PVOID               Sas_pAttachedSES2Device;      /* Association between a device elements and its parent */
                                                       /* Here parent is the SES-2 device or box indicating */
                                                       /* a device element belongs to a certain box */
                                                       /* For a SES-2 sas device itself this pointer is NULL */

	FPTR			   	PhysSubmit;
	FPTR  				HIMSubmit;	/* Bypass IOSort routine */
	FPTR  				pPhysIOCtl;

	ULONG			   	cPhysTicks;	/* Timer ticks for timeout value */
	USHORT			   	PhysType; 	/* SCSI device type */
	USHORT				PhysId;		/* SCSI id of the device */
    USHORT              PhysIdOld;  /* Old Id replaced with PhysId (new id) */
                                    /* when PD is attached behind a SES2 device */
                                    /* In certain situation PD can be discovered */
                                    /* before knowing that this PD is in fact a SES2 */
                                    /* device. So the old mapping of PD needs to be */
                                    /* removed with the new mapping done according to SES2 */
                                    /* page information.                                */
	USHORT				PhysLUN;	/* Some LUN designation	*/
	USHORT				PhysBus;	/* Some expression of the bus */
    USHORT              PhysBusOld; /* Old bus replaced with PhysBus (new bus) */
                                    /* when PD is attached behind a SES2 device */
                                    /* In certain situation PD can be discovered */
                                    /* before knowing that this PD is in fact a SES2 */
                                    /* device. So the old mapping of PD needs to be */
                                    /* removed with the new mapping done according to SES2 */
                                    /* page information.                                */
	USHORT				PhysMapIndex;	/* Index to AD_PhysMap array in AdapterDevice */
										/* indicating the location of the PhysDevice */
    UCHAR               PhysSubState;
	UCHAR				Reserved;

	UINT32			   	PhysCapacity;
	UINT32 				PhysValidTo;/* RAID only	*/
	USHORT				PhysRemovable;
	USHORT				PhysState; 	/* Working, failed, etc. */

	USHORT				PhysTagsMax;
	USHORT				TP_SCSICurrentSpeed; /* from TargetProfile to send in athena ioctl  */
	USHORT				TP_SCSIMaxSpeed;     /* from TargetProfile to send in athena ioctl */
	USHORT				PhysBusWidth;  /* Taken from INQUIRY field to send in athena ioctl */
	USHORT				PhysAtDevice;
	USHORT				PhysMaxRetry;  /* Set as 3, never used */

	UCHAR			   	Phys_HoldOff;   
    UCHAR               bResetPending;   /* used for deviceport reset */
    UCHAR               PendingRemoval;   /* used for deviceport reset */

//    UCHAR               Debug;			/* abort/reset debug */
    UCHAR               PendingRequest;/* Contains the pending request to perform later in timer context*/
    /* Definition for PendingRequest */
    #define PENDING_REQ_RESET_SAS_HARDWARE                   0x01
    #define PENDING_REQ_RESET_ADAPTER                        0x02
    #define PENDING_REQ_RESET_DEVICE_PORT                    0x04

	/*Whether the CHIM queue for this PD is frozen. If so, we must send an IOB
	with function HIM_UNFREEZE_QUEUE by calling SendIOD_UnfrzQ. This field
	becomes TRUE when SendIOD_UnfrzQ fails due to lack of free IODs. Code that
	reads and writes this field is critical.*/
	
	/* TDE - change data type from BOOL to UCHAR
	 * to define PhysTMFRetryCount field.
	 */
	//BOOL			   	bPhys_StartStop; /* Start stop flag for IOCTL to stop sending SCBs to HIM */
	UCHAR				bPhys_StartStop;
	UCHAR				PhysTMFRetryCount;	 /* TMF retry count */

	UCHAR				bPhysDeviceBusy;	/* These are used to handle busy and queue full condition */	
	UCHAR				cPhysBusyProcessed;

	/* TDE - change data type from USHORT to UCHAR
	 * to define PhysTMFState field.
	 */
	//USHORT				cPhysTCount;
	
	UCHAR				cPhysTCount;	/* TDE - Used for device busy, Lun Reset */
										/* device port reset timeout. */

	UCHAR				PhysTMFState;	/* TMF task success, failed, need retry, or unsupported  */

#ifdef SES2_DEVICE  	

    /* 
     * Fields used to defer OSM notification
     * of device inserted.
     */
	USHORT				PhysOSMNotifyType;	/* To defer OSM notification at later time */  
                                            /* after PT mapping is assigned to an inserted */
                                            /* device, this field keeps the information */
                                            /* the types of notificaiton. */
#define DEFER_OSM_NOTIFY_NONE                   0x0000
#define DEFER_OSM_NOTIFY_NEW_SYSDEVICE_ADDED    0x0001
#define DEFER_OSM_NOTIFY_NEW_PHYSDEVICE_ADDED   0x0002
#else

    USHORT				PhysReserved;	/* To align to long */
#endif

	ULONG				PDTimestamp;
	ULONG				CurrentTimeCount;

	pIODescriptor		pIOQueueHead;	/* Head of work queue */
	pIODescriptor		pIOQueueTail;	/* Tail of work queue */
	pIODescriptor		pLowPriStart; /* Head of relative low priority requests*/
	pIODescriptor		pIODsAtHIM;		/* Head of queue for IODs at HIM */
    pIODescriptor       pPhysOvrlapIO;  /* OS IO overlapped queue, IO that */
                                        /* overlapped with backgound tasks IOs */


	HIM_TASK_SET_HANDLE targetTSH;
	BOOL				bSmartDrive;
#if 1  //@TR_PERF_RSV
    ULONG				LastReadLBA;
#else
	ULONG				BalanceLoadFlag ;
#endif
	ULONG				ErrorThreshold;
	BOOL				bConfiguredDrive;
	BOOL				bSmartError;
	BOOL 				bDriveWriteCacheEnabled;
	BOOL                bNoChangeInCacheSetting;
	ULONG				PhysCrntQd ;
	USHORT				QCounter ;

	/*******************/
	/* For Dell DDF Metadata */
	/*******************/
	UCHAR				InqData[20];
	ULONG				PD_Reference;

    UINT32              WorkingActionSet;
    UINT32              SavedActionSet;
    PVOID               SavedEventContext;
    UINT32              LuEventFreeze;
#define CDA_LU_FREEZE_INITIATED_BY_CHIM                             0x01
#define CDA_LU_FREEZE_INITIATED_BY_CDA                              0x02
#define CDA_LU_FREEZE_PENDING_DEVICE_VALIDATION                     0x04
#define CDA_LU_FREEZE_INITIATED_BY_CDA_DUE_TO_PERFORM_LU_RECOVERY   0x08
#define CDA_LU_FREEZE_INITIATED_BY_CDA_DUE_TO_ABORT_TASK_SET        0x10
#define CDA_LU_FREEZE_INITIATED_BY_CDA_DUE_TO_READ_MEH_RECOVERY     0x20
#define CDA_LU_FREEZE_INITIATED_BY_CDA_DUE_TO_WRITE_MEH_RECOVERY    0x40
    UINT32              LuLocalWorkingActionSet;
#define CDA_LU_LOCAL_FREEZE_LOGICAL_UNIT     (1 <<  0)
#define CDA_LU_LOCAL_VALIDATE_LOGICAL_UNIT   (1 <<  4)
#define CDA_LU_LOCAL_UNFREEZE_LOGICAL_UNIT   (1 << 31)

    UINT8               NotReadyIoFailedCount;
    UCHAR               PhysDeviceElementNumber;    /* added to save Device Element Number */
                                                    /* returned by SES-2 page */
                                                    /* Saved to turn on FAULT LED when a */
                                                    /* member is removed. Since SAS object is */
                                                    /* is freed when a member is removed, we need */
                                                    /* to save the device element position number */
                                                    /* in the member PhysDevice.                  */
                                                    /* This is ONLY supported for RAID            */
                                                    /* device. For non-configured/Configured single */
                                                    /* FAULT LED is not supported since when a single */
                                                    /* is removed all data structures related to single */
                                                    /* is freed; whereas a RAID member PhysDevice struct. */
                                                    /* is kept as long as not all members of a RAID are */
                                                    /* removed. */

    /* modepage bit for AWRE */
    UINT8               MPage_AWRE;
    UINT8                  HZ1ArrayMember;
} PhysDevice;
/*********************************************************************************
	Bit definition for the bPhysDeviceBusy byte.
*********************************************************************************/
#define	DEVICE_BUSY				0x01		/* target returns busy status */
#define	DEVICE_QUEUEFULL		0x04		/* target returns queue full status */
#define DEVICE_BUSY_COUNT		5		/* wait for 5 ticks (5 sec apart - 25 sec
										 * before clearing Busy condition. 
										 */
#define DEVICE_BUSY_PROCESSED	4		/* process DEVICE BUSY 4 times. If the
								 		 * drive reports busy again, reset the
								 		 * channel.
								 		 */

#define ADAPTER_BUS				0
#define ADAPTER_XOR				1

#define DELAY_AFTERRESET   		3	/* 15 sec delay after reset being cleared */

/* TDE */
/* Abort task, Logical Unit reset, Device Port reset - all
 * has one timeout defined
 * NOTE:: Depending on the value of the TicksPerSec in device extension, the frequency
 * of the timer running in one second changes, to accomodate this change
 * dynamically timeout has to be multiplied by the number of ticks.
 */
#define TMF_TASK_TIMEOUT	10	/*  10 sec timeout for TMF task */

//#define ABORT_COMMAND_TIMEOUT	3	/*-  15 sec timeout for Abort Commands. */


#define RESET_CHANNEL_TIMEOUT	15	/* 15 sec timeout for Reset commands. */
#define BOARD_RESET_TIMEOUT		15   /* 15 sec timeout for Board reset commands. */
#define DEVICE_BUSY_DELAY		15   /* 15 sec delay after device reports queue
									 * full or busy.
									 */
#define PHYS_DEVICE_ERROR_THRESHOLD	3 /* allow 3 times before marking the drive down */


/* Currently we are not issuing abort task command */
/* abort timedout command 3 times and then do a bus reset.	   */
#define ABORT_THRESHOLD 			3  /* Indicates if abort timed out, send Device Port reset
									   * immediately, no need to retry abort task request.
									   */

/* Support CheckIOBMemoryNeeded */
#define MAX_IOB_FUNCTIONS                    0x50
#define MAX_HIM_BUFFER_DESCRIPTOR_IN_IOB    4

#define IOD_FOR_PAC               1
#define IOD_FOR_TASK_IOD          2
#define IOD_FOR_LU_VALIDATION_IOD 3
#define IOD_FOR_RESET_ADAPTER_IOD 4
#define IOD_FOR_RESET_HARDWARE_IOD 5

/* Initiator Port Information */
typedef struct _PortInformation
{
	UCHAR 	        PortNumber;         /* logical number, index in to AD_PortTSH */
	UCHAR           eventFreeze;        /* indicates all I/Os on this port must be blocked */
    #define PORT_FREEZE_INITIATED_BY_CDA                        0x01
    #define PORT_FREEZE_INITIATED_BY_CHIM                       0x02
    #define PORT_FREEZE_INITIATED_BY_CDA_DUE_TO_PORT_PAC        0x04
    #define PORT_FREEZE_INITIATED_BY_CDA_DUE_TO_DISABLE_PORT    0x08
    #define PORT_FREEZE_INITIATED_BY_CDA_DUE_TO_FAILED_PHY      0x10
    #define PORT_FREEZE_INITIATED_BY_CDA_DUE_TO_RESET_PORT      0x20  
	UCHAR	        PortState; 
    /* Definition for PortState */
    #define PORT_STATE_VALID                0x01
    #define PORT_STATE_VIRTUAL              0x02
    #define PORT_STATE_MISSING              0x04
    UCHAR           PortPendingTaksToPerform;/* Contains the pending tasks to perform later*/
    /* Definition for PendingTaksToPerform */
    
    #define PORT_PENDING_TASK_DEVICES_VALIDATION_REQUIRED    0x01
    #define PORT_PENDING_TASK_DEVICES_LU_VALIDATION_REQUIRED 0x02

	pSasDevice	    pNextSasDevice;	    /* list of all devices connected to this port */
    pSasDevice      pSavedDeviceList;/* List of device on Port before StopAdapter (Per Port) */
	pAdapterDevice  pAD;                /*  pointer to Adapter Device */
    ULONG           SavedActionSet;     /* original action set when port event is generated */
    ULONG           WorkingActionSet;   /* working action set as we progress through port event handling */
    ULONG           SavedEventContext;  /* saved CHIM context for the event */
    PVOID           pPortPACIOD;        /* To issue the PAC on this Port */

} PortInformation, DPTR pPortInformation;


/* Local Adapter Profile structure to copy information from CHIM Adpater Profile pages */
/* Not all the fields are copied from CHIM, only the current used fields are copied from CHIM */
/* If some new fields need to be used, please copy them from Adpater Profile pages provided by CHIM */
typedef struct	_LocalAdapterProfile
{
	/* required fields */
	UINT16		AP_MajorVersion;       /* Profile major version number        */
	UINT16		AP_MinorVersion;       /* Profile minor version number        */

	/* informational only fields */
	UINT16		AP_NumDevices;			/* Max devices (targets+expanders)
										 * adapter can support. 
										 */
	UINT8		AP_NumPhys;				/* Number of phys adapter can support  */
	UINT8		AP_NumPorts;			/* Number of ports adapter can support */
	UCHAR		AP_WorldWideName[HIM_MAX_WWID_SIZE];
										/* Adapter's World Wide Name */

	BOOL		AP_BiosActive;			/* If HIM_TRUE, BIOS is active on
										 * adapter.
										 */
	UINT8		AP_BiosVersionFormat;	/* The version format used for
										 * returning BIOS version information
										 * in field AP_Bios_Version.
										 * One of:
										 *    HIM_BV_FORMAT1,
										 *    HIM_BV_FORMAT_UNKNOWN
										 */
	struct BIOS_VERSION_ {
		union {
			struct BIOS_VERSION_FORMAT1_ {
				UINT8     AP_MajorNumber;
				UINT8     AP_MinorNumber;
				UINT32    AP_BuildNumber;
			} BV_FORMAT1;
		} himu;
	} AP_BiosVersion;

    
    BOOL            AP_DMAForPIOTransfersSupported;

} LocalAdapterProfile, DPTR pLocalAdapterProfile;

#define     CDA_MAX_PHYIDS_PER_DEVICE       0x80


typedef struct _LU_VALIDATION_BUFFER
{
    UINT8   BufferInUseFlag;
    UINT8   LuValidationBuf[256];
    UINT8   Padding[7];
} LU_VALIDATION_BUFFER, *PLU_VALIDATION_BUFFER;

/*********************************************************************************
	Adapter Device
*********************************************************************************/
typedef struct _AdapterDevice
{
	pADAPTER_DEV_EXT pAdapterDevExt ;/* Pointer to Device Extension */


	ULONG            AD_PDCount ;		/* currently not used - TDE */
										/* should be removed */
	/*
		These fields are used for performing discovery. Since it's single threaded
		the parameters travel in these.
	*/
	/* TDE - BEGIN of scratch area - Holds values in device discovery temporarily */
	HIM_TASK_SET_HANDLE  AD_DeviceContext ;
	ULONG            AD_Index ;
	ULONG            AD_PhysTagsMax ;
	pSasDevice       AD_pSasInfo ;  /* Holds for each PD, temporarily */
	UCHAR            AD_SASAddress[ HIM_MAX_WWID_SIZE ]  ;  /* SAS ID as returned in discovery for a device port */
	ULONG            AD_Result ;
	HIM_TASK_SET_HANDLE    AD_DevicePortTSH ;
	HIM_TASK_SET_HANDLE    AD_SAS_AttachedDevicePortTSH ;
	HIM_TASK_SET_HANDLE    AD_PortTSHUnderDiscovery;
#if CDA_ENABLE_FTTT_SUPPORT
    UINT8            AD_SAS_AttachedPhyIdentifier;
    UINT8            AD_Sas_AttachedDeviceWWN[8];
#endif

	UCHAR            AD_Transport ;
	UCHAR            AD_SAS_TargetProtocol ;
	UCHAR            AD_SAS_DeviceType ;
	UCHAR            AD_WorldWideID[ HIM_MAX_WWID_SIZE ]  ; /* world wide ID of Razor chip */

	USHORT           AD_SAS_ConnectionRate ;
	USHORT           AD_SAS_NegotiatedPhyLinkRate ;

	/* AttachedPortId and AttachedTargetId are not currently
	 * implemented in CHIM. This values should be returned in
	 * HIM_DEVICEPORT_PROFILE structure.
	 */
	USHORT            AD_SAS_AttachedPortId ;	/* Identifies port id to which target is attached */	
	USHORT			 AD_SAS_AttachedTargetId;	/* Identifies the target or Bay number */

	USHORT			 AD_SAS_ITNexusLossTime;	/* TDE - added */
	USHORT           AD_DeviceBus ;
	USHORT           AD_DeviceId ;		/* not used */
	USHORT           AD_DeviceLUN ;
	USHORT           AD_DeviceCurrentSpeed ;
	USHORT           AD_DeviceMaxSpeed ;
    USHORT           AD_SAS_DPDiscoveryIndex;
	/* TDE - END of scratch area - Holds values in device discovery temporarily */
	

	UCHAR            AD_PhysLayerType ;
	UCHAR            AD_MaxLunPerId ;
	UCHAR            AD_BusWidth ;
	UCHAR            AD_BusSpeed ;

	ULONG            AD_Capabilities ;
	ULONG            AD_PhysVersion ;
	ULONG            AD_PhysMemoryNeeded ;
	ULONG            AD_MaxTargets ;
	CDA_BAR_INFO     AD_BAR[ 6 ] ;	
    UCHAR            Polling;
    UCHAR            PollingCmdCount;

	UCHAR			AD_State ;

    pIODescriptor	AD_pLuValidationIOD; /* Used for Lu Validation only */
	pIODescriptor	AD_pTaskIOD ;   /* Used for ENABLE_PHY, DISABLE_PHY, PROBE_LU */
	pIODescriptor	AD_pPacIOD ;	/* Used for PAC and RESET_ADAPTER */
    PVOID           AD_pAdapterPACIOD;/* Used for PAC at Adapter Level */
    PVOID           AD_pQuesceIOD;
    pIODescriptor   AD_pResetHardwareIOD;
    pIODescriptor   AD_pResetAdapterIOD;

	pSysDevice		pAD_Parent;
	USHORT			AD_IntTableIndex;/* For ack'ing ints	*/
	HIM_HOST_ID		AD_ChipId ;      /* 0x78709004, etc. */

	USHORT			AD_PCIVendorId;	/* PCI Vendor	*/
	USHORT			AD_PCIDeviceId;	/* PCI Device	*/
    USHORT			AD_PCISubVendorId;
    USHORT			AD_PCISubsystemId;
	UCHAR			AD_PCIDevFunct;	/* PCI Device & Function #s from Find_PCI_Device */
	UCHAR			AD_PCIBus;
	UCHAR			AD_PCIDevice;	/* AKA slot number or (numerical) device id. */
	UCHAR			AD_MemOrIO;
	UCHAR			AD_IntLine;		/* Level for the interrupt for this card */
	UCHAR			cAD_DelayResetCount;
	USHORT			AD_PhysInterface; /* Type of bus, PHYS_SCSI2, PHYS_SSA, etc */
	BOOL			bAD_ResetIsPending;
	BOOL			bAD_ResetStateChked;
	USHORT			AD_Slot;
	DATA_N_PTR		AD_BaseAddr;	/* IO Base Addr			*/
	DATA_N_PTR		AD_MemBase;		/* Mem Base addr		*/
	UCHAR			AD_Type;		/* Bus or xor adapter	*/
	UCHAR			AD_RelativeIndex;/* relative index to the controller */

	/* TDE */
	//UCHAR			Padding;	/*AD_SCSIId;	SCSI ID for the adapter */
	/* status to report to management application 
	 * Cannot use AD_State since it is used to maintain 
	 * adapter state information
	 */
	UCHAR			AD_Status;	
	UCHAR			AD_CrntDevices;	/* Num devices known to be attached	 */

	ULONG			Signature;
	HIM_TASK_SET_HANDLE	AD_AdapterTSH;		/* maybe an array later */
	HIM_TASK_SET_HANDLE	AD_InitTSH;
	VOID HIM_PTR	pAD_AdapterTSCB;    /* maybe an array later */
	VOID HIM_PTR	pAD_DOSState;	  	  /* maybe an array later */
	VOID HIM_PTR	pAD_ProtectedState; /* maybe an array later */
	ULONG			AD_SizeAdapterTSCB;
	pAdapterDevice	pAD_GroupADPtr;	   /* not used yet */
	HIM_HOST_ID		AD_PCIDeviceVendorId; /* 0x78709004, etc. */
	ULONG			AD_MaxIOHandles;	  /* taken from HIM_CONFIGURATION */
	HIM_IO_HANDLE	AD_IOHandles;		 /* pointer to an array of structs */

	VOID HIM_PTR	pAD_CHIMMemory;
	HIM_FUNC_PTRS	AD_HIMFunctions;

	LocalAdapterProfile		AD_Profile;

	pPhysDevice		pAD_AdapterPD;	/* for adapter-specific functions */

	/* MN-RK: SEEPROM Change: included new field for logical ordering of adapters */
	HIM_UINT8		AD_LogicalOrder; 

	UCHAR           eventIOFreeze;
    /*
     * It can have the following bit values
     */
    #define ADAPTER_FREEZE_NONE                                 0x00
    #define ADAPTER_FREEZE_INITIATED_BY_CHIM                    0x01
    #define ADAPTER_FREEZE_INITIATED_BY_CDA_DUE_TO_ADAPTER_PAC  0x02
    #define ADAPTER_FREEZE_INITIATED_BY_CDA_RESET_ADAPTER       0x04
    #define ADAPTER_FREEZE_INITIATED_BY_CDA_ADAPTER_FAILED      0x08
    #define ADAPTER_FREEZE_INITIATED_BY_CDA_RESET_SAS_HARDWARE  0x10
    #define ADAPTER_FRREZE_INITIATED_BY_CDA_RESET_IN_PROGRESS   0x20

    UCHAR           AD_PendingTaksToPerform;/* Contains the pending tasks to perform later*/
    /* Definition for PendingTaksToPerform */
    #define ADAPTER_PENDING_TASK_PORT_VALIDATION_REQUIRED            0x01
    #define ADAPTER_PENDING_TASK_DEVICES_VALIDATION_REQUIRED         0x02
    #define ADAPTER_PENDING_TASK_LU_DISCOVERY_REQUIRED               0x04
    #define ADAPTER_PENDING_TASK_DEVICE_PORT_INITIALIZATION_REQUIRED 0x08
    #define ADAPTER_PENDING_TASK_DEVICES_LU_VALIDATION_REQUIRED      0x10

	HIM_TASK_SET_HANDLE		pAD_PhyTSH[SAS_CHIP_MAX_NUM_PHY] ;
	HIM_TASK_SET_HANDLE		pAD_PortTSH[SAS_CHIP_MAX_NUM_PORTS] ;
	PortInformation			pAD_PortInformation[SAS_CHIP_MAX_NUM_PORTS] ;
	SasDevice		AD_SasDevices[ CDA_MAX_DEVICES_PER_ADAPTER ] ;
	pPhysDevice     pAD_PhysMap[ CDA_MAX_DEVICES_PER_ADAPTER ] ;
   
    /* These two are for special memory requirement per IOB function */
    //HIM_UINT32      MemTable[MAX_HIM_BUFFER_DESCRIPTOR_IN_IOB];
    HIM_UINT32      MaxSize;
    HIM_UINT32      LargestAlignment;  /* tt largest alignment mask of all iob functions */
    
    HIM_UINT32      SavedActionSet; /* original action set for the event provided by CHIM */
    PVOID           SavedEventContext; /* saved HIM context for handling events */
    ULONG           WorkingActionSet; /* action set as we progress through handling adapter events */
    USHORT          HIMReportedFailedPhysBitMask;
    USHORT          EnableRetryOnFailedPhyBitMask;

#ifdef SES2_DEVICE   
    /* Added to support SES-2 device */
    
    UCHAR           CrntSES2Index;
    UCHAR           CrntSES2_LEDIndex;
    UCHAR           WaitingToProcessNewTargets;
    UCHAR           Reserved;
#else
    UCHAR          Reserved;
#endif

    UCHAR           AD_PendingRequest;/* Contains the pending request to perform later in timer context*/

#ifdef CDA_FOR_SATA 
   HIM_WATCHDOG_FUNC	HIMWatchdogFunc;
   HIM_UINT32           HIMWatchdogInterval;   /* in microseconds */
#endif 

   UINT32           TotalChimMemSize;
   UINT8            CdaDelayNewPortEvent;

   HIM_AP_HARDWARE_CONSTRAINTS_PAGE	AD_ConstrainsPage;	 

   PLU_VALIDATION_BUFFER AD_LuValidationBufPtr;
   FPTR	            LastFunc;
   pIODescriptor    pRstIODsQueue;
   UINT16           AD_CMPDDNumber; //Crash Mode Parallel Discover Devices Number
#define     CDA_PARALLEL_DISCOVER_DEVICES_NUMBER    4
#ifdef OSM_DBGMSG_SAS_DUMP
   SAS_DUMP_OPTIONS  debugDumpOptions;  
#endif /* OSM_DBGMSG_SAS_DUMP */

} AdapterDevice;
/*********************************************************************************
	ControllerDevice
*********************************************************************************/
typedef struct _ControllerDevice 		/* structure size is 80 bytes */
{
/* 0x0000 */	pSysDevice			pCD_Parent;
/* 0x0004 */	ULONG				CD_BaseAddr_High;/* On a 64bit machine IO Base Addr				*/
/* 0x0004 */	ULONG				CD_BaseAddr;	/* IO Base Addr				*/
/* 0x0008 */	ULONG				CD_MemBase_High;/* On a 64bit machine Mem Base addr			*/
/* 0x0008 */	ULONG				CD_MemBase;		/* Mem Base addr			*/
/* 0x000C */	USHORT				CD_ControllerVersion;
/* 0x000E */	USHORT				CD_ControllerRevision;
/* 0x0010 */	USHORT				CD_IntTableIndex;/* For ack'ing ints		*/
/* 0x0012 */	USHORT				CD_PCIVendorId;	/* PCI Vendor 				*/
/* 0x0014 */	USHORT				CD_PCIDeviceId;	/* PCI Device 				*/
/* 0x0016 */	USHORT				CD_DiagResults;
/* 0x0018 */	UCHAR				CD_PCIDevFunct;	/* PCI Device & Function #s from Find_PCI_Device */
/* 0x0019 */	UCHAR				CD_PCIBus;
/* 0x001A */ 	UCHAR				CD_PCIDevice;
/* 0x001B */	UCHAR				CD_MemOrIO;		/* True if memory mapped */
/* 0x001C */	UCHAR				CD_IntLine;		/* Level for the interrupt for this card */
/* 0x001D */	UCHAR				CD_PCIType;
/* 0x001E */	UCHAR				cCD_XOREngines;	/* Count of sabres			*/
/* 0x001F */	UCHAR				cCD_Adapters;	/* Count of adapters here	*/
/* 0x0020 */	UCHAR				CD_AdapterBase;	/* Index for use with adapters */
/* 0x0021 */	UCHAR				cCD_TCount;		/* Use this timer count to decide when to issue */
/* 0x0022 */	USHORT				bCD_ResetIsPending;	/* board reset after a channel reset. */
/* 0x0024 */	pSysDevice			pCD_Adapters[MAX_ADAPTERS];
/* 0x0048 */	FPTR				pCD_Submit;

/* 0x004C */	UCHAR				CD_MaxAdapter;	/* Max adapters per controller */
/* 0x004D */	UCHAR				cArrayPause;	/* Number of active array that needs to be paused */
/* 0x004E */	UCHAR				Padding[2];
/* 0x0050 */	FPTR				pOSD_Pause;		/* OSD for pause */
} ControllerDevice;

#endif
