/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**************************************************************************

				PHYS_CONTEXT Structure

07/20/2004  16:56 Manjunat Narayanaswamy
Added constant AD_ADAPTER_EVENTS_PENDING  

07/23/2004  16:56 Manjunat Narayanaswamy
Added constant AD_PORT_EVENTS_PENDING AND AD_PAC_REQUIRED

Pointers to PhysLayer is filled during compile time in an array in device extension.

When creating a new PhysLayerXXX Module:
   1. Define a function PhysLayerXXXX

		void  PhysLayerXXXX( UINT32     FunctionNumber,
					    pPHYS_CONTEXT  pPhysHandle,						
						PVOID          pCmd,
						PVOID          pCmdToAbort )

		Where XXXX is the transport or h/w that this PhysLayer supports.
		e.g. PhysLayerU320

   2. Put the prototype of this name in coreprot.h

   3. Feed this name into raidglbl.h (where DeviceExtension is kept) in the array of
      function pointers called PhysLayers.


PhysLayer can call the following CDA functions directly.
	CdaPortReadWrite() ;
	CdaNotify() ;
	CdaGetPhysicalAddress() ;
	CdaMemZero() ;
	CdaMemAlloc() ;
	CdaMemAllocContiguous() ;


***************************************************************************/
#ifndef PHYSIF_INCLUDED
#define PHYSIF_INCLUDED

#define	CDA_MAX_PHYS_LAYERS			          4
#define	PHYS_INDEX_NEW_DEVICE                 0xFFFFFFFF

#define	PHYS_MINIMUM_POLLING_GRANULARITY      100
#define	PHYS_MINIMUM_COMMAND_TIME_OUT         5000
/************************************************************************
	One of the following functions need to be passed to PhysLayer

Functions                              Parameter1       Parameter2

PHYS_FUNCTION_INIT_STAGE1
PHYS_FUNCTION_INIT_STAGE2
PHYS_FUNCTION_QUIESCE
PHYS_FUNCTION_POLLED_IO                pIOB
PHYS_FUNCTION_ABORT_IO                 pIOB
PHYS_FUNCTION_RESET                    Target/Lun/Chip
PHYS_FUNCTION_GET_NEXT_DEVICE          Ttsh             Index
PHYS_FUNCTION_GET_DEVICE_PROPERTIES    Ttsh             pDeviceProperties
PHYS_FUNCTION_SET_DEVICE_PROPERTIES    Ttsh             pDeviceProperties
PHYS_FUNCTION_VALIDATE_DEVICE          Ttsh

PHYS_FUNCTION_READ_NV                  offset           length
PHYS_FUNCTION_WRITE_NV                 offset           length

PHYS_FUNCTION_SAVE_STATE               Real/Protected
PHYS_FUNCTION_RESTORE_STATE            Real/Protected

************************************************************************/
enum PhysFunctions {
	PHYS_FUNCTION_INIT_STAGE1,
	PHYS_FUNCTION_INIT_STAGE2,
	PHYS_FUNCTION_POLLED_IO,
	PHYS_FUNCTION_QUIESCE,	          /* Wait until all pending I/Os complete. */
	PHYS_FUNCTION_ABORT_IO,
	PHYS_FUNCTION_RESET,
	PHYS_FUNCTION_GET_NEXT_DEVICE,    /* Returns next device in chain as discovered earlier */
	PHYS_FUNCTION_RETURN_DEVICE,
	PHYS_FUNCTION_GET_DEVICE_PROPERTIES,
	PHYS_FUNCTION_SET_DEVICE_PROPERTIES,
	PHYS_FUNCTION_VALIDATE_DEVICE     /* Checks if the device is still connected */
} ;
/*********************************************************************
	The following types are filled by PhysLayer in PhysBusType
*********************************************************************/
enum PhysAdapterTypes {
	PHYS_TYPE_UNKNOWN,
	PHYS_TYPE_SCSI2,
	PHYS_TYPE_SSA,
	PHYS_TYPE_FC,
	PHYS_TYPE_P1394,
	PHYS_TYPE_SAS,
	PHYS_TYPE_ATA,
	PHYS_TYPE_SATA,
	PHYS_TYPE_XOR,
	PHYS_TYPE_SCSI160
} ;
/************************************************************************
	The following device types are filled by PhysLayer in PhysDeviceType
************************************************************************/
enum PhysDeviceTypes {
	PHYS_DEVICE_TYPE_DASD,
	PHYS_DEVICE_TYPE_TAPE,
	PHYS_DEVICE_TYPE_PRINTER,
	PHYS_DEVICE_TYPE_PROCESSOR,
	PHYS_DEVICE_TYPE_WORM,
	PHYS_DEVICE_TYPE_CDROM,
	PHYS_DEVICE_TYPE_SCANNER,
	PHYS_DEVICE_TYPE_OPTICAL,
	PHYS_DEVICE_TYPE_CHANGER,
	PHYS_DEVICE_TYPE_COMM,
	PHYS_DEVICE_TYPE_UNKNOWN,
	PHYS_DEVICE_TYPE_E_EXPANDER,
	PHYS_DEVICE_TYPE_F_EXPANDER
} ;
/************************************************************************
	New discovery function uses the following state definitions
************************************************************************/
enum DiscoveryStates {
	AD_DISCOVER_FIRST,
	AD_NEXT_DEVICE_PORT,
	AD_PAC_COMPLETE,
	AD_PROBE_LU_COMPLETE,
	AD_VALIDATION,
	AD_MD_READ_COMPLETE,
	AD_PAC_REQUIRED
} ;

/* union for SAS/SATA hardware control page */
typedef union HIM_YP_HW_CTRL_PAGE_UNION_
{
   HIM_YP_SAS_HARDWARE_CONTROL_PAGE    sas;
   HIM_YP_SATA_HARDWARE_CONTROL_PAGE   sata;
} HIM_YP_HW_CTRL_PAGE_UNION;

/*****************************************************************************
	The following bit definitions are filled by PhysLayer in PhysCapabilities
*****************************************************************************/
#define	PHYS_CAPABLE_R1_WRITE	1
#define	PHYS_CAPABLE_R1_VERIFY	2
#define	PHYS_CAPABLE_R1_REBUILD 4
/************************************************************************
	New command timeout and chim error handling uses the folliwing states
************************************************************************/
#define	AD_STATE_FROZEN         0x01
#define	AD_STATE_RESET_GOING_ON 0x02
#define	AD_STATE_PAC_GOING_ON   0x04
#define	AD_STATE_PAC_REQUIRED   0x08
#define	AD_STATE_DISCOVER_FIRST 0x10
#define AD_PORT_EVENTS_PENDING  0x20
#define AD_ADAPTER_EVENTS_PENDING  0x40

#define	AD_STATE_OFFLINE        0x80

#define	SAS_FLAGS_DEVICE_VALID	0x01
#define	SAS_FLAGS_EXPANDER		0x02
#define	SAS_FLAGS_ENDDEVICE		0x04
//#define SAS_FLAGS_DISCOVERED_RUNTIME 0x08
#define SAS_FLAGS_DISCOVERED_NEW_DEVICE 0x08

#define SAS_FLAGS_DEVICE_VALIDATION_NEEDED            0x10
#define SAS_FLAGS_DEVICE_MISSING                      0x20
#define SAS_FLAGS_PROBE_LU_REQUIRED                   0x40
#define SAS_FLAGS_DEVICE_PORT_INITIALIZATION_REQUIRED 0x80

#ifdef SES2_DEVICE
#define SAS_FLAGS_DEVICE_VERIFICATION_NEEDED      0x0100
#define SAS_FLAGS_DEVICE_TEST_ENDED               0x0200
#endif
#define SAS_FLAGS_DEVICE_LU_VALIDATION_NEEDED   0x0400
#define SAS_FLAGS_REDISCOVER_THIS_DEVICE        0x0800
#define SAS_FLAGS_TMF_DEVIE_RESET_PENDING 		0x1000

/************************************************************************
	CHIM Stubs
************************************************************************/

/*****************************************************************************
	These are the gateways to call the physical layer. Functions are passed
	inside FunctionNumber.
*****************************************************************************/
#ifndef SCO_DDI8

void	PhysLayerU320(  pAdapterDevice pAD,
					    UINT32         FunctionNumber,
						UINT32         Parameter1,
						UINT32         Parameter2 ) ;

void	PhysLayerSas(   pAdapterDevice pAD,
					    UINT32         FunctionNumber,
						UINT32         Parameter1,
						UINT32         Parameter2 ) ;
#endif /* SCO_DDI8 */

#endif
