/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:raiddef.h

File Description
	Define those structures used for raid manipulation.


***********************************************************************

History :

Jan 13, 2003: Madhuresh.
Added a field for exact LBA on which miscompare or medium error showed up.
see D_LbaToReport

09/10/2002 : 14:30 NAR20038  Manjunath
Reduced RD_PER_DEVICE count and if we run out of resources, we are
handling it gracefully.

05/14/2001 : 19:06 NAR20038  Manjunath
Added D_ReassignmentBuffer in D_Header.

03/22/2001 : 13:54  NAR20038  Manjunath
Fixed compilation warning messages.

12/13/2000 : 11:08 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Added D_pStripeBase and D_pXORBase to DHeader.
2. Removed XORDevice from WriteDescriptor and RecoveryDescriptor

***********************************************************************/
#ifndef RAIDDEF_INCLUDED
#define RAIDDEF_INCLUDED

#include "sysdef.h"

#define ABORT_WRITE             0x80
/*
	Define valid owner types
*/
#define WRITE_OWNER     1
#define RECOVERY_OWNER  2

/* Define the type of operation in RD_bRDControl. */
#define BACKGROUND_READ			0x0001
#define SYSTEM_READ				0x0002

/* define settings used for ioctl functions. These enable rebuild/verify */
#define VERIFY_OPERATION		0x0004
#define REBUILD_OPERATION		0x0008
#define IOCTL_OPERATION			VERIFY_OPERATION + REBUILD_OPERATION

/* Flags used for background VERIFY task */
#define V_REPORT_MISCOMPARES	0x0010
#define V_FIX_MISCOMPARES		0x0020
#define V_FIX_BAD_BLKS			0x0040

/* Define flag to show that the verify operation has done its read. */
#define V_READ_COMPLETE			0x0080
#define ABORT_RECOVERY			0x0100

/* Define flags for cache and disk update */
#define VR_DISK_UPDATE			0x0200
#define VR_CACHE_UPDATE			0x0400

#define QUICK_INIT				0x0800

/* Define a mask to clear these bits    */
#define VR_CLR_EVENTS           ~(VR_DISK_UPDATE + VR_CACHE_UPDATE)

#define BACKGROUND_WRITE        0x1000
#define IOCTL_RECOVERY          0x2000

#define SABRE_REQUEST_RESET		0x4000
#define ZERO_WRITE_SAME			0x8000

/* Define the flag setting for the RD_fReassign field.  */
#define NEED_REASSIGN           0x0001
#define START_REASSIGN          0x0002
#define REASSIGN_SUCCESS        0x0004
#define REASSIGN_FAILURE        0x0008
#define NEED_MEDIUM_ERR_RECOVERY 0x0010
#define IS_VERIFY               0x0020
#define STARTED_VERIFY          0x0040
#define FAILED_DISK             0x0080
#define REASSIGN_DEFERRED_ERROR 0x0100
#define FORCE_MEDIUM_ERROR          0x0200
#define FORCE_MEDIUM_ERROR_WR       0x0400
#define FORCE_MEDIUM_ERROR_RL       0x0800
#define FORCE_MEDIUM_ERROR_WL       0x1000
#define FORCE_MEDIUM_ERROR_RD       0x2000
#define FORCE_MEDIUM_ERROR_END_RD   0x4000
#define RESUME_MID_CHUNK_MED_ERR    0x8000
/*
	Define settings for noting WHEN an error occurred. Important for
	post processing of rebuild functions.
*/
#define FAILED_BEFORE_SUBMISSION  0x80
#define FAILED_AFTER_SUBMISSION   0x81
#define RD_PER_DEVICE             10
/*********************************************************************************
	DHeader

	Describe the header that is common to write and recovery descriptors. This
	header allows resource manipulators to handle the two types of activity
	descriptors in the same way.
*********************************************************************************/

typedef struct  _DHeader               /* structure size is 80 (0x50) bytes */
{
	struct _DHeader DPTR            pD_Next;
	struct _DHeader DPTR            pD_Pending;

	pSysDevice      pD_SysDevice;   /* Sys device parent */
	FPTR            pD_Entry;
	ULONG           D_StartBlock;
	USHORT          cD_Sectors;
	USHORT          D_OwnerType;    /* padding to dword alignment */
	USHORT          D_iDataMember;  /* Index to data member */
	USHORT          D_iParityMember;/* Index to parity member */
	USHORT          D_RetryCount;
	UCHAR DPTR		D_pStripeBase;
	UCHAR DPTR		D_pXORBase;
	UCHAR DPTR		D_ReassignmentBuffer;
	ULONG			D_LbaToReport ;
	UCHAR           D_CDB[ MAX_CDB_LEN ];
	pSGTable        pD_SGTable;
	SGTable         D_SGTable;     /* Only 1 element */

} DHeader;
/*********************************************************************************
	The Recovery descriptor

	This describes a recovery operation over some range of blocks.
*********************************************************************************/
typedef struct  _RecoveryDescriptor
{
	DHeader                 RD;
	/*
		Needed, Queued and Done are used for IODescriptor allocation. When
		Needed = Queued all requests are at devices. When Needed=Done all recovery
		events are done.
	*/
	USHORT          cRD_Available; /* IOD successfully allocated */
	USHORT          cRD_Needed;    /* IOD that need to be issued */
	USHORT          cRD_Queued;    /* IOD that were issued */
	USHORT          cRD_Done;      /* IOD that have been completed */
	USHORT          cRD_PhysDevice;/* Number of physical devices */
									/* that have been issued a request */
	USHORT          bRD_Control;    /*state flags used to control execution */
	USHORT          RD_fReassign;   /*Flags to control reassign     */
	BOOL            bRD_fPhysRead;
	BOOL            bRD_fIOCtlRecovery;
	ULONG           RD_SysReadOriginalLBA;
	pIODescriptor   pRD_FailedIO;
	pIODescriptor   pRD_Recovery[MAX_DEV_MEMBERS];
	pOSDescriptor   pRD_OSD;
	SGTable         RD_DefectListSGTable;
	UCHAR           RD_DefectList[8];

} RecoveryDescriptor;
/*********************************************************************************
	The Recovery Wait Descriptor

	Due to shortage of RDs, we need to queue the recovery request for a later
	processing. RD Wait Descriptors are meant for that.
**********************************************************************************/
typedef struct _RDWaitDescriptor
{
	struct _RDWaitDescriptor DPTR   pRDWait_Next;   /* Link ptr     */

	pOSDescriptor   pRDWait_OSD;    /* Pointer to an OSD  allocated to start an IOCtl operation. */
	FPTR        pRDWait_PostFunc1;  /* Call to this Post function to allocate a RD and
									   other resources to start the IOCtl. */
	FPTR        pRDWait_PostFunc2;  /* Call to this Post function which starts the operation */
	USHORT      cRDWaitIODs;		/* Maximum IODs needed to start the IOCtl operation.*/
	BOOL        bRDWaitFailedIODNeeded; /* Failed IOD needed when running rebuild */
										/* and verify, but not needed for Zero ioctl. */
} RDWaitDescriptor, DPTR pRDWaitDescriptor;     

#endif
