/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: raidglbl.h

File Description


***********************************************************************

History :
06/21/2006 mun24235
changed 32-bit MedErrPath to FeatureNumber and added 16-bit FeatureQualifier

07/26/2004 Ken Chang/Madhuresh
Added pModeSenseBuffer and pModeSenseBufferPhysical so that IOCTL can retrieve
Cache Mode pages (without using the pConfig or scratchbuffer).

10/14/04 Jimmy Z.
Added two more resource descriptor in ADE_CRD[] for RSU IODs and RSU Buffers.

10/13/04 Jimmy Z.
Added two pIODs for meta data update.

10/02/2004 Madhuresh
ADE_OperationMode

09/09/2004 Madhuresh
Added FreeMemoryPhysPtr

08/21/2004 Madhuresh
Removed some unused fields.

07/13/2004 Farrukh
Added two fields CrashDumpMemoryPtr and CrashDumpMemorySize to save 
crashdump uncached memory pointer. 
  
02/13/2004 Madhuresh 
Unused RP has been removed. One RP left casted in ADE so that
for Athena IOCTL, we never have to allocate RP.

11/16/2003 Madhuresh
Embeded CRD has to be an array not just one element.

11/20/2003 Tam
Add 3 fields in AdapterDeviceExtension as per CSMISAS needs for 
CSMI_GetRAIDConfig() call. 
1. CSMI_RAIDMemberIndex:  Index of DeviceMember in the RAID.
2. CSMI_RaidConfigStates:  States from CSMI_SendInquiry() state machine.
3. pInquiryBuffer:  pointer to 64 bytes buffer allocated in CSMI_Init(),
used to store temporary inquiry buffer.           

11/09/2003 Madhuresh
Embedding AD, CD etc into the DeviceExtension to avoid malloc issues
before intializing the CHIM. Removed storagetable from here. ConfigInfoBuffer
is gone.

10/15/2003 Madhuresh
Added version and description related fields. Aligned to CSMISAS doc.

10/13/2003 Madhuresh
Changed pSysDev to pAdapterSD and added pAD for easy access to adapter
device.

10/03/2003 Madhuresh
Added Drivername, Version String and Product name strings.

07/18/2003 : 15:00 RUIZ Tony
Added contants needed for having a varible to scan the number of luns with 
pAdapterDevExt->MaxLunsToScan.

06/16/2003: Madhuresh
Cleaned up unused stale fields and definitions.

Sept 24, 2002: Madhuresh
Four new fields for Hot Swap Rebuild.
When a disk is found at a scsi id which is still referred by the
meta data of any RAID1/10 as its missing member, the newly found disk 
is moved in as a member and a rebuild is initiated.
  
06/18/2002 : 12:05 BUR11729 Farrukh
typedef for struct _ADAPTER_DEV_EXT is forward declared in sysdef.h. 
We only need to define the struct here. Removed typedef from the struct
defination it was giving compilation error for Unix.

06/14/2002 : 16:35 NAR20038 Manjunath
Rolled back changes to ConfigInfoBuffer - will get back to this later when
clean solution is available.

06/14/2002 : 15:50 NAR20038 Manjunath
Changed type of ConfigInfoBuffer to PCICfgHeader from UCHAR[SZ_CONFIG_INFO].
Earlier definition posed problems while compilation in Unix.

05/29/2000 : 18:00 BUR11729 Farrukh
Removed gControllerCount since it is not used.

05/22/2000 : 17:43 BUR11729 Farrukh
ADAPTER_DEV_EXT and  DPTR pADAPTER_DEV_EXT is Forward declared in 
sysdev.h and remote.h since pAdapterDevExt member declaration is 
now changed from PVOID to pADAPTER_DEV_EXT for SysDevice and RemoteCtlPkt.

05/20/2002 : 18:51 BUR11729 Farrukh
added #define RAIDGLBL_INCLUDED to correct compilation error .

05/02/2002 : 12:40 BUR11729 Farrukh
Modified to eliminate compiler errors.

04/21/2002:  13:14 BUR11729 Farrukh
Him Management global pSCSIInitTSCB, gOSMRoutines, InitTSH, HimFunctions
are removed as global.

03/27/2002:  17:38 BUR11729 Farrukh
Moved Global pSysDeviceChild, pWaitingChange, cWaiting,
pSCSIInitTSCB, gOSMRoutines, gMaxIobReserveSize, InitTSH[MAX_CHIMS],
HimFunctions[MAX_CHIMS] to ADAPTER_DEV_EXT

Changed following variable declaration to defines and are moved here from raidglbl.c
szPrimaryTable, szCacheBlock, CacheBlockMask, BytesPerCacheBlock,
cFreeCacheBlocks, CacheShift, CacheMask, CacheNormalMask, RangeShift,
szRangeBlock, cRDWaitAllocs,

Moved These Global to ADAPTER_DEV_EXT
*pOtcBuffer, pPhysOtcBuffer, pBufferVirtualAddress, pRSWaitListHead, rsAccessMutex,
mdt_Mutex.

Moved defination of MAX_CHIMS, MAX_CONTROLLERS to global.h

Moved Debug Globals to global.h

Removed pPhysDevFreeList, InitIODSemaphore, cFreeCacheBlocks, Not used.


03/19/2002:  13:45 BUR11729 Farrukh
Global InitializationTime in moved to ADAPTER_DEV_EXT

03/18/2002:  14:40 BUR11729 Farrukh 
Moved pGlobalNewPD global to ADAPTER_DEV_EXT

02/18/2002:  18:44 BUR11729 Farrukh 
GloblList struct is combined with ADAPTER_DEV_EXT. Renamed ADAPTER_DEV_EXT
member PauseInProgress to ADE_PauseInProgress because GlobalList had a member
with same name and the two structures are merged now. Apparently PauseInProgress 
in both structures are used independently. We need to revisit it more closely.

02/19/2002 : 12:35 BUR11729 Farrukh
Moved GlobalList structure here from sysdev.h

02/18/2002 : 18:34 BUR11729 Farrukh
pSDGlblList has been moved from SysDevice to ADAPTER_DEV_EXT in an attempt
to make all global stored in one place.

02/07/2002 : 15:05 BURN11729 Farrukh
New file. struct _ADAPTER_DEV_EXT is moved here from nt.h for multi-controller
support. This struct is now OS independent an must be used by all OSs to 
access all the Globals. All the Globals will eventuly be accessable via 
this stuct by the time multi-channel support is completely finished. Some
defines are also moved here from nt.h. We have removed the queueing mechanism 
of SRBs and IOCTLS from the code thus making this struct independent of OS.
The pointers for SRB and IOCTL SRB queue have been removed from the struct.
These Globals are now stored in ADAPTER_DEV_EXT 
	SysHeader, pSysHeader, pGlobalSD, pPhysDevFreeList, EventTable
	SysHeaderCD, HeaderSDCRD[MAX_RESOURCE], SysHeaderRP, SysHeaderRP2
	SysHeaderOSD, cSystemControllers

***********************************************************************/
#ifndef RAIDGLBL_INCLUDED
#define RAIDGLBL_INCLUDED

#include	"sysdef.h"


#define	SZ_CONFIG_INFO				2048
#define	cRDWaitAllocs				20

#define NOTIFY_OSDEVICE_DELAY_COUNT 2	// TDE

#define MAX_DDF_IOD_PER_SD          2
#define MODE_SENSE_BUF_SIZE		0x100

/**************************************************************************

					Device Extension

ADAPTER_DEV_EXTENSION is passed for almost all calls to miniport layer.

***************************************************************************/
//#define CONFIG_HARDCODED_RAID 1

typedef struct  _RSUBuffer{
	ReservedSectorLayout   SysRSBuf;
    pRSUBuffer             pSysNextRSUBuffer;
} RSUBuffer;

#ifdef CDA_FOR_SATA
typedef struct VIRTUAL_PHYSICAL_PTRS_
{
    DATA_N_PTR  PhysicalMemPtrToSave;
    DATA_N_PTR  VirtualMemPtrToSave;
    UINT32      VirtualHigh4ByteToSave;
    UINT32      VirtualLow4ByteToSave;
    UINT32      BiosSgpioSet;
}VIRTUAL_PHYSICAL_PTRS, *PVIRTUAL_PHYSICAL_PTRS;
#endif

typedef struct BIOS_INFORMATION_
{
       UINT8	BIOSMajorRevision;   
       UINT8	BIOSMinorRevision;
       USHORT	BIOSReleaseRevision;
       UINT32	BIOSBuildRevision;
}BIOS_INFORMATION, *PBIOS_INFORMATION;

struct _ADAPTER_DEV_EXT
{

	/*
		A set of parameters, if available in registry should be read by OSM
		and programmed to the RAID Engine who keeps them right in the beginning
		of the device extension. Alternatively, these parameters may be stored
		in SEEPROM or NVRAM or Flash depending on the hardware underneath.
	*/
	USHORT          SgMax ;       /* Maximum number of elements in the SGList */
	USHORT          QueueDepth ;  /* Max commands we take from OS/Host. Now set thru regedit */

	UINT16          PhysTagsMax ; /* How many commands to send to each Lu (disk/target) */
	UINT8           NTCrashDetected ;/* Are we to run in reduced resource mode? */

	/* Adapter RAID capability */
	UINT8           ADE_CoercionPolicy ;/* Method applied for Disk Size Coercion? */
	UINT8           ADE_SupportHSR ;   /* HotSwapRebuild Support is required */
	UINT8           ADE_SupportGlobalSpare ;
	UINT8           ADE_SupportDedicatedSpare ;
    UINT8           ADE_MdStampingPolicy ;     /* OEM preference on stamping the MD with user warning or never stamping it? */
    UINT8           ADE_MemoryMapped ;         /* Program hardware/chim using memory mapping or IO mapping */
    UINT8           ADE_ForceFlushOnSata ;     /* Some SATA disks do not flush the cache correctly. Do we force? */
    UINT8           ADE_SupportCsmi ;          /* Per csmisas specs */
    UINT8           ADE_SuppportPsuite ;       /* PSUITENT ioctls are enabled or not */
#ifdef AUTO_REBUILD
	UINT8           ADE_SuppportAutoReconstruct ;
#endif
	USHORT          MaxPhysDevices ;
	USHORT          MaxSysDevices ;
	USHORT          OSDCount ;
	USHORT          IODCount ;
	USHORT          SpecialIODCount ;
	USHORT          RPCount ;
	USHORT          RDCount ;
	USHORT          WDCount ;

    UINT32          ADE_OperationMode ;	/* CDA_OP_MODE_LEAST_MEMORY, CDA_OP_MODE_NO_RAID */

	DATA_N_PTR		BaseAddress;
	ULONG           BAR[6] ;

	USHORT          ADE_PauseInProgress;
	UCHAR           IOCtlInProgress;
        UCHAR           NeedOps;
#if ENABLE_DRIVE_CACHE_SETTING
	UCHAR DPTR		pModeSenseBuffer;			/* Added to support WCE */
	DATA_N_PTR		pModeSenseBufferPhysical;	/* Added to support WCE */
#endif
	DATA_N_PTR      FreeMemoryPtr;		/* Still free space in uncached area */
	DATA_N_PTR		FreeMemoryPhysPtr;
	ULONG           FreeMemorySize;		/* Size of free space */
	/*
		Stores the pointer of memory allocated for future dump drive use
		Note the size of this memory is a not a constant and it is defined 
		pAdapterDevExt->CrashDumpMemLength
	*/
	DATA_N_PTR		CrashDumpMemoryPtr;

	/*
		Linked list of discovered SDs, adapter SD and AD,
		one global SD and PD for rescan etc
	*/
	pSysDevice      pStorageHeader ;
	pSysDevice      pAdapterSD ;
	pAdapterDevice  pAD ;
	pSysDevice      pTestSD ;
	pPhysDevice     pTestPD ;

	/*
		Embedding AD, CD etc into the DeviceExtension to avoid malloc issues
		before intializing the CHIM.
	*/
	SysDevice		ADE_CSD ;
	SysDevice		ADE_ASD ;
	AdapterDevice	ADE_AD ;
	ControllerDevice				ADE_CD ;
	ControllerResourceDescriptor	ADE_CRD[ MAX_RESOURCE ] ;
	/*
		Pointers to the IOCTL packet holder, common internal MD view holder...
	*/
	UCHAR DPTR      IOCTLDataBuffer ;
    /*
     * The flag to lock this buffer before using it is
     * ConfigBufferInUse flag. Set this flag to 1 to lock it, and
     * reset it to zero when a task is done using it.
     */
	PVOID           pConfig ;
	DATA_N_PTR		ConfigPhysAddr;
	UINT32          ConfigBufSize ; 

#ifdef SES2_DEVICE
    UCHAR           I2CDevCount;
    UCHAR           AbortLEDActivity;
    /* 
     * To Enable Device activity indicator LED, we need to allocate
     * a dedicated buffer and an IOD to turn on activity LED
     * in every 1 sec.
     */
    UCHAR           LEDBufferInUse;
    UCHAR           EnableLEDActivityPerSec;

    PVOID           pLEDBuffer;    

    pIODescriptor   pLED_IOD;
#endif


	UINT32          TraceIndex ;
    PVOID           pAP_ParentOSD;

	/*
		Rescan related globals
	*/
	pOSDescriptor   pScanOSD ;
	pIODescriptor   pScanIOD ;
    /*
     * RescanInProgress, bPACInProgress
     * UpdatingCheckPoint, UpdatingPTLMap, and fHsrActive
     * - all these flags used to lock pConfig buffer before
     * using as a scratch buffer for metadata read/write
     * purpose have been renamed to ConfigBufferInUse flag. Whenever
     * a task needs to get this buffer it has to lock it before
     * it can be used. After this buffer usage is done, it
     * has to unlock and start the task who is waiting for this
     * buffer.
     */
    UCHAR           ConfigBufferInUse;    
	//UCHAR           RescanInProgress;
	UCHAR           RescanAfterReset;
	UCHAR           ScanId ;
	UCHAR           ScanLUN ;
	UCHAR           DevicePresent ;
	UCHAR           MaxLunsToScan;
    UCHAR           Unused2[2];
	/*
		Internal MD view (ReservedSector) related globals
	*/
//	UCHAR           DiskFuncModeFlag ;
	//pOSDescriptor   pDiskFuncWriteOSD ;


	pSysDevice      PACPendingList ;
	PVOID           pRescanPending ;
    
	UCHAR           PauseInProgress ;
    UCHAR           Unused3[3];
	//UCHAR           bPACInProgress ;
	//UCHAR           UpdatingCheckPoint ;
	//UCHAR			UpdatingPTLMap;

	SysDevice       SysHeader ;
	pSysDevice      pSysHeader ;
	pSysDevice      pGlobalSD ;
	/*
		The event log is managed as a circular buffer. Once full, more events
		are logged from the start of the queue.
		If there are more than MAX_EVENTS events only the last 16 
		events will be logged.
	*/
 	EventLogList    EventTable ;
	USHORT          cEventsToRead; /* Counter to keep track of the 
									  total events that has been reported*/ 
	
	/*
		Pause/Resume background task related globals
	*/
	pRecoveryDescriptor pPauseRDHead ;

	RemoteCtlPkt    ADE_RP ;		

	OSDescriptor    SysHeaderOSD;
	pPhysDevice     pGlobalNewPD;
	pSysDevice		pIOCTL_CurrentSD ;

	USHORT          cSystemControllers;
	USHORT          ControllerNumber;
	USHORT          InitializationTime;
	USHORT          cFoundAdapters;

	/*
		Old Athena board needed these globals - should go away
	*/
	pRemoteCtlPkt   pWaitingChange ;
	USHORT          cWaiting ;

	/*
		Initialization related globals

		To hold separate queue heads during initialization time
	*/
	pSysDevice      pSingleHead;
	pSysDevice      pArraydHead;
	pSysDevice      pOtherHead;
	pSysDevice      pSysDeviceChild ;
	BOOL            InitOSDSemaphore ;
	ControllerDevice                SysHeaderCD;
	ControllerResourceDescriptor    HeaderSDCRD[MAX_RESOURCE];

	/*
		CHIM requires this size of DMAable memory in each IOB.
	*/
	HIM_UINT32      gMaxIobReserveSize;


	pReservedSectorLayout   pBufferVirtualAddress ;


	//
	//	Pointers to a global Dell DDF MetatData buffer.
	//	-	This is the buffer for the actual read/write of DDF Metadata
	//		to/from the hard disks.
	//	-	This buffer is allocated during MDT_Initialize() using library
	//		function XMemAlloc.
	//	-	For NT, this buffer is 5.5K Bytes of uncached memory.

	UCHAR		DPTR		pDDFbuffer;					// Virtual  address
	DATA_N_PTR				pDDFbufferPhysAddr;			// Physical address
	UCHAR		DPTR		pDDFbufferGlobal;			// Virtual  address
	DATA_N_PTR				pDDFbufferGlobalPhysAddr;	// Physical address
	
	BOOL					bOsGraceFullShutdown;
	// To keep track of any Global updates on all drives
	BOOL					bGlobalUpdateNeeded;
	pSysDevice				pSD_DDF_UD; // DDF Globalupdate currently working with this sysdevice
	USHORT					iMem_DDF_UD; // DDF Globalupdate currently working with this member
	USHORT					NotifyOSDeviceAdded;

	/*
		The following definitions are used to make a queue of IODs using field in IOD,
		so that multiple meta data i/o callers are handled one at a time. This was needed
		to make the functions here re-entrant.
	*/
	pIODescriptor   pRSWaitListHead ;
	PVOID           rsAccessMutex ;
	USHORT          mdt_Mutex ;
	USHORT          mdt_padding ;

	/*
		The following IODs are internally used for meta data update.
	*/
    pIODescriptor           pDDF_RS_IOD[MAX_DDF_IOD_PER_SD];

    /*
		Hot Swap Rebuild (HSR) related globals.
	*/
	pSysDevice      pHsrSD ;			/* HSR is currently working on this RAID sysdevice */
	USHORT          iHsrMissingIndex ;	/* HSR is currently working on this member index of pHsrSD */
    USHORT          Unused4;
	//USHORT          fHsrActive ;		/* NOT USED in RAZOR - Flag denotes Scan in progress for HSR. */

	/*
		Verify/Rebuild buffers
	*/
	RAIDBuffer      RAIDBuffers [ MAX_RAID_BUFFERS_FOR_BG_TASKS ];
        SenseBuffer     SenseBuffers[CDA_MAX_DMA_SENSE_BUFFERS]; /* sense buffers to support long sense data */
                                                                  /* for removable media devices only */

	/*
		Name and other constants for driver and RAID engine.
	*/
    UCHAR           ADE_ProductName[ CDA_PRODUCT_NAME_SIZE ] ;    /* OSM should fill this */
    UCHAR           ADE_DriverName [ CDA_DRIVER_NAME_SIZE ] ;     /* OSM should fill this */
	UINT8           ADE_DriverDescription[ CDA_DRIVER_DESCRIPTION_SIZE ] ;/* OSM should fill this */
	USHORT          ADE_usMajorRevision ;
	USHORT          ADE_usMinorRevision ;
	USHORT          ADE_usBuildRevision ;
	USHORT          ADE_usReleaseRevision ;    /* 5 fields above are as per CSMISAS needs */

	
	USHORT          CSMI_RAIDMemberIndex ;
	USHORT          CSMI_RaidConfigStates ;
	PVOID           pInquiryBuffer ;           /* 3 fields above are as per CSMISAS needs */

#ifdef CONFIG_HARDCODED_RAID	
	
	PVOID			pHC_RSBuffer;
#endif

#if SMART_SUPPORT
	USHORT			Smart_TCount;		       /* time in ticks for S.M.A.R.T event pooling */
#endif    
	UCHAR			TicksPerSec;
	UCHAR			DevIORetryCnt;
    UCHAR           MaxTargetIdAllowed;
    UCHAR           FaultToleranceTimeout;
#ifdef CDA_FOR_SATA
#define     FTTT_DEFAULT_VALUE          0 //Seconds
#else
#define     FTTT_DEFAULT_VALUE          8 //Seconds
#endif
#define     FTTT_MAXIMUM_VALUE          50//Seconds
#define     FTTT_MINIMUM_VALUE          0
    USHORT          ITLNexusLossTimeout;
    UCHAR           RestartFlag;
    //UCHAR           Unused5[1];
	UCHAR           UpdatingMetaDataAtStopCommand;
	
	/*
		Device Map Table - has to go away from here to OSM portion.

		Create a storage table so that request from OS will be easier to handle by reported bus, target id and Lun.
	
	pSysDevice		pStorageTable [MAX_CHANNEL_PER_PCI_FUNCTION][MAX_PER_BUS][MAX_LUN_PER_ID_SUPPORTED];
	*/
#ifdef  SRB_TO_IOD_SGLIST
    SGElement		    * IO_SGElements;
#endif
/*++
 PT MAP Table Definitions */
    pDeviceUniqueMap    pDevicePTMapTable;
    /*
     * Device PT Map Table memory pointers
     */
    DATA_N_PTR      pPTMemPtr;
    LUN_DATA        LunData;
    UINT8           ADE_RaidPath;
    UINT8           ADE_ReservePaths;
    UINT8           ADE_VSAModePath;
    UINT8           ADE_MaxLuns;
    UINT8           ADE_PathFirst;
    UINT8           ADE_OsmMaxPath;
    UINT16          ADE_MaxTargetsAllowed;
    UINT16          HorizontalCount;
    UINT16          VerticalCount;
    UINT16          WidthCnt;
    UINT8           ADE_EnableBootDeviceMap;
#ifdef SES2_DEVICE
    UINT8           ExposeEnclosure;
    UINT8           EnableLED;
#endif
/*--
 PT MAP Table Definitions */
#ifdef CDA_FOR_SATA
    VIRTUAL_PHYSICAL_PTRS   VirtualPhysicalPtrs;
#endif
    UINT16           ADE_MaxSataQDepth;
    UINT8           FTTTProcessingFlag;
    pPhysDevice     pISRIODPD;
BIOS_INFORMATION BiosInformation;
	UINT32          CrashDumpMemLength;

    pIODescriptor   pCSMI_AtHIM;

    PVOID           pCSMIIOD;
    BOOL            CSMIIODUsed;

	UINT16			FeatureNumber;
	UINT16			FeatureQualifier;
#define FEATURE_ENABLED		0x8000
#define MED_ERR_IBM_PCR13	0x0001
#define MED_ERR_MISSION_BAY	0x0002
#define MED_ERR_NON_IBM		0x0003

#define FAIL_VERIFY_BLOCK_RECOVERY_FAILED	0x0001
#define ENABLE_FORCE_ME_TARGET				0x0002
#define BLOCK_RECOVERY_ON_READ_FAILURE		0x0004

#define MEH_FEATURE_QUALIFIER_MISSION_BAY  (FEATURE_ENABLED| \
										    FAIL_VERIFY_BLOCK_RECOVERY_FAILED|\
											BLOCK_RECOVERY_ON_READ_FAILURE)
} ; /* ADAPTER_DEV_EXT , DPTR pADAPTER_DEV_EXT - Forward declared in typedef.h */

//#define TRACE_DEBUG 1
/* DEBUG GLOBAL */
#ifdef TRACE_DEBUG
extern ULONG	ResetErrorCount;
extern ULONG	MediumErrorCount;
extern BOOL	TraceOn;
extern TracesLogList  TraceTable;
#endif


#endif



