/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/******************************************************************************
* File Name : resource.h
*
*   In order to serialize access to resources used by Windows NT, a separate
* pool will be managed for each controller. Here are the structures that will
* manage the resources. It is assumed that the SysDev representing each
* controller will   contain a pointer to an array of these structuresbut that this
* array will be represented by a simple pointer to a structure. Access to
* elements beyond the first one will handled by pointer math.
*
* The pointer to the controller's resources should also be copied into the
* sysdevice representing each array/single attached to the controller. This
* will reduce pointer overhead and allow quick access to the resources.
*
***********************************************************************

History :
09/23/2004 : 14:41 BUR11729 Farrukh
Updated to accommodate memory resources for Device and LU TSCBs.

12/11/2001 : 17:10 NAR20038  Manjunath
Added FreeDiscriptorCount for tracking freeIODs in the list.

12/13/2000 : 11:08 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.


******************************************************************************/
#ifndef RESOURCE_INCLUDED
#define RESOURCE_INCLUDED
typedef struct _ControllerResourceDescriptor
{
 /* 0000 */	PVOID   pFreeListHead;
 /* 0004 */	PVOID   pFreeListTail;
 /* 0008 */	PVOID   pWaitListHead;
 /* 000C */	PVOID   pWaitListTail;
 /* 0010 */ ULONG	FreeDescriptorCount;
 /* 0014 */	PVOID   accessMutex;
} ControllerResourceDescriptor, DPTR pControllerResourceDescriptor;

#define MAX_RESOURCE            15   

#define OSD_RESOURCE		0	/* Descriptor for OSD */
#define IOD_RESOURCE		1	/* Descriptor for IOD */
#define WD_RESOURCE			2	/* Descriptor for Write Descriptor */
#define RD_RESOURCE			3	/* Descriptor for Recovery Descriptor */
#define RD_WAIT_RESOURCE	4	/* Descriptor for Recovery wait Descriptor */
#define RP_RESOURCE			5	/* Descriptor for Remote packet */
#define SD_RESOURCE			6	/* Descriptor for Free Sys device list */
#define PD_RESOURCE			7	/* Descriptor for Free Physical device list */
#define SCB_RESOURCE		8	/* Descriptor for SCB */
#define DEV_TSCB_RESOURCE	9	/* Descriptor for Device TSCB */
#define LU_TSCB_RESOURCE	10	/* Descriptor for LU TSCB */

/* Manju */
#define SPECIAL_IOD_RESOURCE	11 /* Descriptor for Special IOD */

#define RSU_IOD_RESOURCE	12  /* Descriptor for RSU IOD */
#define RSU_BUFFER_RESOURCE	13  /* Descriptor for RSU Buffer */

#define  CSMI_IOD_RESOURCE       14   /* Descriptor for CSMI IOD */
#endif
