/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/*$Header: Y:\chimdev\sas\src\sasosm.h_t   \main\100   Tue Mar 15 22:10:54 2005   hu11135 $*/

/***************************************************************************
*
*  Module Name:   SASOSM.H
*
*  Description:   SAS implementation specific customization
*
*  Owners:        SSG CHIM Team
*
*  Notes:         None
*
***************************************************************************/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* Customization unique to implementation                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/*------------------------------------------------------------------------------

 The table below show supported combinations of SAS hardware & firmware:

                       AIC941X        AIC958X
                       -------        -------
SAS_941X_MODE             X    

SAS_958X_HOST_MODE                       X

SAS_958X_IOP_MODE                        X

 For example, if we would like to enable support for AIC-9410 only then use
   #define SAS_AIC941X          1
   #define SAS_941X_MODE        1
 The rest of AIC & firmware options can be set to 0 to reduce code size.
 
 NOTE : No other firmware mode is supported when SAS_958X_IOP_MODE is 
        selected for SAS_AIC958X. On the other hand both SAS_941X_MODE & 
        SAS_958X_HOST_MODE can be selected to support both hardwares. 

------------------------------------------------------------------------------*/
#include "osmcda.h"

/* SAS hardware supported ----------------------------------------------------*/
/*   This section can be customized to a specific AIC if desired. Code size   */
/*   expands according to numbers of enabled AIC support.                     */

#define SAS_AIC941X                 1        /* Support AIC-9410W family      */

/* DO NOT MODIFY - THIS IS NOT COMPLETELY SUPPORTED YET */
#define SAS_AIC958X                 0        /* Support AIC-9580W family      */

/* Special SAS hardware supported --------------------------------------------*/
#define SAS_COMSTOCK_SUPPORT        0        /* Enable/Disable CHIM compile   */
                                             /* for COMSTOCK board.           */

/* DO NOT MODIFY - THIS IS NOT COMPLETELY SUPPORTED YET */
#define SAS_GALAXY_SUPPORT          0        /* Enable/Disable CHIM compile   */
                                             /* for GALAXY board.             */

/* Firmware mode supported ---------------------------------------------------*/
/*   This section can be customized to a specific firmware if desired.        */
/*   Code size expands according to numbers of enabled firmware support.      */
#define SAS_941X_MODE               1        /* Enable standard firmware      */
                                             /*  mode for AIC-9410 family     */
                                             /* This cannot be selected with  */
                                             /* SAS_958X_IOP_MODE             */

/* DO NOT MODIFY - THIS IS NOT COMPLETELY SUPPORTED YET */                                          
#define SAS_958X_HOST_MODE          0        /* Enable host firmware          */
                                             /*  mode for AIC-958x family     */
                                             /* This cannot be selected with  */
                                             /* SAS_958X_IOP_MODE          */

/* DO NOT MODIFY - THIS IS NOT COMPLETELY SUPPORTED YET */
#define SAS_958X_IOP_MODE           0        /* Enable iop firmware           */
                                             /*  mode for AIC-958x family     */
                                             /* This cannot be selected with  */
                                             /* SAS_941X_MODE and             */
                                             /* SAS_958X_HOST_MODE            */

#define SAS_ATAPI_SUPPORT           1        /* Enable ATAPI support          */

#define SAS_BIOS_SUPPORT            0        /* enable for BIOS supprt        */
#define SAS_DOWNSHIFT_SUPPORT       0        /* enable for donwshift support  */
#define SAS_ASSIGN_EXT_DDB          1        /* Assign off chip DDBS to expanders */


/* SAS hardware operation mode supported -------------------------------------*/
#define SAS_INITIATOR_OPERATION     1        /* Initiator operation mode      */
#define SAS_TARGET_OPERATION        0        /* Target operation mode         */

/* Optimization options ------------------------------------------------------*/
#define SAS_IO_SYNCHRONIZATION_GUARANTEED 0  /* PIO's are guaranteed to be    */
                                             /*  in order and not posted      */
#define SAS_DISCOVER_PROCESS_OPTIMIZATION 1  /* Use SAS 1.1 optimization for
                                                route table configuration     */
                                                
/* General options -----------------------------------------------------------*/
#define SAS_CHIM_SPEC_REV_5_SUPPORT       1  /* Support CHIM Interface change */
                                             /* applied for CHIM Spec Rev 5   */

#define SAS_OVERRIDE_OSM_NVRAM_ROUTINES   1  /* 0 - Use OSM routines for      */
                                             /*     NVRAM access.             */
                                             /* 1 - Use HIM routines for      */
                                             /*     NVRAM access.             */

#define SAS_PROFILE_INFORMATION     1        /* Support CHIM adapter,         */
                                             /*  deviceport and logical unit  */
                                             /*  profiles                     */

#define SAS_SASSELECT_SUPPORT       1        /* SASSelect information exist   */
#define SAS_BIOS_ASPI8DOS           1        /* Access to scratch ram that    */
                                             /*  BIOS/ASPI8DOS have           */

#define SAS_BACKENDISR_OUTSIDE_INTRCONTEXT 0 /* Defer HIMBackEndISR call      */
                                             /* outside of interrupt context  */

#ifdef C_MODEL  
#define SAS_PCI_COMMAND_REG_CHECK   0        /* Enable checking I/O, Memory   */
#else
#define SAS_PCI_COMMAND_REG_CHECK   1        /* Enable checking I/O, Memory   */
                                             /*  and Bus Master               */
#endif /* C_MODEL */

#define SAS_MAX_DEVICE_RESET_FAILURES  3     /* Number of device reset failures*/
                                             /* allowed before transitioning   */
                                             /* the device to invalid state    */

#define SAS_REGISTER_TIMEOUT_DEFAULT  2000   /* The default value (in         */
                                             /* milliseconds) for deciding    */
                                             /* to time out an operation that */
                                             /* is waiting on an ASIC reg     */
                                             /* state change.                 */

#define SAS_DEFAULT_NOTIFY_TIMER_TIMEOUT  4000
                                             /* The default value 4000 (in    */
                                             /* msecs) for time interval for  */
                                             /* transmission interval between */
                                             /* successive NOTIFY             */
                                             /* (ENABLE_SPINUP)primitive      */

#define SAS_PHY_COMINIT_TIMEOUT     10000
                                             /* The default value 10000 (in   */
                                             /* usecs) for Razor to time out  */
                                             /* the COMINIT process during OOB*/
                                             /* sequence with the drives.     */

#define SAS_DISABLE_INTERRUPTS      0        /* Does not enable the HA interrupt */
                                             /* when SCSIInitialize() is called. */

#define SAS_ENABLE_NCQ_SUPPORT      1        /* Enable/Disable CHIM support   */
                                             /* for SATA II Native Command    */
                                             /* Queueing feature.             */
                                             
#define SAS_ENABLE_LSI_ATAPI_WORKAROUND  1   /* LSI expander firmware has a   */
                                             /* problem for their virtual     */
                                             /* ATAPI SEP device. When we do  */
                                             /* REPORT_PHY_SATA, the response */
                                             /* comes in BIG ENDIAN as apposed*/
                                             /* to LITLLE ENDIAN as expected. */
                                             /* So workaround has been added  */
                                             /* which can be enabled by this  */
                                             /* flag                          */
#define SAS_ENABLE_SSP_TLR_SUPPORT  0
                                             /* Enable/Disable Transport layer*/

                                             /* retry support for SSP devices */
                                             /* This is only supported for    */
                                             /* Aurora hardware. This is not  */
                                             /* supported for Razor           */
                                             
#if SAS_BIOS_SUPPORT
#define SAS_BIOS_WORKAROUND_CHIP_RESET   0   /* When 1 will initiate the check
                                                for workaround for not resetting
                                                the chip                       */
#endif /* SAS_BIOS_SUPPORT */
#if SAS_DOWNSHIFT_SUPPORT
#define SAS_VERIFY_SEQUENCER_LOAD   0        /* When set to 1 the code loaded
                                              * into all sequencer instruction
                                              * RAMs will be verified by reading
                                              * back.
                                              */
#define SAS_DISABLE_PCI_PCIX_ERROR_HANDLING 1  
                                             /* Disables PCI or PCI-X error   */
                                             /* handling and reporting.       */
#define SAS_SEEPROM_ACCESS          0        /* Support SEEPROM access        */
#define SAS_SEQ_LOAD_USING_PIO      1        /* 1 - Enable loading of         */
                                             /*     sequencer using PIO       */
                                             /* 0 - Enable loading of         */
                                             /*     sequencer using the HOST  */
                                             /*     OVERLAY DMA feature       */
#define SAS_SENSE_DATA_BYTES_DEFAULT  18     /* Number of sense data bytes    */
                                             /* returned by CHIM              */

#define SAS_DISABLE_PROBE_SUPPORT   1        /* Disable support of HIM_PROBE  */
                                             /*  IOB function                 */

#define SAS_PINSET_DESCRIPTOR_SUPPORT 1      /* Support for manufacture pin   */
                                             /*  set descriptor data else use */
                                             /*  default  defines             */

#else /* !SAS_DOWNSHIFT_SUPPORT */
#define SAS_VERIFY_SEQUENCER_LOAD   1        /* When set to 1 the code loaded
                                              * into all sequencer instruction
                                              * RAMs will be verified by reading
                                              * back.
                                              */
#define SAS_DISABLE_PCI_PCIX_ERROR_HANDLING 1  
                                             /* Disables PCI or PCI-X error   */
                                             /* handling and reporting.       */
#define SAS_SEEPROM_ACCESS          1        /* Support SEEPROM access        */
#define SAS_SEQ_LOAD_USING_PIO      0        /* 1 - Enable loading of         */
                                             /*     sequencer using PIO       */
                                             /* 0 - Enable loading of         */
                                             /*     sequencer using the HOST  */
                                             /*     OVERLAY DMA feature       */


#define SAS_DISABLE_PROBE_SUPPORT   0        /* Disable support of HIM_PROBE  */
                                             /*  IOB function                 */

#define SAS_PINSET_DESCRIPTOR_SUPPORT 1      /* Support for manufacture pin   */
                                             /*  set descriptor data else use */
                                             /*  default  defines             */
#endif /* SAS_DOWNSHIFT_SUPPORT */

/* Specific customer customization options -----------------------------------*/
#define SAS_OEM1_SUPPORT            0        /* Support OEM1 customer         */

#ifdef SES2_DEVICE
#define SAS_I2C_SUPPORT                 1    /* I2C device support            */
#endif

#if (SAS_DOWNSHIFT_SUPPORT || SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT)
#undef  SAS_I2C_SUPPORT                      /* disable for downshift         */
#define SAS_I2C_SUPPORT                 0    /* I2C device support            */
#endif /* (SAS_DOWNSHIFT_SUPPORT || SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT) */

#define SAS_SGPIO_SUPPORT               0    /* SGPIO device support          */
#if (SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT)
#undef  SAS_SGPIO_SUPPORT                    /* disable for comstock          */
#define SAS_SGPIO_SUPPORT               0    /* SGPIO device support          */
#endif /* (SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT) */

#if SAS_SGPIO_SUPPORT
#if (SAS_DOWNSHIFT_SUPPORT || SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT)
#define SAS_SGPIO_SMP_DEVICE_SUPPORT    0    /* SGPIO SMP function support    */
#define SAS_SGPIO_SES_DEVICE_SUPPORT    0    /* SGPIO SES function support    */
#else /* (SAS_DOWNSHIFT_SUPPORT || SAS_COMSTOCK_SUPPORT) */
#define SAS_SGPIO_SMP_DEVICE_SUPPORT    1    /* SGPIO SMP function support    */
#define SAS_SGPIO_SES_DEVICE_SUPPORT    0    /* SGPIO SES function support    */
#endif /* (SAS_DOWNSHIFT_SUPPORT || SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT) */
#endif /* SAS_SGPIO_SUPPORT */

#if SAS_I2C_SUPPORT
#define SAS_I2C_INTERNAL_TDI_SUPPORT    1    /* Use CHIM's PCA9564 I2C channel*/
#define SAS_I2C_EXTERNAL_TDI_SUPPORT    0    /* Use OSM controlled I2C channel*/
#if SAS_I2C_INTERNAL_TDI_SUPPORT && SAS_I2C_EXTERNAL_TDI_SUPPORT
   #error Only one I2C TDI driver can be selected
#endif /* SAS_I2C_INTERNAL_TDI_SUPPORT && SAS_I2C_EXTERNAL_TDI_SUPPORT */
#if (!SAS_I2C_INTERNAL_TDI_SUPPORT) && (!SAS_I2C_EXTERNAL_TDI_SUPPORT)
   #error Atleast one I2C TDI driver should be selected
#endif /* SAS_I2C_INTERNAL_TDI_SUPPORT && SAS_I2C_EXTERNAL_TDI_SUPPORT */
#endif /* SAS_I2C_SUPPORT */

#define SAS_NOTIFY_BIOSDRV_TPGLYCHG     0    /* 1 - to enable the support for */
                                             /*     reporting topology change */
                                             /*     events from BIOS to driver*/
                                             /*     and vice versa.           */
                                             
#define SAS_SAVE_RESTORE_STATE          0    /* 1 - to enable support for Save*/
                                             /*     and Restore h/w state     */
                                             /* 0 - to disable support for    */
                                             /*     Save and Restore h/w state*/
#ifdef SAS_SAVE_RESTORE_STATE
#define SAS_SAVE_SEQUENCER              0    /* 1 - to enable saving and      */
                                             /*     restoring of sequencer,   */
                                             /*     if Save/Restore h/w state */
                                             /*     support is enabled.       */
                                             /* 0 - to disable saving and     */
                                             /*     restoring of sequencer,   */
                                             /*     if Save/Restore h/w state */
                                             /*     support is enabled. NOTE: */
                                             /*     This means sequencer will */
                                             /*     be loaded from Flash for  */
                                             /*     BIOS/ASPI modes and from  */
                                             /*     host memory for CHIM mode,*/
                                             /*     as part of restoring h/w  */
                                             /*     state.                    */
#endif /* SAS_SAVE_RESTORE_STATE */

/* Target Mode Specific Options [SAS_TARGET_OPERATION must be 1]--------------*/
#if SAS_TARGET_OPERATION
#define SAS_SCSI_PROTOCOL_SUPPORT   1        /* SCSI(SSP) Target mode protocol*/
                                             /* support                       */
#define SAS_NUM_INITIATOR_PORTS    24        /* The default number of initiator*/
                                             /* ports to be supported.        */
                                             /* Returned in HIM_CONFIGURATION */
                                             /* numInitiatorPorts.            */
                                             /* This setting reduces the      */
                                             /* numDevices supported by the   */
                                             /* value used here. Maximum      */
                                             /* is currently 24.              */
#define SAS_NUM_INTERNAL_REQUEST_BUFFERS 512
                                             /* The default number of internal*/
                                             /* request buffers to be         */
                                             /* supported. Returned in        */
                                             /* HIM_CONFIGURATION             */
                                             /* numInternalRequestBuffers. The*/
                                             /* maximum is currently 512.     */
#if SAS_SCSI_PROTOCOL_SUPPORT
#define SAS_NUM_SCSI_NEXUS_TSH  65535        /* The default number of SCSI    */
                                             /* nexus TSHs to be supported.   */
                                             /* Returned in HIM_CONFIGURATION */
                                             /* numSCSINexusTSHs. The maximum */
                                             /* is currently 65535.           */
#define SAS_MAX_RECEIVE_CDB_LENGTH    16     /* The default number of bytes   */
                                             /* of CDB supported for CDBs     */
                                             /* received when operatinging in */
                                             /* target mode for SCSI protocol.*/
                                             /* Returned in HIM_CONFIGURATION */
                                             /* maxReceiveCDBLength. The      */
                                             /* maximum is currently 16.      */
#endif /* SAS_SCSI_PROTOCOL_SUPPORT */

/* Internal use options. Do not modify ---------------------------------------*/

#define SAS_SMP_PROTOCOL_SUPPORT    0        /* SMP Target mode protocol      */
                                             /* support                       */

#if SAS_SMP_PROTOCOL_SUPPORT
#define SAS_NUM_SMP_NEXUS_TSH   65535        /* The default number of SMP     */
                                             /* nexus TSHs to be supported.   */
                                             /* Returned in HIM_CONFIGURATION */
                                             /* numSMPNexusTSHs.  The maximum */
                                             /* is currently 65535.           */
#define SAS_MAX_RECEIVE_SMP_REQUEST_LENGTH 0
                                             /* The default number of bytes of*/
                                             /* SMP frame for SMP requests    */
                                             /* received when operating in    */
                                             /* target mode for SMP protocol  */
                                             /* (excluding CRC). Returned in  */
                                             /* HIM_CONFIGURATION             */
                                             /* maxReceiveSMPRequestLength.   */
                                             /* The maximum is currently 0 as */
                                             /* SMP protocol is not supported.*/  
#endif /* SAS_SMP_PROTOCOL_SUPPORT */
#endif /* SAS_TARGET_OPERATION */

/* RAID1 ---------------------------------------------------------------------*/
#define SAS_IROC                    0        /* Support Iroc HW               */
#define SAS_RAID1                   0        /* Support RAID 1 Sequencer      */

/* Debug Mode Specific Options -----------------------------------------------*/
/* Enable debugging features. Debug features are enabled via both compile-    */
/* time and run-time options. To simplify the interface a common set of HIM   */
/* debug defines are used for both compile-time and run-time setting. These   */
/* defines are used as a bitmask of the features to be supported.             */
/*                                                                            */
/* The define SAS_OSM_DEBUG_MODE provides the compile time setting. The OSM   */
/* writer must declare an OSMDebugMode variable large enough to support the   */
/* number of HIM_DEBUG bitmask defines (an extern reference is provided in    */
/* CHIMOSM.H_T) to enable debug output at run-time.                           */
/*                                                                            */
/* For example, SAS_OSM_DEBUG_MODE may be set to HIM_DEBUG_ALL, enabling      */
/* compilation of all debug code. Then OSMDebugMode may be set to             */
/* (HIM_DEBUG_FUNCTION_ENTRY | HIM_DEBUG_FUNCTION_EXIT |                      */
/*  HIM_DEBUG_FUNCTION_ERROR_EXIT) to enable function entry and exit output.  */
/*                                                                            */
/* Please note that; the debug output is under development and not all        */
/* options may be available or provide complete coverage.                     */
/*                                                                            */
/* Output related options are:                                                */
/* HIM_DEBUG_ALL             - all options are set                            */
/* HIM_DEBUG_ERROR           - output for some error condition related to     */
/*                             coding, for example error default case in case */
/*                             statement                                      */
/* HIM_DEBUG_FUNCTION_ENTRY  - output function entry and any function         */
/*                             parameters                                     */
/* HIM_DEBUG_FUNCTION_EXIT   - output function exit with return value, (when  */
/*                             non void function)                             */
/* HIM_DEBUG_FUNCTION_ERROR_EXIT - output function error exit with return     */
/*                                 value (when non void function) and line    */
/*                                 number                                     */
/* HIM_DEBUG_INFO            - output some useful data structure information, */
/*                             for example, done list entry                   */
/* HIM_DEBUG_WARNING         - output some debug warning information          */
/* HIM_DEBUG_DEBUG           - general debug added by CHIM programmer for     */
/*                             specific problem                               */
/* HIM_DEBUG_DEVICE_DISCOVERY- output device discovery related information    */
/* HIM_DEBUG_DUMP            - dump chip registers and chip memory when CHIM  */
/*                             is not running (look below for various Dump    */
/*                             Mode options)                                  */
/*                                                                            */
#ifdef OSM_DBGMSG_SAS_DUMP
#define SAS_OSM_DEBUG_MODE          HIM_DEBUG_ALL
#else /* OSM_DBGMSG_SAS_DUMP */
#define SAS_OSM_DEBUG_MODE          0
#endif /* SAS_OSM_DEBUG_MODE */

/* OSM Dump Mode Specific Options -----------------------------------------------*/
/* Enable debugging features. Debug features are enabled via both compile-    */
/* time and run-time options. To simplify the interface a common set of HIM   */
/* debug defines are used for both compile-time and run-time setting. These   */
/* defines are used as a bitmask of the features to be supported.             */
#ifdef OSM_SAS_DUMP_SHARED_STRUCT
#define SAS_OSM_DUMP_SHARED_STRUCT_MODE     SAS_SHARED_STRUCT_ALL
#else /* OSM_SAS_DUMP_SHARED_STRUCT */
#define SAS_OSM_DUMP_SHARED_STRUCT_MODE     0
#endif /* OSM_SAS_DUMP_SHARED_STRUCT */
#ifdef OSM_SAS_DUMP_CHIP_MEMORY
#define SAS_OSM_DUMP_CHIP_MEMORY_MODE       SAS_CHIP_MEMORY_ALL
#else /* OSM_SAS_DUMP_CHIP_MEMORY */
#define SAS_OSM_DUMP_CHIP_MEMORY_MODE       0
#endif /* OSM_SAS_DUMP_CHIP_MEMORY */
#ifdef OSM_SAS_DUMP_REGISTERS
#define SAS_OSM_DUMP_REGISTERS_MODE         SAS_REGISTERS_ALL
#else /* OSM_SAS_DUMP_REGISTERS */
#define SAS_OSM_DUMP_REGISTERS_MODE         0
#endif /* OSM_SAS_DUMP_REGISTERS */
#ifdef OSM_SAS_DUMP_QUEUE
#define SAS_OSM_DUMP_QUEUE_MODE             SAS_QUEUE_ALL
#else /* OSM_SAS_DUMP_QUEUE */
#define SAS_OSM_DUMP_QUEUE_MODE             0
#endif /* OSM_SAS_DUMP_QUEUE */
#ifdef OSM_SAS_DUMP_STRUCT
#define SAS_OSM_DUMP_STRUCT_MODE            0
#else /* OSM_SAS_DUMP_STRUCT */
#define SAS_OSM_DUMP_STRUCT_MODE            0
#endif /* OSM_SAS_DUMP_STRUCT */


/* Internal use options. Do not modify ---------------------------------------*/
#define SAS_ENABLE_PHY_BYPASS_OOB   0        /* Enable PHY Bypass feature     */
                                             /* 0 - Bypass OOB disabled       */
                                             /* non-zero - value to be loaded */
                                             /* into SAS_LmOOB_MODE register. */
                                             /* Currently only the following  */
                                             /* are tested:                   */
                                             /* (SAS_OOB_MODE_SAS_MODE |      */
                                             /*  SAS_OOB_MODE_PHY_SPEED_15)   */
                                             /* These values are set by       */
                                             /* default when                  */
                                             /* SAS_SIMULATION_SUPPORT is 1.  */

#define SAS_DISABLE_BIST            0        /* Disable ASIC Built-in Self
                                              * Test feature.
                                              */

#define SAS_SIMULATION_SUPPORT      0        /* Enable/Disable CHIM compile   */
                                             /* for simulation environment.   */

#define SAS_ATA_TAGGED_QUEUING_SUPPORT   0   /* Tagged queuing is not         */
                                             /* currently supported for       */
                                             /* HIM_PROTOCOL_ATA targets in   */
                                             /* this CHIM.                    */


#define SAS_NVRAM_8BITS             1        /* Flash part 8bits/16bits       */      

/* Change this option to 1 when testing sequencer from CHIM, bypassing flash  */
/* with SAS_DOWNSHIFT_SUPPORT set to 1 only.                                  */
#define SAS_LOAD_CHIM_SEQUENCERS    0        /* Enable/Disable loading the    */
                                             /* sequencer from CHIM object    */
                                             /* rather than flash memory.     */   

#define SAS_SATA_3G_SUPPORT         0        /* Set to 1 for HW that supports */
                                             /* SATA 3G rate.                 */   
/* Hardware Simulation Only. Do not modify -----------------------------------*/
#if SAS_SIMULATION_SUPPORT

/* Always undefine the option before attempting to assign a simulation value  */

/* SIM: General options ------------------------------------------------------*/

#ifdef SAS_SEEPROM_ACCESS
#undef SAS_SEEPROM_ACCESS
#define SAS_SEEPROM_ACCESS          0        /* Support SEEPROM access        */

#endif /* SAS_SEEPROM_ACCESS */

#ifdef SAS_BIOS_ASPI8DOS
#undef SAS_BIOS_ASPI8DOS
#define SAS_BIOS_ASPI8DOS           0        /* Access to scratch ram that    */
                                             /*  BIOS/ASPI8DOS have           */
#endif /* SAS_BIOS_ASPI8DOS */

#ifdef SAS_SASSELECT_SUPPORT
#undef SAS_SASSELECT_SUPPORT
#define SAS_SASSELECT_SUPPORT       0        /* SASSelect information exist   */
#endif /* SAS_SASSELECT_SUPPORT */

#ifdef SAS_DISABLE_BIST
#undef SAS_DISABLE_BIST
#define SAS_DISABLE_BIST            1        /* Disable ASIC Built-in Self
                                              * Test feature.
                                              */
#endif /* SAS_DISABLE_BIST */

#ifdef SAS_VERIFY_SEQUENCER_LOAD
#undef SAS_VERIFY_SEQUENCER_LOAD
#define SAS_VERIFY_SEQUENCER_LOAD   0        /* Disable verify for simulation */
#endif /* SAS_VERIFY_SEQUENCER_LOAD */

#ifdef SAS_ENABLE_PHY_BYPASS_OOB
#undef SAS_ENABLE_PHY_BYPASS_OOB
/* Set SAS BYPASS OOB at 3.0 G/bit as default */ 
#define SAS_ENABLE_PHY_BYPASS_OOB   (SAS_OOB_MODE_SAS_MODE | SAS_OOB_MODE_PHY_SPEED_30)
#endif /* SAS_ENABLE_PHY_BYPASS_OOB */

#ifdef SAS_PHY_COMINIT_TIMEOUT
#undef SAS_PHY_COMINIT_TIMEOUT
#define SAS_PHY_COMINIT_TIMEOUT     1        /* COMINIT time-out value        */
#endif /* SAS_PHY_COMINIT_TIMEOUT */

/* CHIM will select correct sequencer for simulation based on design */
#if SAS_DOWNSHIFT_SUPPORT
#undef SAS_LOAD_CHIM_SEQUENCERS
#define SAS_LOAD_CHIM_SEQUENCERS    1        /* Enable/Disable loading the    */
                                             /* sequencer from CHIM object    */
                                             /* rather than flash memory.     */   
#endif /* SAS_DOWNSHIFT_SUPPORT */
#endif /* SAS_SIMULATION_SUPPORT */

/* COMSTOCK Only. Do not modify ----------------------------------------------*/
#if (SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT)

/* Always undefine the option before attempting to assign a value for Comstock */

#ifdef SAS_SEEPROM_ACCESS
#undef SAS_SEEPROM_ACCESS
#define SAS_SEEPROM_ACCESS          0        /* Support SEEPROM access        */
#endif /* SAS_SEEPROM_ACCESS */

#ifdef SAS_SASSELECT_SUPPORT
#undef SAS_SASSELECT_SUPPORT
#define SAS_SASSELECT_SUPPORT       0        /* SASSelect information exist   */
#endif /* SAS_SASSELECT_SUPPORT */

#ifdef SAS_DISABLE_BIST
#undef SAS_DISABLE_BIST
#define SAS_DISABLE_BIST            1        /* Disable ASIC Built-in Self
                                              * Test feature.
                                              */
#endif /* SAS_DISABLE_BIST */

#endif /* (SAS_COMSTOCK_SUPPORT || SAS_GALAXY_SUPPORT) */

/* Configuration Option Validity ---------------------------------------------*/

#if ((SAS_AIC941X) || (SAS_AIC958X))
#ifdef HIM_SCATTER_GATHER_FORMAT0
#if (HIM_SCATTER_GATHER_FORMAT0 != 1)
#undef HIM_SCATTER_GATHER_FORMAT0
#define HIM_SCATTER_GATHER_FORMAT0  1        /* Adaptec Common Scatter Gather Format 0   */
#endif /* (HIM_SCATTER_GATHER_FORMAT0 != 1) */
#else /* !ifdef HIM_SCATTER_GATHER_FORMAT0 */
#define HIM_SCATTER_GATHER_FORMAT0  1        /* Adaptec Common Scatter Gather Format 0   */
#endif /* HIM_SCATTER_GATHER_FORMAT0 */
#endif /* (SAS_AIC941X || SAS_AIC958X) */

/* some validation on the firmware mode selection before proceeding */
#if SAS_958X_IOP_MODE
#if SAS_STANDARD_MODE
   #error Razor standard F/W mode cannot be selected with Aurora IOP F/W mode
#elif SAS_958X_HOST_MODE
   #error Aurora host F/W mode cannot be selected with Aurora IOP F/W mode
#endif /* SAS_STANDARD_MODE */
#endif /* SAS_958X_IOP_MODE */
