/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: scsicdb.h

File Description
	Contains defs for all supported SCSI opcodes and structure
	definitions for CDB's

***********************************************************************

History :

01/06/2004 : 11:32   David
Added new SCSI opcodes definition.

11/24/2003 : 13:23   Tam
Added SCSISerialNoData structure.

04/19/2003 : 17:49   NAR20038  Manjunath
Added SCSI_READ_BUFFER and SCSI_WRITE_BUFFER commands.

04/24/2001 : 16:58   NAR20038  Manjunath
Added SCSI_WRITE_VERIFY_BYT_CHK for WRITE AND VERIFY command.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

***********************************************************************/

#ifndef SCSICDB_INCLUDED
#define SCSICDB_INCLUDED

#include	"sysdef.h"

typedef struct SerialNumberFormat {
    UCHAR      PeripheralDeviceType             :5,  
               PeripheralQualifier              :3;
    UCHAR      PageCode;                  /* 0x80                                 */
    UCHAR      Reserved;
    UCHAR      PageLength;                /* sizeof(SCSISerialNoData) - 3           */
    UCHAR      SerialNumber[1];
} SCSISerialNoData, DPTR pSCSISerialNoData;

/*	Extraction of bytes in CDB's */

typedef struct _fb 
{	/* Defines four bytes that are as a long  */
/* 0x0000 */   UCHAR          DByte0;
/* 0x0001 */   UCHAR          DByte1;
/* 0x0002 */   UCHAR          DByte2;
/* 0x0003 */   UCHAR          DByte3;
} fb;

typedef struct _tb 
{  /* Defines two bytes that are as a word  */
/* 0x0000 */   UCHAR          WByte0;
/* 0x0001 */   UCHAR          WByte1;
} tb;

typedef union _LongBytes 
{
/* 0x0000 */   struct _fb		f;
/* 0x0000 */   ULONG			DLong;
} LongBytes, DPTR pLongBytes, *npLongBytes;

typedef union _WordBytes 
{
/* 0x0000 */   struct _tb		t;
/* 0x0000 */   USHORT			DWord;
} WordBytes, DPTR pWordBytes, *npWordBytes;

/* General SCSI definitions */


/* SCSI Operation Codes */

#define SCSI_TEST_UNIT_READY    0x00    /* Test Unit Ready                  */
#define SCSI_REZERO_UNIT        0x01    /* CD-ROM rezero unit               */
#define SCSI_REWIND             0x01    /* Tape Rewind                      */
#define SCSI_REQUEST_SENSE      0x03    /* Request Sense Command            */
#define SCSI_FORMAT             0x04    /* Format                           */
#define SCSI_READ_BLK_LIMITS    0x05    /* Read Block Limits                */
#define SCSI_REQ_AUX_SENSE      0x06    /* Request Auxiliary Sense          */
#define SCSI_REASSIGN_BLOCKS    0x07    /* Reassign Blocks                  */
#define SCSI_READ_6             0x08    /* SCSI 6 byte Read                 */
#define SCSI_WRITE_6            0x0A    /* SCSI 6 byte Write                */
#define SCSI_SEEK_6             0x0B    /* SCSI 6 byte Seek                 */
#define SCSI_WRITE_FILEMARKS    0x10    /* Tape Write Filemarks             */
#define SCSI_SPACE              0x11    /* Tape Space                       */
#define SCSI_INQUIRY            0x12    /* Inquiry command                  */
#define SCSI_RECOVER_BUFFER     0x14    /* Tape Recover Buffer              */
#define SCSI_MODE_SELECT        0x15    /* Mode Select                      */
#define SCSI_RESERVE_UNIT       0x16    /* Tape Reserve Unit                */
#define SCSI_RELEASE_UNIT       0x17    /* Tape Release Unit                */
#define SCSI_ERASE              0x19    /* Tape Erase                       */
#define SCSI_MODE_SENSE         0x1A    /* Mode Sense                       */
#define SCSI_START_STOP_UNIT    0x1B    /* Start/Stop Unit                  */
#define SCSI_LOAD_UNLOAD        0x1B    /* Tape Load/Unload Media           */
#define SCSI_RECEIVE_DIAGNOSTIC_RESULTS 0x1C /* Receive Diagnostic Results  */
#define SCSI_SEND_DIAGNOSTIC    0x1D    /* Send Diagnostic                  */
#define SCSI_LOCK_UNLOCK        0x1E    /* Lock/Unlock drive door           */
#define SCSI_READ_CAPACITY      0x25    /* Read Capacity                    */
#define SCSI_READ_10            0x28    /* SCSI 10 byte Read                */
#define SCSI_WRITE_10           0x2A    /* SCSI 10 byte Write               */
#define SCSI_SEEK_10            0x2B    /* SCSI 10 byte Seek                */
#define SCSI_LOCATE             0x2B    /* Tape Locate                      */
#define SCSI_WRITE_VERIFY_10    0x2E    /* SCSI 10 byte Write w/Verify      */
#define SCSI_VERIFY_10          0x2F    /* SCSI 10 byte Verify              */
#define SCSI_PREFETCH           0x34    /* Prefetch                         */
#define SCSI_SYNC_CACHE		  0x35	/* Flush drive cache	    		 */
#define SCSI_WRITE_BUFFER	  0x3B	/* Write buffer				 */
#define SCSI_READ_BUFFER		  0x3C	/* Read buffer					 */
	#define SCSI_READ_LONG_10  		0x3E	/* Read long						*/
	#define SCSI_WRITE_LONG_10 		0x3F	/* Write long						*/
#define SCSI_WRITE_LONG         0x3F    /* Write long                       */
#define SCSI_WRITE_SAME		  0x41	/* Write same data				 */
#define SCSI_READ_SUB_CHAN      0x42    /* Read Sub-Channel (CD-ROM)        */
#define SCSI_READ_TOC           0x43    /* Read Table of Contents           */
#define SCSI_READ_HEADER        0x44    /* Read Header                      */
#define SCSI_PLAY_AUDIO_10      0x45    /* Play Audio 10 byte               */
#define SCSI_PLAY_MSF           0x47    /* Play Audio - MSF format          */
#define SCSI_PLAY_TRACK_INDEX   0x48    /* Play Audio - Track Index         */
#define SCSI_PLAY_TRACK_REL     0x49    /* Play Audio - Track Relative      */
#define SCSI_PAUSE_RESUME       0x4B    /* Pause/Resume Audio Play          */
#define SCSI_LOG_SELECT         0x4C    /* Log Select                       */
#define SCSI_MODE_SELECT_10     0x55    /* Mode Select - 10 byte            */
#define SCSI_RESERVE_UNIT_10    0x56    /* Tape Reserve Unit - 10 byte      */
#define SCSI_RELEASE_UNIT_10    0x57    /* Tape Release Unit - 10 byte      */
#define SCSI_MODE_SENSE_10      0x5A    /* Mode Sense  - 10 byte            */
#define SCSI_PERS_RESERVE_IN    0x5E    /* Persistent Reserve In            */
#define SCSI_PERS_RESERVE_OUT   0x5F    /* Persistent Reserve Out           */
#define SCSI_REPORT_LUNS		0xA0	/* Report LUNs command				*/
#define SCSI_PLAY_AUDIO_12      0xA5    /* Play Audio - 12 byte             */
#define SCSI_READ_12            0xA8    /* SCSI 12 byte read                */
#define SCSI_PLAY_TRACK_REL_12  0xA9    /* Play Audio - Trace Relative 12   */



/*
**  SCSI Command Descriptor Block
*/

#define MAX_CDB_LEN             12

typedef struct _SCSICDB6 
{		 /* CDB6 - 6 byte Command Descriptor Block*/

/* 0x0000 */        UCHAR   Opcode;             /* CDB Operation Code           */
/* 0x0001 */        UCHAR   Lun_MsbLBA;         /* SCSI LUN & 5 MSB bits of LBA */
/* 0x0002 */        UCHAR   MidLBA;             /* SCSI MID byte of LBA         */
/* 0x0003 */        UCHAR   LsbLBA;             /* SCSI LBA byte of LBA         */
/* 0x0004 */        UCHAR   XferLen;            /* SCSI Xfer length, Alloc length */
/* 0x0005 */        UCHAR   Control;            /* Control byte                 */

} SCSICDB6,DPTR pSCSICDB6, *npSCSICDB6;


typedef struct _SCSICDB10 
{		 /* CDB10 - 10 byte Command Descriptor Block*/

/* 0x0000 */        UCHAR   Opcode;            /* CDB Operation Code           */
/* 0x0001 */        UCHAR   Lun;               /* SCSI                         */
/* 0x0002 */        UCHAR   LBA[4];            /* SCSI LBA MSB->LSB            */
/* 0x0006 */        UCHAR   Res;               /* reserved byte                */
/* 0x0007 */        UCHAR   XferLen[2];        /* SCSI Xfer length (MSB first) */
/* 0x0009 */        UCHAR   Control;           /* Control byte                 */

} SCSICDB10,DPTR pSCSICDB10, *npSCSICDB10;


typedef struct _SCSICDB12 
{		 /* CDB12 - 12 byte Command Descriptor Block*/

/* 0x0000 */        UCHAR   Opcode;            /* CDB Operation Code           */
/* 0x0001 */        UCHAR   Lun;               /* SCSI                         */
/* 0x0002 */        UCHAR   LBA[4];            /* SCSI LBA MSB->LSB            */
/* 0x0006 */        UCHAR   XferLen[4];        /* SCSI Xfer length (MSB first) */
/* 0x000A */        UCHAR   Res;               /* reserved byte                */
/* 0x000B */        UCHAR   Control;           /* Control byte                 */

} SCSICDB12, DPTR pSCSICDB12, *npSCSICDB12;


/* Inquiry Data format */

typedef struct  _SCSIInqData 		/* structure size is 36 bytes */
{		 /* INQ */

/* 0x0000 */        UCHAR    DevType;          /* Periph Qualifier & Periph Dev Type*/
/* 0x0001 */        UCHAR    RMB_TypeMod;      /* rem media bit & Dev Type Modifier */
/* 0x0002 */        UCHAR    Vers;             /* ISO, ECMA, & ANSI versions        */
/* 0x0003 */        UCHAR    RDF;              /* AEN, TRMIOP, & response data format*/
/* 0x0004 */        UCHAR    AddLen;           /* length of additional data         */
/* 0x0005 */        UCHAR    Res1;             /* reserved                          */
/* 0x0006 */        UCHAR    Res2;             /* reserved                          */
/* 0x0007 */        UCHAR    Flags;            /* RelADr,Wbus32,Wbus16,Sync,etc.    */
/* 0x0008 */        UCHAR    VendorID[8];      /* Vendor Identification             */
/* 0x0010 */        UCHAR    ProductID[16];    /* Product Identification            */
/* 0x0020 */        UCHAR    ProductRev[4];    /* Product Revision                  */


} SCSI_INQDATA, DPTR pSCSI_INQDATA;


/*  Inquiry byte 0 masks */


#define SCSI_DEVTYPE        0x1F      /* Peripheral Device Type             */
#define SCSI_PERIPHQUAL     0xE0      /* Peripheral Qualifier               */


/*  Inquiry byte 1 mask */

#define SCSI_REMOVABLE_MEDIA  0x80    /* Removable Media bit (1=removable)  */


/*  Peripheral Device Type definitions */


#define SCSI_DASD                0x00      /* Direct-access Device         */
#define SCSI_SEQACCESS            0x01      /* Sequential-access device     */
#define SCSI_PRINTER             0x02      /* Printer device               */
#define SCSI_PROCESSOR           0x03      /* Processor device             */
#define SCSI_WRITEONCE           0x04      /* Write-once device            */
#define SCSI_CDROM               0x05      /* CD-ROM device                */
#define SCSI_SCANNER             0x06      /* Scanner device               */
#define SCSI_OPTICAL             0x07      /* Optical memory device        */
#define SCSI_MEDCHGR             0x08      /* Medium changer device        */
#define SCSI_COMM                0x09      /* Communications device        */
#define SCSI_ENCLOSURE_SERVICES  0x0D      /* Enclosure Services device    */
#define SCSI_NODEV               0x1F      /* Unknown or no device type    */


/*
** Inquiry flag definitions (Inq data byte 7)
*/

#define SCSI_INQ_RELADR       0x80    /* device supports relative addressing*/
#define SCSI_INQ_WBUS32       0x40    /* device supports 32 bit data xfers  */
#define SCSI_INQ_WBUS16       0x20    /* device supports 16 bit data xfers  */
#define SCSI_INQ_SYNC         0x10    /* device supports synchronous xfer   */
#define SCSI_INQ_LINKED       0x08    /* device supports linked commands    */
#define SCSI_INQ_CMDQUEUE     0x02    /* device supports command queueing   */
#define SCSI_INQ_SFTRE        0x01    /* device supports soft resets */
#define SCSI_WRITE_VERIFY_BYT_CHK	0x02 /* performs byte-by-byte compare of data
 										  * written */

/* SCSI Mode Page codes */
#define SCSI_MODE_ERR_RECOVERY_PAGE 	0x01

/* SCSI Mode sense data page code mask*/
#define SCSI_MP_PAGE_CODE_MASK		0x3F
#endif
