/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: scsisns.h

File Description
	Definition of structure for reporting of errors and for reporting
	events.

***********************************************************************

History :

08/18/2005: Jeff
Added #define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_NONE 0x0
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_MEDIA 0x01
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_LU 0x02
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_SPARE_AREA 0x03

04/19/2005: SKP
Added #define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED 0x10

08/21/2004: Madhuresh
Added an error for access of OCM as CDA_ERROR_HIMRAMSEGMENT_FAILURE.

01/16/2003 : 16:35 NAR20038  Manjunath
Added New event CODE_STATE_DIRTY_NO_VERIFY - application should post unsafe
shutdown event but do not start verify activity.

September 24, 2002: Madhuresh.
CODE_HSR_RECREATE_STARTED defined to notify IOMGR about internally started
rebuild operation due to detection of a disk which is the missing member of 
RAID1/10.

Aug 7, 2002: Madhuresh.
CODE_ARRAY_NEEDS_VERIFY_WITH_AUTOFIX defined as new event so that
iomgr starts the verify approriately.

03/20/2001 : 10:56 AM   NAR20038  Manjunath
Changed MAX_EVENTS from 64 to 16 since in IROC, we don't support more
than 8 disk drives and we have to reduce memory requirements.

01/31/2001 : 10:35 AM   NAR20038  Manjunath
Added event codes CODE_BACKGROUND_OPS_ABORTED. Also added few HIM error
codes so that IOMgr would compile. We can remove them once we clean up
IOMgr.

01/24/2001 : 01:49 PM   NAR20038  Manjunath
Added event codes CODE_ARRAY_NEEDS_INITIALIZE, CODE_ARRAY_NEEDS_VERIFY, 
CODE_ARRAY_NEEDS_REBUILD.

12/13/2000 
12/13/2000 

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Removed code under #if 0

***********************************************************************/

#ifndef SCSISNS_INCLUDED
#define SCSISNS_INCLUDED

#include "sysdef.h"


/*
** SCSI Status byte codes
*/

#define  SCSI_UNIT_GOOD      0x00        /* Good status or none available       */
#define  SCSI_UNIT_CHECK     0x02        /* Check Condition                     */
#define  SCSI_UNIT_MET       0x04        /* Condition met                       */
#define  SCSI_UNIT_BUSY      0x08        /* Target busy                         */
#define  SCSI_UNIT_INTERMED  0x10        /* Intermediate command good           */
#define  SCSI_UNIT_INTMED_GD 0x14        /* Intermediate condition met          */
#define  SCSI_UNIT_RESERV    0x18        /* Reservation conflict                */
#define  SCSI_UNIT_QUEFULL   0x28        /* Queue Full                          */

#define SCSI_STAT_GOOD		  	SCSI_UNIT_GOOD	  /* Good status				  */
#define SCSI_STAT_CHECKCOND	 	SCSI_UNIT_CHECK	  /* SCSI Check Condition		 */
#define SCSI_STAT_CONDMET	   	SCSI_UNIT_MET	  /* Condition Met				*/
#define SCSI_STAT_BUSY		  	SCSI_UNIT_BUSY	  /* Target busy status		   */
#define SCSI_STAT_INTER		 	SCSI_UNIT_INTERMED	  /* Intermediate status		  */
#define SCSI_STAT_INTERCONDMET  SCSI_UNIT_INTMED_GD	  /* Intermediate condition met   */
#define SCSI_STAT_RESCONFLICT   SCSI_UNIT_RESERV	  /* Reservation conflict		 */
#define SCSI_STAT_CMDTERM	   	0x22	  			/* Command Terminated */
#define SCSI_STAT_QUEUEFULL		SCSI_UNIT_QUEFULL	  /* Queue Full */

/*
Define an out of bounds status value. This is returned for those events or
errors that were GENERATED by the driver/host and did not come directly
from a SCSI device.
*/

#define	SCSI_STAT_GENERATED		0x80
#define SCSI_UNIT_STOPPED		0x8A		/* for PC97 Implementation */

/*
**  Request Sense Data format
*/

#define SENSE_DATA_KEY_SPECIFIC_SIZE 17
typedef struct  _SCSISenseData 	  /* structure size is 32 (0x20) bytes	  */
{
/* 0x0000 */	UCHAR   ErrCode_Valid;		/* Error Code & Valid bit	   */
/* 0x0001 */	UCHAR   SegNum;				/* Segment Number			   */
/* 0x0002 */	UCHAR   SenseKey;			/* Sense Key,ILI,EOM, FM		*/
/* 0x0003 */	UCHAR   INFO[4];			/* information field			*/
/* 0x0007 */	UCHAR   AddLen;				/* additional length			*/
/* 0x0008 */	UCHAR   CmdInfo[4];			/* command-specific info		*/
/* 0x000C */	UCHAR   AddSenseCode;		/* additional sense code		*/
/* 0x000D */	UCHAR   AddSenseCodeQual;	/* additional sense code qualifier*/
/* 0x000E */	UCHAR   FieldRepUnitCode;	/* field replaceable unit code  */
/* 0x000F */	UCHAR   KeySpecific[SENSE_DATA_KEY_SPECIFIC_SIZE];	/* Sense-key specific		   */

} SCSISenseData, DPTR pSCSISenseData, NP npSCSISenseData;



/*
**  Sense Data bit masks
*/


/* Byte 0 of sense data */


#define SCSI_ERRCODE_MASK	 0x7F	   /* Error Code				   */
#define SCSI_VALID_MASK	   0x80	   /* Information field valid bit  */
#define SCSI_DEFERRED_ERROR	0x01		  /* Error for an earlier command */



/* Byte 2 of sense data */


#define SCSI_SENSEKEY_MASK	 0xF	   /* Sense key				  */
#define SCSI_INCORRECT_LEN	 0x20	  /* Incorrect lenght indicator */
#define SCSI_SENSE_ENDOFMEDIUM 0x40	  /* End-of-medium bit		  */
#define SCSI_SENSE_FM		  0x80	  /* filemark bit			   */



/*
**  Sense Key definitions
*/


#define SCSI_SK_NOSENSE		  0x0	 /* No sense					 */
#define SCSI_SK_RECERR		   0x1	 /* Recovered Error			  */
#define SCSI_SK_NOTRDY		   0x2	 /* Not Ready Error			  */
#define SCSI_SK_MEDIUMERR		0x3	 /* Medium Error				 */
#define SCSI_SK_HARDWAREERR	  0x4	 /* HardWare Error			   */
#define SCSI_SK_ILLEGALREQ	   0x5	 /* Illegal Request			  */
#define SCSI_SK_UNITATTN		 0x6	 /* Unit Attention			   */
#define SCSI_SK_DATAPROTECT	  0x7	 /* Data Protect Error		   */
#define SCSI_SK_BLANKCHK		 0x8	 /* Blank Check				  */
#define SCSI_SK_COPYABORT		0x0A	/* Copy Aborted				 */
#define SCSI_SK_ABORTEDCMD	   0x0B	/* Aborted Command			  */
#define SCSI_SK_EQUAL			0x0C	/* Equal Comparison satisfied   */
#define SCSI_SK_VOLOVERFLOW	  0x0D	/* Volume Overflow			  */
#define SCSI_SK_MISCOMPARE	   0x0E	/* Miscompare				   */


/*
**  Additional Sense Code definitions
*/


#define ASC_NO_SEEK_COMPLETE		0x02
#define ASC_UNIT_NOT_READY		  0x04
#define ASC_UNIT_COMM_FAIL		  0x08
#define ASC_SERVO_FAIL			  0x09
#define ASC_UNRECOVERED_ERROR	   0x11
#define ASC_NO_ADDRESS_MARK		 0x12
#define ASC_SEEK_POSITIONING_ERROR  0x15
#define ASC_DATA_SYNC_ERROR		 0x16
#define ASC_INVALID_COMMAND_OPCODE  0x20
#define ASC_ILLEGAL_LBA			 0x21
#define ASC_INVALID_FIELD		   0x24
#define ASC_UNIT_NOT_SUPPORTED	  0x25
#define ASC_INVALID_PARMLIST		0x26
#define ASC_WRITE_PROTECT		   0x27
#define ASC_MEDIUM_CHANGED		  0x28
#define ASC_RESET				   0x29
#define ASC_INCOMPATIBLE_CARTRIDGE  0x30
#define ASC_MEDIUM_NOT_PRESENT		0x3A
#define ASC_INTERNAL_TARGET_FAIL	0x44
#define ASC_TOC_RECOVERY_FAIL	   0x57

#if SMART_SUPPORT
#define ASC_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED  0x5D
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED 0x10
/* Jeff  ADPmp87554*/
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_NONE 0x0
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_MEDIA 0x01
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_LU 0x02
#define ASCQ_FAILURE_PREDICTION_THRESHHOLD_EXCEEDED_SPARE_AREA 0x03
#endif

#define ASC_END_USER_AREA		   0x63
#define ASC_ILLEGAL_MODE_FOR_TRACK  0x64

#define ASCV_NOT_AUDIO_TRACK		0x88
#define ASCV_NOT_DATA_TRACK		 0x89
#define ASCV_NOT_AUDIO_PLAY_STATE   0x8A

/* Define translation to general errors from device errors. */

#define SET_RECOVERED_ERROR	 	0
#define SET_HARD_ERROR		  	1
#define SET_NONFATAL_DISK_FAIL  	2
#define SET_FATAL_DISK_FAIL	 	3
#define SET_FATAL_BLK_FAILURE		4
#define SET_NONFATAL_BLK_FAILURE	5
#define SET_FATAL_ARRAY_FAIL		6
#define SET_INFORMATION_EVENT		7
#define SET_ERROR_LOG			8
#define SET_NONFATAL_DISK_MISSING	9	/* one member in RAID5 array */
						/* is missing.	*/

#define SET_USER_FAILED_DISK		10
/*
This last one is just a place holder. It is set whenever the status
is not SCSI_STAT_GENERATED.
*/
/* Since it is not being used currently, it is just commented out for now */
/* #define SET_SCSI_EVENT				9	 */
#define MAX_EVENTS	16
#ifdef SCO_DDI8
typedef struct	_EventLog 			/* structure size is 72 (0x48) bytes	*/
{
/* 0x0000 */	union {
			LDTree			_EL_DevPath;		/* LDTree of the device	*/
			Sas_Device_Address	_EL_Sas_Dev_Info;	/* Sas ID, port, state */
		} _EL_union;
#define EL_DevPath _EL_union._EL_DevPath
#define EL_Sas_Dev_Info _EL_union._EL_Sas_Dev_Info
#else
typedef struct	_EventLog 			/* structure size is 72 (0x48) bytes	*/
{
/* 0x0000 */	union {
			LDTree			EL_DevPath;		/* LDTree of the device	*/
			Sas_Device_Address	EL_Sas_Dev_Info;	/* Sas ID, port, state */
		};
#endif /* SCO_DDI8 */
/* 0x0024 */	UCHAR	  	EL_Status;		/* Status => status generated	*/
											/* by host or driver, 			*/
											/* e.g. SCSI_STAT_GENERATED		*/
/* 0x0025 */	UCHAR		EL_EventQual;	/* Event qualifier => severity 	*/
											/* of the event, 				*/
											/* e.g. SET_FATAL_DISK_FAIL 	*/
/* 0x0026 */	USHORT		Padding;		/* padding for dword alignment	*/
/* 0x0028 */	SCSISenseData	EL_Sense;	/* Buffer contains sense data */
								/* This may contain standard sense */
								/* data those come from SCSI devices */
								/* or those errors/events generated by */
								/* the driver, as for example,	*/
								/* EL_Sense.SenseKey = KEY_DEVICE_EVENT */
								/* EL_Sense.AddSenseCode = CODE_DEVICE_FAILED */
								/* EL_Sense.INFO[] = LBA of access */
} EventLog, DPTR pEventLog;

typedef struct _EventLogList 	/* structure size is 3588 bytes */
{
	USHORT		cEvents;		/* Number of events in the event */
	USHORT		Padding;		/* padding for dword alignment */
								/* log buffer */
	EventLog	Events[MAX_EVENTS];
} EventLogList, DPTR pEventLogList;

/*****************************************************************************/
/* This is for logging history trace */
#define MAX_TRACES					 65536
typedef struct	_TracesLog
{
	UCHAR		Function;		/* where did we log this trace? */
	UCHAR		Opcode;			/* scsi opcode	*/
	UCHAR		Id;				/* scsi id		*/
	UCHAR		Reserved;
	union
	{
		ULONG	Parent;			/* parent or self request address	*/
		struct
		{
			UCHAR		HaStat;	/* host adapter status	*/
			UCHAR		TarStat;/* target status 		*/
			UCHAR		Key;	/* sense key			*/
			UCHAR		Code;	/* additional sense code 	*/
		} status;
	} io;
	ULONG		Lba;			/* block address		*/
	ULONG		Blocks;			/* number of blocks		*/
	ULONG		HashKey;		/* index to the hash table 	*/
	ULONG		SabreBase;		/* sabre address		*/
	ULONG		SGAddress;		/* scatter gather address	*/
	ULONG		SGLength;		/* scatter gather length	*/

} TracesLog, DPTR pTracesLog;

typedef struct _TracesLogList
{
	ULONG		cTraces;
	TracesLog DPTR	pTraces;
} TracesLogList, DPTR pTracesLogList;

/*	The 2 globals that defined in raidglbl.c for debug purposes only.
	They are: TraceOn is set to 0(False) by default to turn off the
	trace and the user can set it to 1(TRUE) to turn on the trace.
	TraceTable+4 has the pointer to the trace table.
	BOOL	TraceOn = FALSE;
	TracesLogList	TraceTable;
*/

/* Functions defined as follow :
	1 - OS request to send in XBeginOSD
		LogTrace(1, pReq->Function, pSD->SysReportedId, 0, 0, 0L, 0L,
				pReq->StartingSector, pReq->NumberOfSectors, 0xFFFFFFFFL, 0,0,0,0);

	2 - IOD in SendToHIM
		LogTrace(2, pSP->Sp_CDB[0], pSP->SP_Tarlun, 0, 0, 0L, 0L,
			pIOD->IOD_StartBlock, pIOD->IOD_cBlocks, (ULONG)pIOD->pIOD_IOParent,
			HashKey(pIOD->IOD_StartBlock),0,debugPtr->SGLBufAddr,debugPtr->SGLByteCnt);

	3 - IOD has error in ReceiveFromHIM
		LogTrace(3, pIOD->IOD_CDB[0], pSP->SP_Tarlun, pSP->SP_HaStat, pSP->SP_TargStat,
			pIOD->IOD_Sense.SenseKey, pIOD->IOD_Sense.AddSenseCode,
			IOD->IOD_StartBlock, pIOD->IOD_cBlocks, 0xFFFFFFFFL,HashKey(pIOD->IOD_StartBlock),0,
			pIOD->pIOD_SGTable->SGList[0].SGLBufAddr,
			pIOD->pIOD_SGTable->SGList[0].SGLByteCnt);

	4 - parity miscompare in IZeroCheckHostMemory
		LogTrace(4, pRD->pRD_FailedIO->IOD_CDB[0], pRD->pRD_FailedIO->IOD_sp.SP_Tarlun,
			pRD->pRD_FailedIO->IOD_sp.SP_HaStat, pRD->pRD_FailedIO->IOD_sp.SP_TargStat,
			pRD->pRD_FailedIO->IOD_Sense.SenseKey, pRD->pRD_FailedIO->IOD_Sense.AddSenseCode,
			pRD->RD.D_StartBlock, pRD->RD.cD_Sectors, 0xFFFFFFFFL,HashKey(pRD->RD.D_StartBlock),
			pRD->RD.D_SabreBase,
			pRD->pRD_FailedIO->pIOD_SGTable->SGList[0].SGLBufAddr,
			pRD->pRD_FailedIO->pIOD_SGTable->SGList[0].SGLByteCnt);

	5 - parity miscompare in IVerifySabreImageDone
		LogTrace(5, pRD->pRD_FailedIO->IOD_CDB[0], pRD->pRD_FailedIO->IOD_sp.SP_Tarlun,
			pRD->pRD_FailedIO->IOD_sp.SP_HaStat, pRD->pRD_FailedIO->IOD_sp.SP_TargStat,
			pRD->pRD_FailedIO->IOD_Sense.SenseKey, pRD->pRD_FailedIO->IOD_Sense.AddSenseCode,
			pRD->RD.D_StartBlock, pRD->RD.cD_Sectors, 0xFFFFFFFFL,HashKey(pRD->RD.D_StartBlock),
			pRD->RD.D_SabreBase,
			pRD->pRD_FailedIO->pIOD_SGTable->SGList[0].SGLBufAddr,
			pRD->pRD_FailedIO->pIOD_SGTable->SGList[0].SGLByteCnt);

	6 - eliminate the write and return success in RAIDRecoveryBeforeSubmission
		LogTrace(6, pFailedIOD->IOD_CDB[0], pFailedIOD->pIOD_PhysParent->PhysId, 0, 0, 0L, 0L,
			pFailedIOD->IOD_StartBlock, pFailedIOD->IOD_cBlocks, (ULONG)pFailedIOD->pIOD_IOParent,
			HashKey(pFailedIOD->IOD_StartBlock),
			((pWriteDescriptor) pFailedIOD->pIOD_IOParent)->WD.D_SabreBase,
			pFailedIOD->pIOD_SGTable->SGList[0].SGLBufAddr,
			pFailedIOD->pIOD_SGTable->SGList[0].SGLByteCnt);

	7 - all the requests that are issued in RAIDRecoveryStartRead
		LogTrace(7, pCrntIOD->IOD_CDB[0], pCrntIOD->pIOD_PhysParent->PhysId, 0, 0, 0L, 0L,
			pCrntIOD->IOD_StartBlock, pCrntIOD->IOD_cBlocks, (ULONG)pCrntIOD->pIOD_IOParent,
			HashKey(pCrntIOD->IOD_StartBlock),0,pCrntIOD->pIOD_SGTable->SGList[0].SGLBufAddr,
			pCrntIOD->pIOD_SGTable->SGList[0].SGLByteCnt);

	8 - OSD in ProcessRW
		LogTrace(8, pOSD->OSD_CDB[0], pSD->SysReportedId, 0, 0, 0L, 0L,
			pOSD->OSD_StartBlock, pOSD->cOSD_Blocks, (ULONG)pOSD,0L,0L,0L,0L);

	9 - all the WDs in ProcessR5Write
		LogTrace(9, pWDChain->WD.D_CDB[0], pSD->SysReportedId, 0, 0, 0L, 0L,
			pWDChain->WD.D_StartBlock, pWDChain->WD.cD_Sectors, (ULONG)pWDChain->pWD_OSD,0,0,0,0);

	10 - RD in IVerifyGainRange
		LogTrace(10, 0, 0, 0, 0, 0L, 0L,
			pRD->RD.D_StartBlock, pIOD->pIOD_PhysParent->PhysValidTo, (ULONG)pRD,
			HashKey(pRD->RD.D_StartBlock),
			pRD->RD.D_SabreBase,pIOD->pIOD_SGTable->SGList[0].SGLBufAddr,
			pIOD->pIOD_SGTable->SGList[0].SGLByteCnt);
*/
/*****************************************************************************/

/*
There are some events in the error/event log that are not exactly
SCSI events. The sense data format is used with some fields redefined.
A Sense Key provides the error type. Other fields such as sense code and
InfoLBA can be used for more detailed information

Define the set of sense information used for reporting events, errors
generated by the host/driver.
*/

/*	R E P O R T E D  S E N S E  I N F O  */
/*
	Event/Error Log
		Key			KEY_EVENT_LOG
		Code			CODE_FAILED
						CODE_COMPLETED_OK
		Info LBA		N/A

	Abort
		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFY_ABORTED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_RECREATE_ABORTED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_ZERO_DEVICE_ABORTED
		Info LBA		N/A


	Currently driver does not log any event for AdministerSpare
	and ChangeDrive.
	AdministerSpare
		Key			KEY_SPARE_MGT
		Code			CODE_NOW_SINGLE
		Info LBA		N/A

		Key			KEY_SPARE_MGT
		Code			CODE_NOW_SPARE
		Info LBA		N/A

		Key			KEY_SPARE_MGT
		Code			CODE_FAILED_MOVE
		Info LBA		N/A

	ChangeDrive
		Key			KEY_SPARE_MGT
		Code			CODE_SPARE_NOW_MEMBER
		Info LBA		N/A

		Key			KEY_SPARE_MGT
		Code			CODE_SPARE_UNFIT
		Info LBA		N/A

	RecreateLogicalDevice,
		Key			KEY_MGT_UPDATE
		Code			CODE_RECREATE_STARTED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_RECREATE_COMPLETED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_RECREATE_FAILED
		Info LBA		N/A
	VerifyLogicalDevice,
		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFY_STARTED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFY_COMPLETED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFY_FAILED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFY_FAILED_AT
		Info LBA		LBA of Current Function

		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFY_FIXED_AT
		Info LBA		LBA of Current Function

		While fixing the parity, if the read/write of the
		parity block fails, we use CODE_VERIFYFIX_FAILED_AT
		to explicitly state the failure to the IO manager.

		Key			KEY_MGT_UPDATE
		Code			CODE_VERIFYFIX_FAILED_AT
		Info LBA		LBA of Current Function

	ZeroLogicalDevice
		Key			KEY_MGT_UPDATE
		Code			CODE_ZERO_DEVICE_STARTED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_ZERO_DEVICE_COMPLETED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_ZERO_DEVICE_FAILED
		Info LBA		N/A

	Format
		Key			KEY_MGT_UPDATE
		Code			CODE_FORMAT_STARTED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_FORMAT_COMPLETED
		Info LBA		N/A

		Key			KEY_MGT_UPDATE
		Code			CODE_FORMAT_FAILED
		Info LBA		N/A

	The following ioclts are not used by IO manager yet.

	RecreateLogicalDeviceRange,
	VerifyLogicalDeviceRange,
	ZeroLogicalDeviceRange
		Key			KEY_MGT_UPDATE
		Code			CODE_ACTION_AT
		Info LBA		LBA of current function

		Key			KEY_MGT_UPDATE
		Code			CODE_ACTION_FAILED_AT
		Info LBA		LBA of current function

These are the key, codes for events generated by devices/drivers.

	Bad Block encountered.
		Key		KEY_DEVICE_EVENT
		Code		CODE_BAD_BLK_REPAIRED
		Info LBA	LBA of the bad block

	Bad Block encountered.
		Key		KEY_DEVICE_EVENT
		Code		CODE_BAD_BLK_FAILED
		Info LBA	LBA of the bad block

	Device failed.
		Key		KEY_DEVICE_EVENT
		Code		CODE_DEVICE_FAILED
		Info LBA	N/A

*/


#define KEY_MGT_UPDATE				0x01
#define KEY_SPARE_MGT				0x02
#define KEY_DEVICE_EVENT			0x03
#define KEY_INVALID_IOCTL			0x04
#define KEY_DEVICE_MGT				0x05
#define KEY_ARRAY_EVENT				0x06
#define KEY_CODE_MGT				0x07 /* For Athena Code Management */
#define KEY_BATTERY_MGT				0x08
#define KEY_SPARE_EVENT				0x09
#define KEY_CACHE_MGT_UPDATE		0x0A
#define KEY_CONFIG_EVENT			0x0B
#define KEY_SAS_EVENT				0x0C

#define CODE_VERIFY_ABORTED			0x01
#define CODE_NOW_SINGLE				0x02
#define CODE_NOW_SPARE				0x03
#define CODE_FAILED_MOVE			0x04
#define CODE_SPARE_NOW_MEMBER		0x05
#define CODE_SPARE_UNFIT			0x06
#define CODE_NOW_CONFIGURED_SINGLE  0x07
#define CODE_NOW_UNCONFIGURED_SINGLE 0x08
#define CODE_FAILED					0x09
#define CODE_ACTION_AT				0x0A
#define CODE_ACTION_FAILED_AT		0x0B
#define CODE_DATA_UNDERRUN_OVERRUN	0x0C
#define CODE_INVALID_TARGET			0x0D
#define CODE_RECREATE_ABORTED		0x0E
#define CODE_ZERO_DEVICE_ABORTED	0x0F
#define	CODE_INVALID_COMMAND_OPCODE	0x40
#define CODE_INVALID_FIELD			0x41

#define CODE_RECREATE_STARTED		0x10
#define CODE_RECREATE_COMPLETED		0x11
#define CODE_RECREATE_FAILED		0x12

#define CODE_VERIFY_STARTED			0x13
#define CODE_VERIFY_COMPLETED		0x14
#define CODE_VERIFY_FAILED			0x15
#define CODE_VERIFY_FAILED_AT		0x16
#define CODE_VERIFY_FIXED_AT		0x17
#define CODE_VERIFYFIX_FAILED_AT	0x18

#define CODE_ZERO_DEVICE_STARTED	0x19
#define CODE_ZERO_DEVICE_COMPLETED	0x1A
#define CODE_ZERO_DEVICE_FAILED 	0x1B

#define CODE_FORMAT_STARTED			0x1C
#define CODE_FORMAT_COMPLETED 		0x1D
#define CODE_FORMAT_FAILED 			0x1E
#define CODE_WRITE_COMPLETED 		0x1F

#define CODE_MOVED_MEMBER			0x25
#define CODE_DEVICE_ADDED			0x26
#define CODE_DEVICE_REMOVED		0x27

#define CODE_RESCAN_COMPLETED		0x28

#define CODE_REACTIVATE_FAILED		0x29
#define CODE_REACTIVATE_STARTED		0x30
#define CODE_REACTIVATE_COMPLETED	0x31

#define CODE_CACHE_ALLOCATION_FAILED		0x32

#define CODE_CACHE_FLUSH_FAILED		0x33
#define CODE_CACHE_FLUSH_STARTED		0x34
#define CODE_CACHE_FLUSH_COMPLETED		0x35

/* At init time, if RedundancyState is not clean, mark dirty and log.
		EventQual	SET_ERROR_LOG
		Key			KEY_DEVICE_EVENT
		Code			CODE_STATE_DIRTY
*/
#define CODE_STATE_DIRTY			0x20

/* Flash utility uses following status for Athena/BIOS code upgrade. */
#define CODE_UPGRADE_FAILED		0x21
#define CODE_UPGRADE_SUCCESS	0x00
#define CODE_REVERT_FAILED		0x22
#define CODE_REVERT_SUCCESS		0x23
#define CODE_OUT_OF_RESOURCES	0x24

#define CODE_PFA_EVENT			0x5D /* sent by SMART drives */

/* 
 * if bios configuration mgmt, starts initialize/verify/rebuild
 * and driver takes over from the point bios stopped, then driver
 * logs these events so that iomgr will start these operations again.
 */
#define CODE_ARRAY_NEEDS_INITIALIZE			0x51
#define CODE_ARRAY_NEEDS_VERIFY				0x52 
#define CODE_ARRAY_NEEDS_REBUILD			0x53
#define CODE_BACKGROUND_OPS_ABORTED         0x54
#define CODE_ARRAY_NEEDS_VERIFY_WITH_AUTOFIX 0x55
#define CODE_STATE_DIRTY_NO_VERIFY			0x56

/*
Define the codes for device event management. Partition the codes
to allow for mgt events to be separate from device events.
*/

#define CODE_BAD_BLK_REPAIRED	0x80
#define CODE_BAD_BLK_FAILED	0x81	/* This is same as 	*/
						  				/* CODE_DEVICE_FAILED, but */
						  				/* more specific of the failure */
#define CODE_DEVICE_FAILED		0x82	/* This implies that either */
						  				/* the member is missing or */
						  				/* nothing can be recovered */
						  				/* from this member. 	*/
#define CODE_DEVICE_CFG_FAILED	0x83	/* Read/Write to reserved */
						  				/* sector failed */
#define CODE_DEVICE_IN_RECOVERY  0x84  /* Driver detected an error */
						  				/* on a member disk block and */
						  				/* started recovery on that member */
						  				/* So if recovery is successful */
						  				/* this could be a good member. */
						  				/* Instead of setting it to */
						  				/* CODE_DEVICE_FAILED, this */
						  				/* would be more specific.*/

#define CODE_BATTERY_FAILED	0x85	/* Driver detected that battery */
										/* is not charging.			*/
#define CODE_WRITE_CACHE_DISABLED	0x86/* Write cache on the drive has been disabled */
#define CODE_PAUSE_UNPAUSE_COMPLETED 0x87
#define CODE_MEDIUM_ERROR			0x88 /* medium error on device */
#define CODE_REPLACE_DISK		    0x95 /* Rebuild disk has fatal error replace it */

/*
	Hot Swap Rebuild. When a disk is found at a scsi id which is
	still referred by the meta data of any RAID1/10 as its missing
	member, the newly found disk is moved in as a member and
	a rebuild is initiated.
*/
#define CODE_HSR_RECREATE_STARTED	0x89 /* Rebuild initiated by Hot Swap Rebuild logic */

/* SAS support */
/* Following additional sense code (AddSenseCode)are for the 
 * sensekey KEY_CONFIG_EVENT.
 * The additional sense code qualifier (AddSenseCodeQual) are defined below as CDA_ERROR_XXX.
 */
#define CODE_DEVICE_DUAL_PORTED				0x90     /* Second port occurrence has been dropped */
#define CODE_DEVICE_CABLE_LOOPBACK			0x91     /* SAS Address and Phy IDs of expander provided in INFO fields - don't know how he would use them? */
#define CODE_ADAPTER_INITIALIZATION_FAILED	0x92     /* Some step in the initialization failed. See ASCQ */
#define CODE_ADAPTER_INITIALIZATION_SUCCESS 0x93
#define CODE_ADAPTER_RESTARTED              0x94    /* Restart Adapter */

//@Maram
#define CODE_UPDATE_BOOTFLAG_SET			0x95
#define CODE_UPDATE_BOOTFLAG_CLEARED		0x96
#define CODE_UPDATE_BOOTFLAG_ALREADY_MARKED 0x97
#define CODE_UPDATE_BOOTFLAG_FAILED			0x98

/*  Metadata related errors: Error codes range: 0xA0 to 0xD0.
 *  CAUTION:  MUST MATCH mdtddf.h for the error codes used here.
 *            DO NOT modify one without the other. 
 */
#define ASC_MDTDDF_ERR_BASE					0xA0

#define	CODE_INIT_MEM_ALLOC		  			(UCHAR)(ASC_MDTDDF_ERR_BASE + 1)
#define	CODE_NO_SD_NUM_MEMBERS				(UCHAR)(ASC_MDTDDF_ERR_BASE + 2)
#define	CODE_NO_MATCHING_VD_GUID			(UCHAR)(ASC_MDTDDF_ERR_BASE + 3)
#define	CODE_NO_MATCHING_PD_REF				(UCHAR)(ASC_MDTDDF_ERR_BASE + 4)
#define	CODE_INVALID_PRIMARY_ELEMENT_COUNT	(UCHAR)(ASC_MDTDDF_ERR_BASE + 5)
#define CODE_INVALID_SECONDARY_ELEMENT_SEQ	(UCHAR)(ASC_MDTDDF_ERR_BASE + 6)
#define	CODE_INVALID_POPULATED_PDE			(UCHAR)(ASC_MDTDDF_ERR_BASE + 7)
#define	CODE_INVALID_PD_REF					(UCHAR)(ASC_MDTDDF_ERR_BASE + 8)
#define	CODE_INVALID_PD_SEQ					(UCHAR)(ASC_MDTDDF_ERR_BASE + 9)
#define	CODE_INVALID_DEVICE_ID				(UCHAR)(ASC_MDTDDF_ERR_BASE + 10)
#define	CODE_INVALID_MEMBER_INDEX			(UCHAR)(ASC_MDTDDF_ERR_BASE + 11)
#define	CODE_UNSUPPORTED_RAID_CFG			(UCHAR)(ASC_MDTDDF_ERR_BASE + 12)
#define	CODE_PD_REFERENCE_NOT_IN_PDE		(UCHAR)(ASC_MDTDDF_ERR_BASE + 13)
#define	CODE_EXCEED_MAX_VDE					(UCHAR)(ASC_MDTDDF_ERR_BASE + 14)
#define	CODE_EXCEED_CDA_MAX_RAIDS_PER_ADAPTER	(UCHAR)(ASC_MDTDDF_ERR_BASE + 15)
#define	CODE_INVALID_POPULATED_VDE			(UCHAR)(ASC_MDTDDF_ERR_BASE + 16)
#define	CODE_VD_NOTFOUND_IN_VDE				(UCHAR)(ASC_MDTDDF_ERR_BASE + 17)
#define	CODE_NULL_pPD						(UCHAR)(ASC_MDTDDF_ERR_BASE + 18)
#define	CODE_INVALID_PRIM_RAID_LEVEL  		(UCHAR)(ASC_MDTDDF_ERR_BASE + 19)

#define CODE_FOREIGN_DEVICE					(UCHAR)(ASC_MDTDDF_ERR_BASE + 20)
#define CODE_INVALID_ANCHOR_HDR_SIGNATURE	(UCHAR)(ASC_MDTDDF_ERR_BASE + 21)
#define CODE_INVALID_PRIMARY_HDR_SIGNATURE	(UCHAR)(ASC_MDTDDF_ERR_BASE + 22)
#define CODE_INCOMPATIBLE_DDF_REV			(UCHAR)(ASC_MDTDDF_ERR_BASE +23)
#define CODE_INCOMPATIBLE_VSLOG_REV			(UCHAR)(ASC_MDTDDF_ERR_BASE +24)

#define CODE_ANCHOR_HDR_CRC					(UCHAR)(ASC_MDTDDF_ERR_BASE + 30)
#define CODE_PRIMARY_HDR_CRC				(UCHAR)(ASC_MDTDDF_ERR_BASE + 31)
#define CODE_ADAPTER_DATA_CRC				(UCHAR)(ASC_MDTDDF_ERR_BASE + 32)
#define CODE_PD_REC_CRC						(UCHAR)(ASC_MDTDDF_ERR_BASE + 33)
#define CODE_VD_REC_CRC						(UCHAR)(ASC_MDTDDF_ERR_BASE + 34)
#define CODE_VD_CFG_REC_CRC					(UCHAR)(ASC_MDTDDF_ERR_BASE + 35)
#define CODE_SA_CFG_REC_CRC					(UCHAR)(ASC_MDTDDF_ERR_BASE + 36)
#define CODE_PD_DATA_CRC					(UCHAR)(ASC_MDTDDF_ERR_BASE + 37)
#define CODE_VS_LOG_CRC						(UCHAR)(ASC_MDTDDF_ERR_BASE + 38)
		   
/* SAS events */
#define CODE_PROBE_LU_FAILED				0xD1
#define CODE_INITIALIZE_DEVICE_PORT_FAILED		0xD2
#define CODE_PAC_FOR_PORT_FAILED			0xD3
#define CODE_PAC_FOR_ADAPTER_FAILED			0xD4

enum CDA_ERROR_CATEGORY_INIT
{
    CDA_ERROR_CATEGORY_INIT,
    CDA_ERROR_BAD_OSM_CONFIG_PAGE,			/* 0x0001 */
    CDA_ERROR_NO_MEMORY_FOR_SD_PD_POOL,		/* 0x0002 */
    CDA_ERROR_NO_MEMORY_FOR_RP_POOL,		/* 0x0003 */
    CDA_ERROR_NO_MEMORY_FOR_RD_POOL,		/* 0x0004 */
    CDA_ERROR_NO_MEMORY_FOR_RDWAIT_POOL,	/* 0x0005 */
    CDA_ERROR_NO_MEMORY_FOR_IOD_POOL,		/* 0x0006 */
    CDA_ERROR_NO_MEMORY_FOR_OSD_POOL,		/* 0x0007 */
    CDA_ERROR_NO_MEMORY_FOR_INIT_OSD,		/* 0x0008 */
    CDA_ERROR_NO_MEMORY_FOR_INIT_IOD,		/* 0x0009 */
    CDA_ERROR_NO_MEMORY_FOR_INIT_SD,		/* 0x000A */
    CDA_ERROR_NO_MEMORY_FOR_INIT_PD,		/* 0x000B */
    CDA_ERROR_NO_MEMORY_FOR_SD,				/* 0x000C */
    CDA_ERROR_NO_MEMORY_FOR_PD,				/* 0x000D */
    CDA_ERROR_NO_MEMORY_FOR_CD,				/* 0x000E */
    CDA_ERROR_NO_MEMORY_FOR_AD,				/* 0x000F */
    CDA_ERROR_NO_MEMORY_FOR_CRD,			/* 0x0010 */
    CDA_ERROR_NO_MEMORY_FOR_RS,				/* 0x0011 */
    CDA_ERROR_NO_MEMORY_FOR_RAID_BUFFERS,	/* 0x0012 */
    CDA_ERROR_HIM_PROFILE_ADAPTER_FAILED,	/* 0x0013 */
	CDA_ERROR_NO_MEMORY_FOR_TASK_IOD,		/* 0x0014 */
    CDA_ERROR_NO_MEMORY_FOR_IO_HANDLES,		/* 0x0015 */
    CDA_ERROR_NO_MEMORY_FOR_ADAPTER_TSCB,	/* 0x0016 */
    CDA_ERROR_NO_MEMORY_FOR_CHIM,			/* 0x0017 */
    CDA_ERROR_HIMSETCONFIG_FAILURE,			/* 0x0018 */
    CDA_ERROR_HIMCREATEADAPTERTSCB_FAILURE,	/* 0x0019 */
    CDA_ERROR_HIMSETUPADAPTERTSCB_FAILURE,	/* 0x001A */
    CDA_ERROR_HIMVERIFYADAPTER_FAILURE,		/* 0x001B */
    CDA_ERROR_HIMSETMEMORYPOINTER_FAILURE,	/* 0x001C */
    CDA_ERROR_HIMGETPHYTSH_FAILURE,			/* 0x001D */
    CDA_ERROR_TOO_MANY_PHYS,				/* 0x001E */
    CDA_ERROR_TOO_MANY_PORTS,				/* 0x001F */
    CDA_ERROR_HIMINITIALIZE_FAILURE,		/* 0x0020 */
    CDA_ERROR_HIMPROFILEADAPTER_FAILURE,	/* 0x0021 */
    CDA_ERROR_HIMGETPORTTSH_FAILURE,		/* 0x0022 */
	CDA_ERROR_HIMENABLEPHY_FAILURE,			/* 0x0023 */
	CDA_ERROR_HIMGETPORTTSH_NOT_CREATED,	/* 0x0024 */
	CDA_ERROR_HIMRAMSEGMENT_FAILURE,		/* 0x0025 */
    CDA_ERROR_NO_MEMORY_FOR_RSU_POOL,       /* 0x0026 */
    CDA_ERROR_NO_MEMORY_FOR_RSU_BUFFER,     /* 0x0027 */
	CDA_ERROR_SET_ENGINE_PROPERTIES_FAILED, /* 0x0028 */
    
    CDA_UNRECOVERABLE_ADAPTER_FAILURE,      /* 0x0029 */
    CDA_UNRECOVERABLE_PHY_FAILURE,          /* 0x0030 */
    CDA_ERROR_NO_OR_BAD_COOKIE_SIGNATURE,   /* 0x0031 */
	CDA_ERROR_HIMADJUSTPROFILEADAPTER_FAILURE, /* 0x0032 */
	CDA_ERROR_NO_MEMORY_FOR_NUM_SG_ELEMENTS,   /* 0x0033 */

    CDA_ERROR_NO_MEMORY_FOR_CSMIIOD_POOL,      /* 0x0034 */
	CDA_ERROR_UNKNOWN  = 0xFFFF             /* 0xFFFF */

} ;


/* HOST ADAPTER RELATED ERRORS */

#define	KEY_HOST_ERROR		0x80

//JHL #if 0
#define  HOST_ABT_HOST  		0x04	/* Command aborted by host	 */
#define  HOST_ABT_HA			0x05	/* Command aborted by host adapter */
#define  HOST_SEL_TO			0x11	/* Selection timeout */
#define  HOST_DU_DO	 		0x12	/* Data overrun/underrun error  */
#define  HOST_BUS_FREE  		0x13	/* Unexpected bus free  */
#define  HOST_PHASE_ERR 		0x14	/* Target bus phase sequence error */
#define  HOST_INV_LINK  		0x17	/* Invalid SCSI linking operation */
#define  HOST_SNS_FAIL  		0x1b	/* Auto-request sense failed	*/
#define  HOST_TAG_REJ   		0x1c	/* Tagged Queuing rejected by target */
#define  HOST_HW_ERROR  		0x20	/* Host adpater hardware error  */
#define  HOST_ABT_FAIL  		0x21	/* Target did'nt respond to ATN(RESET)*/
#define  HOST_RST_HA			0x22	/* SCSI bus reset by host adapter */
#define  HOST_RST_OTHER 		0x23	/* SCSI bus reset by other device */
#define  HOST_NOAVL_INDEX		0x30	/* SCSI bus reset by other device */
#define  HOST_DETECTED_ERR		0x48	/* Initiator detected error (parity) */
//#endif

/*  Reported Sense Information */

/*
		HOST_ABT_HOST
			Key	KEY_HOST_ERROR
			Code	HOST_ABT_HOST
			Info LBA	N/A
		HOST_ABT_HA
			Key	KEY_HOST_ERROR
			Code	HOST_ABT_HA
			Info LBA	N/A
		HOST_SEL_TO
			Key	KEY_HOST_ERROR
			Code	HOST_SEL_TO
			Info LBA	N/A
		HOST_DU_DO
			Key	KEY_HOST_ERROR
			Code	HOST_DU_DO
			Info LBA	N/A
		HOST_BUS_FREE
			Key	KEY_HOST_ERROR
			Code	HOST_BUS_FREE
			Info LBA	N/A
		HOST_PHASE_ERR
			Key	KEY_HOST_ERROR
			Code	HOST_PHASE_ERR
			Info LBA	N/A
		HOST_INV_LINK
			Key	KEY_HOST_ERROR
			Code	HOST_DU_DO
			Info LBA	N/A
		HOST_SNS_FAIL
			Key	KEY_HOST_ERROR
			Code	HOST_SNS_FAIL
			Info LBA	N/A
		HOST_TAG_REJ
			Key	KEY_HOST_ERROR
			Code	HOST_TAG_REJ
			Info LBA	N/A
		HOST_HW_ERROR
			Key	KEY_HOST_ERROR
			Code	HOST_HW_ERROR
			Info LBA	N/A
		HOST_ABT_FAIL
			Key	KEY_HOST_ERROR
			Code	HOST_ABT_FAIL
			Info LBA	N/A
		HOST_RST_HA
			Key	KEY_HOST_ERROR
			Code	HOST_RST_HA
			Info LBA	N/A
		HOST_RST_OTHER
			Key	KEY_HOST_ERROR
			Code	HOST_RST_OTHER
			Info LBA	N/A
		HOST_NOAVL_INDEX
			Key	KEY_HOST_ERROR
			Code	HOST_NOAVL_INDEX
			Info LBA	N/A
*/


#endif

