/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:ses2dev.h

File Description
	Structures for SES-2 Diagnostic Pages


***********************************************************************/

#ifndef SES2DEV_INCLUDED
#define SES2DEV_INCLUDED

#include "sysdef.h"

typedef enum 
{
   SES2_PAGE_SUPPORTED_DIAGNOSTICS      = 0x00,
   SES2_PAGE_CONFIGURATION              = 0x01,   
   SES2_PAGE_ENCLOSURE_STATUS           = 0x02,
   SES2_PAGE_HELP_TEXT                  = 0x03,   
   SES2_PAGE_STRING_IN                  = 0x04,   
   SES2_PAGE_THRESHOLD_IN               = 0x05,   
   SES2_PAGE_ELEMENT_DESCRIPTOR         = 0x07,
   SES2_PAGE_SHORT_ENCLOSURE_STATUS     = 0x08,
   SES2_PAGE_ENCLOSURE_BUSY             = 0x09,
   SES2_PAGE_DEVICE_ELEMENT_STATUS      = 0x0A,
   SES2_PAGE_SUBENCLOSURE_HELP_TEXT     = 0x0B,
   
   SES2_PAGE_SUBENCLOSURE_STRING_IN     = 0x0C,
   SES2_PAGE_SUPPORTED_SES_DIAGNOSTICS  = 0x0D,
   SES2_PAGE_VENDOR_SPECIFIC_SES_PAGE_0 = 0x10,
   SES2_PAGE_VENDOR_PAGE_A0             = 0xA0,
} SES2PageCodes;

typedef enum
{
   SES2_ELEMENT_UNSPECIFIED             = 0x00,
   SES2_ELEMENT_DEVICE                  = 0x01,
   SES2_ELEMENT_POWER_SUPPLY            = 0x02,
   SES2_ELEMENT_FAN                     = 0x03,
   SES2_ELEMENT_TEMPERATURE_SENSOR      = 0x04,
   SES2_ELEMENT_DOOR_LOCK               = 0x05,
   SES2_ELEMENT_SPEAKER                 = 0x06,
   SES2_ELEMENT_ES_CONTROLLER           = 0x07,
   SES2_ELEMENT_SCC_CONTROLLER          = 0x08,
   SES2_ELEMENT_NONVOLATILE_CACHE       = 0x09,
   SES2_ELEMENT_UPS                     = 0x0B,
   SES2_ELEMENT_DISPLAY                 = 0x0C,
   SES2_ELEMENT_KEYPAD                  = 0x0D,
   SES2_ELEMENT_ENCLOSURE               = 0x0E,
   SES2_ELEMENT_SCSI_TRANSCEIVER        = 0x0F,
   SES2_ELEMENT_LANGUAGE                = 0x10,
   SES2_ELEMENT_COMM_PORT               = 0x11,
   SES2_ELEMENT_VOLTAGE_SENSOR          = 0x12,
   SES2_ELEMENT_CURRENT_SENSOR          = 0x13,
   SES2_ELEMENT_SCSI_TARGET_PORT        = 0x14,
   SES2_ELEMENT_SCSI_INITIATOR_PORT     = 0x15,
   SES2_ELEMENT_SIMPLE_SUBENCLOSURE     = 0x16,
   SES2_ELEMENT_ARRAY_DEVICE            = 0x17,
   SES2_ELEMENT_VENDOR_SPECIFIC         = 0x80, /* 0x80 - 0xFF */
} EnumSESElementTypes;


typedef struct _SES2ReceiveDiagnosticResults
{
    UCHAR OperationCode;
    UCHAR PCV:1;
    UCHAR Reserved:7;
    UCHAR PageCode;
    UCHAR AllocationLength[2];
    UCHAR Control;
}SES2ReceiveDiagnosticResults, *PSES2ReceiveDiagnosticResults;

typedef struct _SES2SendDiagnostic
{

    UCHAR OperationCode;
    UCHAR UnitOffL:1;
    UCHAR DevOffL:1;
    UCHAR SelfTest:1;
    UCHAR Reserved:1;
    UCHAR PF:1;
    UCHAR SelfTestCode:3;
    UCHAR ParamLength[2];
    UCHAR Control;
}SES2SendDiagnostic, *PSES2SendDiagnostic;

typedef struct _SES2EnclosureDescriptorHeader
{
   UCHAR EnclosureServiceProcessesCount:3;
   UCHAR Reserved0a:1;
   UCHAR RelativeEnclosureServiceProcessID:3;
   UCHAR Reserved0b:1;
   UCHAR SubEnclosureID;
   UCHAR NumberOfElementTypesSupported;
   UCHAR EnclosureDescriptorLength;
   
} SES2EnclosureDescriptorHeader, *PSES2EnclosureDescriptorHeader;

typedef struct _SES2EnclosureDescriptor
{
   UCHAR EnclosureLogicalID[8];
   UCHAR EnclosureVendorID[8];
   UCHAR ProductID[16];
   UCHAR ProductRevisionLevel[4];
   UCHAR VendorSpecific; // variable length and optional, may not exists in response page
} SES2EnclosureDescriptor, *PSES2EnclosureDescriptor;

typedef struct _SES2ConfigurationPageHeader
{
   UCHAR PageCode;
   UCHAR SubEnclosureCount;
   UCHAR PageLength[2];
   
   UCHAR GenerationCode[4];
  
} SES2ConfigurationPageHeader, *PSES2ConfigurationPageHeader;

typedef struct _SES2TypeDescriptorHeader
{
   UCHAR ElementType;
   UCHAR NumberOfPossibleElements;
   UCHAR SubEnclosureID;
   UCHAR TypeDescriptorTextLength;
} SES2TypeDescriptorHeader, *PSES2TypeDescriptorHeader;

/* Definition of ElementType */
/* There are more element types described in SES-2 specs. */
/* We are interested only array and device type element */
#define ARRAY_DEVICE_TYPE   0x17
#define DEVICE_TYPE         0x01        

typedef struct _SES2DeviceElementStatusPage
{
   UCHAR PageCode;
   UCHAR Reserved;
   UCHAR PageLength[2];   
   UCHAR GenerationCode3;
   UCHAR GenerationCode2;
   UCHAR GenerationCode1;
   UCHAR GenerationCode0;
   UCHAR OverallStatus; // variable length
} SES2DeviceElementStatusPage, *PSES2DeviceElementStatusPage;

typedef struct _SES2DeviceElementStatusDescriptor
{
   UCHAR ProtocolID:4;
   UCHAR bEIP:1;
   UCHAR Reserved0:2;
   UCHAR Invalid:1;
   UCHAR DeviceElementStatusDescriptorLength;
   
} SES2DeviceElementStatusDescriptor, *PSES2DeviceElementStatusDescriptor;

typedef struct _SES2DeviceElementStatusDescriptor_ProtocolSpecEIP0
{
   UCHAR PhyDescriptorCount;
   UCHAR NotAllPhys:1;
   UCHAR Reserved3:5;
   UCHAR DescriptorType:2;
}SES2DeviceElementStatusDescriptor_ProtocolSpecEIP0, *PSES2DeviceElementStatusDescriptor_ProtocolSpecEIP0;

typedef struct _SES2DeviceElementStatusDescriptorEIP1
{
   UCHAR ProtocolID:4;
   UCHAR bEIP:1;
   UCHAR Reserved0:2;
   UCHAR Invalid:1;
   UCHAR DeviceElementStatusDescriptorLength;
   UCHAR Reserved1;
   UCHAR ElementIndex;
   
} SES2DeviceElementStatusDescriptorEIP1, *PSES2DeviceElementStatusDescriptorEIP1;

typedef struct _SES2DeviceElementStatusDescriptor_ProtocolSpecEIP1
{
   UCHAR PhyDescriptorCount;
   UCHAR NotAllPhys:1;
   UCHAR Reserved2:5;
   UCHAR DescriptorType:2;
   UCHAR Reserved3;
   UCHAR BayNumber;
#define CDA_INVALID_BAYNUMBER   0xFF
}SES2DeviceElementStatusDescriptor_ProtocolSpecEIP1, *PSES2DeviceElementStatusDescriptor_ProtocolSpecEIP1;


typedef struct _SES2SasPhyDescriptor
{
   UCHAR Reserved0a:4;
   UCHAR DeviceType:3;
   UCHAR Reserved0b:1;
   UCHAR Reserved1;
   UCHAR Reserved2a:1;
   UCHAR SMPInitiatorPort:1;
   UCHAR STPInitiatorPort:1;
   UCHAR SSPInitiatorPort:1;
   UCHAR Reserved2b:4;
   UCHAR Reserved3a:1;
   UCHAR SMPTargetPort:1;
   UCHAR STPTargetPort:1;
   UCHAR SSPTargetPort:1;
   UCHAR Reserved3b:4;
   UCHAR AttachedSASAddress[8];
   UCHAR SASAddress[8];
   UCHAR PhyIdentifier;
   UCHAR Reserved[7];
} SES2SasPhyDescriptor, *PSES2SasPhyDescriptor;

typedef struct _SES2ElementDescriptorPage
{
   UCHAR PageCode;
   UCHAR Reserved1;
   UCHAR PageLength[2];  
   UCHAR GenerationCode3;
   UCHAR GenerationCode2;
   UCHAR GenerationCode1;
   UCHAR GenerationCode0;
   UCHAR Descriptor; // variable length
} SES2ElementDescriptorPage, *PSES2ElementDescriptorPage;

typedef struct _SES2ElementDescriptor
{
   UCHAR Reserved0;
   UCHAR Reserved1;
   UCHAR DescriptorLength[2];   
   UCHAR Descriptor[1]; // variable length
} SES2ElementDescriptor, *PSES2ElementDescriptor;

typedef struct _SES2EnclosureStatusPage
{
   UCHAR PageCode;
   UCHAR UnrecoverableCondition:1;
   UCHAR CriticalCondition:1;
   UCHAR NoncriticalCondition:1;
   UCHAR InformationCondition:1;
   UCHAR InvalidOperationRequested:1;
   UCHAR Reserved1:3;
   UCHAR PageLength[2];   
   UCHAR GenerationCode3;
   UCHAR GenerationCode2;
   UCHAR GenerationCode1;
   UCHAR GenerationCode0;
   UCHAR OverallStatus; // variable length
} SES2EnclosureStatusPage, *PSES2EnclosureStatusPage;

typedef struct _SES2DeviceElementForEnclosureStatusPage
{
    /* Byte 0 */
    UCHAR ElementStatusCode:4;
    UCHAR Swap:1;
    UCHAR Reserved1:1;
    UCHAR Prdfail:1;
    UCHAR Reserved2:1;
    /* Byte 1 */
    UCHAR SlotAddress;
    /* Byte 2 */
    UCHAR Report:1;
    UCHAR Ident:1;
    UCHAR Rmv:1;
    UCHAR ReadyToInsert:1;
    UCHAR EnclosureBypassedB:1;
    UCHAR EnclosureBypassedA:1;
    UCHAR DoNotRemove:1;
    UCHAR AppClientBypassedA:1;
    /* Byte 3 */
    UCHAR DeviceBypassedB:1;
    UCHAR DeviceBypassedA:1;
    UCHAR BypassedB:1;
    UCHAR BypassedA:1;
    UCHAR DeviceOff:1;
    UCHAR FaultReqstd:1;
    UCHAR FaultSensed:1;
    UCHAR AppClientBypassedB:1;
}SES2DeviceElementForEnclosureStatusPage, *PSES2DeviceElementForEnclosureStatusPage;

/* Definition of Element Status Code */
typedef enum
{
   ELEMENT_STATUS_UNSUPPORTED       = 0x00,
   ELEMENT_STATUS_OK                = 0x01,
   ELEMENT_STATUS_CRITICAL          = 0x02,
   ELEMENT_STATUS_NONCRITICAL       = 0x03,
   ELEMENT_STATUS_UNRECOVERABLE     = 0x04,
   ELEMENT_STATUS_NOTINSTALLED      = 0x05,
   ELEMENT_STATUS_UNKNOWN           = 0x06,
   ELEMENT_STATUS_NOTAVAILABLE      = 0x07,
   ELEMENT_STATUS_RESERVED          = 0x08, /* 0x08 - 0x0F reserved */

} EnumElementStatusCode;



typedef struct _SES2ArrayElementForEnclosureStatusPage
{
    /* Byte 0 */
   UCHAR StatusCode:4;
   UCHAR Swap:1;
   UCHAR Reserved0a:1;
   UCHAR PredictedFailure:1;
   UCHAR Reserved0b:1;
   /* Byte 1 */
   UCHAR RebuildRemapAbort:1;
   UCHAR RebuildRemap:1;
   UCHAR InFailedArray:1;
   UCHAR InCriticalArray:1;
   UCHAR ConsistencyCheck:1;
   UCHAR HotSpare:1;
   UCHAR ReservedDevice:1;
   UCHAR OK:1;
   /* Byte 2 */
   UCHAR Report:1;
   UCHAR Identify:1;
   UCHAR Rmv:1;
   UCHAR ReadyToInsert:1;
   UCHAR EnclosureBypassedB:1;
   UCHAR EnclosureBypassedA:1;
   UCHAR DoNotRemove:1;
   UCHAR AppBypassedA:1;
   /* Byte 3 */
   UCHAR DeviceBypassedB:1;
   UCHAR DeviceBypassedA:1;
   UCHAR BypassedB:1;
   UCHAR BypassedA:1;
   UCHAR DeviceOff:1;
   UCHAR FaultRequested:1;
   UCHAR FaultSensed:1;
   UCHAR AppBypassedB:1;
} SES2ArrayElementForEnclosureStatusPage, *PSES2ArrayElementForEnclosureStatusPage;

typedef struct _SES2EnclosureControlPage
{
   UCHAR PageCode;
   UCHAR UnrecoverableCondition:1;
   UCHAR CriticalCondition:1;
   UCHAR NoncriticalCondition:1;
   UCHAR InformationCondition:1;
   UCHAR Reserved1:4;
   UCHAR PageLength[2];   
   UCHAR GenerationCode3;
   UCHAR GenerationCode2;
   UCHAR GenerationCode1;
   UCHAR GenerationCode0;
   UCHAR OverallControl; // variable length
} SES2EnclosureControlPage, *PSES2EnclosureControlPage;

typedef struct _SES2DeviceElementForEnclosureControlPage
{
   UCHAR Reserved0:4;
   UCHAR ResetSwap:1;
   UCHAR Disable:1;
   UCHAR PredictedFailure:1;
   UCHAR Select:1;

   UCHAR Reserved1;

   UCHAR Reserved2a:1;
   UCHAR RequestIdentify:1;
   UCHAR RequestRemove:1;
   UCHAR RequestInsert:1;
   UCHAR Reserved2b:2;
   UCHAR DoNotRemove:1;
   UCHAR Reserved2c:1;

   UCHAR Reserved3a:2;
   UCHAR EnableBypassB:1;
   UCHAR EnableBypassA:1;
   UCHAR DeviceOff:1;
   UCHAR RequestFault:1;
   UCHAR Reserved3b:2;
} SES2DeviceElementForEnclosureControlPage, *PSES2DeviceElementForEnclosureControlPage;

typedef struct _SES2ArrayElementForEnclosureControlPage
{
   UCHAR Reserved0:4;
   UCHAR ResetSwap:1;
   UCHAR Disable:1;
   UCHAR PredictedFailure:1;
   UCHAR Select:1;

   UCHAR RequestRebuildRemapAbort:1;
   UCHAR RequestRebuildRemap:1;
   UCHAR RequestInFailedArray:1;
   UCHAR RequestInCriticalArray:1;
   UCHAR RequestConsistencyCheck:1;
   UCHAR RequestHotSpare:1;
   UCHAR RequestReservedDevice:1;
   UCHAR RequestOK:1;

   UCHAR Reserved2a:1;
   UCHAR RequestIdentify:1;
   UCHAR RequestRemove:1;
   UCHAR RequestInsert:1;
   UCHAR Reserved2b:2;
   UCHAR DoNotRemove:1;
   UCHAR Active:1;

   UCHAR Reserved3a:2;
   UCHAR EnableBypassB:1;
   UCHAR EnableBypassA:1;
   UCHAR DeviceOff:1;
   UCHAR RequestFault:1;
   UCHAR Reserved3b:2;
} SES2ArrayElementForEnclosureControlPage, *PSES2ArrayElementForEnclosureControlPage;





#define IsDigit(c) ( (UCHAR) (c >= '0' && c <= '9') )
#define IsSpace(c) ( (UCHAR) (c == '\t' || c == '\n' || c == ' ') )

#endif
