/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:sg.h

File Description
	Define those structures used for scatter/gather manipulation


***********************************************************************


History :
05/01/2002 : Madhuresh
SG_MAX moved to sysdef.h for future performance tuneup.

01/24/2001 : 03:22 PM   NAR20038  Manjunath
Removed POWEREDCACHE

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.


***********************************************************************/

#ifndef SG_INCLUDED
#define SG_INCLUDED


/* Define a scatter/gather list entry. Set max values	*/

//#define	SG_MAX	64	// now moved to sysdef.h for performance tuneup.

#define SG_TERMINATION	0x80000000L
#define AD_SG_TERMINATION	0x40000000L

#define MAX_SG_ELEMENT_SIZE 0xFFFFFFFF
/*
	Define a scatter/gather list entry.
*/

#define	ATHENA_SG_MAX	64

typedef  struct   _AthenaSGElement32	/* structure size is 8 (0x08) bytes */
{
	ULONG   	SGLBufAddr;
	ULONG		SGLByteCnt;

}  AthenaSGElement32;

typedef  struct   _AthenaSGElement64	/* structure size is 16 (0x10) bytes */
{
    DATA_N_PTR	SGLBufAddr;
    ULONG    SGLByteCnt;
    ULONG    Pad;
} AthenaSGElement64;

#if _LME
typedef AthenaSGElement64 AthenaSGElement;
#else
typedef AthenaSGElement32 AthenaSGElement;
#endif


typedef struct _AthenaSGTable 		/* structure size is 16 or 24 bytes */
{
	USHORT		SGCount ;
	USHORT      SGListMaximum ;	
	DATA_N_PTR		SGPhysAddr ;
	AthenaSGElement	SGList[1] ;

} AthenaSGTable;

typedef  struct   _SGElement32	/* structure size is 8 (0x0)8 bytes */
{
/* 0x0000 */   ULONG	SGLBufAddr;
/* 0x0004 */   ULONG    SGLBufAddrHigh ;
/* 0x0008 */   ULONG    SGLByteCnt;
/* 0x000C */   ULONG    SGLFlags ;	/* Set termination here */
} SGElement32;

typedef  struct   _SGElement64	/* structure size is 16 (0x10) bytes */
{
/* 0x0000 */   DATA_N_PTR	SGLBufAddr;
/* 0x0008 */   ULONG    SGLByteCnt;
/* 0x000C */   ULONG    SGLFlags ;	/* Set termination here */
} SGElement64;

#if _LME
typedef SGElement64 SGElement;
#else
typedef SGElement32 SGElement;
#endif

#define LOCAL_NUM_SG_ELEMENTS  255

/* SGCount is number of SGElements being used in SGList.*/
/* SGListMaximum is the number of SGElements in SGList  */
/*	SGPhysAddr is abs addr of the SGList.					  */

typedef struct _SGTable 		/* structure size is 16 (0x10) bytes */
{
/* 0x0000 */	USHORT			SGCount;
/* 0x0002 */    USHORT			SGListMaximum;	
#if _LME
/* 0x0004 */	ULONG			Pad;
#endif
/* 0x0008 */	DATA_N_PTR		SGPhysAddr;
/* 0x000x */	SGElement		SGList[1];

} SGTable;


#endif
