/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:sharedds.h

File Description
	Define those structures that are shared across different components
	(BIOS, Driver, etc.)
***********************************************************************

History :

10/29/2004	17:30	Kashif
Added the file to source

***********************************************************************/

#ifndef SHAREDDS_INCLUDED
#define SHAREDDS_INCLUDED

#include "sysdef.h"

/***************************************************************************
*  SasOcmBiosDriverDataStructure definitions
*     This structure is used for saving BIOS and Driver OCM structure settings
***************************************************************************/
typedef struct SasOcmBiosDriverDataStructure_
{
   ULONG	Signature;
   UCHAR	StructureMajorVersion;
   UCHAR	StructureMinorVersion;
   UCHAR	BiosMajorVersion;
   UCHAR	BiosMinorVersion;
   ULONG	BiosBuildNumber;
   UCHAR	BiosAttributes;
   UCHAR	PciSlotNumber;
   USHORT	NumberUnitElements;
   USHORT	SizeUnitElement;
   UCHAR	Reserved[14];
} SasOcmBiosDriverDataStructure;

#define SAS_OCM_BIOS_INSTALLED 0x02
#define SAS_OCM_BIOS_SIGNATURE 0x42494f53   /* "BIOS" in big endian */

/***************************************************************************
*  SasOcmBiosDriverDataUnitElement definitions
*     This structure is used for saving BIOS and Driver OCM unit Identifier
***************************************************************************/
typedef struct SasOcmBiosDriverDataUnitElement_
{
   UCHAR	UnitIdentifierType;
#define SSP_DEVICE                      0x00
#define STP_DEVICE                      0x01
#define RAID_GUID                       0x02
#define SGPIO_DEVICE                    0x03
#define SAS_TOPOLOGY_LOCATION_DEVICE    0x04
#define PROFILE_WWN_DEVICE              0x05
   UCHAR	DosDriveNumber;
   UCHAR	Reserved_RemovableUnit;
   UCHAR	Reserved0[4];
   UCHAR	UnitIdentifierLength;
   UCHAR	Reserved1[4];
   UCHAR	Reserved2[4];
   UCHAR	UnitIdentifier[68];				/* used the max length here, in future this can be defined as a union for different types */
   UCHAR    Reserved3[4];
} SasOcmBiosDriverDataUnitElement;

typedef struct ConfigurableFeaturesSectorHeader_
{
   USHORT	Signature;
   UCHAR	Reserved0[2];
   UCHAR	MajorVersionNumber;
   UCHAR	MinorVersionNumber;
   UCHAR	Reserved1[2];
   UCHAR	NoOfConfigurableFeatureDescriptors;
   UCHAR	ConfigurableFeatureDescriptorSize;
   UCHAR	Reserved2[6];
} ConfigurableFeaturesSectorHeader;


typedef struct ConfigurableFeatureDescriptor_
{
	USHORT	ConfigurableFeatureNumber;
	USHORT	ConfigurableFeatureQualifier;
	UCHAR	ConfigurableFeatureDescription[16];
#define MEDIUM_ERROR_HANDLING_STRING		"MED_ERR_HNDLING\0"
#define MEDIUM_ERROR_HANDLING_STRING_LEN	15
#define MEH_LEVEL_STRING					"MEH_LEVEL2\0"
#define MEH_LEVEL_STRING_LEN				10
} ConfigurableFeatureDescriptor;

#define CONFIGURABLE_FEATURE_SIGNATURE		0x4346
#define CONFIGURABLE_FEATURE_SECTOR_CODE	0x0160
#define CONFIGURABLE_FEATURE_DESCRIPTOR_START_OFFSET 0x10

#define MEDIUM_ERROR_HANDLING_FEATURE_NO	0x0001

/***************************************************************************
*  Adaptec Shared Data Structures definitions
*  Adaptec Shared Data Structures Layout as per Rev 17.0 - Vishwanath Maram
***************************************************************************/

typedef struct _ASD_SETTINGS_LAYOUT {
	UCHAR		ID;

#define NVRAM_PCI_SETTINGS_ID 		0x50
#define NVRAM_PHY_SETTINGS_ID   	0x68
#define NVRAM_BOOT_UNIT_SETTINGS_ID 0x42

	UCHAR		Reserved;
	USHORT		Next_Ptr;
} ASD_SETTINGS_LAYOUT,
  *PASD_SETTINGS_LAYOUT;

typedef struct ASD_PCI_SETTINGS_LAYOUT_
{
   ASD_SETTINGS_LAYOUT SettingsLayout;

   UCHAR			MajorVersion;                      
   UCHAR			MinorVersion;
   USHORT			SettingsCheckSum;           /* settings checksum */
   USHORT			SettingsImageSize;          /* settings Image Size */

   UCHAR			Reserved[22]; //These fields are not defined as these are not used by Driver as of now.

} ASD_PCI_SETTINGS_LAYOUT,
  *PASD_PCI_SETTINGS_LAYOUT;

#define NumberOfBootUnits		1

typedef struct ASD_BOOT_UNIT_SETTINGS_ENTRY_
{
   UCHAR			BootFunctionIdentifierType;     
   UCHAR			BootUnitIdentifierType;
   UCHAR			BootLuAttributes;			/*Reserved[4] SpinUpLu[1] WriteCachePolicy[2] ValidEntry[2]*/
   UCHAR			Reserved1[3];
   UCHAR			BootFunctionIdentifierLength;
   UCHAR			BootUnitIdentifierLength;
   UCHAR			Reserved2[4];
   UCHAR			BootFunctionIdentifier[4];	/*For Rev 17 this value is 4*/
   UCHAR			BootUnitIdentifier[88];		/*check it out???? For Rev 17 this value is 0x44 or 0x58????*/
} ASD_BOOT_UNIT_SETTINGS_ENTRY,
  *PASD_BOOT_UNIT_SETTINGS_ENTRY;

typedef struct ASD_BOOT_UNIT_SETTINGS_LAYOUT_
{
   ASD_SETTINGS_LAYOUT SettingsLayout;
   
   USHORT		NumOfBootUnitSettings;		/*Right now it is considered as 1 for this variable*/                      
   USHORT		Reserved2;
   
   ASD_BOOT_UNIT_SETTINGS_ENTRY	BootUnitEntry[NumberOfBootUnits];	/*As of now number of boot units will be only one*/

} ASD_BOOT_UNIT_SETTINGS_LAYOUT,
  *PASD_BOOT_UNIT_SETTINGS_LAYOUT;

#define NumOfPhys		8

typedef struct ASD_PHY_SETTINGS_LAYOUT_
{
   ASD_SETTINGS_LAYOUT SettingsLayout;
	
   UCHAR			NumberOfPhys;
   UCHAR			Reserved1[3];

   UCHAR			Reserved2[NumOfPhys * 16]; //These fields are not defined as these are not used by Driver as of now.

} ASD_PHY_SETTINGS_LAYOUT,
  *PASD_PHY_SETTINGS_LAYOUT;

#endif

