/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:stats.h

File Description
	Define those structures used for statistic information.

***********************************************************************

History :

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.


***********************************************************************/


#ifndef STAT_INCLUDED
#define STAT_INCLUDED

/*
Define the structure for statistic information per device; that is,
for each physdevice.
*/


/* This represents the distribution of I/O requests:512, 1k, 2k,*/
/* 4k, 8k, 16k, 32k, 64k, 128k, 256k, 512k, and 1024k		*/
#define	MAX_IO_DISTRIBUTION		12
#define	MAX_IO_BLOCK			MAX_IO_DISTRIBUTION-1
#define	MAX_IO_BLOCK_VALUE		1 << MAX_IO_BLOCK

/*
Define statistics kept for each system device. If the device is not
an array the value of SysSplits will always be zero.
*/

typedef	struct	_SysDeviceStats 	/* structure size is 72 bytes */
{
 /* 0000 */	ULONG		SysReqsQd;	/*Total queued to the device	*/
 /* 0004 */ ULONG		cSysTotalReadDone;	/* Total read requests completed*/
 /* 0008 */ ULONG		cSysTotalWriteDone;	/* Total read requests completed*/
 /* 000C */	ULONG		cSysBlocksRead;		/* No. of sector/blocks read */
 /* 0010 */	ULONG		cSysBlocksWritten;	/* No. of sector/blocks written */
 /* 0014 */	ULONG		SysReadBlocksReq[MAX_IO_DISTRIBUTION];
 /* 0044 */	ULONG		SysWriteBlocksReq[MAX_IO_DISTRIBUTION];
} SysDeviceStats;

/*
Define the physical device statistics kept for all devices. If the device
is not block oriented the values of blocks read/written are meaningless.
*/

typedef	struct _PhysDeviceStats 	/* structure size is 124 bytes */
{
 /* 0000 */	ULONG		cPhysBlocksRead;
 /* 0004 */	ULONG		cPhysBlocksWritten;
 /* 0008 */	USHORT		PhysCrntQd;
 /* 000A */	USHORT		Padding;
 /* 000C */ ULONG		cPhysTotalReadDone; 
 /* 0010 */ ULONG		cPhysTotalWriteDone; 
 /* 0014 */	ULONG		PhysReadBlocksReq[MAX_IO_DISTRIBUTION];
 /* 0044 */	ULONG		PhysWriteBlocksReq[MAX_IO_DISTRIBUTION];
 /* 0074 */	ULONG		PhysSoftErrors;
 /* 0078 */	ULONG		PhysHardErrors;
} PhysDeviceStats;

typedef struct _SysDeviceCacheStats			/* structure size is 32 bytes */
{
 /* Number of Blocks Read and Written seen by Cache code. */
 /* 0000 */ ULONG		cCacheBlocksRead;	/* Total no. of blocks read */
											/* in cache block size */
 /* 0004 */ ULONG		cCacheBlocksWritten;/* Total no. of blocks written */
											/* in cache block size */
 /* 0000 */ ULONG		cCacheTotalReadDone;								
 /* 0000 */ ULONG		cCacheTotalWriteDone;								

		/* The sum of cDemand CacheHits, cLookAheadCacheHits, */
		/* cWriteBackCacheHits, and cWriteThruCacheHits gives */
		/* the total read cache hits in cache block size.	  */
 /* 0008 */	ULONG		cDemandCacheHits;
 /* 000C */ ULONG		cLookAheadCacheHits;
 /* 0010 */ ULONG		cWriteBackCacheHits;
 /* 0014 */ ULONG		cWriteThruCacheHits;

 /* 0018 */ ULONG		cParityCacheHits;
 /* 001C */ ULONG		cCacheMergers;

#ifdef _ADAPTIVECACHE
/* 5/15/97 JHC added parameters for adaptive look ahead caching */
	UBYTE				AdaptiveLookAhead;
	UBYTE				PrevAdaptiveLookAhead;
	USHORT				DiskToCacheTime;/*time to transfer a cache blk form disk to cache*/
	USHORT				DiskToHostTime;
	USHORT				CacheToHostTime;
	USHORT				MinAdaptiveTimeout;
	USHORT				MaxAdaptiveTimeout;
	USHORT				WorkingAdaptiveTimeout;
	USHORT				RunningAdaptiveTimeout;
#endif

 /* This reflects I/O distribution from OS I/O requests */
 /* seen by cache code.								*/
 /* 0020 */ ULONG		CacheReadBlocksReq[MAX_IO_DISTRIBUTION];
 /* 0024 */ ULONG		CacheWriteBlocksReq[MAX_IO_DISTRIBUTION];

} SysDeviceCacheStats, DPTR pSysDeviceCacheStats;

#define IO_STAT_PAGE	1	/* Returns IO and Cache statistics */
				/* of an array or single disk device */
#define CACHE_STAT_PAGE	3	/* Returns only Cache statistics */
#define PHYS_STAT_PAGE	4	/* Returns members IO statistics */
typedef struct _PageBlock
{
	USHORT	PageLength;			/* Size (byte count) of a Page */
	USHORT	PageType;			/* Defined as above: Sysdevice I/O, */
								/* Cache I/O, or Physdevice I/O statistics */
	ULONG	PageBufferOffset;	/* Pointer to either SysDeviceStats, */
								/* SysDeviceCacheStats, or PhysDeviceStats */
								/* depending on page type. 		*/
}PageBlock, DPTR pPageBlock;
typedef struct _StatisticsBlock
{
	USHORT		cSB_TotalPage;	/* Number of pages returned */
	USHORT		Padding;
	PageBlock	SB_PageBlock[1];
} StatisticsBlock, DPTR pStatisticsBlock;
/*
L designates those that are LOCALLY defined.
R designates those that are REMOTELY defined. Remote definitions are the
or'ing of the msb into the definition of the local device. Note that
this valid for a BYTE wide item only.
*/

#define REMOTE				0x80
#define ARRAY				0x40
#define LSINGLE				1
#define LRAID0				2
#define LRAID1				3
#define LRAID10				4
#define LRAID5				5
#define LRAID50				6
#define LSPARE				7
#define LOTHER				8
									
#define BUS_ADAPTER			9
#define XOR_ADAPTER			10
#define CONTROLLER			11

#define LSES2_DEVICE        12

/* Members with OCE capability will be mark as */
/* unknown device for our current Athena environment. */
#define LUNKNOWN			13
										
/* Defines for ArrayState */			
#define ARRAY_STATE_OK			0x00
#define ARRAY_STATE_FAILED		0x01
#define ARRAY_STATE_RECOVERY	0x02
/* Define for Redundancy State that gets written to disk reserved sector */
/* This state indicates whether system has shutdown properly. During
   startup if ArrayState is in ARRAY_STATE_OK, then we update the
	reserved sector Redundancy State as ARRAY_STATE_DIRTY, and it stays
	DIRTY until a proper shutdown occurs, then if we find that our
	ArrayState is still ARRAY_STATE_OK, then we update the reserved
	sector Redundancy state from ARRAY_STATE_DIRTY to ARRAY_STATE_OK.
*/	
#define ARRAY_STATE_DIRTY		0x03

/* Bit defines for RedunancySubState */
#define ARRAY_STATE_DIRTY_DURING_BOOT		0x01

/* Defines for ArraySubState
 * This is used with ARRAY_STATE_FAILED condition - the reason
 * why array is in failed state.
 */
#define ARRAY_SUBSTATE_OK				0x0000
#define ARRAY_SUBSTATE_UNINITIALIZED	0x0001  /* Failed to complete zeroing */
												/* or formating devices */
#define ARRAY_SUBSTATE_INITIALIZING		0x0002  /* Format or zero device running */
#define ARRAY_SUBSTATE_FAILED_DRIVES	0x0003	/* More than one drive failed */
#define ARRAY_SUBSTATE_DELETED			0x0004	/* Array deleted */
#define ARRAY_SUBSTATE_LOCKED			0x0005	/* I/O suspended for this array */
#define ARRAY_SUBSTATE_RESCAN			0x0006	/* Arary recognized at rescan */


#define ARRAY_SUBSTATE_METADATA_UPDATE_PENDING	0x0007 /* Auto rebuild/HSR rebuild is pending on reserved sector
														* update to be completed.
													    */
#define ARRAY_SUBSTATE_HSR_START	    0x0008 /* Indicates that HSR should start on this SysDevice */

#define ARRAY_SUBSTATE_RESOURCE_LIMIT 	0x0009 /* resource allocation limit*/
#define ARRAY_SUBSTATE_RESOURCE_CHECK 	0x8000 /* a temporary check mark */

#define ARRAY_SUBSTATE_ZAP_RESERVED_SECTOR  0x000a

/*
 * Defines for ArrayOpActive
 */
#define ARRAY_NO_OP					0x00	/* No RAID management operation is active */
#define ARRAY_VERIFY_OP    			0x01    /* Array verify operation is active */
#define ARRAY_REBUILD_OP			0x02	/* Array recreate operation is active */
#define ARRAY_ZERO_OP				0x03	/* Array zero operation is active */
#define ARRAY_FORMAT_OP				0x04	/* Array Format operation is active */

/* Values for the state of the physical device (PhysState) */
#define PHYS_STATE_ACTIVE				0
#define PHYS_STATE_FAILED				1
/* This missing state is only for IOmanager to distinguish a missing */
/* drive vs. a failed drive. The driver only keep two states defined above.*/
#define PHYS_STATE_MISSING				2
/* This is for IOmanager to distinguish the drive is under reconstructing.	*/
#define PHYS_STATE_RECONSTRUCTING		3



/*
** define for the pPD substate
*/
#define PHYS_SUBSTATE_REBUILDING			0x01

/* For message passing to/from BIOS */

/* Message types for status changes */
#define DOWN_DRIVE_MSG          0x01
#define SWAP_DRIVE_MSG          0x02
#define START_RECREATE_MSG      0x03
#define END_RECREATE_MSG        0x04
#define UP_DRIVE_MSG            0x05
#define DOWN_ARRAY_MSG	        0x06
#define CHANGE_TYPE_MSG         0x07
/* Message direction for status changes */
#define	BIOS_TO_OS				1
#define	OS_TO_BIOS				2
#endif
