/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: sysdef.h

File Description
Now it contains only common performance, features and core definitions
and their default values.

********* DO  NOT  INCLUDE IN APPLICATION OR OSM FILES **********

********* TO BE USED BY ONLY CORE FILES **********

Abstraction of basic data types to allow expression of pointers, etc
across multiple operating systems.

NOTE!
	DPTR    is model independant
	NP      is a near pointer and is VERY model dependant

	All data structures should use these pointer types. NOT the base type
	defined.
***********************************************************************

History :

03/19/2005 : 20:00	Shing-Kin Pun
Added Compilation flag #ifdef (_RAID_10) to enable or disable
MULTI_LEVEL_RAID. This allows the file "sources" in the build directory
to control the conditional compile.
_RAID_10 is set in sources as C_DEFINES= -D_RAID_10.

03/19/2005 : 12:00	Shing-Kin Pun
The following changes are for Multi-level RAID support.
1.  Added Conditional Compile Flag, MULTI_LEVEL_RAID.
2.  Added conditional compile to turn on SUPPORTS_RAID10.
	#define SUPPORTS_RAID_LEVEL (SUPPORTS_RAID0 | SUPPORTS_RAID1 | SUPPORTS_RAID10)

09/14/2004 : 14:18 BURN11729 Farrukh
SG_MAX updated to 0x11 for hibernation to work. This will break Psuite.

08/10/2004 : 13:45 NAR20038  Manjunath
Disabled support for R10.

11/09/03: Madhuresh
Added couple of consts and removed #if 0 block on chips supported struct.

10/29/03: Madhuresh
Added CUSTOM.H. HIM_TS_SCSI to avoid compilation errors.

10/8/03: Madhuresh
Definitions have been moved to OSM shareable files cdadef.h and osmcda.h.
This file should not be included by application any longer. Inlcude all
needed core headers here so that core sources need include only
sysdef.

10/03/2003: Madhuresh
Some definitions temporarily kept here until cdadef.h gets included in
all core files.

07/18/2003 : 15:00 RUIZ Tony 
Removed RE_MAX_LUNS_PER_TARGET and MaxLunPerID.  We are using a varible in the AdapterDevExt 
instead to scan the number of luns.


06/18/2003: Madhuresh
MAX_LOGICAL_ARRAYS changed to MAX_RAID_BUFFERS_FOR_BG_TASKS same as number of
RAIDs on the bus and raid buffers moved into global space instead of the SD.

05/06/2003  13:20 NAR20038 Manjunath
Re-enabled drive write cache since now we know that this is not a cause for
mis-compare during VERIFY under heavy I/O load. Also disabled Double buffering.

04/19/2003  11:00 NAR20038 Manjunath
Changes common to both SCSI and SATA - updated from SnakeEye branch.

04/09/2003  NAR20038  Manjunath
Increased number of global spares supported to 2.

Dec 23, 2002: Madhuresh.
Added EXPERIMENT_BUFFER_SIZE under NEC to add a 32K buffer to each OSD
where we copy all writes to RAIDs to avoid miscompare on NT's Pagefile access.

Oct 27, 2002: Madhuresh.
Turned the drive write cache off

09/27/2002 Farrukh
Number of Dedicated and Global spare is now defined in #ifdef 

09/26/2002 Farrukh
Defined NEC in this file rather then in makefile since we are branching.

09/24/2002 Farrukh
Changed number of global/pool spares support to 1.

09/19/2002 Farrukh
Enabled Global Spare support for NEC

07/16/2002  Farrukh 
More changes made for Exporting Capabilities to IO Manager for IBIS support.
Defines for LINUX added. (Per Rakesh request)

06/11/2002  Farrukh 
Changes made for Exporting Capabilities to IO Manager for IBIS support.
Defines for LINUX added. (Per Rakesh request)

06/04/2002	Madhuresh
New Driver capability related deinitions are added in this file. Also
BALANCE_LOAD_FACTOR is now set to 4.

05/21/2002	Madhuresh
All PCI device IDs are stored in AdaptecChipsSupported struct defined here and
stored in ID.C. Also the driver version and name are declared as strings.

05/15/2002 : 14:29 BUR11729  Farrukh
Modified PRODUCTID_SIZE to 16 characters.  In _SCSIInqData struct ProductID 
is 16 characters.

05/01/2002 : Madhuresh
New constant called BALANCE_LOAD_FACTOR and SG_MAX moved from SG.H to here.
OS specific files to use SG_MAX instead of their local definitions.


12/18/2001 : 14:59 NAR20038  Manjunath
Removed #define PROFILER

07/18/2001 : 18:05 NAR20038  Manjunath
Decreased write same sector count to 0X1000 for NT, some drive f/w
used to cause problems when the count FFFF was given.

04/16/2001 : 14:43 NAR20038  Manjunath
Change PHYS_TAGS_MAX to 64. Now we can send 64 commands to a target.

04/16/2001 : 14:43 NAR20038  Manjunath
Increased write same sector count to 0XFFFF (33 MB) from 0x1000 for NT.

03/26/2001 : 14:43 NAR20038  Manjunath
Moved PRODUCTID_SIZE from driver specific code. Now this constant is
applicable for all OSs and other components like IOMgr.

03/22/2001 : 11:25 NAR20038  Manjunath
MAX_LOGICAL_ARRAYS changed to 2 from 4. Reference for it is
removed from confpage.h

03/20/2001 : 12:12 NAR20038  Manjunath
Updated PRODUCTID_SIZE to 32 characters.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Removed code under #if 0
2. Removed #ifdef ATHENA
3. Removed #define ENABLE_R1WRITE_PROXY
***********************************************************************/

#ifndef SYSDEF_INCLUDED
#define SYSDEF_INCLUDED

#include "cdadef.h"

#define	SAS_BRINGUP             TRUE	
#define AUTO_REBUILD		TRUE
#define SAS                     TRUE
#define AUTO_VERIFY		TRUE
#define AUTO_CLEAR		TRUE

//TDE
/* Drive cache setting does not work on SAS drive */
//KenC: Need to be 1 for WCE support
#define ENABLE_DRIVE_CACHE_SETTING     1 
//KenC: Compile SMaRT for debugging Amrik's Smart problem
#define SMART_SUPPORT                  1    /* Enabled 04/06/2005 */

#ifdef FTTT
#define CDA_ENABLE_FTTT_SUPPORT        1
#else
#define CDA_ENABLE_FTTT_SUPPORT        0
#endif

#define CDA_ENABLE_LU_VALIDATION_SUPPORT  1

/* The following flag enable or diable the */
/* init time disable phy code path */
#define CDA_INITTIME_DISABLE_PHY          1


//
//  MULTI_LEVEL_RAID -  TRUE  = turn ON Multi-level RAID support.
//                      FALSE = turn OFF Multi-level RAID support.
//
//  This flag is used in mdtddf.c and mdtddf.h.

/*
	Peformance and feature related constants
*/
#define SUCCESS                     0
#define FAILURE                     -1
#define ALLOW_PROXY					FALSE	/* To merge the IODs if contiguous */
#define	BALANCE_LOAD_FACTOR			4		/* Number of R1Reads to one drive before switch to other. */

//#ifdef _LME
/* For 64 bit driver reduce the S/G  else allocated memory is not enough */
//#define	SG_MAX						0x11	/* 0x64 - To allow large commands particularly write buffer for drive f/w upgrade */											/* 0x11 - For Hibernation to work. It will break Psuite.						  */
//#else
//#define	SG_MAX						0xFF    /* 0xFF - To allow large commands particularly write buffer for drive f/w upgrade */											/* 0x11 - For Hibernation to work. It will break Psuite.                          */  
//#endif

#define	QUEUE_DEPTH_FOR_OS			256		/* So many OSDs should be allocated */
#define	PHYS_TAGS_MAX				16		/* So many commands will be queued to drive if it can take */
#define DDF_DRIVE_SIZE_COERCION  TRUE		/* TDE - merged code */

#define IOD_TO_VALUE				4000000L/* 4c4b40, 5sec for ChkForIODTimeout() */

#define	SUPPORT_GLOBAL_SPARE	
#define	SUPPORT_HOT_SWAP_REBUILD	TRUE

#define WRITE_SAME_ENABLED          FALSE	/* was TRUE. Temporarily changing for current build */

#define WRITE_SAME_SECTOR_COUNT 	0x1000	/* 32 MB at a time */
#define MIN_RD						20
#define BYTES_PER_SECTOR            512
#define DISK_BYTES_PER_BLOCK        512
#define MAX_SABRE_MAP               1024
#define SECONDS_PER_TICK            5		/* 5 seconds per system tick */

#define PHYS_TICKS_MAX              15    /* This is 7.5 seconds timeout */

#define CONFIG_BUFF_SIZE            4096
#define FIRST_PARTITION             0x1BE
#define PART_HEAD_OFFSET            0x5
#define PART_SPT_OFFSET             0x6
#define PART_SIGNATURE              0xAA55
#define CONFIG_SIGNATURE            "ADAPTEC ARRAY"
#define CONFIG_SIGNATURE_LEN        13
#define NO_BOARDS                   11
#define PERFORMANCE
#define MAXSPARES					1
#define TOP_BRANCH                  2
#define TYPE_ATHENA_PACKET          1
#define TYPE_REMOTE_PACKET          2
#define	PCI_BASE_IS_IO              0x01	/* bit 0 is set in io-mapped bases	 */
#define	PCI_MAX_32_BIT_BARS         6
#define	CDA_MODEPAGE_SIZE           CONFIG_BUFF_SIZE
#define	CDA_BG_BUFFER_SIZE          (2 * MAX_FUNC_SIZE * MAX_RAID_BUFFERS_FOR_BG_TASKS * DISK_BYTES_PER_BLOCK)

#define CDA_DMA_SENSE_BUFFER_SIZE   256
#define CDA_MAX_DMA_SENSE_BUFFERS   4     /* number of sense buffers (256-bytes each) for removable media devices */
                                          /* This means the first 4 removable media devices can support long sense */
#define CDA_DMA_SENSE_BUFFERS_ALLOC_SIZE (CDA_DMA_SENSE_BUFFER_SIZE * CDA_MAX_DMA_SENSE_BUFFERS)

#ifdef SES2_DEVICE
#define LED_BUFFER_SIZE             512
#endif


#define	SAS_CHIP_MAX_NUM_PHY        (8+2) //Includes the Virtual Phy
#define	SAS_CHIP_MAX_NUM_PORTS      ((8*2)+2) //Includes one virtual port
                                              /* Added 8 more ports to support the FTTT */

#ifdef SES2_DEVICE
#define MAX_I2C_DEVICE_COUNT        2
#define LED_BUFFER_SIZE             512
#endif


/* TDE - changed to 100 microsecond for now  until
 * Miramar problem is fixed. This will pass the OS installation.
 * 
 */
//#define	CDA_POLLED_IO_INTERVAL      10      /* in microseconds. Only for commands executed by polling during initialization */
#define	CDA_POLLED_IO_INTERVAL      100     /* in microseconds. Only for commands executed by polling during initialization */
#define	CDA_POLLED_IO_TIMEOUT       60      /* in seconds. Only for commands executed by polling during initialization */
/*
	The following options will issue Mode select to the drive if set to TRUE.
*/
#define DISABLE_DRIVE_WRITE_CACHE	FALSE
#define TEST_PFA					FALSE
#define PRODUCTID_SIZE              16     /* product id string size */
#define PRODUCT_ID_SHIFT            16

/*
	Constants to describe the capabilities of this RAID engine ( Prefixed RE_ )
	We export this to the API in controller modepage.
	Define the feature set supported for management. Set in CF_Capabilities.
*/
#define CAP_SPARE_POOL						0x01L
#define CAP_DEDICATED_SPARE					0x02L
#define CAP_LOGICAL_ARRAYS					0x04L
#define CAP_LAYERED_ARRAYS					0x08L
#define CAP_SEPARATE_STRIPE_SIZE			0x10L
#define CAP_SPAN_CONTROLLERS				0x20L
#define CAP_AUTO_VERIFY						0x40L
#define CAP_AUTO_FIX						0x80L
#define CAP_RESERVED0_FOR_IOM				0x100L
#define CAP_CTLR_SUPPORTS_SAFTE				0x200L
#define CAP_CTLR_SUPPORTS_SES				0x400L
#define CAP_SUPPORTS_DECFAULT_BUS			0x800L

#if		defined (SUPPORT_GLOBAL_SPARE)
#define CAP_FEATURE_SET		(CAP_SPARE_POOL | CAP_LOGICAL_ARRAYS | CAP_AUTO_VERIFY | CAP_AUTO_FIX)
#elif	defined (SUPPORT_DEDICATED_SPARE)
#define CAP_FEATURE_SET		(CAP_DEDICATED_SPARE | CAP_LOGICAL_ARRAYS | CAP_AUTO_VERIFY | CAP_AUTO_FIX)
#else 
#define	CAP_FEATURE_SET		(CAP_DEDICATED_SPARE | CAP_LOGICAL_ARRAYS | CAP_AUTO_VERIFY | CAP_AUTO_FIX)
#endif


/* Define the RAID levels supported.  Set in CF_RaidDefinitions. */

#define SUPPORTS_NO_RAID_LEVEL              0x00
#define SUPPORTS_RAID0						0x01
#define SUPPORTS_RAID1						0x02
#define SUPPORTS_RAID3						0x04
#define SUPPORTS_RAID4						0x08
#define SUPPORTS_RAID5						0x10
#define SUPPORTS_RAID10						0x20
#define SUPPORTS_RAID50						0x40

#define	RE_SUPPORT_CACHING					FALSE	/* Data caching is done by this RAID engine? */
#define	RE_SUPPORT_QUICK_INIT				TRUE	/* Can OS access the RAID while it is partially intialized? */
#define	RE_SUPPORT_CONTAINERS				FALSE	/* Can more than one RAID reside on a disk? */
#define	RE_SUPPORT_MORPHING					FALSE	/* Can morph from one type of RAID to another? */
#define	RE_SUPPORT_OCE						FALSE	/* Online Capacity Expansion? */
#define	RE_SUPPORT_VERIFY					TRUE	/* Read mirrors and compare? */
#define RE_SUPPORT_FORMAT_WITH_CREATION		FALSE	/* After Array creation initialization does not occur automatically */
#define RE_SUPPORT_CLUSTERING				FALSE
#define	RE_SUPPORT_SCRUB					RE_SUPPORT_VERIFY /* High end RAIDs call verify as scrubbing. */
													/* IBM expects no data compare. */
#define RE_SUPPORT_BATTERY					FALSE
#define RE_SUPPORT_PROFILING				FALSE

#ifdef AUTO_REBUILD
#define RE_SUPPORT_AUTO_RECONSTRUCT			TRUE
#else
#define RE_SUPPORT_AUTO_RECONSTRUCT			FALSE
#endif

#define	RE_SUPPORT_CROSS_CHANNEL_RAID		FALSE	/* Can members of a RAID sit on different SCSI Bus? */
#define RE_SUPPORT_BUILD_AT_CREATE			FALSE
#define RE_SUPPORT_RAID_VOLUMES				FALSE
#define RE_SUPPORT_DEVICE_CLEAR				FALSE	/* Define device capabilities. Defined as a bit map in Application */
#define RE_SUPPORT_ZERO_INIT_OS_ARRAY		FALSE   /* Zero the array with a partition  */

/*
	MaxLunPerID is now equated to RE_MAX_LUNS_PER_TARGET which is what
	is exported in the capability modepage.
*/
#define	RE_MAX_BUS_PER_SYSTEM				8	/* Specified by marketing. Limited only by NT's resources. */
#define MAX_CHANNEL_PER_PCI_FUNCTION        1

#define MAX_PER_BUS                         CDA_MAX_DEVICES_PER_ADAPTER

/*
//Tarun: It is defined in wincfg file now: Not used anymore
#define         MAX_LUN_PER_ID_SUPPORTED		32	// Fixed for SCSI 
#define         MIN_LUN_SCANNED                  1
#define         DEF_LUN_SCANNED                  8
#define         MAX_LUN_SCANNED                  MAX_LUN_PER_ID_SUPPORTED
*/
#define MAX_DEV_MEMBERS                     16  /* max members per array - used for structure sizing */
#define RE_MAX_RAID_PER_DRIVE				CDA_MAX_RAIDS_PER_DISK	/* How many RAIDS a disk participates in? */
#define RE_MAX_MEMBERS_PER_RAID				CDA_MAX_DISKS_PER_RAID

#if		defined (SUPPORT_DEDICATED_SPARE)
#define RE_MAX_SPARES_PER_RAID				1	/* Specified by marketing */
#else
#define RE_MAX_SPARES_PER_RAID				0	
#endif

#if		defined (SUPPORT_GLOBAL_SPARE)

#ifdef CDA_FOR_SATA
#define	RE_MAX_SPARES_IN_POOL				1	/* We will support one since there are only a few ports */
#else
#define	RE_MAX_SPARES_IN_POOL				2	/* No pool spares support */
#endif //CDA_FOR_SATA

#else
#define	RE_MAX_SPARES_IN_POOL				0
#endif

#define	RE_MAX_MEMBERS_PER_RAID0			RE_MAX_MEMBERS_PER_RAID
#define	RE_MAX_MEMBERS_PER_RAID1			2
#define	RE_MAX_MEMBERS_PER_RAID3			0
#define	RE_MAX_MEMBERS_PER_RAID4			0
#define	RE_MAX_MEMBERS_PER_RAID5			0
#define	RE_MAX_MEMBERS_PER_RAID10			RE_MAX_MEMBERS_PER_RAID1+RE_MAX_MEMBERS_PER_RAID1
#define	RE_MAX_MEMBERS_PER_RAID50			0
#define	RE_MaxSimpleVolume					7

#define	RE_MIN_MEMBERS_PER_RAID0			2
#define	RE_MIN_MEMBERS_PER_RAID1			2
#define	RE_MIN_MEMBERS_PER_RAID3			0
#define	RE_MIN_MEMBERS_PER_RAID4			0
#define	RE_MIN_MEMBERS_PER_RAID5			0
#define	RE_MIN_MEMBERS_PER_RAID10			4
#define	RE_MIN_MEMBERS_PER_RAID50			0

#define	RE_MaxRAID0							CDA_MAX_RAIDS_PER_ADAPTER
#define	RE_MaxRAID1							CDA_MAX_RAIDS_PER_ADAPTER
#define	RE_MaxRAID3							0
#define	RE_MaxRAID4							0
#define	RE_MaxRAID5							0
#define	RE_MaxRAID10						RE_MAX_MEMBERS_PER_RAID / RE_MIN_MEMBERS_PER_RAID10
#define	RE_MaxRAID50						0

#define	RE_MAX_RAIDS_PER_BUS				CDA_MAX_RAIDS_PER_ADAPTER
#define	RE_MAX_RAIDS_PER_SYSTEM				RE_MAX_RAIDS_PER_BUS * RE_MAX_BUS_PER_SYSTEM
//#define MAX_RAID_BUFFERS_FOR_BG_TASKS		RE_MAX_RAIDS_PER_BUS  /* How many simulatenous background tasks can run */
#define MAX_RAID_BUFFERS_FOR_BG_TASKS		RE_MAX_RAIDS_PER_BUS * 2 /* How many simulatenous background tasks can run */

#define RE_MAX_CONFIGURED_SINGLE_DRIVES		7  /* Max Number of Configured Single Drives that our MetaData resources can support */

/*
	Physical Bus (Back end) on which devices are connected.
*/
#define PHYS_SCSI2		1
#define PHYS_SSA		2
#define PHYS_FC			3
#define PHYS_P1394		4
#define PHYS_SAS		5
#define PHYS_ATA		6
#define PHYS_SATA		7
#define PHYS_SAS_ON_SATA 8


/* Defines for SAS speed */
//#define SAS_SPEED_15GBPS_VALUE 15360
//#define SAS_SPEED_30GBPS_VALUE 30720
/* TS - Define modified to correct the RATE CHECK bug DDTS ISSUE ADPmp68645 */
/* Defines for SAS speed */
#define SAS_SPEED_15GBPS_VALUE 15000
#define SAS_SPEED_30GBPS_VALUE 30000

/*********************************************************************
	SCSI Device Types
	These are defined by the SCSI spec, except DEVICETYPESPARE.

	When used in the SCSIUNIT.Type field, DEVICETYPESPARE denotes
	a spare device, and DEVICETYPEDISK denotes a physical drive, which
	may be an array or a single.
/*********************************************************************/
#define DEVICETYPEDISK			0
#define DEVICETYPETAPE			1
#define DEVICETYPEPRINTER		2
#define DEVICETYPEPROCESSOR		3
#define DEVICETYPEWORM			4
#define DEVICETYPECDROM			5
#define DEVICETYPESCANNER		6
#define DEVICETYPEOPTICAL		7
#define DEVICETYPECHANGER		8
#define DEVICETYPECOMM			9
#define DEVICETYPEUNKNOWN		0x1F
#define DEVICETYPESPARE			0x80
#define	FAKE					0x40
/*********************************************************************
	Define the supported types of redundancy that can be used for the
	redundacy page of some array.

	The RTYPE_SPARE does not represent any redundancy. A spare device can
	be automatically kicked in to replace a bad/faulty device.
/*********************************************************************/
#define RTYPE_NONE				0	/* Used for RAID0 and singles */
#define RTYPE_COPY				1	/* Used for RAID1 and RAID10 */
#define RTYPE_XOR				2	/* Used for RAID5 */
#define RTYPE_PANDQ				3	/* Historical */
#define RTYPE_SPARE				4	/* For the spare disk */
/*
	Max number of members per RAID and Max number of devices controlled
	by this instance, are two different constants, the former has to be
	always smaller of the two.

	Since we have two separate #defines, make sure nobody makes mistakes
*/
#define	MAXARRAYMEMBERS		RE_MAX_MEMBERS_PER_RAID    // set maximun to 6 devices from R0

#if (MAXARRAYMEMBERS > MAX_DEV_MEMBERS)

	#error MAXARRAYMEMBERS MUST be <= MAX_DEV_MEMBERS 

#endif

/*
	Instead of changing all references, map these definitions to what
	OSMs would provide us (in OSMCDA.H).
*/


#define INBYTE(port)				XINBYTE(port)
#define OUTBYTE(port,value)			XOUTBYTE(port,value)
#define INWORD(port)				XINWORD(port)
#define OUTWORD(port,value)			XOUTWORD(port,value)
#define INDWORD(port)				XINDWORD(port)
#define OUTDWORD(port,value)		XOUTDWORD(port,value)

#include "chim.h"


/*
	HIMQueueIOB: transport-specific, if OSM wishes to include it

	Temporary place. This may go away altogether.
*/
typedef struct HIM_TS_SCSI_ {
    HIM_UEXACT8 protocolStatus;
    HIM_BOOLEAN forceUntagged;
    HIM_BOOLEAN disallowDisconnect;
    HIM_BOOLEAN forceSync;
    HIM_BOOLEAN forceAsync;
    HIM_BOOLEAN forceWide;
    HIM_BOOLEAN forceNarrow;
    HIM_BOOLEAN forceReqSenseNego;
    HIM_BOOLEAN parityEnable;
    HIM_UEXACT8 scsiID;
    HIM_UEXACT8 LUN[8];
    HIM_BOOLEAN suppressNego;      /* When set to HIM_TRUE indicates that 
                                    * negotiation should be suppressed for
                                    * given target.
                                    */

    HIM_BUFFER_DESCRIPTOR dvPattern; /* OSM DV data buffer */   

    HIM_BOOLEAN dvIOB;             /* When set to HIM_TRUE indicates this is a
                                    * domain validation IOB.
                                    */ 
    HIM_UEXACT8 selectionTimeout;  /* the value, in milliseconds, of the
                                      desired selection time-out period. */
} HIM_TS_SCSI;

#define	AP_MaxTargets               AP_NumDevices

#include "cdaptmap.h"
#include "cdaerr.h"
#include "raidtype.h"
#include "address.h"
#include "scsisns.h"
#include "scsicdb.h"
#include "remote.h"
#include "stats.h"
#include "resource.h"
#include "athena.h"
#include "confpage.h"
#include "modepage.h"
#include "ioctl.h"
#include "raiddef.h"
#include "iod.h"
#include "sysdev.h"
#include "physdev.h"
#include "remote.h"
#include "osifprot.h"
#include "coreprot.h"
#include "sharedds.h"

#include "raidglbl.h"
#include "mdtddf.h"
#include "physif.h"

#ifdef SES2_DEVICE
#include "ses2dev.h"
#endif

#include "cdaver.h"

#ifdef CONFIG_HARDCODED_RAID


VOID BuildHardCodedRAIDCdaMetadata(pADAPTER_DEV_EXT pAdapterDevExt, USHORT, USHORT);

#endif

#if	1		/* SAS_BRINGUP - TDE */
#ifdef SCO_DDI8
extern char *VikProductID[];
#else // SCO_DDI8
extern UCHAR *VikProductID[];
#endif // SCO_DDI8
extern enum VikPdctIdE {VPI_13x, VPI_UDMA, VPI_IROC};
#endif /* SAS_BRINGUP */

// For DDF Drive Rooming issue
#define DDF_PDREF_CHANGE		1


/* TDE */
/* CDA TMF OperationFlag Definition */
#define ABORT_TASK			1
#define LUN_RESET_TASK		2
#define DEVICE_PORT_RESET	3
#define PORT_RESET			4	/* not used */

#define INTERNAL_IOCTL_TIMEOUT_IN_SECS 60	/* in seconds */

#define DRIVER_STATE_UP_AND_RUNNING     0x01
#define DRIVER_STATE_FINDADAPTER        0x02
#define DRIVER_STATE_HW_INITIALIZE      0x04         
#define DRIVER_STATE_STOP_ADAPTER       0x08
#define DRIVER_STATE_RESTART_ADAPTER    0x10

#define MAX_28_BIT_LBA		0x0FFFFFFF

#endif
