/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename:sysdev.h

File Description
	Define those structures used for system device manipulation.
***********************************************************************

History :

Oct 18, 2002: Madhuresh
LockNextRange: When background op like verify or rebuild enters a new range,
we should lock that range for the IOs. Whenever this happens
all such IODs are flagged and current RD is kept in SD for when
IOPull happens, it examins and tries to lock the range again. SD now has
a new field defined here.

05/22/2000 : 17:50 BUR11729 Farrukh
ADAPTER_DEV_EXT and  DPTR pADAPTER_DEV_EXT is Forward declared in 
sysdev.h and remote.h since pAdapterDevExt member declaration is 
now changed from PVOID to pADAPTER_DEV_EXT for SysDevice and RemoteCtlPkt.

05/15/2002:  13:54 BUR11729 Farrukh
Move out GlobalList structure from here to raidglbl.h

pSDGlblList has been moved from SysDevice to ADAPTER_DEV_EXT in an attempt
to make all global stored in one place.

For Multi-controller support now we also hold the pointer to ADAPTER_DEV_EXT
in SysDevice

12/07/2001 : 11:39  NAR20038  Manjunath
Removed JAGUAR lines.

05/14/2001 : 19:11  NAR20038  Manjunath
Added buffer pointers in SysDevice for array ops. Added 
ArrayOpAbortForShutdown flag. Added UpdatingCheckPoint in global list.

04/13/2001 : 11:58  NAR20038  Manjunath
Added check point counter in SysDevice.

03/22/2001 : 11:08 AM   NAR20038  Manjunath
Fixed compile time warning messages.

12/13/2000 : 11:08 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

***********************************************************************/

#ifndef SYSDEV_INCLUDED
#define SYSDEV_INCLUDED

#include "sysdef.h"


/***********************************************************************************
	Used during PAC handling and Multiple resets handling
***********************************************************************************/
#define PAC_IN_PROGRESS	1	
#define	PAC_ABORTED		2

/***********************************************************************************
	This is embedded into SD to point to functions. Functions are specific
	to the type of SD it is.
***********************************************************************************/
typedef struct		_SysEntryPoints 	/* structure size is 24 bytes */
{
	FPTR   		   	SysDevRead;
	FPTR			SysDevWrite;
	FPTR			SysDevIOCtl;
	FPTR			SysDevRecovery;
	FPTR			SysDevErrHandle;
	FPTR			SysDevMgt;

} SysEntryPoints;
/***********************************************************************************
	Though containers are not supported, the following structure points to
	start and end LBA on physical disk which comprises a container that participates
	in a RAID pointed by this SD.
***********************************************************************************/
typedef struct _SysBounds 				/* structure size is 8 bytes */
{
	ULONG           SB_LowerBound;
	ULONG           SB_UpperBound;

} SysBounds;
/***********************************************************************************
	Verify and Rebuild share a common buffer, kept two per RAID in the system.
	In the past the following structure was embedded into each SD. Now moved to
	Global space (see raidglbl.h).
***********************************************************************************/
typedef struct _RAIDBuffer
{
	UCHAR DPTR      pStripeBase;
	UCHAR DPTR      pXORBase;
	USHORT          InUse;

} RAIDBuffer ;

typedef struct _SenseBuffer
{
	UCHAR DPTR      virtualAddress;
	DATA_N_PTR      physicalAddress;

} SenseBuffer ;

/* Definition of SysHotPlugState */
#define HOTPLUG_REMOVE_DEVICE           0x0001
#define HOTPLUG_METADATA_UPDATE_NEEDED 0x0002


/***********************************************************************************
	S Y S      D E V I C E

	SD is created for each device that needs to be accessed by OS or by the 
	Management Application.
***********************************************************************************/
typedef	struct	_SysDevice
{
	pSysDevice      pNextSysDevice;
	LDTree          SysDeviceMe;
	UCHAR           SysDeviceAccess;/* LSINGLE, LRAID0, etc. */
	UCHAR           SysDeviceType; 	/* Disk, tape, etc. */

	UCHAR			cDevMembers;
	UCHAR			TMFBusyLogicCount;
	ULONG           nBlocks;		/* Reported capacity */


	ULONG           StripeFactor;
	ULONG           StripeShift;
	ULONG           StripeMask;
	ULONG           StripeRoll;		/* cMembers * (cMembers-1)	*/

	USHORT          SysKnownToPTTable; /* Zero if not reported yet */
	USHORT          SysReportedBus;
	USHORT          SysReportedId;
	USHORT          SysReportedLUN;

	ULONG           SysUniqueId;	/* Unique Id for the device	*/
	ULONG           SysTimestamp;
	USHORT          SysPhysDriveIndex;

	USHORT          ArrayState;
	USHORT          ArraySubState;
	USHORT          ArrayOpActive;
	USHORT          ArrayOpBlocked;
	USHORT          ArrayManagementPriority;
	ULONG           ArrayOpStart;
	ULONG           ArrayOpEnd;
	ULONG           ArrayOpCheckPoint;
	BOOL            ArrayAbortOp;

	pSysDevice      pRelationalSD;	/* points parent or first child for dual level RAID */

	/* SysIsGhost is a ghost reference to the sysdevice. If non zero */
	/* Ghosts are not referenced to the IOManager.*/
	/* We need to keep the ghost reference if Sys device is known to the OS. */

	UCHAR           SysIsGhost;
	UCHAR           SysKeepGhostReference;
	USHORT          SysKnownToOS;

	/*
		Meta data access related fields. Each RAID SD has an attached MD buffer to it.
		But newly created RAID may not have this set.

		cSysRSU_PassCount is used during recovery if one of the member
		failed and we update the count before we go and update the reserved
		sector. Therefore the non zero value of this count would prevent us
		to do simultaneous update of the same array members, if another
		member goes down while we are in the process of updating.
	*/
	pIODescriptor           pSysRSU_IOD;
	USHORT                  cSysRSU_PassCount;
	USHORT                  iSysRSU_Member;
	//USHORT					RSU_Active;
	//USHORT					ArrayAbortOpForShutdown; /* Converted to UINT8*/
    UINT8                   ArrayAbortOpForShutdown;
    UINT8                   ArryAbortOpForShutdownComplete;/*Added Work Around for issue ADPmp89330*/
	DATA_N_PTR              RSBufPhysAddr;
	pReservedSectorLayout   pSysRSBuf;

	USHORT                  iSysStartUnit_Member;      /* tracking members during 
                                                          Starting/Stoping drives */
	pControllerResourceDescriptor	pSysResources;

	SysBounds               DevBounds  [MAXARRAYMEMBERS] ;
	PVOID                   DevMembers [MAXARRAYMEMBERS] ;
	SysEntryPoints          SysFunc;
	LogicalRedundancyPage   SysRedundancy;


	/* Points to next Adapter SysDevice waiting for PAC to be sent */
	pSysDevice				pNextPACPendingDevice;
	pADAPTER_DEV_EXT		pAdapterDevExt;

	/*
		When verify or rebuild cannot enter a new range, as a write is pending in that range
		their RD is hooked in here.
	*/
	pRecoveryDescriptor		pHoldVerifyRebuild ;

#if PROFILER
	SysDeviceStats			SysStats;
#endif
    //USHORT                  cMemberReset;     /* not used */

    USHORT                  SysHotPlugState;      

    /******** Following fields added for AutoRebuild Operation ********/
    UCHAR                   RebuildFailedMemberIndex; /* Used for auto rebuild and HSR to save failed member index */

    UCHAR                   RebuildType;        /* Indicates auto rebuild or HSR rebuild */
#define TYPE_AUTO_REBUILD   0x01
#define TYPE_HSR_REBUILD    0x02
    pSysDevice              pRebuildSD;         /* Auto Rebuild operation saves the SD on which */
                                                /* rebuild should start after old member and spare */
                                                /* member PDE entries get cleaned up by the MDT layer */	
    UCHAR                   SpareInUse;          /* Flag is set when a valid spare is allocated for auto
                                                   * auto rebuilding.
                                                   */

    UCHAR                   CheckpointActivity; /* save the activity operation in case */
                                                /* of dirty shutdown and we need to do */
                                                /* auto verify after system is booted */
                                                /* cannot save this operation in Sysdevice */
                                                /* local metadata buffer. It will can wipped out */
                                                /* when we read the metadata at the end of Initialization */
                                                /* during call to LocalUpdateAll since the metadata */
                                                /* on the disk doesn't have this operation bit set. */
    
	UCHAR					ArrayOpActivityStart;
    
    pRecoveryDescriptor     pRD;
    
	UCHAR					ForeignArray;		/* 
												 * To indicate if this a foreign array
												 * (array that we do not know of during 
												 *  init time).
												 */
    UCHAR                  RSUpdateNeeded;
    UCHAR                  IrocFlags;            /* save/restore cache settings,verify with
                                                  * auto fix flags.
                                                  */
#define RS_CHECKPOINT_UPDATE_PENDING    0x01
#define RS_LOCALMEMBER_UPDATE_PENDING    0x02
    UINT8                  HZ1Array;
    ULONG		   InitArrayOpCheckpointLBA;
} SysDevice ;


/* High Byte Definition internal to CDA and	bitmapped */

#define CDA_ARRAY_SUBSTATE_HSR_START	    		0x0100 /* Indicates that HSR should start on this SysDevice */

#define CDA_ARRAY_SUBSTATE_RESOURCE_LIMIT 			0x0200 /* resource allocation limit*/
#define CDA_ARRAY_SUBSTATE_RESOURCE_CHECK 			0x0400 /* a temporary check mark */

#define CDA_ARRAY_SUBSTATE_RECOVER_RESERVED_SECTOR  	0x0800
#define CDA_ARRAY_SUBSTATE_METADATA_UPDATE_PENDING	0x1000 /* Auto rebuild/HSR rebuild is pending on reserved sector
															* update to be completed.
													    	*/


#endif
