/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/*
 * HostRaid Linux OSM Implementation
 *
 * $Id$
 */


#include "hr.h"
#include <linux/notifier.h>
#include <linux/reboot.h>
#include <linux/major.h>
#include <linux/blkdev.h>
#ifdef __x86_64__
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#include <asm/ioctl32.h>
#else
#include <linux/syscalls.h>
#endif
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,4)
#include <linux/buffer_head.h>
#endif

/********************************** Extern Func *******************************/
#ifdef CONFIG_COMPAT 
#if	!(HAVE_COMPAT_IOCTL)
extern int 	register_ioctl32_conversion(unsigned int cmd,
			int(*handler)(unsigned int, unsigned int, 
			unsigned long, struct file *));
extern int 	unregister_ioctl32_conversion(unsigned int cmd);
#endif
#endif
extern void	XMemFreeAllContiguous(PVOID pADExt);
extern void *	XMemAllocContiguous(PVOID pADE, ULONG size);
extern void	XMemFree(uint8_t DPTR pBuf, ULONG nBytes);
/********************************** Constants *********************************/
uint32_t	HR_WATCHDOG_PERIOD = 1;

/************************ Local Function Declarations *************************/
void PostOsmTMF(POSD pOSD);
static int	hr_platform_alloc(struct hr_softc *hr);
static int	hr_watchdog_thread(void *data);
static void	hr_kill_watchdog_thread(struct hr_softc *hr);
static void	hr_destroy_device(void *arg);
static void	hr_add_device(void *arg);
static void 	hr_get_sd_devname(int disknum, char *buffer);
static int 	hr_get_sd_devnum(Scsi_Cmnd *cmd);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
static void 	hr_initiate_kernel_rescan(struct hr_softc *hr,
			 int ch, int id, int lun, int to_add);
#endif

/************************* Error Recovery Functions ***************************/
static int	hr_ehandler_thread(void *data);
static void	hr_kill_ehandler_thread(struct hr_softc *asd);
static 		hr_eh_post_t hr_handle_tmf_req_done;
static void	hr_recover_cmds(struct hr_softc *hr);
static void	hr_abort_ioreq(struct hr_softc *hr,
			struct hr_io_request *ioreq_to_abort,
			struct hr_io_request *ioreq);
static void	hr_reset_lu(struct hr_softc *hr,
			    struct hr_io_request *ioreq_to_reset,
			    struct hr_io_request *ioreq);
static void	hr_reset_device(struct hr_softc *hr,
			struct hr_io_request *ioreq_to_reset,
			struct hr_io_request *ioreq);

/****************************** Helper Functions ******************************/
static void	hr_sleep(struct semaphore *sleep_sem, u_int interval);
static void	hr_sleep_timeout(u_long arg);
static void	hr_setup_dev_dpc_task(struct hr_linux_device *dev,
			      void (*func)(void *));
static int	hr_find_ioreq_in_busyq(struct hr_softc *hr,
			       struct hr_io_request *ioreq);
static int	hr_find_dev_in_devtable(struct hr_softc *hr,
			struct hr_linux_device *dev, u_int *ch,
			 u_int *id, u_int *lun);
static void hr_ioreq_add_timer(struct hr_io_request *ioreq, int timeout);
static int hr_ioreq_delete_timer(struct hr_io_request *ioreq);

#ifdef CONFIG_SMP
#define HR_LOCK_ASSERT(asd) \
	ASSERT(spin_is_locked(&hr->spin_lock))
#else
#define HR_LOCK_ASSERT(hr) 
#endif 

/******************************* Initialization *******************************/
static int		 hr_init(void) __init;
static void		 hr_exit(void) __exit;
static int		 hr_pci_init(void);
static void		 hr_pci_exit(void);
static int		 hr_config(struct hr_softc *hr);
static int		 hr_map_io_handle(struct hr_softc *hr);
static int		 hr_discover_device(struct hr_softc *hr);
static void		 hr_set_name(struct hr_softc *hr, char *name);
static struct hr_softc	*hr_find_softc(struct hr_softc *hr);
static struct hr_softc	*hr_alloc(char *instance_name, hr_dev_softc_t dev);
static void		 hr_free(struct hr_softc *hr);
static int		 hr_next_unit(void);
static int		 hr_register_host(struct hr_softc *hr);
static __inline struct hr_linux_device * hr_get_ldev(struct hr_softc *hr,
				 Scsi_Cmnd *cmd);

static __inline struct hr_linux_device *
hr_get_ldev(struct hr_softc *hr, Scsi_Cmnd *cmd)
{
	struct hr_linux_device *ldev;
	Scsi_Device		*dev;

	dev = cmd->device;
	if ((dev->channel > OSM_MAX_CHANNEL) || 
	    (dev->id > OSM_MAX_TARGET_PER_PATH) ||
	    (dev->lun > OSM_MAX_LUN))
		return (NULL);

	ldev = (struct hr_linux_device *)
			hr->device_table[dev->channel][dev->id][dev->lun];
	if (ldev == NULL)
	return (NULL);

	return (((ldev->flags & HR_DEV_ONLINE) != 0) ? ldev : NULL);
}

/****************************** PCI Entry Points ******************************/
static int	hr_pci_dev_probe(struct pci_dev *pdev,
				 const struct pci_device_id *ent);
static void	hr_pci_dev_remove(struct pci_dev *pdev);

/********************************** Shutdown **********************************/
static int	hr_reboot_notifier(struct notifier_block *block, ulong code,
				   void *arg);
static void	hr_shutdown(struct hr_softc *hr, struct hr_shutdown_info *);
static void	hr_shutdown_complete(struct hr_shutdown_info *sinfo);

/******************************* Queue Handling *******************************/
static inline struct hr_linux_device *
		     	hr_next_device_to_run(struct hr_softc *hr);
static inline void 	hr_check_device_queue(struct hr_softc *hr,
					      struct hr_linux_device *dev);
static inline void 	hr_run_device_queues(struct hr_softc *hr);
static void   hr_run_device_queue(struct hr_softc *hr,
				  struct hr_linux_device *dev);
static inline void	hr_schedule_runq(struct hr_softc *hr);
static inline void 	hr_schedule_unblock(struct hr_softc *hr);
static void     	hr_runq_tasklet(u_long data);
static void     	hr_unblock_tasklet(u_long data);
static void		    hr_flush_commands(struct hr_linux_device  *dev);

static inline struct hr_linux_device *
hr_next_device_to_run(struct hr_softc *hr)
{
	if ((hr->qfrozen != 0) || (hr->flags & HR_SUSPEND_IO))
		return (NULL);
	return (TAILQ_FIRST(&hr->device_runq));
}

static inline void
hr_check_device_queue(struct hr_softc *hr, struct hr_linux_device *dev)
{
	if ((TAILQ_FIRST(&dev->busyq) == NULL) || (dev->qfrozen != 0) ||
		(hr->flags & HR_SUSPEND_IO))
		return;

	hr_run_device_queue(hr, dev);
}

static inline void
hr_run_device_queues(struct hr_softc *hr)
{
	struct hr_linux_device *dev;

	while ((dev = hr_next_device_to_run(hr)) != NULL) {
		TAILQ_REMOVE(&hr->device_runq, dev, links);
		dev->flags &= ~HR_DEV_ON_RUN_LIST;
		hr_check_device_queue(hr, dev);
	}
}

/*
 * Must be called with our lock held.
 */
static inline void
hr_schedule_runq(struct hr_softc *hr)
{
	tasklet_schedule(&hr->platform_data->runq_tasklet);
}

static inline void
hr_schedule_unblock(struct hr_softc *hr)
{
	tasklet_schedule(&hr->platform_data->unblock_tasklet);
}

/*
 * This routine is called from a tasklet, so we must re-acquire
 * our lock prior when accessing data-structures that need protection.
 */
static void
hr_runq_tasklet(u_long data)
{
	struct hr_softc		*hr;
	struct hr_linux_device	*dev;
	u_long			 flags;

	hr = (struct hr_softc *) data;
	hr_lock(hr, &flags);
	while ((dev = hr_next_device_to_run(hr)) != NULL) {
		TAILQ_REMOVE(&hr->device_runq, dev, links);
		dev->flags &= ~ HR_DEV_ON_RUN_LIST;
		hr_check_device_queue(hr, dev);
		/* Yield to our interrupt handler. */
		hr_unlock(hr, &flags);
		hr_lock(hr, &flags);
	}
	hr_unlock(hr, &flags);
}

static void
hr_unblock_tasklet(u_long data)
{
	struct hr_softc	*hr;

	hr = (struct hr_softc *) data;
	scsi_unblock_requests(hr->host);
}

/******************************* Tasklet Support ******************************/
static inline void
hr_setup_tasklets(struct hr_softc *hr)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	tasklet_init(&hr->platform_data->runq_tasklet, hr_runq_tasklet,
		     (u_long) hr);
	tasklet_init(&hr->platform_data->unblock_tasklet, hr_unblock_tasklet,
		     (u_long) hr);
#endif
}

static inline void
hr_teardown_tasklets(struct hr_softc *hr)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	tasklet_kill(&hr->platform_data->runq_tasklet);
	tasklet_kill(&hr->platform_data->unblock_tasklet);
#endif
}

/*************************** Transaction Processing ***************************/
static void   hr_PostOSD(POSD pOSD);
static void   XPostApiIOCtlDonex(POSD pOSD);
static void		hr_ioreq_timeout(Scsi_Cmnd *cmd);
static void   hr_scsi_requeue(struct hr_softc *hr, Scsi_Cmnd *);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static void	hr_scsi_offline(Scsi_Cmnd *);
#endif
static int	hr_map_data(struct hr_softc *hr, POSD pOSD, Scsi_Cmnd *cmd);
static inline void
		hr_unmap_data(struct hr_softc *hr, POSD pOSD, Scsi_Cmnd *cmd);
static inline struct hr_io_request *
		hr_get_ioreq(struct hr_softc *hr);
static inline void	hr_free_ioreq(struct hr_softc *hr,
				      struct hr_io_request *ioreq);
static inline void hr_freeze_hostq(struct hr_softc *hr);
static void hr_release_hostq(struct hr_softc *hr);
static inline void	hr_start_ioreq_timer(struct hr_io_request *ioreq);

/*
 * Release mapped OS specified data.
 */
static inline void
hr_unmap_data(struct hr_softc *hr, POSD pOSD, struct scsi_cmnd *cmd)
{
	if (cmd->use_sg != 0) {
		struct scatterlist *sg;

		sg = (struct scatterlist *)cmd->request_buffer;
		hr_unmap_sg(hr, sg, cmd->use_sg,
			    scsi_to_pci_dma_dir(cmd->sc_data_direction));
	} else if (cmd->request_bufflen != 0) {
		hr_unmap_single(hr,
				((SGTable *)
				pOSD->OSD_pSGTable)->SGList[0].SGLBufAddr,
				cmd->request_bufflen,
				scsi_to_pci_dma_dir(cmd->sc_data_direction));
        }
}

static inline struct hr_io_request *
hr_get_ioreq(struct hr_softc *hr)
{
	struct hr_io_request	*ioreq;

	HR_LOCK_ASSERT(hr);

	if ((ioreq = TAILQ_FIRST(&hr->io_request_freeq)) == NULL) {
		hr_log(HR_RUNTIME_DEBUG, 1, "Running out of free io req.\n");
		return (NULL);
	}
	TAILQ_REMOVE(&hr->io_request_freeq, ioreq, links);
	memset(ioreq,0,sizeof(*ioreq));
	return (ioreq);
}

static inline void
hr_free_ioreq(struct hr_softc *hr, struct hr_io_request *ioreq)
{
	ioreq->flags = 0;
	TAILQ_INSERT_TAIL(&hr->io_request_freeq, ioreq, links); 

	if ((hr->flags & HR_RESOURCE_SHORTAGE) != 0) {
		hr->flags &= ~HR_RESOURCE_SHORTAGE;
		hr_release_hostq(hr);
	}
}

static inline void
hr_freeze_hostq(struct hr_softc *hr)
{
	hr->qfrozen++;
	if (hr->qfrozen == 1)
		scsi_block_requests(hr->host);
}

/* This needs to be called with LOCK being held. */
static void
hr_release_hostq(struct hr_softc *hr)
{
	HR_LOCK_ASSERT(hr);

	if (hr->qfrozen > 0)
		hr->qfrozen--;
	if (hr->qfrozen == 0) {
		hr_schedule_unblock(hr);
		hr_schedule_runq(hr);
	}
}

static inline void
hr_start_ioreq_timer(struct hr_io_request *ioreq)
{
	struct	hr_softc	*hr;
	struct	scsi_cmnd 	*cmd;
	int timeout;

	hr = ioreq->pAdapter;
	cmd = &hcmd_scsi_cmd(ioreq->hcmd);

	ioreq->flags |= HR_IOREQ_TIMER_ACTIVE;

	if(ioreq->dev->incr_timeout & LUN_RESET_TASK)
		timeout = cmd->timeout_per_command + (200 * HZ);
	else {
		timeout = cmd->timeout_per_command;
	}

	hr_ioreq_add_timer( ioreq, timeout);
}

/**************************** Interrupt Processing ****************************/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
irqreturn_t 	
#else
void
#endif
hr_isr(int vec, void *arg, struct pt_regs *regs);

/******************************* Ioctl Interface ******************************/
static int	hr_open(struct inode *inode, struct file *file);
static int	hr_close(struct inode *inode, struct file *file);
static int	hr_ioctl(struct inode *inode, struct file *file,
                       u_int cmd, u_long arg);
#ifdef CONFIG_COMPAT
#if	HAVE_COMPAT_IOCTL
static int	hr_compat_ioctl(struct file *file, u_int cmd,
			 u_long arg);
#endif
#endif

#define HR_IOCTL_PAUSE_LIMIT	24
#define PCI_SUBSYSTEM_VENDORID_OFFSET 0x2C
#define PCI_SUBSYSTEM_DEVICEID_OFFSET 0x2E

/************************* Host Template Entry Points *************************/
static int		
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
hr_proc_info(char * /*proc_buffer*/, char** /*proc_start*/,
	     off_t /*proc_offset*/, int /*proc_length*/, int /*proc_host*/,
				     int /*proc_out*/);
#else
hr_proc_info(struct Scsi_Host *, char * /*proc_start*/, char ** /*proc_offset*/,
	     off_t /*proc_length*/, int /*proc_host*/, int /*proc_out*/);
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static int		hr_detect(Scsi_Host_Template *);
#else
int		hr_detect(Scsi_Host_Template *);
#endif
static const char *	hr_info(struct Scsi_Host * /*hba*/);
static int		hr_queuecommand(Scsi_Cmnd * /*cmnd*/,
				        void (*)(Scsi_Cmnd *) /*done routine*/);
static int		hr_abort(Scsi_Cmnd *);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static int		hr_bios_param(Disk *, kdev_t, int[]);
#else
static int		hr_bios_param(struct scsi_device *,
				      struct block_device *,
				      sector_t, int[]);
#endif

#if DEBUG_BUILD
u_int	DEBUG_LEVEL;
#endif

#ifdef OSM_DBGMSG_SAS_DUMP
u_int	OSMDebugMode = 0xFFFFFFFFUL;
#endif /* OSM_DBGMSG_SAS_DUMP */

/* Redefinition for SCSI definition of SUCCESS and FAILED. */
#ifndef EH_SUCCESS
#define EH_SUCCESS	0x2002
#endif
#ifndef EH_FAILED
#define EH_FAILED	0x2003
#endif

/** File version no. **/
#define ADPT_MAJOR_REV          1
#define ADPT_MINOR_REV 		4
#define ADPT_OSM_REV 		11662
#define ADPT_CHIM_REV 		0

#if (!defined(HR_DRIVER_VERSION))
# define HR_DRIVER_VERSION    STR(ADPT_MAJOR_REV)
#endif
#if (!defined(HR_DRIVER_REVISION))
# define HR_DRIVER_REVISION    STR(ADPT_MINOR_REV)
#endif
#if (!defined(HR_DRIVER_SUBREVISION))
# define HR_DRIVER_SUBREVISION 		 STR(ADPT_OSM_REV)
#endif
#if (!defined(HR_DRIVER_VARIANT))
  /* Add a leading '-' if set none-null please */
# define HR_DRIVER_VARIANT     ""
#endif
#if (!defined(HR_DRIVER_CHIMVERSION))
# define HR_DRIVER_CHIMVERSION 		 STR(ADPT_CHIM_REV)
#endif
#ifdef CDA_FOR_SATA
#define	DRIVERDESC \
"Adaptec " HR_DRIVER_NAME " SATA HostRAID driver v" HR_DRIVER_VERSION"." \
HR_DRIVER_REVISION"."HR_DRIVER_SUBREVISION
#else
#define	DRIVERDESC \
"Adaptec " HR_DRIVER_NAME " SAS/SATA HostRAID driver v" HR_DRIVER_VERSION"." \
HR_DRIVER_REVISION"."HR_DRIVER_SUBREVISION
#endif

MODULE_AUTHOR("Adaptec Inc.");
MODULE_DESCRIPTION(DRIVERDESC);

/**************************** Global Datastructures ***************************/
spinlock_t 			hr_dma_mem_spinlock;
static int 			hr_major;
static int 			hr_driver_shutdown_called;
static int 			hr_pci_driver_registered;
static int 			hr_init_status;
static int 			hr_detect_complete;
static int 			hr_user_cmd_per_lun = 0;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static spinlock_t		hr_shutdown_spinlock;
static int 			hr_units;
static int 			hr_driver_unloading = 0;
#else
spinlock_t			hr_shutdown_spinlock;
int				hr_units;
int				hr_driver_unloading = 0;
static int 			hr_user_max_xfer = 0;
#endif
#ifndef scsi_partsize
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
extern int scsi_partsize(unsigned char *buf, unsigned long capacity,
           unsigned int  *cyls, unsigned int *hds, unsigned int *secs);
#else
extern int scsi_partsize(struct buffer_head *bh, unsigned long capacity,
           unsigned int  *cyls, unsigned int *hds, unsigned int *secs);
#endif
#endif
static int 			hr_reboot_notifier_registered = 0;
struct hr_softc_tailq 		hr_tailq = TAILQ_HEAD_INITIALIZER(hr_tailq);
static Scsi_Host_Template 	hr_host_template;
static struct file_operations 	hr_file_operations;
static struct notifier_block  	hr_reboot_notifier_block;
static struct hr_shutdown_info 	hr_notifier_shutdown_info =
						{ &hr_shutdown_spinlock };
/*
 * Watchdog Thread support.
 */	 
static pid_t			hr_watchdog_pid;
static DECLARE_MUTEX_LOCKED(hr_watchdog_sem);
static DECLARE_MUTEX_LOCKED(hr_watchdog_ending_sem);


/******************* PCI Driver Constants/Data Structures *********************/
/*
 * Routine to perform controller specific
 * probe and initialization.
 */
typedef int (hr_device_setup_t)(struct hr_softc *);

/*
 * Stucture pointed to by our Linux PCI table entries.
 * When Linux finds an entry in our table that matches
 * a device in the system, it will pass us the PCI table
 * entry that matches.  Given that, we can easly access
 * the controller specific setup structure, allowing
 * personalized setup.
 */
struct hr_pci_driver_data
{
	/* Controller Description. */
	char		  *description;

	/* Controller Specific Setup Routine. */
	hr_device_setup_t *setup;
};


/*
 * PCI Entry Data for SAS HostRAID controllers.
 */
static hr_device_setup_t hr_sas_setup;
#define	PCI_CLASS_DEFAULT		0
#define	PCI_CLASS_MASK_DEFAULT		0

#define	PCI_CLASS_RAID			(0x0104 << 8)
/*NVIDIA chips*/
#define	PCI_VENDOR_NVDA			0x10DE
#define PCI_DEVICE_NVDA			0x0055

/*INTEL chips*/
#define	PCI_VENDOR_ICH			0x1028
#define	PCI_VENDOR_ICH5			0x8086
#define	PCI_VENDOR_ICH6R		0x8086
#define	PCI_VENDOR_ICH7R		0x8086
#define	PCI_DEVICE_ICHa			0x0173
#define	PCI_DEVICE_ICHb			0x0183
#define	PCI_DEVICE_ICH5			0x24D1
#define	PCI_DEVICE_ICH5R		0x24DF
#define	PCI_DEVICE_ICH5Rx		0x25B0
#define PCI_DEVICE_ICH6R		0x2652
#define PCI_DEVICE_ESB2			0x2682
#define PCI_DEVICE_ICH7R		0x27C3
#define PCI_DEVICE_ICH7A		0x27C1

/*MARVEL chips*/
#define	PCI_VENDOR_MARV			0x11AB
#define	PCI_VENDOR_MARVADP		0x9005
#define	PCI_DEVICE_88SX5041		0x5041
#define	PCI_DEVICE_88SX5081		0x5081
#define	PCI_DEVICE_88SX6041		0x6041
#define	PCI_DEVICE_88SX6081		0x6081
#define	PCI_DEVICE_88SX6541		0x2419
#define	PCI_DEVICE_0241			0x0241
#define PCI_DEVICE_0243			0x0243

/*Adaptec chips*/
#define	PCI_VENDOR_94XX			0x9005
#define	PCI_CLASS_94XX			(0x0107 << 8)
#define	PCI_CLASS_MASK_94XX		0xFFFF00
#define	PCI_DEVICE_SPIRIT		0x0410
#define PCI_DEVICE_AIC9412		0x0412
#define PCI_DEVICE_AIC9415		0x0415
#define PCI_DEVICE_AIC9416		0x0416
#define PCI_DEVICE_AIC9410		0x041F
#define PCI_DEVICE_AIC9410B		0x041E
#define PCI_DEVICE_AICLITE		0x0430
#define PCI_DEVICE_AICLITEB		0x0432
#define PCI_DEVICE_AICLITEEA		0x043F
#define PCI_DEVICE_AICLITEEB		0x043E

/*Vittese chips*/
#define	PCI_VENDOR_725X			0x9005
#define	PCI_CLASS_725X			(0x0107 << 8)
#define	PCI_CLASS_MASK_725X		0xFFFF00
#define	PCI_DEVICE_7250HR		0x042F
#define	PCI_DEVICE_7250NR		0x042E

/*Silicon Image chips*/
#define	PCI_VENDOR_SIL			0x1095
#define	PCI_CLASS_SIL			(0x0107 << 8)
#define	PCI_CLASS_MASK_SIL		0xFFFF00
#define	PCI_DEVICE_SIL3132		0x0242

/*
 * hnguyen: ADPmp92940, now that the Makefile has the driver name in
 * HR_DRIVER_NAME, we don't need the old way of assigning names to each
 * individual PCI device ID
 */

static struct hr_pci_driver_data hr_driver_data = 
{
#ifdef CDA_FOR_SATA
	"Adaptec " HR_DRIVER_NAME " SATA HostRAID Controller",
#else
	"Adaptec " HR_DRIVER_NAME " SAS/SATA HostRAID Controller",
#endif
	hr_sas_setup
};
#define ADP94XX_PCI_ENTRY(devid)				\
{								\
	PCI_VENDOR_94XX, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_DEFAULT,		\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define ADP725X_PCI_ENTRY(devid)				\
{								\
	PCI_VENDOR_725X, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_DEFAULT,		\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define NVDA_PCI_ENTRY(devid)					\
{								\
	PCI_VENDOR_NVDA, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_RAID,			\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define ICH_PCI_ENTRY(devid)					\
{								\
	PCI_VENDOR_ICH, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_RAID,			\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define ICHx_PCI_ENTRY(devid)					\
{								\
	PCI_VENDOR_ICH5, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_RAID,			\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define MARV_PCI_ENTRY(devid)					\
{								\
	PCI_VENDOR_MARV, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_RAID,			\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define MARVADP_PCI_ENTRY(devid)				\
{								\
	PCI_VENDOR_MARVADP, devid,				\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_RAID,			\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}
#define ADPSIL_PCI_ENTRY(devid)					\
{								\
	PCI_VENDOR_SIL, devid,					\
	PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_RAID,			\
	PCI_CLASS_MASK_DEFAULT, (u_long)&hr_driver_data		\
}

static struct pci_device_id hr_pci_id_table[] = {
#ifdef CDA_FOR_SATA
#if defined(CDA_FOR_ADPAHCI)
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH6R),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH7R),
	ICHx_PCI_ENTRY(PCI_DEVICE_ESB2),
#elif defined(CDA_FOR_AAR81XX)
	MARV_PCI_ENTRY(PCI_DEVICE_88SX6041),
	MARV_PCI_ENTRY(PCI_DEVICE_88SX6081),
	MARVADP_PCI_ENTRY(PCI_DEVICE_0241),
	MARVADP_PCI_ENTRY(PCI_DEVICE_0243),
	
#elif defined(CDA_FOR_ADP3132)
	ADPSIL_PCI_ENTRY(PCI_DEVICE_SIL3132),
#else
	NVDA_PCI_ENTRY(PCI_DEVICE_NVDA),
	ICH_PCI_ENTRY(PCI_DEVICE_ICHa),
	ICH_PCI_ENTRY(PCI_DEVICE_ICHb),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH5),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH5R),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH5Rx),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH6R),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH7R),
	ICHx_PCI_ENTRY(PCI_DEVICE_ICH7A),
	ICHx_PCI_ENTRY(PCI_DEVICE_ESB2),
	MARV_PCI_ENTRY(PCI_DEVICE_88SX5041),
	MARV_PCI_ENTRY(PCI_DEVICE_88SX5081),
	MARV_PCI_ENTRY(PCI_DEVICE_88SX6041),
	MARV_PCI_ENTRY(PCI_DEVICE_88SX6081),
	MARVADP_PCI_ENTRY(PCI_DEVICE_88SX6541),
	MARVADP_PCI_ENTRY(PCI_DEVICE_0241),
	MARVADP_PCI_ENTRY(PCI_DEVICE_0243),
#endif
#else
#if defined(CDA_FOR_AURORA)
	ADP94XX_PCI_ENTRY(PCI_DEVICE_7250HR),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_7250NR),
#else
	ADP94XX_PCI_ENTRY(PCI_DEVICE_SPIRIT),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AIC9410),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AIC9410B),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AIC9412),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AIC9415),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AIC9416),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AICLITE),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AICLITEB),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AICLITEEA),
	ADP94XX_PCI_ENTRY(PCI_DEVICE_AICLITEEB),
#endif
#endif
	{0}
};

MODULE_DEVICE_TABLE(pci, hr_pci_id_table);

struct pci_driver hr_pci_driver = {
	.name		= HR_DRIVER_NAME,
	.probe		= hr_pci_dev_probe,
	.remove		= hr_pci_dev_remove,
	.id_table	= hr_pci_id_table
};
static int hr_pci_driver_registered;

/********************************* Queue Depth ********************************/
#define HR_CMD_PER_LUN_UNTAGGED	2

/******************************* Option Parsing *******************************/
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#ifndef MODULE
static int hr_setup(char *options);

/*
 * When statically compiled into the kernel, our "setup" routine will
 * be called to parse any command line arguments that are prefixed by our
 * driver string and "=".  We only support a single integer parameter
 * that sets our per-lun maximum transaction count.
 *
 * When loaded as a module, hr_user_cmd_per_lun will be initialized by
 * insmod as the driver is loaded.
 */
static int
hr_setup(char *options)
{
	get_options(options, 1, &hr_user_cmd_per_lun);
	return (0);
}
__setup("hr_user_cmd_per_lun=", hr_setup);
#endif
#endif
/******************** Controller Specific Initialization **********************/
static int
hr_sas_setup(struct hr_softc *hr)
{
	/* XXX Get from the CHIM? */
	hr->adapter_scsi_id = 255;
	hr->max_scsi_id = OSM_MAX_TARGET_PER_PATH;
	hr->max_lun = OSM_MAX_LUN;

	hr->max_cmds_per_lun = 128;

	/*
	 * CDA limits this to 256?  The controller should
	 * be able to handle 512.
	 */
#ifdef CDA_FOR_SATA
	hr->can_queue = 16;
#else
	hr->can_queue = HR_MAX_QUEUE;
	hr->dma64_addr = 1;
#endif

	/*
	 * 16MB intrinsic limit in the hardware, but 4MB seems
	 * to offer good performance.
	 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	hr->max_sectors = 128; 
#else
	/*
	 * hr_user_max_xfer should be in k bytes.
	 * nKbytes / 2 = number of blocks. 
	 */
	if(hr_user_max_xfer < 1)
	hr->max_sectors = 128; 
	else
		hr->max_sectors = hr_user_max_xfer * 2;
#endif
	printk("%s max xfer %dK\n", HR_DRIVER_NAME, hr->max_sectors / 2);
	hr->hw_name = HR_DRIVER_NAME;
	return (0);
}

/********************************** Shutdown **********************************/
/*
 * System event (shutdown) handler.
 */
static int
hr_reboot_notifier(struct notifier_block *block, u_long code, void *arg)
{
	struct	hr_softc *hr;
	u_long	hr_flags;

	hr_log(HR_BOOT_DEBUG, 1, "hr_reboot_notifier(%p,%lx,%p)\n",
		     block, code, arg);
	if ((code != SYS_RESTART)
	 && (code != SYS_HALT)
	 && (code != SYS_POWER_OFF)) {
		return NOTIFY_DONE;
	}

	/*
	 * Stop accepting new controller hot-plug events.
	 */
	hr_driver_shutdown_called = 1;

	/*
	 * Count the number of controllers we
	 * need to shutdown.  This count tells
	 * the last controller that shuts-down
	 * to wakup this notification handler.
	 */
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		hr_notifier_shutdown_info.shutdown_cnt++;
	}

	if(!hr_notifier_shutdown_info.shutdown_cnt)
		return NOTIFY_DONE;

	/*
	 * Initiate shutdown processing on
	 * all instances.
	 */
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		hr_lock(hr, &hr_flags);
		CdaSetInitializatonFlag(hr->pAdapterDevExt);
		hr_shutdown(hr, &hr_notifier_shutdown_info);
		CdaResetInitializatonFlag(hr->pAdapterDevExt);
		hr_unlock(hr, &hr_flags);
	}

	/*
	 * Wait for the completions...
	 */
	down(&hr_notifier_shutdown_info.shutdown_sem);
	return (NOTIFY_DONE);
}

/*
 * Shutdown (flush cache, sync metadata, etc.) a given
 * controller instance.  Both the hr_list_lock and the
 * hr_lock are held on entry - the hr_list_lock being
 * necessary since we may indicate shutdown completion
 * immediately and those data structures are protected
 * by the hr_list_lock.
 */
static void
hr_shutdown(struct hr_softc *hr, struct hr_shutdown_info *sinfo)
{
	PVOID pADE;
	POSD	pOSD;
	struct hr_io_request	*io_req;

	pADE = hr->pAdapterDevExt;

	io_req = hr_get_ioreq(hr);
	if (io_req == NULL) {
		printk("%s: did not complete shutdown\n",HR_DRIVER_NAME);
		return;
	}
	TAILQ_INSERT_TAIL(&hr->io_request_busyq, io_req, links);

	pOSD = &(io_req->OSD); 

	if (pOSD == NULL) {
		hr_shutdown_complete(sinfo);
		return;
	}
	memset(pOSD,0,sizeof(*pOSD));
	sinfo->pContext = (PVOID) io_req;
	pOSD->pOSRequest = (PVOID) sinfo;
	pOSD->OSD_Status = 0;
	pOSD->OSD_SenseLen = CDA_MAX_SENSE_LENGTH;
	pOSD->pOSD_Sense = &pOSD->OSD_Sense[0];
	pOSD->pOSD_Post = &hr_rp_shutdown_complete;
	pOSD->OSD_DeviceExtension = hr->pAdapterDevExt;

	/* Shutdown the RAID engine. */
	CdaShutdown(pOSD);
}

/*
 * Post function for the Shutdown Remote Control Packet.
 * May also be called with a NULL RP indicating that the
 * shutdown process could not be initiated due to a lack
 * of RP resources.  In the latter case, the list lock is
 * already held, so don't acquire it again.
 */
void
hr_rp_shutdown_complete(POSD pOSD)
{
	struct hr_shutdown_info *sinfo;
	struct hr_io_request	*io_request;
	struct hr_softc		*hr;
	PVOID			 pADE;
	u_long			 flags;

	pADE = (PVOID) pOSD->OSD_DeviceExtension;
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->pAdapterDevExt == pADE) {
			break;
		}
	}
	sinfo = (struct hr_shutdown_info *)pOSD->pOSRequest;
	io_request = (struct hr_io_request *) sinfo->pContext;

	/*
	 * If an external lock other than the softc
	 * lock is being used to syncronized the
	 * shutdown requests, the lock field in the
	 * sinfo will be initialized.  hr_shutdown_lock()
	 * takes care of honoring this lock for us.
	 */
	hr_shutdown_lock(sinfo, &flags);

	/*
	 * Instance lock protects CDA Core
	 * data structures, but it is, by
	 * definition, always held on entry
	 * into the core.  No need to acquire
	 * it here.
	 */
	TAILQ_REMOVE(&hr->io_request_busyq, io_request, links);
	TAILQ_INSERT_TAIL(&hr->io_request_freeq, io_request, links); 
	/*
	 * Notify the client of this shutdown
	 * that we have completed.
	 */
	hr_shutdown_complete(sinfo);
	hr_shutdown_unlock(sinfo, &flags);


}

static void
hr_shutdown_complete(struct hr_shutdown_info *sinfo)
{
	sinfo->shutdown_cnt--;
	if (sinfo->shutdown_cnt == 0)
		up(&sinfo->shutdown_sem);
}

/******************************* Helper Routines ******************************/
/*
 * Sleep for the specified interval or until
 * some other event ups the passed in semaphore.
 */
static void
hr_sleep(struct semaphore *sleep_sem, u_int interval)
{
	struct timer_list timer;

	init_timer(&timer);
	timer.expires = jiffies + interval;
	timer.data = (u_long) sleep_sem;
	timer.function = hr_sleep_timeout;
	add_timer(&timer);
	down_interruptible(sleep_sem);
	del_timer_sync(&timer);
}

static void
hr_sleep_timeout(u_long arg)
{
	struct semaphore *sleep_sem;

	sleep_sem = (struct semaphore *) arg;
	up(sleep_sem);
}

static void
hr_setup_dev_dpc_task(struct hr_linux_device *dev, void (*func)(void *))
{
	/* 
	 * If the there is an active DPC task, don't schedule the same
	 * task again.
	 */
	if (dev->flags & HR_DEV_DPC_ACTIVE)
		return;
		
	dev->flags |= HR_DEV_DPC_ACTIVE;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,41)
	dev->taskq.routine = func;
	dev->taskq.data = (void *) dev;
	schedule_task(&dev->taskq);
#else
	INIT_WORK(&dev->workq, func, (void *) dev);
	schedule_work(&dev->workq);
#endif
}

static void 
hr_ioreq_add_timer(struct hr_io_request *ioreq, int timeout)
{
	struct scsi_cmnd	*cmd;

	cmd = &hcmd_scsi_cmd(ioreq->hcmd);

	init_timer(&ioreq->eh_timeout);

	if (ioreq->eh_timeout.function)
		del_timer(&ioreq->eh_timeout);

	ioreq->eh_timeout.data = (unsigned long)cmd;
	ioreq->eh_timeout.expires = jiffies + timeout;
	ioreq->eh_timeout.function = (void (*)(unsigned long)) hr_ioreq_timeout;

	add_timer(&ioreq->eh_timeout);
}

static int 
hr_ioreq_delete_timer(struct hr_io_request *ioreq)
{
	int error;

	error = del_timer(&ioreq->eh_timeout);

	ioreq->eh_timeout.data = (unsigned long)NULL;
	ioreq->eh_timeout.function = NULL;

	return error;
}

/*************************** Initialization Routines **************************/
/*
 * First entry into our driver.  Register ourselves with the OS as appropriate
 * for the various versions of Linux.
 */
static int __init
hr_init()
{
	struct pci_device_id *id;

#if DEBUG_BUILD
	SET_DEBUG_LEVEL();
#endif

	printk("%s: %s.%s.%s.%s\n",HR_DRIVER_NAME,HR_DRIVER_VERSION,
		HR_DRIVER_REVISION,HR_DRIVER_SUBREVISION,HR_DRIVER_CHIMVERSION);

	hr_log(HR_BOOT_DEBUG, 1, "hr_init - Enter\n");
	for (id = hr_pci_id_table; id->vendor != 0; id++) {
		hr_log(HR_BOOT_DEBUG, 2,
		       "{ 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%lx }\n",
		       id->vendor, id->device, id->subvendor, id->subdevice,
		       id->class, id->class_mask, id->driver_data);
	}

	init_MUTEX_LOCKED(&hr_notifier_shutdown_info.shutdown_sem);
	hr_shutdown_lockinit();

	hr_log(HR_BOOT_DEBUG, 1, "hr_init - Locks initialized\n");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	hr_detect(&hr_host_template);
#else
	/*
	 * In 2.4.X, we must register our module with
	 * the SCSI layer.  The SCSI layer will then
	 * call our hr_detect() routine to probe for
	 * adapters.
	 */
	scsi_register_module(MODULE_SCSI_HA, &hr_host_template);  
#endif
	hr_log(HR_BOOT_DEBUG, 1, "hr_init - after detect/register\n");
	if (hr_init_status != 0) {
		/*
		 * Linux will only call our exit
		 * routine if our init function
		 * returns 0 status.  Force cleanup
		 * if initialization is not successful.
		 */
		hr_log(HR_BOOT_DEBUG, 1, "hr_init - init status (%d)\n",
		       hr_init_status);
		hr_exit();
	}
	return (hr_init_status);
}

/*
 * Entry point called during module teardown.
 */
static void __exit
hr_exit()
{
	struct hr_softc	*hr;

	hr_log(HR_BOOT_DEBUG, 1, "hr_exit: Enter\n");
	/*
	 * Shutdown our watchdog thread before proceeding to
	 * tearding down per-instance data.  We don't want the
	 * watchdog getting in the way of our cleanup.
	 */
	hr_driver_unloading = 1;
	
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		hr_kill_watchdog_thread(hr);
		hr_kill_ehandler_thread(hr);
	}

	if (hr_major > 0) {
		hr_log(HR_BOOT_DEBUG, 1, "hr_exit: About to devfs_unreg\n");
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
		devfs_unregister_chrdev(hr_major, HR_DRIVER_NAME);
#else
		unregister_chrdev(hr_major, HR_DRIVER_NAME);
#endif
		hr_log(HR_BOOT_DEBUG, 1, "hr_exit: After to devfs_unreg\n");
	}

	if (hr_reboot_notifier_registered)
		unregister_reboot_notifier(&hr_reboot_notifier_block);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	/*
	 * In 2.4 we have to unregister from the PCI core _after_
	 * unregistering from the scsi midlayer to avoid dangling
	 * references.  In later kernels, the PCI code will call us
	 * back to destroy/unregister each instance.
	 */
	scsi_unregister_module(MODULE_SCSI_HA, &hr_host_template);
#endif

	hr_pci_exit();
	hr_log(HR_BOOT_DEBUG, 1, "hr_exit: Finished\n");
}

static void
hr_kill_watchdog_thread(struct hr_softc *hr)
{

	if (hr_watchdog_pid != 0) {
		up(&hr_watchdog_sem);
		/*
		 * Wait for the watchdog thread to exit before continuing
	         * with the unloading processes.
		 */	 
		down_interruptible(&hr_watchdog_ending_sem);
		hr_watchdog_pid = 0;
	}
}
	
/*
 * Register our driver with the PCI subsystem.
 */
static int
hr_pci_init()
{
	int error;
	error = pci_module_init(&hr_pci_driver);
	if (error == 0)
		hr_pci_driver_registered = 1;
	return (error);
}

/*
 * De-register from the PCI subsystem.
 */
static void
hr_pci_exit()
{
	if (hr_pci_driver_registered)
	        pci_unregister_driver(&hr_pci_driver);
}

static void
hr_set_name(struct hr_softc *hr, char *name)
{
	if (hr->unit_name != NULL)
		kfree(hr->unit_name);
	hr->unit_name = name;
}

/*
 * Verify that the given softc is still valid.
 * Valid softcs are still in the softc list.
 * Callers should hold the list lock.
 */
static struct hr_softc *
hr_find_softc(struct hr_softc *hr)
{
	struct hr_softc *list_hr;

	TAILQ_FOREACH(list_hr, &hr_tailq, links) {
		if (list_hr == hr)
			return (hr);
	}
	return (NULL);
}

/*
 * Allocate a softc for the instance named
 * "name".  This routine takes ownership of
 * "name", so our caller should not free it
 * even if this routine returns NULL.
 */
static struct hr_softc *
hr_alloc(char *name, hr_dev_softc_t dev)
{
	struct	hr_softc *hr;

	hr = kmalloc(sizeof(*hr), GFP_ATOMIC);
	if (hr == NULL) {
                kfree(name);
		goto exit;
        }

	memset(hr, 0, sizeof(*hr));

	/* Allocate and initialize platform data specific. */
	if (hr_platform_alloc(hr) != 0) {
		kfree(name);
		kfree(hr);
		hr = NULL;
		goto exit;
	}

	TAILQ_INIT(&hr->device_runq);
	TAILQ_INIT(&hr->hr_resources);
	TAILQ_INIT(&hr->io_request_freeq);
	TAILQ_INIT(&hr->io_request_busyq);
	TAILQ_INIT(&hr->io_request_timedoutq);
	SLIST_INIT(&hr->dma_mem_regions);
	init_MUTEX_LOCKED(&hr->platform_data->ioctl_sem);
	init_MUTEX_LOCKED(&hr->shutdown_info.shutdown_sem);
	hr_lockinit(hr);
	hr->unit = hr_units++;
	hr->unit_name = name;
	hr->dev_softc = dev;
	hr->shutdown_info.lock = NULL;

exit:
	return (hr);
}

static int
hr_platform_alloc(struct hr_softc *hr)
{
	hr->platform_data = kmalloc(sizeof(struct hr_platform_data),
				    GFP_ATOMIC);	
	if (hr->platform_data == NULL)
		return (-ENOMEM);

	memset(hr->platform_data, 0, sizeof(struct hr_platform_data));
	init_MUTEX_LOCKED(&hr->platform_data->ehandler_sem);
	init_MUTEX_LOCKED(&hr->platform_data->ehandler_ending_sem);
	init_MUTEX_LOCKED(&hr->platform_data->eh_sem);

	return (0);
}

/*
 * Cleanup a controller instance.  Called *without*
 * the softc lock held.  At this point, the softc
 * *is not* in the hr_tailq.
 */
static void
hr_free(struct hr_softc *hr)
{
	PVOID	pADE;
	struct hr_resource 	*res;
	struct hr_linux_device 	*ldev = NULL;
	struct hr_io_request	*io_request;
	u_long		 	hr_flags;
	uint8_t		 	bus, target, lun;

	pADE = (PVOID) hr->pAdapterDevExt;

	/*
	 * Indicate that this instance is going away
	 * so that other threads of execution, such
	 * as our ioctl handler, will know to exit
	 * with an error.
	 */
	hr_lock(hr, &hr_flags);
	hr->flags |= HR_TEARDOWN_IN_PROGRESS;
	/*
	 * Wait for any inprogress ioctl request to finish.
	 * We use the semaphore in the shutdown info
	 * to handle the wait event.
	 */
	if ((hr->flags & HR_IOCTL_IN_PROGRESS) != 0) {
		hr_unlock(hr, &hr_flags);
		down(&hr->shutdown_info.shutdown_sem);
	} else {
		hr_unlock(hr, &hr_flags);
	}

	/*
	 * Deregister with the SCSI layer.
	 * This should stop any new transactions
	 * from coming in.  We must do this
	 * without any locks held since the
	 * SCSI layer may sleep here.
	 */
	if (hr->host != NULL) { 
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
		scsi_remove_host(hr->host);
		scsi_host_put(hr->host);
#else
		scsi_unregister(hr->host);
#endif
	}

	/*
	 * If we have successfully registered with 
	 * the Core, shutdown the RAID.
	 */
	if ((hr->flags & HR_CORE_INITIALIZED) != 0) {
		hr_lock(hr, &hr_flags);
		hr->shutdown_info.shutdown_cnt = 1;
		hr_shutdown(hr, &hr->shutdown_info);
		hr_unlock(hr, &hr_flags);

		/*
		 * Wait for Halt.
		 */
		down(&hr->shutdown_info.shutdown_sem);

	}

	if ((hr->flags & HR_IRQ_ALLOCATED) != 0)
		free_irq(hr_pci_dev(hr)->irq, hr);

	mdelay(1);

	/*
	 * Release all PCI resources.
	 */
	while ((res = TAILQ_FIRST(&hr->hr_resources)) != NULL) {

		TAILQ_REMOVE(&hr->hr_resources, res, res_links);
		switch (res->res_type) {
		case PCI_BASE_ADDRESS_SPACE_MEMORY:
			release_mem_region(res->res_barbase, res->res_len);
			break;
		case PCI_BASE_ADDRESS_SPACE_IO:
			release_region(res->res_barbase, res->res_len);
			break;
		default:
			break;
		}
		XMemFree((UCHAR DPTR) res, sizeof(*res));
	}

	/*
	 * Free all Linux Devices.
	 */
	lun = 0;
	for (bus = 0; bus < OSM_MAX_CHANNEL; bus++) {
		for (target = 0; target < OSM_MAX_TARGET_PER_PATH; target++) {
			ldev = (struct hr_linux_device *)
				hr->device_table[bus][target][lun];

			if (ldev != NULL) {
				hr_free_ldev(hr, ldev);
				hr->device_table[bus][target][lun] = NULL;
			}
		}	
	}

	/*
	 * Free all IO Request blocks.
	 */
	hr_lock(hr, &hr_flags);
	while ((io_request = hr_get_ioreq(hr)) != NULL) {
		kfree(io_request);
 	}
	hr_unlock(hr, &hr_flags);

	/*
	 * Release DMA safe memory resources.
	 */
	XMemFreeAllContiguous(pADE);

	TAILQ_REMOVE(&hr_tailq, hr, links);
	/*
	 * XXX What about non dma-safe memory?
	 */

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && \
    LINUX_VERSION_CODE  < KERNEL_VERSION(2,5,0)
	/*
	 * In 2.4 we detach from the scsi midlayer before the PCI
	 * layer invokes our remove callback - and that only occurs
	 * on module unload.  No per-instance detach is provided, so
	 * we must reach inside the PCI subsystem's internals and detach
	 * our driver manually.
	 */
	if (hr->dev_softc != NULL)
		hr->dev_softc->driver = NULL;
#endif
	kfree((uint8_t *)hr->os_mem_region.MEM_pPTTableNonDmableRegion); 
	kfree((uint8_t *)hr->os_mem_region.MEM_pLunTableNonDmableRegion);
	kfree(hr->unit_name);
	kfree(hr->pAdapterDevExt);
	kfree(hr);
}

/*
 * Allocate and initialize a per-device, OSM specific,
 * data structure to hold all per-device OSM state.
 */
struct hr_linux_device *
hr_alloc_ldev(struct hr_softc *hr, void *handle)
{
	struct hr_linux_device *ldev;

	ldev = kmalloc(sizeof(*ldev), GFP_ATOMIC);
	if (ldev == NULL) {
                printk("%s: cannot malloc linux device!\n",HR_DRIVER_NAME);
		return (NULL);
	}
	memset(ldev, 0, sizeof(*ldev));
	ldev->softc = hr;
	ldev->handle = handle;
	TAILQ_INIT(&ldev->busyq);
	hr_log(HR_BOOT_DEBUG, 1, "Allocated ldev with handle %p\n", 
				ldev->handle);
	return (ldev);
}

/*
 * Cleanup I/O queued on and free an hr_linux_device structure.
 */
void
hr_free_ldev(struct hr_softc *hr, struct hr_linux_device *dev)
{
	struct hr_cmd *hcmd;

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,41)
	/*
	 * Ensure no scheduled workqueue entry is running for
	 * this device prior freeing the device.
	 */
	if ((dev->flags & HR_DEV_DPC_ACTIVE) != 0)
		flush_scheduled_work();
#endif	
	/*
	 * Complete all deferred I/O with selection
	 * timeout status.
	 */
	while ((hcmd = TAILQ_FIRST(&dev->busyq)) != NULL) {
		struct scsi_cmnd *cmd;

		TAILQ_REMOVE(&dev->busyq, hcmd, hcmd_links.tqe);
		cmd = &hcmd_scsi_cmd(hcmd);

		hr_log(HR_RUNTIME_DEBUG, 1, "%s, complete pending IO: %p\n",
							 __FUNCTION__, hcmd);
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
		cmd->result = (DID_NO_CONNECT << 16);
#else
		hr_scsi_offline(cmd);
#endif
		cmd->scsi_done(cmd);
	}

	/*
	 * Release memory.
	 */
	kfree(dev);

	/* XXX What about the sysdevice? */
}

/*
 * Find the smallest available unit number to use
 * for a new device.  We don't just use a static
 * count to handle the "repeated hot-(un)plug"
 * scenario.
 */
static int
hr_next_unit()
{
	struct	hr_softc *hr;
	int	unit;

	unit = 0;
retry:
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->unit == unit) {
			unit++;
			goto retry;
		}
	}
	return (unit);
}

static int
hr_register_host(struct hr_softc *hr)
{
	char	 buf[80];
	struct	 hr_pci_driver_data *driver_data;
	struct	 Scsi_Host *host;
	char	*new_name;
	u_long	 s;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	int	 error;
#endif
	driver_data =(struct hr_pci_driver_data *)(hr->pci_entry->driver_data);
	hr_host_template.name = driver_data->description;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	host = scsi_host_alloc(&hr_host_template, sizeof(struct hr_softc *));
#else
	host = scsi_register(&hr_host_template, sizeof(struct hr_softc *));
#endif
	if (host == NULL) {
		return (-ENOMEM);
	}

	*((struct hr_softc **)host->hostdata) = hr;
	hr_lock(hr, &s);
	hr->host = host;
	host->max_id = hr->max_scsi_id;
	host->max_lun = hr->max_lun;
	host->max_channel = OSM_MAX_CHANNEL - 1;	//starting from zero
	host->this_id = hr->adapter_scsi_id;
	host->can_queue = hr->can_queue;
	host->cmd_per_lun = hr->max_cmds_per_lun;
	host->max_sectors = hr->max_sectors;
	host->irq = hr_pci_dev(hr)->irq;
	hr->unit = hr_next_unit();
	hr_assign_host_lock(hr);
	sprintf(buf, "scsi%d", host->host_no);
	new_name = kmalloc(strlen(buf) + 1, GFP_ATOMIC);
	if (new_name != NULL) {
		strcpy(new_name, buf);
		hr_set_name(hr, new_name);
	}
	host->unique_id = hr->unit;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,4) && \
    LINUX_VERSION_CODE  < KERNEL_VERSION(2,5,0)
	scsi_set_pci_device(host, hr->dev_softc);
#endif
	hr_unlock(hr, &s);

	/* 
	 * Create a watchdog thread to monitor any ongoing IO(s) 
	 * or IOCTL(s)
	 */
	hr_watchdog_pid = kernel_thread(hr_watchdog_thread, hr, 0);
	if (hr_watchdog_pid < 0) {
		hr_log(HR_BOOT_DEBUG, 1,
		       "Failed to create watchdog thread.\n");
		return (-hr_watchdog_pid);
	}
	/*
	 * Create an error-recovery thread for handling error recovery
	 * requested by the OS or async event.
	 */
	hr->platform_data->ehandler_pid = kernel_thread(
						hr_ehandler_thread, 
						hr, 0);
	if (hr->platform_data->ehandler_pid < 0) {
		hr_log(HR_BOOT_DEBUG, 1,
		       "Failed to create eh thread.\n");
		return (-hr->platform_data->ehandler_pid);
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	error = scsi_add_host(host, &hr->dev_softc->dev);
	if (error != 0) {
		return (error);
	}
	scsi_scan_host(host);
#endif
	return (0);
}

/*
 * Setup all controller instance state and register with the Core.
 * We do not register the instance with the OS.  That responsibility
 * is left to the caller.
 */
static int
hr_config(struct hr_softc *hr)
{
	int              	error, cda_error;
	uint8_t			i;
	
	hr_log(HR_BOOT_DEBUG, 1, "hr_config: Enter\n");
	error = CdaGetEngineProperties(NULL, &hr->cep);
	hr->pAdapterDevExt = kmalloc(hr->cep.CEP_DeviceExtensionSize,
				     GFP_ATOMIC);

	if(hr->pAdapterDevExt == NULL) {
		printk("hr_config: cannot alloc AdapterDevExt\n");	
		return (-EINVAL);
	}

	hr->pioctl_data_buf =  XMemAllocContiguous(hr->pAdapterDevExt, 
						sizeof(*hr->pioctl_data_buf));

	if(hr->pioctl_data_buf == NULL) {
		printk("hr_config: cannot alloc ioctl pbuf\n");	
		return (-EINVAL);
	}
	memset(hr->pAdapterDevExt,0,hr->cep.CEP_DeviceExtensionSize);

	/* Get the CDA engine properties. */
	error = CdaGetEngineProperties(hr->pAdapterDevExt, &hr->cep);
	if (error != CDA_SUCCESS)
		return (-EINVAL);

	/* Fills in informational CEP fields. */
	hr->cep.CEP_ChipId = ((hr->pci_entry->device << 0x10) | 
			  (hr->pci_entry->vendor));
	

	hr->cep.CEP_SubVendorID = (uint16_t) (hr_pci_read_config(hr->dev_softc, PCI_SUBSYSTEM_VENDORID_OFFSET, 2));
	hr->cep.CEP_SubsystemID = (uint16_t) (hr_pci_read_config(hr->dev_softc, PCI_SUBSYSTEM_DEVICEID_OFFSET, 2)); 


	hr->cep.CEP_BusNumber = hr_get_pci_bus(hr->dev_softc);
	hr->cep.CEP_SlotNumber = hr_get_pci_slot(hr->dev_softc);
	
	/* File version no. */
	hr->cep.CEP_usMajorRevision = ADPT_MAJOR_REV;
	hr->cep.CEP_usMinorRevision = ADPT_MINOR_REV;
	hr->cep.CEP_usBuildRevision = ADPT_OSM_REV;
	hr->cep.CEP_usReleaseRevision = ADPT_CHIM_REV; 
	strcpy(hr->cep.CEP_DriverName, HR_DRIVER_NAME);
	strcpy(hr->cep.CEP_DriverDescription, DRIVERDESC);	 
	
	if (hr->cep.CEP_ChipId == 
			((PCI_DEVICE_SPIRIT << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "Spirit\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
			((PCI_DEVICE_AIC9410 << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-9410\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AIC9410B << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-9410\0");
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AIC9412 << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC8412\0");
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AIC9415 << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC8412\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AIC9416 << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC8412\0");
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AICLITE << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-lite\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AICLITEB << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-liteB\0");
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AICLITEEA << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-liteEA\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_AICLITEEB << 0x10) | PCI_VENDOR_94XX)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-liteEB\0");
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_7250HR << 0x10) | PCI_VENDOR_725X)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-7250LH\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_7250NR << 0x10) | PCI_VENDOR_725X)) {
		strcpy(hr->cep.CEP_ProductName, "AIC-7250LH\0");
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
		}
	else if (hr->cep.CEP_ChipId ==
			 ((PCI_DEVICE_NVDA << 0x10) | PCI_VENDOR_NVDA)) {
		strcpy(hr->cep.CEP_ProductName, "NVIDIA\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICHa << 0x10) | PCI_VENDOR_ICH)) {
		strcpy(hr->cep.CEP_ProductName, "ICH5a\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICHb << 0x10) | PCI_VENDOR_ICH)) {
		strcpy(hr->cep.CEP_ProductName, "ICH5b\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICH5 << 0x10) | PCI_VENDOR_ICH5)) {
		strcpy(hr->cep.CEP_ProductName, "ICH5\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICH5R << 0x10) | PCI_VENDOR_ICH5)) {
		strcpy(hr->cep.CEP_ProductName, "ICH5R\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICH5Rx << 0x10) | PCI_VENDOR_ICH5)) {
		strcpy(hr->cep.CEP_ProductName, "ICH5Rx\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICH6R << 0x10) | PCI_VENDOR_ICH6R)) {
		strcpy(hr->cep.CEP_ProductName, "ICH6R\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICH7R << 0x10) | PCI_VENDOR_ICH6R)) {
		strcpy(hr->cep.CEP_ProductName, "ICH7R\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ICH7A << 0x10) | PCI_VENDOR_ICH6R)) {
		strcpy(hr->cep.CEP_ProductName, "ICH7A\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_ESB2 << 0x10) | PCI_VENDOR_ICH6R)) {
		strcpy(hr->cep.CEP_ProductName, "ESB2\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_88SX5041 << 0x10) | PCI_VENDOR_MARV)) {
		strcpy(hr->cep.CEP_ProductName, "88SX5041\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_88SX5081 << 0x10) | PCI_VENDOR_MARV)) {
		strcpy(hr->cep.CEP_ProductName, "88SX5081\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_88SX6041 << 0x10) | PCI_VENDOR_MARV)) {
		strcpy(hr->cep.CEP_ProductName, "88SX6041\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_88SX6081 << 0x10) | PCI_VENDOR_MARV)) {
		strcpy(hr->cep.CEP_ProductName, "88SX6081\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_88SX6541 << 0x10) | PCI_VENDOR_MARVADP)) {
		strcpy(hr->cep.CEP_ProductName, "88SX6541\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_0241 << 0x10) | PCI_VENDOR_MARVADP)) {
		strcpy(hr->cep.CEP_ProductName, "88SX0241\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_0243 << 0x10) | PCI_VENDOR_MARVADP)) {
		strcpy(hr->cep.CEP_ProductName, "88SX0243\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else if (hr->cep.CEP_ChipId == 
		((PCI_DEVICE_SIL3132 << 0x10) | PCI_VENDOR_SIL)) {
		strcpy(hr->cep.CEP_ProductName, "SIL3132\0");
    		hr->cep.CEP_OperationMode = 0;
		}
	else {
		strcpy(hr->cep.CEP_ProductName, "Unknown\0");		
    		hr->cep.CEP_OperationMode = CDA_OP_MODE_NO_RAID;
	}
	
	hr->cep.CEP_MaxIods = CDA_MAX_IODS;
	hr->cep.CEP_MaxCommandsPerAdapter = 128;
	hr->cep.CEP_MaxOsds = 0;	
    	
	/* 
	* Fill in the CDA Engine Properties based on the setting provided by
	* the user. Hardcoded for now !!.
	*/
	hr->cep.CEP_MaximumSGList = OSM_SG_MAX;
	hr->cep.CEP_MaxTags = hr->can_queue;
	hr->cep.CEP_ITLNexusLossTimer = 4; /* Default to 5 secs. */
	hr->cep.CEP_ForceFlushOnSata = FALSE;
	hr->cep.CEP_CoercionPolicy = 0;	/* DISABLED */
	hr->cep.CEP_MdStampingPolicy = 1;	/* DEFAULT */
	hr->cep.CEP_TicksPerSec = 1;
	hr->cep.CEP_DevIORetryCnt = 5;	
	hr->cep.CEP_EnableBootDeviceMap = 1;
#ifdef CDA_FOR_SATA
	hr->cep.CEP_OsmMaxPath = 1; 
	hr->cep.CEP_RaidPath = 0;
	hr->cep.CEP_ReservePaths = 0;
	hr->cep.CEP_VSAModePath = 6;
	hr->cep.CEP_PathFirst = 1;
	hr->cep.CEP_MemoryMapped = TRUE;	
	/* set to 1 if have prob with mem */
	hr->cep.CEP_FaultToleranceTimeout = 8;
	hr->cep.CEP_MaxTargetIdAllowed = 32;
	hr->cep.CEP_MaxTargetsAllowed = 32;
	hr->cep.CEP_MaxLuns = 32;
#else
	hr->cep.CEP_OsmMaxPath = 8; 
	hr->cep.CEP_RaidPath = 7;
	hr->cep.CEP_ReservePaths = 5;
	hr->cep.CEP_VSAModePath = 6;
	hr->cep.CEP_PathFirst = 0;
	hr->cep.CEP_MemoryMapped = FALSE;
	hr->cep.CEP_FaultToleranceTimeout = 12;
	hr->cep.CEP_MaxTargetIdAllowed = 256;
	hr->cep.CEP_MaxTargetsAllowed = 256;
	hr->cep.CEP_MaxLuns = 128;
#endif
	hr->cep.CEP_PerformanceKnobe = CDA_PERFORMANCE_KNOB_HIGH;

#ifdef SES2_DEVICE
	hr->cep.CEP_ExposeEnclosure = 0;
	hr->cep.CEP_EnableLED = 1;
#endif

	/* Map the PCI Bar Register */
	error = hr_map_io_handle(hr);
	if (error != 0) {
		hr_log(HR_BOOT_DEBUG, 1, "Map IO Handles Failed !!\n");
		return (error);
	}	

	/* Set the CDA Engine properties */
	error = CdaSetEngineProperties(hr->pAdapterDevExt, &hr->cep);
	if (error != CDA_SUCCESS)
		return (-EINVAL);	
	/* 
	 * Get the CDA Engine properties again to calculate DMA and Non-DMA
	 * memory needed.
	 */
	error = CdaGetEngineProperties(hr->pAdapterDevExt, &hr->cep);
	if (error != CDA_SUCCESS)
		return (-EINVAL);	
	/*
	 * Since we manage the memory allocation, set the os_mem_regions to 0.
	 */
	memset(&hr->os_mem_region, 0x0, sizeof(CDA_OS_MEMORY_REGIONS));
	hr->os_mem_region.MEM_pPTTableNonDmableRegion = (unsigned long)
			kmalloc(hr->cep.CEP_PTTableMemorySize, GFP_ATOMIC);
	hr->os_mem_region.MEM_PTTableNonDmableRegionSize =
			 			hr->cep.CEP_PTTableMemorySize;
	hr->os_mem_region.MEM_pLunTableNonDmableRegion = (unsigned long) 
			kmalloc(hr->cep.CEP_LunTableMemorySize, GFP_ATOMIC);
	hr->os_mem_region.MEM_LunTableNonDmableRegionSize =
						 hr->cep.CEP_LunTableMemorySize;
	hr->os_mem_region.MEM_pDmableRegion = 0;
	hr->os_mem_region.MEM_DmableRegionSize = 0;
	hr->os_mem_region.MEM_pNonDmableRegion = 0;
	hr->os_mem_region.MEM_NonDmableRegionSize = 0;
	hr->DriverState |= DRIVER_STATE_HW_INITIALIZE;
	/* Initialize the CDA Engine. */
	error = CdaSysinit(hr->pAdapterDevExt, &hr->os_mem_region);
	if (error != CDA_SUCCESS) {
		hr_log(HR_BOOT_DEBUG, 1, "CdaSysinit Failed (error=0x%x)!!\n",
			error);
		return (-EINVAL);
	}
	cda_error=CdaIntiTimeDiscoverDevices(hr->pAdapterDevExt);
	if( cda_error != CDA_SUCCESS)return(-EINVAL);
	CdaRestartAdapterCleanup(hr->pAdapterDevExt);

	/* Scan for devices and map them */
	error = hr_discover_device(hr);
	if (error != 0){
		printk("hr_discover_device_error=%x\n",error);
		return (error);
	}
	/* Allocate a pool of OSDs. */
	for (i = 0; i < HR_MAX_OSD; i++) {
		struct hr_io_request	*io_request;
		
		io_request = kmalloc(sizeof(*io_request), GFP_ATOMIC);
		if (io_request == NULL) {
			hr_log(HR_BOOT_DEBUG, 1, "Alloc OSD failed !!\n");
			return (-ENOMEM);
		}				     		     
		memset(io_request, 0x0, sizeof(*io_request));
		TAILQ_INSERT_TAIL(&hr->io_request_freeq, io_request, links);
	}
	
	CdaResetInitializatonFlag(hr->pAdapterDevExt);
	hr->DriverState &= ~DRIVER_STATE_HW_INITIALIZE;
	hr->DriverState |= DRIVER_STATE_UP_AND_RUNNING;
	hr->flags |= HR_CORE_INITIALIZED;

	return (0);
}

/* 
 * Map the IO handles for the register access.
 */
static int
hr_map_io_handle(struct hr_softc *hr)
{
	struct hr_resource	*res;
	u_long 		base_addr;
	uint32_t		bar_type;
	uint32_t		bar_len;
	uint8_t			index;
	
	for (index = 0; index < PCI_MAX_32_BIT_BARS; index++) {
	       /*
	 	* Determine BAR size and validate the request.
	        */
		base_addr = pci_resource_start(hr->dev_softc, index);
		bar_type = pci_resource_flags(hr->dev_softc, index);
		bar_len = pci_resource_len(hr->dev_softc, index);

		if ((base_addr == 0) ||	(bar_len == 0))
			continue;
			
		/*
	 	 * See if we have already allocated a resource descriptor
	 	 * for this BAR.  If so, we will reuse the mapping contained
	 	 * in that descriptor.  If not, allocate a descriptor and
	 	 * map the whole BAR.
	 	 */
		TAILQ_FOREACH(res, &hr->hr_resources, res_links) {
			if (res->res_bar == index)
				break;
		}
	
		if (res == NULL) {
			res = (struct hr_resource *)
				kmalloc((size_t) sizeof(*res), GFP_ATOMIC);				       
			if (res == NULL)
				return (ENOMEM);

			res->res_len = bar_len;
			res->res_type = bar_type & PCI_BASE_ADDRESS_SPACE;
			res->res_bar = index;
			res->res_barbase = base_addr;
			
			if (res->res_type == PCI_BASE_ADDRESS_SPACE_IO) {
				/*
			 	 * Reserve this BAR in its entirety
			 	 * with the OS.
			 	 */
				if (request_region(base_addr,
					   bar_len, HR_DRIVER_NAME) == 0) {
					hr_log(HR_BOOT_DEBUG, 1, 
				       "Request Region Failed "
				       "for index=%d, addr=0x%lx, len=0x%x !\n",
				       index, base_addr, bar_len);
					
					kfree(res);
		                	return (ENOMEM);
				}
				
				hr->cep.CEP_BAR[index].BaseAddr = 
						(UINT32) res->res_barbase;
				hr->cep.CEP_BAR[index].MemoryMapped = FALSE;		
			} else {
				u_long	base_page;
				u_long	base_offset;

				/*
			 	 * Reserve this BAR in its entirety
			 	 * with the OS.
			 	 */
				if (request_mem_region(base_addr,
					       bar_len, HR_DRIVER_NAME) == 0) {
					hr_log(HR_BOOT_DEBUG, 1, 
				       "Request Mem Region Failed "
				       "for index=%d, Addr=0x%lx, Len=0x%x !\n",
				       index, base_addr, bar_len);
				
					kfree(res);
		                	return (ENOMEM);
				}
				
				base_page = base_addr & PAGE_MASK;
				base_offset = base_addr - base_page;
				res->res_membase = ioremap_nocache(
						        base_page,
							base_offset + bar_len);
				res->res_membase += base_offset;
				
				if (res->res_membase == NULL) {
					hr_log(HR_OSMIF_DEBUG, 1, 
				       "ioremap_nocache Failed:"
				       " Base = 0x%lx Offset = 0x%lx\n",
				       (u_long)base_page,
				       (u_long)(base_offset + bar_len));
					release_mem_region(base_addr, bar_len);
		                	return (ENOMEM);
				}
				
				hr->cep.CEP_BAR[index].BaseAddr = (DATA_N_PTR)
							res->res_membase;
				hr->cep.CEP_BAR[index].MemoryMapped = TRUE;

			}
			TAILQ_INSERT_TAIL(&hr->hr_resources, res, res_links);
		}
	}

	return (0);		
}

/*************************************************************************

Function Name 
    discover_device		

Function Description
    discover device(s) found at cda.

Input Parameters
	pOsmDevExt          -   pointer to OSM Device Extension.     

Output Parameters
	None

NOTE: This function can only be used at Initialization time
*************************************************************************/
static	int 
hr_discover_device( struct hr_softc *hr)
{
	struct hr_linux_device  *ldev;
	IO_HANDLE		 IoHandle;
	int bus, target;
  
	for(bus=0;bus<OSM_MAX_CHANNEL;bus++) {
		for(target=0;target<OSM_MAX_TARGET_PER_PATH;target++) {
			memset(&IoHandle, 0, sizeof(IO_HANDLE));
			CdaGetPTSlot(hr->pAdapterDevExt, bus, target,0,
				     &IoHandle);
			if (IoHandle.pHandle == NULL)
				continue;

			ldev = hr_alloc_ldev(hr, IoHandle.pHandle);
				if(ldev == NULL) {
					printk("Discover: ldev no mem\n");
				continue;
				}
				ldev->flags |= HR_DEV_ONLINE;
				hr->device_table[bus][target][0] = ldev;
		}
	}

    return 0;
} 

/******************************* Ioctl Interface ******************************/
/*
 * This is called each time the driver is opened.
 * Only allow an open if an instance corresponding
 * to the minor exists.
 */
static int
hr_open(struct inode *inode, struct file *file)
{
	struct	hr_softc *hr;
	int	minor;

	hr_log(HR_RUNTIME_DEBUG, 1, "hr_open(%p,%p)\n", inode, file);

	minor = MINOR(inode->i_rdev);
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (minor == 0)
			break;
		minor--;
	}

	if(hr != NULL)
		file->private_data = inode;

	return (hr == NULL ? -ENODEV : 0);
}

/*
 * Driver close() entry point. In the case of a pass-thru close,
 * It resumes the queue and calls the target driver event handler
 * if one is present.
 */
static int
hr_close(struct inode *inode, struct file *file)
{
	UNREFERENCED_PARAMETER(inode);

	hr_log(HR_RUNTIME_DEBUG, 1, "hr_close(%p,%p)\n",
		  inode, file);

	file->private_data = NULL;

	return (0);
}

#ifdef CONFIG_COMPAT
#if	HAVE_COMPAT_IOCTL
static int hr_compat_ioctl(struct file *file, u_int cmd, u_long arg)
{
        return hr_ioctl( file->private_data, file, cmd, arg);
}
#endif
#endif


/*
 * This is the driver's ioctl entrypoint.  We copy all user data into
 * kernel buffers up-front, issue the request, sleep waiting for the
 * request to complete, copy the data back into the user process, and
 * finally return status.
 *
 * Input Parameters
 *	inode	specifies the major and minor device numbers.
 *	file    specifies the operations that are allowed.
 *	cmd	command passes to ioctl system call.
 *	arg	pointer to athena package.
 */
static int
hr_ioctl(struct inode *inode, struct file *file, u_int cmd, u_long arg)
{
	struct hr_softc *hr;
	struct hr_disk_info diskinfo;
	struct hr_linux_device *ldev;
	AthenaSGElement	*pSG;
	uint8_t		*pbuf;
	void 		*pUserCSMI;
	PVOID pADE;
	POSD	pOSD;
	pAthenaPkt	pUserAP;
	pAthenaPkt	pAP;
	u_long		 flags;
	u_long		 datalen;
	u_int		 i;
	uint32_t	PktSize;
	int		 error;
	int		 minor;

	UNREFERENCED_PARAMETER(file);

	hr_log(HR_RUNTIME_DEBUG, 1, "hr_ioctl(%p,%p,%x,%lx\n",
					inode, file, cmd, arg);

	minor = MINOR(inode->i_rdev);

	/*
	 * Find our instance.
	 */
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (minor == 0) {
			break;
		}
		minor--;
	}

	if (hr == NULL)
		return (-ENODEV);

	hr_lock(hr, &flags);

	if(!(hr->DriverState & DRIVER_STATE_UP_AND_RUNNING)) {
		hr_unlock(hr, &flags);
		return (-EBUSY);
	}


	/*
	 * We only allow one ioctl request to be
	 * pending at a time.  Notify the user if
	 * we are already busy or being torn down.
	 */
	if ((hr->flags & (HR_TEARDOWN_IN_PROGRESS|HR_IOCTL_IN_PROGRESS 
			| HR_SUSPEND_IO)) != 0) {
		hr_unlock(hr, &flags);
		return (-EBUSY);
	}

	/*
	 * Indicate that an ioctl is in progress.  This
	 * prevents other threads from accessing ioctl 
	 * resources or destroying this instance.
	 */
	hr->flags |= HR_IOCTL_IN_PROGRESS;

	pUserCSMI = NULL;
	pUserAP = NULL;
	pbuf = NULL;
	pAP = NULL;
	pADE = NULL;

	datalen = PktSize = 0;
	pOSD = &hr->pioctl_data_buf->OSD;
	memset(pOSD,0,sizeof(*pOSD));
	pOSD->OSD_CDB[0] = CDA_EXECUTE_IOCTL;

	switch(cmd){
		case CC_CSMI_SAS_GET_DRIVER_INFO: 
			pUserCSMI = (PCSMI_SAS_DRIVER_INFO_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_DRIVER_INFO_BUFFER);
			break;
		case CC_CSMI_SAS_GET_CNTLR_CONFIG:
			pUserCSMI = (PCSMI_SAS_CNTLR_CONFIG_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_CNTLR_CONFIG_BUFFER);
			break;
		case CC_CSMI_SAS_GET_CNTLR_STATUS:
			pUserCSMI = (PCSMI_SAS_CNTLR_STATUS_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_CNTLR_STATUS_BUFFER);
			break;
		case CC_CSMI_SAS_FIRMWARE_DOWNLOAD:
			pUserCSMI = (PCSMI_SAS_FIRMWARE_DOWNLOAD_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_FIRMWARE_DOWNLOAD_BUFFER);
			break;
		case CC_CSMI_SAS_GET_RAID_INFO:
			pUserCSMI = (PCSMI_SAS_RAID_INFO_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_RAID_INFO_BUFFER);
			break;
		case CC_CSMI_SAS_GET_RAID_CONFIG: 
			pUserCSMI = (PCSMI_SAS_RAID_CONFIG_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_RAID_CONFIG_BUFFER);
			break;
		case CC_CSMI_SAS_GET_PHY_INFO:
			pUserCSMI = (PCSMI_SAS_PHY_INFO_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_PHY_INFO_BUFFER);
			break;
		case CC_CSMI_SAS_SET_PHY_INFO:
			pUserCSMI = (PCSMI_SAS_PHY_INFO_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_PHY_INFO_BUFFER);
			break;
#ifndef	CDA_FOR_SATA
		case CC_CSMI_SAS_GET_LINK_ERRORS:
			pUserCSMI = (PCSMI_SAS_LINK_ERRORS_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_LINK_ERRORS_BUFFER);
			break;
		case CC_CSMI_SAS_SMP_PASSTHRU:
			pUserCSMI = (PCSMI_SAS_SMP_PASSTHRU_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_SMP_PASSTHRU_BUFFER);
			break;
		case CC_CSMI_SAS_SSP_PASSTHRU:
			{
			PCSMI_SAS_SSP_PASSTHRU_BUFFER pCSMI;
			pUserCSMI = pCSMI = (PCSMI_SAS_SSP_PASSTHRU_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_SSP_PASSTHRU_BUFFER) +
				pCSMI->Parameters.uDataLength;
			}
			break;
#endif	//if not CDA_FOR_SATA
		case CC_CSMI_SAS_STP_PASSTHRU:
			{
			PCSMI_SAS_STP_PASSTHRU_BUFFER pCSMI;
			pUserCSMI = pCSMI = (PCSMI_SAS_STP_PASSTHRU_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_STP_PASSTHRU_BUFFER) + 
				pCSMI->Parameters.uDataLength;
			}
			break;
		case CC_CSMI_SAS_GET_SATA_SIGNATURE:
			pUserCSMI = (PCSMI_SAS_SATA_SIGNATURE_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_SATA_SIGNATURE_BUFFER);
			break;
		case CC_CSMI_SAS_GET_SCSI_ADDRESS:
			pUserCSMI = (PCSMI_SAS_GET_SCSI_ADDRESS_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_GET_SCSI_ADDRESS_BUFFER);
			break;
		case CC_CSMI_SAS_GET_DEVICE_ADDRESS:
			pUserCSMI = (PCSMI_SAS_GET_DEVICE_ADDRESS_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_GET_DEVICE_ADDRESS_BUFFER);
			break;
		case CC_CSMI_SAS_TASK_MANAGEMENT:
			pUserCSMI = (PCSMI_SAS_SSP_TASK_IU_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_SSP_TASK_IU_BUFFER);
			break;
		case CC_CSMI_SAS_GET_CONNECTOR_INFO:
			pUserCSMI = (PCSMI_SAS_CONNECTOR_INFO_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_CONNECTOR_INFO_BUFFER);
			break;
		case CC_CSMI_SAS_PHY_CONTROL:
			pUserCSMI = (PCSMI_SAS_PHY_CONTROL_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_PHY_CONTROL_BUFFER);
			break;
#ifndef	CDA_FOR_SATA
		case CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG:
			pUserCSMI = (PCSMI_SAS_GET_ADPT_CNTLR_CONFIG_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_GET_ADPT_CNTLR_CONFIG_BUFFER);
			break;
		case CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES:
			pUserCSMI = (PCSMI_SAS_NV_SEGMENT_PROPERTIES_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_NV_SEGMENT_PROPERTIES_BUFFER);
			break;
		case CC_CSMI_SAS_READ_NV_SEGMENT:
			{
			PCSMI_SAS_READ_NV_SEGMENT_BUFFER readnvseg;
			pUserCSMI = readnvseg =
					(PCSMI_SAS_READ_NV_SEGMENT_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_READ_NV_SEGMENT_BUFFER) +
					 readnvseg->Information.uBytesToRead;
			}
			break;
		case CC_CSMI_SAS_WRITE_NV_SEGMENT:
			{
			PCSMI_SAS_WRITE_NV_SEGMENT_BUFFER writenvseg;
			pUserCSMI = writenvseg  =
					(PCSMI_SAS_WRITE_NV_SEGMENT_BUFFER)arg;
			PktSize = sizeof(CSMI_SAS_WRITE_NV_SEGMENT_BUFFER) +
					 writenvseg->Information.uBufferLength;
			}
			break;
#endif	//if not CDA_FOR_SATA
		case ADAPTEC_IOCTL: 
			pUserAP = (pAthenaPkt)arg;
			PktSize = 0;
			pAP = &hr->pioctl_data_buf->AP;
			pbuf = (uint8_t *)(hr->pioctl_data_buf->pbuf);
			pADE = hr->pAdapterDevExt;

			hr_unlock(hr, &flags);

			if (access_ok(VERIFY_READ, pUserAP,
					sizeof(AthenaPkt)) == 0) {
				error = -EPERM;
				goto ioctl_done_unlocked;
			}

			if(copy_from_user(pAP, pUserAP, sizeof(AthenaPkt))) {
				error = -EPERM;
				goto ioctl_done_unlocked;
			}

			hr_lock(hr, &flags);

			/*
	 		* Divert any I/O indicated by the S/G list through our
	 		* static ioctl data buffer.
	 		*/
			pSG = pAP->AP_SGTable.SGList;
			for (datalen = 0, i = 0; i < pAP->AP_SGTable.SGCount;
								i++, pSG++) {
		
				/*
		 		 * If the request cannot fit in our static
		 		 * buffer 8k, error out.
		 		 */
				if (datalen + pSG->SGLByteCnt > 0x2000) {
					error = -EINVAL;
					goto ioctl_done_locked;
				}

				/*
		 		 * If the request contains a write phase, we
		 		 * must copy-in the data before executing the
		 		 * command.
		 		 */
				if ((pAP->AP_Direction == DATA_OUT) ||
					 (pAP->AP_Direction == BIDIRECTIONAL)) {

					hr_unlock(hr, &flags);
					if (access_ok(VERIFY_READ,
						pSG->SGLBufAddr,
				      		pSG->SGLByteCnt) == 0) {
						error = -EPERM;
						goto ioctl_done_unlocked;
					}
					if(copy_from_user(pbuf + datalen,
				       		(void *)pSG->SGLBufAddr,
				       		pSG->SGLByteCnt)) {
						error = -EINVAL;
						goto ioctl_done_unlocked;
					}
					hr_lock(hr, &flags);
				}
				datalen += pSG->SGLByteCnt;
			}

			if (pAP->AP_Direction == DATA_IN) {
				/*
		 		* Zero the buffer so that untransferred
		 		* data appears cleared.
		 		*/ 
				memset(pbuf, 0, datalen);
			}
		break;
		default:
			error = -EINVAL;
			goto ioctl_done_locked;
	} //switch

	if(cmd == ADAPTEC_IOCTL) {
		if (pAP->AP_SCSICmd.Opcode == IOCTL_GET_DISK_INFO) {
			if(GetPTLFromLDTree(&pAP->AP_Target, &diskinfo, pADE)) {
				error = -EPERM;
				goto ioctl_done_locked;
			}
			diskinfo.cnum = minor;
			if(hr->device_table[diskinfo.bus][diskinfo.target] \
						[diskinfo.lun] != NULL) {
				ldev = hr->device_table[diskinfo.bus] \
						[diskinfo.target][diskinfo.lun];
				if((ldev->flags & HR_DEV_ONLINE) != 0) {
					hr_get_sd_devname(ldev->rq_dev,
							 diskinfo.name);
					pSG = pAP->AP_SGTable.SGList;	

					hr_unlock(hr, &flags);
					copy_to_user((void *)pSG->SGLBufAddr,
				  			(void *)&diskinfo,
							 sizeof(diskinfo));

					error = 0;
					goto ioctl_done_unlocked;
				}
			}
			error = -EINVAL;
			goto ioctl_done_locked;
		}
		pOSD->OSD_CDB[1] = CDA_CDB_ATHENA_IOCTL;
		pOSD->OSD_pDataBuffer = (DATA_N_PTR *)hr->pioctl_data_buf;
	} else {
		pOSD->OSD_CDB[1] = CDA_CDB_CSMI_IOCTL;
		pOSD->OSD_CDB[2] = (uint8_t)cmd;
		pOSD->OSD_CDB[3] = (uint8_t)(cmd >> 8);
		pOSD->OSD_CDB[4] = (uint8_t)(cmd >> 16);
		pOSD->OSD_CDB[5] = (uint8_t)(cmd >> 24);
		/* adding timeout value */
		pOSD->OSD_CDB[9] = 30;
		if(capable(CAP_SYS_ADMIN))
			pOSD->OSD_CDB[6] = CSMI_SECURITY_FULL_ACCESS;

		hr_unlock(hr, &flags);

		if (access_ok(VERIFY_READ, pUserCSMI, PktSize) == 0) {
			error = -EPERM;
			goto ioctl_done_unlocked;
		}
		if(PktSize > 0x2000) {
			pOSD->OSD_pDataBuffer = kmalloc(PktSize, GFP_ATOMIC);
			if(copy_from_user(pOSD->OSD_pDataBuffer,
						pUserCSMI, PktSize)) {
				error = -EPERM;
				goto ioctl_done_unlocked;
			}
		} else {
			if(copy_from_user(hr->pioctl_data_buf->pbuf,
						pUserCSMI, PktSize)) {
				error = -EPERM;
				goto ioctl_done_unlocked;
			}
			pOSD->OSD_pDataBuffer = (DATA_N_PTR *)
						hr->pioctl_data_buf->pbuf;
		}


		pOSD->OSD_DataTransferLength = PktSize;
		hr_lock(hr, &flags);
	}

	pOSD->pOSRequest = (PVOID) hr;
	pOSD->OSD_Status = 0;
	pOSD->OSD_SenseLen = CDA_MAX_SENSE_LENGTH;
	pOSD->pOSD_Sense = &pOSD->OSD_Sense[0];
	pOSD->OSD_DeviceExtension = hr->pAdapterDevExt;
	pOSD->pOSD_Post = (FPTR) &XPostApiIOCtlDonex;

	CdaExecuteOSD(pOSD);		
	hr_unlock(hr, &flags);
	down(&hr->platform_data->ioctl_sem);

	/*
	 * Process completed OSD.  The ioctl is considered
	 * "successful" unless we can't copyout data or
	 * status.
	 */
	error = 0;

	if(cmd == ADAPTEC_IOCTL) {

		/*
		 * User space might swapped out while we were sleeping.
		 * Do another user copy here. 
		 */

		if (access_ok(VERIFY_READ, &pUserAP->AP_SGTable,
					 sizeof(AthenaSGTable)) == 0) {
			error = -EPERM;
			goto ioctl_done_unlocked;
		}

		if(copy_from_user(&pAP->AP_SGTable, &pUserAP->AP_SGTable,
						 sizeof(AthenaSGTable))) {
			error = -EPERM;
			goto ioctl_done_unlocked;
		}

		put_user(pUserAP->AP_Status, &pOSD->OSD_Status);

		if (pAP->AP_Status != CDA_SUCCESS) {
			/*
		 	* Copy status and sense information
		 	* into user's request.
		 	*/
			copy_to_user(&pUserAP->AP_Sense, &pAP->AP_Sense, 32);

			error = -EPERM;
			goto ioctl_done_unlocked;
		}

		if (datalen != 0
	 	&& (pAP->AP_Direction == DATA_IN
	  	|| pAP->AP_Direction == BIDIRECTIONAL)) {
			pSG = pAP->AP_SGTable.SGList;
			for (i = 0, datalen = 0;
		     	i < pAP->AP_SGTable.SGCount; i++, pSG++) {
				if (access_ok(VERIFY_WRITE, pSG->SGLBufAddr,
					      pSG->SGLByteCnt) == 0) {
					error = -EPERM;
					goto ioctl_done_unlocked;
				}

				if(copy_to_user((void *)pSG->SGLBufAddr,
					     pbuf + datalen,
				     	pSG->SGLByteCnt)) {
					error = -EPERM;
					goto ioctl_done_unlocked;
				}

				datalen += pSG->SGLByteCnt;
			}
		}
	} else {
		if(pOSD->OSD_Status != CSMI_SAS_STATUS_SUCCESS) {
			error = -EPERM;
			if(PktSize > 0x2000) {
				if(pOSD->OSD_pDataBuffer != NULL)	
					kfree(pOSD->OSD_pDataBuffer);	
			}
			printk("hr_ioctl: cmd failed\n");
			goto ioctl_done_unlocked;
		}
		if(PktSize > 0x2000) {
			if (access_ok(VERIFY_WRITE, pUserCSMI,
					 pOSD->OSD_DataTransferLength) == 0) {
				error = -EPERM;
				if(pOSD->OSD_pDataBuffer != NULL)	
					kfree(pOSD->OSD_pDataBuffer);	
				goto ioctl_done_unlocked;
			}
			if(copy_to_user(pUserCSMI, pOSD->OSD_pDataBuffer,
						pOSD->OSD_DataTransferLength)) {
				error = -EPERM;
				goto ioctl_done_unlocked;
			}


			if(pOSD->OSD_pDataBuffer != NULL)	
					kfree(pOSD->OSD_pDataBuffer);	

		} else {
			if (access_ok(VERIFY_WRITE, pUserCSMI, PktSize) == 0) {
				error = -EPERM;
				printk("hr_ioctl: verify_write failed\n");
				goto ioctl_done_unlocked;
			}

			if(copy_to_user(pUserCSMI,
					 hr->pioctl_data_buf->pbuf, PktSize)) {
				error = -EPERM;
				goto ioctl_done_unlocked;
			}

		}

	}

ioctl_done_unlocked:
	hr_lock(hr, &flags);
ioctl_done_locked:
	hr->flags &= ~HR_IOCTL_IN_PROGRESS;
	if ((hr->flags & HR_TEARDOWN_IN_PROGRESS) != 0)
		up(&hr->shutdown_info.shutdown_sem);
	hr_unlock(hr, &flags);
	return (error);
}


VOID
XPostApiIOCtlDonex(POSD pOSD)
{
	struct hr_softc *hr;

	hr = (struct hr_softc *) pOSD->pOSRequest;
	/* hacking:pADAPTER_DEV_EXT is not supposed to be visible. */
	if(pOSD->OSD_CDB[1] == CDA_CDB_CSMI_IOCTL)
    	((pADAPTER_DEV_EXT)(hr->pAdapterDevExt))->IOCtlInProgress = FALSE;
	up(&hr->platform_data->ioctl_sem);
} /* XPostApiIOCtlDonex() - end */


/**************************** Interrupt Processing ****************************/
/*
 * Driver interrupt handler entry point. Called by the kernel when a
 * host adapter interrupt occurs.
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
void
#else
irqreturn_t
#endif
hr_isr(int vec, void *arg, struct pt_regs *regs)
{
	struct hr_softc	*hr;
	PVOID pADE;
	u_long		 flags;
	uint8_t          interruptAcknowledged;

	interruptAcknowledged = FALSE;
	UNREFERENCED_PARAMETER(vec);
	UNREFERENCED_PARAMETER(regs);

	hr = (struct hr_softc *)arg;
	pADE = hr->pAdapterDevExt;

	/*
	 * Extract the Adapter Device and ask it
	 * to look for interrupts to service.
	 */
	hr_lock(hr, &flags);

	interruptAcknowledged =	CdaIoIsr(pADE);

	/*
	 * Now that we've serviced an interrupt, we
	 * may be now be able to issue more I/O.  Kick
	 * the queues.  We do this from a tasklet so
	 * operate from interrupt context (and thus
	 * defer other interrupts) for as short as short
	 * a period as possible.
	 */
	if (hr_next_device_to_run(hr) != NULL)
                hr_schedule_runq(hr);
	hr_unlock(hr, &flags);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	return interruptAcknowledged;
#endif
}

/*
 * Handle an OSD completion from the CDA Core.  We convert/copy OSD
 * status into the OSes SCSI command structure and return status to
 * the OS.  Since Linux defers all completions via a bottom half handler,
 * there is no need to drop our lock or defer these completions to
 * another context.
 */
static void
hr_PostOSD(POSD pOSD)
{
	struct hr_softc		*hr;
	struct hr_cmd		*hcmd;
	struct hr_io_request	*io_request;
	struct hr_linux_device	*dev;
	Scsi_Cmnd		*cmd;

	/*
	 * Convert OSD status to Linux status.  "result"
	 * is a 32bit integer separated into 4 different
	 * byte codes:
	 *
	 * 0:	SCSI Status Byte
	 * 1:	SCSI Message Byte - Always 0 -> Command Complete
	 * 2:	Host Status Byte
	 * 3:	Driver Status Byte - Flags for status available, etc.
	 */
	io_request = (struct hr_io_request *) pOSD->pOSRequest;
	hcmd = io_request->hcmd;
	dev = io_request->dev;
	hr = io_request->pAdapter;
	cmd = &hcmd_scsi_cmd(hcmd);

	/*
	hr_log(HR_RUNTIME_DEBUG, 2, "\n%s:%d - CMD status = 0x%x cdb0=0x%x "
		 "cdb1=0x%x cdb2=0x%x cdb3=0x%x\n", __FUNCTION__, __LINE__,
		 pOSD->OSD_Status,cmd->cmnd[0], cmd->cmnd[1],
		cmd->cmnd[2],cmd->cmnd[3]);
	*/

	/* cmd is back from CDA */
	io_request->flags &= ~HR_IOREQ_ACTIVE;	

	dev->active--;
	TAILQ_REMOVE(&hr->io_request_busyq, io_request, links);
	hr_unmap_data(hr, pOSD, cmd);

	/*
         * Guard against stale sense data.  The Linux mid-layer
	 * assumes that sense was retrieved anytime the first byte
	 * of the sense buffer looks "sane".
         */     
	cmd->sense_buffer[0] = 0;
	cmd->resid = 0;

	switch(pOSD->OSD_Status) {
	case SCSI_STAT_GOOD:
	case SCSI_STAT_CONDMET:
	case SCSI_STAT_INTER:
	case SCSI_STAT_INTERCONDMET:
	case SCSI_STAT_RESCONFLICT:
	case SCSI_STAT_BUSY:
	case SCSI_STAT_QUEUEFULL:
		cmd->result = (DID_OK << 16) | pOSD->OSD_Status;
		dev->incr_timeout = 0;
		break;
	
	case SCSI_STAT_CHECKCOND:
	case SCSI_STAT_CMDTERM:
	{
		u_int sense_size;

		/*
		 * Record the status byte.
		 */
		cmd->result = (DID_OK << 16) | pOSD->OSD_Status;

		/*
                 * Copy sense information to the OS's cmd
                 * structure if it is available.  We don't
		 * have sense residual information available,
		 * so we always keep the sense buffer cleared
		 * out so that copying extra bytes to the OS
		 * is harmless.
                 */
		sense_size = MIN(pOSD->OSD_SenseLen, sizeof(cmd->sense_buffer));
		
		memcpy(cmd->sense_buffer, pOSD->pOSD_Sense, sense_size);
		if (sense_size < sizeof(cmd->sense_buffer))
			memset(&cmd->sense_buffer[sense_size], 0,
			       sizeof(cmd->sense_buffer) - sense_size);
		memset(&pOSD->OSD_Sense[0], 0, pOSD->OSD_SenseLen);
		cmd->result |= (DRIVER_SENSE << 24);
		/*
		hr_log(HR_RUNTIME_DEBUG, 1, "\npOSD: %p, status = 0x%x "
		 "cmd->result=0x%x, sb0=0x%x sb2=0x%x sb12=0x%x sb13=0x%x\n", 
		pOSD, pOSD->OSD_Status,cmd->result, (char)cmd->sense_buffer[0],
		(char)cmd->sense_buffer[2],(char)cmd->sense_buffer[12],
						(char)cmd->sense_buffer[13]);
		*/
		break;
	}
	
	case CDA_ASC_ABORTED_ON_REQUEST:
		cmd->result = (DID_ABORT << 16);
		break;
	
	case SCSI_STAT_GENERATED:
	{
		pSCSISenseData	pOSD_Sense;
					
		pOSD_Sense = (pSCSISenseData) pOSD->pOSD_Sense;
		/*
		 * Sense information contains HIM error code data.
		 */
		if (pOSD_Sense->SenseKey != KEY_HOST_ERROR) {
			/*
			 * Shouldn't happen???  Just complete
			 * without error.
			 */
			cmd->result = (DID_OK << 16);
			break;
		}

		switch (pOSD_Sense->AddSenseCode) {
		case CDA_ASC_DATA_OVERRUN:
			cmd->result = (DID_OK << 16);
			break;
		
		case HIM_IOB_DATA_UNDERRUN:
		{
			unsigned long resid;
			resid = *((PULONG)(&(pOSD_Sense->INFO[0])));
			cmd->result = (DID_OK << 16);
			cmd->resid = resid;
			break;
		}
		
		case CDA_ASC_NO_RESPONSE:
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
			cmd->result = (DID_NO_CONNECT << 16);
#else
			hr_scsi_offline(cmd);
#endif
			dev->qfrozen = 0;
			break;

		case CDA_ASC_CONNECTION_FAILED:
			cmd->result = (DID_TIME_OUT << 16);
			break;
		//mdt case CDA_ASC_PARITY_ERROR:
		//mdt	cmd->result = (DID_PARITY << 16);
		//mdt	break;
#if _NOTYET_
		case HIM_IOB_TARGET_RESERVED:
			/*
			 * Reservation conflict.  Report via
			 * nomal SCSI status.
			 */
			cmd->result = (DID_OK << 16) | SCSI_STAT_RESCONFLICT;
			break;
#endif
		case CDA_ASC_ABORT_FAILED:
		default:
			cmd->result = (DID_ERROR << 16);
			break;
		}
		break;
	}

	default:
    		printk("PostOSD: DID_ERROR opcode 0x%x, OSD_Status 0x%x\n",
		 			cmd->cmnd[0], pOSD->OSD_Status);
		cmd->result = (DID_ERROR << 16);
		break;
	}

	if ((dev->flags & HR_DEV_HOT_REMOVED) != 0) {
		/*
	 	 * If the device has been removed and all active IOs on 
		 * the device have been completed, schedule the device to
		 * be destroyed.
		 */
		if (TAILQ_EMPTY(&dev->busyq) && (dev->active == 0) && 
		   ((dev->flags & HR_DEV_DESTROY_WAS_ACTIVE) != 0)) {
			/*
			 * Schedule a deferred process task to destroy
		         * the device.
			 */	 
			hr_setup_dev_dpc_task(dev, hr_destroy_device);
		}
	} 
	
	/*
	 * remove our ioreq timer before posting.
 	 */
	if ((io_request->flags & HR_IOREQ_TIMER_ACTIVE) != 0)
		hr_ioreq_delete_timer(io_request);

	/*
	 * Only free the io req if it has NOT timedout.
         * For io req that has timedout, error recovery is invoked and
	 * the timedout SCB will be free in the error recovery path.
	 */	 
	if ((io_request->flags & (HR_IOREQ_TIMEDOUT | HR_IOREQ_RESET_LU)) == 0)
	{
		hr_free_ioreq(hr, io_request);
	}

	/*
	 * Now that we've free'ed up an OSD, let the queue
	 * run if it was held up waiting for OSDs.
	 */
	if ((hr->flags & HR_RESOURCE_SHORTAGE) != 0) {
		hr->flags &= ~HR_RESOURCE_SHORTAGE;
		hr->qfrozen--;
	}

	cmd->scsi_done(cmd);

}


/*
 * Mark a command as needing to be requeued.  Due to numerous bugs in the
 * Linux SCSI layer, this is not as straight-forward as it might seem.
 * What really happens is that we lie and say that the device aborted the
 * command.  This will trigger Linux to requeue it.  The requeue is held
 * off by setting scsi_block_requests() whenever our controller queue is
 * frozen.
 */
static void
hr_scsi_requeue(struct hr_softc *hr, Scsi_Cmnd *cmd)
{

	cmd->result = 0x02; /* SCSI_STATUS_CHECK_COND */
	cmd->result |= DID_OK << 16;
	cmd->result |= DRIVER_SENSE << 24;
	memset(cmd->sense_buffer, 0, sizeof(cmd->sense_buffer));
	cmd->sense_buffer[0] = SSD_ERRCODE_VALID | SSD_CURRENT_ERROR;
	cmd->sense_buffer[2] = SSD_KEY_ABORTED_COMMAND;
	cmd->scsi_done(cmd);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static void
hr_scsi_offline(Scsi_Cmnd *cmd)
{
	cmd->result = DID_NO_CONNECT << 16;
	cmd->result |= DRIVER_SENSE << 24;
	memset(cmd->sense_buffer, 0, sizeof(cmd->sense_buffer));
	cmd->sense_buffer[0] = SSD_ERRCODE_VALID | SSD_CURRENT_ERROR;
	cmd->sense_buffer[2] = SSD_KEY_NOT_READY;
}
#endif


/**********************************************************************

Function List of tables OS uses to interface to the driver methods.

Function Description
	Reboot notification, Passthrough node and Scsi layer tables.

Input Parameters
	lock_handle	handle used to unlock the code.

Output Parameters
	None

**********************************************************************/

static struct notifier_block hr_reboot_notifier_block = {
	hr_reboot_notifier
}; /* hr_reboot_notifier_block - end */

static struct file_operations hr_file_operations = {
	ioctl:           hr_ioctl,
	open:            hr_open,
#ifdef CONFIG_COMPAT
#if	HAVE_COMPAT_IOCTL
	compat_ioctl:	 hr_compat_ioctl,
#endif
#endif
	release:         hr_close,
};

/****************************** PCI Entry Points ******************************/
static int
hr_pci_dev_probe(struct pci_dev *pdev, const struct pci_device_id *entry)
{
	char		 buf[80];
	struct hr_softc	*hr;
	hr_dev_softc_t	 dev;
	struct hr_pci_driver_data  *driver_data;
	char		*name;
	int		 error;

	hr_log(HR_BOOT_DEBUG, 1, "hr_pci_dev_probe: Enter\n");
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	if (hr_detect_complete) {
		/*
		 * XXX Verify that we cannot safely register
		 *     in a 2.4.X kernel.
		 */
		printk(HR_DRIVER_NAME ": ignoring PCI device found after "
		       "initialization\n");
		return (-ENODEV);
	}
#endif

	/*
	 * Some BIOSen report the same device multiple times.
	 */
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		struct pci_dev *probed_pdev;

		probed_pdev = hr_pci_dev(hr);
		if (probed_pdev->bus->number == pdev->bus->number
		 && probed_pdev->devfn == pdev->devfn)
			break;
	}
	if (hr != NULL) {
		/* Skip duplicate. */
		hr_log(HR_BOOT_DEBUG, 1, "hr_pci_dev_probe: Skip Dup\n");
		return (-ENODEV);
	}

	dev = hr_pci_dev_to_dev(pdev);
	driver_data = (struct hr_pci_driver_data *)(entry->driver_data);
	if (driver_data == NULL) {
		hr_log(HR_BOOT_DEBUG, 1, "hr_pci_dev_probe: No Driver Data\n");
		return (-ENODEV);
	}

	/*
	 * Allocate a softc for this card and
	 * set it up for attachment by our
	 * common detect routine.  Here we
	 * provide a temporary name for our
	 * instance that will later be replaced
	 * by the "scsi name" the OS gives to
	 * this device.
	 */
	sprintf(buf, "hr_pci:%d:%d:%d",
		hr_get_pci_bus(dev),
		hr_get_pci_slot(dev),
		hr_get_pci_function(dev));
	name = kmalloc(strlen(buf) + 1, GFP_ATOMIC);
	if (name == NULL) {
		hr_log(HR_BOOT_DEBUG, 1,
		       "hr_pci_dev_probe: Name malloc failed\n");
		return (-ENOMEM);
	}
	strcpy(name, buf);
	hr = hr_alloc(name, dev);
	if (hr == NULL) {
		/*
		 * hr_alloc takes ownership of the name
		 * string even if it fails, so don't free
		 * something that is already free'd here.
		 */
		hr_log(HR_BOOT_DEBUG, 1,
		       "hr_pci_dev_probe: hr_alloc failed\n");
		kfree(name);
		return (-ENOMEM);
	}

	hr->pci_entry = entry;
	hr->dev_softc = dev;

	/*
	 * Perform controller specific setup.
	 */
	error = driver_data->setup(hr);
	if (error != 0) {
		hr_free(hr);
		return (-ENODEV);
	}
	TAILQ_INSERT_TAIL(&hr_tailq, hr, links);

	if (pci_enable_device(pdev)) {
		hr_free(hr);
		return (-ENODEV);
	}
	pci_set_master(pdev);

	/*
	 * Setup the dma transfer mask to 64-bit only the following conditions
	 * are met:
	 *    (a) the system is running on 64-bit or 'PAE' mode.
	 *    (b) the controller can do DMA transfer above 4GB boundary.
	 *    (c) dma address can be above 4GB boundary.
	 * Otherwise set the dma transfer mask to 32-bit.
	 */
	 if ((hr->dma64_addr == 1) && (HR_DMA_64BIT_SUPPORT == 1)) {
		uint64_t    mask;
		
		mask = 0xFFFFFFFFFFFFFFFFULL;
	 	if (hr_set_dma_mask(hr, mask) != 0) {
			/* 
			 * Failed to set dma mask to 64-bit, throttle down
			 * to 32-bit instead.
			 */
			 mask = 0xFFFFFFFFULL;
			 if (hr_set_dma_mask(hr, mask) != 0) {
			 	/* If this also failed, we need to exit. */
				hr_log(HR_BOOT_DEBUG, 1, "Failed to set DMA "
					"mask. \n");
				hr_free(hr);
				return (-ENODEV);
			}	 
	 	}
		hr->hw_dma_mask = (dma_addr_t) mask;
	 } else {
	 	if (hr_set_dma_mask(hr, 0xFFFFFFFF) != 0) {
			hr_log(HR_BOOT_DEBUG, 1, "Failed to set DMA mask. \n");
			hr_free(hr);
			return (-ENODEV);
		}
		hr->hw_dma_mask = 0xFFFFFFFF;
	}

	/*
	 * Since we have not registered our interrupt
	 * handler yet, and the mid-layer doesn't know
	 * anything about this instance, there cannot
	 * be any driver re-entrance for this instance
	 * up to this point.  hr_config() will register
	 * our interrupt handler, so pick up the lock.
	 */
	//mdtmdt hr_lock(hr, &flags);
	error = hr_config(hr);
	if (error != 0) {
		printk("hr_pci_dev_probe: hr_config error %d\n", error);
		//mdtmdt hr_unlock(hr, &flags);
		hr_free(hr);
		return (-ENODEV);
	}
//mdtmdt	hr_unlock(hr, &flags);

	pci_set_drvdata(pdev, hr);

	error = request_irq(hr_pci_dev(hr)->irq, hr_isr,
			    SA_SHIRQ, HR_DRIVER_NAME, hr);
	if (error != 0) {
		hr_log(HR_BOOT_DEBUG, 1, "=FAIL\n");
		hr_log(HR_BOOT_DEBUG, 0, "Could not register irq %d\n",
		       hr_pci_dev(hr)->irq);
		hr_free(hr);
		return (-ENODEV);
	}
	hr->flags |= HR_IRQ_ALLOCATED;

	return (error);
}

static void
hr_pci_dev_remove(struct pci_dev *pdev)
{
	struct hr_softc *hr;

	/*
	 * We should be able to just perform
	 * the free directly, but check our
	 * list for extra sanity.
	 */
	hr = hr_find_softc((struct hr_softc *)pci_get_drvdata(pdev));
	if (hr != NULL) {
		hr_free(hr);
	}
}
/*************************** Device Queue Management **************************/
/*
 * Dispatch commands to the Core for this device until we have
 * exceeded the limit of commands that we are allowed to send or
 * the device becomes frozen.
 */
static void
hr_run_device_queue(struct hr_softc *hr, struct hr_linux_device *dev)
{
	struct hr_cmd	 	*hcmd;
	struct scsi_cmnd 	*cmd;
	struct hr_io_request	*io_request;
	int		  	error;

	HR_LOCK_ASSERT(hr);

	if ((dev->flags & HR_DEV_ON_RUN_LIST) != 0)
		panic("running device on run list");
	
	while ((hcmd = TAILQ_FIRST(&dev->busyq)) != NULL && dev->qfrozen == 0) {
		POSD	pOSD;

		/*
		 * Schedule us to run later.  The only reason we are not
		 * running is because the whole controller Q is frozen.
		 */
		if (hr->qfrozen != 0) {
			TAILQ_INSERT_TAIL(&hr->device_runq, dev, links);
			dev->flags |= HR_DEV_ON_RUN_LIST;

			hr_log(HR_RUNTIME_DEBUG, 1, "%s, hr->qfrozen: %x\n",
						 __FUNCTION__, hr->qfrozen);

			return;
		}

		cmd = &hcmd_scsi_cmd(hcmd);

		TAILQ_REMOVE(&dev->busyq, hcmd, hcmd_links.tqe);

		if ((dev->flags & (HR_DEV_HOT_REMOVED|HR_DEV_OFFLINE)) != 0) {
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
			cmd->result = (DID_NO_CONNECT << 16);
#else
			hr_scsi_offline(cmd);
#endif

			hr_log(HR_RUNTIME_DEBUG, 1, "%s, dev removed or "
			 "offline, complete cmd: %p\n", __FUNCTION__, hcmd);

			cmd->scsi_done(cmd);
			continue;
		}

		/*
		 * Before dequeueing this command ensure that
		 * we have sufficient resources to queue it to
		 * the core.  We do not support splitting I/O
		 * across OSDs, so only a single OSD is required.
		 */
		io_request = hr_get_ioreq(hr);
		if (io_request == NULL) {
			/*
			 * Re-queue this device to wait for OSD
			 * resources to become available.
			 */

			hr_log(HR_RUNTIME_DEBUG, 1, "%s, out of io_request\n",
								 __FUNCTION__);

			TAILQ_INSERT_TAIL(&dev->busyq, hcmd, hcmd_links.tqe);
			TAILQ_INSERT_HEAD(&hr->device_runq,
					  dev, links);
			dev->flags |= HR_DEV_ON_RUN_LIST;
			hr->flags |= HR_RESOURCE_SHORTAGE;
			hr->qfrozen++;
			return;
                }

		/*
		 * Fill in OSD fields.
		 */ 
		cmd->host_scribble = (char *) io_request;
		io_request->hcmd = hcmd;
		io_request->pAdapter = hr;
		io_request->dev = dev;
		pOSD = &(io_request->OSD); 
		pOSD->pOSRequest = (PVOID) io_request;
		pOSD->OSD_pSGTable = &(io_request->SGPool);
		pOSD->OSD_Status = 0;
		pOSD->OSD_SenseLen = sizeof(cmd->sense_buffer);
		pOSD->pOSD_Sense = &pOSD->OSD_Sense[0];
		pOSD->pOSD_Post = &hr_PostOSD;
		pOSD->OSD_DeviceExtension = hr->pAdapterDevExt;
		pOSD->OSD_DeviceHandle = dev->handle; 		
		pOSD->cOSD_TCount = 6; 
		pOSD->MemberIOStat = RESET_MEMBER_IO_STAT; 

		/*
		 * XXX What about real timeout values?
		 */
		pOSD->OSD_RetryCount = 3;

		switch (cmd->sc_data_direction) {
		case SCSI_DATA_WRITE:
			pOSD->OSD_Direction = CDA_DIRECTION_DATA_OUT;
			break;
		case SCSI_DATA_READ:
			pOSD->OSD_Direction = CDA_DIRECTION_DATA_IN;
			break;
		case SCSI_DATA_NONE:
		case SCSI_DATA_UNKNOWN:
		default:
			//mdt pOSD->OSD_Direction = IONone;
			pOSD->OSD_Direction = CDA_DIRECTION_BIDIRECTIONAL;
			break;
		}

		/*
		 * Initialize S/G Data.
		 */
		error = hr_map_data(hr, pOSD, cmd);
		if (error != 0) {
		

			hr_log(HR_RUNTIME_DEBUG, 1, "%s, error: hr_map_data, "
						"cmd: %p\n", __FUNCTION__, cmd);

			cmd->result = DID_ERROR << 16;
			cmd->scsi_done(cmd);
			hr_free_ioreq(hr, io_request);
			continue;
		}
		
		memcpy(pOSD->OSD_CDB, cmd->cmnd, cmd->cmd_len);
		pOSD->OSD_cCDB = cmd->cmd_len;
		pOSD->OSD_StartBlock = 0;
		pOSD->cOSD_Blocks = 0;
		pOSD->OSD_SenseLen = CDA_MAX_SENSE_LENGTH;

		/* Add the IO Req to the busy q.*/
		TAILQ_INSERT_TAIL(&hr->io_request_busyq, io_request, links);

		/* Start our own internal timer. */
		hr_start_ioreq_timer(io_request);

		/* cmd is sent to CDA */
		io_request->flags |= HR_IOREQ_ACTIVE;	

		CdaExecuteOSD(pOSD);
		dev->commands_issued++;
		dev->active++;
	}
}

/*
 * Map OS specified data into buffers accessible by
 * our device, formatted for consumption by the CDA Core.
 */
static int
hr_map_data(struct hr_softc *hr, POSD pOSD, struct scsi_cmnd *cmd)
{
	SGElement *curSG;

	curSG = &(((SGTable *) pOSD->OSD_pSGTable)->SGList[0]);
	
	if (cmd->use_sg != 0) {
		struct scatterlist *cur_seg;
		u_int		    nseg;
		int		    dir;
		cur_seg = (struct scatterlist *)cmd->request_buffer;

		dir = scsi_to_pci_dma_dir(cmd->sc_data_direction);
		nseg = hr_map_sg(hr, cur_seg, cmd->use_sg, dir);
		if (nseg > OSM_SG_MAX)
			return (ENOMEM);

		if((cur_seg->page) && (!pOSD->OSD_pDataBuffer)) {
				pOSD->OSD_pDataBuffer = 
				(DATA_N_PTR *)(page_address(cur_seg->page) + 
							cur_seg->offset);
		}


		((SGTable *) pOSD->OSD_pSGTable)->SGCount = nseg;

		for (; nseg > 0; nseg--, cur_seg++, curSG++) {

			curSG->SGLBufAddr = sg_dma_address(cur_seg);
			curSG->SGLByteCnt = sg_dma_len(cur_seg);

		}

	} else if (cmd->request_bufflen != 0) {
		int	   dir;

		dir = scsi_to_pci_dma_dir(cmd->sc_data_direction);
		((SGTable *) pOSD->OSD_pSGTable)->SGCount = 1;
		curSG->SGLBufAddr = hr_map_single(hr, cmd->request_buffer,
						  cmd->request_bufflen, dir);
		curSG->SGLByteCnt = cmd->request_bufflen;
                pOSD->OSD_pDataBuffer = cmd->request_buffer;
	} else {
		((SGTable *) pOSD->OSD_pSGTable)->SGCount = 0;
		curSG->SGLBufAddr = 0;
		curSG->SGLByteCnt = 0;
	}
	return (0);
}

static int
hr_find_ioreq_in_busyq(struct hr_softc *hr, struct hr_io_request *ioreq)
{
	struct hr_io_request	*tmp_ioreq;
	int			 found;

	found = FALSE;
	TAILQ_FOREACH(tmp_ioreq, &hr->io_request_busyq, links) {
		if (tmp_ioreq == ioreq) {
			found = TRUE;
			break;
		}
	}

	return (found);
}

void
hr_handle_tmf_req(struct hr_softc *hr, POSD pOSD, u_int event, u_short dev_flag)
{
	struct hr_io_request	*ioreq;

	printk("hr_handle_tmf_req: event 0x%x\n", event);
	switch (event) {
	case CDA_NOTIFY_EVENT_ABORT_TASK:
	{
		if (pOSD == NULL) {
			printk("hr_handle_tmf_req: pOSD is NULL\n");
			break;
		} else {
		struct scsi_cmnd 	*cmd;

		/* IO Req to be aborted. */
		ioreq = (struct hr_io_request *) pOSD->pOSRequest;
		if (ioreq == NULL) {
			hr_log(HR_ERROR_DEBUG, 1, "Corrupted ioreq!\n");
			/* This shouldn't happen. Corrupted OSD !! */
			break;
		}

		cmd = &hcmd_scsi_cmd(ioreq->hcmd);

		if (hr_find_ioreq_in_busyq(hr, ioreq) == FALSE) {
			/* IO Req is not the busyq. Corrupted IO Req!! */
			hr_log(HR_ERROR_DEBUG, 1,
			       "Failed to find ioreq in busyq.\n");
			break;
		}

		/*
	 	 	 * Set the level of error recovery for the error handler
		 	 * thread to perform.
	 	 	 */
			ioreq->eh_state = HR_EH_ABORT_REQ; //iob request abort
			ioreq->eh_post = hr_handle_tmf_req_done;
			ioreq->flags |= HR_IOREQ_TIMEDOUT;

			hr_log(HR_RUNTIME_DEBUG, 1, "enqueue timeoutq, ioreq: "
								 "%p\n", ioreq);

			TAILQ_INSERT_TAIL(&hr->io_request_timedoutq, ioreq,
					  timedout_links);
			/* Wake up the error handler thread. */
			up(&hr->platform_data->ehandler_sem);

		break;
	}
	}

	case CDA_NOTIFY_EVENT_RESET_DEVICE_PORT:
	{
		hr_log(HR_RUNTIME_DEBUG, 1,"RESET DEV PORT"
					" EVENT: pOSD: %p\n", pOSD);
		
		if (pOSD == NULL) {
			struct hr_linux_device	*dev;
			uint16_t		 path_id;
			uint16_t		 target_id;

			/*
		   	 * This is just a notification from CDA telling us
	      		 * that Reset Device Port is initiated by CDA. 
			 */
			path_id = (dev_flag >> 8);
			target_id = (dev_flag & 0x00FF);

			/* 
			 * This won't work for multi-lun device as we
			 * assume it is on LUN 0.
			 * CDA need to fix this!!
			 */
			dev = hr->device_table[path_id][target_id][0];

		    hr_log(HR_RUNTIME_DEBUG, 1, "dev: %p, pid: %x, t_id: %x\n",
						 dev, path_id, target_id);

			if ((dev == NULL) || 
			    (dev->flags & HR_DEV_TIMER_ACTIVE) != 0)
				break;

			if (((dev->flags & HR_DEV_IN_RECOVERY) != 0) ||
			(!TAILQ_EMPTY(&hr->io_request_timedoutq))) {
		    		printk("hr_handle_tmf_req: dev: %p, pid: %x,"
				 " t_id: %x, pending recovery -> do nothing\n",
				 dev, path_id, target_id);
				break;
			}

			/*
			 * Ideally, we would like to freeze the device q and
			 * and resume upon the reset completion, but CDA
			 * doesn't have mechanism to notify us when the
			 * reset is completed.
			 * So, we just make the assumption that the reset
			 * will be done in 5 secs.
			 */
			dev->qfrozen++;
      		} else {
			struct hr_linux_device	*dev;
			ioreq = (struct hr_io_request *) pOSD->pOSRequest;
			dev = ioreq->dev;

			if (ioreq == NULL) {
				hr_log(HR_ERROR_DEBUG, 1, "Corrupted ioreq!\n");
				/* This shouldn't happen. Corrupted OSD !! */
				break;
			}
		
			if (hr_find_ioreq_in_busyq(hr, ioreq) == FALSE) {
				hr_log(HR_ERROR_DEBUG, 1,
					"Failed to find ioreq in busyq.\n");
				break;
			}
			if (((dev->flags & HR_DEV_IN_RECOVERY) != 0) ||
			(!TAILQ_EMPTY(&hr->io_request_timedoutq))) {
		    		printk("hr_handle_tmf_req: dev: %p,"
				" pending recovery -> do nothing2\n", dev);
				break;
			}
			/*
	 	 	 * Set the level of error recovery for the error handler
		 	 * thread to perform.
	 	 	 */
			ioreq->eh_state = HR_EH_DEV_RESET_REQ;
			ioreq->eh_post = hr_handle_tmf_req_done;
			ioreq->flags |= (HR_IOREQ_RESET_LU | HR_IOREQ_TIMEDOUT);

			hr_log(HR_RUNTIME_DEBUG, 1, "enqueue timeoutq, "
							 "ioreq: %p\n", ioreq);

			TAILQ_INSERT_TAIL(&hr->io_request_timedoutq, ioreq,
					  timedout_links);
			/* Wake up the error handler thread. */
			up(&hr->platform_data->ehandler_sem);
		}
		break;
	}

	case CDA_NOTIFY_EVENT_RESET_OCCURING_ON_ADAPTER:
		hr_log(HR_ERROR_DEBUG, 1, "RESET OCCURING ON ADAPTER EVENT.\n");
		
		break;
	
	default:
		hr_log(HR_ERROR_DEBUG, 1, "Unhandled TMF Event!!\n");
		break;
	}

	return;
}

static void
hr_handle_tmf_req_done(struct hr_softc *hr, struct hr_io_request *ioreq)
{
	hr_log(HR_RUNTIME_DEBUG, 1, "TMF done,  ioreq: %p.\n", ioreq);
}


static void
hr_ioreq_timeout(Scsi_Cmnd *cmd)
{
	struct hr_softc		*hr;
	struct hr_linux_device	*dev;
	struct hr_cmd		*hcmd;
	struct hr_io_request	*ioreq;
	u_long			 flags;
	int			 found;

	hr = *(struct hr_softc **) cmd->device->host->hostdata;

	hr_lock(hr, &flags);

	dev = hr_get_ldev(hr, cmd);
	if (dev == NULL) {
		printk("(scsi%d: Ch %d Id %d Lun %d): ",
			  cmd->device->host->host_no, cmd->device->channel,
			  cmd->device->id, cmd->device->lun);
		printk("Is not an active device.\n");
		goto exit;
	}

	found = FALSE;
	/* 
	 * Check if the cmd is still pending in the device busyq which
	 * means it hasn't been submitted yet. 
	 */
	TAILQ_FOREACH(hcmd, &dev->busyq, hcmd_links.tqe) {
		if (&hcmd_scsi_cmd(hcmd) == cmd) {
			/* Found it. */
			found = TRUE;
            break;
    }
	}
	if (found == TRUE) {
		/* 
		 * Command to be aborted still pending in the device
		 * busy queue.
		 */
		printk("%s, (scsi%d: Ch %d Id %d Lun %d): ",
			  __FUNCTION__, cmd->device->host->host_no,
				 cmd->device->channel, cmd->device->id,
						 cmd->device->lun);
		printk("Cmd %p found on device queue.\n", cmd);
		TAILQ_REMOVE(&dev->busyq, hcmd, hcmd_links.tqe);
		cmd->result = DID_ABORT << 16;	
		cmd->scsi_done(cmd);	
		goto exit;
	}

	/* When sending abort, cda/chim could return all outstanding cmd */
	/* Therefore we want only one abort at a time.  Restart timer */
	/* if Abort pending or there is one in timedoutq .MDT. */

	ioreq = (struct hr_io_request *) cmd->host_scribble;


	if ((ioreq->flags & HR_IOREQ_TIMER_ACTIVE) != 0) {

		ioreq->flags &= ~HR_IOREQ_TIMER_ACTIVE;

		if((ioreq->flags & (HR_IOREQ_TIMEDOUT | HR_IOREQ_RESET_LU)) ||
						(ioreq->dev->qfrozen != 0)) {
	        	printk("hr_ioreq_timedout: cmd 0x%p ioreq is already "
					"in EH, give it another timer\n",cmd);
			hr_start_ioreq_timer(ioreq);
			goto exit;
		}

		hr_log(HR_RUNTIME_DEBUG, 1, "%s, abort req: ioreq: %p.\n",
							 __FUNCTION__, ioreq);
	       printk("hr_ioreq_timedout:cmd 0x%p (%d,%d,%d) "
		 "opcode 0x%x: Enter\n", cmd, cmd->device->channel,
		 cmd->device->id, cmd->device->lun,cmd->cmnd[0]);

		/* Set the error recovery engine to reset lu this cmd. */
		/* @mdt Abort on a timed out sata is not going to work */
		/* since chim will start IT_NEXUS loss timer for sata */
		ioreq->eh_state = HR_EH_LU_RESET_REQ;	
		/* iob request reset or cmd time expires */
		ioreq->flags |= (HR_IOREQ_TIMEDOUT | HR_IOREQ_RESET_LU);

		ioreq->eh_post = hr_handle_tmf_req_done;

		/* Add the IO request to the timedout queue. */
		TAILQ_INSERT_TAIL(&hr->io_request_timedoutq, ioreq,
				  timedout_links);

		/* Wake up the error handler thread. */
		up(&hr->platform_data->ehandler_sem);
	}

exit:
	hr_unlock(hr, &flags);
    return;
}

/***************************** Watchdog Functions ******************************/
/*
 * Handler for the periodic watchdog event timer.
 * Softc list lock is held on entry.
 */
void
hr_watchdog(struct hr_softc *hr)
{
	u_long	flags;

	hr_lock(hr, &flags);
	if ((hr->flags & HR_TEARDOWN_IN_PROGRESS ) != 0) {
		hr_unlock(hr, &flags);
		return ;
	}
	CdaTimerTask(hr->pAdapterDevExt);
	hr_unlock(hr, &flags);
	/* give other spinning threads a chance */
	hr_lock(hr, &flags);
	if(!(hr->flags & HR_SUSPEND_IO)) {
		struct hr_linux_device	*dev;
		int i,j;
		for(i=0;i<OSM_MAX_CHANNEL;i++)
	    		for(j=0;j<OSM_MAX_TARGET_PER_PATH;j++) {
				dev = (struct hr_linux_device *)
						 hr->device_table[i][j][0];
				if(dev != NULL)
				if ((hr->qfrozen == 0) && 
				(!(hr->flags & HR_SUSPEND_IO)) && 
				(dev->flags & HR_DEV_ON_RUN_LIST) == 0) {
					TAILQ_INSERT_TAIL(&hr->device_runq,
								 dev, links);
					dev->flags |= HR_DEV_ON_RUN_LIST;
					hr_run_device_queues(hr);
				}
			}
		hr->flags &= ~HR_RESUME_RUNQ;
	}
	hr_unlock(hr, &flags);
}

/*
 * This watchdog thread is to monitor any ongoing IO(s) or IOCTL(s) that
 * might timedout.
 */
static int
hr_watchdog_thread(void *data)
{
	struct hr_softc *hr;

	hr = (struct hr_softc *) data;

	lock_kernel();

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,60)
	/* Don't care about any signals. */
	siginitsetinv(&current->blocked, 0);
	daemonize();
	sprintf(current->comm, "hr_wd_%d", hr->unit);
#else
	daemonize("hr_wd_%d", hr->unit);
	current->flags |= PF_FREEZE;
#endif

	unlock_kernel();
	
	while (1) {
		/* Check if we've been signaled to exit. */
		if (hr_driver_unloading)
			break;

		hr_sleep(&hr_watchdog_sem, HR_WATCHDOG_PERIOD * HZ);
		TAILQ_FOREACH(hr, &hr_tailq, links) {
			if ((hr->flags & HR_CORE_INITIALIZED) != 0)
				hr_watchdog(hr);
		}
	}

	up(&hr_watchdog_ending_sem);
	return (0);
}

/************************* Host Template Entry Points *************************/
/*
 * Entry point for read and write operations to
 * our driver node in the procfs filesystem.
 */
static int
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
hr_proc_info(char *proc_buffer, char **proc_start, off_t proc_offset,
	     int proc_length, int proc_hostno, int proc_out)
{
	struct	hr_softc  *hr;
	struct	Scsi_Host *host;
#else
hr_proc_info(struct Scsi_Host *host, char *proc_buffer, char **proc_start,
	     off_t proc_offset, int proc_length, int proc_out)
{
#endif
	char	mybuf[256];
	int	write_len;

	/*
	 * We do not support writes.
	 */
	if (proc_out == TRUE)
		return (-ENOSYS);

	/*
	 * The position where the requested data can
	 * be found (i.e. offset bytes into our total
	 * output) is at the start of the buffer since
	 * we never write more data than is requested.
	 */
	if (proc_start != NULL)
		*proc_start = proc_buffer;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	host = NULL;
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr->host->host_no == proc_hostno) {
			host = hr->host;
			break;
		}
	}
#endif

	if (host == NULL)
		return (-ENODEV);

	write_len = sprintf(mybuf, "%s\n%s\nMajor: %d\nBuild Date: %s, time: %s\n", DRIVERDESC, hr_info(host),
						 hr_major, __DATE__, __TIME__);
	write_len = MIN(proc_length, write_len - proc_offset);

	/*
	 * write_len will be negative for attempts to read
	 * beyond the last byte of our generated data.
	 * Don't treat this or 0 byte requests as errors.
	 */
	if (write_len <= 0)
		return (0);
	memcpy(proc_buffer, &mybuf[proc_offset], write_len);
	return (write_len);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static int
#else
/*
 * Detect our hardware.
 *
 * In 2.4.X, this entry point is called directly from the Linux
 * SCSI mid-layer after we register our module.  In 2.5.X, we
 * call this routine ourselves via our module initialization
 * routine.
 */
int
#endif
hr_detect(Scsi_Host_Template *unused)
{
	struct	hr_softc *hr;
	int	error;
	int	found;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	/*
	 * It is a bug that the upper layer takes
	 * this lock just prior to calling us.
	 */
	spin_unlock_irq(&io_request_lock);
#endif

	found = 0;
	/*
	 * Sanity checking of Linux SCSI data structures so
	 * that some of our hacks^H^H^H^H^Hassumptions aren't
	 * violated.
	 */
	if (offsetof(struct hr_cmd_internal, end) > 
	    offsetof(struct scsi_cmnd, host_scribble)) {
		printk("hr_detect: SCSI data structures changed.\n");
		printk("hr_detect: Unable to attach\n");
		hr_init_status = -EINVAL;
		goto done;
	}

	/*
	 * Determine an appropriate size for our Scatter Gatther lists.
	 */
	hr_host_template.sg_tablesize = OSM_SG_MAX;

	/*
	 * Register ourselves as a character device so that
	 * we can receive ioctl requests.
	 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	hr_major = devfs_register_chrdev(0, HR_DRIVER_NAME,
					 &hr_file_operations);

#else
	hr_major = register_chrdev(0, HR_DRIVER_NAME,
					 &hr_file_operations);
#endif

	hr_log(HR_BOOT_DEBUG, 1, "hr_detect: devfs_register == %d\n", hr_major);
	if (hr_major < 0) {
		/*
		 * HostRAID doesn't function very well without
		 * the monitor, so clean up all found controllers
		 * and report no hardware found if we can't register
		 * our device node.
		 */
		hr_init_status = -ENODEV;
		goto done;
	}
#ifdef CONFIG_COMPAT 
#if	!(HAVE_COMPAT_IOCTL)
	register_ioctl32_conversion((uint32_t)
		ADAPTEC_IOCTL, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_DRIVER_INFO, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_CNTLR_CONFIG, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_CNTLR_STATUS, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_FIRMWARE_DOWNLOAD, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_RAID_INFO, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_RAID_CONFIG, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_PHY_INFO, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_SET_PHY_INFO, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_LINK_ERRORS, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_SATA_SIGNATURE, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_SCSI_ADDRESS, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_TASK_MANAGEMENT, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_CONNECTOR_INFO, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_PHY_CONTROL, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_SMP_PASSTHRU, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_SSP_PASSTHRU, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_STP_PASSTHRU, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_READ_NV_SEGMENT, (void *) sys_ioctl);
	register_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_WRITE_NV_SEGMENT, (void *) sys_ioctl);
#endif
#endif

	/*
	 * Register our driver with the PCI subsystem.  This
	 * will result in PCI layer calls back into our driver
	 * for each found device.
	 */
	error = hr_pci_init();
	hr_log(HR_BOOT_DEBUG, 1, "hr_detect: hr_pci_init == %d\n", error);
	if (error != 0) {
		hr_init_status = error;
		goto done;
	}

	/*
	 * Register with the SCSI layer all controllers we've found.
	 */
	TAILQ_FOREACH(hr, &hr_tailq, links) {
		if (hr_register_host(hr) == 0)
			found++;
	}

	/*
	 * Hook shutdown events.
	 */
	error = register_reboot_notifier(&hr_reboot_notifier_block);
	hr_log(HR_BOOT_DEBUG, 1, "hr_detect: register_reboot == %d\n", error);
	if (error == 0)
		hr_reboot_notifier_registered = 1;

	/*
	 * Indicate that our intial detection
	 * pass has completed.  If the PCI
	 * subsystem presents a hot-plug device
	 * to us in the future, hr_dete_complete
	 * being non-zero indicates that the
	 * host should be registered immediately
	 * with the OS.
	 */
	hr_detect_complete++;
done:
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	spin_lock_irq(&io_request_lock);
#endif
	hr_log(HR_BOOT_DEBUG, 1, "hr_detect: found == %d\n", found);
	return (found);
}

/*
 * Return an informative string
 */
static const char *
hr_info(struct Scsi_Host *host)
{
	struct	hr_softc *hr;
	struct	hr_pci_driver_data *driver_data;
	const	char *descr;
	
	if (host == NULL)
		return (NULL);

	/*
	 * XXX Provide more information??
	 */
	descr = NULL;
	hr = *((struct hr_softc **)host->hostdata);
	hr = hr_find_softc(hr);
	if (hr != NULL) {
		driver_data =
		    (struct hr_pci_driver_data *)(hr->pci_entry->driver_data);
		descr = driver_data->description;
	}
	return (descr);
}

/*
 * Queue an OS initiated I/O to the CDA Core.
 */
static int
hr_queuecommand(Scsi_Cmnd *cmd, void (*scsi_done)(Scsi_Cmnd *))
{
	struct hr_softc		*hr;
	struct hr_linux_device	*dev;

	/*
	hr_log(HR_RUNTIME_DEBUG, 2,
	       "hr_queuecmd(%d,%d,%d) opcode 0x%x: Enter\n",
	       cmd->device->channel, cmd->device->id,
	       cmd->device->lun,cmd->cmnd[0]);
	*/

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	hr = *((struct hr_softc **)cmd->host->hostdata);
#else
	hr = *((struct hr_softc **)cmd->device->host->hostdata);
#endif
	/*
	 * Save the callback on completion function.
	 */
	cmd->scsi_done = scsi_done;

	hr_midlayer_entrypoint_lock(hr);

	/*
	 * Return the request to the upper layer if we are frozen.
	 */
	if ((hr->qfrozen != 0) || (hr->flags & HR_SUSPEND_IO)) {
		printk("hr_queuecmd(%d,%d,%d): QFrozen\n",
		       cmd->device->channel, cmd->device->id,
		       cmd->device->lun);
	}

	/*
	 * If the command doesn't belong to any devices that we know of,
	 * return it.
	 */
	dev = hr_get_ldev(hr, cmd);
	if (dev == NULL) {
		hr_log(HR_RUNTIME_DEBUG, 1, "hr_queuecmd(%d,%d,%d) SELTO\n",
		       cmd->device->channel, cmd->device->id,
		       cmd->device->lun);
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
		cmd->result = (DID_NO_CONNECT << 16);
#else
		hr_scsi_offline(cmd);
#endif
		cmd->scsi_done(cmd);
		hr_midlayer_entrypoint_unlock(hr);
		return 0;
	}

	if ((dev->flags & HR_DEV_HOT_REMOVED) != 0) {
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
		cmd->result = (DID_NO_CONNECT << 16);
#else
		hr_scsi_offline(cmd);
#endif
		cmd->scsi_done(cmd);
		hr_midlayer_entrypoint_unlock(hr);
		hr_log(HR_RUNTIME_DEBUG, 1, "hr_queuecmd(%d,%d,%d) "
			"device already hot removed\n", cmd->device->channel,
			 		cmd->device->id, cmd->device->lun);
		return (0);
	}

	if (cmd->cmnd[0] == SCSI_READ_6 || cmd->cmnd[0] == SCSI_READ_10) {
		dev->rq_dev = hr_get_sd_devnum(cmd);
	}

	if(dev->incr_timeout & LUN_RESET_TASK)
		hr_timer_reset(&cmd->eh_timeout, cmd->timeout_per_command +
					 500*HZ, cmd->eh_timeout.function, cmd);
	else
		hr_timer_reset(&cmd->eh_timeout, cmd->timeout_per_command +
					 100*HZ, cmd->eh_timeout.function, cmd);

	cmd->result = 0;
	TAILQ_INSERT_TAIL(&dev->busyq, (struct hr_cmd *)cmd, hcmd_links.tqe);
	if ((hr->qfrozen == 0) && (!(hr->flags & HR_SUSPEND_IO)) && 
		(dev->flags & HR_DEV_ON_RUN_LIST) == 0) {
		TAILQ_INSERT_TAIL(&hr->device_runq, dev, links);
		dev->flags |= HR_DEV_ON_RUN_LIST;
		hr_run_device_queues(hr);
	}
	
	hr_midlayer_entrypoint_unlock(hr);
	return (0);
} /* hr_queuecommand() - end */

/*
 * Return the disk geometry for the given SCSI device.
 */
static int
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
hr_bios_param(struct scsi_device *sdev, struct block_device *bdev,
	      sector_t capacity, int geom[])
{
	unsigned char		*res;
#else
hr_bios_param(Disk *disk, kdev_t dev, int geom[])
{
	struct block_device	*bdev;
	u_long	capacity = disk->capacity;
#endif
	struct	buffer_head *bh;
	int	 ret;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	if (bdev == NULL) {
		return -1;
	}

	res = kmalloc(66, GFP_KERNEL);

	if (res == NULL) {
		return -1;
	}

	bh = __bread(bdev, 0, block_size(bdev));
#else
	bh = bread(MKDEV(MAJOR(dev), MINOR(dev)&~0xf), 0, block_size(dev));
#endif

	if (bh == NULL) {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
		kfree(res);
#endif
		return -1;
	}

	/*
	 * try to infer mapping from partition table
	 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	memcpy(res, bh->b_data + 0x1be, 66);

	ret = scsi_partsize(res, (unsigned long)capacity,
		(unsigned int *)geom + 2, (unsigned int *)geom + 0,
		(unsigned int *)geom + 1);
#else
	ret = scsi_partsize(bh, (unsigned long)capacity,
		(unsigned int *)geom + 2, (unsigned int *)geom + 0,
		(unsigned int *)geom + 1);
#endif

		brelse(bh);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	kfree(res);
#endif

	if ((ret == 0) && (geom[0] <= 255) && (geom[1] <= 63)) {
		/*
		 * The partition table had the right answer.
		 */
		return 0;
	}

	/*
	 * if something went wrong, then apparently we have to return
	 * a geometry with more than 1024 cylinders
	 */
	geom[0] = 64; // heads
	geom[1] = 32; // sectors
	geom[2] = hr_sector_div(capacity, geom[0], geom[1]);

	if (geom[2] >= 1024) {
		geom[0] = 255;
		geom[1] = 63;
		geom[2] = hr_sector_div(capacity, geom[0], geom[1]);
	}

	if (capacity > 65535*63*255) {
		geom[2] = 65535;
	} else {
		geom[2] = (unsigned long)capacity / (geom[0] * geom[1]);
	}

	return 0;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
/*
 * Entry routine for unloading the driver.
 */
static int
hr_release(struct Scsi_Host *host)
{
	struct	hr_softc *hr;

	if (host != NULL) {
		/* 
		 * We should be able to just perform
		 * the free directly, but check our
		 * list for extra sanity.
		 */
#ifdef CONFIG_COMPAT 
#if	!(HAVE_COMPAT_IOCTL)
	unregister_ioctl32_conversion((uint32_t) ADAPTEC_IOCTL);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_DRIVER_INFO);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_CNTLR_CONFIG);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_CNTLR_STATUS);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_FIRMWARE_DOWNLOAD);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_RAID_INFO);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_RAID_CONFIG);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_PHY_INFO);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_SET_PHY_INFO);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_LINK_ERRORS);
	unregister_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_SATA_SIGNATURE);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_GET_SCSI_ADDRESS);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_TASK_MANAGEMENT);
	unregister_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_CONNECTOR_INFO);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_PHY_CONTROL);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_SMP_PASSTHRU);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_SSP_PASSTHRU);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_STP_PASSTHRU);
	unregister_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG);
	unregister_ioctl32_conversion((uint32_t)
		 CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_READ_NV_SEGMENT);
	unregister_ioctl32_conversion((uint32_t) CC_CSMI_SAS_WRITE_NV_SEGMENT);
#endif
#endif
		hr = hr_find_softc(*(struct hr_softc **)host->hostdata);
		if (hr != NULL) {
			hr_free(hr);
		}
	}

	return (0);
}
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
static int
hr_linux_slave_alloc(Scsi_Device *device)
{
	return (0);
}

static int
hr_linux_slave_configure(Scsi_Device *device)
{
	struct	hr_softc *hr;
	struct	hr_linux_device *dev;

	hr = *((struct hr_softc **)device->host->hostdata);

	hr_midlayer_entrypoint_lock(hr);
	/*
	 * Since Linux has attached to the device, configure
	 * it so we don't free and allocate the device
	 * structure on every command.
	 */
	dev = hr->device_table[device->channel][device->id][device->lun];
	if (dev != NULL) {
		dev->flags &= ~HR_DEV_UNCONFIGURED;
		dev->flags |= HR_DEV_SLAVE_CONFIGURED;
		dev->scsi_device = device;
		if (device->tagged_supported != 0) {
			hr_user_cmd_per_lun = MIN(hr_user_cmd_per_lun,
						 hr->max_cmds_per_lun);
			if (hr_user_cmd_per_lun < 1) {
				device->queue_depth =
					CdaGetQueueDepth(hr->pAdapterDevExt,
								 dev->handle);
			} else {
				device->queue_depth = hr_user_cmd_per_lun;
			}
			printk("scsi%d:%d:%d: Tagged Queuing enabled. "
			       "Depth %d\n", device->channel,
			       device->id, device->lun, device->queue_depth);
		} else {
			/*
			 * We allow the OS to queue 2 untagged
			 * transactions to us at any time even
			 * though we can only execute them
			 * serially on the controller/device.
			 * This should remove some latency.
			 */
         if( (device->type == TYPE_TAPE) || (device->type == TYPE_ROM) ) {
         /* for CD-ROM/TAPE device, queue depth has to be 1 */
            device->queue_depth = 1;
         }
         else {
            device->queue_depth = HR_CMD_PER_LUN_UNTAGGED;
         }
			printk("scsi%d:%d:%d: Tagged Queuing disabled. "
				 "Depth %d\n", device->channel, device->id,
					 device->lun, device->queue_depth);
		}
	}
	hr_midlayer_entrypoint_unlock(hr);

	return (0);
}

static void
hr_linux_slave_destroy(Scsi_Device *device)
{
	struct	hr_softc *hr;
	struct hr_linux_device	*dev;
	u_long			 flags;

	hr = *((struct hr_softc **)device->host->hostdata);

	hr_lock(hr, &flags);

	dev = hr->device_table[device->channel][device->id][device->lun];
	if (dev == NULL) {
		hr_unlock(hr, &flags);
		return;
	}

	if ((dev->flags & HR_DEV_SLAVE_CONFIGURED) != 0) {
		if (TAILQ_EMPTY(&dev->busyq) && (dev->active == 0) && 
		   ((dev->flags & HR_DEV_DESTROY_WAS_ACTIVE) != 0)) {
			/* Only free the device if it is idle. */	
			hr_free_ldev(hr, dev);
			hr->device_table[device->channel][device->id]
					[device->lun] = NULL;
		}
	}
	hr_unlock(hr, &flags);
}
#else
/*
 * Fill in the device queue depths for each
 * device attached to our controller.
 */
static void
hr_select_queue_depths(struct Scsi_Host *host,
		       Scsi_Device *scsi_devices)
{
	struct hr_softc *hr;
	struct hr_linux_device *hr_ldev;
	Scsi_Device	*device;
	Scsi_Device	*ldev;

	hr = *((struct hr_softc **)host->hostdata);

	for (device = scsi_devices; device != NULL; device = device->next) {

		if (device->host != host)
			continue;

		/*
		 * Watch out for duplicate devices.  This works around
		 * some quirks in how the SCSI scanning code does its
		 * device management.
		 */
		for (ldev = scsi_devices; ldev != device; ldev = ldev->next) {
			if (ldev->host == device->host
			 && ldev->channel == device->channel
			 && ldev->id == device->id
			 && ldev->lun == device->lun)
				break;
                }
		/* Skip duplicate. */
		if (ldev != device)
			continue;

		if (device->tagged_supported != 0) {
			hr_user_cmd_per_lun = MIN(hr_user_cmd_per_lun,
							 hr->max_cmds_per_lun);
			if (hr_user_cmd_per_lun < 1) {
				hr_ldev = (struct hr_linux_device *)
				hr->device_table[device->channel][device->id] \
								[device->lun];
				hr_user_cmd_per_lun = 
					CdaGetQueueDepth(hr->pAdapterDevExt,
							hr_ldev->handle);
			}

			device->queue_depth = hr_user_cmd_per_lun;
			printk("scsi%d:%d:%d: Tagged Queuing enabled. "
			       "Depth %d\n", host->host_no,
			       device->id, device->lun, device->queue_depth);
		} else {
			/*
			 * We allow the OS to queue 2 untagged
			 * transactions to us at any time even
			 * though we can only execute them
			 * serially on the controller/device.
			 * This should remove some latency.
			 */

         if( (device->type == TYPE_TAPE) || (device->type == TYPE_ROM) ) {
         /* for CD-ROM/TAPE device, queue depth has to be 1 */
            device->queue_depth = 1;
         }
         else {
            device->queue_depth = HR_CMD_PER_LUN_UNTAGGED;
         }
			printk("scsi%d:%d:%d: Tagged Queuing disabled. "
			 "Depth %d\n", host->host_no, device->id, device->lun,
							 device->queue_depth);
		}
	}

}
#endif

void
hr_update_dev_status(struct hr_softc *hr, void *pDevHandle, u_short member_idx,
		     u_int evt)
{
	struct hr_linux_device	*dev;
	uint32_t	EmbeddedPT;
	uint16_t	PathId;
	uint16_t	TargetId;
	uint16_t	Lun;
	IO_HANDLE		 IoHandle;
	uint16_t	notifycda = 0;

	hr_log(HR_RUNTIME_DEBUG, 1, "%s, Event to handle: 0x%x.\n",
						 __FUNCTION__, evt);

	EmbeddedPT = (uint32_t) ((unsigned long)pDevHandle);
	PathId = (uint16_t) ((EmbeddedPT & 0xFFFF0000) >> 16);
	TargetId = (uint16_t) (EmbeddedPT & 0x0000FFFF);
	Lun = member_idx;
	dev = NULL;

	if((PathId > OSM_MAX_CHANNEL) || (TargetId > OSM_MAX_TARGET_PER_PATH) ||
		(Lun != 0))
		return;

	printk("hr_update_dev_status: event 0x%x %d %d %d\n",
		evt, PathId,TargetId,Lun);

	memset(&IoHandle, 0, sizeof(IO_HANDLE));

	switch (evt) {
	case SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_NEW_SLOT:
		CdaGetPTSlot(hr->pAdapterDevExt, PathId, TargetId,0, &IoHandle);
		if(IoHandle.pHandle != NULL) {
			if((GetSysDeviceAccess(IoHandle.pHandle)) > 0x2 &&
                           (GetSysDeviceAccess(IoHandle.pHandle)) != LOTHER )
				return;

			dev = hr_alloc_ldev(hr, IoHandle.pHandle);
		}
		if(dev == NULL) {
			return;
		}
		dev->flags |= HR_DEV_ONLINE;
		hr->device_table[PathId][TargetId][Lun] = dev;

		hr_log(HR_ERROR_DEBUG, 1,
		       "New device mapped at slot %d:%d:%d.\n",
		       PathId, TargetId, Lun);

		hr_setup_dev_dpc_task(dev, hr_add_device);

		break ;
	case SLOTS_LASTSTATE_OLD_DEVICE_REAPPEARED_AT_OLD_SLOT:
		CdaGetPTSlot(hr->pAdapterDevExt, PathId, TargetId,0, &IoHandle);
		if(IoHandle.pHandle != NULL) {
			/* 
			 * This covers the case where the device
			 * was previously turned offline but not removed
			 */
			if(hr->device_table[PathId][TargetId][Lun] != NULL) {
				dev = hr->device_table[PathId][TargetId][Lun];
				/* handle might have changed */
				dev->handle = IoHandle.pHandle;
				dev->flags = 0;
				dev->flags |= HR_DEV_ONLINE;
				printk("hr_update_dev_status: old dev alloc "
						"dev %d %d\n", PathId,TargetId);
			return;
		}
			dev = hr_alloc_ldev(hr, IoHandle.pHandle);
		}
		if(dev == NULL) {
			return;
		}
		dev->flags |= HR_DEV_ONLINE;
		hr->device_table[PathId][TargetId][Lun] = dev;
		printk("hr_update_dev_status:alloc dev %d %d\n",
						 PathId,TargetId);
		hr_setup_dev_dpc_task(dev, hr_add_device);
		break ;
	case SLOTS_LASTSTATE_NEW_DEVICE_ADDED_AT_OLD_SLOT_OR_DEVICE_CHANGED:
		dev = hr->device_table[PathId][TargetId][Lun];
		if(dev == NULL) {
			/* 
			 * if not visible to OS, turn it
			 * ONLINE then add it to OS 
			 */
			CdaAcknowledgeDeviceChangeAtSameSlot((PVOID)
					 hr->pAdapterDevExt,PathId,TargetId);

			CdaGetPTSlot(hr->pAdapterDevExt, PathId, TargetId,0,
				    				 &IoHandle);
			if(IoHandle.pHandle != NULL) {
				dev = hr_alloc_ldev(hr, IoHandle.pHandle);
			}
			if(dev == NULL) {
				return;
			}
			dev->flags |= HR_DEV_ONLINE;
			hr->device_table[PathId][TargetId][Lun] = dev;
			printk("hr_update_dev_status:alloc dev %d %d\n",
							 PathId,TargetId);
			hr_setup_dev_dpc_task(dev, hr_add_device);
		} else {
		/*
		 * it is known to OS, remove it,
		 * turn it new one ONLINE then add
		 */
			dev->flags &= ~HR_DEV_ONLINE;
			dev->flags |= HR_DEV_HOT_REMOVED;

			/* Check if any IO(s) pending for this device. */
			if (dev->active > 0) {
				hr_log(HR_ERROR_DEBUG, 1,
					 "Device is still ACTIVE.\n");
				dev->flags |= HR_DEV_DESTROY_WAS_ACTIVE;
			} else {
			/*
		 	 * No more IO(s) pending for this device and the
		 	 * device is not in recovery mode, Schedule a
	   	 	 * dpc task to destroy the device.
	       	 	 */
			if ((dev->flags & HR_DEV_IN_RECOVERY) == 0)
				hr_setup_dev_dpc_task(dev, hr_destroy_device);
			}
			CdaAcknowledgeDeviceChangeAtSameSlot((PVOID)
					 hr->pAdapterDevExt,PathId,TargetId);

			CdaGetPTSlot(hr->pAdapterDevExt, PathId, TargetId, 0,
				     &IoHandle);
			if(IoHandle.pHandle != NULL) {
				dev = hr_alloc_ldev(hr, IoHandle.pHandle);
			}
			if(dev == NULL) {
				return;
			}
			dev->flags |= HR_DEV_ONLINE;
			hr->device_table[PathId][TargetId][Lun] = dev;
			printk("hr_update_dev_status:alloc dev %d %d\n",
							 PathId,TargetId);
			hr_setup_dev_dpc_task(dev, hr_add_device);
		}

		break ;
	case SLOTS_LASTSTATE_OLD_DEVICE_REMOVED_AT_OLD_SLOT:
		notifycda = 1;

	case SLOTS_LASTSTATE_OLD_DEVICE_GOT_OFFLINE_AT_OLD_SLOT:

		dev = hr->device_table[PathId][TargetId][Lun];
		if (dev == NULL) {
			hr_log(HR_ERROR_DEBUG, 1, "Dev is not found on the "
		       					"Dev Table.\n");
			if(notifycda)
				CdaAcknowledgeTargetDeletion((PVOID)
					 hr->pAdapterDevExt,PathId,TargetId);
			return;
		}
		dev->flags &= ~HR_DEV_ONLINE;
		dev->flags |= HR_DEV_HOT_REMOVED;

		/* Check if any IO(s) pending for this device. */
		if (dev->active > 0) {
			hr_log(HR_ERROR_DEBUG, 1, "Device is still ACTIVE.\n");
			dev->flags |= HR_DEV_DESTROY_WAS_ACTIVE;
		} else {
			/*
			 * No more IO(s) pending for this device and the
			 * device is not in recovery mode, Schedule a
		   	 * dpc task to destroy the device.
		       	 */
			if ((dev->flags & HR_DEV_IN_RECOVERY) == 0)
				hr_setup_dev_dpc_task(dev, hr_destroy_device);
		}
		if(notifycda)
			CdaAcknowledgeTargetDeletion((PVOID)
				 hr->pAdapterDevExt,PathId,TargetId);

		break ;
	default:
		break;
	}
}

static int
hr_find_dev_in_devtable(struct hr_softc *hr, struct hr_linux_device *dev,
			u_int *ch, u_int *id, u_int *lun)
{
	u_int	bus;
	u_int	tgt;

	/*
	 * TODO: Need to add multi-lun support. 
	 *	 For now, we set the lun to 0.
	 */
	*lun = 0;
	for (bus = 0; bus < OSM_MAX_CHANNEL; bus++) {
		for (tgt = 0; tgt < OSM_MAX_TARGET_PER_PATH; tgt++) {
			if (hr->device_table[bus][tgt][0] == dev) {
				*ch = bus;
				*id = tgt;

				return (TRUE);
			}
		}
	}

	hr_log(HR_ERROR_DEBUG, 1, "Unable to find DEV in DEVICE TABLE.\n");

	return (FALSE);
}


static void
hr_flush_commands(struct hr_linux_device  *dev)
{
	struct hr_cmd	 	*hcmd;
	struct scsi_cmnd 	*cmd;

	//BEGIN;

	/*
	 * Complete all deferred I/O with NO_CONNECT status
	 */
	while ((hcmd = TAILQ_FIRST(&dev->busyq)) != NULL) {

		TAILQ_REMOVE(&dev->busyq, hcmd, hcmd_links.tqe);
		cmd = &hcmd_scsi_cmd(hcmd);

		hr_log(HR_RUNTIME_DEBUG, 1, "%s, complete pending IO: %p\n",
						 __FUNCTION__, hcmd);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
		cmd->result = (DID_NO_CONNECT << 16);
#else
		hr_scsi_offline(cmd);
#endif
		cmd->scsi_done(cmd);
	}

	//END;

}

static void
hr_destroy_device(void *arg)
{
	struct hr_softc		*hr;
	struct hr_linux_device	*dev;
	u_long			 flags;
	int			 ch;
	int			 id;
	int			 lun;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	Scsi_Device		*device;
	int			 busy;
#endif

	dev = (struct hr_linux_device *)arg;
	hr = dev->softc;

	hr_lock(hr, &flags);

	dev->flags &= ~HR_DEV_DPC_ACTIVE;

	if (hr_find_dev_in_devtable(hr, dev, &ch, &id, &lun) == FALSE) {
		hr_log(HR_ERROR_DEBUG, 1, "Trying to remove invalid DEV.\n");
		hr_unlock(hr, &flags);
		return;
	}

	/* flush all pending IOs in OSM */
	hr_flush_commands(dev);

	hr_log(HR_RUNTIME_DEBUG, 1, "%s: device %d:%d:%d:%d "
		 "has been removed.\n", HR_DRIVER_NAME,
		 hr->host->host_no, ch, id, lun);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	/*
	 * Find the Scsi_Device associated with the SCSI address,
	 * and mark it as changed, invalidating the cache. This deals
	 * with changes to existing device IDs.
	 */
	busy = FALSE;
	for (device = hr->host->host_queue; device != NULL;
	     device = device->next) {
		if ((device->channel != ch) || (device->id != id) ||
		    				(device->lun != lun))
			continue;

		busy = (device->access_count != 0);

#if _TBR_		
		if (busy == TRUE) {
			//device->changed = TRUE;
			//device->removable = TRUE;
			//device->online = FALSE;
		}
#endif		
	}

	hr_unlock(hr, &flags);

	if (busy == FALSE)
		hr_initiate_kernel_rescan(hr, ch, id, lun, FALSE);
#else
	hr_unlock(hr, &flags);
	if(dev->scsi_device == NULL)
		printk("hr_destroy_device: scsi_device is null");
	else
		scsi_remove_device(dev->scsi_device);
#endif
	hr_free_ldev(hr, dev);
	hr->device_table[ch][id][lun] = NULL;
}

static void
hr_add_device(void *arg)
{
	struct hr_softc		*hr;
	struct hr_linux_device	*dev;
	u_long			 flags;
	int			 ch;
	int			 id;
	int			 lun;

	dev = (struct hr_linux_device *)arg;
	hr = dev->softc;

	hr_lock(hr, &flags);

	dev->flags &= ~HR_DEV_DPC_ACTIVE;

	if (hr_find_dev_in_devtable(hr, dev, &ch, &id, &lun) == FALSE) {
		hr_log(HR_ERROR_DEBUG, 1, "Trying to add invalid DEV.\n");
		hr_unlock(hr, &flags);
		return;
	}

	hr_unlock(hr, &flags);

	/* Report the new target found to the user or OS. */
	printk("New device attached at ");
	printk("Host: scsi%d Channel: %d Id: %d Lun: %d\n",
		hr->host->host_no, ch, id, lun);

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
	/*
	 * For 2.6 kernel, notify the scsi layer that a new had been
	 * device added.
	 */
	scsi_add_device(hr->host, ch, id, lun);
#else
	/*
	 * For 2.4 kernel, we need an "ugly hack" to tell the scsi layer
	 * to notify that a device had been added.
	 */ 
	hr_initiate_kernel_rescan(hr, ch, id , lun, TRUE);
#endif
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
static void
hr_initiate_kernel_rescan(struct hr_softc *hr, int ch, int id, int lun,
			  int to_add)
{
	struct proc_dir_entry	*entry;
	char			 buffer[80];
	mm_segment_t		 orig_addr_limit;

	/*
	 * scan_scsis(hr->host, 1, bus, target, lun)
	 * is not exported as accessible, so we need to go around it
	 * another way. So, we look for the "proc/scsi/scsi" entry in
	 * the proc filesystem (using proc_scsi as a shortcut) and send
	 * it a message. This deals with new devices that have
	 * appeared. If the device has gone offline, scan_scsis will
	 * also discover this, but we do not want the device to go
	 * away. We do not need to check the access_count for the
	 * device since we are not wanting the devices to go away.
	 */
	if (proc_scsi == NULL)
		return;

	for (entry = proc_scsi->subdir; entry; entry = entry->next) {
		if ((entry->low_ino == 0) || (entry->namelen != 4) ||
		     (memcmp ("scsi", entry->name, 4) != 0)) {
			continue;
		}
		if (entry->write_proc == NULL)
			continue;

		sprintf(buffer,
		       ((to_add == TRUE) ?
			"scsi add-single-device %d %d %d %d\n" :
			"scsi remove-single-device %d %d %d %d\n"),
			hr->host->host_no, ch, id, lun);

		/*
		 * Make sure that write_proc can copy in from
		 * buffer by giving us kernel address
		 * privaleges for a short moment.
		 */
		orig_addr_limit = current->addr_limit;
		current->addr_limit = KERNEL_DS;
		entry->write_proc(NULL, buffer, strlen(buffer), NULL);
		current->addr_limit = orig_addr_limit;
	}
}
#endif

/******************************* OS Error Handling ****************************/

/*
 * Function:
 *	hr_ehandler_thread()
 *
 * Description:
 *	Thread to handle error recovery.
 */
static int
hr_ehandler_thread(void *data)
{
	struct hr_softc	*hr;
	u_long		 flags;	

	hr = (struct hr_softc *) data;

	lock_kernel();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,60)
	/*
	 * Don't care about any signals.
	 */
	siginitsetinv(&current->blocked, 0);
	daemonize();
	sprintf(current->comm, "hr_eh_%d", hr->unit);
#else
	daemonize("asd_eh_%d", hr->unit);
	current->flags |= PF_FREEZE;	
#endif
	unlock_kernel();

	while (1) {
		/*
		 * Use down_interruptible() rather than down() to
		 * avoid this thread to be counted in the load average as
		 * a running process.
		 */
		down_interruptible(&hr->platform_data->ehandler_sem);

		hr_log(HR_RUNTIME_DEBUG, 1, "hr_eh wakeup\n");

		/* Check if we've been signaled to exit. */
		hr_lock(hr, &flags);
		if ((hr->platform_data->flags & HR_RECOVERY_SHUTDOWN) != 0) {
			hr_unlock(hr, &flags);
			break;
		}
		/* 
		 * Check if any timedout commands that required 
		 * error handling. 
		 */
		if (TAILQ_EMPTY(&hr->io_request_timedoutq)) {
			hr_unlock(hr, &flags);
			continue;
		}
		hr_unlock(hr, &flags);

		hr_recover_cmds(hr);
	}
	up(&hr->platform_data->ehandler_ending_sem);

	return (0);
}

/*
 * Function:
 * 	hr_kill_ehandler_thread()
 * 
 * Description:
 * 	Kill the error handling thread.
 */
static void
hr_kill_ehandler_thread(struct hr_softc *hr)
{
	u_long	flags;

	hr_lock(hr, &flags);

	if (hr->platform_data->ehandler_pid != 0) {
		hr->platform_data->flags |= HR_RECOVERY_SHUTDOWN;
		hr_unlock(hr, &flags);
		up(&hr->platform_data->ehandler_sem);
		hr->platform_data->ehandler_pid = 0;
	} else {
		hr_unlock(hr, &flags);
	}
}

static void
hr_recover_cmds(struct hr_softc *hr)
{
	struct hr_io_request	*ioreq;
	struct hr_io_request	*free_ioreq;
	u_long		 	 flags;

	if (TAILQ_EMPTY(&hr->io_request_timedoutq)) {
		hr_log(HR_ERROR_DEBUG, 1,
		       "Timedout io reqs already completed.\n");
		goto exit;
	}
	
	TAILQ_FOREACH(ioreq, &hr->io_request_timedoutq, timedout_links) {

		hr_log(HR_RUNTIME_DEBUG, 1, "%s: ioreq: %p, eh_state: %x\n",
					 __FUNCTION__, ioreq, ioreq->eh_state);

		hr_lock(hr, &flags);
		/* 
		 * Error recovery is in progress for this io req.
		 * Proceed to next one.
		 */
		if ((ioreq->eh_state & HR_EH_IN_PROGRESS) != 0) {
			hr_unlock(hr, &flags);
			continue;
		}

		/* Error recovery is completed for this io req. */
		if (ioreq->eh_state == HR_EH_DONE) {
			hr_unlock(hr, &flags);
			goto done;
		}
		/*
		 * Only allowed one Error Recovery ongoing for a particular
		 * device.
		 */
		if ((ioreq->dev->flags & HR_DEV_IN_RECOVERY) != 0) {
			hr_unlock(hr, &flags);
			continue;
		}
		/*
		 * Acquire a free io req from the reserved pool to be used
	         * for error recovery purpose.
		 */	 
		if ((free_ioreq = hr_get_ioreq(hr)) == NULL) {
			hr_log(HR_ERROR_DEBUG, 1, "Failed to acquire a "
				"free io_req for error recovery.\n");
			hr_unlock(hr, &flags);
			continue;
		}

		free_ioreq->pAdapter = hr;
		free_ioreq->dev = ioreq->dev;
		ioreq->dev->flags |= HR_DEV_IN_RECOVERY;

		/* Freeze the device's queue. */
		ioreq->dev->qfrozen++;

		/* Initialize the state. */
		ioreq->eh_state |= HR_EH_IN_PROGRESS;
		free_ioreq->eh_state = HR_EH_INITIATED;
		free_ioreq->eh_status = HR_EH_SUCCEED;

		switch (ioreq->eh_state & HR_EH_LEVEL_MASK) {
		case HR_EH_ABORT_REQ:
			hr_log(HR_ERROR_DEBUG, 1, "ABORT ER REQ.\n");
			hr_abort_ioreq(hr, ioreq, free_ioreq);
			break;

		case HR_EH_LU_RESET_REQ:
			hr_log(HR_ERROR_DEBUG, 1, "LU RESET ER REQ.\n");
			hr_reset_lu(hr, ioreq, free_ioreq);
			break;

		case HR_EH_DEV_RESET_REQ:
			hr_log(HR_ERROR_DEBUG, 1, "DEV RESET ER REQ.\n");
			hr_reset_device(hr, ioreq, free_ioreq);
			break;

		default:
			hr_log(HR_ERROR_DEBUG, 1, "Unknown ER Level.\n");
			ioreq->eh_state = HR_EH_DONE;
			ioreq->eh_status = HR_EH_FAILED;
			break;
		}
		hr_unlock(hr, &flags);

done:
		if (ioreq->eh_state == HR_EH_DONE) {
			hr_lock(hr, &flags);
			TAILQ_REMOVE(&hr->io_request_timedoutq, ioreq,
				     timedout_links);
			ioreq->flags &= ~(HR_IOREQ_TIMEDOUT | HR_IOREQ_RESET_LU);
			ioreq->dev->flags &= ~HR_DEV_IN_RECOVERY;
			/* Unfreeze the device's queue. */
			ioreq->dev->qfrozen--;
			ioreq->eh_post(hr, ioreq);

	//		if (ioreq->eh_status != HR_EH_FAILED) 
			{
				struct hr_linux_device	*dev;
				/*
				 * Schedule a timer to run the device
				 * queue if the device is not frozen.
				 */
				dev = ioreq->dev;
				if ((dev != NULL) && (dev->qfrozen == 0)) {
					if (dev->flags & HR_DEV_HOT_REMOVED) {
						hr_setup_dev_dpc_task(
							dev, hr_destroy_device);
					}
				} 
			}

			/*
			 * free the io request if it was
			 *  returned from CDA already 
			 */
			if ((ioreq->flags & HR_IOREQ_ACTIVE) == 0)
			{
				hr_log(HR_RUNTIME_DEBUG, 1, "%s, free ioreq: "
				 "%p\n", __FUNCTION__, ioreq);
				hr_free_ioreq(hr, ioreq);
			}

			hr_unlock(hr, &flags);
		}
	}

exit:
	return;
}

static void
hr_abort_ioreq(struct hr_softc *hr, struct hr_io_request *ioreq_to_abort,
	       struct hr_io_request *ioreq)
{
	POSD	pOSD;


	hr_log(HR_RUNTIME_DEBUG, 1, "ioreq_to_abort: %p, State: 0x%x ioreq: "
		 "%p, State: 0x%x.\n", ioreq_to_abort, ioreq_to_abort->eh_state,
		 				ioreq, ioreq->eh_state);

	switch (ioreq->eh_state) {
	case HR_EH_INITIATED:
		/* Fill in the TMF OSD info needed by the CDA. */
		pOSD = &ioreq->OSD; 
		pOSD->OSD_DeviceExtension = hr->pAdapterDevExt;
		pOSD->cOSD_Reference = 0;
		pOSD->pOSRequest = ioreq;
		pOSD->OSD_DeviceHandle = ioreq_to_abort->dev->handle;
		pOSD->pOSD_OsmNext = &ioreq_to_abort->OSD; /* OSD to abort */
		pOSD->OSD_QueueTag = ABORT_TASK;
		pOSD->OSD_CustomFlag = 1;
		pOSD->cOSD_TCount = 0;
		pOSD->cOSD_TCount++;
		pOSD->pOSD_Post = hr_abort_ioreq_done;

		ioreq->eh_state = HR_EH_ABORT_REQ;
		hr_abort_ioreq(hr, ioreq_to_abort, ioreq);
		break;

	case HR_EH_ABORT_REQ:
		/* Issue TMF request to the CDA. */
		TAILQ_INSERT_TAIL(&hr->io_request_busyq, ioreq, links);
		CdaTMFRequest(&ioreq->OSD, ioreq_to_abort->dev->handle,
			      ABORT_TASK);
		break;	

	case HR_EH_DONE:
		ioreq_to_abort->eh_state = ioreq->eh_state;
		ioreq_to_abort->eh_status = ioreq->eh_status;
		TAILQ_REMOVE(&hr->io_request_busyq, ioreq, links);
		hr_free_ioreq(hr, ioreq);
		up(&hr->platform_data->ehandler_sem);
		break;

	default:
		hr_log(HR_ERROR_DEBUG, 1, "ioreq: %p, unknown ioreq state: "
			"%x, do nothing, free ioreq\n", ioreq, ioreq->eh_state);

		ioreq_to_abort->eh_state = HR_EH_DONE;
		ioreq_to_abort->eh_status = HR_EH_FAILED;
		hr_free_ioreq(hr, ioreq);
		up(&hr->platform_data->ehandler_sem);
		break;
	}
}

void
hr_abort_ioreq_done(POSD pOSD)
{
	struct hr_io_request	*ioreq;
	
	ioreq = (struct hr_io_request *) pOSD->pOSRequest;

	hr_log(HR_RUNTIME_DEBUG, 1, "pOSD: %p, pOSD_Status: 0x%x, ioreq: %p\n",
	      				 pOSD, pOSD->OSD_Status, ioreq);

	ioreq->eh_state = HR_EH_DONE;
	if (pOSD->OSD_Status == CDA_SUCCESS) {
		ioreq->eh_status = HR_EH_SUCCEED;
	} else {
		hr_log(HR_ERROR_DEBUG, 1, "%s, OSD Status: 0x%x.\n",
		       	__FUNCTION__, pOSD->OSD_Status);
		ioreq->eh_status = HR_EH_FAILED;
	}
		
	hr_abort_ioreq(ioreq->pAdapter, pOSD->pOSD_OsmNext->pOSRequest, ioreq);
}

void
hr_reset_lu_done(POSD pOSD)
{
	struct hr_io_request	*ioreq;
	
	ioreq = (struct hr_io_request *) pOSD->pOSRequest;

	printk("%s, pOSD: %p, pOSD_Status: 0x%x, ioreq: %p\n",
	       __FUNCTION__, pOSD, pOSD->OSD_Status, ioreq);

	ioreq->eh_state = HR_EH_DONE;
	if (pOSD->OSD_Status == CDA_SUCCESS) {
		ioreq->eh_status = HR_EH_SUCCEED;
	} else {
		hr_log(HR_ERROR_DEBUG, 1, "%s, OSD Status: 0x%x.\n",
		       __FUNCTION__, pOSD->OSD_Status);
		ioreq->eh_status = HR_EH_FAILED;
	}
		
	hr_reset_lu(ioreq->pAdapter, pOSD->pOSD_OsmNext->pOSRequest, ioreq);

}

static void
hr_reset_lu(struct hr_softc *hr, struct hr_io_request *ioreq_to_reset,
	    struct hr_io_request *ioreq)
{
	POSD	pOSD;


	printk("hr_reset_lu: ioreq_to_reset: %p, State: %x, ioreq: %p, State: "
		 "0x%x.\n", ioreq_to_reset, ioreq_to_reset->eh_state,
		 ioreq, ioreq->eh_state);

	switch (ioreq->eh_state) {
	case HR_EH_INITIATED:
		/* Fill in the TMF OSD info needed by the CDA. */
		pOSD = &ioreq->OSD; 
		pOSD->OSD_DeviceExtension = hr->pAdapterDevExt;
		pOSD->cOSD_Reference = 0;
		pOSD->pOSRequest = ioreq;
		pOSD->OSD_DeviceHandle = ioreq_to_reset->dev->handle;
		pOSD->pOSD_OsmNext = &ioreq_to_reset->OSD; /* OSD to abort */
		//mdtmdtm pOSD->OSD_QueueTag = ABORT_TASK;
		pOSD->OSD_QueueTag = LUN_RESET_TASK;
		pOSD->OSD_CustomFlag = 1;
		pOSD->cOSD_TCount = 0;
		pOSD->cOSD_TCount++;
		pOSD->pOSD_Post = hr_reset_lu_done;

		ioreq->eh_state = HR_EH_LU_RESET_REQ;
		hr_reset_lu(hr, ioreq_to_reset, ioreq);
		break;

	case HR_EH_LU_RESET_REQ:
		/* Issue TMF request to the CDA. */
		TAILQ_INSERT_TAIL(&hr->io_request_busyq, ioreq, links);
		CdaTMFRequest(&ioreq->OSD, ioreq_to_reset->dev->handle,
			      LUN_RESET_TASK);
	     	ioreq->dev->incr_timeout |= LUN_RESET_TASK;
		break;

	case HR_EH_DONE:
		ioreq_to_reset->eh_state = ioreq->eh_state;
		ioreq_to_reset->eh_status = ioreq->eh_status;
		TAILQ_REMOVE(&hr->io_request_busyq, ioreq, links);
		hr_free_ioreq(hr, ioreq);
		up(&hr->platform_data->ehandler_sem);
		break;

	default:
		hr_log(HR_ERROR_DEBUG, 1, "ioreq: %p, unknown ioreq state: " 
			"%x, do nothing, free ioreq\n", ioreq, ioreq->eh_state);
		printk("hr_reset_lu: ioreq: %p, unknown ioreq state:  %x, do "
			 "nothing, free ioreq\n", ioreq, ioreq->eh_state);
		ioreq_to_reset->eh_state = HR_EH_DONE;
		ioreq_to_reset->eh_status = HR_EH_FAILED;
		hr_free_ioreq(hr, ioreq);
		up(&hr->platform_data->ehandler_sem);
		break;
	}
}


void
hr_reset_device_done(POSD pOSD)
{
	struct hr_io_request	*ioreq;
	
	ioreq = (struct hr_io_request *) pOSD->pOSRequest;

	hr_log(HR_RUNTIME_DEBUG, 1, "%s, pOSD: %p, pOSD_Status: 0x%x, "
		 "ioreq: %p\n", __FUNCTION__, pOSD, pOSD->OSD_Status, ioreq);

	ioreq->eh_state = HR_EH_DONE;
	if (pOSD->OSD_Status == CDA_SUCCESS) {
		ioreq->eh_status = HR_EH_SUCCEED;
	} else {
		hr_log(HR_ERROR_DEBUG, 1, "%s, OSD Status: 0x%x.\n",
		       __FUNCTION__, pOSD->OSD_Status);
		ioreq->eh_status = HR_EH_FAILED;
	}
		
	hr_reset_device(ioreq->pAdapter, pOSD->pOSD_OsmNext->pOSRequest, ioreq);

}

static void
hr_reset_device(struct hr_softc *hr, struct hr_io_request *ioreq_to_reset,
		struct hr_io_request *ioreq)
{
	POSD	pOSD;


	hr_log(HR_RUNTIME_DEBUG, 1, "ioreq_to_reset: %p, State: %x, "
		 "ioreq: %p, State: 0x%x.\n", ioreq_to_reset,
		 ioreq_to_reset->eh_state, ioreq, ioreq->eh_state);

	switch (ioreq->eh_state) {
	case HR_EH_INITIATED:
		/* Fill in the TMF OSD info needed by the CDA. */
		pOSD = &ioreq->OSD; 
		pOSD->OSD_DeviceExtension = hr->pAdapterDevExt;
		pOSD->cOSD_Reference = 0;
		pOSD->pOSRequest = ioreq;
		pOSD->OSD_DeviceHandle = ioreq_to_reset->dev->handle;
		pOSD->pOSD_OsmNext = &ioreq_to_reset->OSD; /* OSD to abort */
		pOSD->OSD_QueueTag = DEVICE_PORT_RESET;
		pOSD->OSD_CustomFlag = 1;
		pOSD->cOSD_TCount = 0;
		pOSD->cOSD_TCount++;
		pOSD->pOSD_Post = hr_reset_device_done;

		ioreq->eh_state = HR_EH_DEV_RESET_REQ;
		hr_reset_device(hr, ioreq_to_reset, ioreq);
		break;

	case HR_EH_DEV_RESET_REQ:
		/* Issue TMF request to the CDA. */
		TAILQ_INSERT_TAIL(&hr->io_request_busyq, ioreq, links);
		CdaTMFRequest(&ioreq->OSD, ioreq_to_reset->dev->handle,
			      DEVICE_PORT_RESET);
		break;

	case HR_EH_DONE:
		ioreq_to_reset->eh_state = ioreq->eh_state;
		ioreq_to_reset->eh_status = ioreq->eh_status;
		TAILQ_REMOVE(&hr->io_request_busyq, ioreq, links);
		hr_free_ioreq(hr, ioreq);
		up(&hr->platform_data->ehandler_sem);
		break;

	default:
		hr_log(HR_ERROR_DEBUG, 1, "ioreq: %p, unknown ioreq state: "
			"%x, do nothing, free ioreq\n", ioreq, ioreq->eh_state);
		ioreq_to_reset->eh_state = HR_EH_DONE;
		ioreq_to_reset->eh_status = HR_EH_FAILED;
		hr_free_ioreq(hr, ioreq);
		up(&hr->platform_data->ehandler_sem);
		break;
	}
}

/*
 * hr_abort()
 *
 * Description:
 * 	Perform abort for the requested command.
 */
static int
hr_abort(Scsi_Cmnd *cmd)
{
	struct hr_softc		*hr;
	int		 	 retval;
	struct hr_linux_device	*dev;
	struct hr_cmd		*hcmd;
	struct hr_io_request	*ioreq;
	int			 found;

	printk("(scsi%d: Ch %d Id %d Lun %d): ",
		cmd->device->host->host_no, cmd->device->channel,
			 cmd->device->id, cmd->device->lun);
	printk("Abort requested for SCSI cmd %p, opcode 0x%x.\n",
		 			cmd, cmd->cmnd[0]);

	hr_midlayer_entrypoint_lock(hr);
	
	hr = *(struct hr_softc **) cmd->device->host->hostdata;

	dev = hr_get_ldev(hr, cmd);
	if (dev == NULL) {
		printk("(scsi%d: Ch %d Id %d Lun %d): ",
			  cmd->device->host->host_no, cmd->device->channel, 
			  cmd->device->id, cmd->device->lun);
		printk("Is not an active device.\n");
		retval = EH_SUCCESS;
		goto exit;
	}


	found = FALSE;
	/* 
	 * Check if the cmd is still pending in the device busyq which
	 * means it hasn't been submitted yet. 
	 */
	TAILQ_FOREACH(hcmd, &dev->busyq, hcmd_links.tqe) {
		if (&hcmd_scsi_cmd(hcmd) == cmd) {
			/* Found it. */
			found = TRUE;
            		break;
    		}
	}

	if (found == TRUE) {
		/* 
		 * Command to be aborted still pending in the device
		 * busy queue.
		 */
		printk("%s, (scsi%d: Ch %d Id %d Lun %d): ",
			  __FUNCTION__, cmd->device->host->host_no,
			 cmd->device->channel, cmd->device->id,
			 cmd->device->lun);
		printk("Cmd %p found on device queue.\n", cmd);
		TAILQ_REMOVE(&dev->busyq, hcmd, hcmd_links.tqe);
		cmd->result = DID_ABORT << 16;	
		cmd->scsi_done(cmd);	
		retval = EH_SUCCESS;
		goto exit;
	}

	
	/* When sending abort, cda/chim could return all outstanding cmd */
	/* Therefore we want only one abort at a time.  Restart timer */
	/* if Abort pending or there is one in timedoutq .MDT. */
	ioreq = (struct hr_io_request *) cmd->host_scribble;


	if ((ioreq->flags & HR_IOREQ_TIMER_ACTIVE) != 0) {

		ioreq->flags &= ~HR_IOREQ_TIMER_ACTIVE;

		hr_ioreq_delete_timer(ioreq);

		if((ioreq->flags & ( HR_IOREQ_TIMEDOUT | HR_IOREQ_RESET_LU)) ||
						(dev->qfrozen != 0)) {
			retval = EH_SUCCESS;
			goto exit;
		}
		hr_log(HR_RUNTIME_DEBUG, 1, "%s, abort req: ioreq: %p.\n",
						 __FUNCTION__, ioreq);
	        printk("hr_ioreq_timedout:cmd 0x%p (%d,%d,%d) "
			 "opcode 0x%x: Enter\n", cmd, cmd->device->channel,
			 cmd->device->id, cmd->device->lun,cmd->cmnd[0]);

		/* Set the error recovery engine to reset lu this cmd. */
		/* @mdt Abort on a timed out sata is not going to work */
		/* since chim will start IT_NEXUS loss timer for sata */
		ioreq->eh_state = HR_EH_DEV_RESET_REQ;

		ioreq->flags |= ( HR_IOREQ_TIMEDOUT | HR_IOREQ_RESET_LU);

		ioreq->eh_post = hr_handle_tmf_req_done;

		/* Add the IO request to the timedout queue. */
		TAILQ_INSERT_TAIL(&hr->io_request_timedoutq, ioreq,
				  timedout_links);

		/* Wake up the error handler thread. */
		up(&hr->platform_data->ehandler_sem);

		retval = EH_SUCCESS;
		goto exit;
	}
	/* if we are here, then our internal EH is hosed */
	/* let the OS remove device gracefully */ 
	retval = EH_SUCCESS;

exit:
	hr_midlayer_entrypoint_unlock(hr);
	return (retval);
}

/**************************************************************************

Function Name: hr_get_sd_devname()

Function Description
	To convert disk number to /dev/sdX.

Input Parameters
	value		value to evaluate

Output Parameters
	returns the absolute value of the input parameter.

**************************************************************************/
static void hr_get_sd_devname(int disknum, char *buffer)
{
	if (disknum < 0) {
		sprintf(buffer, "%s", "");
		return;
	}

	if (disknum < 26)
		sprintf(buffer, "sd%c", 'a' + disknum);
	else 	{
		unsigned int min1;
		unsigned int min2;
		/*
		 * For larger numbers of disks, we need to go to a new
		 * naming scheme.
		 */
		min1 = disknum / 26;
		min2 = disknum % 26;
		sprintf(buffer, "sd%c%c", 'a' + min1 - 1, 'a' + min2);
		}
}

/**************************************************************************

Function Name: hr_get_sd_devnum()

Function Description
	To convert major/minor to OS exposed disk number

Input Parameters
	Scsi Command structure

Output Parameters
	returns mapped disk number

**************************************************************************/
static int hr_get_sd_devnum(Scsi_Cmnd *cmd)
{
	int dmaj, dfmin, devnum;

#ifndef SD_MAJORS
#define SD_MAJORS	16
#endif
	dmaj = 0;
	dfmin = 0;
	devnum = -1;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	dmaj = MAJOR(cmd->request.rq_dev);
	dfmin = MINOR(cmd->request.rq_dev);
#else
	/* get disk num based on major/minor of the 
	 * device referred by this command 
	 */
	if (cmd->request->rq_disk) {
		dmaj = cmd->request->rq_disk->major;
		dfmin = cmd->request->rq_disk->first_minor;
	}
#endif
	if (dmaj == SCSI_DISK0_MAJOR) {
		devnum = dfmin / SD_MAJORS;
	}
	if (dmaj >= SCSI_DISK1_MAJOR && dmaj <= SCSI_DISK7_MAJOR) {
		/* we are here because we have bypassed first 16 whole disks */
		devnum = 15 + ((dmaj % SCSI_DISK1_MAJOR) * SD_MAJORS) 
			 + dfmin / SD_MAJORS;
	}
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
	if (dmaj >= SCSI_DISK8_MAJOR  && dmaj <= SCSI_DISK15_MAJOR) {
		/* we are here because we have bypassed first 128 whole disks */
		devnum = 127 + ((dmaj % SCSI_DISK8_MAJOR) * SD_MAJORS) 
			 + dfmin / SD_MAJORS;
	}
#endif
	return devnum;
}

/****************************** Host Template *********************************/
#if (defined CONFIG_HIGHIO)
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,10)
/* Assume RedHat Distribution with its different HIGHIO conventions. */
#define HR_TEMPLATE_DMA_SETTINGS()	\
	.can_dma_32		= 1,	\
	.single_sg_okay		= 1,
#else
#define HR_TEMPLATE_DMA_SETTINGS()	\
	.highmem_io		= 1,
#endif
#else
#define AIC_TEMPLATE_DMA_SETTINGS()
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
static Scsi_Host_Template hr_host_template = {
	.module			 THIS_MODULE,
	.name			 HR_DRIVER_NAME,
	.proc_name 		 HR_DRIVER_NAME,
	.proc_info               hr_proc_info,
	.detect                  hr_detect,
	.release                 hr_release,
	.info                    hr_info,
	.queuecommand            hr_queuecommand,
	.eh_abort_handler        hr_abort,
	.bios_param              hr_bios_param,
	.select_queue_depths     hr_select_queue_depths,
	.use_clustering          ENABLE_CLUSTERING,
	.use_new_eh_code         1,
	HR_TEMPLATE_DMA_SETTINGS()
};

#else
static Scsi_Host_Template hr_host_template = {
	.module			= THIS_MODULE,
	.name			= HR_DRIVER_NAME,
	.proc_name		= HR_DRIVER_NAME,
	.info                   = hr_info,
	.proc_info		= hr_proc_info,
	.queuecommand           = hr_queuecommand,
	.eh_abort_handler       = hr_abort,
	.bios_param             = hr_bios_param,
	.use_clustering         = ENABLE_CLUSTERING,
	.slave_alloc		= hr_linux_slave_alloc,
	.slave_configure	= hr_linux_slave_configure,
	.slave_destroy		= hr_linux_slave_destroy,
};
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,41)
EXPORT_NO_SYMBOLS;
#endif

#ifdef MODULE_LICENSE
MODULE_LICENSE("Adaptec");
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
MODULE_PARM(hr_user_cmd_per_lun, "i");
MODULE_PARM_DESC(hr_user_cmd_per_lun,"Per-device queue depth");
#else
module_param(hr_user_cmd_per_lun, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(hr_user_cmd_per_lun,"Per-device queue depth");
module_param(hr_user_max_xfer, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(hr_user_max_xfer,"Adapter max transfer per req in K bytes"
	", Warning: data corruption may occurs if set Xfer too high;"
	" Do it at your own risk");
#endif

module_init(hr_init);
module_exit(hr_exit);
